@@ -29,9 +29,11 @@
 #include "Parser.h"
 #include "Protect.h"
 
-namespace JSC {
+namespace {
 
-class Recompiler {
+using namespace JSC;
+
+class Recompiler : public MarkedBlock::VoidFunctor {
 public:
     Recompiler(Debugger*);
     ~Recompiler();
@@ -82,6 +84,10 @@ inline void Recompiler::operator()(JSCell* cell)
         m_sourceProviders.add(executable->source().provider(), exec);
 }
 
+} // namespace
+
+namespace JSC {
+
 Debugger::~Debugger()
 {
     HashSet<JSGlobalObject*>::iterator end = m_globalObjects.end();
@@ -112,7 +118,7 @@ void Debugger::recompileAllJSFunctions(JSGlobalData* globalData)
         return;
 
     Recompiler recompiler(this);
-    globalData->heap.forEach(recompiler);
+    globalData->heap.forEachCell(recompiler);
 }
 
 JSValue evaluateInGlobalCallFrame(const UString& script, JSValue& exception, JSGlobalObject* globalObject)
