@@ -26,10 +26,10 @@
 #include "CallFrame.h"
 #include "CommonIdentifiers.h"
 #include "Identifier.h"
-#include "JSNumberCell.h"
 #include "PropertyDescriptor.h"
 #include "PropertySlot.h"
 #include "RopeImpl.h"
+#include "Structure.h"
 
 namespace JSC {
 
@@ -59,9 +59,6 @@ namespace JSC {
     JSString* jsOwnedString(JSGlobalData*, const UString&); 
     JSString* jsOwnedString(ExecState*, const UString&); 
 
-    typedef void (*JSStringFinalizerCallback)(JSString*, void* context);
-    JSString* jsStringWithFinalizer(ExecState*, const UString&, JSStringFinalizerCallback callback, void* context);
-
     class JS_EXPORTCLASS JSString : public JSCell {
     public:
         friend class JIT;
@@ -69,6 +66,8 @@ namespace JSC {
         friend class SpecializedThunkJIT;
         friend struct ThunkHelpers;
 
+        typedef JSCell Base;
+
         class RopeBuilder {
         public:
             RopeBuilder(unsigned fiberCount)
@@ -93,7 +92,7 @@ namespace JSC {
             {
                 if (jsString->isRope()) {
                     for (unsigned i = 0; i < jsString->m_fiberCount; ++i)
-                        append(jsString->m_other.m_fibers[i]);
+                        append(jsString->m_fibers[i]);
                 } else
                     append(jsString->string());
             }
@@ -183,146 +182,234 @@ namespace JSC {
 
                 Vector<WorkItem, 16> m_workQueue;
         };
-
-        ALWAYS_INLINE JSString(JSGlobalData* globalData, const UString& value)
-            : JSCell(globalData->stringStructure.get())
+        
+    private:
+        ALWAYS_INLINE JSString(JSGlobalData& globalData, const UString& value)
+            : JSCell(globalData, globalData.stringStructure.get())
             , m_length(value.length())
             , m_value(value)
             , m_fiberCount(0)
         {
-            ASSERT(!m_value.isNull());
-            Heap::heap(this)->reportExtraMemoryCost(value.impl()->cost());
         }
 
         enum HasOtherOwnerType { HasOtherOwner };
-        JSString(JSGlobalData* globalData, const UString& value, HasOtherOwnerType)
-            : JSCell(globalData->stringStructure.get())
+        JSString(JSGlobalData& globalData, const UString& value, HasOtherOwnerType)
+            : JSCell(globalData, globalData.stringStructure.get())
             , m_length(value.length())
             , m_value(value)
             , m_fiberCount(0)
         {
-            ASSERT(!m_value.isNull());
         }
-        JSString(JSGlobalData* globalData, PassRefPtr<StringImpl> value, HasOtherOwnerType)
-            : JSCell(globalData->stringStructure.get())
+        JSString(JSGlobalData& globalData, PassRefPtr<StringImpl> value, HasOtherOwnerType)
+            : JSCell(globalData, globalData.stringStructure.get())
             , m_length(value->length())
             , m_value(value)
             , m_fiberCount(0)
         {
-            ASSERT(!m_value.isNull());
         }
-        JSString(JSGlobalData* globalData, PassRefPtr<RopeImpl> rope)
-            : JSCell(globalData->stringStructure.get())
+        JSString(JSGlobalData& globalData, PassRefPtr<RopeImpl> rope)
+            : JSCell(globalData, globalData.stringStructure.get())
             , m_length(rope->length())
             , m_fiberCount(1)
         {
-            m_other.m_fibers[0] = rope.leakRef();
         }
         // This constructor constructs a new string by concatenating s1 & s2.
         // This should only be called with fiberCount <= 3.
-        JSString(JSGlobalData* globalData, unsigned fiberCount, JSString* s1, JSString* s2)
-            : JSCell(globalData->stringStructure.get())
+        JSString(JSGlobalData& globalData, unsigned fiberCount, JSString* s1, JSString* s2)
+            : JSCell(globalData, globalData.stringStructure.get())
             , m_length(s1->length() + s2->length())
             , m_fiberCount(fiberCount)
         {
-            ASSERT(fiberCount <= s_maxInternalRopeLength);
-            unsigned index = 0;
-            appendStringInConstruct(index, s1);
-            appendStringInConstruct(index, s2);
-            ASSERT(fiberCount == index);
         }
         // This constructor constructs a new string by concatenating s1 & s2.
         // This should only be called with fiberCount <= 3.
-        JSString(JSGlobalData* globalData, unsigned fiberCount, JSString* s1, const UString& u2)
-            : JSCell(globalData->stringStructure.get())
+        JSString(JSGlobalData& globalData, unsigned fiberCount, JSString* s1, const UString& u2)
+            : JSCell(globalData, globalData.stringStructure.get())
             , m_length(s1->length() + u2.length())
             , m_fiberCount(fiberCount)
         {
-            ASSERT(fiberCount <= s_maxInternalRopeLength);
-            unsigned index = 0;
-            appendStringInConstruct(index, s1);
-            appendStringInConstruct(index, u2);
-            ASSERT(fiberCount == index);
         }
         // This constructor constructs a new string by concatenating s1 & s2.
         // This should only be called with fiberCount <= 3.
-        JSString(JSGlobalData* globalData, unsigned fiberCount, const UString& u1, JSString* s2)
-            : JSCell(globalData->stringStructure.get())
+        JSString(JSGlobalData& globalData, unsigned fiberCount, const UString& u1, JSString* s2)
+            : JSCell(globalData, globalData.stringStructure.get())
             , m_length(u1.length() + s2->length())
             , m_fiberCount(fiberCount)
         {
-            ASSERT(fiberCount <= s_maxInternalRopeLength);
-            unsigned index = 0;
-            appendStringInConstruct(index, u1);
-            appendStringInConstruct(index, s2);
-            ASSERT(fiberCount == index);
         }
-        // This constructor constructs a new string by concatenating v1, v2 & v3.
-        // This should only be called with fiberCount <= 3 ... which since every
-        // value must require a fiberCount of at least one implies that the length
-        // for each value must be exactly 1!
-        JSString(ExecState* exec, JSValue v1, JSValue v2, JSValue v3)
-            : JSCell(exec->globalData().stringStructure.get())
+        JSString(ExecState* exec)
+            : JSCell(exec->globalData(), exec->globalData().stringStructure.get())
             , m_length(0)
             , m_fiberCount(s_maxInternalRopeLength)
         {
-            unsigned index = 0;
-            appendValueInConstructAndIncrementLength(exec, index, v1);
-            appendValueInConstructAndIncrementLength(exec, index, v2);
-            appendValueInConstructAndIncrementLength(exec, index, v3);
-            ASSERT(index == s_maxInternalRopeLength);
         }
 
         // This constructor constructs a new string by concatenating u1 & u2.
-        JSString(JSGlobalData* globalData, const UString& u1, const UString& u2)
-            : JSCell(globalData->stringStructure.get())
+        JSString(JSGlobalData& globalData, const UString& u1, const UString& u2)
+            : JSCell(globalData, globalData.stringStructure.get())
             , m_length(u1.length() + u2.length())
             , m_fiberCount(2)
         {
-            unsigned index = 0;
-            appendStringInConstruct(index, u1);
-            appendStringInConstruct(index, u2);
-            ASSERT(index <= s_maxInternalRopeLength);
         }
 
         // This constructor constructs a new string by concatenating u1, u2 & u3.
-        JSString(JSGlobalData* globalData, const UString& u1, const UString& u2, const UString& u3)
-            : JSCell(globalData->stringStructure.get())
+        JSString(JSGlobalData& globalData, const UString& u1, const UString& u2, const UString& u3)
+            : JSCell(globalData, globalData.stringStructure.get())
             , m_length(u1.length() + u2.length() + u3.length())
             , m_fiberCount(s_maxInternalRopeLength)
         {
+        }
+
+        void finishCreation(JSGlobalData& globalData, const UString& value)
+        {
+            Base::finishCreation(globalData);
+            ASSERT(!m_value.isNull());
+            Heap::heap(this)->reportExtraMemoryCost(value.impl()->cost());
+        }
+
+        void finishCreation(JSGlobalData& globalData)
+        {
+            Base::finishCreation(globalData);
+            ASSERT(!m_value.isNull());
+        }
+
+        void finishCreation(JSGlobalData& globalData, PassRefPtr<RopeImpl> rope)
+        {
+            Base::finishCreation(globalData);
+            m_fibers[0] = rope.leakRef();
+        }
+
+        void finishCreation(JSGlobalData& globalData, unsigned fiberCount, JSString* s1, JSString* s2)
+        {
+            Base::finishCreation(globalData);
+            ASSERT_UNUSED(fiberCount, fiberCount <= s_maxInternalRopeLength);
             unsigned index = 0;
-            appendStringInConstruct(index, u1);
-            appendStringInConstruct(index, u2);
-            appendStringInConstruct(index, u3);
+            appendStringInCreate(index, s1);
+            appendStringInCreate(index, s2);
+            ASSERT(fiberCount == index);
+        }
+
+        void finishCreation(JSGlobalData& globalData, unsigned fiberCount, JSString* s1, const UString& u2)
+        {
+            Base::finishCreation(globalData);
+            ASSERT_UNUSED(fiberCount, fiberCount <= s_maxInternalRopeLength);
+            unsigned index = 0;
+            appendStringInCreate(index, s1);
+            appendStringInCreate(index, u2);
+            ASSERT(fiberCount == index);
+        }
+
+        void finishCreation(JSGlobalData& globalData, unsigned fiberCount, const UString& u1, JSString* s2)
+        {
+            Base::finishCreation(globalData);
+            ASSERT_UNUSED(fiberCount, fiberCount <= s_maxInternalRopeLength);
+            unsigned index = 0;
+            appendStringInCreate(index, u1);
+            appendStringInCreate(index, s2);
+            ASSERT(fiberCount == index);
+        }
+
+        // Fills in the new string by concatenating v1, v2 & v3.
+        // This should only be called with fiberCount <= 3 ... which since every
+        // value must require a fiberCount of at least one implies that the length
+        // for each value must be exactly 1!
+        void finishCreation(ExecState* exec, JSValue v1, JSValue v2, JSValue v3)
+        {
+            Base::finishCreation(exec->globalData());
+            unsigned index = 0;
+            appendValueInCreateAndIncrementLength(exec, index, v1);
+            appendValueInCreateAndIncrementLength(exec, index, v2);
+            appendValueInCreateAndIncrementLength(exec, index, v3);
+            ASSERT(index == s_maxInternalRopeLength);
+        }
+
+        void finishCreation(JSGlobalData& globalData, const UString& u1, const UString& u2)
+        {
+            Base::finishCreation(globalData);
+            unsigned index = 0;
+            appendStringInCreate(index, u1);
+            appendStringInCreate(index, u2);
             ASSERT(index <= s_maxInternalRopeLength);
         }
 
-        JSString(JSGlobalData* globalData, const UString& value, JSStringFinalizerCallback finalizer, void* context)
-            : JSCell(globalData->stringStructure.get())
-            , m_length(value.length())
-            , m_value(value)
-            , m_fiberCount(0)
+        void finishCreation(JSGlobalData& globalData, const UString& u1, const UString& u2, const UString& u3)
         {
-            ASSERT(!m_value.isNull());
-            // nasty hack because we can't union non-POD types
-            m_other.m_finalizerCallback = finalizer;
-            m_other.m_finalizerContext = context;
-            Heap::heap(this)->reportExtraMemoryCost(value.impl()->cost());
+            Base::finishCreation(globalData);
+            unsigned index = 0;
+            appendStringInCreate(index, u1);
+            appendStringInCreate(index, u2);
+            appendStringInCreate(index, u3);
+            ASSERT(index <= s_maxInternalRopeLength);
+        }
+
+    public:
+        static JSString* create(JSGlobalData& globalData, const UString& value)
+        {
+            JSString* newString = new (allocateCell<JSString>(globalData.heap)) JSString(globalData, value);
+            newString->finishCreation(globalData, value);
+            return newString;
+        }
+        static JSString* createHasOtherOwner(JSGlobalData& globalData, const UString& value)
+        {
+            JSString* newString = new (allocateCell<JSString>(globalData.heap)) JSString(globalData, value, HasOtherOwner);
+            newString->finishCreation(globalData, value);
+            return newString;
+        }
+        static JSString* createHasOtherOwner(JSGlobalData& globalData, PassRefPtr<StringImpl> value)
+        {
+            JSString* newString = new (allocateCell<JSString>(globalData.heap)) JSString(globalData, value, HasOtherOwner);
+            newString->finishCreation(globalData);
+            return newString;
+        }
+        static JSString* create(JSGlobalData& globalData, PassRefPtr<RopeImpl> rope)
+        {
+            RefPtr<RopeImpl> tempRope = rope;
+            JSString* newString = new (allocateCell<JSString>(globalData.heap)) JSString(globalData, tempRope);
+            newString->finishCreation(globalData, tempRope);
+            return newString;
+        }
+        static JSString* create(JSGlobalData& globalData, unsigned fiberCount, JSString* s1, JSString* s2)
+        {
+            JSString* newString = new (allocateCell<JSString>(globalData.heap)) JSString(globalData, fiberCount, s1, s2);
+            newString->finishCreation(globalData, fiberCount, s1, s2);
+            return newString;
+        }
+        static JSString* create(JSGlobalData& globalData, unsigned fiberCount, JSString* s1, const UString& u2)
+        {
+            JSString* newString = new (allocateCell<JSString>(globalData.heap)) JSString(globalData, fiberCount, s1, u2);
+            newString->finishCreation(globalData, fiberCount, s1, u2);
+            return newString;
+        }
+        static JSString* create(JSGlobalData& globalData, unsigned fiberCount, const UString& u1, JSString* s2)
+        {
+            JSString* newString = new (allocateCell<JSString>(globalData.heap)) JSString(globalData, fiberCount, u1, s2);
+            newString->finishCreation(globalData, fiberCount, u1, s2);
+            return newString;
+        }
+        static JSString* create(ExecState* exec, JSValue v1, JSValue v2, JSValue v3)
+        {
+            JSString* newString = new (allocateCell<JSString>(*exec->heap())) JSString(exec);
+            newString->finishCreation(exec, v1, v2, v3);
+            return newString;
+        }
+        static JSString* create(JSGlobalData& globalData, const UString& u1, const UString& u2)
+        {
+            JSString* newString = new (allocateCell<JSString>(globalData.heap)) JSString(globalData, u1, u2);
+            newString->finishCreation(globalData, u1, u2);
+            return newString;
+        }
+        static JSString* create(JSGlobalData& globalData, const UString& u1, const UString& u2, const UString& u3)
+        {
+            JSString* newString = new (allocateCell<JSString>(globalData.heap)) JSString(globalData, u1, u2, u3);
+            newString->finishCreation(globalData, u1, u2, u3);
+            return newString;
         }
 
         ~JSString()
         {
             ASSERT(vptr() == JSGlobalData::jsStringVPtr);
-            if (!m_fiberCount) {
-                if (m_other.m_finalizerCallback)
-                    m_other.m_finalizerCallback(this, m_other.m_finalizerContext);
-            } else {
-                unsigned i = 0;
-                do
-                    RopeImpl::deref(m_other.m_fibers[i]);
-                while (++i < m_fiberCount);
-            }
+            for (unsigned i = 0; i < m_fiberCount; ++i)
+                RopeImpl::deref(m_fibers[i]);
         }
 
         const UString& value(ExecState* exec) const
@@ -339,6 +426,7 @@ namespace JSC {
         }
         unsigned length() { return m_length; }
 
+        JSValue toPrimitive(ExecState*, PreferredPrimitiveType) const;
         bool getStringPropertySlot(ExecState*, const Identifier& propertyName, PropertySlot&);
         bool getStringPropertySlot(ExecState*, unsigned propertyName, PropertySlot&);
         bool getStringPropertyDescriptor(ExecState*, const Identifier& propertyName, PropertyDescriptor&);
@@ -349,60 +437,66 @@ namespace JSC {
 
         JSValue replaceCharacter(ExecState*, UChar, const UString& replacement);
 
-        static PassRefPtr<Structure> createStructure(JSGlobalData& globalData, JSValue proto) { return Structure::create(globalData, proto, TypeInfo(StringType, OverridesGetOwnPropertySlot | NeedsThisConversion), AnonymousSlotCount, 0); }
+        static Structure* createStructure(JSGlobalData& globalData, JSGlobalObject* globalObject, JSValue proto)
+        {
+            return Structure::create(globalData, globalObject, proto, TypeInfo(StringType, OverridesGetOwnPropertySlot), &s_info);
+        }
+        
+        static const ClassInfo s_info;
 
     private:
-        enum VPtrStealingHackType { VPtrStealingHack };
         JSString(VPtrStealingHackType) 
-            : JSCell(0)
+            : JSCell(VPtrStealingHack)
             , m_fiberCount(0)
         {
         }
 
         void resolveRope(ExecState*) const;
+        void resolveRopeSlowCase(ExecState*, UChar*) const;
+        void outOfMemory(ExecState*) const;
         JSString* substringFromRope(ExecState*, unsigned offset, unsigned length);
 
-        void appendStringInConstruct(unsigned& index, const UString& string)
+        void appendStringInCreate(unsigned& index, const UString& string)
         {
             StringImpl* impl = string.impl();
             impl->ref();
-            m_other.m_fibers[index++] = impl;
+            m_fibers[index++] = impl;
+            Heap::heap(this)->reportExtraMemoryCost(string.impl()->cost());
         }
 
-        void appendStringInConstruct(unsigned& index, JSString* jsString)
+        void appendStringInCreate(unsigned& index, JSString* jsString)
         {
             if (jsString->isRope()) {
                 for (unsigned i = 0; i < jsString->m_fiberCount; ++i) {
-                    RopeImpl::Fiber fiber = jsString->m_other.m_fibers[i];
+                    RopeImpl::Fiber fiber = jsString->m_fibers[i];
                     fiber->ref();
-                    m_other.m_fibers[index++] = fiber;
+                    m_fibers[index++] = fiber;
                 }
             } else
-                appendStringInConstruct(index, jsString->string());
+                appendStringInCreate(index, jsString->string());
         }
 
-        void appendValueInConstructAndIncrementLength(ExecState* exec, unsigned& index, JSValue v)
+        void appendValueInCreateAndIncrementLength(ExecState* exec, unsigned& index, JSValue v)
         {
             if (v.isString()) {
                 ASSERT(v.asCell()->isString());
                 JSString* s = static_cast<JSString*>(v.asCell());
                 ASSERT(s->fiberCount() == 1);
-                appendStringInConstruct(index, s);
+                appendStringInCreate(index, s);
                 m_length += s->length();
             } else {
                 UString u(v.toString(exec));
                 StringImpl* impl = u.impl();
                 impl->ref();
-                m_other.m_fibers[index++] = impl;
+                m_fibers[index++] = impl;
                 m_length += u.length();
             }
         }
 
-        virtual JSValue toPrimitive(ExecState*, PreferredPrimitiveType) const;
         virtual bool getPrimitiveNumber(ExecState*, double& number, JSValue& value);
         virtual bool toBoolean(ExecState*) const;
         virtual double toNumber(ExecState*) const;
-        virtual JSObject* toObject(ExecState*) const;
+        virtual JSObject* toObject(ExecState*, JSGlobalObject*) const;
         virtual UString toString(ExecState*) const;
 
         virtual JSObject* toThisObject(ExecState*) const;
@@ -418,17 +512,7 @@ namespace JSC {
         unsigned m_length;
         mutable UString m_value;
         mutable unsigned m_fiberCount;
-        // This structure exists to support a temporary workaround for a GC issue.
-        struct JSStringFinalizerStruct {
-            JSStringFinalizerStruct() : m_finalizerCallback(0) {}
-            union {
-                mutable FixedArray<RopeImpl::Fiber, s_maxInternalRopeLength> m_fibers;
-                struct {
-                    JSStringFinalizerCallback m_finalizerCallback;
-                    void* m_finalizerContext;
-                };
-            };
-        } m_other;
+        mutable FixedArray<RopeImpl::Fiber, s_maxInternalRopeLength> m_fibers;
 
         bool isRope() const { return m_fiberCount; }
         UString& string() { ASSERT(!isRope()); return m_value; }
@@ -439,7 +523,6 @@ namespace JSC {
         friend JSValue jsString(ExecState* exec, JSString* s1, const UString& u2);
         friend JSValue jsString(ExecState* exec, Register* strings, unsigned count);
         friend JSValue jsString(ExecState* exec, JSValue thisValue);
-        friend JSString* jsStringWithFinalizer(ExecState*, const UString&, JSStringFinalizerCallback callback, void* context);
         friend JSString* jsSubstring(ExecState* exec, JSString* s, unsigned offset, unsigned length);
     };
 
@@ -470,17 +553,17 @@ namespace JSC {
     {
         if (c <= maxSingleCharacterString)
             return globalData->smallStrings.singleCharacterString(globalData, c);
-        return fixupVPtr(globalData, new (globalData) JSString(globalData, UString(&c, 1)));
+        return fixupVPtr(globalData, JSString::create(*globalData, UString(&c, 1)));
     }
 
     inline JSString* jsSingleCharacterSubstring(ExecState* exec, const UString& s, unsigned offset)
     {
         JSGlobalData* globalData = &exec->globalData();
         ASSERT(offset < static_cast<unsigned>(s.length()));
-        UChar c = s.characters()[offset];
+        UChar c = s[offset];
         if (c <= maxSingleCharacterString)
             return globalData->smallStrings.singleCharacterString(globalData, c);
-        return fixupVPtr(globalData, new (globalData) JSString(globalData, UString(StringImpl::create(s.impl(), offset, 1))));
+        return fixupVPtr(globalData, JSString::create(*globalData, UString(StringImpl::create(s.impl(), offset, 1))));
     }
 
     inline JSString* jsNontrivialString(JSGlobalData* globalData, const char* s)
@@ -488,13 +571,13 @@ namespace JSC {
         ASSERT(s);
         ASSERT(s[0]);
         ASSERT(s[1]);
-        return fixupVPtr(globalData, new (globalData) JSString(globalData, s));
+        return fixupVPtr(globalData, JSString::create(*globalData, s));
     }
 
     inline JSString* jsNontrivialString(JSGlobalData* globalData, const UString& s)
     {
         ASSERT(s.length() > 1);
-        return fixupVPtr(globalData, new (globalData) JSString(globalData, s));
+        return fixupVPtr(globalData, JSString::create(*globalData, s));
     }
 
     inline JSString* JSString::getIndex(ExecState* exec, unsigned i)
@@ -512,20 +595,13 @@ namespace JSC {
         if (!size)
             return globalData->smallStrings.emptyString(globalData);
         if (size == 1) {
-            UChar c = s.characters()[0];
+            UChar c = s[0];
             if (c <= maxSingleCharacterString)
                 return globalData->smallStrings.singleCharacterString(globalData, c);
         }
-        return fixupVPtr(globalData, new (globalData) JSString(globalData, s));
+        return fixupVPtr(globalData, JSString::create(*globalData, s));
     }
 
-    inline JSString* jsStringWithFinalizer(ExecState* exec, const UString& s, JSStringFinalizerCallback callback, void* context)
-    {
-        ASSERT(s.length() && (s.length() > 1 || s.characters()[0] > maxSingleCharacterString));
-        JSGlobalData* globalData = &exec->globalData();
-        return fixupVPtr(globalData, new (globalData) JSString(globalData, s, callback, context));
-    }
-    
     inline JSString* jsSubstring(ExecState* exec, JSString* s, unsigned offset, unsigned length)
     {
         ASSERT(offset <= static_cast<unsigned>(s->length()));
@@ -547,11 +623,11 @@ namespace JSC {
         if (!length)
             return globalData->smallStrings.emptyString(globalData);
         if (length == 1) {
-            UChar c = s.characters()[offset];
+            UChar c = s[offset];
             if (c <= maxSingleCharacterString)
                 return globalData->smallStrings.singleCharacterString(globalData, c);
         }
-        return fixupVPtr(globalData, new (globalData) JSString(globalData, UString(StringImpl::create(s.impl(), offset, length)), JSString::HasOtherOwner));
+        return fixupVPtr(globalData, JSString::createHasOtherOwner(*globalData, UString(StringImpl::create(s.impl(), offset, length))));
     }
 
     inline JSString* jsOwnedString(JSGlobalData* globalData, const UString& s)
@@ -560,11 +636,11 @@ namespace JSC {
         if (!size)
             return globalData->smallStrings.emptyString(globalData);
         if (size == 1) {
-            UChar c = s.characters()[0];
+            UChar c = s[0];
             if (c <= maxSingleCharacterString)
                 return globalData->smallStrings.singleCharacterString(globalData, c);
         }
-        return fixupVPtr(globalData, new (globalData) JSString(globalData, s, JSString::HasOtherOwner));
+        return fixupVPtr(globalData, JSString::createHasOtherOwner(*globalData, s));
     }
 
     inline JSString* jsEmptyString(ExecState* exec) { return jsEmptyString(&exec->globalData()); }
