@@ -35,6 +35,8 @@
 #include "FocusController.h"
 #include "Frame.h"
 #include "FrameLoader.h"
+#include "FrameSelection.h"
+#include "HTMLNames.h"
 #include "LocalizedStrings.h"
 #include "NodeList.h"
 #include "NotImplemented.h"
@@ -47,11 +49,14 @@
 #include "RenderTheme.h"
 #include "RenderView.h"
 #include "RenderWidget.h"
-#include "SelectionController.h"
+#include "RenderedPosition.h"
+#include "TextCheckerClient.h"
 #include "TextIterator.h"
 #include "htmlediting.h"
 #include "visible_units.h"
 #include <wtf/StdLibExtras.h>
+#include <wtf/text/StringBuilder.h>
+#include <wtf/text/WTFString.h>
 #include <wtf/unicode/CharacterNames.h>
 
 using namespace std;
@@ -82,6 +87,229 @@ void AccessibilityObject::detach()
 #endif    
 }
 
+bool AccessibilityObject::isAccessibilityObjectSearchMatch(AccessibilityObject* axObject, AccessibilitySearchPredicate* axSearchPredicate)
+{
+    if (!axObject || !axSearchPredicate)
+        return false;
+    
+    switch (axSearchPredicate->axSearchKey) {
+    // The AnyTypeSearchKey matches any non-null AccessibilityObject.
+    case AnyTypeSearchKey:
+        return true;
+        
+    case BlockquoteSameLevelSearchKey:
+        return axSearchPredicate->axStartObject
+            && axObject->isBlockquote()
+            && axObject->blockquoteLevel() == axSearchPredicate->axStartObject->blockquoteLevel();
+        
+    case BlockquoteSearchKey:
+        return axObject->isBlockquote();
+        
+    case BoldFontSearchKey:
+        return axObject->hasBoldFont();
+        
+    case ButtonSearchKey:
+        return axObject->isButton();
+        
+    case CheckBoxSearchKey:
+        return axObject->isCheckbox();
+        
+    case ControlSearchKey:
+        return axObject->isControl();
+        
+    case DifferentTypeSearchKey:
+        return axSearchPredicate->axStartObject
+            && axObject->roleValue() != axSearchPredicate->axStartObject->roleValue();
+        
+    case FontChangeSearchKey:
+        return axSearchPredicate->axStartObject
+            && !axObject->hasSameFont(axSearchPredicate->axStartObject->renderer());
+        
+    case FontColorChangeSearchKey:
+        return axSearchPredicate->axStartObject
+            && !axObject->hasSameFontColor(axSearchPredicate->axStartObject->renderer());
+        
+    // FIXME: Handle this search key.
+    case FrameSearchKey:
+        return false;
+        
+    case GraphicSearchKey:
+        return axObject->isImage();
+        
+    case HeadingLevel1SearchKey:
+        return axObject->headingLevel() == 1;
+        
+    case HeadingLevel2SearchKey:
+        return axObject->headingLevel() == 2;
+        
+    case HeadingLevel3SearchKey:
+        return axObject->headingLevel() == 3;
+        
+    case HeadingLevel4SearchKey:
+        return axObject->headingLevel() == 4;
+        
+    case HeadingLevel5SearchKey:
+        return axObject->headingLevel() == 5;
+        
+    case HeadingLevel6SearchKey:
+        return axObject->headingLevel() == 6;
+        
+    case HeadingSameLevelSearchKey:
+        return axSearchPredicate->axStartObject
+            && axObject->isHeading()
+            && axObject->headingLevel() == axSearchPredicate->axStartObject->headingLevel();
+        
+    case HeadingSearchKey:
+        return axObject->isHeading();
+        
+    case ItalicFontSearchKey:
+        return axObject->hasItalicFont();
+        
+    case LandmarkSearchKey:
+        return axObject->isLandmark();
+        
+    case LinkSearchKey:
+        return axObject->isLink();
+        
+    case ListSearchKey:
+        return axObject->isList();
+        
+    case LiveRegionSearchKey:
+        return axObject->supportsARIALiveRegion();
+        
+    case MisspelledWordSearchKey:
+        return axObject->hasMisspelling();
+        
+    case PlainTextSearchKey:
+        return axObject->hasPlainText();
+        
+    case RadioGroupSearchKey:
+        return axObject->isRadioGroup();
+        
+    case SameTypeSearchKey:
+        return axSearchPredicate->axStartObject
+            && axObject->roleValue() == axSearchPredicate->axStartObject->roleValue();
+        
+    case StaticTextSearchKey:
+        return axObject->hasStaticText();
+        
+    case StyleChangeSearchKey:
+        return axSearchPredicate->axStartObject
+            && !axObject->hasSameStyle(axSearchPredicate->axStartObject->renderer());
+        
+    case TableSameLevelSearchKey:
+        return axSearchPredicate->axStartObject
+            && axObject->isAccessibilityTable()
+            && axObject->tableLevel() == axSearchPredicate->axStartObject->tableLevel();
+        
+    case TableSearchKey:
+        return axObject->isAccessibilityTable();
+        
+    case TextFieldSearchKey:
+        return axObject->isTextControl();
+        
+    case UnderlineSearchKey:
+        return axObject->hasUnderline();
+        
+    case UnvisitedLinkSearchKey:
+        return axObject->isUnvisited();
+        
+    case VisitedLinkSearchKey:
+        return axObject->isVisited();
+        
+    default:
+        return false;
+    }
+}
+
+bool AccessibilityObject::isAccessibilityTextSearchMatch(AccessibilityObject* axObject, AccessibilitySearchPredicate* axSearchPredicate)
+{
+    if (!axObject || !axSearchPredicate)
+        return false;
+    
+    return axObject->accessibilityObjectContainsText(axSearchPredicate->searchText);
+}
+
+bool AccessibilityObject::accessibilityObjectContainsText(String* text) const
+{
+    // If text is null or empty we return true.
+    return !text
+        || text->isEmpty()
+        || title().contains(*text, false)
+        || accessibilityDescription().contains(*text, false)
+        || stringValue().contains(*text, false);
+}
+
+bool AccessibilityObject::isBlockquote() const
+{
+    return node() && node()->hasTagName(blockquoteTag);
+}
+
+bool AccessibilityObject::isLandmark() const
+{
+    AccessibilityRole role = roleValue();
+    
+    return role == LandmarkApplicationRole
+        || role == LandmarkBannerRole
+        || role == LandmarkComplementaryRole
+        || role == LandmarkContentInfoRole
+        || role == LandmarkMainRole
+        || role == LandmarkNavigationRole
+        || role == LandmarkSearchRole;
+}
+
+bool AccessibilityObject::hasMisspelling() const
+{
+    if (!node())
+        return false;
+    
+    Document* document = node()->document();
+    if (!document)
+        return false;
+    
+    Frame* frame = document->frame();
+    if (!frame)
+        return false;
+    
+    Editor* editor = frame->editor();
+    if (!editor)
+        return false;
+    
+    TextCheckerClient* textChecker = editor->textChecker();
+    if (!textChecker)
+        return false;
+    
+    const UChar* chars = stringValue().characters();
+    int charsLength = stringValue().length();
+    bool isMisspelled = false;
+    
+#if USE(UNIFIED_TEXT_CHECKING)
+    Vector<TextCheckingResult> results;
+    textChecker->checkTextOfParagraph(chars, charsLength, TextCheckingTypeSpelling, results);
+    if (!results.isEmpty())
+        isMisspelled = true;
+#else
+    int misspellingLength = 0;
+    int misspellingLocation = -1;
+    textChecker->checkSpellingOfString(chars, charsLength, &misspellingLocation, &misspellingLength);
+    if (misspellingLength || misspellingLocation != -1)
+        isMisspelled = true;
+#endif
+    
+    return isMisspelled;
+}
+
+int AccessibilityObject::blockquoteLevel() const
+{
+    int level = 0;
+    for (Node* elementNode = node(); elementNode; elementNode = elementNode->parentNode()) {
+        if (elementNode->hasTagName(blockquoteTag))
+            ++level;
+    }
+    
+    return level;
+}
+
 AccessibilityObject* AccessibilityObject::parentObjectUnignored() const
 {
     AccessibilityObject* parent;
@@ -120,6 +348,40 @@ AccessibilityObject* AccessibilityObject::firstAccessibleObjectFromNode(const No
     return accessibleObject;
 }
 
+void AccessibilityObject::accessibleObjectsWithAccessibilitySearchPredicate(AccessibilitySearchPredicate* axSearchPredicate, AccessibilityChildrenVector& axResults)
+{
+    ASSERT(AXObjectCache::accessibilityEnabled());
+    
+    if (!axSearchPredicate)
+        return;
+    
+    AccessibilityChildrenVector axChildren;
+    if (axSearchPredicate->axContainerObject)
+        axChildren.append(axSearchPredicate->axContainerObject);
+    
+    bool isSearchDirectionNext = (axSearchPredicate->axSearchDirection == SearchDirectionNext);
+    bool didFindAXStartObject = (!axSearchPredicate->axStartObject);
+    
+    // FIXME: Iterate the AccessibilityObject cache creating and adding objects if nessesary.
+    while (!axChildren.isEmpty() && axResults.size() < axSearchPredicate->resultsLimit) {
+        AccessibilityObject* axChild = axChildren.last().get();
+        axChildren.removeLast();
+        
+        if (didFindAXStartObject) {
+            if (isAccessibilityObjectSearchMatch(axChild, axSearchPredicate)
+                && isAccessibilityTextSearchMatch(axChild, axSearchPredicate))
+                axResults.append(axChild);
+        } else if (axChild == axSearchPredicate->axStartObject)
+            didFindAXStartObject = true;
+        
+        AccessibilityChildrenVector axGrandchildren = axChild->children();
+        unsigned axGrandchildrenSize = axChild->children().size();
+        for (unsigned i = (isSearchDirectionNext) ? axGrandchildrenSize : 0; (isSearchDirectionNext) ? i > 0 : i < axGrandchildrenSize; (isSearchDirectionNext) ? i-- : i++)
+            // FIXME: Handle attachments.
+            axChildren.append(axGrandchildren.at((isSearchDirectionNext) ? i - 1 : i).get());
+    }
+}
+
 bool AccessibilityObject::isARIAInput(AccessibilityRole ariaRole)
 {
     return ariaRole == RadioButtonRole || ariaRole == CheckBoxRole || ariaRole == TextFieldRole;
@@ -131,10 +393,10 @@ bool AccessibilityObject::isARIAControl(AccessibilityRole ariaRole)
     || ariaRole == ComboBoxRole || ariaRole == SliderRole; 
 }
 
-IntPoint AccessibilityObject::clickPoint() const
+LayoutPoint AccessibilityObject::clickPoint() const
 {
-    IntRect rect = elementRect();
-    return IntPoint(rect.x() + rect.width() / 2, rect.y() + rect.height() / 2);
+    LayoutRect rect = elementRect();
+    return LayoutPoint(rect.x() + rect.width() / 2, rect.y() + rect.height() / 2);
 }
 
 bool AccessibilityObject::press() const
@@ -215,22 +477,15 @@ static VisiblePosition updateAXLineStartForVisiblePosition(const VisiblePosition
     // So let's update the position to include that.
     VisiblePosition tempPosition;
     VisiblePosition startPosition = visiblePosition;
-    Position p;
-    RenderObject* renderer;
     while (true) {
         tempPosition = startPosition.previous();
-        if (tempPosition.isNull())
+        if (tempPosition.isNull() || tempPosition.isNull())
             break;
-        p = tempPosition.deepEquivalent();
-        if (!p.deprecatedNode())
-            break;
-        renderer = p.deprecatedNode()->renderer();
+        Position p = tempPosition.deepEquivalent();
+        RenderObject* renderer = p.deprecatedNode()->renderer();
         if (!renderer || (renderer->isRenderBlock() && !p.deprecatedEditingOffset()))
             break;
-        InlineBox* box;
-        int ignoredCaretOffset;
-        p.getInlineBoxAndOffset(tempPosition.affinity(), box, ignoredCaretOffset);
-        if (box)
+        if (!RenderedPosition(tempPosition).isNull())
             break;
         startPosition = tempPosition;
     }
@@ -430,12 +685,7 @@ String AccessibilityObject::listMarkerTextForNodeAndPosition(Node* node, const V
     // Append text, plus the period that follows the text.
     // FIXME: Not all list marker styles are followed by a period, but this
     // sounds much better when there is a synthesized pause because of a period.
-    Vector<UChar> resultVector;
-    resultVector.append(markerText.characters(), markerText.length());
-    resultVector.append('.');
-    resultVector.append(' ');
-    
-    return String::adopt(resultVector);
+    return markerText + ". ";
 }
     
 String AccessibilityObject::stringForVisiblePositionRange(const VisiblePositionRange& visiblePositionRange) const
@@ -443,7 +693,7 @@ String AccessibilityObject::stringForVisiblePositionRange(const VisiblePositionR
     if (visiblePositionRange.isNull())
         return String();
 
-    Vector<UChar> resultVector;
+    StringBuilder builder;
     RefPtr<Range> range = makeRange(visiblePositionRange.start, visiblePositionRange.end);
     for (TextIterator it(range.get()); !it.atEnd(); it.advance()) {
         // non-zero length means textual node, zero length means replaced node (AKA "attachments" in AX)
@@ -451,9 +701,9 @@ String AccessibilityObject::stringForVisiblePositionRange(const VisiblePositionR
             // Add a textual representation for list marker text
             String listMarkerText = listMarkerTextForNodeAndPosition(it.node(), visiblePositionRange.start);
             if (!listMarkerText.isEmpty())
-                resultVector.append(listMarkerText.characters(), listMarkerText.length());
-                
-            resultVector.append(it.characters(), it.length());
+                builder.append(listMarkerText);
+
+            builder.append(it.characters(), it.length());
         } else {
             // locate the node and starting offset for this replaced range
             int exception = 0;
@@ -462,11 +712,11 @@ String AccessibilityObject::stringForVisiblePositionRange(const VisiblePositionR
             int offset = it.range()->startOffset(exception);
 
             if (replacedNodeNeedsCharacter(node->childNode(offset)))
-                resultVector.append(objectReplacementCharacter);
+                builder.append(objectReplacementCharacter);
         }
     }
 
-    return String::adopt(resultVector);
+    return builder.toString();
 }
 
 int AccessibilityObject::lengthForVisiblePositionRange(const VisiblePositionRange& visiblePositionRange) const
@@ -719,6 +969,13 @@ unsigned AccessibilityObject::doAXLineForIndex(unsigned index)
     return lineForPosition(visiblePositionForIndex(index, false));
 }
     
+void AccessibilityObject::updateBackingStore()
+{
+    // Updating the layout may delete this object.
+    if (Document* document = this->document())
+        document->updateLayoutIgnorePendingStylesheets();
+}
+
 Document* AccessibilityObject::document() const
 {
     FrameView* frameView = documentFrameView();
@@ -727,6 +984,14 @@ Document* AccessibilityObject::document() const
     
     return frameView->frame()->document();
 }
+    
+Page* AccessibilityObject::page() const
+{
+    Document* document = this->document();
+    if (!document)
+        return 0;
+    return document->page();
+}
 
 FrameView* AccessibilityObject::documentFrameView() const 
 { 
@@ -887,7 +1152,7 @@ const AtomicString& AccessibilityObject::getAttribute(const QualifiedName& attri
 // Lacking concrete evidence of orientation, horizontal means width > height. vertical is height > width;
 AccessibilityOrientation AccessibilityObject::orientation() const
 {
-    IntRect bounds = elementRect();
+    LayoutRect bounds = elementRect();
     if (bounds.size().width() > bounds.size().height())
         return AccessibilityOrientationHorizontal;
     if (bounds.size().height() > bounds.size().width())
@@ -939,7 +1204,7 @@ static ARIARoleMap* createARIARoleMap()
         { "math", DocumentMathRole },
         { "menu", MenuRole },
         { "menubar", MenuBarRole },
-        // "menuitem" isn't here because it may map to different roles depending on the parent element's role
+        { "menuitem", MenuItemRole },
         { "menuitemcheckbox", MenuItemRole },
         { "menuitemradio", MenuItemRole },
         { "note", DocumentNoteRole },
@@ -980,8 +1245,21 @@ static ARIARoleMap* createARIARoleMap()
 AccessibilityRole AccessibilityObject::ariaRoleToWebCoreRole(const String& value)
 {
     ASSERT(!value.isEmpty());
+    
     static const ARIARoleMap* roleMap = createARIARoleMap();
-    return roleMap->get(value);
+
+    Vector<String> roleVector;
+    value.split(' ', roleVector);
+    AccessibilityRole role = UnknownRole;
+    unsigned size = roleVector.size();
+    for (unsigned i = 0; i < size; ++i) {
+        String roleName = roleVector[i];
+        role = roleMap->get(roleName);
+        if (role)
+            return role;
+    }
+    
+    return role;
 }
 
 const AtomicString& AccessibilityObject::placeholderValue() const
@@ -1017,14 +1295,14 @@ bool AccessibilityObject::supportsARIALiveRegion() const
     return equalIgnoringCase(liveRegion, "polite") || equalIgnoringCase(liveRegion, "assertive");
 }
 
-AccessibilityObject* AccessibilityObject::elementAccessibilityHitTest(const IntPoint& point) const
+AccessibilityObject* AccessibilityObject::elementAccessibilityHitTest(const LayoutPoint& point) const
 { 
     // Send the hit test back into the sub-frame if necessary.
     if (isAttachment()) {
         Widget* widget = widgetForAttachmentView();
         // Normalize the point for the widget's bounds.
         if (widget && widget->isFrameView())
-            return axObjectCache()->getOrCreate(widget)->accessibilityHitTest(IntPoint(point - widget->frameRect().location()));
+            return axObjectCache()->getOrCreate(widget)->accessibilityHitTest(toPoint(point - widget->frameRect().location()));
     }
 
     return const_cast<AccessibilityObject*>(this); 
