@@ -30,7 +30,7 @@
 #ifndef AccessibilityObject_h
 #define AccessibilityObject_h
 
-#include "IntRect.h"
+#include "LayoutTypes.h"
 #include "VisiblePosition.h"
 #include "VisibleSelection.h"
 #include <wtf/Forward.h>
@@ -49,7 +49,6 @@
 typedef struct _NSRange NSRange;
 
 #ifdef __OBJC__
-@class AccessibilityObjectWrapper;
 @class NSArray;
 @class NSAttributedString;
 @class NSData;
@@ -57,6 +56,7 @@ typedef struct _NSRange NSRange;
 @class NSString;
 @class NSValue;
 @class NSView;
+@class WebAccessibilityObjectWrapper;
 #else
 class NSArray;
 class NSAttributedString;
@@ -68,13 +68,20 @@ class NSView;
 #if PLATFORM(GTK)
 typedef struct _AtkObject AtkObject;
 typedef struct _AtkObject AccessibilityObjectWrapper;
+#elif PLATFORM(MAC)
+class WebAccessibilityObjectWrapper;
 #else
 class AccessibilityObjectWrapper;
 #endif
 #endif
 
+#if PLATFORM(MAC)
+typedef WebAccessibilityObjectWrapper AccessibilityObjectWrapper;
+#endif
+
 namespace WebCore {
 
+class AccessibilityObject;
 class AXObjectCache;
 class Element;
 class Frame;
@@ -84,6 +91,7 @@ class HTMLAreaElement;
 class IntPoint;
 class IntSize;
 class Node;
+class Page;
 class RenderObject;
 class RenderListItem;
 class VisibleSelection;
@@ -92,87 +100,48 @@ class Widget;
 typedef unsigned AXID;
 
 enum AccessibilityRole {
-    UnknownRole = 1,
-    ButtonRole,
-    RadioButtonRole,
-    CheckBoxRole,
-    SliderRole,
-    TabGroupRole,
-    TextFieldRole,
-    StaticTextRole,
-    TextAreaRole,
-    ScrollAreaRole,
-    PopUpButtonRole,
-    MenuButtonRole,
-    TableRole,
+    AnnotationRole = 1,
     ApplicationRole,
-    GroupRole,
-    RadioGroupRole,
-    ListRole,
-    ScrollBarRole,
-    ValueIndicatorRole,
-    ImageRole,
-    MenuBarRole,
-    MenuRole,
-    MenuItemRole,
-    ColumnRole,
-    RowRole,
-    ToolbarRole,
-    BusyIndicatorRole,
-    ProgressIndicatorRole,
-    WindowRole,
-    DrawerRole,
-    SystemWideRole,
-    OutlineRole,
-    IncrementorRole,
+    ApplicationAlertRole,
+    ApplicationAlertDialogRole,
+    ApplicationDialogRole,
+    ApplicationLogRole,
+    ApplicationMarqueeRole,
+    ApplicationStatusRole,
+    ApplicationTimerRole, 
     BrowserRole,
-    ComboBoxRole,
-    SplitGroupRole,
-    SplitterRole,
-    ColorWellRole,
-    GrowAreaRole,
-    SheetRole,
-    HelpTagRole,
-    MatteRole,
-    RulerRole,
-    RulerMarkerRole,
-    LinkRole,
-    DisclosureTriangleRole,
-    GridRole,
+    BusyIndicatorRole,
+    ButtonRole,
     CellRole, 
+    CheckBoxRole,
+    ColorWellRole,
+    ColumnRole,
     ColumnHeaderRole,
-    RowHeaderRole,
-    // AppKit includes SortButtonRole but it is misnamed and really a subrole of ButtonRole so we do not include it here.
-
-    // WebCore-specific roles
-    WebCoreLinkRole,
-    ImageMapLinkRole,
-    ImageMapRole,
-    ListMarkerRole,
-    WebAreaRole,
-    HeadingRole,
-    ListBoxRole,
-    ListBoxOptionRole,
-    TableHeaderContainerRole,
+    ComboBoxRole,
     DefinitionListTermRole,
     DefinitionListDefinitionRole,
-    AnnotationRole,
-    SliderThumbRole,
-    IgnoredRole,
-    PresentationalRole,
-    TabRole,
-    TabListRole,
-    TabPanelRole,
-    TreeRole,
-    TreeGridRole,
-    TreeItemRole,
     DirectoryRole,
+    DisclosureTriangleRole,
+    DivRole,
+    DocumentRole,
+    DocumentArticleRole,
+    DocumentMathRole,
+    DocumentNoteRole,
+    DocumentRegionRole,            
+    DrawerRole,
     EditableTextRole,
-    ListItemRole,
-    MenuListPopupRole,
-    MenuListOptionRole,
-
-    // ARIA Grouping roles
+    FormRole,
+    GridRole,
+    GroupRole,
+    GrowAreaRole,
+    HeadingRole,
+    HelpTagRole,
+    IgnoredRole,
+    ImageRole,
+    ImageMapRole,
+    ImageMapLinkRole,
+    IncrementorRole,
+    LabelRole,
     LandmarkApplicationRole,
     LandmarkBannerRole,
     LandmarkComplementaryRole,
@@ -180,22 +149,57 @@ enum AccessibilityRole {
     LandmarkMainRole,
     LandmarkNavigationRole,
     LandmarkSearchRole,
-    
-    ApplicationAlertRole,
-    ApplicationAlertDialogRole,
-    ApplicationDialogRole,
-    ApplicationLogRole,
-    ApplicationMarqueeRole,
-    ApplicationStatusRole,
-    ApplicationTimerRole,
-    
-    DocumentRole,
-    DocumentArticleRole,
-    DocumentMathRole,
-    DocumentNoteRole,
-    DocumentRegionRole,
-    
-    UserInterfaceTooltipRole
+    LinkRole,
+    ListRole,
+    ListBoxRole,
+    ListBoxOptionRole,
+    ListItemRole,
+    ListMarkerRole,
+    MatteRole,
+    MenuRole,
+    MenuBarRole,
+    MenuButtonRole,
+    MenuItemRole,
+    MenuListPopupRole,
+    MenuListOptionRole,
+    OutlineRole,
+    ParagraphRole,
+    PopUpButtonRole,
+    PresentationalRole,
+    ProgressIndicatorRole,
+    RadioButtonRole,
+    RadioGroupRole,
+    RowHeaderRole,
+    RowRole,
+    RulerRole,
+    RulerMarkerRole,
+    ScrollAreaRole,
+    ScrollBarRole,
+    SheetRole,
+    SliderRole,
+    SliderThumbRole,
+    SplitGroupRole,
+    SplitterRole,
+    StaticTextRole,
+    SystemWideRole,
+    TabGroupRole,
+    TabListRole,
+    TabPanelRole,            
+    TabRole,
+    TableRole,
+    TableHeaderContainerRole,
+    TextAreaRole,
+    TreeRole,
+    TreeGridRole,
+    TreeItemRole,
+    TextFieldRole,
+    ToolbarRole,
+    UnknownRole,
+    UserInterfaceTooltipRole,
+    ValueIndicatorRole,            
+    WebAreaRole,
+    WebCoreLinkRole,
+    WindowRole,
 };
 
 enum AccessibilityOrientation {
@@ -221,6 +225,60 @@ enum AccessibilitySortDirection {
     SortDirectionDescending,
 };
 
+enum AccessibilitySearchDirection {
+    SearchDirectionNext = 1,
+    SearchDirectionPrevious
+};
+
+enum AccessibilitySearchKey {
+    AnyTypeSearchKey = 1,
+    BlockquoteSameLevelSearchKey,
+    BlockquoteSearchKey,
+    BoldFontSearchKey,
+    ButtonSearchKey,
+    CheckBoxSearchKey,
+    ControlSearchKey,
+    DifferentTypeSearchKey,
+    FontChangeSearchKey,
+    FontColorChangeSearchKey,
+    FrameSearchKey,
+    GraphicSearchKey,
+    HeadingLevel1SearchKey,
+    HeadingLevel2SearchKey,
+    HeadingLevel3SearchKey,
+    HeadingLevel4SearchKey,
+    HeadingLevel5SearchKey,
+    HeadingLevel6SearchKey,
+    HeadingSameLevelSearchKey,
+    HeadingSearchKey,
+    ItalicFontSearchKey,
+    LandmarkSearchKey,
+    LinkSearchKey,
+    ListSearchKey,
+    LiveRegionSearchKey,
+    MisspelledWordSearchKey,
+    PlainTextSearchKey,
+    RadioGroupSearchKey,
+    SameTypeSearchKey,
+    StaticTextSearchKey,
+    StyleChangeSearchKey,
+    TableSameLevelSearchKey,
+    TableSearchKey,
+    TextFieldSearchKey,
+    UnderlineSearchKey,
+    UnvisitedLinkSearchKey,
+    VisitedLinkSearchKey
+};
+
+struct AccessibilitySearchPredicate {
+    AccessibilityObject* axContainerObject;
+    AccessibilityObject* axStartObject;
+    AccessibilitySearchDirection axSearchDirection;
+    AccessibilitySearchKey axSearchKey;
+    String* searchText;
+    unsigned resultsLimit;
+};
+
 struct VisiblePositionRange {
 
     VisiblePosition start;
@@ -257,6 +315,10 @@ struct PlainTextRange {
 class AccessibilityObject : public RefCounted<AccessibilityObject> {
 protected:
     AccessibilityObject();
+    
+    // Should only be called by accessibleObjectsWithAccessibilitySearchPredicate for AccessibilityObject searching.
+    static bool isAccessibilityObjectSearchMatch(AccessibilityObject*, AccessibilitySearchPredicate*);
+    static bool isAccessibilityTextSearchMatch(AccessibilityObject*, AccessibilitySearchPredicate*);
 public:
     virtual ~AccessibilityObject();
     virtual void detach();
@@ -267,6 +329,8 @@ public:
     virtual bool isAccessibilityScrollbar() const { return false; }
     virtual bool isAccessibilityScrollView() const { return false; }
     
+    bool accessibilityObjectContainsText(String *) const;
+    
     virtual bool isAnchor() const { return false; }
     virtual bool isAttachment() const { return false; }
     virtual bool isHeading() const { return false; }
@@ -317,6 +381,8 @@ public:
     bool isListItem() const { return roleValue() == ListItemRole; }
     bool isCheckboxOrRadio() const { return isCheckbox() || isRadioButton(); }
     bool isScrollView() const { return roleValue() == ScrollAreaRole; }
+    bool isBlockquote() const;
+    bool isLandmark() const;
     
     virtual bool isChecked() const { return false; }
     virtual bool isEnabled() const { return false; }
@@ -329,6 +395,7 @@ public:
     virtual bool isOffScreen() const { return false; }
     virtual bool isPressed() const { return false; }
     virtual bool isReadOnly() const { return false; }
+    virtual bool isUnvisited() const { return false; }
     virtual bool isVisited() const { return false; }
     virtual bool isRequired() const { return false; }
     virtual bool isLinked() const { return false; }
@@ -337,6 +404,16 @@ public:
     virtual bool isCollapsed() const { return false; }
     virtual void setIsExpanded(bool) { }
 
+    virtual bool hasBoldFont() const { return false; }
+    virtual bool hasItalicFont() const { return false; }
+    bool hasMisspelling() const;
+    virtual bool hasPlainText() const { return false; }
+    virtual bool hasSameFont(RenderObject*) const { return false; }
+    virtual bool hasSameFontColor(RenderObject*) const { return false; }
+    virtual bool hasSameStyle(RenderObject*) const { return false; }
+    bool hasStaticText() const { return roleValue() == StaticTextRole; }
+    virtual bool hasUnderline() const { return false; }
+
     virtual bool canSetFocusAttribute() const { return false; }
     virtual bool canSetTextRangeAttributes() const { return false; }
     virtual bool canSetValueAttribute() const { return false; }
@@ -346,17 +423,21 @@ public:
     virtual bool canSetExpandedAttribute() const { return false; }
     
     // A programmatic way to set a name on an AccessibleObject.
-    virtual void setAccessibleName(String&) { }
+    virtual void setAccessibleName(const AtomicString&) { }
     
     virtual Node* node() const { return 0; }
+    virtual RenderObject* renderer() const { return 0; }
     virtual bool accessibilityIsIgnored() const  { return true; }
 
+    int blockquoteLevel() const;
     virtual int headingLevel() const { return 0; }
+    virtual int tableLevel() const { return 0; }
     virtual AccessibilityButtonState checkboxOrRadioValue() const;
     virtual String valueDescription() const { return String(); }
     virtual float valueForRange() const { return 0.0f; }
     virtual float maxValueForRange() const { return 0.0f; }
     virtual float minValueForRange() const { return 0.0f; }
+    virtual float stepValueForRange() const { return 0.0f; }
     virtual AccessibilityObject* selectedRadioButton() { return 0; }
     virtual AccessibilityObject* selectedTabItem() { return 0; }    
     virtual int layoutCount() const { return 0; }
@@ -381,9 +462,9 @@ public:
     virtual void determineARIADropEffects(Vector<String>&) { }
     
     // Called on the root AX object to return the deepest available element.
-    virtual AccessibilityObject* accessibilityHitTest(const IntPoint&) const { return 0; }
+    virtual AccessibilityObject* accessibilityHitTest(const LayoutPoint&) const { return 0; }
     // Called on the AX object after the render tree determines which is the right AccessibilityRenderObject.
-    virtual AccessibilityObject* elementAccessibilityHitTest(const IntPoint&) const;
+    virtual AccessibilityObject* elementAccessibilityHitTest(const LayoutPoint&) const;
 
     virtual AccessibilityObject* focusedUIElement() const;
 
@@ -395,11 +476,13 @@ public:
     virtual AccessibilityObject* parentObjectUnignored() const;
     virtual AccessibilityObject* parentObjectIfExists() const { return 0; }
     static AccessibilityObject* firstAccessibleObjectFromNode(const Node*);
+    static void accessibleObjectsWithAccessibilitySearchPredicate(AccessibilitySearchPredicate*, AccessibilityChildrenVector&);
 
     virtual AccessibilityObject* observableObject() const { return 0; }
     virtual void linkedUIElements(AccessibilityChildrenVector&) const { }
     virtual AccessibilityObject* titleUIElement() const { return 0; }
     virtual bool exposesTitleUIElement() const { return true; }
+    virtual AccessibilityObject* correspondingLabelForControlElement() const { return 0; }
     virtual AccessibilityObject* correspondingControlForLabelElement() const { return 0; }
     virtual AccessibilityObject* scrollBar(AccessibilityOrientation) const { return 0; }
     
@@ -420,10 +503,10 @@ public:
     static AccessibilityObject* anchorElementForNode(Node*);
     virtual Element* anchorElement() const { return 0; }
     virtual Element* actionElement() const { return 0; }
-    virtual IntRect boundingBoxRect() const { return IntRect(); }
-    virtual IntRect elementRect() const = 0;
-    virtual IntSize size() const { return elementRect().size(); }
-    virtual IntPoint clickPoint() const;
+    virtual LayoutRect boundingBoxRect() const { return LayoutRect(); }
+    virtual LayoutRect elementRect() const = 0;
+    virtual LayoutSize size() const { return elementRect().size(); }
+    virtual LayoutPoint clickPoint() const;
 
     virtual PlainTextRange selectedTextRange() const { return PlainTextRange(); }
     unsigned selectionStart() const { return selectedTextRange().start; }
@@ -442,6 +525,7 @@ public:
     const String& actionVerb() const;
     virtual Widget* widget() const { return 0; }
     virtual Widget* widgetForAttachmentView() const { return 0; }
+    Page* page() const;
     virtual Document* document() const;
     virtual FrameView* topDocumentFrameView() const { return 0; }
     virtual FrameView* documentFrameView() const;
@@ -472,6 +556,8 @@ public:
     virtual bool canHaveChildren() const { return true; }
     virtual bool hasChildren() const { return m_haveChildren; }
     virtual void updateChildrenIfNecessary();
+    virtual void clearChildren();
+
     virtual void selectedChildren(AccessibilityChildrenVector&) { }
     virtual void visibleChildren(AccessibilityChildrenVector&) { }
     virtual void tabChildren(AccessibilityChildrenVector&) { }
@@ -497,7 +583,7 @@ public:
     VisiblePositionRange visiblePositionRangeForRange(const PlainTextRange&) const;
 
     String stringForVisiblePositionRange(const VisiblePositionRange&) const;
-    virtual IntRect boundsForVisiblePositionRange(const VisiblePositionRange&) const { return IntRect(); }
+    virtual LayoutRect boundsForVisiblePositionRange(const VisiblePositionRange&) const { return LayoutRect(); }
     int lengthForVisiblePositionRange(const VisiblePositionRange&) const;
     virtual void setSelectedVisiblePositionRange(const VisiblePositionRange&) const { }
 
@@ -528,7 +614,7 @@ public:
     PlainTextRange doAXStyleRangeForIndex(unsigned) const;
 
     virtual String doAXStringForRange(const PlainTextRange&) const { return String(); }
-    virtual IntRect doAXBoundsForRange(const PlainTextRange&) const { return IntRect(); }
+    virtual LayoutRect doAXBoundsForRange(const PlainTextRange&) const { return LayoutRect(); }
     String listMarkerTextForNodeAndPosition(Node*, const VisiblePosition&) const;
 
     unsigned doAXLineForIndex(unsigned);
@@ -584,7 +670,7 @@ public:
 
     // allows for an AccessibilityObject to update its render tree or perform
     // other operations update type operations
-    virtual void updateBackingStore() { }
+    void updateBackingStore();
     
 protected:
     AXID m_id;
@@ -592,7 +678,6 @@ protected:
     mutable bool m_haveChildren;
     AccessibilityRole m_role;
     
-    virtual void clearChildren();
     virtual bool isDetached() const { return true; }
     
 #if PLATFORM(GTK)
@@ -604,7 +689,7 @@ protected:
 #endif
 
 #if PLATFORM(MAC)
-    RetainPtr<AccessibilityObjectWrapper> m_wrapper;
+    RetainPtr<WebAccessibilityObjectWrapper> m_wrapper;
 #elif PLATFORM(WIN) && !OS(WINCE)
     COMPtr<AccessibilityObjectWrapper> m_wrapper;
 #elif PLATFORM(GTK)
