@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2007, 2008, 2009, 2010 Apple Inc. All rights reserved.
+ * Copyright (C) 2011 Google Inc. All rights reserved.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -31,6 +32,7 @@
 #include "JSEventListener.h"
 #include "JSEventSource.h"
 #include "JSFloat32Array.h"
+#include "JSFloat64Array.h"
 #include "JSHTMLCollection.h"
 #include "JSHistory.h"
 #include "JSImageConstructor.h"
@@ -46,7 +48,6 @@
 #include "JSUint8Array.h"
 #include "JSWebKitCSSMatrix.h"
 #include "JSWebKitPoint.h"
-#include "JSWorker.h"
 #include "JSXMLHttpRequest.h"
 #include "JSXSLTProcessor.h"
 #include "Location.h"
@@ -56,6 +57,10 @@
 #include "SharedWorkerRepository.h"
 #include <runtime/JSFunction.h>
 
+#if ENABLE(WORKERS)
+#include "JSWorker.h"
+#endif
+
 #if ENABLE(SHARED_WORKERS)
 #include "JSSharedWorker.h"
 #endif
@@ -72,40 +77,22 @@ using namespace JSC;
 
 namespace WebCore {
 
-void JSDOMWindow::markChildren(MarkStack& markStack)
+void JSDOMWindow::visitChildren(SlotVisitor& visitor)
 {
-    Base::markChildren(markStack);
+    ASSERT_GC_OBJECT_INHERITS(this, &s_info);
+    COMPILE_ASSERT(StructureFlags & OverridesVisitChildren, OverridesVisitChildrenWithoutSettingFlag);
+    ASSERT(structure()->typeInfo().overridesVisitChildren());
+    Base::visitChildren(visitor);
 
-    impl()->markJSEventListeners(markStack);
-
-    JSGlobalData& globalData = *Heap::heap(this)->globalData();
-
-    markDOMObjectWrapper(markStack, globalData, impl()->optionalConsole());
-    markDOMObjectWrapper(markStack, globalData, impl()->optionalHistory());
-    markDOMObjectWrapper(markStack, globalData, impl()->optionalLocationbar());
-    markDOMObjectWrapper(markStack, globalData, impl()->optionalMenubar());
-    markDOMObjectWrapper(markStack, globalData, impl()->optionalNavigator());
-    markDOMObjectWrapper(markStack, globalData, impl()->optionalPersonalbar());
-    markDOMObjectWrapper(markStack, globalData, impl()->optionalScreen());
-    markDOMObjectWrapper(markStack, globalData, impl()->optionalScrollbars());
-    markDOMObjectWrapper(markStack, globalData, impl()->optionalSelection());
-    markDOMObjectWrapper(markStack, globalData, impl()->optionalStatusbar());
-    markDOMObjectWrapper(markStack, globalData, impl()->optionalToolbar());
-    markDOMObjectWrapper(markStack, globalData, impl()->optionalLocation());
-    markDOMObjectWrapper(markStack, globalData, impl()->optionalMedia());
-#if ENABLE(DOM_STORAGE)
-    markDOMObjectWrapper(markStack, globalData, impl()->optionalSessionStorage());
-    markDOMObjectWrapper(markStack, globalData, impl()->optionalLocalStorage());
-#endif
-#if ENABLE(OFFLINE_WEB_APPLICATIONS)
-    markDOMObjectWrapper(markStack, globalData, impl()->optionalApplicationCache());
-#endif
+    impl()->visitJSEventListeners(visitor);
+    if (Frame* frame = impl()->frame())
+        visitor.addOpaqueRoot(frame);
 }
 
 template<NativeFunction nativeFunction, int length>
 JSValue nonCachingStaticFunctionGetter(ExecState* exec, JSValue, const Identifier& propertyName)
 {
-    return new (exec) JSFunction(exec, exec->lexicalGlobalObject(), exec->lexicalGlobalObject()->functionStructure(), length, propertyName, nativeFunction);
+    return JSFunction::create(exec, exec->lexicalGlobalObject(), exec->lexicalGlobalObject()->functionStructure(), length, propertyName, nativeFunction);
 }
 
 static JSValue childFrameGetter(ExecState* exec, JSValue slotBase, const Identifier& propertyName)
@@ -129,8 +116,8 @@ static JSValue namedItemGetter(ExecState* exec, JSValue slotBase, const Identifi
 
     RefPtr<HTMLCollection> collection = document->windowNamedItems(identifierToString(propertyName));
     if (collection->length() == 1)
-        return toJS(exec, collection->firstItem());
-    return toJS(exec, collection.get());
+        return toJS(exec, thisObj, collection->firstItem());
+    return toJS(exec, thisObj, collection.get());
 }
 
 bool JSDOMWindow::getOwnPropertySlot(ExecState* exec, const Identifier& propertyName, PropertySlot& slot)
@@ -141,7 +128,9 @@ bool JSDOMWindow::getOwnPropertySlot(ExecState* exec, const Identifier& property
 
     const HashEntry* entry;
 
-    // We don't want any properties other than "close" and "closed" on a closed window.
+    // We don't want any properties other than "close" and "closed" on a frameless window (i.e. one whose page got closed,
+    // or whose iframe got removed).
+    // FIXME: This doesn't fully match Firefox, which allows at least toString in addition to those.
     if (!impl()->frame()) {
         // The following code is safe for cross-domain and same domain use.
         // It ignores any custom properties that might be set on the DOMWindow (including a custom prototype).
@@ -441,24 +430,24 @@ JSValue JSDOMWindow::lookupSetter(ExecState* exec, const Identifier& propertyNam
 JSValue JSDOMWindow::history(ExecState* exec) const
 {
     History* history = impl()->history();
-    if (DOMObject* wrapper = getCachedDOMObjectWrapper(exec, history))
+    if (JSDOMWrapper* wrapper = getCachedWrapper(currentWorld(exec), history))
         return wrapper;
 
     JSDOMWindow* window = const_cast<JSDOMWindow*>(this);
-    JSHistory* jsHistory = new (exec) JSHistory(getDOMStructure<JSHistory>(exec, window), window, history);
-    cacheDOMObjectWrapper(exec, history, jsHistory);
+    JSHistory* jsHistory = JSHistory::create(getDOMStructure<JSHistory>(exec, window), window, history);
+    cacheWrapper(currentWorld(exec), history, jsHistory);
     return jsHistory;
 }
 
 JSValue JSDOMWindow::location(ExecState* exec) const
 {
     Location* location = impl()->location();
-    if (DOMObject* wrapper = getCachedDOMObjectWrapper(exec, location))
+    if (JSDOMWrapper* wrapper = getCachedWrapper(currentWorld(exec), location))
         return wrapper;
 
     JSDOMWindow* window = const_cast<JSDOMWindow*>(this);
-    JSLocation* jsLocation = new (exec) JSLocation(getDOMStructure<JSLocation>(exec, window), window, location);
-    cacheDOMObjectWrapper(exec, location, jsLocation);
+    JSLocation* jsLocation = JSLocation::create(getDOMStructure<JSLocation>(exec, window), window, location);
+    cacheWrapper(currentWorld(exec), location, jsLocation);
     return jsLocation;
 }
 
@@ -490,15 +479,13 @@ JSValue JSDOMWindow::event(ExecState* exec) const
     Event* event = currentEvent();
     if (!event)
         return jsUndefined();
-    return toJS(exec, event);
+    return toJS(exec, const_cast<JSDOMWindow*>(this), event);
 }
 
-#if ENABLE(EVENTSOURCE)
 JSValue JSDOMWindow::eventSource(ExecState* exec) const
 {
     return getDOMConstructor<JSEventSourceConstructor>(exec, this);
 }
-#endif
 
 JSValue JSDOMWindow::image(ExecState* exec) const
 {
@@ -569,6 +556,11 @@ JSValue JSDOMWindow::float32Array(ExecState* exec) const
     return getDOMConstructor<JSFloat32ArrayConstructor>(exec, this);
 }
 
+JSValue JSDOMWindow::float64Array(ExecState* exec) const
+{
+    return getDOMConstructor<JSFloat64ArrayConstructor>(exec, this);
+}
+
 JSValue JSDOMWindow::dataView(ExecState* exec) const
 {
     return getDOMConstructor<JSDataViewConstructor>(exec, this);
@@ -653,7 +645,6 @@ class DialogHandler {
 public:
     explicit DialogHandler(ExecState* exec)
         : m_exec(exec)
-        , m_globalObject(0)
     {
     }
 
@@ -662,25 +653,27 @@ public:
 
 private:
     ExecState* m_exec;
-    JSDOMWindow* m_globalObject;
+    RefPtr<Frame> m_frame;
 };
 
 inline void DialogHandler::dialogCreated(DOMWindow* dialog)
 {
+    m_frame = dialog->frame();
     // FIXME: This looks like a leak between the normal world and an isolated
     //        world if dialogArguments comes from an isolated world.
-    m_globalObject = toJSDOMWindow(dialog->frame(), normalWorld(m_exec->globalData()));
+    JSDOMWindow* globalObject = toJSDOMWindow(m_frame.get(), normalWorld(m_exec->globalData()));
     if (JSValue dialogArguments = m_exec->argument(1))
-        m_globalObject->putDirect(m_exec->globalData(), Identifier(m_exec, "dialogArguments"), dialogArguments);
+        globalObject->putDirect(m_exec->globalData(), Identifier(m_exec, "dialogArguments"), dialogArguments);
 }
 
 inline JSValue DialogHandler::returnValue() const
 {
-    if (!m_globalObject)
+    JSDOMWindow* globalObject = toJSDOMWindow(m_frame.get(), normalWorld(m_exec->globalData()));
+    if (!globalObject)
         return jsUndefined();
     Identifier identifier(m_exec, "returnValue");
     PropertySlot slot;
-    if (!m_globalObject->JSGlobalObject::getOwnPropertySlot(m_exec, identifier, slot))
+    if (!globalObject->getOwnPropertySlot(m_exec, identifier, slot))
         return jsUndefined();
     return slot.getValue(m_exec, identifier);
 }
@@ -708,7 +701,7 @@ JSValue JSDOMWindow::showModalDialog(ExecState* exec)
 
 JSValue JSDOMWindow::postMessage(ExecState* exec)
 {
-    PassRefPtr<SerializedScriptValue> message = SerializedScriptValue::create(exec, exec->argument(0));
+    RefPtr<SerializedScriptValue> message = SerializedScriptValue::create(exec, exec->argument(0));
 
     if (exec->hadException())
         return jsUndefined();
@@ -724,17 +717,27 @@ JSValue JSDOMWindow::postMessage(ExecState* exec)
         return jsUndefined();
 
     ExceptionCode ec = 0;
-    impl()->postMessage(message, &messagePorts, targetOrigin, activeDOMWindow(exec), ec);
+    impl()->postMessage(message.release(), &messagePorts, targetOrigin, activeDOMWindow(exec), ec);
     setDOMException(exec, ec);
 
     return jsUndefined();
 }
 
+JSValue JSDOMWindow::webkitPostMessage(ExecState* exec)
+{
+    return postMessage(exec);
+}
+
 JSValue JSDOMWindow::setTimeout(ExecState* exec)
 {
-    OwnPtr<ScheduledAction> action = ScheduledAction::create(exec, currentWorld(exec));
+    ContentSecurityPolicy* contentSecurityPolicy = impl()->document() ? impl()->document()->contentSecurityPolicy() : 0;
+    OwnPtr<ScheduledAction> action = ScheduledAction::create(exec, currentWorld(exec), contentSecurityPolicy);
     if (exec->hadException())
         return jsUndefined();
+
+    if (!action)
+        return jsNumber(0);
+
     int delay = exec->argument(1).toInt32(exec);
 
     ExceptionCode ec = 0;
@@ -746,11 +749,15 @@ JSValue JSDOMWindow::setTimeout(ExecState* exec)
 
 JSValue JSDOMWindow::setInterval(ExecState* exec)
 {
-    OwnPtr<ScheduledAction> action = ScheduledAction::create(exec, currentWorld(exec));
+    ContentSecurityPolicy* contentSecurityPolicy = impl()->document() ? impl()->document()->contentSecurityPolicy() : 0;
+    OwnPtr<ScheduledAction> action = ScheduledAction::create(exec, currentWorld(exec), contentSecurityPolicy);
     if (exec->hadException())
         return jsUndefined();
     int delay = exec->argument(1).toInt32(exec);
 
+    if (!action)
+        return jsNumber(0);
+
     ExceptionCode ec = 0;
     int result = impl()->setInterval(action.release(), delay, ec);
     setDOMException(exec, ec);
