@@ -31,30 +31,34 @@
  */
 
 #include "config.h"
-#include "JSInjectedScriptHost.h"
 
 #if ENABLE(INSPECTOR)
 
-#if ENABLE(DATABASE)
+#include "JSInjectedScriptHost.h"
+
+#if ENABLE(SQL_DATABASE)
 #include "Database.h"
 #include "JSDatabase.h"
 #endif
 #include "ExceptionCode.h"
 #include "InjectedScriptHost.h"
+#include "InspectorDebuggerAgent.h"
 #include "InspectorValues.h"
+#include "JSHTMLAllCollection.h"
+#include "JSHTMLCollection.h"
 #include "JSNode.h"
+#include "JSNodeList.h"
 #include "ScriptValue.h"
 #if ENABLE(DOM_STORAGE)
 #include "Storage.h"
 #include "JSStorage.h"
 #endif
+#include <runtime/DateInstance.h>
+#include <runtime/Error.h>
+#include <runtime/JSArray.h>
+#include <runtime/JSFunction.h>
 #include <runtime/JSLock.h>
-
-#if ENABLE(JAVASCRIPT_DEBUGGER)
-#include "JavaScriptCallFrame.h"
-#include "JSJavaScriptCallFrame.h"
-#include "ScriptDebugServer.h"
-#endif
+#include <runtime/RegExpObject.h>
 
 using namespace JSC;
 
@@ -70,22 +74,29 @@ Node* InjectedScriptHost::scriptValueAsNode(ScriptValue value)
 ScriptValue InjectedScriptHost::nodeAsScriptValue(ScriptState* state, Node* node)
 {
     JSLock lock(SilenceAssertionsOnly);
-    return ScriptValue(state->globalData(), toJS(state, node));
+    return ScriptValue(state->globalData(), toJS(state, deprecatedGlobalObjectForPrototype(state), node));
 }
 
-JSValue JSInjectedScriptHost::currentCallFrame(ExecState* exec)
+JSValue JSInjectedScriptHost::evaluate(ExecState* exec)
 {
-#if ENABLE(JAVASCRIPT_DEBUGGER)
-    JavaScriptCallFrame* callFrame = ScriptDebugServer::shared().currentCallFrame();
-    if (!callFrame || !callFrame->isValid())
+    JSValue expression = exec->argument(0);
+    if (!expression.isString())
+        return throwError(exec, createError(exec, "String argument expected."));
+    JSGlobalObject* globalObject = exec->lexicalGlobalObject();
+    JSFunction* evalFunction = globalObject->evalFunction();
+    CallData callData;
+    CallType callType = evalFunction->getCallData(callData);
+    if (callType == CallTypeNone)
         return jsUndefined();
+    MarkedArgumentBuffer args;
+    args.append(expression);
 
-    JSLock lock(SilenceAssertionsOnly);
-    return toJS(exec, callFrame);
-#else
-    UNUSED_PARAM(exec);
-    return jsUndefined();
-#endif
+    bool wasEvalEnabled = globalObject->evalEnabled();
+    globalObject->setEvalEnabled(true);
+    JSValue result = JSC::call(exec, evalFunction, callType, callData, exec->globalThisValue(), args);
+    globalObject->setEvalEnabled(wasEvalEnabled);
+
+    return result;
 }
 
 JSValue JSInjectedScriptHost::inspectedNode(ExecState* exec)
@@ -98,7 +109,7 @@ JSValue JSInjectedScriptHost::inspectedNode(ExecState* exec)
         return jsUndefined();
 
     JSLock lock(SilenceAssertionsOnly);
-    return toJS(exec, node);
+    return toJS(exec, globalObject(), node);
 }
 
 JSValue JSInjectedScriptHost::internalConstructorName(ExecState* exec)
@@ -110,12 +121,48 @@ JSValue JSInjectedScriptHost::internalConstructorName(ExecState* exec)
     return jsString(exec, result);
 }
 
+JSValue JSInjectedScriptHost::isHTMLAllCollection(ExecState* exec)
+{
+    if (exec->argumentCount() < 1)
+        return jsUndefined();
+
+    JSValue value = exec->argument(0);
+    return jsBoolean(value.inherits(&JSHTMLAllCollection::s_info));
+}
+
+JSValue JSInjectedScriptHost::type(ExecState* exec)
+{
+    if (exec->argumentCount() < 1)
+        return jsUndefined();
+
+    JSValue value = exec->argument(0);
+    if (value.isString())
+        return jsString(exec, String("string"));
+    if (value.inherits(&JSArray::s_info))
+        return jsString(exec, String("array"));
+    if (value.isBoolean())
+        return jsString(exec, String("boolean"));
+    if (value.isNumber())
+        return jsString(exec, String("number"));
+    if (value.inherits(&DateInstance::s_info))
+        return jsString(exec, String("date"));
+    if (value.inherits(&RegExpObject::s_info))
+        return jsString(exec, String("regexp"));
+    if (value.inherits(&JSNode::s_info))
+        return jsString(exec, String("node"));
+    if (value.inherits(&JSNodeList::s_info))
+        return jsString(exec, String("array"));
+    if (value.inherits(&JSHTMLCollection::s_info))
+        return jsString(exec, String("array"));
+    return jsUndefined();
+}
+
 JSValue JSInjectedScriptHost::inspect(ExecState* exec)
 {
     if (exec->argumentCount() >= 2) {
-        ScriptValue objectId(exec->globalData(), exec->argument(0));
+        ScriptValue object(exec->globalData(), exec->argument(0));
         ScriptValue hints(exec->globalData(), exec->argument(1));
-        impl()->inspectImpl(objectId.toInspectorValue(exec), hints.toInspectorValue(exec));
+        impl()->inspectImpl(object.toInspectorValue(exec), hints.toInspectorValue(exec));
     }
     return jsUndefined();
 }
@@ -124,7 +171,7 @@ JSValue JSInjectedScriptHost::databaseId(ExecState* exec)
 {
     if (exec->argumentCount() < 1)
         return jsUndefined();
-#if ENABLE(DATABASE)
+#if ENABLE(SQL_DATABASE)
     Database* database = toDatabase(exec->argument(0));
     if (database)
         return jsNumber(impl()->databaseIdImpl(database));
