@@ -47,6 +47,7 @@
 #include "RenderObject.h"
 #include "Settings.h"
 #include "SimpleFontData.h"
+#include "WebKitFontFamilyNames.h"
 #include <wtf/text/AtomicString.h>
 
 #if ENABLE(SVG)
@@ -245,13 +246,14 @@ void CSSFontSelector::addFontFaceRule(const CSSFontFaceRule* fontFaceRule)
             Settings* settings = m_document ? m_document->frame() ? m_document->frame()->settings() : 0 : 0;
             bool allowDownloading = foundSVGFont || (settings && settings->downloadableBinaryFontsEnabled());
             if (allowDownloading && item->isSupportedFormat() && m_document) {
-                CachedFont* cachedFont = m_document->cachedResourceLoader()->requestFont(item->resource());
+                ResourceRequest request(m_document->completeURL(item->resource()));
+                CachedFont* cachedFont = m_document->cachedResourceLoader()->requestFont(request);
                 if (cachedFont) {
+                    source = new CSSFontFaceSource(item->resource(), cachedFont);
 #if ENABLE(SVG_FONTS)
                     if (foundSVGFont)
-                        cachedFont->setSVGFont(true);
+                        source->setHasExternalSVGFont(true);
 #endif
-                    source = new CSSFontFaceSource(item->resource(), cachedFont);
                 }
             }
         } else {
@@ -295,19 +297,22 @@ void CSSFontSelector::addFontFaceRule(const CSSFontFaceRule* fontFaceRule)
             String familyName;
             switch (item->getIdent()) {
                 case CSSValueSerif:
-                    familyName = "-webkit-serif";
+                    familyName = serifFamily;
                     break;
                 case CSSValueSansSerif:
-                    familyName = "-webkit-sans-serif";
+                    familyName = sansSerifFamily;
                     break;
                 case CSSValueCursive:
-                    familyName = "-webkit-cursive";
+                    familyName = cursiveFamily;
                     break;
                 case CSSValueFantasy:
-                    familyName = "-webkit-fantasy";
+                    familyName = fantasyFamily;
                     break;
                 case CSSValueMonospace:
-                    familyName = "-webkit-monospace";
+                    familyName = monospaceFamily;
+                    break;
+                case CSSValueWebkitPictograph:
+                    familyName = pictographFamily;
                     break;
                 default:
                     break;
@@ -378,6 +383,16 @@ void CSSFontSelector::fontCacheInvalidated()
     dispatchInvalidationCallbacks();
 }
 
+void CSSFontSelector::retireCustomFont(FontData* fontData)
+{
+    if (m_document)
+        m_document->retireCustomFont(fontData);
+    else {
+        GlyphPageTreeNode::pruneTreeCustomFontData(fontData);
+        delete fontData;
+    }
+}
+
 static FontData* fontDataForGenericFamily(Document* document, const FontDescription& fontDescription, const AtomicString& familyName)
 {
     if (!document || !document->frame())
@@ -386,20 +401,24 @@ static FontData* fontDataForGenericFamily(Document* document, const FontDescript
     const Settings* settings = document->frame()->settings();
     if (!settings)
         return 0;
-    
+
     AtomicString genericFamily;
-    if (familyName == "-webkit-serif")
-        genericFamily = settings->serifFontFamily();
-    else if (familyName == "-webkit-sans-serif")
-        genericFamily = settings->sansSerifFontFamily();
-    else if (familyName == "-webkit-cursive")
-        genericFamily = settings->cursiveFontFamily();
-    else if (familyName == "-webkit-fantasy")
-        genericFamily = settings->fantasyFontFamily();
-    else if (familyName == "-webkit-monospace")
-        genericFamily = settings->fixedFontFamily();
-    else if (familyName == "-webkit-standard")
-        genericFamily = settings->standardFontFamily();
+    UScriptCode script = fontDescription.script();
+
+    if (familyName == serifFamily)
+         genericFamily = settings->serifFontFamily(script);
+    else if (familyName == sansSerifFamily)
+         genericFamily = settings->sansSerifFontFamily(script);
+    else if (familyName == cursiveFamily)
+         genericFamily = settings->cursiveFontFamily(script);
+    else if (familyName == fantasyFamily)
+         genericFamily = settings->fantasyFontFamily(script);
+    else if (familyName == monospaceFamily)
+         genericFamily = settings->fixedFontFamily(script);
+    else if (familyName == pictographFamily)
+         genericFamily = settings->pictographFontFamily(script);
+    else if (familyName == standardFamily)
+         genericFamily = settings->standardFontFamily(script);
 
     if (!genericFamily.isEmpty())
         return fontCache()->getCachedFontData(fontDescription, genericFamily);
@@ -494,6 +513,8 @@ FontData* CSSFontSelector::getFontData(const FontDescription& fontDescription, c
     if (m_fontFaces.isEmpty()) {
         if (familyName.startsWith("-webkit-"))
             return fontDataForGenericFamily(m_document, fontDescription, familyName);
+        if (fontDescription.genericFamily() == FontDescription::StandardFamily && !fontDescription.isSpecifiedFont())
+            return fontDataForGenericFamily(m_document, fontDescription, "-webkit-standard");
         return 0;
     }
 
@@ -504,6 +525,8 @@ FontData* CSSFontSelector::getFontData(const FontDescription& fontDescription, c
     if (!familyFontFaces || familyFontFaces->isEmpty()) {
         // If we were handed a generic family, but there was no match, go ahead and return the correct font based off our
         // settings.
+        if (fontDescription.genericFamily() == FontDescription::StandardFamily && !fontDescription.isSpecifiedFont())
+            return fontDataForGenericFamily(m_document, fontDescription, "-webkit-standard");
         return fontDataForGenericFamily(m_document, fontDescription, familyName);
     }
 
