@@ -35,7 +35,7 @@
 #include "ExceptionCode.h"
 #include "InspectorInstrumentation.h"
 #include "StyledElement.h"
-#include <wtf/text/StringConcatenate.h>
+#include <wtf/text/WTFString.h>
 
 using namespace std;
 
@@ -290,7 +290,7 @@ String CSSMutableStyleDeclaration::borderSpacingValue(const int properties[2]) c
     String verticalValueCSSText = verticalValue->cssText();
     if (horizontalValueCSSText == verticalValueCSSText)
         return horizontalValueCSSText;
-    return makeString(horizontalValueCSSText, ' ', verticalValueCSSText);
+    return horizontalValueCSSText + ' ' + verticalValueCSSText;
 }
 
 String CSSMutableStyleDeclaration::get4Values(const int* properties) const
@@ -571,8 +571,7 @@ bool CSSMutableStyleDeclaration::setProperty(int propertyID, const String& value
 
     // When replacing an existing property value, this moves the property to the end of the list.
     // Firefox preserves the position, and MSIE moves the property to the beginning.
-    CSSParser parser(useStrictParsing());
-    bool success = parser.parseValue(this, propertyID, value, important);
+    bool success = CSSParser::parseValue(this, propertyID, value, important, useStrictParsing());
     if (!success) {
         // CSS DOM requires raising SYNTAX_ERR here, but this is too dangerous for compatibility,
         // see <http://bugs.webkit.org/show_bug.cgi?id=7296>.
@@ -605,6 +604,15 @@ bool CSSMutableStyleDeclaration::setProperty(int propertyID, int value, bool imp
     return true;
 }
 
+bool CSSMutableStyleDeclaration::setProperty(int propertyID, double value, CSSPrimitiveValue::UnitTypes unit, bool important, bool notifyChanged)
+{
+    CSSProperty property(propertyID, CSSPrimitiveValue::create(value, unit), important);
+    setPropertyInternal(property);
+    if (notifyChanged)
+        setNeedsStyleRecalc();
+    return true;
+}
+
 void CSSMutableStyleDeclaration::setStringProperty(int propertyId, const String &value, CSSPrimitiveValue::UnitTypes type, bool important)
 {
     ASSERT(!m_iteratorCount);
@@ -634,17 +642,11 @@ void CSSMutableStyleDeclaration::parseDeclaration(const String& styleDeclaration
 void CSSMutableStyleDeclaration::addParsedProperties(const CSSProperty* const* properties, int numProperties)
 {
     ASSERT(!m_iteratorCount);
-    
+
     m_properties.reserveCapacity(numProperties);
-    
-    for (int i = 0; i < numProperties; ++i) {
-        // Only add properties that have no !important counterpart present
-        if (!getPropertyPriority(properties[i]->id()) || properties[i]->isImportant()) {
-            removeProperty(properties[i]->id(), false);
-            ASSERT(properties[i]);
-            m_properties.append(*properties[i]);
-        }
-    }
+    for (int i = 0; i < numProperties; ++i)
+        addParsedProperty(*properties[i]);
+
     // FIXME: This probably should have a call to setNeedsStyleRecalc() if something changed. We may also wish to add
     // a notifyChanged argument to this function to follow the model of other functions in this class.
 }
@@ -653,7 +655,11 @@ void CSSMutableStyleDeclaration::addParsedProperty(const CSSProperty& property)
 {
     ASSERT(!m_iteratorCount);
 
-    setPropertyInternal(property);
+    // Only add properties that have no !important counterpart present
+    if (!getPropertyPriority(property.id()) || property.isImportant()) {
+        removeProperty(property.id(), false);
+        m_properties.append(property);
+    }
 }
 
 void CSSMutableStyleDeclaration::setLengthProperty(int propertyId, const String& value, bool important, bool /*multiLength*/)
@@ -796,6 +802,9 @@ static const int blockProperties[] = {
     CSSPropertyPageBreakAfter,
     CSSPropertyPageBreakBefore,
     CSSPropertyPageBreakInside,
+    CSSPropertyWebkitRegionBreakAfter,
+    CSSPropertyWebkitRegionBreakBefore,
+    CSSPropertyWebkitRegionBreakInside,
     CSSPropertyTextAlign,
     CSSPropertyTextIndent,
     CSSPropertyWidows
