@@ -33,6 +33,7 @@
 #include "ColorSpace.h"
 #include "CSSPrimitiveValue.h"
 #include "CSSValueKeywords.h"
+#include "FontDescription.h"
 #include "FontSmoothingMode.h"
 #include "GraphicsTypes.h"
 #include "Path.h"
@@ -42,9 +43,97 @@
 #include "TextOrientation.h"
 #include "TextRenderingMode.h"
 #include "ThemeTypes.h"
+#include "UnicodeBidi.h"
+
+#include <wtf/MathExtras.h>
 
 namespace WebCore {
 
+template<> inline CSSPrimitiveValue::CSSPrimitiveValue(short i)
+    : m_type(CSS_NUMBER)
+    , m_hasCachedCSSText(false)
+{
+    m_value.num = static_cast<double>(i);
+}
+
+template<> inline CSSPrimitiveValue::operator short() const
+{
+    if (m_type == CSS_NUMBER)
+        return clampTo<short>(m_value.num);
+
+    ASSERT_NOT_REACHED();
+    return 0;
+}
+
+template<> inline CSSPrimitiveValue::CSSPrimitiveValue(unsigned short i)
+    : m_type(CSS_NUMBER)
+    , m_hasCachedCSSText(false)
+{
+    m_value.num = static_cast<double>(i);
+}
+
+template<> inline CSSPrimitiveValue::operator unsigned short() const
+{
+    if (m_type == CSS_NUMBER)
+        return clampTo<unsigned short>(m_value.num);
+
+    ASSERT_NOT_REACHED();
+    return 0;
+}
+
+// FIXME: Temp build fix for Symbian. 
+//Symbian treats unsigned short as int and below specilization contructor becomes duplicate incase of symbian.
+#if !OS(SYMBIAN)
+template<> inline CSSPrimitiveValue::CSSPrimitiveValue(int i)
+    : m_type(CSS_NUMBER)
+    , m_hasCachedCSSText(false)
+{
+    m_value.num = static_cast<double>(i);
+}
+
+template<> inline CSSPrimitiveValue::operator int() const
+{
+    if (m_type == CSS_NUMBER)
+        return clampTo<int>(m_value.num);
+
+    ASSERT_NOT_REACHED();
+    return 0;
+}
+#endif
+
+template<> inline CSSPrimitiveValue::CSSPrimitiveValue(unsigned i)
+    : m_type(CSS_NUMBER)
+    , m_hasCachedCSSText(false)
+{
+    m_value.num = static_cast<double>(i);
+}
+
+template<> inline CSSPrimitiveValue::operator unsigned() const
+{
+    if (m_type == CSS_NUMBER)
+        return clampTo<unsigned>(m_value.num);
+
+    ASSERT_NOT_REACHED();
+    return 0;
+}
+
+
+template<> inline CSSPrimitiveValue::CSSPrimitiveValue(float i)
+    : m_type(CSS_NUMBER)
+    , m_hasCachedCSSText(false)
+{
+    m_value.num = static_cast<double>(i);
+}
+
+template<> inline CSSPrimitiveValue::operator float() const
+{
+    if (m_type == CSS_NUMBER)
+        return clampTo<float>(m_value.num);
+
+    ASSERT_NOT_REACHED();
+    return 0.0f;
+}
+
 template<> inline CSSPrimitiveValue::CSSPrimitiveValue(EBorderStyle e)
     : m_type(CSS_IDENT)
     , m_hasCachedCSSText(false)
@@ -85,9 +174,18 @@ template<> inline CSSPrimitiveValue::CSSPrimitiveValue(EBorderStyle e)
 
 template<> inline CSSPrimitiveValue::operator EBorderStyle() const
 {
+    if (m_value.ident == CSSValueAuto) // Valid for CSS outline-style
+        return DOTTED;
     return (EBorderStyle)(m_value.ident - CSSValueNone);
 }
 
+template<> inline CSSPrimitiveValue::operator OutlineIsAuto() const
+{
+    if (m_value.ident == CSSValueAuto)
+        return AUTO_ON;
+    return AUTO_OFF;
+}
+
 template<> inline CSSPrimitiveValue::CSSPrimitiveValue(CompositeOperator e)
     : m_type(CSS_IDENT)
     , m_hasCachedCSSText(false)
@@ -129,9 +227,6 @@ template<> inline CSSPrimitiveValue::CSSPrimitiveValue(CompositeOperator e)
         case CompositePlusDarker:
             m_value.ident = CSSValuePlusDarker;
             break;
-        case CompositeHighlight:
-            m_value.ident = CSSValueHighlight;
-            break;
         case CompositePlusLighter:
             m_value.ident = CSSValuePlusLighter;
             break;
@@ -165,8 +260,6 @@ template<> inline CSSPrimitiveValue::operator CompositeOperator() const
             return CompositeXOR;
         case CSSValuePlusDarker:
             return CompositePlusDarker;
-        case CSSValueHighlight:
-            return CompositeHighlight;
         case CSSValuePlusLighter:
             return CompositePlusLighter;
         default:
@@ -299,9 +392,6 @@ template<> inline CSSPrimitiveValue::CSSPrimitiveValue(ControlPart e)
         case RatingLevelIndicatorPart:
             m_value.ident = CSSValueRatingLevelIndicator;
             break;
-        case OuterSpinButtonPart:
-            m_value.ident = CSSValueOuterSpinButton;
-            break;
         case ProgressBarPart:
 #if ENABLE(PROGRESS_TAG)
             m_value.ident = CSSValueProgressBar;
@@ -367,6 +457,34 @@ template<> inline CSSPrimitiveValue::operator ControlPart() const
         return ControlPart(m_value.ident - CSSValueCheckbox + 1);
 }
 
+template<> inline CSSPrimitiveValue::CSSPrimitiveValue(EBackfaceVisibility e)
+    : m_type(CSS_IDENT)
+    , m_hasCachedCSSText(false)
+{
+    switch (e) {
+    case BackfaceVisibilityVisible:
+        m_value.ident = CSSValueVisible;
+        break;
+    case BackfaceVisibilityHidden:
+        m_value.ident = CSSValueHidden;
+        break;
+    }
+}
+
+template<> inline CSSPrimitiveValue::operator EBackfaceVisibility() const
+{
+    switch (m_value.ident) {
+    case CSSValueVisible:
+        return BackfaceVisibilityVisible;
+    case CSSValueHidden:
+        return BackfaceVisibilityHidden;
+    default:
+        ASSERT_NOT_REACHED();
+        return BackfaceVisibilityHidden;
+    }
+}
+
+
 template<> inline CSSPrimitiveValue::CSSPrimitiveValue(EFillAttachment e)
     : m_type(CSS_IDENT)
     , m_hasCachedCSSText(false)
@@ -524,6 +642,33 @@ template<> inline CSSPrimitiveValue::operator EBoxAlignment() const
     }
 }
 
+template<> inline CSSPrimitiveValue::CSSPrimitiveValue(EBoxSizing e)
+    : m_type(CSS_IDENT)
+    , m_hasCachedCSSText(false)
+{
+    switch (e) {
+    case BORDER_BOX:
+        m_value.ident = CSSValueBorderBox;
+        break;
+    case CONTENT_BOX:
+        m_value.ident = CSSValueContentBox;
+        break;
+    }
+}
+
+template<> inline CSSPrimitiveValue::operator EBoxSizing() const
+{
+    switch (m_value.ident) {
+    case CSSValueBorderBox:
+        return BORDER_BOX;
+    case CSSValueContentBox:
+        return CONTENT_BOX;
+    default:
+        ASSERT_NOT_REACHED();
+        return BORDER_BOX;
+    }
+}
+
 template<> inline CSSPrimitiveValue::CSSPrimitiveValue(EBoxDirection e)
     : m_type(CSS_IDENT)
     , m_hasCachedCSSText(false)
@@ -870,6 +1015,14 @@ template<> inline CSSPrimitiveValue::CSSPrimitiveValue(EDisplay e)
         case INLINE_BOX:
             m_value.ident = CSSValueWebkitInlineBox;
             break;
+#if ENABLE(CSS3_FLEXBOX)
+        case FLEXBOX:
+            m_value.ident = CSSValueWebkitFlexbox;
+            break;
+        case INLINE_FLEXBOX:
+            m_value.ident = CSSValueWebkitInlineFlexbox;
+            break;
+#endif
         case NONE:
             m_value.ident = CSSValueNone;
             break;
@@ -910,20 +1063,179 @@ template<> inline CSSPrimitiveValue::operator EEmptyCell() const
     }
 }
 
+template<> inline CSSPrimitiveValue::CSSPrimitiveValue(EFlexAlign e)
+    : m_type(CSS_IDENT)
+    , m_hasCachedCSSText(false)
+{
+    switch (e) {
+    case AlignStart:
+        m_value.ident = CSSValueStart;
+        break;
+    case AlignEnd:
+        m_value.ident = CSSValueEnd;
+        break;
+    case AlignCenter:
+        m_value.ident = CSSValueCenter;
+        break;
+    case AlignStretch:
+        m_value.ident = CSSValueStretch;
+        break;
+    case AlignBaseline:
+        m_value.ident = CSSValueBaseline;
+        break;
+    }
+}
+
+template<> inline CSSPrimitiveValue::operator EFlexAlign() const
+{
+    switch (m_value.ident) {
+    case CSSValueStart:
+        return AlignStart;
+    case CSSValueEnd:
+        return AlignEnd;
+    case CSSValueCenter:
+        return AlignCenter;
+    case CSSValueStretch:
+        return AlignStretch;
+    case CSSValueBaseline:
+        return AlignBaseline;
+    default:
+        ASSERT_NOT_REACHED();
+        return AlignStart;
+    }
+}
+
+template<> inline CSSPrimitiveValue::CSSPrimitiveValue(EFlexPack e)
+    : m_type(CSS_IDENT)
+    , m_hasCachedCSSText(false)
+{
+    switch (e) {
+    case PackStart:
+        m_value.ident = CSSValueStart;
+        break;
+    case PackEnd:
+        m_value.ident = CSSValueEnd;
+        break;
+    case PackCenter:
+        m_value.ident = CSSValueCenter;
+        break;
+    case PackJustify:
+        m_value.ident = CSSValueJustify;
+        break;
+    }
+}
+
+template<> inline CSSPrimitiveValue::operator EFlexPack() const
+{
+    switch (m_value.ident) {
+    case CSSValueStart:
+        return PackStart;
+    case CSSValueEnd:
+        return PackEnd;
+    case CSSValueCenter:
+        return PackCenter;
+    case CSSValueJustify:
+        return PackJustify;
+    default:
+        ASSERT_NOT_REACHED();
+        return PackStart;
+    }
+}
+
+template<> inline CSSPrimitiveValue::CSSPrimitiveValue(EFlexFlow e)
+    : m_type(CSS_IDENT)
+    , m_hasCachedCSSText(false)
+{
+    switch (e) {
+    case FlowRow:
+        m_value.ident = CSSValueRow;
+        break;
+    case FlowRowReverse:
+        m_value.ident = CSSValueRowReverse;
+        break;
+    case FlowColumn:
+        m_value.ident = CSSValueColumn;
+        break;
+    case FlowColumnReverse:
+        m_value.ident = CSSValueColumnReverse;
+        break;
+    case FlowHorizontal:
+        m_value.ident = CSSValueHorizontal;
+        break;
+    case FlowHorizontalReverse:
+        m_value.ident = CSSValueHorizontalReverse;
+        break;
+    case FlowHorizontalLeftToRight:
+        m_value.ident = CSSValueHorizontalLtr;
+        break;
+    case FlowHorizontalRightToLeft:
+        m_value.ident = CSSValueHorizontalRtl;
+        break;
+    case FlowVertical:
+        m_value.ident = CSSValueVertical;
+        break;
+    case FlowVerticalReverse:
+        m_value.ident = CSSValueVerticalReverse;
+        break;
+    case FlowVerticalTopToBottom:
+        m_value.ident = CSSValueVerticalTtb;
+        break;
+    case FlowVerticalBottomToTop:
+        m_value.ident = CSSValueVerticalBtt;
+        break;
+    }
+}
+
+template<> inline CSSPrimitiveValue::operator EFlexFlow() const
+{
+    switch (m_value.ident) {
+    case CSSValueRow:
+        return FlowRow;
+    case CSSValueRowReverse:
+        return FlowRowReverse;
+    case CSSValueColumn:
+        return FlowColumn;
+    case CSSValueColumnReverse:
+        return FlowColumnReverse;
+    case CSSValueHorizontal:
+        return FlowHorizontal;
+    case CSSValueHorizontalReverse:
+        return FlowHorizontalReverse;
+    case CSSValueHorizontalLtr:
+        return FlowHorizontalLeftToRight;
+    case CSSValueHorizontalRtl:
+        return FlowHorizontalRightToLeft;
+    case CSSValueVertical:
+        return FlowVertical;
+    case CSSValueVerticalReverse:
+        return FlowVerticalReverse;
+    case CSSValueVerticalTtb:
+        return FlowVerticalTopToBottom;
+    case CSSValueVerticalBtt:
+        return FlowVerticalBottomToTop;
+    default:
+        ASSERT_NOT_REACHED();
+        return FlowRow;
+    }
+}
+
 template<> inline CSSPrimitiveValue::CSSPrimitiveValue(EFloat e)
     : m_type(CSS_IDENT)
     , m_hasCachedCSSText(false)
 {
     switch (e) {
-        case FNONE:
+        case NoFloat:
             m_value.ident = CSSValueNone;
             break;
-        case FLEFT:
+        case LeftFloat:
             m_value.ident = CSSValueLeft;
             break;
-        case FRIGHT:
+        case RightFloat:
             m_value.ident = CSSValueRight;
             break;
+        case PositionedFloat:
+            m_value.ident = CSSValueWebkitPositioned;
+            break;
     }
 }
 
@@ -931,15 +1243,17 @@ template<> inline CSSPrimitiveValue::operator EFloat() const
 {
     switch (m_value.ident) {
         case CSSValueLeft:
-            return FLEFT;
+            return LeftFloat;
         case CSSValueRight:
-            return FRIGHT;
+            return RightFloat;
         case CSSValueNone:
         case CSSValueCenter:  // Non-standard CSS value
-            return FNONE;
+            return NoFloat;
+        case CSSValueWebkitPositioned:
+            return PositionedFloat;
         default:
             ASSERT_NOT_REACHED();
-            return FNONE;
+            return NoFloat;
     }
 }
 
@@ -1319,6 +1633,33 @@ template<> inline CSSPrimitiveValue::operator EMarqueeBehavior() const
     }
 }
 
+template<> inline CSSPrimitiveValue::CSSPrimitiveValue(RegionOverflow e)
+    : m_type(CSS_IDENT)
+    , m_hasCachedCSSText(false)
+{
+    switch (e) {
+    case AutoRegionOverflow:
+        m_value.ident = CSSValueAuto;
+        break;
+    case BreakRegionOverflow:
+        m_value.ident = CSSValueWebkitBreak;
+        break;
+    }
+}
+
+template<> inline CSSPrimitiveValue::operator RegionOverflow() const
+{
+    switch (m_value.ident) {
+    case CSSValueAuto:
+        return AutoRegionOverflow;
+    case CSSValueWebkitBreak:
+        return BreakRegionOverflow;
+    default:
+        ASSERT_NOT_REACHED();
+        return AutoRegionOverflow;
+    }
+}
+
 template<> inline CSSPrimitiveValue::CSSPrimitiveValue(EMarqueeDirection e)
     : m_type(CSS_IDENT)
     , m_hasCachedCSSText(false)
@@ -1617,30 +1958,36 @@ template<> inline CSSPrimitiveValue::CSSPrimitiveValue(ETextAlign e)
     , m_hasCachedCSSText(false)
 {
     switch (e) {
-        case TAAUTO:
-            m_value.ident = CSSValueWebkitAuto;
-            break;
-        case LEFT:
-            m_value.ident = CSSValueLeft;
-            break;
-        case RIGHT:
-            m_value.ident = CSSValueRight;
-            break;
-        case CENTER:
-            m_value.ident = CSSValueCenter;
-            break;
-        case JUSTIFY:
-            m_value.ident = CSSValueJustify;
-            break;
-        case WEBKIT_LEFT:
-            m_value.ident = CSSValueWebkitLeft;
-            break;
-        case WEBKIT_RIGHT:
-            m_value.ident = CSSValueWebkitRight;
-            break;
-        case WEBKIT_CENTER:
-            m_value.ident = CSSValueWebkitCenter;
-            break;
+    case TAAUTO:
+        m_value.ident = CSSValueWebkitAuto;
+        break;
+    case TASTART:
+        m_value.ident = CSSValueStart;
+        break;
+    case TAEND:
+        m_value.ident = CSSValueEnd;
+        break;
+    case LEFT:
+        m_value.ident = CSSValueLeft;
+        break;
+    case RIGHT:
+        m_value.ident = CSSValueRight;
+        break;
+    case CENTER:
+        m_value.ident = CSSValueCenter;
+        break;
+    case JUSTIFY:
+        m_value.ident = CSSValueJustify;
+        break;
+    case WEBKIT_LEFT:
+        m_value.ident = CSSValueWebkitLeft;
+        break;
+    case WEBKIT_RIGHT:
+        m_value.ident = CSSValueWebkitRight;
+        break;
+    case WEBKIT_CENTER:
+        m_value.ident = CSSValueWebkitCenter;
+        break;
     }
 }
 
@@ -1648,9 +1995,9 @@ template<> inline CSSPrimitiveValue::operator ETextAlign() const
 {
     switch (m_value.ident) {
         case CSSValueStart:
+            return TASTART;
         case CSSValueEnd:
-            ASSERT_NOT_REACHED(); // Depends on direction, thus should be handled by the caller.
-            return LEFT;
+            return TAEND;
         default:
             return static_cast<ETextAlign>(m_value.ident - CSSValueWebkitAuto);
     }
@@ -1735,30 +2082,40 @@ template<> inline CSSPrimitiveValue::CSSPrimitiveValue(EUnicodeBidi e)
     , m_hasCachedCSSText(false)
 {
     switch (e) {
-        case UBNormal:
-            m_value.ident = CSSValueNormal;
-            break;
-        case Embed:
-            m_value.ident = CSSValueEmbed;
-            break;
-        case Override:
-            m_value.ident = CSSValueBidiOverride;
-            break;
+    case UBNormal:
+        m_value.ident = CSSValueNormal;
+        break;
+    case Embed:
+        m_value.ident = CSSValueEmbed;
+        break;
+    case Override:
+        m_value.ident = CSSValueBidiOverride;
+        break;
+    case Isolate:
+        m_value.ident = CSSValueWebkitIsolate;
+        break;
+    case Plaintext:
+        m_value.ident = CSSValueWebkitPlaintext;
+        break;
     }
 }
 
 template<> inline CSSPrimitiveValue::operator EUnicodeBidi() const
 {
     switch (m_value.ident) {
-        case CSSValueNormal:
-            return UBNormal; 
-        case CSSValueEmbed:
-            return Embed; 
-        case CSSValueBidiOverride:
-            return Override;
-        default:
-            ASSERT_NOT_REACHED();
-            return UBNormal;
+    case CSSValueNormal:
+        return UBNormal;
+    case CSSValueEmbed:
+        return Embed;
+    case CSSValueBidiOverride:
+        return Override;
+    case CSSValueWebkitIsolate:
+        return Isolate;
+    case CSSValueWebkitPlaintext:
+        return Plaintext;
+    default:
+        ASSERT_NOT_REACHED();
+        return UBNormal;
     }
 }
 
@@ -1845,6 +2202,70 @@ template<> inline CSSPrimitiveValue::operator EUserSelect() const
     }
 }
 
+template<> inline CSSPrimitiveValue::CSSPrimitiveValue(EVerticalAlign a)
+    : m_type(CSS_IDENT)
+    , m_hasCachedCSSText(false)
+{
+    switch (a) {
+    case TOP:
+        m_value.ident = CSSValueTop;
+        break;
+    case BOTTOM:
+        m_value.ident = CSSValueBottom;
+        break;
+    case MIDDLE:
+        m_value.ident = CSSValueMiddle;
+        break;
+    case BASELINE:
+        m_value.ident = CSSValueBaseline;
+        break;
+    case TEXT_BOTTOM:
+        m_value.ident = CSSValueTextBottom;
+        break;
+    case TEXT_TOP:
+        m_value.ident = CSSValueTextTop;
+        break;
+    case SUB:
+        m_value.ident = CSSValueSub;
+        break;
+    case SUPER:
+        m_value.ident = CSSValueSuper;
+        break;
+    case BASELINE_MIDDLE:
+        m_value.ident = CSSValueWebkitBaselineMiddle;
+        break;
+    case LENGTH:
+        m_value.ident = CSSValueInvalid;
+    }
+}
+
+template<> inline CSSPrimitiveValue::operator EVerticalAlign() const
+{
+    switch (m_value.ident) {
+    case CSSValueTop:
+        return TOP;
+    case CSSValueBottom:
+        return BOTTOM;
+    case CSSValueMiddle:
+        return MIDDLE;
+    case CSSValueBaseline:
+        return BASELINE;
+    case CSSValueTextBottom:
+        return TEXT_BOTTOM;
+    case CSSValueTextTop:
+        return TEXT_TOP;
+    case CSSValueSub:
+        return SUB;
+    case CSSValueSuper:
+        return SUPER;
+    case CSSValueWebkitBaselineMiddle:
+        return BASELINE_MIDDLE;
+    default:
+        ASSERT_NOT_REACHED();
+        return TOP;
+    }
+}
+
 template<> inline CSSPrimitiveValue::CSSPrimitiveValue(EVisibility e)
     : m_type(CSS_IDENT)
     , m_hasCachedCSSText(false)
@@ -2101,6 +2522,33 @@ template<> inline CSSPrimitiveValue::operator TextEmphasisPosition() const
     }
 }
 
+template<> inline CSSPrimitiveValue::CSSPrimitiveValue(TextOverflow overflow)
+    : m_type(CSS_IDENT)
+    , m_hasCachedCSSText(false)
+{
+    switch (overflow) {
+    case TextOverflowClip:
+        m_value.ident = CSSValueClip;
+        break;
+    case TextOverflowEllipsis:
+        m_value.ident = CSSValueEllipsis;
+        break;
+    }
+}
+
+template<> inline CSSPrimitiveValue::operator TextOverflow() const
+{
+    switch (m_value.ident) {
+    case CSSValueClip:
+        return TextOverflowClip;
+    case CSSValueEllipsis:
+        return TextOverflowEllipsis;
+    default:
+        ASSERT_NOT_REACHED();
+        return TextOverflowClip;
+    }
+}
+
 template<> inline CSSPrimitiveValue::CSSPrimitiveValue(TextEmphasisFill fill)
     : m_type(CSS_IDENT)
     , m_hasCachedCSSText(false)
@@ -2312,6 +2760,135 @@ template<> inline CSSPrimitiveValue::operator FontSmoothingMode() const
     return AutoSmoothing;
 }
 
+template<> inline CSSPrimitiveValue::CSSPrimitiveValue(FontWeight weight)
+    : m_type(CSS_IDENT)
+    , m_hasCachedCSSText(false)
+{
+    switch (weight) {
+    case FontWeight900:
+        m_value.ident = CSSValue900;
+        return;
+    case FontWeight800:
+        m_value.ident = CSSValue800;
+        return;
+    case FontWeight700:
+        m_value.ident = CSSValue700;
+        return;
+    case FontWeight600:
+        m_value.ident = CSSValue600;
+        return;
+    case FontWeight500:
+        m_value.ident = CSSValue500;
+        return;
+    case FontWeight400:
+        m_value.ident = CSSValue400;
+        return;
+    case FontWeight300:
+        m_value.ident = CSSValue300;
+        return;
+    case FontWeight200:
+        m_value.ident = CSSValue200;
+        return;
+    case FontWeight100:
+        m_value.ident = CSSValue100;
+        return;
+    }
+
+    ASSERT_NOT_REACHED();
+    m_value.ident = CSSValueNormal;
+}
+
+template<> inline CSSPrimitiveValue::operator FontWeight() const
+{
+    switch (m_value.ident) {
+    case CSSValueBold:
+        return FontWeightBold;
+    case CSSValueNormal:
+        return FontWeightNormal;
+    case CSSValue900:
+        return FontWeight900;
+    case CSSValue800:
+        return FontWeight800;
+    case CSSValue700:
+        return FontWeight700;
+    case CSSValue600:
+        return FontWeight600;
+    case CSSValue500:
+        return FontWeight500;
+    case CSSValue400:
+        return FontWeight400;
+    case CSSValue300:
+        return FontWeight300;
+    case CSSValue200:
+        return FontWeight200;
+    case CSSValue100:
+        return FontWeight100;
+    }
+
+    ASSERT_NOT_REACHED();
+    return FontWeightNormal;
+}
+
+template<> inline CSSPrimitiveValue::CSSPrimitiveValue(FontItalic italic)
+    : m_type(CSS_IDENT)
+    , m_hasCachedCSSText(false)
+{
+    switch (italic) {
+    case FontItalicOff:
+        m_value.ident = CSSValueNormal;
+        return;
+    case FontItalicOn:
+        m_value.ident = CSSValueItalic;
+        return;
+    }
+
+    ASSERT_NOT_REACHED();
+    m_value.ident = CSSValueNormal;
+}
+
+template<> inline CSSPrimitiveValue::operator FontItalic() const
+{
+    switch (m_value.ident) {
+    case CSSValueOblique:
+    // FIXME: oblique is the same as italic for the moment...
+    case CSSValueItalic:
+        return FontItalicOn;
+    case CSSValueNormal:
+        return FontItalicOff;
+    }
+    ASSERT_NOT_REACHED();
+    return FontItalicOff;
+}
+
+template<> inline CSSPrimitiveValue::CSSPrimitiveValue(FontSmallCaps smallCaps)
+    : m_type(CSS_IDENT)
+    , m_hasCachedCSSText(false)
+{
+    switch (smallCaps) {
+    case FontSmallCapsOff:
+        m_value.ident = CSSValueNormal;
+        return;
+    case FontSmallCapsOn:
+        m_value.ident = CSSValueSmallCaps;
+        return;
+    }
+
+    ASSERT_NOT_REACHED();
+    m_value.ident = CSSValueNormal;
+}
+
+template<> inline CSSPrimitiveValue::operator FontSmallCaps() const
+{
+    switch (m_value.ident) {
+    case CSSValueSmallCaps:
+        return FontSmallCapsOn;
+    case CSSValueNormal:
+        return FontSmallCapsOff;
+    }
+    ASSERT_NOT_REACHED();
+    return FontSmallCapsOff;
+}
+
 template<> inline CSSPrimitiveValue::CSSPrimitiveValue(TextRenderingMode e)
     : m_type(CSS_IDENT)
     , m_hasCachedCSSText(false)
@@ -2439,6 +3016,33 @@ template<> inline CSSPrimitiveValue::CSSPrimitiveValue(ESpeak e)
     }
 }
     
+template<> inline CSSPrimitiveValue::operator Order() const
+{
+    switch (m_value.ident) {
+    case CSSValueLogical:
+        return LogicalOrder;
+    case CSSValueVisual:
+        return VisualOrder;
+    default:
+        ASSERT_NOT_REACHED();
+        return LogicalOrder;
+    }
+}
+
+template<> inline CSSPrimitiveValue::CSSPrimitiveValue(Order e)
+    : m_type(CSS_IDENT)
+    , m_hasCachedCSSText(false)
+{
+    switch (e) {
+    case LogicalOrder:
+        m_value.ident = CSSValueLogical;
+        break;
+    case VisualOrder:
+        m_value.ident = CSSValueVisual;
+        break;
+    }
+}
+
 template<> inline CSSPrimitiveValue::operator ESpeak() const
 {
     switch (m_value.ident) {
@@ -2631,6 +3235,128 @@ template<> inline CSSPrimitiveValue::operator EAlignmentBaseline() const
     }
 }
 
+#endif
+
+template<> inline CSSPrimitiveValue::CSSPrimitiveValue(EBorderCollapse e)
+    : m_type(CSS_IDENT)
+    , m_hasCachedCSSText(false)
+{
+    switch (e) {
+    case BSEPARATE:
+        m_value.ident = CSSValueSeparate;
+        break;
+    case BCOLLAPSE:
+        m_value.ident = CSSValueCollapse;
+        break;
+    }
+}
+
+template<> inline CSSPrimitiveValue::operator EBorderCollapse() const
+{
+    switch (m_value.ident) {
+    case CSSValueSeparate:
+        return BSEPARATE;
+    case CSSValueCollapse:
+        return BCOLLAPSE;
+    default:
+        ASSERT_NOT_REACHED();
+        return BSEPARATE;
+    }
+}
+
+template<> inline CSSPrimitiveValue::CSSPrimitiveValue(EBorderFit e)
+    : m_type(CSS_IDENT)
+    , m_hasCachedCSSText(false)
+{
+    switch (e) {
+    case BorderFitBorder:
+        m_value.ident = CSSValueBorder;
+        break;
+    case BorderFitLines:
+        m_value.ident = CSSValueLines;
+        break;
+    }
+}
+
+template<> inline CSSPrimitiveValue::operator EBorderFit() const
+{
+    switch (m_value.ident) {
+    case CSSValueBorder:
+        return BorderFitBorder;
+    case CSSValueLines:
+        return BorderFitLines;
+    default:
+        ASSERT_NOT_REACHED();
+        return BorderFitLines;
+    }
+}
+
+template<> inline CSSPrimitiveValue::CSSPrimitiveValue(EImageRendering e)
+    : m_type(CSS_IDENT)
+    , m_hasCachedCSSText(false)
+{
+    switch (e) {
+    case ImageRenderingAuto:
+        m_value.ident = CSSValueAuto;
+        break;
+    case ImageRenderingOptimizeSpeed:
+        m_value.ident = CSSValueOptimizespeed;
+        break;
+    case ImageRenderingOptimizeQuality:
+        m_value.ident = CSSValueOptimizequality;
+        break;
+    case ImageRenderingOptimizeContrast:
+        m_value.ident = CSSValueWebkitOptimizeContrast;
+        break;
+    }
+}
+
+template<> inline CSSPrimitiveValue::operator EImageRendering() const
+{
+    switch (m_value.ident) {
+    case CSSValueAuto:
+        return ImageRenderingAuto;
+    case CSSValueOptimizespeed:
+        return ImageRenderingOptimizeSpeed;
+    case CSSValueOptimizequality:
+        return ImageRenderingOptimizeQuality;
+    case CSSValueWebkitOptimizeContrast:
+        return ImageRenderingOptimizeContrast;
+    default:
+        ASSERT_NOT_REACHED();
+        return ImageRenderingAuto;
+    }
+}
+
+template<> inline CSSPrimitiveValue::CSSPrimitiveValue(ETransformStyle3D e)
+    : m_type(CSS_IDENT)
+    , m_hasCachedCSSText(false)
+{
+    switch (e) {
+    case TransformStyle3DFlat:
+        m_value.ident = CSSValueFlat;
+        break;
+    case TransformStyle3DPreserve3D:
+        m_value.ident = CSSValuePreserve3d;
+        break;
+    }
+}
+
+template<> inline CSSPrimitiveValue::operator ETransformStyle3D() const
+{
+    switch (m_value.ident) {
+    case CSSValueFlat:
+        return TransformStyle3DFlat;
+    case CSSValuePreserve3d:
+        return TransformStyle3DPreserve3D;
+    default:
+        ASSERT_NOT_REACHED();
+        return TransformStyle3DFlat;
+    }
+}
+
+#if ENABLE(SVG)
+
 template<> inline CSSPrimitiveValue::CSSPrimitiveValue(EColorInterpolation e)
     : m_type(CSS_IDENT)
     , m_hasCachedCSSText(false)
@@ -2772,72 +3498,40 @@ template<> inline CSSPrimitiveValue::operator EDominantBaseline() const
     }
 }
 
-template<> inline CSSPrimitiveValue::CSSPrimitiveValue(EImageRendering e)
-    : m_type(CSS_IDENT)
-    , m_hasCachedCSSText(false)
-{
-    switch (e) {
-        case IR_AUTO:
-            m_value.ident = CSSValueAuto;
-            break;
-        case IR_OPTIMIZESPEED:
-            m_value.ident = CSSValueOptimizespeed;
-            break;
-        case IR_OPTIMIZEQUALITY:
-            m_value.ident = CSSValueOptimizequality;
-            break;
-    }
-}
-
-template<> inline CSSPrimitiveValue::operator EImageRendering() const
-{
-    switch (m_value.ident) {
-        case CSSValueAuto:
-            return IR_AUTO;
-        case CSSValueOptimizespeed:
-            return IR_OPTIMIZESPEED;
-        case CSSValueOptimizequality:
-            return IR_OPTIMIZEQUALITY;
-        default:
-            ASSERT_NOT_REACHED();
-            return IR_AUTO;
-    }
-}
-
 template<> inline CSSPrimitiveValue::CSSPrimitiveValue(EShapeRendering e)
     : m_type(CSS_IDENT)
     , m_hasCachedCSSText(false)
 {
     switch (e) {
-        case IR_AUTO:
-            m_value.ident = CSSValueAuto;
-            break;
-        case IR_OPTIMIZESPEED:
-            m_value.ident = CSSValueOptimizespeed;
-            break;
-        case SR_CRISPEDGES:
-            m_value.ident = CSSValueCrispedges;
-            break;
-        case SR_GEOMETRICPRECISION:
-            m_value.ident = CSSValueGeometricprecision;
-            break;
+    case SR_AUTO:
+        m_value.ident = CSSValueAuto;
+        break;
+    case SR_OPTIMIZESPEED:
+        m_value.ident = CSSValueOptimizespeed;
+        break;
+    case SR_CRISPEDGES:
+        m_value.ident = CSSValueCrispedges;
+        break;
+    case SR_GEOMETRICPRECISION:
+        m_value.ident = CSSValueGeometricprecision;
+        break;
     }
 }
 
 template<> inline CSSPrimitiveValue::operator EShapeRendering() const
 {
     switch (m_value.ident) {
-        case CSSValueAuto:
-            return SR_AUTO;
-        case CSSValueOptimizespeed:
-            return SR_OPTIMIZESPEED;
-        case CSSValueCrispedges:
-            return SR_CRISPEDGES;
-        case CSSValueGeometricprecision:
-            return SR_GEOMETRICPRECISION;
-        default:
-            ASSERT_NOT_REACHED();
-            return SR_AUTO;
+    case CSSValueAuto:
+        return SR_AUTO;
+    case CSSValueOptimizespeed:
+        return SR_OPTIMIZESPEED;
+    case CSSValueCrispedges:
+        return SR_CRISPEDGES;
+    case CSSValueGeometricprecision:
+        return SR_GEOMETRICPRECISION;
+    default:
+        ASSERT_NOT_REACHED();
+        return SR_AUTO;
     }
 }
 
