@@ -214,13 +214,6 @@ void CSSStyleSelector::applySVGProperty(int id, CSSValue* value)
                 svgstyle->setJoinStyle(*primitiveValue);
             break;
         }
-        case CSSPropertyImageRendering:
-        {
-            HANDLE_INHERIT_AND_INITIAL(imageRendering, ImageRendering)
-            if (primitiveValue)
-                svgstyle->setImageRendering(*primitiveValue);
-            break;
-        }
         case CSSPropertyShapeRendering:
         {
             HANDLE_INHERIT_AND_INITIAL(shapeRendering, ShapeRendering)
@@ -273,8 +266,10 @@ void CSSStyleSelector::applySVGProperty(int id, CSSValue* value)
         case CSSPropertyStrokeDasharray:
         {
             HANDLE_INHERIT_AND_INITIAL(strokeDashArray, StrokeDashArray)
-            if (!value->isValueList())
+            if (!value->isValueList()) {
+                svgstyle->setStrokeDashArray(SVGRenderStyle::initialStrokeDashArray());
                 break;
+            }
 
             CSSValueList* dashes = static_cast<CSSValueList*>(value);
 
@@ -366,7 +361,7 @@ void CSSStyleSelector::applySVGProperty(int id, CSSValue* value)
             else
                 return;
 
-            svgstyle->setMarkerStartResource(SVGURIReference::getTarget(s));
+            svgstyle->setMarkerStartResource(SVGURIReference::fragmentIdentifierFromIRIString(s, m_element->document()));
             break;
         }
         case CSSPropertyMarkerMid:
@@ -382,7 +377,7 @@ void CSSStyleSelector::applySVGProperty(int id, CSSValue* value)
             else
                 return;
 
-            svgstyle->setMarkerMidResource(SVGURIReference::getTarget(s));
+            svgstyle->setMarkerMidResource(SVGURIReference::fragmentIdentifierFromIRIString(s, m_element->document()));
             break;
         }
         case CSSPropertyMarkerEnd:
@@ -398,7 +393,7 @@ void CSSStyleSelector::applySVGProperty(int id, CSSValue* value)
             else
                 return;
 
-            svgstyle->setMarkerEndResource(SVGURIReference::getTarget(s));
+            svgstyle->setMarkerEndResource(SVGURIReference::fragmentIdentifierFromIRIString(s, m_element->document()));
             break;
         }
         case CSSPropertyStrokeLinecap:
@@ -437,7 +432,7 @@ void CSSStyleSelector::applySVGProperty(int id, CSSValue* value)
             else
                 return;
 
-            svgstyle->setFilterResource(SVGURIReference::getTarget(s));
+            svgstyle->setFilterResource(SVGURIReference::fragmentIdentifierFromIRIString(s, m_element->document()));
             break;
         }
         case CSSPropertyMask:
@@ -453,7 +448,7 @@ void CSSStyleSelector::applySVGProperty(int id, CSSValue* value)
             else
                 return;
             
-            svgstyle->setMaskerResource(SVGURIReference::getTarget(s));
+            svgstyle->setMaskerResource(SVGURIReference::fragmentIdentifierFromIRIString(s, m_element->document()));
             break;
         }
         case CSSPropertyClipPath:
@@ -469,7 +464,7 @@ void CSSStyleSelector::applySVGProperty(int id, CSSValue* value)
             else
                 return;
 
-            svgstyle->setClipperResource(SVGURIReference::getTarget(s));
+            svgstyle->setClipperResource(SVGURIReference::fragmentIdentifierFromIRIString(s, m_element->document()));
             break;
         }
         case CSSPropertyTextAnchor:
@@ -562,9 +557,9 @@ void CSSStyleSelector::applySVGProperty(int id, CSSValue* value)
             break;
         case CSSPropertyWebkitSvgShadow: {
             if (isInherit)
-                return svgstyle->setShadow(m_parentStyle->svgStyle()->shadow() ? new ShadowData(*m_parentStyle->svgStyle()->shadow()) : 0);
+                return svgstyle->setShadow(adoptPtr(m_parentStyle->svgStyle()->shadow() ? new ShadowData(*m_parentStyle->svgStyle()->shadow()) : 0));
             if (isInitial || primitiveValue) // initial | none
-                return svgstyle->setShadow(0);
+                return svgstyle->setShadow(nullptr);
 
             if (!value->isValueList())
                 return;
@@ -577,9 +572,9 @@ void CSSStyleSelector::applySVGProperty(int id, CSSValue* value)
             if (!firstValue->isShadowValue())
                 return;
             ShadowValue* item = static_cast<ShadowValue*>(firstValue);
-            int x = item->x->computeLengthInt(style(), m_rootElementStyle);
-            int y = item->y->computeLengthInt(style(), m_rootElementStyle);
-            int blur = item->blur ? item->blur->computeLengthInt(style(), m_rootElementStyle) : 0;
+            int x = item->x->computeLength<int>(style(), m_rootElementStyle);
+            int y = item->y->computeLength<int>(style(), m_rootElementStyle);
+            int blur = item->blur ? item->blur->computeLength<int>(style(), m_rootElementStyle) : 0;
             Color color;
             if (item->color)
                 color = getColorFromPrimitiveValue(item->color.get());
@@ -588,8 +583,8 @@ void CSSStyleSelector::applySVGProperty(int id, CSSValue* value)
             ASSERT(!item->spread);
             ASSERT(!item->style);
                 
-            ShadowData* shadowData = new ShadowData(x, y, blur, 0, Normal, false, color.isValid() ? color : Color::transparent);
-            svgstyle->setShadow(shadowData);
+            OwnPtr<ShadowData> shadowData = adoptPtr(new ShadowData(x, y, blur, 0, Normal, false, color.isValid() ? color : Color::transparent));
+            svgstyle->setShadow(shadowData.release());
             return;
         }
         case CSSPropertyVectorEffect: {
