@@ -44,16 +44,20 @@
 #include "Frame.h"
 #include "FrameView.h"
 #include "HTMLElement.h"
+#include "HTMLFrameOwnerElement.h"
 #include "HTMLNames.h"
 #include "HTMLParserIdioms.h"
 #include "InspectorInstrumentation.h"
 #include "NodeList.h"
 #include "NodeRenderStyle.h"
+#include "NodeRenderingContext.h"
 #include "Page.h"
 #include "RenderLayer.h"
 #include "RenderView.h"
 #include "RenderWidget.h"
 #include "Settings.h"
+#include "ShadowRoot.h"
+#include "Text.h"
 #include "TextIterator.h"
 #include "WebKitAnimationList.h"
 #include "XMLNames.h"
@@ -89,7 +93,13 @@ public:
 
         if (!m_pushedStyleSelector)
             return;
+
+        // This tells us that our pushed style selector is in a bad state,
+        // so we should just bail out in that scenario.
         ASSERT(m_pushedStyleSelector == m_parent->document()->styleSelector());
+        if (m_pushedStyleSelector != m_parent->document()->styleSelector())
+            return;
+
         m_pushedStyleSelector->popParent(m_parent); 
     }
 
@@ -121,9 +131,9 @@ inline ElementRareData* Element::ensureRareData()
     return static_cast<ElementRareData*>(Node::ensureRareData());
 }
     
-NodeRareData* Element::createRareData()
+OwnPtr<NodeRareData> Element::createRareData()
 {
-    return new ElementRareData;
+    return adoptPtr(new ElementRareData);
 }
 
 DEFINE_VIRTUAL_ATTRIBUTE_EVENT_LISTENER(Element, blur);
@@ -131,48 +141,6 @@ DEFINE_VIRTUAL_ATTRIBUTE_EVENT_LISTENER(Element, error);
 DEFINE_VIRTUAL_ATTRIBUTE_EVENT_LISTENER(Element, focus);
 DEFINE_VIRTUAL_ATTRIBUTE_EVENT_LISTENER(Element, load);
 
-PassRefPtr<DocumentFragment> Element::deprecatedCreateContextualFragment(const String& markup, FragmentScriptingPermission scriptingPermission)
-{
-    RefPtr<DocumentFragment> fragment = document()->createDocumentFragment();
-
-    if (document()->isHTMLDocument())
-        fragment->parseHTML(markup, this, scriptingPermission);
-    else {
-        if (!fragment->parseXML(markup, this, scriptingPermission))
-            // FIXME: We should propagate a syntax error exception out here.
-            return 0;
-    }
-
-    // Exceptions are ignored because none ought to happen here.
-    ExceptionCode ignoredExceptionCode;
-
-    // We need to pop <html> and <body> elements and remove <head> to
-    // accommodate folks passing complete HTML documents to make the
-    // child of an element.
-
-    RefPtr<Node> nextNode;
-    for (RefPtr<Node> node = fragment->firstChild(); node; node = nextNode) {
-        nextNode = node->nextSibling();
-        if (node->hasTagName(htmlTag) || node->hasTagName(headTag) || node->hasTagName(bodyTag)) {
-            HTMLElement* element = toHTMLElement(node.get());
-            Node* firstChild = element->firstChild();
-            if (firstChild)
-                nextNode = firstChild;
-            RefPtr<Node> nextChild;
-            for (RefPtr<Node> child = firstChild; child; child = nextChild) {
-                nextChild = child->nextSibling();
-                element->removeChild(child.get(), ignoredExceptionCode);
-                ASSERT(!ignoredExceptionCode);
-                fragment->insertBefore(child, element, ignoredExceptionCode);
-                ASSERT(!ignoredExceptionCode);
-            }
-            fragment->removeChild(element, ignoredExceptionCode);
-            ASSERT(!ignoredExceptionCode);
-        }
-    }
-    return fragment.release();
-}
-
 PassRefPtr<Node> Element::cloneNode(bool deep)
 {
     return deep ? cloneElementWithChildren() : cloneElementWithoutChildren();
@@ -201,11 +169,15 @@ PassRefPtr<Element> Element::cloneElementWithoutChildren()
     return clone.release();
 }
 
-PassRefPtr<Element> Element::cloneElementWithoutAttributesAndChildren() const
+PassRefPtr<Element> Element::cloneElementWithoutAttributesAndChildren()
 {
     return document()->createElement(tagQName(), false);
 }
 
+void Element::copyNonAttributeProperties(const Element*)
+{
+}
+
 void Element::removeAttribute(const QualifiedName& name, ExceptionCode& ec)
 {
     if (m_attributeMap) {
@@ -221,12 +193,6 @@ void Element::setAttribute(const QualifiedName& name, const AtomicString& value)
     ExceptionCode ec;
     setAttribute(name, value, ec);
 }
-    
-void Element::setCStringAttribute(const QualifiedName& name, const char* cStringValue)
-{
-    ExceptionCode ec;
-    setAttribute(name, AtomicString(cStringValue), ec);
-}
 
 void Element::setBooleanAttribute(const QualifiedName& name, bool b)
 {
@@ -264,41 +230,48 @@ const AtomicString& Element::getAttribute(const QualifiedName& name) const
 void Element::scrollIntoView(bool alignToTop) 
 {
     document()->updateLayoutIgnorePendingStylesheets();
-    IntRect bounds = getRect();    
-    if (renderer()) {
-        // Align to the top / bottom and to the closest edge.
-        if (alignToTop)
-            renderer()->enclosingLayer()->scrollRectToVisible(bounds, false, ScrollAlignment::alignToEdgeIfNeeded, ScrollAlignment::alignTopAlways);
-        else
-            renderer()->enclosingLayer()->scrollRectToVisible(bounds, false, ScrollAlignment::alignToEdgeIfNeeded, ScrollAlignment::alignBottomAlways);
-    }
+
+    if (!renderer())
+        return;
+
+    LayoutRect bounds = getRect();
+    // Align to the top / bottom and to the closest edge.
+    if (alignToTop)
+        renderer()->enclosingLayer()->scrollRectToVisible(bounds, ScrollAlignment::alignToEdgeIfNeeded, ScrollAlignment::alignTopAlways);
+    else
+        renderer()->enclosingLayer()->scrollRectToVisible(bounds, ScrollAlignment::alignToEdgeIfNeeded, ScrollAlignment::alignBottomAlways);
 }
 
 void Element::scrollIntoViewIfNeeded(bool centerIfNeeded)
 {
     document()->updateLayoutIgnorePendingStylesheets();
-    IntRect bounds = getRect();    
-    if (renderer()) {
-        if (centerIfNeeded)
-            renderer()->enclosingLayer()->scrollRectToVisible(bounds, false, ScrollAlignment::alignCenterIfNeeded, ScrollAlignment::alignCenterIfNeeded);
-        else
-            renderer()->enclosingLayer()->scrollRectToVisible(bounds, false, ScrollAlignment::alignToEdgeIfNeeded, ScrollAlignment::alignToEdgeIfNeeded);
-    }
+
+    if (!renderer())
+        return;
+
+    LayoutRect bounds = getRect();
+    if (centerIfNeeded)
+        renderer()->enclosingLayer()->scrollRectToVisible(bounds, ScrollAlignment::alignCenterIfNeeded, ScrollAlignment::alignCenterIfNeeded);
+    else
+        renderer()->enclosingLayer()->scrollRectToVisible(bounds, ScrollAlignment::alignToEdgeIfNeeded, ScrollAlignment::alignToEdgeIfNeeded);
 }
 
 void Element::scrollByUnits(int units, ScrollGranularity granularity)
 {
     document()->updateLayoutIgnorePendingStylesheets();
-    if (RenderObject *rend = renderer()) {
-        if (rend->hasOverflowClip()) {
-            ScrollDirection direction = ScrollDown;
-            if (units < 0) {
-                direction = ScrollUp;
-                units = -units;
-            }
-            toRenderBox(rend)->layer()->scroll(direction, granularity, units);
-        }
+
+    if (!renderer())
+        return;
+
+    if (!renderer()->hasOverflowClip())
+        return;
+
+    ScrollDirection direction = ScrollDown;
+    if (units < 0) {
+        direction = ScrollUp;
+        units = -units;
     }
+    toRenderBox(renderer())->layer()->scroll(direction, granularity, units);
 }
 
 void Element::scrollByLines(int lines)
@@ -334,7 +307,7 @@ static float localZoomForRenderer(RenderObject* renderer)
     return zoomFactor;
 }
 
-static int adjustForLocalZoom(int value, RenderObject* renderer)
+static LayoutUnit adjustForLocalZoom(LayoutUnit value, RenderObject* renderer)
 {
     float zoomFactor = localZoomForRenderer(renderer);
     if (zoomFactor == 1)
@@ -342,7 +315,7 @@ static int adjustForLocalZoom(int value, RenderObject* renderer)
     // Needed because computeLengthInt truncates (rather than rounds) when scaling up.
     if (zoomFactor > 1)
         value++;
-    return static_cast<int>(value / zoomFactor);
+    return static_cast<LayoutUnit>(value / zoomFactor);
 }
 
 int Element::offsetLeft()
@@ -445,7 +418,7 @@ int Element::clientHeight()
     return 0;
 }
 
-int Element::scrollLeft() const
+int Element::scrollLeft()
 {
     document()->updateLayoutIgnorePendingStylesheets();
     if (RenderBox* rend = renderBox())
@@ -453,7 +426,7 @@ int Element::scrollLeft() const
     return 0;
 }
 
-int Element::scrollTop() const
+int Element::scrollTop()
 {
     document()->updateLayoutIgnorePendingStylesheets();
     if (RenderBox* rend = renderBox())
@@ -475,7 +448,7 @@ void Element::setScrollTop(int newTop)
         rend->setScrollTop(static_cast<int>(newTop * rend->style()->effectiveZoom()));
 }
 
-int Element::scrollWidth() const
+int Element::scrollWidth()
 {
     document()->updateLayoutIgnorePendingStylesheets();
     if (RenderBox* rend = renderBox())
@@ -483,7 +456,7 @@ int Element::scrollWidth() const
     return 0;
 }
 
-int Element::scrollHeight() const
+int Element::scrollHeight()
 {
     document()->updateLayoutIgnorePendingStylesheets();
     if (RenderBox* rend = renderBox())
@@ -491,19 +464,19 @@ int Element::scrollHeight() const
     return 0;
 }
 
-IntRect Element::boundsInWindowSpace() const
+LayoutRect Element::boundsInWindowSpace()
 {
     document()->updateLayoutIgnorePendingStylesheets();
 
     FrameView* view = document()->view();
     if (!view)
-        return IntRect();
+        return LayoutRect();
 
     Vector<FloatQuad> quads;
 #if ENABLE(SVG)
     if (isSVGElement() && renderer()) {
         // Get the bounding rectangle from the SVG model.
-        const SVGElement* svgElement = static_cast<const SVGElement*>(this);
+        SVGElement* svgElement = static_cast<SVGElement*>(this);
         FloatRect localRect;
         if (svgElement->boundingBox(localRect))
             quads.append(renderer()->localToAbsoluteQuad(localRect));
@@ -516,9 +489,9 @@ IntRect Element::boundsInWindowSpace() const
     }
 
     if (quads.isEmpty())
-        return IntRect();
+        return LayoutRect();
 
-    IntRect result = quads[0].enclosingBoundingBox();
+    LayoutRect result = quads[0].enclosingBoundingBox();
     for (size_t i = 1; i < quads.size(); ++i)
         result.unite(quads[i].enclosingBoundingBox());
 
@@ -526,7 +499,7 @@ IntRect Element::boundsInWindowSpace() const
     return result;
 }
 
-PassRefPtr<ClientRectList> Element::getClientRects() const
+PassRefPtr<ClientRectList> Element::getClientRects()
 {
     document()->updateLayoutIgnorePendingStylesheets();
 
@@ -540,18 +513,24 @@ PassRefPtr<ClientRectList> Element::getClientRects() const
     Vector<FloatQuad> quads;
     renderBoxModelObject->absoluteQuads(quads);
 
+    float pageScale = 1;
+    if (Page* page = document()->page()) 
+        pageScale = page->pageScaleFactor();
+
     if (FrameView* view = document()->view()) {
-        IntRect visibleContentRect = view->visibleContentRect();
+        LayoutRect visibleContentRect = view->visibleContentRect();
         for (size_t i = 0; i < quads.size(); ++i) {
             quads[i].move(-visibleContentRect.x(), -visibleContentRect.y());
             adjustFloatQuadForAbsoluteZoom(quads[i], renderBoxModelObject);
+            if (pageScale != 1)
+                adjustFloatQuadForPageScale(quads[i], pageScale);
         }
     }
 
     return ClientRectList::create(quads);
 }
 
-PassRefPtr<ClientRect> Element::getBoundingClientRect() const
+PassRefPtr<ClientRect> Element::getBoundingClientRect()
 {
     document()->updateLayoutIgnorePendingStylesheets();
 
@@ -559,7 +538,7 @@ PassRefPtr<ClientRect> Element::getBoundingClientRect() const
 #if ENABLE(SVG)
     if (isSVGElement() && renderer()) {
         // Get the bounding rectangle from the SVG model.
-        const SVGElement* svgElement = static_cast<const SVGElement*>(this);
+        SVGElement* svgElement = static_cast<SVGElement*>(this);
         FloatRect localRect;
         if (svgElement->boundingBox(localRect))
             quads.append(renderer()->localToAbsoluteQuad(localRect));
@@ -579,18 +558,21 @@ PassRefPtr<ClientRect> Element::getBoundingClientRect() const
         result.unite(quads[i].boundingBox());
 
     if (FrameView* view = document()->view()) {
-        IntRect visibleContentRect = view->visibleContentRect();
+        LayoutRect visibleContentRect = view->visibleContentRect();
         result.move(-visibleContentRect.x(), -visibleContentRect.y());
     }
 
     adjustFloatRectForAbsoluteZoom(result, renderer());
+    if (Page* page = document()->page())
+        adjustFloatRectForPageScale(result, page->pageScaleFactor());
+
     return ClientRect::create(result);
 }
     
-IntRect Element::screenRect() const
+LayoutRect Element::screenRect() const
 {
     if (!renderer())
-        return IntRect();
+        return LayoutRect();
     return renderer()->view()->frameView()->contentsToScreen(renderer()->absoluteBoundingBoxRect());
 }
 
@@ -640,22 +622,20 @@ void Element::setAttribute(const AtomicString& name, const AtomicString& value,
 #endif
 
     const AtomicString& localName = shouldIgnoreAttributeCase(this) ? name.lower() : name;
-
+    QualifiedName attributeName(nullAtom, localName, nullAtom);
+    
     // Allocate attribute map if necessary.
     Attribute* old = attributes(false)->getAttributeItem(localName, false);
 
     document()->incDOMTreeVersion();
 
-    // FIXME: This check is probably not correct for the case where the document has an id attribute
-    // with a non-null namespace, because it will return true if the local name happens to match
-    // but the namespace does not.
-    if (localName == document()->idAttributeName().localName())
+    if (isIdAttributeName(old ? old->name() : attributeName))
         updateId(old ? old->value() : nullAtom, value);
 
     if (old && value.isNull())
         m_attributeMap->removeAttribute(old->name());
     else if (!old && !value.isNull())
-        m_attributeMap->addAttribute(createAttribute(QualifiedName(nullAtom, localName, nullAtom), value));
+        m_attributeMap->addAttribute(createAttribute(attributeName, value));
     else if (old && !value.isNull()) {
         if (Attr* attrNode = old->attr())
             attrNode->setValue(value);
@@ -710,6 +690,8 @@ PassRefPtr<Attribute> Element::createAttribute(const QualifiedName& name, const
 
 void Element::attributeChanged(Attribute* attr, bool)
 {
+    if (isIdAttributeName(attr->name()))
+        idAttributeChanged(attr);
     recalcStyleIfNeededAfterAttributeChanged(attr);
     updateAfterAttributeChanged(attr);
 }
@@ -748,6 +730,20 @@ void Element::recalcStyleIfNeededAfterAttributeChanged(Attribute* attr)
         setNeedsStyleRecalc();
 }
 
+void Element::idAttributeChanged(Attribute* attr)
+{
+    setHasID(!attr->isNull());
+    if (attributeMap()) {
+        if (attr->isNull())
+            attributeMap()->setIdForStyleResolution(nullAtom);
+        else if (document()->inQuirksMode())
+            attributeMap()->setIdForStyleResolution(attr->value().lower());
+        else
+            attributeMap()->setIdForStyleResolution(attr->value());
+    }
+    setNeedsStyleRecalc();
+}
+    
 // Returns true is the given attribute is an event handler.
 // We consider an event handler any attribute that begins with "on".
 // It is a simple solution that has the advantage of not requiring any
@@ -896,12 +892,21 @@ void Element::setChangedSinceLastFormControlChangeEvent(bool)
 {
 }
 
+void Element::willRemove()
+{
+#if ENABLE(FULLSCREEN_API)
+    if (containsFullScreenElement())
+        setContainsFullScreenElementOnAncestorsCrossingFrameBoundaries(false);
+#endif
+    ContainerNode::willRemove();
+}
+
 void Element::insertedIntoDocument()
 {
     // need to do superclass processing first so inDocument() is true
     // by the time we reach updateId
     ContainerNode::insertedIntoDocument();
-    if (Node* shadow = shadowRoot())
+    if (ShadowRoot* shadow = shadowRoot())
         shadow->insertedIntoDocument();
 
     if (hasID()) {
@@ -924,7 +929,7 @@ void Element::removedFromDocument()
     }
 
     ContainerNode::removedFromDocument();
-    if (Node* shadow = shadowRoot())
+    if (ShadowRoot* shadow = shadowRoot())
         shadow->removedFromDocument();
 }
 
@@ -933,8 +938,13 @@ void Element::insertedIntoTree(bool deep)
     ContainerNode::insertedIntoTree(deep);
     if (!deep)
         return;
-    if (Node* shadow = shadowRoot())
+    if (ShadowRoot* shadow = shadowRoot())
         shadow->insertedIntoTree(true);
+
+#if ENABLE(FULLSCREEN_API)
+    if (containsFullScreenElement() && parentElement() && !parentElement()->containsFullScreenElement())
+        setContainsFullScreenElementOnAncestorsCrossingFrameBoundaries(true);
+#endif
 }
 
 void Element::removedFromTree(bool deep)
@@ -942,7 +952,7 @@ void Element::removedFromTree(bool deep)
     ContainerNode::removedFromTree(deep);
     if (!deep)
         return;
-    if (Node* shadow = shadowRoot())
+    if (ShadowRoot* shadow = shadowRoot())
         shadow->removedFromTree(true);
 }
 
@@ -954,13 +964,16 @@ void Element::attach()
     createRendererIfNeeded();
     
     StyleSelectorParentPusher parentPusher(this);
+
     if (firstChild())
         parentPusher.push();
     ContainerNode::attach();
-    if (Node* shadow = shadowRoot()) {
+
+    if (ShadowRoot* shadow = shadowRoot()) {
         parentPusher.push();
         shadow->attach();
     }
+
     if (hasRareData()) {   
         ElementRareData* data = rareData();
         if (data->needsFocusAppearanceUpdateSoonAfterAttach()) {
@@ -982,7 +995,7 @@ void Element::detach()
     if (hasRareData())
         rareData()->resetComputedStyle();
     ContainerNode::detach();
-    if (Node* shadow = shadowRoot())
+    if (ShadowRoot* shadow = shadowRoot())
         shadow->detach();
 
     RenderWidget::resumeWidgetHierarchyUpdates();
@@ -1029,27 +1042,50 @@ bool Element::pseudoStyleCacheIsInvalid(const RenderStyle* currentStyle, RenderS
     return false;
 }
 
+PassRefPtr<RenderStyle> Element::customStyleForRenderer()
+{
+    ASSERT_NOT_REACHED(); 
+    return 0; 
+}
+
+PassRefPtr<RenderStyle> Element::styleForRenderer()
+{
+    if (hasCustomStyleForRenderer())
+        return customStyleForRenderer();
+    return document()->styleSelector()->styleForElement(static_cast<Element*>(this), 0, true/*allowSharing*/);
+}
+
 void Element::recalcStyle(StyleChange change)
 {
+    if (hasCustomWillOrDidRecalcStyle()) {
+        if (!willRecalcStyle(change))
+            return;
+    }
+
     // Ref currentStyle in case it would otherwise be deleted when setRenderStyle() is called.
     RefPtr<RenderStyle> currentStyle(renderStyle());
-    bool hasParentStyle = parentOrHostNode() ? parentOrHostNode()->renderStyle() : false;
-    bool hasDirectAdjacentRules = currentStyle && currentStyle->childrenAffectedByDirectAdjacentRules();
+    bool hasParentStyle = parentNodeForRenderingAndStyle() ? static_cast<bool>(parentNodeForRenderingAndStyle()->renderStyle()) : false;
+    bool hasIndirectAdjacentRules = currentStyle && currentStyle->childrenAffectedByForwardPositionalRules();
 
     if ((change > NoChange || needsStyleRecalc())) {
-        if (hasRareData())
-            rareData()->resetComputedStyle();
+        if (hasRareData()) {
+            ElementRareData* data = rareData();
+            data->resetComputedStyle();
+            data->m_styleAffectedByEmpty = false;
+        }
     }
     if (hasParentStyle && (change >= Inherit || needsStyleRecalc())) {
-        RefPtr<RenderStyle> newStyle = document()->styleSelector()->styleForElement(this);
+        RefPtr<RenderStyle> newStyle = styleForRenderer();
         StyleChange ch = diff(currentStyle.get(), newStyle.get());
         if (ch == Detach || !currentStyle) {
-            if (attached())
-                detach();
-            attach(); // FIXME: The style gets computed twice by calling attach. We could do better if we passed the style along.
-            // attach recalulates the style for all children. No need to do it twice.
+            // FIXME: The style gets computed twice by calling attach. We could do better if we passed the style along.
+            reattach();
+            // attach recalculates the style for all children. No need to do it twice.
             clearNeedsStyleRecalc();
             clearChildNeedsStyleRecalc();
+
+            if (hasCustomWillOrDidRecalcStyle())
+                didRecalcStyle(change);
             return;
         }
 
@@ -1070,8 +1106,8 @@ void Element::recalcStyle(StyleChange change)
                 newStyle->setChildrenAffectedByFirstChildRules();
             if (currentStyle->childrenAffectedByLastChildRules())
                 newStyle->setChildrenAffectedByLastChildRules();
-            if (currentStyle->childrenAffectedByDirectAdjacentRules())
-                newStyle->setChildrenAffectedByDirectAdjacentRules();
+            if (currentStyle->affectedByDirectAdjacentRules())
+                newStyle->setAffectedByDirectAdjacentRules();
         }
 
         if (ch != NoChange || pseudoStyleCacheIsInvalid(currentStyle.get(), newStyle.get()) || (change == Force && renderer() && renderer()->requiresForcedStyleRecalcPropagation())) {
@@ -1101,52 +1137,91 @@ void Element::recalcStyle(StyleChange change)
     // FIXME: This check is good enough for :hover + foo, but it is not good enough for :hover + foo + bar.
     // For now we will just worry about the common case, since it's a lot trickier to get the second case right
     // without doing way too much re-resolution.
-    bool forceCheckOfNextElementSibling = false;
+    bool previousSiblingHadDirectAdjacentRules = false;
+    bool forceCheckOfAnyElementSibling = false;
     for (Node *n = firstChild(); n; n = n->nextSibling()) {
-        bool childRulesChanged = n->needsStyleRecalc() && n->styleChangeType() == FullStyleChange;
-        if (forceCheckOfNextElementSibling && n->isElementNode())
-            n->setNeedsStyleRecalc();
-        if (change >= Inherit || n->isTextNode() || n->childNeedsStyleRecalc() || n->needsStyleRecalc()) {
+        if (n->isTextNode()) {
             parentPusher.push();
-            n->recalcStyle(change);
+            static_cast<Text*>(n)->recalcTextStyle(change);
+            continue;
+        } 
+        if (!n->isElementNode()) 
+            continue;
+        Element* element = static_cast<Element*>(n);
+        bool childRulesChanged = element->needsStyleRecalc() && element->styleChangeType() == FullStyleChange;
+        bool childAffectedByDirectAdjacentRules = element->renderStyle() ? element->renderStyle()->affectedByDirectAdjacentRules() : previousSiblingHadDirectAdjacentRules;
+        if (childAffectedByDirectAdjacentRules || forceCheckOfAnyElementSibling)
+            element->setNeedsStyleRecalc();
+        if (change >= Inherit || element->childNeedsStyleRecalc() || element->needsStyleRecalc()) {
+            parentPusher.push();
+            element->recalcStyle(change);
         }
-        if (n->isElementNode())
-            forceCheckOfNextElementSibling = childRulesChanged && hasDirectAdjacentRules;
+        previousSiblingHadDirectAdjacentRules = childAffectedByDirectAdjacentRules;
+        forceCheckOfAnyElementSibling = forceCheckOfAnyElementSibling || (childRulesChanged && hasIndirectAdjacentRules);
     }
     // FIXME: This does not care about sibling combinators. Will be necessary in XBL2 world.
-    if (Node* shadow = shadowRoot()) {
-        if (change >= Inherit || shadow->isTextNode() || shadow->childNeedsStyleRecalc() || shadow->needsStyleRecalc()) {
+    if (ShadowRoot* shadow = shadowRoot()) {
+        if (change >= Inherit || shadow->childNeedsStyleRecalc() || shadow->needsStyleRecalc()) {
             parentPusher.push();
-            shadow->recalcStyle(change);
+            shadow->recalcShadowTreeStyle(change);
         }
     }
 
     clearNeedsStyleRecalc();
     clearChildNeedsStyleRecalc();
+    
+    if (hasCustomWillOrDidRecalcStyle())
+        didRecalcStyle(change);
 }
 
-Node* Element::shadowRoot()
+ShadowRoot* Element::shadowRoot() const
 {
     return hasRareData() ? rareData()->m_shadowRoot : 0;
 }
 
-void Element::setShadowRoot(PassRefPtr<Node> node)
+static bool validateShadowRoot(Document* document, ShadowRoot* shadowRoot, ExceptionCode& ec)
 {
-    // FIXME: Because today this is never called from script directly, we don't have to worry
-    // about compromising DOM tree integrity (eg. node being a parent of this). However,
-    // once we implement XBL2, we will have to add integrity checks here.
-    removeShadowRoot();
+    if (!shadowRoot)
+        return true;
 
-    RefPtr<Node> newRoot = node;
-    if (!newRoot)
+    if (shadowRoot->shadowHost()) {
+        ec = HIERARCHY_REQUEST_ERR;
+        return false;
+    }
+
+    if (shadowRoot->document() != document) {
+        ec = WRONG_DOCUMENT_ERR;
+        return false;
+    }
+
+    return true;
+}
+
+void Element::setShadowRoot(PassRefPtr<ShadowRoot> prpShadowRoot, ExceptionCode& ec)
+{
+    RefPtr<ShadowRoot> shadowRoot = prpShadowRoot;
+    if (!validateShadowRoot(document(), shadowRoot.get(), ec))
         return;
 
-    ensureRareData()->m_shadowRoot = newRoot.get();
-    newRoot->setShadowHost(this);
+    removeShadowRoot();
+
+    ensureRareData()->m_shadowRoot = shadowRoot.get();
+    shadowRoot->setShadowHost(this);
     if (inDocument())
-        newRoot->insertedIntoDocument();
-    if (attached() && !newRoot->attached())
-        newRoot->lazyAttach();
+        shadowRoot->insertedIntoDocument();
+    if (attached())
+        shadowRoot->lazyAttach();
+}
+
+ShadowRoot* Element::ensureShadowRoot()
+{
+    if (ShadowRoot* existingRoot = shadowRoot())
+        return existingRoot;
+
+    ExceptionCode ec = 0;
+    setShadowRoot(ShadowRoot::create(document()), ec);
+    ASSERT(!ec);
+    return shadowRoot();
 }
 
 void Element::removeShadowRoot()
@@ -1155,10 +1230,16 @@ void Element::removeShadowRoot()
         return;
 
     ElementRareData* data = rareData();
-    if (RefPtr<Node> oldRoot = data->m_shadowRoot) {
+    if (RefPtr<ShadowRoot> oldRoot = data->m_shadowRoot) {
         data->m_shadowRoot = 0;
         document()->removeFocusedNodeOfSubtree(oldRoot.get());
+
+        // Remove from rendering tree
+        if (oldRoot->attached())
+            oldRoot->detach();
+
         oldRoot->setShadowHost(0);
+        oldRoot->setTreeScopeRecursively(document());
         if (oldRoot->inDocument())
             oldRoot->removedFromDocument();
         else
@@ -1166,7 +1247,25 @@ void Element::removeShadowRoot()
     }
 }
 
-bool Element::childTypeAllowed(NodeType type)
+const AtomicString& Element::shadowPseudoId() const
+{
+    return hasRareData() ? rareData()->m_shadowPseudoId : nullAtom;
+}
+
+void Element::setShadowPseudoId(const AtomicString& id, ExceptionCode& ec)
+{
+    if (!hasRareData() && id == nullAtom)
+        return;
+
+    if (!CSSSelector::isUnknownPseudoType(id)) {
+        ec = SYNTAX_ERR;
+        return;
+    }
+
+    ensureRareData()->m_shadowPseudoId = id;
+}
+
+bool Element::childTypeAllowed(NodeType type) const
 {
     switch (type) {
     case ELEMENT_NODE:
@@ -1184,10 +1283,10 @@ bool Element::childTypeAllowed(NodeType type)
 
 static void checkForEmptyStyleChange(Element* element, RenderStyle* style)
 {
-    if (!style)
+    if (!style && !element->styleAffectedByEmpty())
         return;
 
-    if (style->affectedByEmpty() && (!style->emptyState() || element->hasChildNodes()))
+    if (!style || (style->affectedByEmpty() && (!style->emptyState() || element->hasChildNodes())))
         element->setNeedsStyleRecalc();
 }
 
@@ -1244,17 +1343,6 @@ static void checkForSiblingStyleChanges(Element* e, RenderStyle* style, bool fin
             newLastChild->setNeedsStyleRecalc();
     }
 
-    // The + selector.  We need to invalidate the first element following the insertion point.  It is the only possible element
-    // that could be affected by this DOM change.
-    if (style->childrenAffectedByDirectAdjacentRules() && afterChange) {
-        Node* firstElementAfterInsertion = 0;
-        for (firstElementAfterInsertion = afterChange;
-             firstElementAfterInsertion && !firstElementAfterInsertion->isElementNode();
-             firstElementAfterInsertion = firstElementAfterInsertion->nextSibling()) {};
-        if (firstElementAfterInsertion && firstElementAfterInsertion->attached())
-            firstElementAfterInsertion->setNeedsStyleRecalc();
-    }
-
     // Forward positional selectors include the ~ selector, nth-child, nth-of-type, first-of-type and only-of-type.
     // Backward positional selectors include nth-last-child, nth-last-of-type, last-of-type and only-of-type.
     // We have to invalidate everything following the insertion point in the forward case, and everything before the insertion point in the
@@ -1322,31 +1410,6 @@ void Element::dispatchAttrAdditionEvent(Attribute*)
 #endif
 }
 
-String Element::openTagStartToString() const
-{
-    String result = "<" + nodeName();
-
-    NamedNodeMap* attrMap = attributes(true);
-
-    if (attrMap) {
-        unsigned numAttrs = attrMap->length();
-        for (unsigned i = 0; i < numAttrs; i++) {
-            result += " ";
-
-            Attribute *attribute = attrMap->attributeItem(i);
-            result += attribute->name().toString();
-            if (!attribute->value().isNull()) {
-                result += "=\"";
-                // FIXME: substitute entities for any instances of " or '
-                result += attribute->value();
-                result += "\"";
-            }
-        }
-    }
-
-    return result;
-}
-
 #ifndef NDEBUG
 void Element::formatForDebugger(char* buffer, unsigned length) const
 {
@@ -1442,6 +1505,7 @@ void Element::removeAttribute(const String& name, ExceptionCode& ec)
     String localName = shouldIgnoreAttributeCase(this) ? name.lower() : name;
 
     if (m_attributeMap) {
+        ec = 0;
         m_attributeMap->removeNamedItem(localName, ec);
         if (ec == NOT_FOUND_ERR)
             ec = 0;
@@ -1524,6 +1588,8 @@ void Element::focus(bool restorePreviousSelection)
         // If a focus event handler changes the focus to a different node it
         // does not make sense to continue and update appearence.
         protect = this;
+        if (shadowRoot() && page->focusController()->transferFocusToElementInShadowRoot(this, restorePreviousSelection))
+            return;
         if (!page->focusController()->setFocusedNode(this, doc->frame()))
             return;
     }
@@ -1574,7 +1640,7 @@ void Element::blur()
     }
 }
 
-String Element::innerText() const
+String Element::innerText()
 {
     // We need to update layout, since plainText uses line boxes in the render tree.
     document()->updateLayoutIgnorePendingStylesheets();
@@ -1585,7 +1651,7 @@ String Element::innerText() const
     return plainText(rangeOfContents(const_cast<Element*>(this)).get());
 }
 
-String Element::outerText() const
+String Element::outerText()
 {
     // Getting outerText is the same as getting innerText, only
     // setting is different. You would think this should get the plain
@@ -1600,12 +1666,12 @@ String Element::title() const
     return String();
 }
 
-IntSize Element::minimumSizeForResizing() const
+LayoutSize Element::minimumSizeForResizing() const
 {
     return hasRareData() ? rareData()->m_minimumSizeForResizing : defaultMinimumSizeForResizing();
 }
 
-void Element::setMinimumSizeForResizing(const IntSize& size)
+void Element::setMinimumSizeForResizing(const LayoutSize& size)
 {
     if (size == defaultMinimumSizeForResizing() && !hasRareData())
         return;
@@ -1631,6 +1697,17 @@ RenderStyle* Element::computedStyle(PseudoId pseudoElementSpecifier)
     return pseudoElementSpecifier ? data->m_computedStyle->getCachedPseudoStyle(pseudoElementSpecifier) : data->m_computedStyle.get();
 }
 
+void Element::setStyleAffectedByEmpty()
+{
+    ElementRareData* data = ensureRareData();
+    data->m_styleAffectedByEmpty = true;
+}
+
+bool Element::styleAffectedByEmpty() const
+{
+    return hasRareData() && rareData()->m_styleAffectedByEmpty;
+}
+
 AtomicString Element::computeInheritedLanguage() const
 {
     const Node* n = this;
@@ -1745,7 +1822,7 @@ bool Element::webkitMatchesSelector(const String& selector, ExceptionCode& ec)
         return false;
     }
 
-    CSSStyleSelector::SelectorChecker selectorChecker(document(), strictParsing);
+    SelectorChecker selectorChecker(document(), strictParsing);
     for (CSSSelector* selector = selectorList.first(); selector; selector = CSSSelectorList::next(selector)) {
         if (selectorChecker.checkSelector(selector, this))
             return true;
@@ -1777,13 +1854,6 @@ DOMStringMap* Element::dataset()
     return data->m_datasetDOMStringMap.get();
 }
 
-DOMStringMap* Element::optionalDataset() const
-{
-    if (!hasRareData())
-        return 0;
-    return rareData()->m_datasetDOMStringMap.get();
-}
-
 KURL Element::getURLAttribute(const QualifiedName& name) const
 {
 #if !ASSERT_DISABLED
@@ -1847,16 +1917,39 @@ bool Element::childShouldCreateRenderer(Node* child) const
 #if ENABLE(FULLSCREEN_API)
 void Element::webkitRequestFullScreen(unsigned short flags)
 {
-    document()->webkitRequestFullScreenForElement(this, flags);
+    document()->requestFullScreenForElement(this, flags, Document::EnforceIFrameAllowFulScreenRequirement);
+}
+
+bool Element::containsFullScreenElement() const
+{
+    return hasRareData() ? rareData()->m_containsFullScreenElement : false;
+}
+
+void Element::setContainsFullScreenElement(bool flag)
+{
+    ensureRareData()->m_containsFullScreenElement = flag;
+    setNeedsStyleRecalc(SyntheticStyleChange);
+}
+
+static Element* parentCrossingFrameBoundaries(Element* element)
+{
+    ASSERT(element);
+    return element->parentElement() ? element->parentElement() : element->document()->ownerElement();
+}
+
+void Element::setContainsFullScreenElementOnAncestorsCrossingFrameBoundaries(bool flag)
+{
+    Element* element = this;
+    while ((element = parentCrossingFrameBoundaries(element)))
+        element->setContainsFullScreenElement(flag);
 }
 #endif    
 
 SpellcheckAttributeState Element::spellcheckAttributeState() const
 {
-    if (!hasAttribute(HTMLNames::spellcheckAttr))
-        return SpellcheckAttributeDefault;
-
     const AtomicString& value = getAttribute(HTMLNames::spellcheckAttr);
+    if (value == nullAtom)
+        return SpellcheckAttributeDefault;
     if (equalIgnoringCase(value, "true") || equalIgnoringCase(value, ""))
         return SpellcheckAttributeTrue;
     if (equalIgnoringCase(value, "false"))
@@ -1879,7 +1972,12 @@ bool Element::isSpellCheckingEnabled() const
         }
 
         ContainerNode* parent = const_cast<Element*>(element)->parentOrHostNode();
-        element = (parent && parent->isElementNode()) ? toElement(parent) : 0;
+        if (parent && parent->isElementNode())
+            element = toElement(parent);
+        else if (parent && parent->isShadowRoot())
+            element = toElement(parent->parentOrHostNode());
+        else
+            element = 0;
     }
 
     return true;
