@@ -32,6 +32,7 @@
 #include "DeleteSelectionCommand.h"
 #include "Document.h"
 #include "DocumentFragment.h"
+#include "DocumentMarkerController.h"
 #include "EditorInsertAction.h"
 #include "Frame.h"
 #include "HTMLElement.h"
@@ -42,7 +43,6 @@
 #include "InsertNodeBeforeCommand.h"
 #include "InsertParagraphSeparatorCommand.h"
 #include "InsertTextCommand.h"
-#include "JoinTextNodesCommand.h"
 #include "MergeIdenticalElementsCommand.h"
 #include "Range.h"
 #include "RemoveCSSPropertyCommand.h"
@@ -103,6 +103,17 @@ void CompositeEditCommand::applyCommandToComposite(PassRefPtr<EditCommand> cmd)
     m_commands.append(cmd);
 }
 
+void CompositeEditCommand::applyCommandToComposite(PassRefPtr<CompositeEditCommand> command, const VisibleSelection& selection)
+{
+    command->setParent(this);
+    if (selection != command->endingSelection()) {
+        command->setStartingSelection(selection);
+        command->setEndingSelection(selection);
+    }
+    command->apply();
+    m_commands.append(command);
+}
+
 void CompositeEditCommand::applyStyle(const EditingStyle* style, EditAction editingAction)
 {
     applyCommandToComposite(ApplyStyleCommand::create(document(), style, editingAction));
@@ -146,6 +157,7 @@ void CompositeEditCommand::insertNodeAfter(PassRefPtr<Node> insertChild, PassRef
     ASSERT(!refChild->hasTagName(bodyTag));
     ContainerNode* parent = refChild->parentNode();
     ASSERT(parent);
+    ASSERT(!parent->isShadowRoot());
     if (parent->lastChild() == refChild)
         appendNode(insertChild, parent);
     else {
@@ -204,7 +216,7 @@ void CompositeEditCommand::removeChildrenInRange(PassRefPtr<Node> node, unsigned
 
 void CompositeEditCommand::removeNode(PassRefPtr<Node> node)
 {
-    if (!node || !node->parentNode())
+    if (!node || !node->nonShadowBoundaryParentNode())
         return;
     applyCommandToComposite(RemoveNodeCommand::create(node));
 }
@@ -246,8 +258,10 @@ static bool hasARenderedDescendant(Node* node)
     return false;
 }
 
-void CompositeEditCommand::prune(PassRefPtr<Node> node)
+void CompositeEditCommand::prune(PassRefPtr<Node> prpNode)
 {
+    RefPtr<Node> node = prpNode;
+
     while (node) {
         // If you change this rule you may have to add an updateLayout() here.
         RenderObject* renderer = node->renderer();
@@ -292,41 +306,10 @@ void CompositeEditCommand::splitTextNodeContainingElement(PassRefPtr<Text> text,
     applyCommandToComposite(SplitTextNodeContainingElementCommand::create(text, offset));
 }
 
-void CompositeEditCommand::joinTextNodes(PassRefPtr<Text> text1, PassRefPtr<Text> text2)
-{
-    applyCommandToComposite(JoinTextNodesCommand::create(text1, text2));
-}
-
-void CompositeEditCommand::inputText(const String& text, bool selectInsertedText)
-{
-    unsigned offset = 0;
-    unsigned length = text.length();
-    RefPtr<Range> startRange = Range::create(document(), firstPositionInNode(document()->documentElement()), endingSelection().start());
-    unsigned startIndex = TextIterator::rangeLength(startRange.get());
-    size_t newline;
-    do {
-        newline = text.find('\n', offset);
-        if (newline != offset) {
-            RefPtr<InsertTextCommand> command = InsertTextCommand::create(document());
-            applyCommandToComposite(command);
-            int substringLength = newline == notFound ? length - offset : newline - offset;
-            command->input(text.substring(offset, substringLength), false);
-        }
-        if (newline != notFound)
-            insertLineBreak();
-            
-        offset = newline + 1;
-    } while (newline != notFound && offset != length);
-    
-    if (selectInsertedText) {
-        RefPtr<Range> selectedRange = TextIterator::rangeFromLocationAndLength(document()->documentElement(), startIndex, length);
-        setEndingSelection(VisibleSelection(selectedRange.get()));
-    }
-}
-
 void CompositeEditCommand::insertTextIntoNode(PassRefPtr<Text> node, unsigned offset, const String& text)
 {
-    applyCommandToComposite(InsertIntoTextNodeCommand::create(node, offset, text));
+    if (!text.isEmpty())
+        applyCommandToComposite(InsertIntoTextNodeCommand::create(node, offset, text));
 }
 
 void CompositeEditCommand::deleteTextFromNode(PassRefPtr<Text> node, unsigned offset, unsigned count)
@@ -337,7 +320,41 @@ void CompositeEditCommand::deleteTextFromNode(PassRefPtr<Text> node, unsigned of
 void CompositeEditCommand::replaceTextInNode(PassRefPtr<Text> node, unsigned offset, unsigned count, const String& replacementText)
 {
     applyCommandToComposite(DeleteFromTextNodeCommand::create(node.get(), offset, count));
-    applyCommandToComposite(InsertIntoTextNodeCommand::create(node, offset, replacementText));
+    if (!replacementText.isEmpty())
+        applyCommandToComposite(InsertIntoTextNodeCommand::create(node, offset, replacementText));
+}
+
+Position CompositeEditCommand::replaceSelectedTextInNode(const String& text)
+{
+    Position start = endingSelection().start();
+    Position end = endingSelection().end();
+    if (start.containerNode() != end.containerNode() || !start.containerNode()->isTextNode() || isTabSpanTextNode(start.containerNode()))
+        return Position();
+
+    RefPtr<Text> textNode = start.containerText();
+    replaceTextInNode(textNode, start.offsetInContainerNode(), end.offsetInContainerNode() - start.offsetInContainerNode(), text);
+
+    return Position(textNode.release(), start.offsetInContainerNode() + text.length());
+}
+
+static void copyMarkers(const Vector<DocumentMarker*>& markerPointers, Vector<DocumentMarker>& markers)
+{
+    size_t arraySize = markerPointers.size();
+    markers.reserveCapacity(arraySize);
+    for (size_t i = 0; i < arraySize; ++i)
+        markers.append(*markerPointers[i]);
+}
+
+void CompositeEditCommand::replaceTextInNodePreservingMarkers(PassRefPtr<Text> prpNode, unsigned offset, unsigned count, const String& replacementText)
+{
+    RefPtr<Text> node(prpNode);
+    DocumentMarkerController* markerController = document()->markers();
+    Vector<DocumentMarker> markers;
+    copyMarkers(markerController->markersInRange(Range::create(document(), node, offset, node, offset + count).get(), DocumentMarker::AllMarkers()), markers);
+    replaceTextInNode(node, offset, count, replacementText);
+    RefPtr<Range> newRange = Range::create(document(), node, offset, node, offset + replacementText.length());
+    for (size_t i = 0; i < markers.size(); ++i)
+        markerController->addMarker(newRange.get(), markers[i].type(), markers[i].description());
 }
 
 Position CompositeEditCommand::positionOutsideTabSpan(const Position& pos)
@@ -345,10 +362,18 @@ Position CompositeEditCommand::positionOutsideTabSpan(const Position& pos)
     if (!isTabSpanTextNode(pos.anchorNode()))
         return pos;
 
-    if (pos.anchorType() == Position::PositionIsAfterAnchor)
-        return positionInParentAfterNode(pos.anchorNode());
-    if (pos.anchorType() == Position::PositionIsBeforeAnchor)
+    switch (pos.anchorType()) {
+    case Position::PositionIsBeforeChildren:
+    case Position::PositionIsAfterChildren:
+        ASSERT_NOT_REACHED();
+        return pos;
+    case Position::PositionIsOffsetInAnchor:
+        break;
+    case Position::PositionIsBeforeAnchor:
         return positionInParentBeforeNode(pos.anchorNode());
+    case Position::PositionIsAfterAnchor:
+        return positionInParentAfterNode(pos.anchorNode());
+    }
 
     Node* tabSpan = tabSpanNode(pos.containerNode());
 
@@ -446,8 +471,10 @@ void CompositeEditCommand::rebalanceWhitespaceAt(const Position& position)
     rebalanceWhitespaceOnTextSubstring(static_cast<Text*>(node), position.offsetInContainerNode(), position.offsetInContainerNode());
 }
 
-void CompositeEditCommand::rebalanceWhitespaceOnTextSubstring(RefPtr<Text> textNode, int startOffset, int endOffset)
+void CompositeEditCommand::rebalanceWhitespaceOnTextSubstring(PassRefPtr<Text> prpTextNode, int startOffset, int endOffset)
 {
+    RefPtr<Text> textNode = prpTextNode;
+
     String text = textNode->data();
     ASSERT(!text.isEmpty());
 
@@ -464,8 +491,8 @@ void CompositeEditCommand::rebalanceWhitespaceOnTextSubstring(RefPtr<Text> textN
     if (!length)
         return;
 
-    VisiblePosition visibleUpstreamPos(Position(textNode, upstream, Position::PositionIsOffsetInAnchor));
-    VisiblePosition visibleDownstreamPos(Position(textNode, downstream, Position::PositionIsOffsetInAnchor));
+    VisiblePosition visibleUpstreamPos(Position(textNode, upstream));
+    VisiblePosition visibleDownstreamPos(Position(textNode, downstream));
     
     String string = text.substring(upstream, length);
     String rebalancedString = stringWithRebalancedWhitespace(string,
@@ -475,7 +502,7 @@ void CompositeEditCommand::rebalanceWhitespaceOnTextSubstring(RefPtr<Text> textN
                                                              isEndOfParagraph(visibleDownstreamPos) || (unsigned)downstream == text.length());
     
     if (string != rebalancedString)
-        replaceTextInNode(textNode, upstream, length, rebalancedString);
+        replaceTextInNodePreservingMarkers(textNode.release(), upstream, length, rebalancedString);
 }
 
 void CompositeEditCommand::prepareWhitespaceAtPositionForSplit(Position& position)
@@ -501,9 +528,9 @@ void CompositeEditCommand::prepareWhitespaceAtPositionForSplit(Position& positio
     Position previous(previousVisiblePos.deepEquivalent());
     
     if (isCollapsibleWhitespace(previousVisiblePos.characterAfter()) && previous.deprecatedNode()->isTextNode() && !previous.deprecatedNode()->hasTagName(brTag))
-        replaceTextInNode(static_cast<Text*>(previous.deprecatedNode()), previous.deprecatedEditingOffset(), 1, nonBreakingSpaceString());
+        replaceTextInNodePreservingMarkers(static_cast<Text*>(previous.deprecatedNode()), previous.deprecatedEditingOffset(), 1, nonBreakingSpaceString());
     if (isCollapsibleWhitespace(visiblePos.characterAfter()) && position.deprecatedNode()->isTextNode() && !position.deprecatedNode()->hasTagName(brTag))
-        replaceTextInNode(static_cast<Text*>(position.deprecatedNode()), position.deprecatedEditingOffset(), 1, nonBreakingSpaceString());
+        replaceTextInNodePreservingMarkers(static_cast<Text*>(position.deprecatedNode()), position.deprecatedEditingOffset(), 1, nonBreakingSpaceString());
 }
 
 void CompositeEditCommand::rebalanceWhitespace()
@@ -781,7 +808,7 @@ void CompositeEditCommand::cloneParagraphUnderNewElement(Position& start, Positi
     appendNode(topNode, blockElement);
     RefPtr<Node> lastNode = topNode;
 
-    if (start.deprecatedNode() != outerNode) {
+    if (start.deprecatedNode() != outerNode && lastNode->isElementNode()) {
         Vector<RefPtr<Node> > ancestors;
         
         // Insert each node from innerNode to outerNode (excluded) in a list.
@@ -922,6 +949,7 @@ void CompositeEditCommand::moveParagraphs(const VisiblePosition& startOfParagrap
     int startIndex = -1;
     int endIndex = -1;
     int destinationIndex = -1;
+    bool originalIsDirectional = endingSelection().isDirectional();
     if (preserveSelection && !endingSelection().isNone()) {
         VisiblePosition visibleStart = endingSelection().visibleStart();
         VisiblePosition visibleEnd = endingSelection().visibleEnd();
@@ -954,7 +982,7 @@ void CompositeEditCommand::moveParagraphs(const VisiblePosition& startOfParagrap
     // When we paste a fragment, spaces after the end and before the start are treated as though they were rendered.
     Position start = startOfParagraphToMove.deepEquivalent().downstream();
     Position end = endOfParagraphToMove.deepEquivalent().upstream();
-    
+     
     // start and end can't be used directly to create a Range; they are "editing positions"
     Position startRangeCompliant = start.parentAnchoredEquivalent();
     Position endRangeCompliant = end.parentAnchoredEquivalent();
@@ -1007,7 +1035,7 @@ void CompositeEditCommand::moveParagraphs(const VisiblePosition& startOfParagrap
     RefPtr<Range> startToDestinationRange(Range::create(document(), firstPositionInNode(document()->documentElement()), destination.deepEquivalent().parentAnchoredEquivalent()));
     destinationIndex = TextIterator::rangeLength(startToDestinationRange.get(), true);
 
-    setEndingSelection(destination);
+    setEndingSelection(VisibleSelection(destination, originalIsDirectional));
     ASSERT(endingSelection().isCaretOrRange());
     ReplaceSelectionCommand::CommandOptions options = ReplaceSelectionCommand::SelectReplacement | ReplaceSelectionCommand::MovingParagraph;
     if (!preserveStyle)
@@ -1030,7 +1058,7 @@ void CompositeEditCommand::moveParagraphs(const VisiblePosition& startOfParagrap
         RefPtr<Range> start = TextIterator::rangeFromLocationAndLength(document()->documentElement(), destinationIndex + startIndex, 0, true);
         RefPtr<Range> end = TextIterator::rangeFromLocationAndLength(document()->documentElement(), destinationIndex + endIndex, 0, true);
         if (start && end)
-            setEndingSelection(VisibleSelection(start->startPosition(), end->startPosition(), DOWNSTREAM));
+            setEndingSelection(VisibleSelection(start->startPosition(), end->startPosition(), DOWNSTREAM, originalIsDirectional));
     }
 }
 
@@ -1048,7 +1076,7 @@ bool CompositeEditCommand::breakOutOfEmptyListItem()
     // FIXME: Can't we do something better when the immediate parent wasn't a list node?
     if (!listNode
         || (!listNode->hasTagName(ulTag) && !listNode->hasTagName(olTag))
-        || !listNode->isContentEditable()
+        || !listNode->rendererIsEditable()
         || listNode == emptyListItem->rootEditableElement())
         return false;
 
@@ -1089,7 +1117,7 @@ bool CompositeEditCommand::breakOutOfEmptyListItem()
     }
 
     appendBlockPlaceholder(newBlock);
-    setEndingSelection(VisibleSelection(firstPositionInNode(newBlock.get()), DOWNSTREAM));
+    setEndingSelection(VisibleSelection(firstPositionInNode(newBlock.get()), DOWNSTREAM, endingSelection().isDirectional()));
 
     style->prepareToApplyAt(endingSelection().start());
     if (!style->isEmpty())
@@ -1127,7 +1155,7 @@ bool CompositeEditCommand::breakOutOfEmptyMailBlockquotedParagraph()
     // a second one.
     if (!isStartOfParagraph(atBR))
         insertNodeBefore(createBreakElement(document()), br);
-    setEndingSelection(VisibleSelection(atBR));
+    setEndingSelection(VisibleSelection(atBR, endingSelection().isDirectional()));
     
     // If this is an empty paragraph there must be a line break here.
     if (!lineBreakExistsAtVisiblePosition(caret))
@@ -1217,23 +1245,27 @@ Position CompositeEditCommand::positionAvoidingSpecialElementBoundary(const Posi
 
 // Splits the tree parent by parent until we reach the specified ancestor. We use VisiblePositions
 // to determine if the split is necessary. Returns the last split node.
-PassRefPtr<Node> CompositeEditCommand::splitTreeToNode(Node* start, Node* end, bool splitAncestor)
+PassRefPtr<Node> CompositeEditCommand::splitTreeToNode(Node* start, Node* end, bool shouldSplitAncestor)
 {
+    ASSERT(start);
+    ASSERT(end);
     ASSERT(start != end);
 
     RefPtr<Node> node;
-    for (node = start; node && node->parentNode() != end; node = node->parentNode()) {
+    if (shouldSplitAncestor && end->parentNode())
+        end = end->parentNode();
+
+    RefPtr<Node> endNode = end;
+    for (node = start; node && node->parentNode() != endNode; node = node->parentNode()) {
         if (!node->parentNode()->isElementNode())
             break;
-        VisiblePosition positionInParent(firstPositionInNode(node->parentNode()), DOWNSTREAM);
-        VisiblePosition positionInNode(firstPositionInOrBeforeNode(node.get()), DOWNSTREAM);
+        // Do not split a node when doing so introduces an empty node.
+        VisiblePosition positionInParent = firstPositionInNode(node->parentNode());
+        VisiblePosition positionInNode = firstPositionInOrBeforeNode(node.get());
         if (positionInParent != positionInNode)
-            applyCommandToComposite(SplitElementCommand::create(static_cast<Element*>(node->parentNode()), node));
-    }
-    if (splitAncestor) {
-        splitElement(static_cast<Element*>(end), node);
-        return node->parentNode();
+            splitElement(static_cast<Element*>(node->parentNode()), node);
     }
+
     return node.release();
 }
 
