@@ -39,11 +39,13 @@
 #include "Element.h"
 #include "EventNames.h"
 #include "Frame.h"
+#include "FrameSelection.h"
 #include "HTMLElement.h"
 #include "HTMLInputElement.h"
 #include "HTMLInterchange.h"
 #include "HTMLNames.h"
-#include "SelectionController.h"
+#include "NodeList.h"
+#include "RenderObject.h"
 #include "SmartReplace.h"
 #include "TextIterator.h"
 #include "htmlediting.h"
@@ -137,7 +139,7 @@ ReplacementFragment::ReplacementFragment(Document* document, DocumentFragment* f
     if (!m_fragment->firstChild())
         return;
     
-    Element* editableRoot = selection.rootEditableElement();
+    RefPtr<Element> editableRoot = selection.rootEditableElement();
     ASSERT(editableRoot);
     if (!editableRoot)
         return;
@@ -147,13 +149,17 @@ ReplacementFragment::ReplacementFragment(Document* document, DocumentFragment* f
     if (!editableRoot->getAttributeEventListener(eventNames().webkitBeforeTextInsertedEvent) &&
         // FIXME: Remove these checks once textareas and textfields actually register an event handler.
         !(shadowAncestorNode && shadowAncestorNode->renderer() && shadowAncestorNode->renderer()->isTextControl()) &&
-        editableRoot->isContentRichlyEditable()) {
+        editableRoot->rendererIsRichlyEditable()) {
         removeInterchangeNodes(m_fragment.get());
         return;
     }
 
-    Node* styleNode = selection.base().deprecatedNode();
-    RefPtr<StyledElement> holder = insertFragmentForTestRendering(styleNode);
+    RefPtr<Node> styleNode = selection.base().deprecatedNode();
+    RefPtr<StyledElement> holder = insertFragmentForTestRendering(styleNode.get());
+    if (!holder) {
+        removeInterchangeNodes(m_fragment.get());
+        return;
+    }
     
     RefPtr<Range> range = VisibleSelection::selectionFromContentsOfNode(holder.get()).toNormalizedRange();
     String text = plainText(range.get());
@@ -162,14 +168,14 @@ ReplacementFragment::ReplacementFragment(Document* document, DocumentFragment* f
     ExceptionCode ec = 0;
     editableRoot->dispatchEvent(evt, ec);
     ASSERT(ec == 0);
-    if (text != evt->text() || !editableRoot->isContentRichlyEditable()) {
+    if (text != evt->text() || !editableRoot->rendererIsRichlyEditable()) {
         restoreTestRenderingNodesToFragment(holder.get());
         removeNode(holder);
 
         m_fragment = createFragmentFromText(selection.toNormalizedRange().get(), evt->text());
         if (!m_fragment->firstChild())
             return;
-        holder = insertFragmentForTestRendering(styleNode);
+        holder = insertFragmentForTestRendering(styleNode.get());
     }
     
     removeInterchangeNodes(holder.get());
@@ -211,7 +217,7 @@ void ReplacementFragment::removeNode(PassRefPtr<Node> node)
     if (!node)
         return;
     
-    ContainerNode* parent = node->parentNode();
+    ContainerNode* parent = node->nonShadowBoundaryParentNode();
     if (!parent)
         return;
     
@@ -225,7 +231,7 @@ void ReplacementFragment::insertNodeBefore(PassRefPtr<Node> node, Node* refNode)
     if (!node || !refNode)
         return;
         
-    ContainerNode* parent = refNode->parentNode();
+    ContainerNode* parent = refNode->nonShadowBoundaryParentNode();
     if (!parent)
         return;
         
@@ -245,9 +251,10 @@ PassRefPtr<StyledElement> ReplacementFragment::insertFragmentForTestRendering(No
     ExceptionCode ec = 0;
 
     // Copy the whitespace and user-select style from the context onto this element.
+    // Walk up past <br> elements which may be placeholders and might have their own specified styles.
     // FIXME: We should examine other style properties to see if they would be appropriate to consider during the test rendering.
     Node* n = context;
-    while (n && !n->isElementNode())
+    while (n && (!n->isElementNode() || n->hasTagName(brTag)))
         n = n->parentNode();
     if (n) {
         RefPtr<CSSComputedStyleDeclaration> conFontStyle = computedStyle(n);
@@ -285,7 +292,7 @@ void ReplacementFragment::restoreTestRenderingNodesToFragment(StyledElement* hol
 
 void ReplacementFragment::removeUnrenderedNodes(Node* holder)
 {
-    Vector<Node*> unrendered;
+    Vector<RefPtr<Node> > unrendered;
 
     for (Node* node = holder->firstChild(); node; node = node->traverseNextNode(holder))
         if (!isNodeRendered(node) && !isTableStructureNode(node))
@@ -469,25 +476,59 @@ bool ReplaceSelectionCommand::shouldMerge(const VisiblePosition& source, const V
 
 // Style rules that match just inserted elements could change their appearance, like
 // a div inserted into a document with div { display:inline; }.
-void ReplaceSelectionCommand::negateStyleRulesThatAffectAppearance()
+void ReplaceSelectionCommand::removeRedundantStylesAndKeepStyleSpanInline()
 {
-    for (RefPtr<Node> node = m_firstNodeInserted.get(); node; node = node->traverseNextNode()) {
+    RefPtr<Node> pastEndNode = m_lastLeafInserted ? m_lastLeafInserted->traverseNextNode() : 0;
+    RefPtr<Node> next;
+    for (RefPtr<Node> node = m_firstNodeInserted.get(); node && node != pastEndNode; node = next) {
         // FIXME: <rdar://problem/5371536> Style rules that match pasted content can change it's appearance
-        if (isStyleSpan(node.get())) {
-            HTMLElement* e = toHTMLElement(node.get());
+
+        next = node->traverseNextNode();
+        if (!node->isStyledElement())
+            continue;
+
+        StyledElement* element = static_cast<StyledElement*>(node.get());
+
+        CSSMutableStyleDeclaration* inlineStyle = element->inlineStyleDecl();
+        RefPtr<EditingStyle> newInlineStyle = EditingStyle::create(inlineStyle);
+        if (inlineStyle) {
+            ContainerNode* context = element->parentNode();
+
+            // If Mail wraps the fragment with a Paste as Quotation blockquote, or if you're pasting into a quoted region,
+            // styles from blockquoteNode are allowed to override those from the source document, see <rdar://problem/4930986> and <rdar://problem/5089327>.
+            Node* blockquoteNode = isMailPasteAsQuotationNode(context) ? context : enclosingNodeOfType(firstPositionInNode(context), isBlockNodeToRetainAppearance, CanCrossEditingBoundary);
+            if (blockquoteNode)
+                newInlineStyle->removeStyleFromRulesAndContext(element, document()->documentElement());
+
+            newInlineStyle->removeStyleFromRulesAndContext(element, context);
+        }
+
+        if (!inlineStyle || newInlineStyle->isEmpty()) {
+            if (isStyleSpanOrSpanWithOnlyStyleAttribute(element))
+                removeNodePreservingChildren(element);
+            else
+                removeNodeAttribute(element, styleAttr);
+        } else if (newInlineStyle->style()->length() != inlineStyle->length())
+            setNodeAttribute(element, styleAttr, newInlineStyle->style()->cssText());
+
+        // WebKit used to not add display: inline and float: none on copy.
+        // Keep this code around for backward compatibility
+        if (isLegacyAppleStyleSpan(element)) {
+            if (!element->firstChild()) {
+                removeNodePreservingChildren(element);
+                continue;
+            }
             // There are other styles that style rules can give to style spans,
             // but these are the two important ones because they'll prevent
             // inserted content from appearing in the right paragraph.
             // FIXME: Hyatt is concerned that selectively using display:inline will give inconsistent
             // results. We already know one issue because td elements ignore their display property
             // in quirks mode (which Mail.app is always in). We should look for an alternative.
-            if (isBlock(e))
-                e->getInlineStyleDecl()->setProperty(CSSPropertyDisplay, CSSValueInline);
-            if (e->renderer() && e->renderer()->style()->floating() != FNONE)
-                e->getInlineStyleDecl()->setProperty(CSSPropertyFloat, CSSValueNone);
+            if (isBlock(element))
+                element->getInlineStyleDecl()->setProperty(CSSPropertyDisplay, CSSValueInline);
+            if (element->renderer() && element->renderer()->style()->isFloating())
+                element->getInlineStyleDecl()->setProperty(CSSPropertyFloat, CSSValueNone);
         }
-        if (node == m_lastLeafInserted)
-            break;
     }
 }
 
@@ -548,6 +589,22 @@ VisiblePosition ReplaceSelectionCommand::positionAtStartOfInsertedContent()
     return VisiblePosition(nextCandidate(positionInParentBeforeNode(m_firstNodeInserted.get())));
 }
 
+static void removeHeadContents(ReplacementFragment& fragment)
+{
+    Node* next = 0;
+    for (Node* node = fragment.firstChild(); node; node = next) {
+        if (node->hasTagName(baseTag)
+            || node->hasTagName(linkTag)
+            || node->hasTagName(metaTag)
+            || node->hasTagName(styleTag)
+            || node->hasTagName(titleTag)) {
+            next = node->traverseNextSibling();
+            fragment.removeNode(node);
+        } else
+            next = node->traverseNextNode();
+    }
+}
+
 // Remove style spans before insertion if they are unnecessary.  It's faster because we'll 
 // avoid doing a layout.
 static bool handleStyleSpansBeforeInsertion(ReplacementFragment& fragment, const Position& insertionPos)
@@ -561,29 +618,20 @@ static bool handleStyleSpansBeforeInsertion(ReplacementFragment& fragment, const
 
     // Either there are no style spans in the fragment or a WebKit client has added content to the fragment
     // before inserting it.  Look for and handle style spans after insertion.
-    if (!isStyleSpan(topNode))
+    if (!isLegacyAppleStyleSpan(topNode))
         return false;
 
-    Node* sourceDocumentStyleSpan = topNode;
-    RefPtr<Node> copiedRangeStyleSpan = sourceDocumentStyleSpan->firstChild();
-
+    Node* wrappingStyleSpan = topNode;
     RefPtr<EditingStyle> styleAtInsertionPos = EditingStyle::create(insertionPos.parentAnchoredEquivalent());
     String styleText = styleAtInsertionPos->style()->cssText();
 
     // FIXME: This string comparison is a naive way of comparing two styles.
     // We should be taking the diff and check that the diff is empty.
-    if (styleText == static_cast<Element*>(sourceDocumentStyleSpan)->getAttribute(styleAttr)) {
-        fragment.removeNodePreservingChildren(sourceDocumentStyleSpan);
-        if (!isStyleSpan(copiedRangeStyleSpan.get()))
-            return true;
-    }
+    if (styleText != static_cast<Element*>(wrappingStyleSpan)->getAttribute(styleAttr))
+        return false;
 
-    if (isStyleSpan(copiedRangeStyleSpan.get()) && styleText == static_cast<Element*>(copiedRangeStyleSpan.get())->getAttribute(styleAttr)) {
-        fragment.removeNodePreservingChildren(copiedRangeStyleSpan.get());
-        return true;
-    }
-
-    return false;
+    fragment.removeNodePreservingChildren(wrappingStyleSpan);
+    return true;
 }
 
 // At copy time, WebKit wraps copied content in a span that contains the source document's 
@@ -596,86 +644,45 @@ static bool handleStyleSpansBeforeInsertion(ReplacementFragment& fragment, const
 // or at copy time.
 void ReplaceSelectionCommand::handleStyleSpans()
 {
-    Node* sourceDocumentStyleSpan = 0;
-    Node* copiedRangeStyleSpan = 0;
+    HTMLElement* wrappingStyleSpan = 0;
     // The style span that contains the source document's default style should be at
     // the top of the fragment, but Mail sometimes adds a wrapper (for Paste As Quotation),
     // so search for the top level style span instead of assuming it's at the top.
     for (Node* node = m_firstNodeInserted.get(); node; node = node->traverseNextNode()) {
-        if (isStyleSpan(node)) {
-            sourceDocumentStyleSpan = node;
-            // If the copied Range's common ancestor had user applied inheritable styles
-            // on it, they'll be on a second style span, just below the one that holds the 
-            // document defaults.
-            if (isStyleSpan(node->firstChild()))
-                copiedRangeStyleSpan = node->firstChild();
+        if (isLegacyAppleStyleSpan(node)) {
+            wrappingStyleSpan = toHTMLElement(node);
             break;
         }
     }
     
     // There might not be any style spans if we're pasting from another application or if 
     // we are here because of a document.execCommand("InsertHTML", ...) call.
-    if (!sourceDocumentStyleSpan)
+    if (!wrappingStyleSpan)
         return;
 
-    RefPtr<EditingStyle> sourceDocumentStyle = EditingStyle::create(toHTMLElement(sourceDocumentStyleSpan)->getInlineStyleDecl());
-    ContainerNode* context = sourceDocumentStyleSpan->parentNode();
+    RefPtr<EditingStyle> style = EditingStyle::create(wrappingStyleSpan->getInlineStyleDecl());
+    ContainerNode* context = wrappingStyleSpan->parentNode();
 
     // If Mail wraps the fragment with a Paste as Quotation blockquote, or if you're pasting into a quoted region,
     // styles from blockquoteNode are allowed to override those from the source document, see <rdar://problem/4930986> and <rdar://problem/5089327>.
     Node* blockquoteNode = isMailPasteAsQuotationNode(context) ? context : enclosingNodeOfType(firstPositionInNode(context), isMailBlockquote, CanCrossEditingBoundary);
-    if (blockquoteNode) {
-        sourceDocumentStyle->removeStyleConflictingWithStyleOfNode(blockquoteNode);
-        context = blockquoteNode->parentNode();
-    }
+    if (blockquoteNode)
+        context = document()->documentElement();
 
     // This operation requires that only editing styles to be removed from sourceDocumentStyle.
-    sourceDocumentStyle->prepareToApplyAt(firstPositionInNode(context));
+    style->prepareToApplyAt(firstPositionInNode(context));
 
     // Remove block properties in the span's style. This prevents properties that probably have no effect 
     // currently from affecting blocks later if the style is cloned for a new block element during a future 
     // editing operation.
     // FIXME: They *can* have an effect currently if blocks beneath the style span aren't individually marked
     // with block styles by the editing engine used to style them.  WebKit doesn't do this, but others might.
-    sourceDocumentStyle->removeBlockProperties();
+    style->removeBlockProperties();
 
-    // The styles on sourceDocumentStyleSpan are all redundant, and there is no copiedRangeStyleSpan
-    // to consider.  We're finished.
-    if (sourceDocumentStyle->isEmpty() && !copiedRangeStyleSpan) {
-        removeNodePreservingChildren(sourceDocumentStyleSpan);
-        return;
-    }
-
-    // There are non-redundant styles on sourceDocumentStyleSpan, but there is no
-    // copiedRangeStyleSpan.  Remove the span, because it could be surrounding block elements,
-    // and apply the styles to its children.
-    if (!sourceDocumentStyle->isEmpty() && !copiedRangeStyleSpan) {
-        copyStyleToChildren(sourceDocumentStyleSpan, sourceDocumentStyle->style()); 
-        removeNodePreservingChildren(sourceDocumentStyleSpan);
-        return;
-    }
-    
-    RefPtr<EditingStyle> copiedRangeStyle = EditingStyle::create(toHTMLElement(copiedRangeStyleSpan)->getInlineStyleDecl());
-
-    // We're going to put sourceDocumentStyleSpan's non-redundant styles onto copiedRangeStyleSpan,
-    // as long as they aren't overridden by ones on copiedRangeStyleSpan.
-    copiedRangeStyle->style()->merge(sourceDocumentStyle->style(), false);
-
-    removeNodePreservingChildren(sourceDocumentStyleSpan);
-
-    // Remove redundant styles.
-    context = copiedRangeStyleSpan->parentNode();
-    copiedRangeStyle->prepareToApplyAt(firstPositionInNode(context));
-    copiedRangeStyle->removeBlockProperties();
-    if (copiedRangeStyle->isEmpty()) {
-        removeNodePreservingChildren(copiedRangeStyleSpan);
-        return;
-    }
-
-    // Clear the redundant styles from the span's style attribute.
-    // FIXME: If font-family:-webkit-monospace is non-redundant, then the font-size should stay, even if it
-    // appears redundant.
-    setNodeAttribute(static_cast<Element*>(copiedRangeStyleSpan), styleAttr, copiedRangeStyle->style()->cssText());
+    if (style->isEmpty() || !wrappingStyleSpan->firstChild())
+        removeNodePreservingChildren(wrappingStyleSpan);
+    else
+        setNodeAttribute(wrappingStyleSpan, styleAttr, style->style()->cssText());
 }
 
 // Take the style attribute of a span and apply it to it's children instead.  This allows us to
@@ -776,6 +783,27 @@ static Node* enclosingInline(Node* node)
     return node;
 }
 
+static bool isInlineNodeWithStyle(const Node* node)
+{
+    // We don't want to skip over any block elements.
+    if (isBlock(node))
+        return false;
+
+    if (!node->isHTMLElement())
+        return false;
+
+    // We can skip over elements whose class attribute is
+    // one of our internal classes.
+    const HTMLElement* element = static_cast<const HTMLElement*>(node);
+    AtomicString classAttributeValue = element->getAttribute(classAttr);
+    if (classAttributeValue == AppleTabSpanClass
+        || classAttributeValue == AppleConvertedSpace
+        || classAttributeValue == ApplePasteAsQuotation)
+        return true;
+
+    return EditingStyle::elementIsStyledSpanOrHTMLEquivalent(element);
+}
+    
 void ReplaceSelectionCommand::doApply()
 {
     VisibleSelection selection = endingSelection();
@@ -898,9 +926,6 @@ void ReplaceSelectionCommand::doApply()
         else if (isStartOfBlock(visibleInsertionPos))
             insertionPos = positionInParentBeforeNode(startBlock);
     }
-
-    // Paste into run of tabs splits the tab span.
-    insertionPos = positionOutsideTabSpan(insertionPos);
     
     // Paste at start or end of link goes outside of link.
     insertionPos = positionAvoidingSpecialElementBoundary(insertionPos);
@@ -909,14 +934,40 @@ void ReplaceSelectionCommand::doApply()
     // any work performed after this that queries or uses the typing style.
     if (Frame* frame = document()->frame())
         frame->selection()->clearTypingStyle();
-    
-    bool handledStyleSpans = handleStyleSpansBeforeInsertion(fragment, insertionPos);
+
+    removeHeadContents(fragment);
 
     // We don't want the destination to end up inside nodes that weren't selected.  To avoid that, we move the
     // position forward without changing the visible position so we're still at the same visible location, but
     // outside of preceding tags.
     insertionPos = positionAvoidingPrecedingNodes(insertionPos);
 
+    // Paste into run of tabs splits the tab span.
+    insertionPos = positionOutsideTabSpan(insertionPos);
+
+    bool handledStyleSpans = handleStyleSpansBeforeInsertion(fragment, insertionPos);
+
+    // If we are not trying to match the destination style we prefer a position
+    // that is outside inline elements that provide style.
+    // This way we can produce a less verbose markup.
+    // We can skip this optimization for fragments not wrapped in one of
+    // our style spans and for positions inside list items
+    // since insertAsListItems already does the right thing.
+    if (!m_matchStyle && !enclosingList(insertionPos.containerNode())) {
+        if (insertionPos.containerNode()->isTextNode() && insertionPos.offsetInContainerNode() && !insertionPos.atLastEditingPositionForNode()) {
+            splitTextNode(insertionPos.containerText(), insertionPos.offsetInContainerNode());
+            insertionPos = firstPositionInNode(insertionPos.containerNode());
+        }
+
+        if (RefPtr<Node> nodeToSplitTo = highestEnclosingNodeOfType(insertionPos, isInlineNodeWithStyle, CannotCrossEditingBoundary,
+            enclosingBlock(insertionPos.containerNode()))) {
+            if (insertionPos.containerNode() != nodeToSplitTo->parentNode()) {
+                nodeToSplitTo = splitTreeToNode(insertionPos.anchorNode(), nodeToSplitTo->parentNode()).get();
+                insertionPos = positionInParentBeforeNode(nodeToSplitTo.get());
+            }
+        }
+    }
+
     // FIXME: When pasting rich content we're often prevented from heading down the fast path by style spans.  Try
     // again here if they've been removed.
     
@@ -942,8 +993,8 @@ void ReplaceSelectionCommand::doApply()
     fragment.removeNode(refNode);
 
     Node* blockStart = enclosingBlock(insertionPos.deprecatedNode());
-    if ((isListElement(refNode.get()) || (isStyleSpan(refNode.get()) && isListElement(refNode->firstChild())))
-        && blockStart->renderer()->isListItem())
+    if ((isListElement(refNode.get()) || (isLegacyAppleStyleSpan(refNode.get()) && isListElement(refNode->firstChild())))
+        && blockStart && blockStart->renderer()->isListItem())
         refNode = insertAsListItems(refNode, blockStart, insertionPos);
     else
         insertNodeAtAndUpdateNodesInserted(refNode, insertionPos);
@@ -971,7 +1022,7 @@ void ReplaceSelectionCommand::doApply()
     
     removeUnrenderedTextNodesAtEnds();
     
-    negateStyleRulesThatAffectAppearance();
+    removeRedundantStylesAndKeepStyleSpanInline();
     
     if (!handledStyleSpans)
         handleStyleSpans();
@@ -1070,10 +1121,9 @@ void ReplaceSelectionCommand::doApply()
     
     // Add spaces for smart replace.
     if (m_smartReplace && currentRoot) {
-        // Disable smart replace for password fields.
-        Node* start = currentRoot->shadowAncestorNode();
-        if (start->hasTagName(inputTag) && static_cast<HTMLInputElement*>(start)->isPasswordField())
-            m_smartReplace = false;
+        Element* textControl = enclosingTextFormControl(firstPositionInNode(currentRoot));
+        if (textControl && textControl->hasTagName(inputTag) && static_cast<HTMLInputElement*>(textControl)->isPasswordField())
+            m_smartReplace = false; // Disable smart replace for password fields.
     }
     if (m_smartReplace) {
         bool needsTrailingSpace = !isEndOfParagraph(endOfInsertedContent) &&
@@ -1167,9 +1217,9 @@ void ReplaceSelectionCommand::completeHTMLReplacement(const Position &lastPositi
         return;
     
     if (m_selectReplacement)
-        setEndingSelection(VisibleSelection(start, end, SEL_DEFAULT_AFFINITY));
+        setEndingSelection(VisibleSelection(start, end, SEL_DEFAULT_AFFINITY, endingSelection().isDirectional()));
     else
-        setEndingSelection(VisibleSelection(end, SEL_DEFAULT_AFFINITY));
+        setEndingSelection(VisibleSelection(end, SEL_DEFAULT_AFFINITY, endingSelection().isDirectional()));
 }
 
 EditAction ReplaceSelectionCommand::editingAction() const
@@ -1200,8 +1250,10 @@ void ReplaceSelectionCommand::insertNodeBeforeAndUpdateNodesInserted(PassRefPtr<
 
 // If the user is inserting a list into an existing list, instead of nesting the list,
 // we put the list items into the existing list.
-Node* ReplaceSelectionCommand::insertAsListItems(PassRefPtr<Node> listElement, Node* insertionBlock, const Position& insertPos)
+Node* ReplaceSelectionCommand::insertAsListItems(PassRefPtr<Node> prpListElement, Node* insertionBlock, const Position& insertPos)
 {
+    RefPtr<Node> listElement = prpListElement;
+
     while (listElement->hasChildNodes() && isListElement(listElement->firstChild()) && listElement->childNodeCount() == 1)
         listElement = listElement->firstChild();
 
@@ -1268,20 +1320,12 @@ bool ReplaceSelectionCommand::performTrivialReplace(const ReplacementFragment& f
     
     Text* textNode = static_cast<Text*>(fragment.firstChild());
     // Our fragment creation code handles tabs, spaces, and newlines, so we don't have to worry about those here.
-    String text(textNode->data());
-    
-    Position start = endingSelection().start().parentAnchoredEquivalent();
-    Position end = endingSelection().end().parentAnchoredEquivalent();
-    ASSERT(start.anchorType() == Position::PositionIsOffsetInAnchor);
-    ASSERT(end.anchorType() == Position::PositionIsOffsetInAnchor);
 
-    if (start.containerNode() != end.containerNode() || !start.containerNode()->isTextNode())
+    Position start = endingSelection().start();
+    Position end = replaceSelectedTextInNode(textNode->data());
+    if (end.isNull())
         return false;
 
-    replaceTextInNode(static_cast<Text*>(start.containerNode()), start.offsetInContainerNode(), end.offsetInContainerNode() - start.offsetInContainerNode(), text);
-
-    end = Position(start.containerNode(), start.offsetInContainerNode() + text.length(), Position::PositionIsOffsetInAnchor);
-
     VisibleSelection selectionAfterReplace(m_selectReplacement ? start : end, end);
 
     setEndingSelection(selectionAfterReplace);
