@@ -27,6 +27,7 @@
 #include "Attribute.h"
 #include "EventNames.h"
 #include "Frame.h"
+#include "FrameLoaderClient.h"
 #include "FrameLoaderTypes.h"
 #include "HTMLImageElement.h"
 #include "HTMLNames.h"
@@ -37,8 +38,8 @@
 #include "PingLoader.h"
 #include "RenderImage.h"
 #include "ResourceHandle.h"
+#include "SecurityOrigin.h"
 #include "Settings.h"
-#include "UserGestureIndicator.h"
 
 namespace WebCore {
 
@@ -72,7 +73,7 @@ static unsigned parsePortFromStringPosition(const String& value, unsigned portSt
 
 bool HTMLAnchorElement::supportsFocus() const
 {
-    if (isContentEditable())
+    if (rendererIsEditable())
         return HTMLElement::supportsFocus();
     // If not a link we should still be able to focus the element if it has tabIndex.
     return isLink() || HTMLElement::supportsFocus();
@@ -147,14 +148,11 @@ void HTMLAnchorElement::defaultEventHandler(Event* event)
         }
 
         if (isLinkClick(event) && treatLinkAsLiveForEventType(eventType(event))) {
-            String url = stripLeadingAndTrailingHTMLSpaces(getAttribute(hrefAttr));
-            appendServerMapMousePosition(url, event);
-            handleLinkClick(event, document(), url, getAttribute(targetAttr), hasRel(RelationNoReferrer));
-            sendPings(document()->completeURL(url));
+            handleClick(event);
             return;
         }
 
-        if (isContentEditable()) {
+        if (rendererIsEditable()) {
             // This keeps track of the editable block that the selection was in (if it was in one) just before the link was clicked
             // for the LiveWhenNotFocused editable link behavior
             if (event->type() == eventNames().mousedownEvent && event->isMouseEvent() && static_cast<MouseEvent*>(event)->button() != RightButton && document()->frame() && document()->frame()->selection()) {
@@ -174,7 +172,7 @@ void HTMLAnchorElement::defaultEventHandler(Event* event)
 
 void HTMLAnchorElement::setActive(bool down, bool pause)
 {
-    if (isContentEditable()) {
+    if (rendererIsEditable()) {
         EditableLinkBehavior editableLinkBehavior = EditableLinkDefaultBehavior;
         if (Settings* settings = document()->settings())
             editableLinkBehavior = settings->editableLinkBehavior();
@@ -247,7 +245,7 @@ bool HTMLAnchorElement::canStartSelection() const
     // FIXME: We probably want this same behavior in SVGAElement too
     if (!isLink())
         return HTMLElement::canStartSelection();
-    return isContentEditable();
+    return rendererIsEditable();
 }
 
 bool HTMLAnchorElement::draggable() const
@@ -304,7 +302,7 @@ String HTMLAnchorElement::target() const
 String HTMLAnchorElement::hash() const
 {
     String fragmentIdentifier = href().fragmentIdentifier();
-    return fragmentIdentifier.isEmpty() ? "" : "#" + fragmentIdentifier;
+    return fragmentIdentifier.isEmpty() ? emptyString() : "#" + fragmentIdentifier;
 }
 
 void HTMLAnchorElement::setHash(const String& value)
@@ -405,7 +403,10 @@ void HTMLAnchorElement::setPathname(const String& value)
 
 String HTMLAnchorElement::port() const
 {
-    return String::number(href().port());
+    if (href().hasPort())
+        return String::number(href().port());
+
+    return emptyString();
 }
 
 void HTMLAnchorElement::setPort(const String& value)
@@ -441,7 +442,7 @@ void HTMLAnchorElement::setProtocol(const String& value)
 String HTMLAnchorElement::search() const
 {
     String query = href().query();
-    return query.isEmpty() ? "" : "?" + query;
+    return query.isEmpty() ? emptyString() : "?" + query;
 }
 
 String HTMLAnchorElement::origin() const
@@ -467,7 +468,7 @@ void HTMLAnchorElement::setSearch(const String& value)
     setHref(url.string());
 }
 
-String HTMLAnchorElement::text() const
+String HTMLAnchorElement::text()
 {
     return innerText();
 }
@@ -492,6 +493,37 @@ void HTMLAnchorElement::sendPings(const KURL& destinationURL)
         PingLoader::sendPing(document()->frame(), document()->completeURL(pingURLs[i]), destinationURL);
 }
 
+void HTMLAnchorElement::handleClick(Event* event)
+{
+    event->setDefaultHandled();
+
+    Frame* frame = document()->frame();
+    if (!frame)
+        return;
+
+    String url = stripLeadingAndTrailingHTMLSpaces(fastGetAttribute(hrefAttr));
+    appendServerMapMousePosition(url, event);
+    KURL kurl = document()->completeURL(url);
+
+#if ENABLE(DOWNLOAD_ATTRIBUTE)
+    if (hasAttribute(downloadAttr)) {
+        ResourceRequest request(kurl);
+
+        if (!hasRel(RelationNoReferrer)) {
+            String referrer = frame->loader()->outgoingReferrer();
+            if (!referrer.isEmpty() && !SecurityOrigin::shouldHideReferrer(kurl, referrer))
+                request.setHTTPReferrer(referrer);
+            frame->loader()->addExtraFieldsToMainResourceRequest(request);
+        }
+
+        frame->loader()->client()->startDownload(request, fastGetAttribute(downloadAttr));
+    } else
+#endif
+        frame->loader()->urlSelected(kurl, target(), event, false, false, hasRel(RelationNoReferrer) ? NoReferrer : SendReferrer);
+
+    sendPings(kurl);
+}
+
 HTMLAnchorElement::EventType HTMLAnchorElement::eventType(Event* event)
 {
     if (!event->isMouseEvent())
@@ -501,7 +533,7 @@ HTMLAnchorElement::EventType HTMLAnchorElement::eventType(Event* event)
 
 bool HTMLAnchorElement::treatLinkAsLiveForEventType(EventType eventType) const
 {
-    if (!isContentEditable())
+    if (!rendererIsEditable())
         return true;
 
     Settings* settings = document()->settings();
