@@ -34,7 +34,7 @@
 
 #if PLATFORM(CHROMIUM) || PLATFORM(QT)
 #define DefaultInterpolationQuality InterpolationMedium
-#elif PLATFORM(CG)
+#elif USE(CG)
 #define DefaultInterpolationQuality InterpolationLow
 #else
 #define DefaultInterpolationQuality InterpolationDefault
@@ -97,9 +97,10 @@ public:
     // Used for rendering
     void didDraw(const FloatRect&);
 
-    void paint(GraphicsContext*, const IntRect&);
+    void paint(GraphicsContext*, const LayoutRect&, bool useLowQualityScale = false);
 
     GraphicsContext* drawingContext() const;
+    GraphicsContext* existingDrawingContext() const;
 
     CanvasRenderingContext* renderingContext() const { return m_context.get(); }
 
@@ -107,12 +108,13 @@ public:
     Image* copiedImage() const;
     void clearCopiedImage();
     PassRefPtr<ImageData> getImageData();
+    void makePresentationCopy();
+    void clearPresentationCopy();
 
-    IntRect convertLogicalToDevice(const FloatRect&) const;
-    IntSize convertLogicalToDevice(const FloatSize&) const;
-    IntSize convertToValidDeviceSize(float width, float height) const;
+    FloatRect convertLogicalToDevice(const FloatRect&) const;
+    FloatSize convertLogicalToDevice(const FloatSize&) const;
 
-    const SecurityOrigin& securityOrigin() const;
+    SecurityOrigin* securityOrigin() const;
     void setOriginTainted() { m_originClean = false; }
     bool originClean() const { return m_originClean; }
 
@@ -125,6 +127,9 @@ public:
 #endif
 
     void makeRenderingResultsAvailable();
+    bool hasCreatedImageBuffer() const { return m_hasCreatedImageBuffer; }
+
+    bool shouldAccelerate(const IntSize&) const;
 
 private:
     HTMLCanvasElement(const QualifiedName&, Document*);
@@ -137,7 +142,6 @@ private:
     void createImageBuffer() const;
 
     void setSurfaceSize(const IntSize&);
-    bool hasCreatedImageBuffer() const { return m_hasCreatedImageBuffer; }
 
     HashSet<CanvasObserver*> m_observers;
 
@@ -150,13 +154,14 @@ private:
     bool m_ignoreReset;
     FloatRect m_dirtyRect;
 
-    float m_pageScaleFactor;
+    float m_deviceScaleFactor;
     bool m_originClean;
 
     // m_createdImageBuffer means we tried to malloc the buffer.  We didn't necessarily get it.
     mutable bool m_hasCreatedImageBuffer;
     mutable OwnPtr<ImageBuffer> m_imageBuffer;
     
+    mutable RefPtr<Image> m_presentedImage;
     mutable RefPtr<Image> m_copiedImage; // FIXME: This is temporary for platforms that have to copy the image buffer to render (and for CSSCanvasValue).
 };
 
