@@ -58,11 +58,6 @@ PassRefPtr<HTMLSelectElement> HTMLSelectElement::create(const QualifiedName& tag
     return adoptRef(new HTMLSelectElement(tagName, document, form));
 }
 
-void HTMLSelectElement::recalcStyle(StyleChange change)
-{
-    HTMLFormControlElementWithState::recalcStyle(change);
-}
-
 const AtomicString& HTMLSelectElement::formControlType() const
 {
     DEFINE_STATIC_LOCAL(const AtomicString, selectMultiple, ("select-multiple"));
@@ -84,7 +79,6 @@ void HTMLSelectElement::deselectItems(HTMLOptionElement* excludeElement)
 void HTMLSelectElement::setSelectedIndex(int optionIndex, bool deselect)
 {
     SelectElement::setSelectedIndex(m_data, this, optionIndex, deselect, false, false);
-    setNeedsValidityCheck();
 }
 
 void HTMLSelectElement::setSelectedIndexByUser(int optionIndex, bool deselect, bool fireOnChangeNow, bool allowMultipleSelection)
@@ -107,7 +101,6 @@ void HTMLSelectElement::setSelectedIndexByUser(int optionIndex, bool deselect, b
         return;
     
     SelectElement::setSelectedIndex(m_data, this, optionIndex, deselect, fireOnChangeNow, true);
-    setNeedsValidityCheck();
 }
 
 bool HTMLSelectElement::hasPlaceholderLabelOption() const
@@ -131,7 +124,7 @@ bool HTMLSelectElement::hasPlaceholderLabelOption() const
     if (listIndex < 0)
         return false;
     HTMLOptionElement* option = static_cast<HTMLOptionElement*>(listItems()[listIndex]);
-    return !option->disabled() && !listIndex && option->value().isEmpty();
+    return !listIndex && option->value().isEmpty();
 }
 
 bool HTMLSelectElement::valueMissing() const
@@ -246,9 +239,8 @@ void HTMLSelectElement::restoreFormControlState(const String& state)
     setNeedsValidityCheck();
 }
 
-void HTMLSelectElement::parseMappedAttribute(Attribute* attr) 
+void HTMLSelectElement::parseMappedAttribute(Attribute* attr)
 {
-    bool oldUsesMenuList = m_data.usesMenuList();
     if (attr->name() == sizeAttr) {
         int oldSize = m_data.size();
         // Set the attribute value to a number.
@@ -265,9 +257,8 @@ void HTMLSelectElement::parseMappedAttribute(Attribute* attr)
 
         m_data.setSize(size);
         setNeedsValidityCheck();
-        if ((oldUsesMenuList != m_data.usesMenuList() || (!oldUsesMenuList && m_data.size() != oldSize)) && attached()) {
-            detach();
-            attach();
+        if (m_data.size() != oldSize && attached()) {
+            reattach();
             setRecalcListItems();
         }
     } else if (attr->name() == multipleAttr)
@@ -370,16 +361,16 @@ void HTMLSelectElement::reset()
     setNeedsValidityCheck();
 }
 
-void HTMLSelectElement::dispatchFocusEvent()
+void HTMLSelectElement::dispatchFocusEvent(PassRefPtr<Node> oldFocusedNode)
 {
     SelectElement::dispatchFocusEvent(m_data, this);
-    HTMLFormControlElementWithState::dispatchFocusEvent();
+    HTMLFormControlElementWithState::dispatchFocusEvent(oldFocusedNode);
 }
 
-void HTMLSelectElement::dispatchBlurEvent()
+void HTMLSelectElement::dispatchBlurEvent(PassRefPtr<Node> newFocusedNode)
 {
     SelectElement::dispatchBlurEvent(m_data, this);
-    HTMLFormControlElementWithState::dispatchBlurEvent();
+    HTMLFormControlElementWithState::dispatchBlurEvent(newFocusedNode);
 }
 
 void HTMLSelectElement::defaultEventHandler(Event* event)
@@ -434,12 +425,14 @@ void HTMLSelectElement::accessKeySetSelectedIndex(int index)
     
 void HTMLSelectElement::setMultiple(bool multiple)
 {
+    bool oldMultiple = this->multiple();
     int oldSelectedIndex = selectedIndex();
     setAttribute(multipleAttr, multiple ? "" : 0);
 
     // Restore selectedIndex after changing the multiple flag to preserve
     // selection as single-line and multi-line has different defaults.
-    setSelectedIndex(oldSelectedIndex);
+    if (oldMultiple != this->multiple())
+        setSelectedIndex(oldSelectedIndex);
 }
 
 void HTMLSelectElement::setSize(int size)
