@@ -39,6 +39,7 @@
 #include "Page.h"
 #include "RenderLayer.h"
 #include "RenderTextControlSingleLine.h"
+#include "ScriptController.h"
 #include "ScrollbarTheme.h"
 #include "SpeechInput.h"
 #include "SpeechInputEvent.h"
@@ -47,63 +48,34 @@ namespace WebCore {
 
 using namespace HTMLNames;
 
-TextControlInnerElement::TextControlInnerElement(Document* document, HTMLElement* shadowParent)
+TextControlInnerElement::TextControlInnerElement(Document* document)
     : HTMLDivElement(divTag, document)
 {
-    setShadowHost(shadowParent);
+    setHasCustomStyleForRenderer();
 }
 
-PassRefPtr<TextControlInnerElement> TextControlInnerElement::create(HTMLElement* shadowParent)
+PassRefPtr<TextControlInnerElement> TextControlInnerElement::create(Document* document)
 {
-    return adoptRef(new TextControlInnerElement(shadowParent->document(), shadowParent));
+    return adoptRef(new TextControlInnerElement(document));
 }
 
-void TextControlInnerElement::attachInnerElement(Node* parent, PassRefPtr<RenderStyle> style, RenderArena* arena)
+PassRefPtr<RenderStyle> TextControlInnerElement::customStyleForRenderer()
 {
-    // When adding these elements, create the renderer & style first before adding to the DOM.
-    // Otherwise, the render tree will create some anonymous blocks that will mess up our layout.
-
-    // Create the renderer with the specified style
-    RenderObject* renderer = createRenderer(arena, style.get());
-    if (renderer) {
-        setRenderer(renderer);
-        renderer->setStyle(style);
-    }
-
-    // Set these explicitly since this normally happens during an attach()
-    setAttached();
-    setInDocument();
-
-    // For elements not yet in shadow DOM, add the node to the DOM normally.
-    if (!isShadowRoot()) {
-        // FIXME: This code seems very wrong.  Why are we magically adding |this| to the DOM here?
-        //        We shouldn't be calling parser API methods outside of the parser!
-        parent->deprecatedParserAddChild(this);
-    }
-
-    // Add the renderer to the render tree
-    if (renderer)
-        parent->renderer()->addChild(renderer);
-}
-
-void TextControlInnerElement::detach()
-{
-    HTMLDivElement::detach();
-    // FIXME: Remove once shadow DOM uses Element::setShadowRoot().
-    if (shadowHost())
-        setShadowHost(0);
+    RenderTextControlSingleLine* parentRenderer = toRenderTextControlSingleLine(shadowAncestorNode()->renderer());
+    return parentRenderer->createInnerBlockStyle(parentRenderer->style());
 }
 
 // ----------------------------
 
-inline TextControlInnerTextElement::TextControlInnerTextElement(Document* document, HTMLElement* shadowParent)
-    : TextControlInnerElement(document, shadowParent)
+inline TextControlInnerTextElement::TextControlInnerTextElement(Document* document)
+    : HTMLDivElement(divTag, document)
 {
+    setHasCustomStyleForRenderer();
 }
 
-PassRefPtr<TextControlInnerTextElement> TextControlInnerTextElement::create(Document* document, HTMLElement* shadowParent)
+PassRefPtr<TextControlInnerTextElement> TextControlInnerTextElement::create(Document* document)
 {
-    return adoptRef(new TextControlInnerTextElement(document, shadowParent));
+    return adoptRef(new TextControlInnerTextElement(document));
 }
 
 void TextControlInnerTextElement::defaultEventHandler(Event* event)
@@ -134,10 +106,16 @@ RenderObject* TextControlInnerTextElement::createRenderer(RenderArena* arena, Re
     return new (arena) RenderTextControlInnerBlock(this, multiLine);
 }
 
+PassRefPtr<RenderStyle> TextControlInnerTextElement::customStyleForRenderer()
+{
+    RenderTextControl* parentRenderer = toRenderTextControl(shadowAncestorNode()->renderer());
+    return parentRenderer->createInnerTextStyle(parentRenderer->style());
+}
+
 // ----------------------------
 
 inline SearchFieldResultsButtonElement::SearchFieldResultsButtonElement(Document* document)
-    : TextControlInnerElement(document)
+    : HTMLDivElement(divTag, document)
 {
 }
 
@@ -146,6 +124,24 @@ PassRefPtr<SearchFieldResultsButtonElement> SearchFieldResultsButtonElement::cre
     return adoptRef(new SearchFieldResultsButtonElement(document));
 }
 
+const AtomicString& SearchFieldResultsButtonElement::shadowPseudoId() const
+{
+    DEFINE_STATIC_LOCAL(AtomicString, resultsId, ("-webkit-search-results-button"));
+    DEFINE_STATIC_LOCAL(AtomicString, resultsDecorationId, ("-webkit-search-results-decoration"));
+    DEFINE_STATIC_LOCAL(AtomicString, decorationId, ("-webkit-search-decoration"));
+    Node* host = shadowAncestorNode();
+    if (!host)
+        return resultsId;
+    if (HTMLInputElement* input = host->toInputElement()) {
+        if (input->maxResults() < 0)
+            return decorationId;
+        if (input->maxResults() > 0)
+            return resultsId;
+        return resultsDecorationId;
+    }
+    return resultsId;
+}
+
 void SearchFieldResultsButtonElement::defaultEventHandler(Event* event)
 {
     // On mousedown, bring up a menu, if needed
@@ -168,7 +164,7 @@ void SearchFieldResultsButtonElement::defaultEventHandler(Event* event)
 // ----------------------------
 
 inline SearchFieldCancelButtonElement::SearchFieldCancelButtonElement(Document* document)
-    : TextControlInnerElement(document)
+    : HTMLDivElement(divTag, document)
     , m_capturing(false)
 {
 }
@@ -178,13 +174,19 @@ PassRefPtr<SearchFieldCancelButtonElement> SearchFieldCancelButtonElement::creat
     return adoptRef(new SearchFieldCancelButtonElement(document));
 }
 
+const AtomicString& SearchFieldCancelButtonElement::shadowPseudoId() const
+{
+    DEFINE_STATIC_LOCAL(AtomicString, pseudoId, ("-webkit-search-cancel-button"));
+    return pseudoId;
+}
+
 void SearchFieldCancelButtonElement::detach()
 {
     if (m_capturing) {
         if (Frame* frame = document()->frame())
             frame->eventHandler()->setCapturingMouseEventsNode(0);
     }
-    TextControlInnerElement::detach();
+    HTMLDivElement::detach();
 }
 
 
@@ -192,6 +194,12 @@ void SearchFieldCancelButtonElement::defaultEventHandler(Event* event)
 {
     // If the element is visible, on mouseup, clear the value, and set selection
     RefPtr<HTMLInputElement> input(static_cast<HTMLInputElement*>(shadowAncestorNode()));
+    if (input->disabled() || input->isReadOnlyFormControl()) {
+        if (!event->defaultHandled())
+            HTMLDivElement::defaultEventHandler(event);
+        return;
+    }
+
     if (event->type() == eventNames().mousedownEvent && event->isMouseEvent() && static_cast<MouseEvent*>(event)->button() == LeftButton) {
         if (renderer() && renderer()->visibleToHitTesting()) {
             if (Frame* frame = document()->frame()) {
@@ -224,8 +232,8 @@ void SearchFieldCancelButtonElement::defaultEventHandler(Event* event)
 
 // ----------------------------
 
-inline SpinButtonElement::SpinButtonElement(HTMLElement* shadowParent)
-    : TextControlInnerElement(shadowParent->document(), shadowParent)
+inline SpinButtonElement::SpinButtonElement(Document* document)
+    : HTMLDivElement(divTag, document)
     , m_capturing(false)
     , m_upDownState(Indeterminate)
     , m_pressStartingState(Indeterminate)
@@ -233,21 +241,21 @@ inline SpinButtonElement::SpinButtonElement(HTMLElement* shadowParent)
 {
 }
 
-PassRefPtr<SpinButtonElement> SpinButtonElement::create(HTMLElement* shadowParent)
+PassRefPtr<SpinButtonElement> SpinButtonElement::create(Document* document)
 {
-    return adoptRef(new SpinButtonElement(shadowParent));
+    return adoptRef(new SpinButtonElement(document));
+}
+
+const AtomicString& SpinButtonElement::shadowPseudoId() const
+{
+    DEFINE_STATIC_LOCAL(AtomicString, innerPseudoId, ("-webkit-inner-spin-button"));
+    return innerPseudoId;
 }
 
 void SpinButtonElement::detach()
 {
-    stopRepeatingTimer();
-    if (m_capturing) {
-        if (Frame* frame = document()->frame()) {
-            frame->eventHandler()->setCapturingMouseEventsNode(0);
-            m_capturing = false;
-        }
-    }
-    TextControlInnerElement::detach();
+    releaseCapture();
+    HTMLDivElement::detach();
 }
 
 void SpinButtonElement::defaultEventHandler(Event* event)
@@ -303,21 +311,25 @@ void SpinButtonElement::defaultEventHandler(Event* event)
             m_upDownState = local.y() < box->height() / 2 ? Up : Down;
             if (m_upDownState != oldUpDownState)
                 renderer()->repaint();
-        } else {
-            if (m_capturing) {
-                stopRepeatingTimer();
-                if (Frame* frame = document()->frame()) {
-                    frame->eventHandler()->setCapturingMouseEventsNode(0);
-                    m_capturing = false;
-                }
-            }
-        }
+        } else
+            releaseCapture();
     }
 
     if (!event->defaultHandled())
         HTMLDivElement::defaultEventHandler(event);
 }
 
+void SpinButtonElement::releaseCapture()
+{
+    stopRepeatingTimer();
+    if (m_capturing) {
+        if (Frame* frame = document()->frame()) {
+            frame->eventHandler()->setCapturingMouseEventsNode(0);
+            m_capturing = false;
+        }
+    }
+}
+
 void SpinButtonElement::startRepeatingTimer()
 {
     m_pressStartingState = m_upDownState;
@@ -349,7 +361,7 @@ void SpinButtonElement::setHovered(bool flag)
 {
     if (!hovered() && flag)
         m_upDownState = Indeterminate;
-    TextControlInnerElement::setHovered(flag);
+    HTMLDivElement::setHovered(flag);
 }
 
 
@@ -357,11 +369,11 @@ void SpinButtonElement::setHovered(bool flag)
 
 #if ENABLE(INPUT_SPEECH)
 
-inline InputFieldSpeechButtonElement::InputFieldSpeechButtonElement(HTMLElement* shadowParent)
-    : TextControlInnerElement(shadowParent->document(), shadowParent)
+inline InputFieldSpeechButtonElement::InputFieldSpeechButtonElement(Document* document)
+    : HTMLDivElement(divTag, document)
     , m_capturing(false)
     , m_state(Idle)
-    , m_listenerId(document()->page()->speechInput()->registerListener(this))
+    , m_listenerId(0)
 {
 }
 
@@ -375,15 +387,15 @@ InputFieldSpeechButtonElement::~InputFieldSpeechButtonElement()
     }
 }
 
-PassRefPtr<InputFieldSpeechButtonElement> InputFieldSpeechButtonElement::create(HTMLElement* shadowParent)
+PassRefPtr<InputFieldSpeechButtonElement> InputFieldSpeechButtonElement::create(Document* document)
 {
-    return adoptRef(new InputFieldSpeechButtonElement(shadowParent));
+    return adoptRef(new InputFieldSpeechButtonElement(document));
 }
 
 void InputFieldSpeechButtonElement::defaultEventHandler(Event* event)
 {
     // For privacy reasons, only allow clicks directly coming from the user.
-    if (!event->fromUserGesture()) {
+    if (!ScriptController::processingUserGesture()) {
         HTMLDivElement::defaultEventHandler(event);
         return;
     }
@@ -393,6 +405,12 @@ void InputFieldSpeechButtonElement::defaultEventHandler(Event* event)
     // here, we take a temporary reference.
     RefPtr<HTMLInputElement> input(static_cast<HTMLInputElement*>(shadowAncestorNode()));
 
+    if (input->disabled() || input->isReadOnlyFormControl()) {
+        if (!event->defaultHandled())
+            HTMLDivElement::defaultEventHandler(event);
+        return;
+    }
+
     // On mouse down, select the text and set focus.
     if (event->type() == eventNames().mousedownEvent && event->isMouseEvent() && static_cast<MouseEvent*>(event)->button() == LeftButton) {
         if (renderer() && renderer()->visibleToHitTesting()) {
@@ -416,18 +434,13 @@ void InputFieldSpeechButtonElement::defaultEventHandler(Event* event)
         }
     }
 
-    if (event->type() == eventNames().clickEvent) {
+    if (event->type() == eventNames().clickEvent && m_listenerId) {
         switch (m_state) {
-        case Idle: {
-              AtomicString language = input->computeInheritedLanguage();
-              String grammar = input->getAttribute(webkitgrammarAttr);
-              IntRect rect = input->renderer()->absoluteBoundingBoxRect();
-              if (speechInput()->startRecognition(m_listenerId, rect, language, grammar, document()->securityOrigin()))
-                  setState(Recording);
-            }
+        case Idle:
+            startSpeechInput();
             break;
         case Recording:
-            speechInput()->stopRecording(m_listenerId);
+            stopSpeechInput();
             break;
         case Recognizing:
             // Nothing to do here, we will continue to wait for results.
@@ -471,6 +484,9 @@ void InputFieldSpeechButtonElement::setRecognitionResult(int, const SpeechInputR
     // remove the input element from DOM. To make sure it remains valid until we finish our work
     // here, we take a temporary reference.
     RefPtr<HTMLInputElement> input(static_cast<HTMLInputElement*>(shadowAncestorNode()));
+    if (input->disabled() || input->isReadOnlyFormControl())
+        return;
+
     RefPtr<InputFieldSpeechButtonElement> holdRefButton(this);
     input->setValue(results.isEmpty() ? "" : results[0]->utterance());
     input->dispatchEvent(SpeechInputEvent::create(eventNames().webkitspeechchangeEvent, results));
@@ -481,6 +497,13 @@ void InputFieldSpeechButtonElement::setRecognitionResult(int, const SpeechInputR
         renderer()->repaint();
 }
 
+void InputFieldSpeechButtonElement::attach()
+{
+    ASSERT(!m_listenerId);
+    m_listenerId = document()->page()->speechInput()->registerListener(this);
+    HTMLDivElement::attach();
+}
+
 void InputFieldSpeechButtonElement::detach()
 {
     if (m_capturing) {
@@ -495,7 +518,32 @@ void InputFieldSpeechButtonElement::detach()
         m_listenerId = 0;
     }
 
-    TextControlInnerElement::detach();
+    HTMLDivElement::detach();
+}
+
+void InputFieldSpeechButtonElement::startSpeechInput()
+{
+    if (m_state != Idle)
+        return;
+
+    RefPtr<HTMLInputElement> input = static_cast<HTMLInputElement*>(shadowAncestorNode());
+    AtomicString language = input->computeInheritedLanguage();
+    String grammar = input->getAttribute(webkitgrammarAttr);
+    IntRect rect = renderer()->absoluteBoundingBoxRect();
+    if (speechInput()->startRecognition(m_listenerId, rect, language, grammar, document()->securityOrigin()))
+        setState(Recording);
+}
+
+void InputFieldSpeechButtonElement::stopSpeechInput()
+{
+    if (m_state == Recording)
+        speechInput()->stopRecording(m_listenerId);
+}
+
+const AtomicString& InputFieldSpeechButtonElement::shadowPseudoId() const
+{
+    DEFINE_STATIC_LOCAL(AtomicString, pseudoId, ("-webkit-input-speech-button"));
+    return pseudoId;
 }
 
 #endif // ENABLE(INPUT_SPEECH)
