@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2009 Apple Inc. All rights reserved.
- * Copyright (C) 2009-2011 Google Inc. All rights reserved.
+ * Copyright (C) 2011 Google Inc. All rights reserved.
  * Copyright (C) 2009 Joseph Pecoraro
  *
  * Redistribution and use in source and binary forms, with or without
@@ -47,6 +47,7 @@
 #include "ContainerNode.h"
 #include "Cookie.h"
 #include "CookieJar.h"
+#include "DOMNodeHighlighter.h"
 #include "DOMWindow.h"
 #include "Document.h"
 #include "DocumentType.h"
@@ -57,18 +58,21 @@
 #include "EventTarget.h"
 #include "Frame.h"
 #include "FrameTree.h"
+#include "HitTestResult.h"
 #include "HTMLElement.h"
 #include "HTMLFrameOwnerElement.h"
 #include "InjectedScriptManager.h"
+#include "InspectorClient.h"
 #include "InspectorFrontend.h"
+#include "InspectorPageAgent.h"
 #include "InspectorState.h"
 #include "InstrumentingAgents.h"
+#include "IntRect.h"
 #include "MutationEvent.h"
 #include "Node.h"
 #include "NodeList.h"
 #include "Page.h"
 #include "Pasteboard.h"
-#include "PlatformString.h"
 #include "RenderStyle.h"
 #include "RenderStyleConstants.h"
 #include "ScriptEventListener.h"
@@ -82,12 +86,11 @@
 #include "markup.h"
 
 #include <wtf/text/CString.h>
-#include <wtf/text/StringConcatenate.h>
+#include <wtf/text/WTFString.h>
 #include <wtf/HashSet.h>
 #include <wtf/ListHashSet.h>
 #include <wtf/OwnPtr.h>
 #include <wtf/Vector.h>
-#include <wtf/text/AtomicString.h>
 
 namespace WebCore {
 
@@ -95,6 +98,40 @@ namespace DOMAgentState {
 static const char documentRequested[] = "documentRequested";
 };
 
+static Color parseColor(const RefPtr<InspectorObject>* colorObject)
+{
+    if (!colorObject || !(*colorObject))
+        return Color::transparent;
+
+    int r;
+    int g;
+    int b;
+    bool success = (*colorObject)->getNumber("r", &r);
+    success |= (*colorObject)->getNumber("g", &g);
+    success |= (*colorObject)->getNumber("b", &b);
+    if (!success)
+        return Color::transparent;
+
+    double a;
+    success = (*colorObject)->getNumber("a", &a);
+    if (!success)
+        return Color(r, g, b);
+
+    // Clamp alpha to the [0..1] range.
+    if (a < 0)
+        a = 0;
+    else if (a > 1)
+        a = 1;
+
+    return Color(r, g, b, static_cast<int>(a * 255));
+}
+
+static Color parseConfigColor(const String& fieldName, InspectorObject* configObject)
+{
+    const RefPtr<InspectorObject> colorObject = configObject->getObject(fieldName);
+    return parseColor(&colorObject);
+}
+
 class MatchJob {
 public:
     virtual void match(ListHashSet<Node*>& resultCollector) = 0;
@@ -247,27 +284,33 @@ void RevalidateStyleAttributeTask::scheduleFor(Element* element)
 void RevalidateStyleAttributeTask::onTimer(Timer<RevalidateStyleAttributeTask>*)
 {
     // The timer is stopped on m_domAgent destruction, so this method will never be called after m_domAgent has been destroyed.
+    Vector<Element*> elements;
     for (HashSet<RefPtr<Element> >::iterator it = m_elements.begin(), end = m_elements.end(); it != end; ++it)
-        m_domAgent->didModifyDOMAttr(it->get());
+        elements.append(it->get());
+    m_domAgent->styleAttributeInvalidated(elements);
 
     m_elements.clear();
 }
 
-InspectorDOMAgent::InspectorDOMAgent(InstrumentingAgents* instrumentingAgents, Page* inspectedPage, InspectorState* inspectorState, InjectedScriptManager* injectedScriptManager)
+InspectorDOMAgent::InspectorDOMAgent(InstrumentingAgents* instrumentingAgents, InspectorPageAgent* pageAgent, InspectorClient* client, InspectorState* inspectorState, InjectedScriptManager* injectedScriptManager)
     : m_instrumentingAgents(instrumentingAgents)
-    , m_inspectedPage(inspectedPage)
+    , m_pageAgent(pageAgent)
+    , m_client(client)
     , m_inspectorState(inspectorState)
     , m_injectedScriptManager(injectedScriptManager)
     , m_frontend(0)
     , m_domListener(0)
     , m_lastNodeId(1)
     , m_matchJobsTimer(this, &InspectorDOMAgent::onMatchJobsTimer)
+    , m_searchingForNode(false)
 {
 }
 
 InspectorDOMAgent::~InspectorDOMAgent()
 {
     reset();
+    ASSERT(!m_highlightData || (!m_highlightData->node && !m_highlightData->rect));
+    ASSERT(!m_searchingForNode);
 }
 
 void InspectorDOMAgent::setFrontend(InspectorFrontend* frontend)
@@ -275,12 +318,20 @@ void InspectorDOMAgent::setFrontend(InspectorFrontend* frontend)
     ASSERT(!m_frontend);
     m_frontend = frontend->dom();
     m_instrumentingAgents->setInspectorDOMAgent(this);
-    m_document = m_inspectedPage->mainFrame()->document();
+    m_document = m_pageAgent->mainFrame()->document();
+
+    if (m_nodeToFocus)
+        focusNode();
 }
 
 void InspectorDOMAgent::clearFrontend()
 {
     ASSERT(m_frontend);
+    setSearchingForNode(false, 0);
+
+    ErrorString error;
+    hideHighlight(&error);
+
     m_frontend = 0;
     m_instrumentingAgents->setInspectorDOMAgent(0);
     m_inspectorState->setBoolean(DOMAgentState::documentRequested, false);
@@ -291,7 +342,7 @@ void InspectorDOMAgent::restore()
 {
     // Reset document to avoid early return from setDocument.
     m_document = 0;
-    setDocument(m_inspectedPage->mainFrame()->document());
+    setDocument(m_pageAgent->mainFrame()->document());
 }
 
 Vector<Document*> InspectorDOMAgent::documents()
@@ -306,6 +357,11 @@ Vector<Document*> InspectorDOMAgent::documents()
     return result;
 }
 
+Node* InspectorDOMAgent::highlightedNode() const
+{
+    return m_highlightData ? m_highlightData->node.get() : 0;
+}
+
 void InspectorDOMAgent::reset()
 {
     ErrorString error;
@@ -330,7 +386,11 @@ void InspectorDOMAgent::setDocument(Document* doc)
 
     m_document = doc;
 
-    if (!doc && m_inspectorState->getBoolean(DOMAgentState::documentRequested))
+    if (!m_inspectorState->getBoolean(DOMAgentState::documentRequested))
+        return;
+
+    // Immediately communicate 0 document or document that has finished loading.
+    if (!doc || !doc->parsing())
         m_frontend->documentUpdated();
 }
 
@@ -340,9 +400,9 @@ void InspectorDOMAgent::releaseDanglingNodes()
     m_danglingNodeToIdMaps.clear();
 }
 
-long InspectorDOMAgent::bind(Node* node, NodeToIdMap* nodesMap)
+int InspectorDOMAgent::bind(Node* node, NodeToIdMap* nodesMap)
 {
-    long id = nodesMap->get(node);
+    int id = nodesMap->get(node);
     if (id)
         return id;
     id = m_lastNodeId++;
@@ -360,7 +420,7 @@ void InspectorDOMAgent::unbind(Node* node, NodeToIdMap* nodesMap)
             m_domListener->didRemoveDocument(frameOwner->contentDocument());
     }
 
-    long id = nodesMap->get(node);
+    int id = nodesMap->get(node);
     if (!id)
         return;
     m_idToNode.remove(id);
@@ -377,61 +437,43 @@ void InspectorDOMAgent::unbind(Node* node, NodeToIdMap* nodesMap)
     }
 }
 
-Node* InspectorDOMAgent::assertNode(ErrorString* errorString, long nodeId)
+Node* InspectorDOMAgent::assertNode(ErrorString* errorString, int nodeId)
 {
     Node* node = nodeForId(nodeId);
     if (!node) {
-        *errorString = "Could not find node with given id.";
+        *errorString = "Could not find node with given id";
         return 0;
     }
     return node;
 }
 
-Element* InspectorDOMAgent::assertElement(ErrorString* errorString, long nodeId)
+Element* InspectorDOMAgent::assertElement(ErrorString* errorString, int nodeId)
 {
     Node* node = assertNode(errorString, nodeId);
     if (!node)
         return 0;
 
     if (node->nodeType() != Node::ELEMENT_NODE) {
-        *errorString = "Node is not an Element.";
+        *errorString = "Node is not an Element";
         return 0;
     }
     return toElement(node);
 }
 
 
-HTMLElement* InspectorDOMAgent::assertHTMLElement(ErrorString* errorString, long nodeId)
+HTMLElement* InspectorDOMAgent::assertHTMLElement(ErrorString* errorString, int nodeId)
 {
     Element* element = assertElement(errorString, nodeId);
     if (!element)
         return 0;
 
     if (!element->isHTMLElement()) {
-        *errorString = "Node is not an HTML Element.";
+        *errorString = "Node is not an HTML Element";
         return 0;
     }
     return toHTMLElement(element);
 }
 
-Node* InspectorDOMAgent::nodeToSelectOn(ErrorString* errorString, long nodeId, bool documentWide)
-{
-    Node* node;
-    if (!nodeId) {
-        node = m_document.get();
-        if (!node)
-            *errorString = "No document to query on.";
-    } else
-        node = assertNode(errorString, nodeId);
-
-    if (!node)
-        return 0;
-
-    if (documentWide && nodeId)
-        node = node->ownerDocument();
-    return node;
-}
-
 void InspectorDOMAgent::getDocument(ErrorString*, RefPtr<InspectorObject>* root)
 {
     m_inspectorState->setBoolean(DOMAgentState::documentRequested, true);
@@ -444,11 +486,10 @@ void InspectorDOMAgent::getDocument(ErrorString*, RefPtr<InspectorObject>* root)
     reset();
     m_document = doc;
 
-    if (!m_documentNodeToIdMap.contains(m_document))
-        *root = buildObjectForNode(m_document.get(), 2, &m_documentNodeToIdMap);
+    *root = buildObjectForNode(m_document.get(), 2, &m_documentNodeToIdMap);
 }
 
-void InspectorDOMAgent::pushChildNodesToFrontend(long nodeId)
+void InspectorDOMAgent::pushChildNodesToFrontend(int nodeId)
 {
     Node* node = nodeForId(nodeId);
     if (!node || (node->nodeType() != Node::ELEMENT_NODE && node->nodeType() != Node::DOCUMENT_NODE && node->nodeType() != Node::DOCUMENT_FRAGMENT_NODE))
@@ -458,7 +499,6 @@ void InspectorDOMAgent::pushChildNodesToFrontend(long nodeId)
 
     NodeToIdMap* nodeMap = m_idToNodesMap.get(nodeId);
     RefPtr<InspectorArray> children = buildArrayForContainerChildren(node, 1, nodeMap);
-    m_childrenRequested.add(nodeId);
     m_frontend->setChildNodes(nodeId, children.release());
 }
 
@@ -470,33 +510,33 @@ void InspectorDOMAgent::discardBindings()
     m_childrenRequested.clear();
 }
 
-Node* InspectorDOMAgent::nodeForId(long id)
+Node* InspectorDOMAgent::nodeForId(int id)
 {
     if (!id)
         return 0;
 
-    HashMap<long, Node*>::iterator it = m_idToNode.find(id);
+    HashMap<int, Node*>::iterator it = m_idToNode.find(id);
     if (it != m_idToNode.end())
         return it->second;
     return 0;
 }
 
-void InspectorDOMAgent::getChildNodes(ErrorString*, long nodeId)
+void InspectorDOMAgent::requestChildNodes(ErrorString*, int nodeId)
 {
     pushChildNodesToFrontend(nodeId);
 }
 
-void InspectorDOMAgent::querySelector(ErrorString* errorString, long nodeId, const String& selectors, bool documentWide, long* elementId)
+void InspectorDOMAgent::querySelector(ErrorString* errorString, int nodeId, const String& selectors, int* elementId)
 {
     *elementId = 0;
-    Node* node = nodeToSelectOn(errorString, nodeId, documentWide);
+    Node* node = assertNode(errorString, nodeId);
     if (!node)
         return;
 
     ExceptionCode ec = 0;
     RefPtr<Element> element = node->querySelector(selectors, ec);
     if (ec) {
-        *errorString = "DOM Error while querying.";
+        *errorString = "DOM Error while querying";
         return;
     }
 
@@ -504,16 +544,16 @@ void InspectorDOMAgent::querySelector(ErrorString* errorString, long nodeId, con
         *elementId = pushNodePathToFrontend(element.get());
 }
 
-void InspectorDOMAgent::querySelectorAll(ErrorString* errorString, long nodeId, const String& selectors, bool documentWide, RefPtr<InspectorArray>* result)
+void InspectorDOMAgent::querySelectorAll(ErrorString* errorString, int nodeId, const String& selectors, RefPtr<InspectorArray>* result)
 {
-    Node* node = nodeToSelectOn(errorString, nodeId, documentWide);
+    Node* node = assertNode(errorString, nodeId);
     if (!node)
         return;
 
     ExceptionCode ec = 0;
     RefPtr<NodeList> nodes = node->querySelectorAll(selectors, ec);
     if (ec) {
-        *errorString = "DOM Error while querying.";
+        *errorString = "DOM Error while querying";
         return;
     }
 
@@ -521,7 +561,7 @@ void InspectorDOMAgent::querySelectorAll(ErrorString* errorString, long nodeId,
         (*result)->pushNumber(pushNodePathToFrontend(nodes->item(i)));
 }
 
-long InspectorDOMAgent::pushNodePathToFrontend(Node* nodeToPush)
+int InspectorDOMAgent::pushNodePathToFrontend(Node* nodeToPush)
 {
     ASSERT(nodeToPush);  // Invalid input
 
@@ -531,7 +571,7 @@ long InspectorDOMAgent::pushNodePathToFrontend(Node* nodeToPush)
         return 0;
 
     // Return id in case the node is known.
-    long result = m_documentNodeToIdMap.get(nodeToPush);
+    int result = m_documentNodeToIdMap.get(nodeToPush);
     if (result)
         return result;
 
@@ -560,59 +600,114 @@ long InspectorDOMAgent::pushNodePathToFrontend(Node* nodeToPush)
 
     NodeToIdMap* map = danglingMap ? danglingMap : &m_documentNodeToIdMap;
     for (int i = path.size() - 1; i >= 0; --i) {
-        long nodeId = map->get(path.at(i));
+        int nodeId = map->get(path.at(i));
         ASSERT(nodeId);
         pushChildNodesToFrontend(nodeId);
     }
     return map->get(nodeToPush);
 }
 
-long InspectorDOMAgent::boundNodeId(Node* node)
+int InspectorDOMAgent::boundNodeId(Node* node)
 {
     return m_documentNodeToIdMap.get(node);
 }
 
-void InspectorDOMAgent::setAttribute(ErrorString* errorString, long elementId, const String& name, const String& value)
+void InspectorDOMAgent::setAttributeValue(ErrorString* errorString, int elementId, const String& name, const String& value)
 {
     Element* element = assertElement(errorString, elementId);
-    if (element) {
-        ExceptionCode ec = 0;
-        element->setAttribute(name, value, ec);
+    if (!element)
+        return;
+
+    ExceptionCode ec = 0;
+    element->setAttribute(name, value, ec);
+    if (ec)
+        *errorString = "Internal error: could not set attribute value.";
+}
+
+void InspectorDOMAgent::setAttributesText(ErrorString* errorString, int elementId, const String& text, const String* const name)
+{
+    Element* element = assertElement(errorString, elementId);
+    if (!element)
+        return;
+
+    ExceptionCode ec = 0;
+    RefPtr<Element> parsedElement = element->document()->createElement("span", ec);
+    if (ec) {
+        *errorString = "Internal error: could not set attribute value.";
+        return;
+    }
+
+    toHTMLElement(parsedElement.get())->setInnerHTML("<span " + text + "></span>", ec);
+    if (ec) {
+        *errorString = "Could not parse value as attributes.";
+        return;
+    }
+
+    Node* child = parsedElement->firstChild();
+    if (!child) {
+        *errorString = "Could not parse value as attributes.";
+        return;
+    }
+
+    const NamedNodeMap* attrMap = toHTMLElement(child)->attributes(true);
+    if (!attrMap && name) {
+        element->removeAttribute(*name, ec);
         if (ec)
-            *errorString = "Exception while setting attribute value.";
+            *errorString = "Could not remove attribute.";
+        return;
+    }
+
+    bool foundOriginalAttribute = false;
+    unsigned numAttrs = attrMap->length();
+    for (unsigned i = 0; i < numAttrs; ++i) {
+        // Add attribute pair
+        const Attribute *attribute = attrMap->attributeItem(i);
+        foundOriginalAttribute = foundOriginalAttribute || (name && attribute->name().toString() == *name);
+        element->setAttribute(attribute->name(), attribute->value(), ec);
+        if (ec) {
+            *errorString = "Internal error: could not set attribute value.";
+            return;
+        }
+    }
+
+    if (!foundOriginalAttribute && name) {
+        element->removeAttribute(*name, ec);
+        if (ec)
+            *errorString = "Could not remove attribute.";
+        return;
     }
 }
 
-void InspectorDOMAgent::removeAttribute(ErrorString* errorString, long elementId, const String& name)
+void InspectorDOMAgent::removeAttribute(ErrorString* errorString, int elementId, const String& name)
 {
     Element* element = assertElement(errorString, elementId);
     if (element) {
         ExceptionCode ec = 0;
         element->removeAttribute(name, ec);
         if (ec)
-            *errorString = "Exception while removing attribute.";
+            *errorString = "Exception while removing attribute";
     }
 }
 
-void InspectorDOMAgent::removeNode(ErrorString*, long nodeId, long* outNodeId)
+void InspectorDOMAgent::removeNode(ErrorString* errorString, int nodeId)
 {
-    Node* node = nodeForId(nodeId);
+    Node* node = assertNode(errorString, nodeId);
     if (!node)
         return;
 
     ContainerNode* parentNode = node->parentNode();
-    if (!parentNode)
+    if (!parentNode) {
+        *errorString = "Can not remove detached node";
         return;
+    }
 
     ExceptionCode ec = 0;
     parentNode->removeChild(node, ec);
     if (ec)
-        return;
-
-    *outNodeId = nodeId;
+        *errorString = "Could not remove node due to DOM exception";
 }
 
-void InspectorDOMAgent::setNodeName(ErrorString*, long nodeId, const String& tagName, long* newId)
+void InspectorDOMAgent::setNodeName(ErrorString*, int nodeId, const String& tagName, int* newId)
 {
     *newId = 0;
 
@@ -627,7 +722,6 @@ void InspectorDOMAgent::setNodeName(ErrorString*, long nodeId, const String& tag
 
     // Copy over the original node's attributes.
     Element* oldElem = static_cast<Element*>(oldNode);
-    newElem->copyNonAttributeProperties(oldElem);
     if (oldElem->attributes())
         newElem->attributes()->setAttributes(*(oldElem->attributes(true)));
 
@@ -649,14 +743,14 @@ void InspectorDOMAgent::setNodeName(ErrorString*, long nodeId, const String& tag
         pushChildNodesToFrontend(*newId);
 }
 
-void InspectorDOMAgent::getOuterHTML(ErrorString* errorString, long nodeId, WTF::String* outerHTML)
+void InspectorDOMAgent::getOuterHTML(ErrorString* errorString, int nodeId, WTF::String* outerHTML)
 {
     HTMLElement* element = assertHTMLElement(errorString, nodeId);
     if (element)
         *outerHTML = element->outerHTML();
 }
 
-void InspectorDOMAgent::setOuterHTML(ErrorString* errorString, long nodeId, const String& outerHTML, long* newId)
+void InspectorDOMAgent::setOuterHTML(ErrorString* errorString, int nodeId, const String& outerHTML, int* newId)
 {
     HTMLElement* htmlElement = assertHTMLElement(errorString, nodeId);
     if (!htmlElement)
@@ -693,20 +787,25 @@ void InspectorDOMAgent::setOuterHTML(ErrorString* errorString, long nodeId, cons
         pushChildNodesToFrontend(*newId);
 }
 
-void InspectorDOMAgent::setNodeValue(ErrorString*, long nodeId, const String& value, bool* success)
+void InspectorDOMAgent::setNodeValue(ErrorString* errorString, int nodeId, const String& value)
 {
-    Node* node = nodeForId(nodeId);
-    if (node && (node->nodeType() == Node::TEXT_NODE)) {
-        Text* text_node = static_cast<Text*>(node);
-        ExceptionCode ec = 0;
-        text_node->replaceWholeText(value, ec);
-        *success = !ec;
+    Node* node = assertNode(errorString, nodeId);
+    if (!node)
+        return;
+
+    if (node->nodeType() != Node::TEXT_NODE) {
+        *errorString = "Can only set value of text nodes";
         return;
     }
-    *success = false;
+
+    Text* textNode = static_cast<Text*>(node);
+    ExceptionCode ec = 0;
+    textNode->replaceWholeText(value, ec);
+    if (ec)
+        *errorString = "DOM Error while setting the node value";
 }
 
-void InspectorDOMAgent::getEventListenersForNode(ErrorString*, long nodeId, RefPtr<InspectorArray>* listenersArray)
+void InspectorDOMAgent::getEventListenersForNode(ErrorString*, int nodeId, RefPtr<InspectorArray>* listenersArray)
 {
     Node* node = nodeForId(nodeId);
     EventTargetData* d;
@@ -716,29 +815,26 @@ void InspectorDOMAgent::getEventListenersForNode(ErrorString*, long nodeId, RefP
         return;
 
     // Get the list of event types this Node is concerned with
-    Vector<AtomicString> eventTypes;
-    const EventListenerMap& listenerMap = d->eventListenerMap;
-    EventListenerMap::const_iterator end = listenerMap.end();
-    for (EventListenerMap::const_iterator iter = listenerMap.begin(); iter != end; ++iter)
-        eventTypes.append(iter->first);
+    Vector<AtomicString> eventTypes = d->eventListenerMap.eventTypes();
 
     // Quick break if no useful listeners
     size_t eventTypesLength = eventTypes.size();
     if (!eventTypesLength)
         return;
 
-    // The Node's Event Ancestors (not including self)
-    Vector<EventContext> ancestors;
-    node->getEventAncestors(ancestors, node);
+    // The Node's Ancestors (not including self)
+    Vector<ContainerNode*> ancestors;
+    for (ContainerNode* ancestor = node->parentOrHostNode(); ancestor; ancestor = ancestor->parentOrHostNode())
+        ancestors.append(ancestor);
 
     // Nodes and their Listeners for the concerned event types (order is top to bottom)
     Vector<EventListenerInfo> eventInformation;
     for (size_t i = ancestors.size(); i; --i) {
-        Node* ancestor = ancestors[i - 1].node();
+        ContainerNode* ancestor = ancestors[i - 1];
         for (size_t j = 0; j < eventTypesLength; ++j) {
             AtomicString& type = eventTypes[j];
             if (ancestor->hasEventListeners(type))
-                eventInformation.append(EventListenerInfo(static_cast<Node*>(ancestor), type, ancestor->getEventListeners(type)));
+                eventInformation.append(EventListenerInfo(ancestor, type, ancestor->getEventListeners(type)));
         }
     }
 
@@ -772,7 +868,7 @@ void InspectorDOMAgent::getEventListenersForNode(ErrorString*, long nodeId, RefP
     }
 }
 
-void InspectorDOMAgent::performSearch(ErrorString* error, const String& whitespaceTrimmedQuery, bool runSynchronously)
+void InspectorDOMAgent::performSearch(ErrorString* error, const String& whitespaceTrimmedQuery, const bool* const runSynchronously)
 {
     // FIXME: Few things are missing here:
     // 1) Search works with node granularity - number of matches within node is not calculated.
@@ -847,7 +943,7 @@ void InspectorDOMAgent::performSearch(ErrorString* error, const String& whitespa
         m_pendingMatchJobs.append(new MatchXPathJob(document, whitespaceTrimmedQuery));
     }
 
-    if (runSynchronously) {
+    if (runSynchronously && *runSynchronously) {
         // For tests.
         ListHashSet<Node*> resultCollector;
         for (Deque<MatchJob*>::iterator it = m_pendingMatchJobs.begin(); it != m_pendingMatchJobs.end(); ++it)
@@ -868,19 +964,215 @@ void InspectorDOMAgent::cancelSearch(ErrorString*)
     m_searchResults.clear();
 }
 
-void InspectorDOMAgent::resolveNode(ErrorString* error, long nodeId, const String& objectGroup, RefPtr<InspectorObject>* result)
+bool InspectorDOMAgent::handleMousePress()
 {
+    if (!m_searchingForNode)
+        return false;
+
+    if (m_highlightData && m_highlightData->node) {
+        RefPtr<Node> node = m_highlightData->node;
+        setSearchingForNode(false, 0);
+        inspect(node.get());
+    }
+    return true;
+}
+
+void InspectorDOMAgent::inspect(Node* node)
+{
+    if (node->nodeType() != Node::ELEMENT_NODE && node->nodeType() != Node::DOCUMENT_NODE)
+        node = node->parentNode();
+    m_nodeToFocus = node;
+
+    focusNode();
+}
+
+void InspectorDOMAgent::focusNode()
+{
+    if (!m_frontend)
+        return;
+
+    ASSERT(m_nodeToFocus);
+
+    RefPtr<Node> node = m_nodeToFocus.get();
+    m_nodeToFocus = 0;
+
+    Document* document = node->ownerDocument();
+    if (!document)
+        return;
+    Frame* frame = document->frame();
+    if (!frame)
+        return;
+
+    InjectedScript injectedScript = m_injectedScriptManager->injectedScriptFor(mainWorldScriptState(frame));
+    if (injectedScript.hasNoValue())
+        return;
+
+    injectedScript.inspectNode(node.get());
+}
+
+void InspectorDOMAgent::mouseDidMoveOverElement(const HitTestResult& result, unsigned)
+{
+    if (!m_searchingForNode || !m_highlightData)
+        return;
+
+    Node* node = result.innerNode();
+    while (node && node->nodeType() == Node::TEXT_NODE)
+        node = node->parentNode();
+    if (node) {
+        m_highlightData->node = node;
+        highlight();
+    }
+}
+
+void InspectorDOMAgent::setSearchingForNode(bool enabled, InspectorObject* highlightConfig)
+{
+    if (m_searchingForNode == enabled)
+        return;
+    m_searchingForNode = enabled;
+    if (enabled)
+        setHighlightDataFromConfig(highlightConfig);
+    else {
+        ErrorString error;
+        hideHighlight(&error);
+        m_highlightData.clear();
+    }
+}
+
+void InspectorDOMAgent::setInspectModeEnabled(ErrorString*, bool enabled, const RefPtr<InspectorObject>* highlightConfig)
+{
+    setSearchingForNode(enabled, highlightConfig ? highlightConfig->get() : 0);
+}
+
+bool InspectorDOMAgent::setHighlightDataFromConfig(InspectorObject* highlightConfig)
+{
+    if (!highlightConfig) {
+        m_highlightData.clear();
+        return false;
+    }
+
+    m_highlightData = adoptPtr(new HighlightData());
+    bool showInfo = false; // Default: false (do not show a tooltip).
+    highlightConfig->getBoolean("showInfo", &showInfo);
+    m_highlightData->showInfo = showInfo;
+    m_highlightData->content = parseConfigColor("contentColor", highlightConfig);
+    m_highlightData->contentOutline = parseConfigColor("contentOutlineColor", highlightConfig);
+    m_highlightData->padding = parseConfigColor("paddingColor", highlightConfig);
+    m_highlightData->paddingOutline = parseConfigColor("paddingOutlineColor", highlightConfig);
+    m_highlightData->border = parseConfigColor("borderColor", highlightConfig);
+    m_highlightData->borderOutline = parseConfigColor("borderOutlineColor", highlightConfig);
+    m_highlightData->margin = parseConfigColor("marginColor", highlightConfig);
+    m_highlightData->marginOutline = parseConfigColor("marginOutlineColor", highlightConfig);
+    return true;
+}
+
+void InspectorDOMAgent::highlight()
+{
+    // This method requires m_highlightData to have been filled in by its client.
+    ASSERT(m_highlightData);
+    m_client->highlight();
+}
+
+void InspectorDOMAgent::highlightRect(ErrorString*, int x, int y, int width, int height, const RefPtr<InspectorObject>* color, const RefPtr<InspectorObject>* outlineColor)
+{
+    m_highlightData = adoptPtr(new HighlightData());
+    m_highlightData->rect = adoptPtr(new IntRect(x, y, width, height));
+    m_highlightData->content = parseColor(color);
+    m_highlightData->contentOutline = parseColor(outlineColor);
+    m_client->highlight();
+}
+
+void InspectorDOMAgent::highlightNode(
+    ErrorString*,
+    int nodeId,
+    const RefPtr<InspectorObject> highlightConfig)
+{
+    if (Node* node = nodeForId(nodeId)) {
+        if (setHighlightDataFromConfig(highlightConfig.get())) {
+            m_highlightData->node = node;
+            highlight();
+        }
+    }
+}
+
+void InspectorDOMAgent::highlightFrame(
+    ErrorString*,
+    const String& frameId,
+    const RefPtr<InspectorObject>* color,
+    const RefPtr<InspectorObject>* outlineColor)
+{
+    Frame* frame = m_pageAgent->frameForId(frameId);
+    if (frame && frame->ownerElement()) {
+        m_highlightData = adoptPtr(new HighlightData());
+        m_highlightData->node = frame->ownerElement();
+        m_highlightData->showInfo = true; // Always show tooltips for frames.
+        m_highlightData->content = parseColor(color);
+        m_highlightData->contentOutline = parseColor(outlineColor);
+        highlight();
+    }
+}
+
+void InspectorDOMAgent::hideHighlight(ErrorString*)
+{
+    if (m_highlightData) {
+        m_highlightData->node.clear();
+        m_highlightData->rect.clear();
+    }
+    m_client->hideHighlight();
+}
+
+void InspectorDOMAgent::moveTo(ErrorString* error, int nodeId, int targetElementId, const int* const anchorNodeId, int* newNodeId)
+{
+    Node* node = assertNode(error, nodeId);
+    if (!node)
+        return;
+
+    Element* targetElement = assertElement(error, targetElementId);
+    if (!targetElement)
+        return;
+
+    Node* anchorNode = 0;
+    if (anchorNodeId && *anchorNodeId) {
+        anchorNode = assertNode(error, *anchorNodeId);
+        if (!anchorNode)
+            return;
+        if (anchorNode->parentNode() != targetElement) {
+            *error = "Anchor node must be child of the target element.";
+            return;
+        }
+    }
+
+    ExceptionCode ec = 0;
+    bool success = targetElement->insertBefore(node, anchorNode, ec);
+    if (ec || !success) {
+        *error = "Could not drop node.";
+        return;
+    }
+    *newNodeId = pushNodePathToFrontend(node);
+}
+
+void InspectorDOMAgent::resolveNode(ErrorString* error, int nodeId, const String* const objectGroup, RefPtr<InspectorObject>* result)
+{
+    String objectGroupName = objectGroup ? *objectGroup : "";
     Node* node = nodeForId(nodeId);
     if (!node) {
         *error = "No node with given id found.";
         return;
     }
-    *result = resolveNode(node, objectGroup);
+    *result = resolveNode(node, objectGroupName);
 }
 
-void InspectorDOMAgent::pushNodeToFrontend(ErrorString*, PassRefPtr<InspectorObject> objectId, long* nodeId)
+void InspectorDOMAgent::getAttributes(ErrorString* errorString, int nodeId, RefPtr<InspectorArray>* result)
 {
-    InjectedScript injectedScript = m_injectedScriptManager->injectedScriptForObjectId(objectId.get());
+    Element* element = assertElement(errorString, nodeId);
+    if (!element)
+        return;
+
+    *result = buildArrayForElementAttributes(element);
+}
+
+void InspectorDOMAgent::requestNode(ErrorString*, const String& objectId, int* nodeId)
+{
+    InjectedScript injectedScript = m_injectedScriptManager->injectedScriptForObjectId(objectId);
     Node* node = injectedScript.nodeForObjectId(objectId);
     if (node)
         *nodeId = pushNodePathToFrontend(node);
@@ -899,7 +1191,7 @@ PassRefPtr<InspectorObject> InspectorDOMAgent::buildObjectForNode(Node* node, in
 {
     RefPtr<InspectorObject> value = InspectorObject::create();
 
-    long id = bind(node, nodesMap);
+    int id = bind(node, nodesMap);
     String nodeName;
     String localName;
     String nodeValue;
@@ -923,7 +1215,7 @@ PassRefPtr<InspectorObject> InspectorDOMAgent::buildObjectForNode(Node* node, in
             break;
     }
 
-    value->setNumber("id", id);
+    value->setNumber("nodeId", id);
     value->setNumber("nodeType", node->nodeType());
     value->setString("nodeName", nodeName);
     value->setString("localName", localName);
@@ -946,6 +1238,7 @@ PassRefPtr<InspectorObject> InspectorDOMAgent::buildObjectForNode(Node* node, in
         } else if (node->nodeType() == Node::DOCUMENT_NODE) {
             Document* document = static_cast<Document*>(node);
             value->setString("documentURL", documentURLString(document));
+            value->setString("xmlVersion", document->xmlVersion());
         }
     } else if (node->nodeType() == Node::DOCUMENT_TYPE_NODE) {
         DocumentType* docType = static_cast<DocumentType*>(node);
@@ -983,14 +1276,15 @@ PassRefPtr<InspectorArray> InspectorDOMAgent::buildArrayForContainerChildren(Nod
     Node* child = innerFirstChild(container);
 
     if (depth == 0) {
-        // Special case the_only text child.
+        // Special-case the only text child - pretend that container's children have been requested.
         if (child && child->nodeType() == Node::TEXT_NODE && !innerNextSibling(child))
-            children->pushObject(buildObjectForNode(child, 0, nodesMap));
+            return buildArrayForContainerChildren(container, 1, nodesMap);
         return children.release();
-    } else if (depth > 0) {
-        depth--;
     }
 
+    depth--;
+    m_childrenRequested.add(bind(container, nodesMap));
+
     while (child) {
         children->pushObject(buildObjectForNode(child, depth, nodesMap));
         child = innerNextSibling(child);
@@ -1006,12 +1300,14 @@ PassRefPtr<InspectorObject> InspectorDOMAgent::buildObjectForEventListener(const
     value->setBoolean("useCapture", registeredEventListener.useCapture);
     value->setBoolean("isAttribute", eventListener->isAttribute());
     value->setNumber("nodeId", pushNodePathToFrontend(node));
-    value->setString("listenerBody", eventListenerHandlerBody(node->document(), eventListener.get()));
+    value->setString("handlerBody", eventListenerHandlerBody(node->document(), eventListener.get()));
     String sourceName;
     int lineNumber;
     if (eventListenerHandlerLocation(node->document(), eventListener.get(), sourceName, lineNumber)) {
-        value->setString("sourceName", sourceName);
-        value->setNumber("lineNumber", lineNumber);
+        RefPtr<InspectorObject> location = InspectorObject::create();
+        location->setString("scriptId", sourceName);
+        location->setNumber("lineNumber", lineNumber);
+        value->setObject("location", location);
     }
     return value.release();
 }
@@ -1085,7 +1381,7 @@ void InspectorDOMAgent::loadEventFired(Document* document)
     if (!frameOwner)
         return;
 
-    long frameOwnerId = m_documentNodeToIdMap.get(frameOwner);
+    int frameOwnerId = m_documentNodeToIdMap.get(frameOwner);
     if (!frameOwnerId)
         return;
 
@@ -1094,11 +1390,11 @@ void InspectorDOMAgent::loadEventFired(Document* document)
         m_frontend->childNodeCountUpdated(frameOwnerId, innerChildNodeCount(frameOwner));
     } else {
         // Re-add frame owner element together with its new children.
-        long parentId = m_documentNodeToIdMap.get(innerParentNode(frameOwner));
+        int parentId = m_documentNodeToIdMap.get(innerParentNode(frameOwner));
         m_frontend->childNodeRemoved(parentId, frameOwnerId);
         RefPtr<InspectorObject> value = buildObjectForNode(frameOwner, 0, &m_documentNodeToIdMap);
         Node* previousSibling = innerPreviousSibling(frameOwner);
-        long prevId = previousSibling ? m_documentNodeToIdMap.get(previousSibling) : 0;
+        int prevId = previousSibling ? m_documentNodeToIdMap.get(previousSibling) : 0;
         m_frontend->childNodeInserted(parentId, prevId, value.release());
         // Invalidate children requested flag for the element.
         m_childrenRequested.remove(m_childrenRequested.find(frameOwnerId));
@@ -1114,7 +1410,7 @@ void InspectorDOMAgent::didInsertDOMNode(Node* node)
     unbind(node, &m_documentNodeToIdMap);
 
     ContainerNode* parent = node->parentNode();
-    long parentId = m_documentNodeToIdMap.get(parent);
+    int parentId = m_documentNodeToIdMap.get(parent);
     // Return if parent is not mapped yet.
     if (!parentId)
         return;
@@ -1125,7 +1421,7 @@ void InspectorDOMAgent::didInsertDOMNode(Node* node)
     } else {
         // Children have been requested -> return value of a new child.
         Node* prevSibling = innerPreviousSibling(node);
-        long prevId = prevSibling ? m_documentNodeToIdMap.get(prevSibling) : 0;
+        int prevId = prevSibling ? m_documentNodeToIdMap.get(prevSibling) : 0;
         RefPtr<InspectorObject> value = buildObjectForNode(node, 0, &m_documentNodeToIdMap);
         m_frontend->childNodeInserted(parentId, prevId, value.release());
     }
@@ -1137,7 +1433,7 @@ void InspectorDOMAgent::didRemoveDOMNode(Node* node)
         return;
 
     ContainerNode* parent = node->parentNode();
-    long parentId = m_documentNodeToIdMap.get(parent);
+    int parentId = m_documentNodeToIdMap.get(parent);
     // If parent is not mapped yet -> ignore the event.
     if (!parentId)
         return;
@@ -1156,7 +1452,7 @@ void InspectorDOMAgent::didRemoveDOMNode(Node* node)
 
 void InspectorDOMAgent::didModifyDOMAttr(Element* element)
 {
-    long id = m_documentNodeToIdMap.get(element);
+    int id = boundNodeId(element);
     // If node is not mapped yet -> ignore the event.
     if (!id)
         return;
@@ -1164,12 +1460,29 @@ void InspectorDOMAgent::didModifyDOMAttr(Element* element)
     if (m_domListener)
         m_domListener->didModifyDOMAttr(element);
 
-    m_frontend->attributesUpdated(id, buildArrayForElementAttributes(element));
+    m_frontend->attributesUpdated(id);
+}
+
+void InspectorDOMAgent::styleAttributeInvalidated(const Vector<Element*>& elements)
+{
+    RefPtr<InspectorArray> nodeIds = InspectorArray::create();
+    for (unsigned i = 0, size = elements.size(); i < size; ++i) {
+        Element* element = elements.at(i);
+        int id = boundNodeId(element);
+        // If node is not mapped yet -> ignore the event.
+        if (!id)
+            continue;
+
+        if (m_domListener)
+            m_domListener->didModifyDOMAttr(element);
+        nodeIds->pushNumber(id);
+    }
+    m_frontend->inlineStyleInvalidated(nodeIds.release());
 }
 
 void InspectorDOMAgent::characterDataModified(CharacterData* characterData)
 {
-    long id = m_documentNodeToIdMap.get(characterData);
+    int id = m_documentNodeToIdMap.get(characterData);
     if (!id)
         return;
     m_frontend->characterDataModified(id, characterData->data());
@@ -1177,13 +1490,13 @@ void InspectorDOMAgent::characterDataModified(CharacterData* characterData)
 
 void InspectorDOMAgent::didInvalidateStyleAttr(Node* node)
 {
-    long id = m_documentNodeToIdMap.get(node);
+    int id = m_documentNodeToIdMap.get(node);
     // If node is not mapped yet -> ignore the event.
     if (!id)
         return;
 
     if (!m_revalidateStyleAttrTask)
-        m_revalidateStyleAttrTask = new RevalidateStyleAttributeTask(this);
+        m_revalidateStyleAttrTask = adoptPtr(new RevalidateStyleAttributeTask(this));
     m_revalidateStyleAttrTask->scheduleFor(static_cast<Element*>(node));
 }
 
@@ -1218,14 +1531,6 @@ Node* InspectorDOMAgent::nodeForPath(const String& path)
     return node;
 }
 
-PassRefPtr<InspectorArray> InspectorDOMAgent::toArray(const Vector<String>& data)
-{
-    RefPtr<InspectorArray> result = InspectorArray::create();
-    for (unsigned i = 0; i < data.size(); ++i)
-        result->pushString(data[i]);
-    return result.release();
-}
-
 void InspectorDOMAgent::onMatchJobsTimer(Timer<InspectorDOMAgent>*)
 {
     if (!m_pendingMatchJobs.size()) {
@@ -1251,12 +1556,12 @@ void InspectorDOMAgent::reportNodesAsSearchResults(ListHashSet<Node*>& resultCol
         if (m_searchResults.contains(*it))
             continue;
         m_searchResults.add(*it);
-        nodeIds->pushNumber(static_cast<long long>(pushNodePathToFrontend(*it)));
+        nodeIds->pushNumber(pushNodePathToFrontend(*it));
     }
     m_frontend->searchResults(nodeIds.release());
 }
 
-void InspectorDOMAgent::copyNode(ErrorString*, long nodeId)
+void InspectorDOMAgent::copyNode(ErrorString*, int nodeId)
 {
     Node* node = nodeForId(nodeId);
     if (!node)
@@ -1265,7 +1570,7 @@ void InspectorDOMAgent::copyNode(ErrorString*, long nodeId)
     Pasteboard::generalPasteboard()->writePlainText(markup);
 }
 
-void InspectorDOMAgent::pushNodeByPathToFrontend(ErrorString*, const String& path, long* nodeId)
+void InspectorDOMAgent::pushNodeByPathToFrontend(ErrorString*, const String& path, int* nodeId)
 {
     if (Node* node = nodeForPath(path))
         *nodeId = pushNodePathToFrontend(node);
@@ -1273,7 +1578,7 @@ void InspectorDOMAgent::pushNodeByPathToFrontend(ErrorString*, const String& pat
 
 PassRefPtr<InspectorObject> InspectorDOMAgent::resolveNode(Node* node, const String& objectGroup)
 {
-    Document* document = node->ownerDocument();
+    Document* document = node->isDocumentNode() ? node->document() : node->ownerDocument();
     Frame* frame = document ? document->frame() : 0;
     if (!frame)
         return 0;
@@ -1285,6 +1590,14 @@ PassRefPtr<InspectorObject> InspectorDOMAgent::resolveNode(Node* node, const Str
     return injectedScript.wrapNode(node, objectGroup);
 }
 
+void InspectorDOMAgent::drawHighlight(GraphicsContext& context) const
+{
+    if (!m_highlightData)
+        return;
+
+    DOMNodeHighlighter::drawHighlight(context, m_highlightData->node ? m_highlightData->node->document() : m_document.get(), m_highlightData.get());
+}
+
 } // namespace WebCore
 
 #endif // ENABLE(INSPECTOR)
