@@ -43,6 +43,7 @@
 #include "FrameLoaderClient.h"
 #include "FrameTree.h"
 #include "HistoryItem.h"
+#include "InspectorInstrumentation.h"
 #include "Logging.h"
 #include "MainResourceLoader.h"
 #include "Page.h"
@@ -54,7 +55,7 @@
 #include <wtf/text/CString.h>
 #include <wtf/unicode/Unicode.h>
 
-#if ENABLE(WEB_ARCHIVE)
+#if ENABLE(WEB_ARCHIVE) || ENABLE(MHTML)
 #include "ArchiveFactory.h"
 #endif
 
@@ -177,17 +178,17 @@ void DocumentLoader::setRequest(const ResourceRequest& req)
     KURL oldURL = m_request.url();
     m_request = req;
 
-    // Only send webView:didReceiveServerRedirectForProvisionalLoadForFrame: if URL changed (and is non-null).
+    // Only dispatchDidReceiveServerRedirectForProvisionalLoad() if URL changed (and is non-null).
     // Also, don't send it when replacing unreachable URLs with alternate content.
     if (!handlingUnreachableURL && !req.url().isNull() && oldURL != req.url())
-        frameLoader()->didReceiveServerRedirectForProvisionalLoadForFrame();
+        frameLoader()->client()->dispatchDidReceiveServerRedirectForProvisionalLoad();
 }
 
 void DocumentLoader::setMainDocumentError(const ResourceError& error)
 {
     m_mainDocumentError = error;    
-    frameLoader()->setMainDocumentError(this, error);
- }
+    frameLoader()->client()->setMainDocumentError(this, error);
+}
 
 void DocumentLoader::clearErrors()
 {
@@ -300,14 +301,14 @@ void DocumentLoader::commitLoad(const char* data, int length)
     FrameLoader* frameLoader = DocumentLoader::frameLoader();
     if (!frameLoader)
         return;
-#if ENABLE(WEB_ARCHIVE)
+#if ENABLE(WEB_ARCHIVE) || ENABLE(MHTML)
     if (ArchiveFactory::isArchiveMimeType(response().mimeType()))
         return;
 #endif
     frameLoader->client()->committedLoad(this, data, length);
 }
 
-void DocumentLoader::commitData(const char* bytes, int length)
+void DocumentLoader::commitData(const char* bytes, size_t length)
 {
     // Set the text encoding.  This is safe to call multiple times.
     bool userChosen = true;
@@ -341,7 +342,7 @@ void DocumentLoader::setupForReplaceByMIMEType(const String& newMIMEType)
     String oldMIMEType = m_response.mimeType();
     
     if (!doesProgressiveLoad(oldMIMEType)) {
-        frameLoader()->revertToProvisional(this);
+        frameLoader()->client()->revertToProvisionalState(this);
         setupForReplace();
         RefPtr<SharedBuffer> resourceData = mainResourceData();
         commitLoad(resourceData->data(), resourceData->size());
@@ -354,13 +355,13 @@ void DocumentLoader::setupForReplaceByMIMEType(const String& newMIMEType)
     m_gotFirstByte = false;
     
     if (doesProgressiveLoad(newMIMEType)) {
-        frameLoader()->revertToProvisional(this);
+        frameLoader()->client()->revertToProvisionalState(this);
         setupForReplace();
     }
     
     stopLoadingSubresources();
     stopLoadingPlugIns();
-#if ENABLE(WEB_ARCHIVE)
+#if ENABLE(WEB_ARCHIVE) || ENABLE(MHTML)
     clearArchiveResources();
 #endif
 }
@@ -399,9 +400,11 @@ void DocumentLoader::attachToFrame()
 void DocumentLoader::detachFromFrame()
 {
     ASSERT(m_frame);
+
 #if ENABLE(OFFLINE_WEB_APPLICATIONS)
     m_applicationCacheHost->setDOMApplicationCache(0);
 #endif
+    InspectorInstrumentation::loaderDetachedFromFrame(m_frame, this);
     m_frame = 0;
 }
 
@@ -436,11 +439,9 @@ bool DocumentLoader::isLoadingInAPISense() const
     // Once a frame has loaded, we no longer need to consider subresources,
     // but we still need to consider subframes.
     if (frameLoader()->state() != FrameStateComplete) {
-        if (!m_primaryLoadComplete && isLoading())
-            return true;
-        if (!m_subresourceLoaders.isEmpty())
-            return true;
         Document* doc = m_frame->document();
+        if ((!m_primaryLoadComplete || !m_frame->document()->loadEventFinished()) && isLoading())
+            return true;
         if (doc->cachedResourceLoader()->requestCount())
             return true;
         if (DocumentParser* parser = doc->parser())
@@ -450,7 +451,7 @@ bool DocumentLoader::isLoadingInAPISense() const
     return frameLoader()->subframeIsLoading();
 }
 
-#if ENABLE(WEB_ARCHIVE)
+#if ENABLE(WEB_ARCHIVE) || ENABLE(MHTML)
 void DocumentLoader::addAllArchiveResources(Archive* archive)
 {
     if (!m_archiveResourceCollection)
@@ -477,9 +478,9 @@ void DocumentLoader::addArchiveResource(PassRefPtr<ArchiveResource> resource)
     m_archiveResourceCollection->addResource(resource);
 }
 
-PassRefPtr<Archive> DocumentLoader::popArchiveForSubframe(const String& frameName)
+PassRefPtr<Archive> DocumentLoader::popArchiveForSubframe(const String& frameName, const KURL& url)
 {
-    return m_archiveResourceCollection ? m_archiveResourceCollection->popSubframeArchive(frameName) : 0;
+    return m_archiveResourceCollection ? m_archiveResourceCollection->popSubframeArchive(frameName, url) : PassRefPtr<Archive>(0);
 }
 
 void DocumentLoader::clearArchiveResources()
@@ -497,7 +498,7 @@ SharedBuffer* DocumentLoader::parsedArchiveData() const
 {
     return m_parsedArchiveData.get();
 }
-#endif // ENABLE(WEB_ARCHIVE)
+#endif // ENABLE(WEB_ARCHIVE) || ENABLE(MHTML)
 
 ArchiveResource* DocumentLoader::archiveResourceForURL(const KURL& url) const
 {
@@ -589,7 +590,19 @@ void DocumentLoader::substituteResourceDeliveryTimerFired(Timer<DocumentLoader>*
             SharedBuffer* data = resource->data();
         
             loader->didReceiveResponse(resource->response());
+
+            // Calling ResourceLoader::didReceiveResponse can end up cancelling the load,
+            // so we need to check if the loader has reached its terminal state.
+            if (loader->reachedTerminalState())
+                return;
+
             loader->didReceiveData(data->data(), data->size(), data->size(), true);
+
+            // Calling ResourceLoader::didReceiveData can end up cancelling the load,
+            // so we need to check if the loader has reached its terminal state.
+            if (loader->reachedTerminalState())
+                return;
+
             loader->didFinishLoading(0);
         } else {
             // A null resource means that we should fail the load.
@@ -615,26 +628,34 @@ void DocumentLoader::cancelPendingSubstituteLoad(ResourceLoader* loader)
         m_substituteResourceDeliveryTimer.stop();
 }
 
-#if ENABLE(WEB_ARCHIVE)
+#if ENABLE(WEB_ARCHIVE) || ENABLE(MHTML)
 bool DocumentLoader::scheduleArchiveLoad(ResourceLoader* loader, const ResourceRequest& request, const KURL& originalURL)
 {
-    ArchiveResource* resource = 0;
-    
-    if (request.url() == originalURL)
-        resource = archiveResourceForURL(originalURL);
+    if (request.url() == originalURL) {
+        if (ArchiveResource* resource = archiveResourceForURL(originalURL)) {
+            m_pendingSubstituteResources.set(loader, resource);
+            deliverSubstituteResourcesAfterDelay();
+            return true;
+        }
+    }
 
-    if (!resource) {
+    Archive* archive = frameLoader()->archive();
+    if (!archive)
+        return false;
+
+    switch (archive->type()) {
+#if ENABLE(WEB_ARCHIVE)
+    case Archive::WebArchive:
         // WebArchiveDebugMode means we fail loads instead of trying to fetch them from the network if they're not in the archive.
-        bool shouldFailLoad = m_frame->settings()->webArchiveDebugModeEnabled() && ArchiveFactory::isArchiveMimeType(responseMIMEType());
-
-        if (!shouldFailLoad)
-            return false;
+        return m_frame->settings()->webArchiveDebugModeEnabled() && ArchiveFactory::isArchiveMimeType(responseMIMEType());
+#endif
+#if ENABLE(MHTML)
+    case Archive::MHTML:
+        return true; // Always fail the load for resources not included in the MHTML.
+#endif
+    default:
+        return false;
     }
-    
-    m_pendingSubstituteResources.set(loader, resource);
-    deliverSubstituteResourcesAfterDelay();
-    
-    return true;
 }
 #endif // ENABLE(WEB_ARCHIVE)
 
@@ -649,7 +670,7 @@ void DocumentLoader::stopRecordingResponses()
     m_stopRecordingResponses = true;
 }
 
-void DocumentLoader::setTitle(const String& title)
+void DocumentLoader::setTitle(const StringWithDirection& title)
 {
     if (title.isEmpty())
         return;
@@ -661,14 +682,19 @@ void DocumentLoader::setTitle(const String& title)
     }
 }
 
-void DocumentLoader::setIconURL(const String& iconURL)
+IconURL DocumentLoader::iconURL(IconType iconType) const
 {
-    if (iconURL.isEmpty())
+    return m_iconURLs[toIconIndex(iconType)];
+}
+
+void DocumentLoader::setIconURL(const IconURL& url)
+{
+    if (url.m_iconURL.isEmpty())
         return;
 
-    if (m_pageIconURL != iconURL) {
-        m_pageIconURL = iconURL;
-        frameLoader()->didChangeIcons(this);
+    if (iconURL(url.m_iconType).m_iconURL != url.m_iconURL) {
+        m_iconURLs[toIconIndex(url.m_iconType)] = url;
+        frameLoader()->didChangeIcons(this, url.m_iconType);
     }
 }
 
@@ -703,6 +729,18 @@ const KURL& DocumentLoader::responseURL() const
     return m_response.url();
 }
 
+KURL DocumentLoader::documentURL() const
+{
+    KURL url = substituteData().responseURL();
+    if (url.isEmpty())
+        url = requestURL();
+    if (url.isEmpty())
+        url = responseURL();
+    if (url.isEmpty())
+        url = blankURL();
+    return url;
+}
+
 const String& DocumentLoader::responseMIMEType() const
 {
     return m_response.mimeType();
@@ -821,26 +859,21 @@ void DocumentLoader::transferLoadingResourcesFromPage(Page* oldPage)
     FrameLoader* loader = frameLoader();
     ASSERT(loader);
 
-    const ResourceRequest& request = originalRequest();
-    if (isLoadingMainResource()) {
-        loader->dispatchTransferLoadingResourceFromPage(
-            m_mainResourceLoader->identifier(), this, request, oldPage);
-    }
+    if (isLoadingMainResource())
+        loader->dispatchTransferLoadingResourceFromPage(m_mainResourceLoader.get(), originalRequest(), oldPage);
 
     if (isLoadingSubresources()) {
         ResourceLoaderSet::const_iterator it = m_subresourceLoaders.begin();
         ResourceLoaderSet::const_iterator end = m_subresourceLoaders.end();
-        for (; it != end; ++it) {
-            loader->dispatchTransferLoadingResourceFromPage(
-                (*it)->identifier(), this, request, oldPage);
-        }
+        for (; it != end; ++it)
+            loader->dispatchTransferLoadingResourceFromPage((*it).get(), (*it)->originalRequest(), oldPage);
     }
 }
 
 void DocumentLoader::iconLoadDecisionAvailable()
 {
     if (m_frame)
-        m_frame->loader()->iconLoadDecisionReceived(iconDatabase().synchronousLoadDecisionForIconURL(KURL(frameLoader()->iconURL()), this));
+        m_frame->loader()->icon()->loadDecisionReceived(iconDatabase().synchronousLoadDecisionForIconURL(frameLoader()->icon()->url(), this));
 }
 
 static void iconLoadDecisionCallback(IconLoadDecision decision, void* context)
@@ -858,8 +891,10 @@ void DocumentLoader::getIconLoadDecisionForIconURL(const String& urlString)
 
 void DocumentLoader::continueIconLoadWithDecision(IconLoadDecision decision)
 {
+    ASSERT(m_iconLoadDecisionCallback);
+    m_iconLoadDecisionCallback = 0;
     if (m_frame)
-        m_frame->loader()->continueIconLoadWithDecision(decision);
+        m_frame->loader()->icon()->continueLoadWithDecision(decision);
 }
 
 static void iconDataCallback(SharedBuffer*, void*)
@@ -875,4 +910,12 @@ void DocumentLoader::getIconDataForIconURL(const String& urlString)
     iconDatabase().iconDataForIconURL(urlString, m_iconDataCallback);
 }
 
+void DocumentLoader::handledOnloadEvents()
+{
+    m_wasOnloadHandled = true;
+#if ENABLE(OFFLINE_WEB_APPLICATIONS)
+    applicationCacheHost()->stopDeferringEvents();
+#endif
+}
+
 } // namespace WebCore
