@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006, 2007, 2008, 2009 Apple Inc. All rights reserved.
+ * Copyright (C) 2006, 2007, 2008, 2009, 2011 Apple Inc. All rights reserved.
  * Copyright (C) 2008, 2009 Torch Mobile Inc. All rights reserved. (http://www.torchmobile.com/)
  * Copyright (C) Research In Motion Limited 2009. All rights reserved.
  *
@@ -35,11 +35,12 @@
 #include "FrameLoaderStateMachine.h"
 #include "FrameLoaderTypes.h"
 #include "HistoryController.h"
-#include "IconDatabaseBase.h"
+#include "IconController.h"
+#include "IconURL.h"
 #include "PolicyChecker.h"
+#include "ResourceHandle.h"
 #include "ResourceLoadNotifier.h"
 #include "SubframeLoader.h"
-#include "ThreadableLoader.h"
 #include "Timer.h"
 #include <wtf/Forward.h>
 #include <wtf/HashSet.h>
@@ -47,39 +48,27 @@
 namespace WebCore {
 
 class Archive;
-class AuthenticationChallenge;
 class CachedFrameBase;
 class CachedPage;
 class CachedResource;
 class Chrome;
 class DOMWrapperWorld;
-class Document;
 class DocumentLoader;
 class Event;
-class FormData;
 class FormState;
 class FormSubmission;
-class Frame;
 class FrameLoaderClient;
 class FrameNetworkingContext;
-class HistoryItem;
-class HTMLFormElement;
-class IconLoader;
 class NavigationAction;
 class NetworkingContext;
 class Page;
-class ProtectionSpace;
 class ResourceError;
-class ResourceLoader;
 class ResourceRequest;
 class ResourceResponse;
-class ScriptSourceCode;
-class ScriptValue;
 class SecurityOrigin;
 class SerializedScriptValue;
-class SharedBuffer;
+class StringWithDirection;
 class SubstituteData;
-class TextResourceDecoder;
 
 struct FrameLoadRequest;
 struct WindowFeatures;
@@ -100,43 +89,51 @@ public:
     HistoryController* history() const { return &m_history; }
     ResourceLoadNotifier* notifier() const { return &m_notifer; }
     SubframeLoader* subframeLoader() const { return &m_subframeLoader; }
+    IconController* icon() const { return &m_icon; }
 
-    // FIXME: This is not cool, people. There are too many different functions that all start loads.
-    // We should aim to consolidate these into a smaller set of functions, and try to reuse more of
-    // the logic by extracting common code paths.
-
+    void prepareForHistoryNavigation();
     void prepareForLoadStart();
     void setupForReplace();
-    void setupForReplaceByMIMEType(const String& newMIMEType);
 
+    // FIXME: These are all functions which start loads. We have too many.
     void loadURLIntoChildFrame(const KURL&, const String& referer, Frame*);
-
     void loadFrameRequest(const FrameLoadRequest&, bool lockHistory, bool lockBackForwardList,  // Called by submitForm, calls loadPostRequest and loadURL.
         PassRefPtr<Event>, PassRefPtr<FormState>, ReferrerPolicy);
 
     void load(const ResourceRequest&, bool lockHistory);                                        // Called by WebFrame, calls load(ResourceRequest, SubstituteData).
     void load(const ResourceRequest&, const SubstituteData&, bool lockHistory);                 // Called both by WebFrame and internally, calls load(DocumentLoader*).
     void load(const ResourceRequest&, const String& frameName, bool lockHistory);               // Called by WebPluginController.
-
-#if ENABLE(WEB_ARCHIVE)
+#if ENABLE(WEB_ARCHIVE) || ENABLE(MHTML)
     void loadArchive(PassRefPtr<Archive>);
 #endif
+    unsigned long loadResourceSynchronously(const ResourceRequest&, StoredCredentials, ResourceError&, ResourceResponse&, Vector<char>& data);
+
+    void changeLocation(SecurityOrigin*, const KURL&, const String& referrer, bool lockHistory = true, bool lockBackForwardList = true, bool refresh = false);
+    void urlSelected(const KURL&, const String& target, PassRefPtr<Event>, bool lockHistory, bool lockBackForwardList, ReferrerPolicy);
+    void submitForm(PassRefPtr<FormSubmission>);
+
+    void reload(bool endToEndReload = false);
+    void reloadWithOverrideEncoding(const String& overrideEncoding);
+
+    void open(CachedFrameBase&);
+    void loadItem(HistoryItem*, FrameLoadType);
 
     static void reportLocalLoadFailed(Frame*, const String& url);
 
-    unsigned long loadResourceSynchronously(const ResourceRequest&, StoredCredentials, ResourceError&, ResourceResponse&, Vector<char>& data);
-
-    bool canHandleRequest(const ResourceRequest&);
-
-    // Also not cool.
+    // FIXME: These are all functions which stop loads. We have too many.
     void stopAllLoaders(ClearProvisionalItemPolicy = ShouldClearProvisionalItem);
     void stopForUserCancel(bool deferCheckLoadComplete = false);
+    void stop();
+    void stopLoading(UnloadEventPolicy);
+    bool closeURL();
+    void cancelAndClear();
+    void clear(bool clearWindowProperties = true, bool clearScriptObjects = true, bool clearFrameView = true);
 
     bool isLoadingMainResource() const { return m_isLoadingMainResource; }
     bool isLoading() const;
     bool frameHasLoaded() const;
     void transferLoadingResourcesFromPage(Page*);
-    void dispatchTransferLoadingResourceFromPage(unsigned long, DocumentLoader*, const ResourceRequest&, Page*);
+    void dispatchTransferLoadingResourceFromPage(ResourceLoader*, const ResourceRequest&, Page*);
 
     int numPendingOrLoadingRequests(bool recurse) const;
     String referrer() const;
@@ -150,10 +147,6 @@ public:
     FrameState state() const { return m_state; }
     static double timeOfLastCompletedLoad();
 
-    bool shouldUseCredentialStorage(ResourceLoader*);
-#if USE(PROTECTION_SPACE_AUTH_CALLBACK)
-    bool canAuthenticateAgainstProtectionSpace(ResourceLoader* loader, const ProtectionSpace& protectionSpace);
-#endif
     const ResourceRequest& originalRequest() const;
     const ResourceRequest& initialRequest() const;
     void receivedMainResourceError(const ResourceError&, bool isComplete);
@@ -161,46 +154,31 @@ public:
     bool willLoadMediaElementURL(KURL&);
 
     void handleFallbackContent();
-    bool isStopping() const;
 
     void finishedLoading();
 
     ResourceError cancelledError(const ResourceRequest&) const;
-    ResourceError fileDoesNotExistError(const ResourceResponse&) const;
-    ResourceError blockedError(const ResourceRequest&) const;
-    ResourceError cannotShowURLError(const ResourceRequest&) const;
-    ResourceError interruptionForPolicyChangeError(const ResourceRequest&) const;
 
     bool isHostedByObjectElement() const;
     bool isLoadingMainFrame() const;
-    bool canShowMIMEType(const String& MIMEType) const;
-    bool representationExistsForURLScheme(const String& URLScheme);
-    String generatedMIMETypeForURLScheme(const String& URLScheme);
 
-    void reload(bool endToEndReload = false);
-    void reloadWithOverrideEncoding(const String& overrideEncoding);
-
-    void didReceiveServerRedirectForProvisionalLoadForFrame();
     void finishedLoadingDocument(DocumentLoader*);
     bool isReplacing() const;
     void setReplacing();
-    void revertToProvisional(DocumentLoader*);
-    void setMainDocumentError(DocumentLoader*, const ResourceError&);
     void mainReceivedCompleteError(DocumentLoader*, const ResourceError&);
     bool subframeIsLoading() const;
     void willChangeTitle(DocumentLoader*);
     void didChangeTitle(DocumentLoader*);
-    void didChangeIcons(DocumentLoader*);
+    void didChangeIcons(DocumentLoader*, IconType);
 
     FrameLoadType loadType() const;
 
     CachePolicy subresourceCachePolicy() const;
 
     void didFirstLayout();
-
     void didFirstVisuallyNonEmptyLayout();
 
-    void loadedResourceFromMemoryCache(const CachedResource*);
+    void loadedResourceFromMemoryCache(CachedResource*);
     void tellClientAboutPastMemoryCacheLoads();
 
     void checkLoadComplete();
@@ -210,21 +188,12 @@ public:
     void addExtraFieldsToSubresourceRequest(ResourceRequest&);
     void addExtraFieldsToMainResourceRequest(ResourceRequest&);
     
-    static void addHTTPOriginIfNeeded(ResourceRequest&, String origin);
+    static void addHTTPOriginIfNeeded(ResourceRequest&, const String& origin);
 
     FrameLoaderClient* client() const { return m_client; }
 
     void setDefersLoading(bool);
 
-    void changeLocation(PassRefPtr<SecurityOrigin>, const KURL&, const String& referrer, bool lockHistory = true, bool lockBackForwardList = true, bool refresh = false);
-    void urlSelected(const KURL&, const String& target, PassRefPtr<Event>, bool lockHistory, bool lockBackForwardList, ReferrerPolicy);
-
-    void submitForm(PassRefPtr<FormSubmission>);
-
-    void stop();
-    void stopLoading(UnloadEventPolicy);
-    bool closeURL();
-
     void didExplicitOpen();
 
     // Callbacks from DocumentWriter
@@ -232,11 +201,6 @@ public:
     void didEndDocument();
     void willSetEncoding();
 
-    KURL iconURL();
-    void commitIconURLToIconDatabase(const KURL&);
-
-    KURL baseURL() const;
-
     void handledOnloadEvents();
     String userAgent(const KURL&) const;
 
@@ -254,14 +218,12 @@ public:
 
     // Mixed content related functions.
     static bool isMixedContent(SecurityOrigin* context, const KURL&);
-    void checkIfDisplayInsecureContent(SecurityOrigin* context, const KURL&);
-    void checkIfRunInsecureContent(SecurityOrigin* context, const KURL&);
+    bool checkIfDisplayInsecureContent(SecurityOrigin* context, const KURL&);
+    bool checkIfRunInsecureContent(SecurityOrigin* context, const KURL&);
 
     Frame* opener();
     void setOpener(Frame*);
 
-    bool isProcessingUserGesture();
-
     void resetMultipleFormSubmissionProtection();
 
     void checkCallImplicitClose();
@@ -278,43 +240,23 @@ public:
 
     bool isComplete() const;
 
-    KURL completeURL(const String& url);
-
-    void cancelAndClear();
-
-    void setTitle(const String&);
-    void setIconURL(const String&);
+    void setTitle(const StringWithDirection&);
 
     void commitProvisionalLoad();
-    bool isLoadingFromCachedPage() const { return m_loadingFromCachedPage; }
 
     FrameLoaderStateMachine* stateMachine() const { return &m_stateMachine; }
 
-    void startIconLoader();
-    void iconLoadDecisionReceived(IconLoadDecision);
-    void continueIconLoadWithDecision(IconLoadDecision);
-
     bool shouldAllowNavigation(Frame* targetFrame) const;
     Frame* findFrameForNavigation(const AtomicString& name);
 
-    void applyUserAgent(ResourceRequest& request);
+    void applyUserAgent(ResourceRequest&);
 
     bool shouldInterruptLoadForXFrameOptions(const String&, const KURL&);
 
-    void open(CachedFrameBase&);
-
-#if ENABLE(PLUGIN_PROXY_FOR_VIDEO)
-    void hideMediaPlayerProxyPlugin(Widget*);
-    void showMediaPlayerProxyPlugin(Widget*);
-#endif
-
-    // FIXME: Should these really be public?
     void completed();
     bool allAncestorsAreComplete() const; // including this
-    bool allChildrenAreComplete() const; // immediate children, not all descendants
     void clientRedirected(const KURL&, double delay, double fireDate, bool lockBackForwardList);
     void clientRedirectCancelledOrFinished(bool cancelWithLoadInProgress);
-    void loadItem(HistoryItem*, FrameLoadType);
 
     // FIXME: This is public because this asynchronous callback from the FrameLoaderClient
     // uses the policy machinery (and therefore is called via the PolicyChecker).  Once we
@@ -323,9 +265,7 @@ public:
     
     bool suppressOpenerInNewFrame() const { return m_suppressOpenerInNewFrame; }
 
-    static ObjectContentType defaultObjectContentType(const KURL& url, const String& mimeType);
-
-    void clear(bool clearWindowProperties = true, bool clearScriptObjects = true, bool clearFrameView = true);
+    static ObjectContentType defaultObjectContentType(const KURL&, const String& mimeType, bool shouldPreferPlugInsForImages);
 
     bool quickRedirectComing() const { return m_quickRedirectComing; }
 
@@ -333,11 +273,23 @@ public:
     
     void started();
 
-    bool pageDismissalEventBeingDispatched() const { return m_pageDismissalEventBeingDispatched; }
+    enum PageDismissalType {
+        NoDismissal = 0,
+        BeforeUnloadDismissal = 1,
+        PageHideDismissal = 2,
+        UnloadDismissal = 3
+    };
+    PageDismissalType pageDismissalEventBeingDispatched() const { return m_pageDismissalEventBeingDispatched; }
 
     NetworkingContext* networkingContext() const;
 
+#if ENABLE(WEB_ARCHIVE) || ENABLE(MHTML)
+    Archive* archive() const { return m_archive.get(); }
+#endif
+
 private:
+    bool allChildrenAreComplete() const; // immediate children, not all descendants
+
     void checkTimerFired(Timer<FrameLoader>*);
     
     void loadSameDocumentItem(HistoryItem*);
@@ -350,18 +302,12 @@ private:
     void updateFirstPartyForCookies();
     void setFirstPartyForCookies(const KURL&);
     
-    void addExtraFieldsToRequest(ResourceRequest&, FrameLoadType loadType, bool isMainResource, bool cookiePolicyURLFromRequest);
-
-    // Also not cool.
-    void stopLoadingSubframes(ClearProvisionalItemPolicy);
+    void addExtraFieldsToRequest(ResourceRequest&, FrameLoadType, bool isMainResource);
 
     void clearProvisionalLoad();
-    void markLoadComplete();
     void transitionToCommitted(PassRefPtr<CachedPage>);
     void frameLoadCompleted();
 
-    void mainReceivedError(const ResourceError&, bool isComplete);
-
     static void callContinueLoadAfterNavigationPolicy(void*, const ResourceRequest&, PassRefPtr<FormState>, bool shouldContinue);
     static void callContinueLoadAfterNewWindowPolicy(void*, const ResourceRequest&, PassRefPtr<FormState>, const String& frameName, const NavigationAction&, bool shouldContinue);
     static void callContinueFragmentScrollAfterNavigationPolicy(void*, const ResourceRequest&, PassRefPtr<FormState>, bool shouldContinue);
@@ -406,8 +352,6 @@ private:
 
     void requestFromDelegate(ResourceRequest&, unsigned long& identifier, ResourceError&);
 
-    void recursiveCheckLoadComplete();
-
     void detachChildren();
     void closeAndRemoveChild(Frame*);
 
@@ -415,14 +359,12 @@ private:
 
     void provisionalLoadStarted();
 
-    bool didOpenURL(const KURL&);
+    bool didOpenURL();
 
     void scheduleCheckCompleted();
     void scheduleCheckLoadComplete();
     void startCheckCompleteTimer();
 
-    KURL originalRequestURL() const;
-
     bool shouldTreatURLAsSameAsCurrent(const KURL&) const;
 
     void updateSandboxFlags();
@@ -435,6 +377,7 @@ private:
     mutable ResourceLoadNotifier m_notifer;
     mutable SubframeLoader m_subframeLoader;
     mutable FrameLoaderStateMachine m_stateMachine;
+    mutable IconController m_icon;
 
     FrameState m_state;
     FrameLoadType m_loadType;
@@ -459,16 +402,13 @@ private:
 
     bool m_didCallImplicitClose;
     bool m_wasUnloadEventEmitted;
-    bool m_pageDismissalEventBeingDispatched;
+    PageDismissalType m_pageDismissalEventBeingDispatched;
     bool m_isComplete;
     bool m_isLoadingMainResource;
 
     RefPtr<SerializedScriptValue> m_pendingStateObject;
 
-    KURL m_workingURL;
-
-    OwnPtr<IconLoader> m_iconLoader;
-    bool m_mayLoadIconLater;
+    bool m_hasReceivedFirstData;
 
     bool m_needsClear;
 
@@ -490,6 +430,10 @@ private:
 
     RefPtr<FrameNetworkingContext> m_networkingContext;
 
+#if ENABLE(WEB_ARCHIVE) || ENABLE(MHTML)
+    RefPtr<Archive> m_archive;
+#endif
+
     KURL m_previousUrl;
 };
 
