@@ -49,26 +49,19 @@ namespace WebCore {
 
 PassRefPtr<SharedBuffer> ResourceLoader::resourceData()
 {
-    if (m_resourceData)
-        return m_resourceData;
-
-    if (ResourceHandle::supportsBufferedData() && m_handle)
-        return m_handle->bufferedData();
-    
-    return 0;
+    return m_resourceData;
 }
 
-ResourceLoader::ResourceLoader(Frame* frame, bool sendResourceLoadCallbacks, bool shouldContentSniff)
+ResourceLoader::ResourceLoader(Frame* frame, ResourceLoaderOptions options)
     : m_frame(frame)
     , m_documentLoader(frame->loader()->activeDocumentLoader())
     , m_identifier(0)
     , m_reachedTerminalState(false)
+    , m_calledWillCancel(false)
     , m_cancelled(false)
     , m_calledDidFinishLoad(false)
-    , m_sendResourceLoadCallbacks(sendResourceLoadCallbacks)
-    , m_shouldContentSniff(shouldContentSniff)
-    , m_shouldBufferData(true)
     , m_defersLoading(frame->page()->defersLoading())
+    , m_options(options)
 {
 }
 
@@ -131,11 +124,11 @@ bool ResourceLoader::init(const ResourceRequest& r)
 
     willSendRequest(clientRequest, ResourceResponse());
     if (clientRequest.isNull()) {
-        didFail(frameLoader()->cancelledError(m_request));
+        didFail(cancelledError());
         return false;
     }
 
-    m_request = clientRequest;
+    m_originalRequest = m_request = clientRequest;
     return true;
 }
 
@@ -145,7 +138,7 @@ void ResourceLoader::start()
     ASSERT(!m_request.isNull());
     ASSERT(m_deferredRequest.isNull());
 
-#if ENABLE(WEB_ARCHIVE)
+#if ENABLE(WEB_ARCHIVE) || ENABLE(MHTML)
     if (m_documentLoader->scheduleArchiveLoad(this, m_request, m_request.url()))
         return;
 #endif
@@ -161,7 +154,7 @@ void ResourceLoader::start()
     }
 
     if (!m_reachedTerminalState)
-        m_handle = ResourceHandle::create(m_frame->loader()->networkingContext(), m_request, this, m_defersLoading, m_shouldContentSniff);
+        m_handle = ResourceHandle::create(m_frame->loader()->networkingContext(), m_request, this, m_defersLoading, m_options.sniffContent == SniffContent);
 }
 
 void ResourceLoader::setDefersLoading(bool defers)
@@ -183,19 +176,19 @@ FrameLoader* ResourceLoader::frameLoader() const
     return m_frame->loader();
 }
 
-void ResourceLoader::setShouldBufferData(bool shouldBufferData)
+void ResourceLoader::setShouldBufferData(DataBufferingPolicy shouldBufferData)
 { 
-    m_shouldBufferData = shouldBufferData; 
+    m_options.shouldBufferData = shouldBufferData; 
 
     // Reset any already buffered data
-    if (!m_shouldBufferData)
+    if (!shouldBufferData)
         m_resourceData = 0;
 }
     
 
 void ResourceLoader::addData(const char* data, int length, bool allAtOnce)
 {
-    if (!m_shouldBufferData)
+    if (m_options.shouldBufferData == DoNotBufferData)
         return;
 
     if (allAtOnce) {
@@ -203,16 +196,10 @@ void ResourceLoader::addData(const char* data, int length, bool allAtOnce)
         return;
     }
         
-    if (ResourceHandle::supportsBufferedData()) {
-        // Buffer data only if the connection has handed us the data because is has stopped buffering it.
-        if (m_resourceData)
-            m_resourceData->append(data, length);
-    } else {
-        if (!m_resourceData)
-            m_resourceData = SharedBuffer::create(data, length);
-        else
-            m_resourceData->append(data, length);
-    }
+    if (!m_resourceData)
+        m_resourceData = SharedBuffer::create(data, length);
+    else
+        m_resourceData->append(data, length);
 }
 
 void ResourceLoader::clearResourceData()
@@ -223,13 +210,19 @@ void ResourceLoader::clearResourceData()
 
 void ResourceLoader::willSendRequest(ResourceRequest& request, const ResourceResponse& redirectResponse)
 {
+#if ENABLE(OFFLINE_WEB_APPLICATIONS)
+    if (!fastMallocSize(documentLoader()->applicationCacheHost()))
+        CRASH();
+#endif
+    if (!fastMallocSize(documentLoader()->frame()))
+        CRASH();
     // Protect this in this delegate method since the additional processing can do
     // anything including possibly derefing this; one example of this is Radar 3266216.
     RefPtr<ResourceLoader> protector(this);
 
     ASSERT(!m_reachedTerminalState);
 
-    if (m_sendResourceLoadCallbacks) {
+    if (m_options.sendLoadCallbacks == SendCallbacks) {
         if (!m_identifier) {
             m_identifier = m_frame->page()->progress()->createUniqueIdentifier();
             frameLoader()->notifier()->assignIdentifierToInitialRequest(m_identifier, documentLoader(), request);
@@ -245,10 +238,22 @@ void ResourceLoader::willSendRequest(ResourceRequest& request, const ResourceRes
 
 void ResourceLoader::didSendData(unsigned long long, unsigned long long)
 {
+#if ENABLE(OFFLINE_WEB_APPLICATIONS)
+    if (!fastMallocSize(documentLoader()->applicationCacheHost()))
+        CRASH();
+#endif
+    if (!fastMallocSize(documentLoader()->frame()))
+        CRASH();
 }
 
 void ResourceLoader::didReceiveResponse(const ResourceResponse& r)
 {
+#if ENABLE(OFFLINE_WEB_APPLICATIONS)
+    if (!fastMallocSize(documentLoader()->applicationCacheHost()))
+        CRASH();
+#endif
+    if (!fastMallocSize(documentLoader()->frame()))
+        CRASH();
     ASSERT(!m_reachedTerminalState);
 
     // Protect this in this delegate method since the additional processing can do
@@ -260,12 +265,18 @@ void ResourceLoader::didReceiveResponse(const ResourceResponse& r)
     if (FormData* data = m_request.httpBody())
         data->removeGeneratedFilesIfNeeded();
         
-    if (m_sendResourceLoadCallbacks)
+    if (m_options.sendLoadCallbacks == SendCallbacks)
         frameLoader()->notifier()->didReceiveResponse(this, m_response);
 }
 
-void ResourceLoader::didReceiveData(const char* data, int length, long long lengthReceived, bool allAtOnce)
+void ResourceLoader::didReceiveData(const char* data, int length, long long encodedDataLength, bool allAtOnce)
 {
+#if ENABLE(OFFLINE_WEB_APPLICATIONS)
+    if (!m_cancelled && !fastMallocSize(documentLoader()->applicationCacheHost()))
+        CRASH();
+#endif
+    if (!m_cancelled && !fastMallocSize(documentLoader()->frame()))
+        CRASH();
     // The following assertions are not quite valid here, since a subclass
     // might override didReceiveData in a way that invalidates them. This
     // happens with the steps listed in 3266216
@@ -280,13 +291,13 @@ void ResourceLoader::didReceiveData(const char* data, int length, long long leng
     // FIXME: If we get a resource with more than 2B bytes, this code won't do the right thing.
     // However, with today's computers and networking speeds, this won't happen in practice.
     // Could be an issue with a giant local file.
-    if (m_sendResourceLoadCallbacks && m_frame)
-        frameLoader()->notifier()->didReceiveData(this, data, length, static_cast<int>(lengthReceived));
+    if (m_options.sendLoadCallbacks == SendCallbacks && m_frame)
+        frameLoader()->notifier()->didReceiveData(this, data, length, static_cast<int>(encodedDataLength));
 }
 
 void ResourceLoader::willStopBufferingData(const char* data, int length)
 {
-    if (!m_shouldBufferData)
+    if (m_options.shouldBufferData == DoNotBufferData)
         return;
 
     ASSERT(!m_resourceData);
@@ -314,7 +325,7 @@ void ResourceLoader::didFinishLoadingOnePart(double finishTime)
     if (m_calledDidFinishLoad)
         return;
     m_calledDidFinishLoad = true;
-    if (m_sendResourceLoadCallbacks)
+    if (m_options.sendLoadCallbacks == SendCallbacks)
         frameLoader()->notifier()->didFinishLoad(this, finishTime);
 }
 
@@ -331,36 +342,7 @@ void ResourceLoader::didFail(const ResourceError& error)
     if (FormData* data = m_request.httpBody())
         data->removeGeneratedFilesIfNeeded();
 
-    if (m_sendResourceLoadCallbacks && !m_calledDidFinishLoad)
-        frameLoader()->notifier()->didFailToLoad(this, error);
-
-    releaseResources();
-}
-
-void ResourceLoader::didCancel(const ResourceError& error)
-{
-    ASSERT(!m_cancelled);
-    ASSERT(!m_reachedTerminalState);
-
-    if (FormData* data = m_request.httpBody())
-        data->removeGeneratedFilesIfNeeded();
-
-    // This flag prevents bad behavior when loads that finish cause the
-    // load itself to be cancelled (which could happen with a javascript that 
-    // changes the window location). This is used to prevent both the body
-    // of this method and the body of connectionDidFinishLoading: running
-    // for a single delegate. Canceling wins.
-    m_cancelled = true;
-    
-    if (m_handle)
-        m_handle->clearAuthentication();
-
-    m_documentLoader->cancelPendingSubstituteLoad(this);
-    if (m_handle) {
-        m_handle->cancel();
-        m_handle = 0;
-    }
-    if (m_sendResourceLoadCallbacks && m_identifier && !m_calledDidFinishLoad)
+    if (m_options.sendLoadCallbacks == SendCallbacks && !m_calledDidFinishLoad)
         frameLoader()->notifier()->didFailToLoad(this, error);
 
     releaseResources();
@@ -373,17 +355,53 @@ void ResourceLoader::cancel()
 
 void ResourceLoader::cancel(const ResourceError& error)
 {
+    // If the load has already completed - succeeded, failed, or previously cancelled - do nothing.
     if (m_reachedTerminalState)
         return;
-    if (!error.isNull())
-        didCancel(error);
-    else
-        didCancel(cancelledError());
-}
+       
+    ResourceError nonNullError = error.isNull() ? cancelledError() : error;
+    
+    // willCancel() and didFailToLoad() both call out to clients that might do 
+    // something causing the last reference to this object to go away.
+    RefPtr<ResourceLoader> protector(this);
+    
+    // If we re-enter cancel() from inside willCancel(), we want to pick up from where we left 
+    // off without re-running willCancel()
+    if (!m_calledWillCancel) {
+        m_calledWillCancel = true;
+        
+        willCancel(nonNullError);
+    }
 
-const ResourceResponse& ResourceLoader::response() const
-{
-    return m_response;
+    // If we re-enter cancel() from inside didFailToLoad(), we want to pick up from where we 
+    // left off without redoing any of this work.
+    if (!m_cancelled) {
+        m_cancelled = true;
+        
+        if (FormData* data = m_request.httpBody())
+            data->removeGeneratedFilesIfNeeded();
+
+        if (m_handle)
+            m_handle->clearAuthentication();
+
+        m_documentLoader->cancelPendingSubstituteLoad(this);
+        if (m_handle) {
+            m_handle->cancel();
+            m_handle = 0;
+        }
+
+        if (m_options.sendLoadCallbacks == SendCallbacks && m_identifier && !m_calledDidFinishLoad)
+            frameLoader()->notifier()->didFailToLoad(this, nonNullError);
+    }
+
+    // If cancel() completed from within the call to willCancel() or didFailToLoad(),
+    // we don't want to redo didCancel() or releasesResources().
+    if (m_reachedTerminalState)
+        return;
+
+    didCancel(nonNullError);
+            
+    releaseResources();
 }
 
 ResourceError ResourceLoader::cancelledError()
@@ -393,12 +411,12 @@ ResourceError ResourceLoader::cancelledError()
 
 ResourceError ResourceLoader::blockedError()
 {
-    return frameLoader()->blockedError(m_request);
+    return frameLoader()->client()->blockedError(m_request);
 }
 
 ResourceError ResourceLoader::cannotShowURLError()
 {
-    return frameLoader()->cannotShowURLError(m_request);
+    return frameLoader()->client()->cannotShowURLError(m_request);
 }
 
 void ResourceLoader::willSendRequest(ResourceHandle*, ResourceRequest& request, const ResourceResponse& redirectResponse)
@@ -424,20 +442,32 @@ void ResourceLoader::didReceiveResponse(ResourceHandle*, const ResourceResponse&
     didReceiveResponse(response);
 }
 
-void ResourceLoader::didReceiveData(ResourceHandle*, const char* data, int length, int lengthReceived)
+void ResourceLoader::didReceiveData(ResourceHandle*, const char* data, int length, int encodedDataLength)
 {
     InspectorInstrumentationCookie cookie = InspectorInstrumentation::willReceiveResourceData(m_frame.get(), identifier());
-    didReceiveData(data, length, lengthReceived, false);
+    didReceiveData(data, length, encodedDataLength, false);
     InspectorInstrumentation::didReceiveResourceData(cookie);
 }
 
 void ResourceLoader::didFinishLoading(ResourceHandle*, double finishTime)
 {
+#if ENABLE(OFFLINE_WEB_APPLICATIONS)
+    if (!fastMallocSize(documentLoader()->applicationCacheHost()))
+        CRASH();
+#endif
+    if (!fastMallocSize(documentLoader()->frame()))
+        CRASH();
     didFinishLoading(finishTime);
 }
 
 void ResourceLoader::didFail(ResourceHandle*, const ResourceError& error)
 {
+#if ENABLE(OFFLINE_WEB_APPLICATIONS)
+    if (!fastMallocSize(documentLoader()->applicationCacheHost()))
+        CRASH();
+#endif
+    if (!fastMallocSize(documentLoader()->frame()))
+        CRASH();
 #if ENABLE(OFFLINE_WEB_APPLICATIONS)
     if (documentLoader()->applicationCacheHost()->maybeLoadFallbackForError(this, error))
         return;
@@ -447,18 +477,40 @@ void ResourceLoader::didFail(ResourceHandle*, const ResourceError& error)
 
 void ResourceLoader::wasBlocked(ResourceHandle*)
 {
+#if ENABLE(OFFLINE_WEB_APPLICATIONS)
+    if (!fastMallocSize(documentLoader()->applicationCacheHost()))
+        CRASH();
+#endif
+    if (!fastMallocSize(documentLoader()->frame()))
+        CRASH();
     didFail(blockedError());
 }
 
 void ResourceLoader::cannotShowURL(ResourceHandle*)
 {
+#if ENABLE(OFFLINE_WEB_APPLICATIONS)
+    if (!fastMallocSize(documentLoader()->applicationCacheHost()))
+        CRASH();
+#endif
+    if (!fastMallocSize(documentLoader()->frame()))
+        CRASH();
     didFail(cannotShowURLError());
 }
 
 bool ResourceLoader::shouldUseCredentialStorage()
 {
+#if ENABLE(OFFLINE_WEB_APPLICATIONS)
+    if (!fastMallocSize(documentLoader()->applicationCacheHost()))
+        CRASH();
+#endif
+    if (!fastMallocSize(documentLoader()->frame()))
+        CRASH();
+
+    if (m_options.allowCredentials == DoNotAllowStoredCredentials)
+        return false;
+    
     RefPtr<ResourceLoader> protector(this);
-    return frameLoader()->shouldUseCredentialStorage(this);
+    return frameLoader()->client()->shouldUseCredentialStorage(documentLoader(), identifier());
 }
 
 void ResourceLoader::didReceiveAuthenticationChallenge(const AuthenticationChallenge& challenge)
@@ -481,7 +533,7 @@ void ResourceLoader::didCancelAuthenticationChallenge(const AuthenticationChalle
 bool ResourceLoader::canAuthenticateAgainstProtectionSpace(const ProtectionSpace& protectionSpace)
 {
     RefPtr<ResourceLoader> protector(this);
-    return frameLoader()->canAuthenticateAgainstProtectionSpace(this, protectionSpace);
+    return frameLoader()->client()->canAuthenticateAgainstProtectionSpace(documentLoader(), identifier(), protectionSpace);
 }
 #endif
 
@@ -492,6 +544,12 @@ void ResourceLoader::receivedCancellation(const AuthenticationChallenge&)
 
 void ResourceLoader::willCacheResponse(ResourceHandle*, CacheStoragePolicy& policy)
 {
+#if ENABLE(OFFLINE_WEB_APPLICATIONS)
+    if (!fastMallocSize(documentLoader()->applicationCacheHost()))
+        CRASH();
+#endif
+    if (!fastMallocSize(documentLoader()->frame()))
+        CRASH();
     // <rdar://problem/7249553> - There are reports of crashes with this method being called
     // with a null m_frame->settings(), which can only happen if the frame doesn't have a page.
     // Sadly we have no reproducible cases of this.
