@@ -38,14 +38,18 @@
 #include "SQLiteStatement.h"
 #include "SQLiteTransaction.h"
 #include "SecurityOrigin.h"
+#include "UUID.h"
 #include <wtf/text/CString.h>
 #include <wtf/StdLibExtras.h>
 #include <wtf/StringExtras.h>
+#include <wtf/text/StringBuilder.h>
 
 using namespace std;
 
 namespace WebCore {
 
+static const char flatFileSubdirectory[] = "ApplicationCache";
+
 template <class T>
 class StorageIDJournal {
 public:  
@@ -90,7 +94,7 @@ static unsigned urlHostHash(const KURL& url)
     unsigned hostStart = url.hostStart();
     unsigned hostEnd = url.hostEnd();
     
-    return AlreadyHashed::avoidDeletedValue(WTF::StringHasher::createHash(url.string().characters() + hostStart, hostEnd - hostStart));
+    return AlreadyHashed::avoidDeletedValue(StringHasher::computeHash(url.string().characters() + hostStart, hostEnd - hostStart));
 }
 
 ApplicationCacheGroup* ApplicationCacheStorage::loadCacheGroup(const KURL& manifestURL)
@@ -153,6 +157,11 @@ ApplicationCacheGroup* ApplicationCacheStorage::findOrCreateCacheGroup(const KUR
     return group;
 }
 
+ApplicationCacheGroup* ApplicationCacheStorage::findInMemoryCacheGroup(const KURL& manifestURL) const
+{
+    return m_cachesInMemory.get(manifestURL);
+}
+
 void ApplicationCacheStorage::loadManifestHostHashes()
 {
     static bool hasLoadedHashes = false;
@@ -389,7 +398,7 @@ int64_t ApplicationCacheStorage::spaceNeeded(int64_t cacheToSave)
     if (!getFileSize(m_cacheFile, fileSize))
         return 0;
 
-    int64_t currentSize = fileSize;
+    int64_t currentSize = fileSize + flatFileAreaSize();
 
     // Determine the amount of free space we have available.
     int64_t totalAvailableSize = 0;
@@ -425,7 +434,7 @@ void ApplicationCacheStorage::setDefaultOriginQuota(int64_t quota)
     m_defaultOriginQuota = quota;
 }
 
-bool ApplicationCacheStorage::quotaForOrigin(const SecurityOrigin* origin, int64_t& quota)
+bool ApplicationCacheStorage::calculateQuotaForOrigin(const SecurityOrigin* origin, int64_t& quota)
 {
     // If an Origin record doesn't exist, then the COUNT will be 0 and quota will be 0.
     // Using the count to determine if a record existed or not is a safe way to determine
@@ -448,7 +457,7 @@ bool ApplicationCacheStorage::quotaForOrigin(const SecurityOrigin* origin, int64
     return false;
 }
 
-bool ApplicationCacheStorage::usageForOrigin(const SecurityOrigin* origin, int64_t& usage)
+bool ApplicationCacheStorage::calculateUsageForOrigin(const SecurityOrigin* origin, int64_t& usage)
 {
     // If an Origins record doesn't exist, then the SUM will be null,
     // which will become 0, as expected, when converting to a number.
@@ -472,7 +481,7 @@ bool ApplicationCacheStorage::usageForOrigin(const SecurityOrigin* origin, int64
     return false;
 }
 
-bool ApplicationCacheStorage::remainingSizeForOriginExcludingCache(const SecurityOrigin* origin, ApplicationCache* cache, int64_t& remainingSize)
+bool ApplicationCacheStorage::calculateRemainingSizeForOriginExcludingCache(const SecurityOrigin* origin, ApplicationCache* cache, int64_t& remainingSize)
 {
     openDatabase(false);
     if (!m_database.isOpen())
@@ -511,7 +520,7 @@ bool ApplicationCacheStorage::remainingSizeForOriginExcludingCache(const Securit
     if (result == SQLResultRow) {
         int64_t numberOfCaches = statement.getColumnInt64(0);
         if (numberOfCaches == 0)
-            quotaForOrigin(origin, remainingSize);
+            calculateQuotaForOrigin(origin, remainingSize);
         else
             remainingSize = statement.getColumnInt64(1);
         return true;
@@ -555,7 +564,7 @@ bool ApplicationCacheStorage::executeSQLCommand(const String& sql)
 // Update the schemaVersion when the schema of any the Application Cache
 // SQLite tables changes. This allows the database to be rebuilt when
 // a new, incompatible change has been introduced to the database schema.
-static const int schemaVersion = 6;
+static const int schemaVersion = 7;
     
 void ApplicationCacheStorage::verifySchemaVersion()
 {
@@ -563,7 +572,7 @@ void ApplicationCacheStorage::verifySchemaVersion()
     if (version == schemaVersion)
         return;
 
-    m_database.clearAllTables();
+    deleteTables();
 
     // Update user version.
     SQLiteTransaction setDatabaseVersion(m_database);
@@ -613,7 +622,8 @@ void ApplicationCacheStorage::openDatabase(bool createIfDoesNotExist)
     executeSQLCommand("CREATE TABLE IF NOT EXISTS CacheEntries (cache INTEGER NOT NULL ON CONFLICT FAIL, type INTEGER, resource INTEGER NOT NULL)");
     executeSQLCommand("CREATE TABLE IF NOT EXISTS CacheResources (id INTEGER PRIMARY KEY AUTOINCREMENT, url TEXT NOT NULL ON CONFLICT FAIL, "
                       "statusCode INTEGER NOT NULL, responseURL TEXT NOT NULL, mimeType TEXT, textEncodingName TEXT, headers TEXT, data INTEGER NOT NULL ON CONFLICT FAIL)");
-    executeSQLCommand("CREATE TABLE IF NOT EXISTS CacheResourceData (id INTEGER PRIMARY KEY AUTOINCREMENT, data BLOB)");
+    executeSQLCommand("CREATE TABLE IF NOT EXISTS CacheResourceData (id INTEGER PRIMARY KEY AUTOINCREMENT, data BLOB, path TEXT)");
+    executeSQLCommand("CREATE TABLE IF NOT EXISTS DeletedCacheResources (id INTEGER PRIMARY KEY AUTOINCREMENT, path TEXT)");
     executeSQLCommand("CREATE TABLE IF NOT EXISTS Origins (origin TEXT UNIQUE ON CONFLICT IGNORE, quota INTEGER NOT NULL ON CONFLICT FAIL)");
 
     // When a cache is deleted, all its entries and its whitelist should be deleted.
@@ -636,6 +646,15 @@ void ApplicationCacheStorage::openDatabase(bool createIfDoesNotExist)
                       " FOR EACH ROW BEGIN"
                       "  DELETE FROM CacheResourceData WHERE id = OLD.data;"
                       " END");
+    
+    // When a cache resource is deleted, if it contains a non-empty path, that path should
+    // be added to the DeletedCacheResources table so the flat file at that path can
+    // be deleted at a later time.
+    executeSQLCommand("CREATE TRIGGER IF NOT EXISTS CacheResourceDataDeleted AFTER DELETE ON CacheResourceData"
+                      " FOR EACH ROW"
+                      " WHEN OLD.path NOT NULL BEGIN"
+                      "  INSERT INTO DeletedCacheResources (path) values (OLD.path);"
+                      " END");
 }
 
 bool ApplicationCacheStorage::executeStatement(SQLiteStatement& statement)
@@ -767,32 +786,68 @@ bool ApplicationCacheStorage::store(ApplicationCacheResource* resource, unsigned
         return false;
 
     // First, insert the data
-    SQLiteStatement dataStatement(m_database, "INSERT INTO CacheResourceData (data) VALUES (?)");
+    SQLiteStatement dataStatement(m_database, "INSERT INTO CacheResourceData (data, path) VALUES (?, ?)");
     if (dataStatement.prepare() != SQLResultOk)
         return false;
     
-    if (resource->data()->size())
-        dataStatement.bindBlob(1, resource->data()->data(), resource->data()->size());
+
+    String fullPath;
+    if (!resource->path().isEmpty())
+        dataStatement.bindText(2, pathGetFileName(resource->path()));
+    else if (shouldStoreResourceAsFlatFile(resource)) {
+        // First, check to see if creating the flat file would violate the maximum total quota. We don't need
+        // to check the per-origin quota here, as it was already checked in storeNewestCache().
+        if (m_database.totalSize() + flatFileAreaSize() + resource->data()->size() > m_maximumSize) {
+            m_isMaximumSizeReached = true;
+            return false;
+        }
+        
+        String flatFileDirectory = pathByAppendingComponent(m_cacheDirectory, flatFileSubdirectory);
+        makeAllDirectories(flatFileDirectory);
+
+        String extension;
+        
+        String fileName = resource->response().suggestedFilename();
+        size_t dotIndex = fileName.reverseFind('.');
+        if (dotIndex != notFound && dotIndex < (fileName.length() - 1))
+            extension = fileName.substring(dotIndex);
+
+        String path;
+        if (!writeDataToUniqueFileInDirectory(resource->data(), flatFileDirectory, path, extension))
+            return false;
+        
+        fullPath = pathByAppendingComponent(flatFileDirectory, path);
+        resource->setPath(fullPath);
+        dataStatement.bindText(2, path);
+    } else {
+        if (resource->data()->size())
+            dataStatement.bindBlob(1, resource->data()->data(), resource->data()->size());
+    }
     
-    if (!dataStatement.executeCommand())
+    if (!dataStatement.executeCommand()) {
+        // Clean up the file which we may have written to:
+        if (!fullPath.isEmpty())
+            deleteFile(fullPath);
+
         return false;
+    }
 
     unsigned dataId = static_cast<unsigned>(m_database.lastInsertRowID());
 
     // Then, insert the resource
     
     // Serialize the headers
-    Vector<UChar> stringBuilder;
+    StringBuilder stringBuilder;
     
     HTTPHeaderMap::const_iterator end = resource->response().httpHeaderFields().end();
     for (HTTPHeaderMap::const_iterator it = resource->response().httpHeaderFields().begin(); it!= end; ++it) {
-        stringBuilder.append(it->first.characters(), it->first.length());
+        stringBuilder.append(it->first);
         stringBuilder.append((UChar)':');
-        stringBuilder.append(it->second.characters(), it->second.length());
+        stringBuilder.append(it->second);
         stringBuilder.append((UChar)'\n');
     }
     
-    String headers = String::adopt(stringBuilder);
+    String headers = stringBuilder.toString();
     
     SQLiteStatement resourceStatement(m_database, "INSERT INTO CacheResources (url, statusCode, responseURL, headers, data, mimeType, textEncodingName) VALUES (?, ?, ?, ?, ?, ?, ?)");
     if (resourceStatement.prepare() != SQLResultOk)
@@ -826,6 +881,12 @@ bool ApplicationCacheStorage::store(ApplicationCacheResource* resource, unsigned
     if (!executeStatement(entryStatement))
         return false;
     
+    // Did we successfully write the resource data to a file? If so,
+    // release the resource's data and free up a potentially large amount
+    // of memory:
+    if (!fullPath.isEmpty())
+        resource->data()->clear();
+
     resource->setStorageID(resourceId);
     return true;
 }
@@ -856,7 +917,7 @@ bool ApplicationCacheStorage::store(ApplicationCacheResource* resource, Applicat
         return false;
  
     m_isMaximumSizeReached = false;
-    m_database.setMaximumSize(m_maximumSize);
+    m_database.setMaximumSize(m_maximumSize - flatFileAreaSize());
 
     SQLiteTransaction storeResourceTransaction(m_database);
     storeResourceTransaction.begin();
@@ -895,6 +956,28 @@ bool ApplicationCacheStorage::ensureOriginRecord(const SecurityOrigin* origin)
     return true;
 }
 
+bool ApplicationCacheStorage::checkOriginQuota(ApplicationCacheGroup* group, ApplicationCache* oldCache, ApplicationCache* newCache, int64_t& totalSpaceNeeded)
+{
+    // Check if the oldCache with the newCache would reach the per-origin quota.
+    int64_t remainingSpaceInOrigin;
+    const SecurityOrigin* origin = group->origin();
+    if (calculateRemainingSizeForOriginExcludingCache(origin, oldCache, remainingSpaceInOrigin)) {
+        if (remainingSpaceInOrigin < newCache->estimatedSizeInStorage()) {
+            int64_t quota;
+            if (calculateQuotaForOrigin(origin, quota)) {
+                totalSpaceNeeded = quota - remainingSpaceInOrigin + newCache->estimatedSizeInStorage();
+                return false;
+            }
+
+            ASSERT_NOT_REACHED();
+            totalSpaceNeeded = 0;
+            return false;
+        }
+    }
+
+    return true;
+}
+
 bool ApplicationCacheStorage::storeNewestCache(ApplicationCacheGroup* group, ApplicationCache* oldCache, FailureReason& failureReason)
 {
     openDatabase(true);
@@ -903,19 +986,17 @@ bool ApplicationCacheStorage::storeNewestCache(ApplicationCacheGroup* group, App
         return false;
 
     m_isMaximumSizeReached = false;
-    m_database.setMaximumSize(m_maximumSize);
+    m_database.setMaximumSize(m_maximumSize - flatFileAreaSize());
 
     SQLiteTransaction storeCacheTransaction(m_database);
     
     storeCacheTransaction.begin();
 
     // Check if this would reach the per-origin quota.
-    int64_t remainingSpaceInOrigin;
-    if (remainingSizeForOriginExcludingCache(group->origin(), oldCache, remainingSpaceInOrigin)) {
-        if (remainingSpaceInOrigin < group->newestCache()->estimatedSizeInStorage()) {
-            failureReason = OriginQuotaReached;
-            return false;
-        }
+    int64_t totalSpaceNeededIgnored;
+    if (!checkOriginQuota(group, oldCache, group->newestCache(), totalSpaceNeededIgnored)) {
+        failureReason = OriginQuotaReached;
+        return false;
     }
 
     GroupStorageIDJournal groupStorageIDJournal;
@@ -1003,7 +1084,7 @@ static inline void parseHeaders(const String& headers, ResourceResponse& respons
 PassRefPtr<ApplicationCache> ApplicationCacheStorage::loadCache(unsigned storageID)
 {
     SQLiteStatement cacheStatement(m_database, 
-                                   "SELECT url, type, mimeType, textEncodingName, headers, CacheResourceData.data FROM CacheEntries INNER JOIN CacheResources ON CacheEntries.resource=CacheResources.id "
+                                   "SELECT url, statusCode, type, mimeType, textEncodingName, headers, CacheResourceData.data, CacheResourceData.path FROM CacheEntries INNER JOIN CacheResources ON CacheEntries.resource=CacheResources.id "
                                    "INNER JOIN CacheResourceData ON CacheResourceData.id=CacheResources.data WHERE CacheEntries.cache=?");
     if (cacheStatement.prepare() != SQLResultOk) {
         LOG_ERROR("Could not prepare cache statement, error \"%s\"", m_database.lastErrorMsg());
@@ -1013,27 +1094,41 @@ PassRefPtr<ApplicationCache> ApplicationCacheStorage::loadCache(unsigned storage
     cacheStatement.bindInt64(1, storageID);
 
     RefPtr<ApplicationCache> cache = ApplicationCache::create();
-    
+
+    String flatFileDirectory = pathByAppendingComponent(m_cacheDirectory, flatFileSubdirectory);
+
     int result;
     while ((result = cacheStatement.step()) == SQLResultRow) {
         KURL url(ParsedURLString, cacheStatement.getColumnText(0));
         
-        unsigned type = static_cast<unsigned>(cacheStatement.getColumnInt64(1));
+        int httpStatusCode = cacheStatement.getColumnInt(1);
+
+        unsigned type = static_cast<unsigned>(cacheStatement.getColumnInt64(2));
 
         Vector<char> blob;
-        cacheStatement.getColumnBlobAsVector(5, blob);
+        cacheStatement.getColumnBlobAsVector(6, blob);
         
         RefPtr<SharedBuffer> data = SharedBuffer::adoptVector(blob);
         
-        String mimeType = cacheStatement.getColumnText(2);
-        String textEncodingName = cacheStatement.getColumnText(3);
+        String path = cacheStatement.getColumnText(7);
+        long long size = 0;
+        if (path.isEmpty())
+            size = data->size();
+        else {
+            path = pathByAppendingComponent(flatFileDirectory, path);
+            getFileSize(path, size);
+        }
         
-        ResourceResponse response(url, mimeType, data->size(), textEncodingName, "");
+        String mimeType = cacheStatement.getColumnText(3);
+        String textEncodingName = cacheStatement.getColumnText(4);
+        
+        ResourceResponse response(url, mimeType, size, textEncodingName, "");
+        response.setHTTPStatusCode(httpStatusCode);
 
-        String headers = cacheStatement.getColumnText(4);
+        String headers = cacheStatement.getColumnText(5);
         parseHeaders(headers, response);
         
-        RefPtr<ApplicationCacheResource> resource = ApplicationCacheResource::create(url, response, type, data.release());
+        RefPtr<ApplicationCacheResource> resource = ApplicationCacheResource::create(url, response, type, data.release(), path);
 
         if (type & ApplicationCacheResource::Manifest)
             cache->setManifestResource(resource.release());
@@ -1127,6 +1222,8 @@ void ApplicationCacheStorage::remove(ApplicationCache* cache)
 
         cache->group()->clearStorageID();
     }
+    
+    checkForDeletedResources();
 }    
 
 void ApplicationCacheStorage::empty()
@@ -1147,7 +1244,51 @@ void ApplicationCacheStorage::empty()
     CacheGroupMap::const_iterator end = m_cachesInMemory.end();
     for (CacheGroupMap::const_iterator it = m_cachesInMemory.begin(); it != end; ++it)
         it->second->clearStorageID();
-}    
+    
+    checkForDeletedResources();
+}
+    
+void ApplicationCacheStorage::deleteTables()
+{
+    empty();
+    m_database.clearAllTables();
+}
+    
+bool ApplicationCacheStorage::shouldStoreResourceAsFlatFile(ApplicationCacheResource* resource)
+{
+    return resource->response().mimeType().startsWith("audio/", false) 
+        || resource->response().mimeType().startsWith("video/", false);
+}
+    
+bool ApplicationCacheStorage::writeDataToUniqueFileInDirectory(SharedBuffer* data, const String& directory, String& path, const String& fileExtension)
+{
+    String fullPath;
+    
+    do {
+        path = encodeForFileName(createCanonicalUUIDString()) + fileExtension;
+        // Guard against the above function being called on a platform which does not implement
+        // createCanonicalUUIDString().
+        ASSERT(!path.isEmpty());
+        if (path.isEmpty())
+            return false;
+        
+        fullPath = pathByAppendingComponent(directory, path);
+    } while (directoryName(fullPath) != directory || fileExists(fullPath));
+    
+    PlatformFileHandle handle = openFile(fullPath, OpenForWrite);
+    if (!handle)
+        return false;
+    
+    int64_t writtenBytes = writeToFile(handle, data->data(), data->size());
+    closeFile(handle);
+    
+    if (writtenBytes != static_cast<int64_t>(data->size())) {
+        deleteFile(fullPath);
+        return false;
+    }
+    
+    return true;
+}
 
 bool ApplicationCacheStorage::storeCopyOfCache(const String& cacheDirectory, ApplicationCacheHost* cacheHost)
 {
@@ -1166,7 +1307,7 @@ bool ApplicationCacheStorage::storeCopyOfCache(const String& cacheDirectory, App
     for (ApplicationCache::ResourceMap::const_iterator it = cache->begin(); it != end; ++it) {
         ApplicationCacheResource* resource = it->second.get();
         
-        RefPtr<ApplicationCacheResource> resourceCopy = ApplicationCacheResource::create(resource->url(), resource->response(), resource->type(), resource->data());
+        RefPtr<ApplicationCacheResource> resourceCopy = ApplicationCacheResource::create(resource->url(), resource->response(), resource->type(), resource->data(), resource->path());
         
         cacheCopy->addResource(resourceCopy.release());
     }
@@ -1274,6 +1415,9 @@ bool ApplicationCacheStorage::deleteCacheGroup(const String& manifestURL)
     }
 
     deleteTransaction.commit();
+    
+    checkForDeletedResources();
+    
     return true;
 }
 
@@ -1291,6 +1435,71 @@ void ApplicationCacheStorage::checkForMaxSizeReached()
     if (m_database.lastError() == SQLResultFull)
         m_isMaximumSizeReached = true;
 }
+    
+void ApplicationCacheStorage::checkForDeletedResources()
+{
+    openDatabase(false);
+    if (!m_database.isOpen())
+        return;
+
+    // Select only the paths in DeletedCacheResources that do not also appear in CacheResourceData:
+    SQLiteStatement selectPaths(m_database, "SELECT DeletedCacheResources.path "
+        "FROM DeletedCacheResources "
+        "LEFT JOIN CacheResourceData "
+        "ON DeletedCacheResources.path = CacheResourceData.path "
+        "WHERE (SELECT DeletedCacheResources.path == CacheResourceData.path) IS NULL");
+    
+    if (selectPaths.prepare() != SQLResultOk)
+        return;
+    
+    if (selectPaths.step() != SQLResultRow)
+        return;
+    
+    do {
+        String path = selectPaths.getColumnText(0);
+        if (path.isEmpty())
+            continue;
+        
+        String flatFileDirectory = pathByAppendingComponent(m_cacheDirectory, flatFileSubdirectory);
+        String fullPath = pathByAppendingComponent(flatFileDirectory, path);
+        
+        // Don't exit the flatFileDirectory! This should only happen if the "path" entry contains a directory 
+        // component, but protect against it regardless.
+        if (directoryName(fullPath) != flatFileDirectory)
+            continue;
+        
+        deleteFile(fullPath);
+    } while (selectPaths.step() == SQLResultRow);
+    
+    executeSQLCommand("DELETE FROM DeletedCacheResources");
+}
+    
+long long ApplicationCacheStorage::flatFileAreaSize()
+{
+    openDatabase(false);
+    if (!m_database.isOpen())
+        return 0;
+    
+    SQLiteStatement selectPaths(m_database, "SELECT path FROM CacheResourceData WHERE path NOT NULL");
+
+    if (selectPaths.prepare() != SQLResultOk) {
+        LOG_ERROR("Could not load flat file cache resource data, error \"%s\"", m_database.lastErrorMsg());
+        return 0;
+    }
+
+    long long totalSize = 0;
+    String flatFileDirectory = pathByAppendingComponent(m_cacheDirectory, flatFileSubdirectory);
+    while (selectPaths.step() == SQLResultRow) {
+        String path = selectPaths.getColumnText(0);
+        String fullPath = pathByAppendingComponent(flatFileDirectory, path);
+        long long pathSize = 0;
+        if (!getFileSize(fullPath, pathSize))
+            continue;
+        totalSize += pathSize;
+    }
+    
+    return totalSize;
+}
 
 void ApplicationCacheStorage::getOriginsWithCache(HashSet<RefPtr<SecurityOrigin>, SecurityOriginHash>& origins)
 {
@@ -1309,24 +1518,6 @@ void ApplicationCacheStorage::getOriginsWithCache(HashSet<RefPtr<SecurityOrigin>
     }
 }
 
-void ApplicationCacheStorage::deleteEntriesForOrigin(SecurityOrigin* origin)
-{
-    Vector<KURL> urls;
-    if (!manifestURLs(&urls)) {
-        LOG_ERROR("Failed to retrieve ApplicationCache manifest URLs");
-        return;
-    }
-
-    // Multiple manifest URLs might share the same SecurityOrigin, so we might be creating extra, wasted origins here.
-    // The current schema doesn't allow for a more efficient way of deleting by origin.
-    size_t count = urls.size();
-    for (size_t i = 0; i < count; ++i) {
-        RefPtr<SecurityOrigin> manifestOrigin = SecurityOrigin::create(urls[i]);
-        if (manifestOrigin->isSameSchemeHostPort(origin))
-            deleteCacheGroup(urls[i]);
-    }
-}
-
 void ApplicationCacheStorage::deleteAllEntries()
 {
     empty();
