@@ -31,6 +31,7 @@
 #include "CachedResourceHandle.h"
 #include "CachedResourceLoader.h"
 #include "CachedResourceRequest.h"
+#include "CrossOriginAccessControl.h"
 #include "Frame.h"
 #include "FrameLoaderClient.h"
 #include "KURL.h"
@@ -64,6 +65,10 @@ static ResourceLoadPriority defaultPriorityForResourceType(CachedResource::Type
 #if ENABLE(LINK_PREFETCH)
         case CachedResource::LinkPrefetch:
             return ResourceLoadPriorityVeryLow;
+        case CachedResource::LinkPrerender:
+            return ResourceLoadPriorityVeryLow;
+        case CachedResource::LinkSubresource:
+            return ResourceLoadPriorityVeryLow;
 #endif
     }
     ASSERT_NOT_REACHED();
@@ -74,9 +79,8 @@ static ResourceLoadPriority defaultPriorityForResourceType(CachedResource::Type
 static RefCountedLeakCounter cachedResourceLeakCounter("CachedResource");
 #endif
 
-CachedResource::CachedResource(const String& url, Type type)
-    : m_url(url)
-    , m_request(0)
+CachedResource::CachedResource(const ResourceRequest& request, Type type)
+    : m_resourceRequest(request)
     , m_loadPriority(defaultPriorityForResourceType(type))
     , m_responseTimestamp(currentTime())
     , m_lastDecodedAccessTime(0)
@@ -88,11 +92,11 @@ CachedResource::CachedResource(const String& url, Type type)
     , m_preloadResult(PreloadNotReferenced)
     , m_inLiveDecodedResourcesList(false)
     , m_requestedFromNetworkingLayer(false)
-    , m_sendResourceLoadCallbacks(true)
     , m_inCache(false)
     , m_loading(false)
     , m_type(type)
     , m_status(Pending)
+    , m_options(SendCallbacks, SniffContent, BufferData, AllowStoredCredentials)
 #ifndef NDEBUG
     , m_deleted(false)
     , m_lruIndex(0)
@@ -127,11 +131,24 @@ CachedResource::~CachedResource()
         m_owningCachedResourceLoader->removeCachedResource(this);
 }
 
-void CachedResource::load(CachedResourceLoader* cachedResourceLoader, bool incremental, SecurityCheckPolicy securityCheck, bool sendResourceLoadCallbacks)
+void CachedResource::load(CachedResourceLoader* cachedResourceLoader, bool incremental, SecurityCheckPolicy securityCheck)
 {
-    m_sendResourceLoadCallbacks = sendResourceLoadCallbacks;
-    cachedResourceLoader->load(this, incremental, securityCheck, sendResourceLoadCallbacks);
     m_loading = true;
+    m_request = CachedResourceRequest::load(cachedResourceLoader, this, incremental, securityCheck, m_options);
+    if (m_request) {
+        m_status = Pending;
+        cachedResourceLoader->incrementRequestCount(this);
+    }
+}
+
+void CachedResource::checkNotify()
+{
+    if (isLoading())
+        return;
+
+    CachedResourceClientWalker w(m_clients);
+    while (CachedResourceClient* c = w.next())
+        c->notifyFinished(this);
 }
 
 void CachedResource::data(PassRefPtr<SharedBuffer>, bool allDataReceived)
@@ -140,9 +157,17 @@ void CachedResource::data(PassRefPtr<SharedBuffer>, bool allDataReceived)
         return;
     
     setLoading(false);
-    CachedResourceClientWalker w(m_clients);
-    while (CachedResourceClient* c = w.next())
-        c->notifyFinished(this);
+    checkNotify();
+}
+
+void CachedResource::error(CachedResource::Status status)
+{
+    setStatus(status);
+    ASSERT(errorOccurred());
+    m_data.clear();
+
+    setLoading(false);
+    checkNotify();
 }
 
 void CachedResource::finish()
@@ -150,6 +175,12 @@ void CachedResource::finish()
     m_status = Cached;
 }
 
+bool CachedResource::passesAccessControlCheck(SecurityOrigin* securityOrigin)
+{
+    String errorDescription;
+    return WebCore::passesAccessControlCheck(m_response, resourceRequest().allowCookies() ? AllowStoredCredentials : DoNotAllowStoredCredentials, securityOrigin, errorDescription);
+}
+
 bool CachedResource::isExpired() const
 {
     if (m_response.isNull())
@@ -226,13 +257,21 @@ CachedMetadata* CachedResource::cachedMetadata(unsigned dataTypeID) const
     return m_cachedMetadata.get();
 }
 
-void CachedResource::setRequest(CachedResourceRequest* request)
+void CachedResource::stopLoading()
 {
-    if (request && !m_request)
-        m_status = Pending;
-    m_request = request;
-    if (canDelete() && !inCache())
-        delete this;
+    ASSERT(m_request);            
+    m_request.clear();
+
+    CachedResourceHandle<CachedResource> protect(this);
+
+    // All loads finish with data(allDataReceived = true) or error(), except for
+    // canceled loads, which silently set our request to 0. Be sure to notify our
+    // client in that case, so we don't seem to continue loading forever.
+    if (isLoading()) {
+        setLoading(false);
+        setStatus(Canceled);
+        checkNotify();
+    }
 }
 
 void CachedResource::addClient(CachedResourceClient* client)
@@ -486,10 +525,7 @@ bool CachedResource::canUseCacheValidator() const
 
     if (m_response.cacheControlContainsNoStore())
         return false;
-
-    DEFINE_STATIC_LOCAL(const AtomicString, lastModifiedHeader, ("last-modified"));
-    DEFINE_STATIC_LOCAL(const AtomicString, eTagHeader, ("etag"));
-    return !m_response.httpHeaderField(lastModifiedHeader).isEmpty() || !m_response.httpHeaderField(eTagHeader).isEmpty();
+    return m_response.hasCacheValidatorFields();
 }
 
 bool CachedResource::mustRevalidateDueToCacheHeaders(CachePolicy cachePolicy) const
@@ -580,11 +616,8 @@ bool CachedResource::wasPurged() const
 
 unsigned CachedResource::overheadSize() const
 {
-    return sizeof(CachedResource) + m_response.memoryUsage() + 576;
-    /*
-        576 = 192 +                   // average size of m_url
-              384;                    // average size of m_clients hash map
-    */
+    static const int kAverageClientsHashMapSize = 384;
+    return sizeof(CachedResource) + m_response.memoryUsage() + kAverageClientsHashMapSize + m_resourceRequest.url().string().length() * 2;
 }
     
 void CachedResource::setLoadPriority(ResourceLoadPriority loadPriority) 
