@@ -34,6 +34,7 @@
 #include "CachedScript.h"
 #include "CachedXSLStyleSheet.h"
 #include "Console.h"
+#include "ContentSecurityPolicy.h"
 #include "DOMWindow.h"
 #include "Document.h"
 #include "Frame.h"
@@ -46,31 +47,36 @@
 #include "ResourceLoadScheduler.h"
 #include "SecurityOrigin.h"
 #include "Settings.h"
+#include <wtf/UnusedParam.h>
 #include <wtf/text/CString.h>
-#include <wtf/text/StringConcatenate.h>
+#include <wtf/text/WTFString.h>
 
 #define PRELOAD_DEBUG 0
 
 namespace WebCore {
 
-static CachedResource* createResource(CachedResource::Type type, const KURL& url, const String& charset)
+static CachedResource* createResource(CachedResource::Type type, ResourceRequest& request, const String& charset)
 {
     switch (type) {
     case CachedResource::ImageResource:
-        return new CachedImage(url.string());
+        return new CachedImage(request);
     case CachedResource::CSSStyleSheet:
-        return new CachedCSSStyleSheet(url.string(), charset);
+        return new CachedCSSStyleSheet(request, charset);
     case CachedResource::Script:
-        return new CachedScript(url.string(), charset);
+        return new CachedScript(request, charset);
     case CachedResource::FontResource:
-        return new CachedFont(url.string());
+        return new CachedFont(request);
 #if ENABLE(XSLT)
     case CachedResource::XSLStyleSheet:
-        return new CachedXSLStyleSheet(url.string());
+        return new CachedXSLStyleSheet(request);
 #endif
 #if ENABLE(LINK_PREFETCH)
     case CachedResource::LinkPrefetch:
-        return new CachedResource(url.string(), CachedResource::LinkPrefetch);
+        return new CachedResource(request, CachedResource::LinkPrefetch);
+    case CachedResource::LinkPrerender:
+        return new CachedResource(request, CachedResource::LinkPrerender);
+    case CachedResource::LinkSubresource:
+        return new CachedResource(request, CachedResource::LinkSubresource);
 #endif
     }
     ASSERT_NOT_REACHED();
@@ -80,7 +86,7 @@ static CachedResource* createResource(CachedResource::Type type, const KURL& url
 CachedResourceLoader::CachedResourceLoader(Document* document)
     : m_document(document)
     , m_requestCount(0)
-    , m_loadDoneActionTimer(this, &CachedResourceLoader::loadDoneActionTimerFired)
+    , m_garbageCollectDocumentResourcesTimer(this, &CachedResourceLoader::garbageCollectDocumentResourcesTimerFired)
     , m_autoLoadImages(true)
     , m_loadFinishing(false)
     , m_allowStaleResources(false)
@@ -91,7 +97,6 @@ CachedResourceLoader::~CachedResourceLoader()
 {
     m_document = 0;
 
-    cancelRequests();
     clearPreloads();
     DocumentResourceMap::iterator end = m_documentResources.end();
     for (DocumentResourceMap::iterator it = m_documentResources.begin(); it != end; ++it)
@@ -118,54 +123,52 @@ Frame* CachedResourceLoader::frame() const
     return m_document ? m_document->frame() : 0;
 }
 
-CachedImage* CachedResourceLoader::requestImage(const String& url)
+CachedImage* CachedResourceLoader::requestImage(ResourceRequest& request)
 {
     if (Frame* f = frame()) {
-        Settings* settings = f->settings();
-        if (!f->loader()->client()->allowImages(!settings || settings->areImagesEnabled()))
-            return 0;
-
-        if (f->loader()->pageDismissalEventBeingDispatched()) {
-            KURL completeURL = m_document->completeURL(url);
-            if (completeURL.isValid() && canRequest(CachedResource::ImageResource, completeURL))
-                PingLoader::loadImage(f, completeURL);
+        if (f->loader()->pageDismissalEventBeingDispatched() != FrameLoader::NoDismissal) {
+            KURL requestURL = request.url();
+            if (requestURL.isValid() && canRequest(CachedResource::ImageResource, requestURL))
+                PingLoader::loadImage(f, requestURL);
             return 0;
         }
     }
-    CachedImage* resource = static_cast<CachedImage*>(requestResource(CachedResource::ImageResource, url, String()));
-    if (autoLoadImages() && resource && resource->stillNeedsLoad()) {
-        resource->setLoading(true);
-        load(resource, true);
-    }
+    CachedImage* resource = static_cast<CachedImage*>(requestResource(CachedResource::ImageResource, request, String()));
+    if (autoLoadImages() && resource && resource->stillNeedsLoad())
+        resource->load(this);
     return resource;
 }
 
-CachedFont* CachedResourceLoader::requestFont(const String& url)
+CachedFont* CachedResourceLoader::requestFont(ResourceRequest& request)
 {
-    return static_cast<CachedFont*>(requestResource(CachedResource::FontResource, url, String()));
+    return static_cast<CachedFont*>(requestResource(CachedResource::FontResource, request, String()));
 }
 
-CachedCSSStyleSheet* CachedResourceLoader::requestCSSStyleSheet(const String& url, const String& charset, ResourceLoadPriority priority)
+CachedCSSStyleSheet* CachedResourceLoader::requestCSSStyleSheet(ResourceRequest& request, const String& charset, ResourceLoadPriority priority)
 {
-    return static_cast<CachedCSSStyleSheet*>(requestResource(CachedResource::CSSStyleSheet, url, charset, priority));
+    return static_cast<CachedCSSStyleSheet*>(requestResource(CachedResource::CSSStyleSheet, request, charset, priority));
 }
 
-CachedCSSStyleSheet* CachedResourceLoader::requestUserCSSStyleSheet(const String& requestURL, const String& charset)
+CachedCSSStyleSheet* CachedResourceLoader::requestUserCSSStyleSheet(ResourceRequest& request, const String& charset)
 {
-    KURL url = MemoryCache::removeFragmentIdentifierIfNeeded(KURL(KURL(), requestURL));
+    KURL url = MemoryCache::removeFragmentIdentifierIfNeeded(request.url());
 
     if (CachedResource* existing = memoryCache()->resourceForURL(url)) {
         if (existing->type() == CachedResource::CSSStyleSheet)
             return static_cast<CachedCSSStyleSheet*>(existing);
         memoryCache()->remove(existing);
     }
-    CachedCSSStyleSheet* userSheet = new CachedCSSStyleSheet(url, charset);
+    if (url.string() != request.url())
+        request.setURL(url);
+
+    CachedCSSStyleSheet* userSheet = new CachedCSSStyleSheet(request, charset);
     
     bool inCache = memoryCache()->add(userSheet);
     if (!inCache)
         userSheet->setInCache(true);
-    
-    userSheet->load(this, /*incremental*/ false, SkipSecurityCheck, /*sendResourceLoadCallbacks*/ false);
+
+    userSheet->setResourceLoaderOptions(ResourceLoaderOptions(DoNotSendCallbacks, SniffContent, BufferData, AllowStoredCredentials));
+    userSheet->load(this, /*incremental*/ false, SkipSecurityCheck);
 
     if (!inCache)
         userSheet->setInCache(false);
@@ -173,28 +176,71 @@ CachedCSSStyleSheet* CachedResourceLoader::requestUserCSSStyleSheet(const String
     return userSheet;
 }
 
-CachedScript* CachedResourceLoader::requestScript(const String& url, const String& charset)
+CachedScript* CachedResourceLoader::requestScript(ResourceRequest& request, const String& charset)
 {
-    return static_cast<CachedScript*>(requestResource(CachedResource::Script, url, charset));
+    return static_cast<CachedScript*>(requestResource(CachedResource::Script, request, charset));
 }
 
 #if ENABLE(XSLT)
-CachedXSLStyleSheet* CachedResourceLoader::requestXSLStyleSheet(const String& url)
+CachedXSLStyleSheet* CachedResourceLoader::requestXSLStyleSheet(ResourceRequest& request)
 {
-    return static_cast<CachedXSLStyleSheet*>(requestResource(CachedResource::XSLStyleSheet, url, String()));
+    return static_cast<CachedXSLStyleSheet*>(requestResource(CachedResource::XSLStyleSheet, request, String()));
 }
 #endif
 
 #if ENABLE(LINK_PREFETCH)
-CachedResource* CachedResourceLoader::requestLinkPrefetch(const String& url)
+CachedResource* CachedResourceLoader::requestLinkResource(CachedResource::Type type, ResourceRequest& request, ResourceLoadPriority priority)
 {
     ASSERT(frame());
-    return requestResource(CachedResource::LinkPrefetch, url, String());
+    ASSERT(type == CachedResource::LinkPrefetch || type == CachedResource::LinkPrerender || type == CachedResource::LinkSubresource);
+    return requestResource(type, request, String(), priority);
 }
 #endif
 
-bool CachedResourceLoader::canRequest(CachedResource::Type type, const KURL& url)
+bool CachedResourceLoader::checkInsecureContent(CachedResource::Type type, const KURL& url) const
 {
+    switch (type) {
+    case CachedResource::Script:
+#if ENABLE(XSLT)
+    case CachedResource::XSLStyleSheet:
+#endif
+    case CachedResource::CSSStyleSheet:
+        // These resource can inject script into the current document (Script,
+        // XSL) or exfiltrate the content of the current document (CSS).
+        if (Frame* f = frame())
+            if (!f->loader()->checkIfRunInsecureContent(m_document->securityOrigin(), url))
+                return false;
+        break;
+    case CachedResource::ImageResource:
+    case CachedResource::FontResource: {
+        // These resources can corrupt only the frame's pixels.
+        if (Frame* f = frame()) {
+            Frame* top = f->tree()->top();
+            if (!top->loader()->checkIfDisplayInsecureContent(top->document()->securityOrigin(), url))
+                return false;
+        }
+        break;
+    }
+#if ENABLE(LINK_PREFETCH)
+    case CachedResource::LinkPrefetch:
+    case CachedResource::LinkPrerender:
+    case CachedResource::LinkSubresource:
+        // Prefetch cannot affect the current document.
+        break;
+#endif
+    }
+    return true;
+}
+
+bool CachedResourceLoader::canRequest(CachedResource::Type type, const KURL& url, bool forPreload)
+{
+    if (!document()->securityOrigin()->canDisplay(url)) {
+        if (!forPreload)
+            FrameLoader::reportLocalLoadFailed(document()->frame(), url.string());
+        LOG(ResourceLoading, "CachedResourceLoader::requestResource URL was not allowed by SecurityOrigin::canDisplay");
+        return 0;
+    }
+
     // Some types of resources can be loaded only from the same origin.  Other
     // types of resources, like Images, Scripts, and CSS, can be loaded from
     // any URL.
@@ -205,6 +251,8 @@ bool CachedResourceLoader::canRequest(CachedResource::Type type, const KURL& url
     case CachedResource::FontResource:
 #if ENABLE(LINK_PREFETCH)
     case CachedResource::LinkPrefetch:
+    case CachedResource::LinkPrerender:
+    case CachedResource::LinkSubresource:
 #endif
         // These types of resources can be loaded from any origin.
         // FIXME: Are we sure about CachedResource::FontResource?
@@ -217,53 +265,58 @@ bool CachedResourceLoader::canRequest(CachedResource::Type type, const KURL& url
         }
         break;
 #endif
-    default:
-        ASSERT_NOT_REACHED();
-        break;
     }
 
     // Given that the load is allowed by the same-origin policy, we should
     // check whether the load passes the mixed-content policy.
     //
-    // Note: Currently, we always allow mixed content, but we generate a
-    //       callback to the FrameLoaderClient in case the embedder wants to
-    //       update any security indicators.
-    // 
+    // FIXME: Should we consider forPreload here?
+    if (!checkInsecureContent(type, url))
+        return false;
+
+    // FIXME: Consider letting the embedder block mixed content loads.
+
     switch (type) {
     case CachedResource::Script:
+        if (!m_document->contentSecurityPolicy()->allowScriptFromSource(url))
+            return false;
+        break;
 #if ENABLE(XSLT)
     case CachedResource::XSLStyleSheet:
 #endif
-        // These resource can inject script into the current document.
-        if (Frame* f = frame())
-            f->loader()->checkIfRunInsecureContent(m_document->securityOrigin(), url);
+    case CachedResource::CSSStyleSheet:
+        if (!m_document->contentSecurityPolicy()->allowStyleFromSource(url))
+            return false;
         break;
     case CachedResource::ImageResource:
-    case CachedResource::CSSStyleSheet:
-    case CachedResource::FontResource: {
-        // These resources can corrupt only the frame's pixels.
-        if (Frame* f = frame()) {
-            Frame* top = f->tree()->top();
-            top->loader()->checkIfDisplayInsecureContent(top->document()->securityOrigin(), url);
+        if (!m_document->contentSecurityPolicy()->allowImageFromSource(url))
+            return false;
+
+        if (frame()) {
+            Settings* settings = frame()->settings();
+            if (!frame()->loader()->client()->allowImage(!settings || settings->areImagesEnabled(), url))
+                return false;
         }
         break;
+    case CachedResource::FontResource: {
+        if (!m_document->contentSecurityPolicy()->allowFontFromSource(url))
+            return false;
+        break;
     }
 #if ENABLE(LINK_PREFETCH)
     case CachedResource::LinkPrefetch:
-        // Prefetch cannot affect the current document.
+    case CachedResource::LinkPrerender:
+    case CachedResource::LinkSubresource:
         break;
 #endif
-    default:
-        ASSERT_NOT_REACHED();
-        break;
     }
-    // FIXME: Consider letting the embedder block mixed content loads.
+
     return true;
 }
 
-CachedResource* CachedResourceLoader::requestResource(CachedResource::Type type, const String& resourceURL, const String& charset, ResourceLoadPriority priority, bool forPreload)
+CachedResource* CachedResourceLoader::requestResource(CachedResource::Type type, ResourceRequest& request, const String& charset, ResourceLoadPriority priority, bool forPreload, const ResourceLoaderOptions& options)
 {
-    KURL url = m_document->completeURL(resourceURL);
+    KURL url = request.url();
     
     LOG(ResourceLoading, "CachedResourceLoader::requestResource '%s', charset '%s', priority=%d, forPreload=%u", url.string().latin1().data(), charset.latin1().data(), priority, forPreload);
     
@@ -272,17 +325,9 @@ CachedResource* CachedResourceLoader::requestResource(CachedResource::Type type,
 
     if (!url.isValid())
         return 0;
-    
-    if (!canRequest(type, url))
-        return 0;
 
-    // FIXME: Figure out what is the correct way to merge this security check with the one above.
-    if (!document()->securityOrigin()->canDisplay(url)) {
-        if (!forPreload)
-            FrameLoader::reportLocalLoadFailed(document()->frame(), url.string());
-        LOG(ResourceLoading, "CachedResourceLoader::requestResource URL was not allowed by SecurityOrigin::canDisplay");
+    if (!canRequest(type, url, forPreload))
         return 0;
-    }
 
     if (memoryCache()->disabled()) {
         DocumentResourceMap::iterator it = m_documentResources.find(url.string());
@@ -295,13 +340,16 @@ CachedResource* CachedResourceLoader::requestResource(CachedResource::Type type,
     // See if we can use an existing resource from the cache.
     CachedResource* resource = memoryCache()->resourceForURL(url);
 
-    switch (determineRevalidationPolicy(type, forPreload, resource)) {
+    if (request.url() != url)
+        request.setURL(url);
+
+    switch (determineRevalidationPolicy(type, request, forPreload, resource)) {
     case Load:
-        resource = loadResource(type, url, charset, priority);
+        resource = loadResource(type, request, charset, priority, options);
         break;
     case Reload:
         memoryCache()->remove(resource);
-        resource = loadResource(type, url, charset, priority);
+        resource = loadResource(type, request, charset, priority, options);
         break;
     case Revalidate:
         resource = revalidateResource(resource, priority);
@@ -331,7 +379,8 @@ CachedResource* CachedResourceLoader::revalidateResource(CachedResource* resourc
     
     // Copy the URL out of the resource to be revalidated in case it gets deleted by the remove() call below.
     String url = resource->url();
-    CachedResource* newResource = createResource(resource->type(), KURL(ParsedURLString, url), resource->encoding());
+    bool urlProtocolIsData = resource->url().protocolIsData();
+    CachedResource* newResource = createResource(resource->type(), resource->resourceRequest(), resource->encoding());
     
     LOG(ResourceLoading, "Resource %p created to revalidate %p", newResource, resource);
     newResource->setResourceToRevalidate(resource);
@@ -341,18 +390,19 @@ CachedResource* CachedResourceLoader::revalidateResource(CachedResource* resourc
     
     newResource->setLoadPriority(priority);
     newResource->load(this);
-    
-    m_validatedURLs.add(url);
+
+    if (!urlProtocolIsData)
+        m_validatedURLs.add(url);
     return newResource;
 }
 
-CachedResource* CachedResourceLoader::loadResource(CachedResource::Type type, const KURL& url, const String& charset, ResourceLoadPriority priority)
+CachedResource* CachedResourceLoader::loadResource(CachedResource::Type type, ResourceRequest& request, const String& charset, ResourceLoadPriority priority, const ResourceLoaderOptions& options)
 {
-    ASSERT(!memoryCache()->resourceForURL(url));
+    ASSERT(!memoryCache()->resourceForURL(request.url()));
     
-    LOG(ResourceLoading, "Loading CachedResource for '%s'.", url.string().latin1().data());
+    LOG(ResourceLoading, "Loading CachedResource for '%s'.", request.url().string().latin1().data());
     
-    CachedResource* resource = createResource(type, url, charset);
+    CachedResource* resource = createResource(type, request, charset);
     
     bool inCache = memoryCache()->add(resource);
     
@@ -362,6 +412,7 @@ CachedResource* CachedResourceLoader::loadResource(CachedResource::Type type, co
         resource->setInCache(true);
     
     resource->setLoadPriority(priority);
+    resource->setResourceLoaderOptions(options);
     resource->load(this);
     
     if (!inCache) {
@@ -378,11 +429,12 @@ CachedResource* CachedResourceLoader::loadResource(CachedResource::Type type, co
         return 0;
     }
 
-    m_validatedURLs.add(url.string());
+    if (!request.url().protocolIsData())
+        m_validatedURLs.add(request.url());
     return resource;
 }
 
-CachedResourceLoader::RevalidationPolicy CachedResourceLoader::determineRevalidationPolicy(CachedResource::Type type, bool forPreload, CachedResource* existingResource) const
+CachedResourceLoader::RevalidationPolicy CachedResourceLoader::determineRevalidationPolicy(CachedResource::Type type, ResourceRequest& request, bool forPreload, CachedResource* existingResource) const
 {
     if (!existingResource)
         return Load;
@@ -415,10 +467,21 @@ CachedResourceLoader::RevalidationPolicy CachedResourceLoader::determineRevalida
         return Reload;
     }
 
-    // Avoid loading the same resource multiple times for a single document, even if the cache policies would tell us to.
-    if (m_validatedURLs.contains(existingResource->url()))
+    // If credentials were sent with the previous request and won't be
+    // with this one, or vice versa, re-fetch the resource.
+    //
+    // This helps with the case where the server sends back
+    // "Access-Control-Allow-Origin: *" all the time, but some of the
+    // client's requests are made without CORS and some with.
+    if (existingResource->resourceRequest().allowCookies() != request.allowCookies()) {
+        LOG(ResourceLoading, "CachedResourceLoader::determineRevalidationPolicy reloading due to difference in credentials settings.");
+        return Reload;
+    }
+
+    // During the initial load, avoid loading the same resource multiple times for a single document, even if the cache policies would tell us to.
+    if (!document()->loadEventFinished() && m_validatedURLs.contains(existingResource->url()))
         return Use;
-    
+
     // CachePolicyReload always reloads
     if (cachePolicy() == CachePolicyReload) {
         LOG(ResourceLoading, "CachedResourceLoader::determineRevalidationPolicy reloading due to CachePolicyReload.");
@@ -461,9 +524,11 @@ void CachedResourceLoader::printAccessDeniedMessage(const KURL& url) const
     if (!settings || settings->privateBrowsingEnabled())
         return;
 
-    String message = m_document->url().isNull() ?
-        makeString("Unsafe attempt to load URL ", url.string(), '.') :
-        makeString("Unsafe attempt to load URL ", url.string(), " from frame with URL ", m_document->url().string(), ". Domains, protocols and ports must match.\n");
+    String message;
+    if (m_document->url().isNull())
+        message = "Unsafe attempt to load URL " + url.string() + '.';
+    else
+        message = "Unsafe attempt to load URL " + url.string() + " from frame with URL " + m_document->url().string() + ". Domains, protocols and ports must match.\n";
 
     // FIXME: provide a real line number and source URL.
     frame()->domWindow()->console()->addMessage(OtherMessageSource, LogMessageType, ErrorMessageLevel, message, 1, String());
@@ -486,7 +551,7 @@ void CachedResourceLoader::setAutoLoadImages(bool enable)
             CachedImage* image = const_cast<CachedImage*>(static_cast<const CachedImage*>(resource));
 
             if (image->stillNeedsLoad())
-                load(image, true);
+                image->load(this);
         }
     }
 }
@@ -506,38 +571,39 @@ void CachedResourceLoader::removeCachedResource(CachedResource* resource) const
     m_documentResources.remove(resource->url());
 }
 
-void CachedResourceLoader::load(CachedResource* resource, bool incremental, SecurityCheckPolicy securityCheck, bool sendResourceLoadCallbacks)
-{
-    incrementRequestCount(resource);
-
-    RefPtr<CachedResourceRequest> request = CachedResourceRequest::load(this, resource, incremental, securityCheck, sendResourceLoadCallbacks);
-    if (request)
-        m_requests.add(request);
-}
-
-void CachedResourceLoader::loadDone(CachedResourceRequest* request)
+void CachedResourceLoader::loadDone()
 {
     m_loadFinishing = false;
-    RefPtr<CachedResourceRequest> protect(request);
-    if (request)
-        m_requests.remove(request);
     if (frame())
         frame()->loader()->loadDone();
-
-    if (!request) {
-        // If the request passed to this function is null, loadDone finished synchronously from when
-        // the load was started, so we want to kick off our next set of loads (via checkForPendingPreloads
-        // and servePendingRequests) asynchronously.
-        m_loadDoneActionTimer.startOneShot(0);
-        return;
-    }
-
     performPostLoadActions();
+
+    if (!m_garbageCollectDocumentResourcesTimer.isActive())
+        m_garbageCollectDocumentResourcesTimer.startOneShot(0);
 }
 
-void CachedResourceLoader::loadDoneActionTimerFired(Timer<CachedResourceLoader>*)
+// Garbage collecting m_documentResources is a workaround for the
+// CachedResourceHandles on the RHS being strong references. Ideally this
+// would be a weak map, however CachedResourceHandles perform additional
+// bookkeeping on CachedResources, so instead pseudo-GC them -- when the
+// reference count reaches 1, m_documentResources is the only reference, so
+// remove it from the map.
+void CachedResourceLoader::garbageCollectDocumentResourcesTimerFired(Timer<CachedResourceLoader>* timer)
 {
-    performPostLoadActions();
+    ASSERT_UNUSED(timer, timer == &m_garbageCollectDocumentResourcesTimer);
+
+    typedef Vector<String, 10> StringVector;
+    StringVector resourcesToDelete;
+
+    for (DocumentResourceMap::iterator it = m_documentResources.begin(); it != m_documentResources.end(); ++it) {
+        if (it->second->hasOneHandle()) {
+            resourcesToDelete.append(it->first);
+            it->second->setOwningCachedResourceLoader(0);
+        }
+    }
+
+    for (StringVector::const_iterator it = resourcesToDelete.begin(); it != resourcesToDelete.end(); ++it)
+        m_documentResources.remove(*it);
 }
 
 void CachedResourceLoader::performPostLoadActions()
@@ -546,18 +612,6 @@ void CachedResourceLoader::performPostLoadActions()
     resourceLoadScheduler()->servePendingRequests();
 }
 
-void CachedResourceLoader::cancelRequests()
-{
-    clearPendingPreloads();
-    Vector<CachedResourceRequest*, 256> requestsToCancel;
-    RequestSet::iterator end = m_requests.end();
-    for (RequestSet::iterator i = m_requests.begin(); i != end; ++i)
-        requestsToCancel.append((*i).get());
-
-    for (unsigned i = 0; i < requestsToCancel.size(); ++i)
-        requestsToCancel[i]->didFail(true);
-}
-
 void CachedResourceLoader::notifyLoadedFromMemoryCache(CachedResource* resource)
 {
     if (!resource || !frame() || resource->status() != CachedResource::Cached)
@@ -569,7 +623,7 @@ void CachedResourceLoader::notifyLoadedFromMemoryCache(CachedResource* resource)
 
 void CachedResourceLoader::incrementRequestCount(const CachedResource* res)
 {
-    if (res->isPrefetch())
+    if (res->isLinkResource())
         return;
 
     ++m_requestCount;
@@ -577,7 +631,7 @@ void CachedResourceLoader::incrementRequestCount(const CachedResource* res)
 
 void CachedResourceLoader::decrementRequestCount(const CachedResource* res)
 {
-    if (res->isPrefetch())
+    if (res->isLinkResource())
         return;
 
     --m_requestCount;
@@ -591,17 +645,21 @@ int CachedResourceLoader::requestCount()
     return m_requestCount;
 }
     
-void CachedResourceLoader::preload(CachedResource::Type type, const String& url, const String& charset, bool referencedFromBody)
+void CachedResourceLoader::preload(CachedResource::Type type, ResourceRequest& request, const String& charset, bool referencedFromBody)
 {
+    // FIXME: Rip this out when we are sure it is no longer necessary (even for mobile).
+    UNUSED_PARAM(referencedFromBody);
+
     bool hasRendering = m_document->body() && m_document->body()->renderer();
-    if (!hasRendering && (referencedFromBody || type == CachedResource::ImageResource)) {
-        // Don't preload images or body resources before we have something to draw. This prevents
-        // preloads from body delaying first display when bandwidth is limited.
-        PendingPreload pendingPreload = { type, url, charset };
+    bool canBlockParser = type == CachedResource::Script || type == CachedResource::CSSStyleSheet;
+    if (!hasRendering && !canBlockParser) {
+        // Don't preload subresources that can't block the parser before we have something to draw.
+        // This helps prevent preloads from delaying first display when bandwidth is limited.
+        PendingPreload pendingPreload = { type, request, charset };
         m_pendingPreloads.append(pendingPreload);
         return;
     }
-    requestPreload(type, url, charset);
+    requestPreload(type, request, charset);
 }
 
 void CachedResourceLoader::checkForPendingPreloads() 
@@ -611,19 +669,19 @@ void CachedResourceLoader::checkForPendingPreloads()
     while (!m_pendingPreloads.isEmpty()) {
         PendingPreload preload = m_pendingPreloads.takeFirst();
         // Don't request preload if the resource already loaded normally (this will result in double load if the page is being reloaded with cached results ignored).
-        if (!cachedResource(m_document->completeURL(preload.m_url)))
-            requestPreload(preload.m_type, preload.m_url, preload.m_charset);
+        if (!cachedResource(preload.m_request.url()))
+            requestPreload(preload.m_type, preload.m_request, preload.m_charset);
     }
     m_pendingPreloads.clear();
 }
 
-void CachedResourceLoader::requestPreload(CachedResource::Type type, const String& url, const String& charset)
+void CachedResourceLoader::requestPreload(CachedResource::Type type, ResourceRequest& request, const String& charset)
 {
     String encoding;
     if (type == CachedResource::Script || type == CachedResource::CSSStyleSheet)
         encoding = charset.isEmpty() ? m_document->charset() : charset;
 
-    CachedResource* resource = requestResource(type, url, encoding, ResourceLoadPriorityUnresolved, true);
+    CachedResource* resource = requestResource(type, request, encoding, ResourceLoadPriorityUnresolved, true);
     if (!resource || (m_preloads && m_preloads->contains(resource)))
         return;
     resource->increasePreloadCount();
@@ -637,6 +695,28 @@ void CachedResourceLoader::requestPreload(CachedResource::Type type, const Strin
 #endif
 }
 
+bool CachedResourceLoader::isPreloaded(const String& urlString) const
+{
+    const KURL& url = m_document->completeURL(urlString);
+
+    if (m_preloads) {
+        ListHashSet<CachedResource*>::iterator end = m_preloads->end();
+        for (ListHashSet<CachedResource*>::iterator it = m_preloads->begin(); it != end; ++it) {
+            CachedResource* resource = *it;
+            if (resource->url() == url)
+                return true;
+        }
+    }
+
+    Deque<PendingPreload>::const_iterator dequeEnd = m_pendingPreloads.end();
+    for (Deque<PendingPreload>::const_iterator it = m_pendingPreloads.begin(); it != dequeEnd; ++it) {
+        PendingPreload pendingPreload = *it;
+        if (pendingPreload.m_request.url() == url)
+            return true;
+    }
+    return false;
+}
+
 void CachedResourceLoader::clearPreloads()
 {
 #if PRELOAD_DEBUG
