@@ -27,10 +27,8 @@
 #ifndef DOMWindow_h
 #define DOMWindow_h
 
+#include "EventTarget.h"
 #include "KURL.h"
-#include "MessagePort.h"
-#include "SecurityOrigin.h"
-#include "Timer.h"
 
 namespace WebCore {
 
@@ -51,6 +49,7 @@ namespace WebCore {
     class EventListener;
     class FileSystemCallback;
     class FloatRect;
+    class Frame;
     class History;
     class IDBFactory;
     class Location;
@@ -62,8 +61,10 @@ namespace WebCore {
     class PostMessageTimer;
     class ScheduledAction;
     class Screen;
+    class SecurityOrigin;
     class SerializedScriptValue;
     class Storage;
+    class StorageInfo;
     class StyleMedia;
     class WebKitPoint;
 
@@ -73,6 +74,8 @@ namespace WebCore {
 
     struct WindowFeatures;
 
+    typedef Vector<RefPtr<MessagePort>, 1> MessagePortArray;
+
     typedef int ExceptionCode;
 
     enum SetLocationLocking { LockHistoryBasedOnGestureState, LockHistoryAndBackForwardList };
@@ -92,7 +95,7 @@ namespace WebCore {
 
         PassRefPtr<MediaQueryList> matchMedia(const String&);
 
-        void setSecurityOrigin(SecurityOrigin* securityOrigin) { m_securityOrigin = securityOrigin; }
+        void setSecurityOrigin(SecurityOrigin*);
         SecurityOrigin* securityOrigin() const { return m_securityOrigin.get(); }
 
         void setURL(const KURL& url) { m_url = url; }
@@ -228,7 +231,7 @@ namespace WebCore {
         void postMessage(PassRefPtr<SerializedScriptValue> message, const MessagePortArray*, const String& targetOrigin, DOMWindow* source, ExceptionCode&);
         // FIXME: remove this when we update the ObjC bindings (bug #28774).
         void postMessage(PassRefPtr<SerializedScriptValue> message, MessagePort*, const String& targetOrigin, DOMWindow* source, ExceptionCode&);
-        void postMessageTimerFired(PostMessageTimer*);
+        void postMessageTimerFired(PassOwnPtr<PostMessageTimer>);
 
         void scrollBy(int x, int y) const;
         void scrollTo(int x, int y) const;
@@ -284,8 +287,6 @@ namespace WebCore {
         DEFINE_ATTRIBUTE_EVENT_LISTENER(ended);
         DEFINE_ATTRIBUTE_EVENT_LISTENER(error);
         DEFINE_ATTRIBUTE_EVENT_LISTENER(focus);
-        DEFINE_ATTRIBUTE_EVENT_LISTENER(formchange);
-        DEFINE_ATTRIBUTE_EVENT_LISTENER(forminput);
         DEFINE_ATTRIBUTE_EVENT_LISTENER(hashchange);
         DEFINE_ATTRIBUTE_EVENT_LISTENER(input);
         DEFINE_ATTRIBUTE_EVENT_LISTENER(invalid);
@@ -341,22 +342,6 @@ namespace WebCore {
 
         void finishedLoading();
 
-        // These functions are used for GC marking. See JSDOMWindow::markChildren(MarkStack&) in JSDOMWindowCustom.cpp.
-        Screen* optionalScreen() const { return m_screen.get(); }
-        DOMSelection* optionalSelection() const { return m_selection.get(); }
-        History* optionalHistory() const { return m_history.get(); }
-        Crypto* optionalCrypto() const { return m_crypto.get(); }
-        BarInfo* optionalLocationbar() const { return m_locationbar.get(); }
-        BarInfo* optionalMenubar() const { return m_menubar.get(); }
-        BarInfo* optionalPersonalbar() const { return m_personalbar.get(); }
-        BarInfo* optionalScrollbars() const { return m_scrollbars.get(); }
-        BarInfo* optionalStatusbar() const { return m_statusbar.get(); }
-        BarInfo* optionalToolbar() const { return m_toolbar.get(); }
-        Console* optionalConsole() const { return m_console.get(); }
-        Navigator* optionalNavigator() const { return m_navigator.get(); }
-        Location* optionalLocation() const { return m_location.get(); }
-        StyleMedia* optionalMedia() const { return m_media.get(); }
-
         using RefCounted<DOMWindow>::ref;
         using RefCounted<DOMWindow>::deref;
 
@@ -364,8 +349,7 @@ namespace WebCore {
         DOMURL* webkitURL() const;
 #endif
 
-#if ENABLE(DATABASE)
-        // HTML 5 client-side database
+#if ENABLE(SQL_DATABASE)
         PassRefPtr<Database> openDatabase(const String& name, const String& version, const String& displayName, unsigned long estimatedSize, PassRefPtr<DatabaseCallback> creationCallback, ExceptionCode&);
 #endif
 
@@ -378,8 +362,6 @@ namespace WebCore {
         // HTML 5 key/value storage
         Storage* sessionStorage(ExceptionCode&) const;
         Storage* localStorage(ExceptionCode&) const;
-        Storage* optionalSessionStorage() const { return m_sessionStorage.get(); }
-        Storage* optionalLocalStorage() const { return m_localStorage.get(); }
 #endif
 
 #if ENABLE(FILE_SYSTEM)
@@ -388,9 +370,10 @@ namespace WebCore {
         enum FileSystemType {
             TEMPORARY,
             PERSISTENT,
+            EXTERNAL,
         };
-        void requestFileSystem(int type, long long size, PassRefPtr<FileSystemCallback>, PassRefPtr<ErrorCallback>);
-        void resolveLocalFileSystemURL(const String&, PassRefPtr<EntryCallback>, PassRefPtr<ErrorCallback>);
+        void webkitRequestFileSystem(int type, long long size, PassRefPtr<FileSystemCallback>, PassRefPtr<ErrorCallback>);
+        void webkitResolveLocalFileSystemURL(const String&, PassRefPtr<EntryCallback>, PassRefPtr<ErrorCallback>);
 #endif
 
 #if ENABLE(INDEXED_DATABASE)
@@ -401,6 +384,10 @@ namespace WebCore {
         NotificationCenter* webkitNotifications() const;
 #endif
 
+#if ENABLE(QUOTA)
+        StorageInfo* webkitStorageInfo() const;
+#endif
+
 #if ENABLE(OFFLINE_WEB_APPLICATIONS)
         DOMApplicationCache* applicationCache() const;
         DOMApplicationCache* optionalApplicationCache() const { return m_applicationCache.get(); }
@@ -424,7 +411,6 @@ namespace WebCore {
 
 #if ENABLE(WEB_TIMING)
         Performance* performance() const;
-        Performance* optionalPerformance() const { return m_performance.get(); }
 #endif
 
     private:
@@ -435,8 +421,6 @@ namespace WebCore {
         virtual EventTargetData* eventTargetData();
         virtual EventTargetData* ensureEventTargetData();
 
-        void printTimerFired(Timer<DOMWindow>*);
-
         static Frame* createWindow(const String& urlString, const AtomicString& frameName, const WindowFeatures&,
             DOMWindow* activeWindow, Frame* firstFrame, Frame* openerFrame,
             PrepareDialogFunction = 0, void* functionContext = 0);
@@ -462,8 +446,6 @@ namespace WebCore {
         mutable RefPtr<Location> m_location;
         mutable RefPtr<StyleMedia> m_media;
 
-        Timer<DOMWindow> m_printTimer;
-
         EventTargetData m_eventTargetData;
 
         String m_status;
@@ -493,6 +475,10 @@ namespace WebCore {
 #if ENABLE(BLOB)
         mutable RefPtr<DOMURL> m_domURL;
 #endif
+
+#if ENABLE(QUOTA)
+        mutable RefPtr<StorageInfo> m_storageInfo;
+#endif
     };
 
     inline String DOMWindow::status() const
