@@ -43,6 +43,7 @@
 #include "FloatRect.h"
 #include "Frame.h"
 #include "FrameLoader.h"
+#include "FrameSelection.h"
 #include "FrameView.h"
 #include "HTMLAnchorElement.h"
 #include "HTMLInputElement.h"
@@ -60,7 +61,7 @@
 #include "RenderView.h"
 #include "ReplaceSelectionCommand.h"
 #include "ResourceRequest.h"
-#include "SelectionController.h"
+#include "SecurityOrigin.h"
 #include "Settings.h"
 #include "Text.h"
 #include "TextEvent.h"
@@ -141,7 +142,7 @@ static PassRefPtr<DocumentFragment> documentFragmentFromDragData(DragData* dragD
     return 0;
 }
 
-bool DragController::dragIsMove(SelectionController* selection, DragData* dragData)
+bool DragController::dragIsMove(FrameSelection* selection, DragData* dragData)
 {
     return m_documentUnderMouse == m_dragInitiator && selection->isContentEditable() && !isCopyKeyDown(dragData);
 }
@@ -157,6 +158,8 @@ void DragController::dragEnded()
     m_dragInitiator = 0;
     m_didInitiateDrag = false;
     m_page->dragCaretController()->clear();
+    
+    m_client->dragEnded();
 }
 
 DragOperation DragController::dragEntered(DragData* dragData)
@@ -252,20 +255,13 @@ static HTMLInputElement* asFileInput(Node* node)
 {
     ASSERT(node);
 
-    // The button for a FILE input is a sub element with no set input type
-    // In order to get around this problem we assume any non-FILE input element
-    // is this internal button, and try querying the shadow parent node.
-    if (node->hasTagName(HTMLNames::inputTag) && node->isShadowRoot() && !static_cast<HTMLInputElement*>(node)->isFileUpload())
-        node = node->shadowHost();
+    HTMLInputElement* inputElement = node->toInputElement();
 
-    if (!node || !node->hasTagName(HTMLNames::inputTag))
-        return 0;
+    // If this is a button inside of the a file input, move up to the file input.
+    if (inputElement && inputElement->isTextButton() && inputElement->treeScope()->isShadowRoot())
+        inputElement = inputElement->treeScope()->shadowHost()->toInputElement();
 
-    HTMLInputElement* inputElement = static_cast<HTMLInputElement*>(node);
-    if (!inputElement->isFileUpload())
-        return 0;
-
-    return inputElement;
+    return inputElement && inputElement->isFileUpload() ? inputElement : 0;
 }
 
 // This can return null if an empty document is loaded.
@@ -329,10 +325,8 @@ bool DragController::tryDocumentDrag(DragData* dragData, DragDestinationAction a
         Element* element = elementUnderMouse(m_documentUnderMouse.get(), point);
         if (!element)
             return false;
-        if (!asFileInput(element)) {
-            VisibleSelection dragCaret = m_documentUnderMouse->frame()->visiblePositionForPoint(point);
-            m_page->dragCaretController()->setSelection(dragCaret);
-        }
+        if (!asFileInput(element))
+            m_page->dragCaretController()->setCaretPosition(m_documentUnderMouse->frame()->visiblePositionForPoint(point));
 
         Frame* innerFrame = element->document()->frame();
         operation = dragIsMove(innerFrame->selection(), dragData) ? DragOperationMove : DragOperationCopy;
@@ -353,7 +347,7 @@ DragOperation DragController::operationForLoad(DragData* dragData)
 {
     ASSERT(dragData);
     Document* doc = m_page->mainFrame()->documentAtPoint(dragData->clientPosition());
-    if (doc && (m_didInitiateDrag || doc->isPluginDocument() || doc->inDesignMode()))
+    if (doc && (m_didInitiateDrag || doc->isPluginDocument() || doc->rendererIsEditable()))
         return DragOperationNone;
     return dragOperation(dragData);
 }
@@ -371,10 +365,9 @@ static bool setSelectionToDragCaret(Frame* frame, VisibleSelection& dragCaret, R
 
 bool DragController::dispatchTextInputEventFor(Frame* innerFrame, DragData* dragData)
 {
-    ASSERT(!m_page->dragCaretController()->isNone());
-    VisibleSelection dragCaret(m_page->dragCaretController()->selection());
-    String text = dragCaret.isContentRichlyEditable() ? "" : dragData->asPlainText(innerFrame);
-    Node* target = innerFrame->editor()->findEventTargetFrom(dragCaret);
+    ASSERT(m_page->dragCaretController()->hasCaret());
+    String text = m_page->dragCaretController()->isContentRichlyEditable() ? "" : dragData->asPlainText(innerFrame);
+    Node* target = innerFrame->editor()->findEventTargetFrom(m_page->dragCaretController()->caretPosition());
     ExceptionCode ec = 0;
     return target->dispatchEvent(TextEvent::createForDrop(innerFrame->domWindow(), text), ec);
 }
@@ -394,7 +387,7 @@ bool DragController::concludeEditDrag(DragData* dragData)
     Frame* innerFrame = element->ownerDocument()->frame();
     ASSERT(innerFrame);
 
-    if (!m_page->dragCaretController()->isNone() && !dispatchTextInputEventFor(innerFrame, dragData))
+    if (m_page->dragCaretController()->hasCaret() && !dispatchTextInputEventFor(innerFrame, dragData))
         return true;
 
     if (dragData->containsColor()) {
@@ -429,18 +422,11 @@ bool DragController::concludeEditDrag(DragData* dragData)
         if (filenames.isEmpty())
             return false;
 
-        // Ugly. For security none of the APIs available to us can set the input value
-        // on file inputs. Even forcing a change in HTMLInputElement doesn't work as
-        // RenderFileUploadControl clears the file when doing updateFromElement().
-        RenderFileUploadControl* renderer = toRenderFileUploadControl(fileInput->renderer());
-        if (!renderer)
-            return false;
-
-        renderer->receiveDroppedFiles(filenames);
+        fileInput->receiveDroppedFiles(filenames);
         return true;
     }
 
-    VisibleSelection dragCaret(m_page->dragCaretController()->selection());
+    VisibleSelection dragCaret = m_page->dragCaretController()->caretPosition();
     m_page->dragCaretController()->clear();
     RefPtr<Range> range = dragCaret.toNormalizedRange();
 
@@ -449,12 +435,11 @@ bool DragController::concludeEditDrag(DragData* dragData)
     if (!range)
         return false;
     CachedResourceLoader* cachedResourceLoader = range->ownerDocument()->cachedResourceLoader();
-    cachedResourceLoader->setAllowStaleResources(true);
+    ResourceCacheValidationSuppressor validationSuppressor(cachedResourceLoader);
     if (dragIsMove(innerFrame->selection(), dragData) || dragCaret.isContentRichlyEditable()) {
         bool chosePlainText = false;
         RefPtr<DocumentFragment> fragment = documentFragmentFromDragData(dragData, innerFrame, range, true, chosePlainText);
         if (!fragment || !innerFrame->editor()->shouldInsertFragment(fragment, range, EditorInsertActionDropped)) {
-            cachedResourceLoader->setAllowStaleResources(false);
             return false;
         }
 
@@ -478,7 +463,6 @@ bool DragController::concludeEditDrag(DragData* dragData)
     } else {
         String text = dragData->asPlainText(innerFrame);
         if (text.isEmpty() || !innerFrame->editor()->shouldInsertText(text, range.get(), EditorInsertActionDropped)) {
-            cachedResourceLoader->setAllowStaleResources(false);
             return false;
         }
 
@@ -486,7 +470,6 @@ bool DragController::concludeEditDrag(DragData* dragData)
         if (setSelectionToDragCaret(innerFrame, dragCaret, range, point))
             applyCommand(ReplaceSelectionCommand::create(m_documentUnderMouse.get(), createFragmentFromText(range.get(), text),  ReplaceSelectionCommand::SelectReplacement | ReplaceSelectionCommand::MatchStyle | ReplaceSelectionCommand::PreventNesting));
     }
-    cachedResourceLoader->setAllowStaleResources(false);
 
     return true;
 }
@@ -511,7 +494,7 @@ bool DragController::canProcessDrag(DragData* dragData)
     if (dragData->containsFiles() && asFileInput(result.innerNonSharedNode()))
         return true;
 
-    if (!result.innerNonSharedNode()->isContentEditable())
+    if (!result.innerNonSharedNode()->rendererIsEditable())
         return false;
 
     if (m_didInitiateDrag && m_documentUnderMouse == m_dragInitiator && result.isSelected())
@@ -571,37 +554,46 @@ bool DragController::tryDHTMLDrag(DragData* dragData, DragOperation& operation)
     return true;
 }
 
-bool DragController::mayStartDragAtEventLocation(const Frame* frame, const IntPoint& framePos, Node* node)
+Node* DragController::draggableNode(const Frame* src, Node* startNode, const IntPoint& dragOrigin, DragState& state) const
 {
-    ASSERT(frame);
-    ASSERT(frame->settings());
+    state.m_dragType = (src->selection()->contains(dragOrigin)) ? DragSourceActionSelection : DragSourceActionNone;
 
-    if (!frame->view() || !frame->contentRenderer())
-        return false;
+    for (const RenderObject* renderer = startNode->renderer(); renderer; renderer = renderer->parent()) {
+        Node* node = renderer->node();
+        if (!node)
+            // Anonymous render blocks don't correspond to actual DOM nodes, so we skip over them
+            // for the purposes of finding a draggable node.
+            continue;
+        if (!(state.m_dragType & DragSourceActionSelection) && node->isTextNode() && node->canStartSelection())
+            // In this case we have a click in the unselected portion of text. If this text is
+            // selectable, we want to start the selection process instead of looking for a parent
+            // to try to drag.
+            return 0;
+        if (node->isElementNode()) {
+            EUserDrag dragMode = renderer->style()->userDrag();
+            if ((m_dragSourceAction & DragSourceActionDHTML) && dragMode == DRAG_ELEMENT) {
+                state.m_dragType = static_cast<DragSourceAction>(state.m_dragType | DragSourceActionDHTML);
+                return node;
+            }
+            if (dragMode == DRAG_AUTO) {
+                if ((m_dragSourceAction & DragSourceActionImage)
+                    && node->hasTagName(HTMLNames::imgTag)
+                    && src->settings()->loadsImagesAutomatically()) {
+                    state.m_dragType = static_cast<DragSourceAction>(state.m_dragType | DragSourceActionImage);
+                    return node;
+                }
+                if ((m_dragSourceAction & DragSourceActionLink)
+                    && node->hasTagName(HTMLNames::aTag)
+                    && static_cast<HTMLAnchorElement*>(node)->isLiveLink()) {
+                    state.m_dragType = static_cast<DragSourceAction>(state.m_dragType | DragSourceActionLink);
+                    return node;
+                }
+            }
+        }
+    }
 
-    HitTestResult mouseDownTarget = HitTestResult(framePos);
-
-    mouseDownTarget = frame->eventHandler()->hitTestResultAtPoint(framePos, true);
-    if (node)
-        mouseDownTarget.setInnerNonSharedNode(node);
-
-    if (mouseDownTarget.image()
-        && !mouseDownTarget.absoluteImageURL().isEmpty()
-        && frame->settings()->loadsImagesAutomatically()
-        && m_dragSourceAction & DragSourceActionImage)
-        return true;
-
-    if (!mouseDownTarget.absoluteLinkURL().isEmpty()
-        && m_dragSourceAction & DragSourceActionLink
-        && mouseDownTarget.isLiveLink()
-        && mouseDownTarget.URLElement()->renderer() && mouseDownTarget.URLElement()->renderer()->style()->userDrag() != DRAG_NONE)
-        return true;
-
-    if (mouseDownTarget.isSelected()
-        && m_dragSourceAction & DragSourceActionSelection)
-        return true;
-
-    return false;
+    // We either have nothing to drag or we have a selection and we're not over a draggable element.
+    return (state.m_dragType & DragSourceActionSelection) ? startNode : 0;
 }
 
 static CachedImage* getCachedImage(Element* element)
@@ -663,19 +655,21 @@ static IntPoint dragLocForSelectionDrag(Frame* src)
     return IntPoint(xpos, ypos);
 }
 
-bool DragController::startDrag(Frame* src, Clipboard* clipboard, DragOperation srcOp, const PlatformMouseEvent& dragEvent, const IntPoint& dragOrigin, bool isDHTMLDrag)
+bool DragController::startDrag(Frame* src, const DragState& state, DragOperation srcOp, const PlatformMouseEvent& dragEvent, const IntPoint& dragOrigin)
 {
     ASSERT(src);
-    ASSERT(clipboard);
 
     if (!src->view() || !src->contentRenderer())
         return false;
 
-    HitTestResult dragSource = HitTestResult(dragOrigin);
-    dragSource = src->eventHandler()->hitTestResultAtPoint(dragOrigin, true);
-    KURL linkURL = dragSource.absoluteLinkURL();
-    KURL imageURL = dragSource.absoluteImageURL();
-    bool isSelected = dragSource.isSelected();
+    HitTestResult hitTestResult = src->eventHandler()->hitTestResultAtPoint(dragOrigin, true);
+    if (!state.m_dragSrc->contains(hitTestResult.innerNode()))
+        // The original node being dragged isn't under the drag origin anymore... maybe it was
+        // hidden or moved out from under the cursor. Regardless, we don't want to start a drag on
+        // something that's not actually under the drag origin.
+        return false;
+    KURL linkURL = hitTestResult.absoluteLinkURL();
+    KURL imageURL = hitTestResult.absoluteImageURL();
 
     IntPoint mouseDraggedPoint = src->view()->windowToContents(dragEvent.pos());
 
@@ -686,14 +680,14 @@ bool DragController::startDrag(Frame* src, Clipboard* clipboard, DragOperation s
     IntPoint dragLoc(0, 0);
     IntPoint dragImageOffset(0, 0);
 
-    if (isDHTMLDrag)
+    Clipboard* clipboard = state.m_dragClipboard.get();
+    if (state.m_dragType == DragSourceActionDHTML)
         dragImage = clipboard->createDragImage(dragImageOffset);
-    else {
-        // This drag operation is not a DHTML drag and may go outside the WebView.
-        // We provide a default set of allowed drag operations that follows from:
+    if (state.m_dragType == DragSourceActionSelection || !imageURL.isEmpty() || !linkURL.isEmpty())
+        // Selection, image, and link drags receive a default set of allowed drag operations that
+        // follows from:
         // http://trac.webkit.org/browser/trunk/WebKit/mac/WebView/WebHTMLView.mm?rev=48526#L3430
-        m_sourceDragOperation = (DragOperation)(DragOperationGeneric | DragOperationCopy);
-    }
+        m_sourceDragOperation = static_cast<DragOperation>(m_sourceDragOperation | DragOperationGeneric | DragOperationCopy);
 
     // We allow DHTML/JS to set the drag image, even if its a link, image or text we're dragging.
     // This is in the spirit of the IE API, which allows overriding of pasteboard data and DragOp.
@@ -704,26 +698,44 @@ bool DragController::startDrag(Frame* src, Clipboard* clipboard, DragOperation s
 
     bool startedDrag = true; // optimism - we almost always manage to start the drag
 
-    Node* node = dragSource.innerNonSharedNode();
+    Node* node = state.m_dragSrc.get();
 
     Image* image = getImage(static_cast<Element*>(node));
-    if (!imageURL.isEmpty() && node && node->isElementNode() && image && !image->isNull()
-            && (m_dragSourceAction & DragSourceActionImage)) {
+    if (state.m_dragType == DragSourceActionSelection) {
+        if (!clipboard->hasData()) {
+            if (enclosingTextFormControl(src->selection()->start()))
+                clipboard->writePlainText(src->editor()->selectedText());
+            else {
+                RefPtr<Range> selectionRange = src->selection()->toNormalizedRange();
+                ASSERT(selectionRange);
+
+                clipboard->writeRange(selectionRange.get(), src);
+            }
+        }
+        m_client->willPerformDragSourceAction(DragSourceActionSelection, dragOrigin, clipboard);
+        if (!dragImage) {
+            dragImage = createDragImageForSelection(src);
+            dragLoc = dragLocForSelectionDrag(src);
+            m_dragOffset = IntPoint(dragOrigin.x() - dragLoc.x(), dragOrigin.y() - dragLoc.y());
+        }
+        doSystemDrag(dragImage, dragLoc, dragOrigin, clipboard, src, false);
+    } else if (!imageURL.isEmpty() && node && node->isElementNode() && image && !image->isNull()
+               && (m_dragSourceAction & DragSourceActionImage)) {
         // We shouldn't be starting a drag for an image that can't provide an extension.
         // This is an early detection for problems encountered later upon drop.
         ASSERT(!image->filenameExtension().isEmpty());
         Element* element = static_cast<Element*>(node);
         if (!clipboard->hasData()) {
             m_draggingImageURL = imageURL;
-            prepareClipboardForImageDrag(src, clipboard, element, linkURL, imageURL, dragSource.altDisplayString());
+            prepareClipboardForImageDrag(src, clipboard, element, linkURL, imageURL, hitTestResult.altDisplayString());
         }
 
         m_client->willPerformDragSourceAction(DragSourceActionImage, dragOrigin, clipboard);
 
         if (!dragImage) {
-            IntRect imageRect = dragSource.imageRect();
+            IntRect imageRect = hitTestResult.imageRect();
             imageRect.setLocation(m_page->mainFrame()->view()->windowToContents(src->view()->contentsToWindow(imageRect.location())));
-            doImageDrag(element, dragOrigin, dragSource.imageRect(), clipboard, src, m_dragOffset);
+            doImageDrag(element, dragOrigin, hitTestResult.imageRect(), clipboard, src, m_dragOffset);
         } else
             // DHTML defined drag image
             doSystemDrag(dragImage, dragLoc, dragOrigin, clipboard, src, false);
@@ -732,7 +744,7 @@ bool DragController::startDrag(Frame* src, Clipboard* clipboard, DragOperation s
         if (!clipboard->hasData())
             // Simplify whitespace so the title put on the clipboard resembles what the user sees
             // on the web page. This includes replacing newlines with spaces.
-            clipboard->writeURL(linkURL, dragSource.textContent().simplifyWhiteSpace(), src);
+            clipboard->writeURL(linkURL, hitTestResult.textContent().simplifyWhiteSpace(), src);
 
         if (src->selection()->isCaret() && src->selection()->isContentEditable()) {
             // a user can initiate a drag on a link without having any text
@@ -746,37 +758,19 @@ bool DragController::startDrag(Frame* src, Clipboard* clipboard, DragOperation s
 
         m_client->willPerformDragSourceAction(DragSourceActionLink, dragOrigin, clipboard);
         if (!dragImage) {
-            dragImage = createDragImageForLink(linkURL, dragSource.textContent(), src);
+            dragImage = createDragImageForLink(linkURL, hitTestResult.textContent(), src);
             IntSize size = dragImageSize(dragImage);
             m_dragOffset = IntPoint(-size.width() / 2, -LinkDragBorderInset);
             dragLoc = IntPoint(mouseDraggedPoint.x() + m_dragOffset.x(), mouseDraggedPoint.y() + m_dragOffset.y());
         }
         doSystemDrag(dragImage, dragLoc, mouseDraggedPoint, clipboard, src, true);
-    } else if (isSelected && (m_dragSourceAction & DragSourceActionSelection)) {
-        if (!clipboard->hasData()) {
-            if (isNodeInTextFormControl(src->selection()->start().deprecatedNode()))
-                clipboard->writePlainText(src->editor()->selectedText());
-            else {
-                RefPtr<Range> selectionRange = src->selection()->toNormalizedRange();
-                ASSERT(selectionRange);
-
-                clipboard->writeRange(selectionRange.get(), src);
-            }
-        }
-        m_client->willPerformDragSourceAction(DragSourceActionSelection, dragOrigin, clipboard);
-        if (!dragImage) {
-            dragImage = createDragImageForSelection(src);
-            dragLoc = dragLocForSelectionDrag(src);
-            m_dragOffset = IntPoint((int)(dragOrigin.x() - dragLoc.x()), (int)(dragOrigin.y() - dragLoc.y()));
-        }
-        doSystemDrag(dragImage, dragLoc, dragOrigin, clipboard, src, false);
-    } else if (isDHTMLDrag) {
+    } else if (state.m_dragType == DragSourceActionDHTML) {
         ASSERT(m_dragSourceAction & DragSourceActionDHTML);
         m_client->willPerformDragSourceAction(DragSourceActionDHTML, dragOrigin, clipboard);
         doSystemDrag(dragImage, dragLoc, dragOrigin, clipboard, src, false);
     } else {
-        // Only way I know to get here is if to get here is if the original element clicked on in the mousedown is no longer
-        // under the mousedown point, so linkURL, imageURL and isSelected are all false/empty.
+        // draggableNode() determined an image or link node was draggable, but it turns out the
+        // image or link had no URL, so there is nothing to drag.
         startedDrag = false;
     }
 
@@ -819,8 +813,7 @@ void DragController::doImageDrag(Element* element, const IntPoint& dragOrigin, c
             origin = IntPoint(DragIconRightInset - dragImageSize(dragImage).width(), DragIconBottomInset);
     }
 
-    dragImageOffset.setX(mouseDownPoint.x() + origin.x());
-    dragImageOffset.setY(mouseDownPoint.y() + origin.y());
+    dragImageOffset = mouseDownPoint + origin;
     doSystemDrag(dragImage, dragImageOffset, dragOrigin, clipboard, frame, false);
 
     deleteDragImage(dragImage);
@@ -850,8 +843,8 @@ void DragController::placeDragCaret(const IntPoint& windowPoint)
     if (!frameView)
         return;
     IntPoint framePoint = frameView->windowToContents(windowPoint);
-    VisibleSelection dragCaret(frame->visiblePositionForPoint(framePoint));
-    m_page->dragCaretController()->setSelection(dragCaret);
+
+    m_page->dragCaretController()->setCaretPosition(frame->visiblePositionForPoint(framePoint));
 }
 
 } // namespace WebCore
