@@ -5,7 +5,7 @@
  *                     2000 Simon Hausmann <hausmann@kde.org>
  *                     2000 Stefan Schimanski <1Stein@gmx.de>
  *                     2001 George Staikos <staikos@kde.org>
- * Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010 Apple Inc. All rights reserved.
+ * Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Apple Inc. All rights reserved.
  * Copyright (C) 2005 Alexey Proskuryakov <ap@nypop.com>
  * Copyright (C) 2008 Nokia Corporation and/or its subsidiary(-ies)
  * Copyright (C) 2008 Eric Seidel <eric@webkit.org>
@@ -30,6 +30,7 @@
 #include "Frame.h"
 
 #include "ApplyStyleCommand.h"
+#include "BackForwardController.h"
 #include "CSSComputedStyleDeclaration.h"
 #include "CSSMutableStyleDeclaration.h"
 #include "CSSProperty.h"
@@ -57,8 +58,10 @@
 #include "HTMLNames.h"
 #include "HTMLTableCellElement.h"
 #include "HitTestResult.h"
+#include "InspectorInstrumentation.h"
 #include "Logging.h"
 #include "MediaFeatureNames.h"
+#include "MediaStreamFrameController.h"
 #include "Navigator.h"
 #include "NodeList.h"
 #include "Page.h"
@@ -70,13 +73,16 @@
 #include "RenderTextControl.h"
 #include "RenderTheme.h"
 #include "RenderView.h"
+#include "RuntimeEnabledFeatures.h"
 #include "ScriptController.h"
 #include "ScriptSourceCode.h"
+#include "ScriptValue.h"
 #include "Settings.h"
 #include "TextIterator.h"
 #include "TextResourceDecoder.h"
 #include "UserContentURLPattern.h"
 #include "UserTypingGestureIndicator.h"
+#include "WebKitFontFamilyNames.h"
 #include "XMLNSNames.h"
 #include "XMLNames.h"
 #include "htmlediting.h"
@@ -108,10 +114,6 @@
 #include "TiledBackingStore.h"
 #endif
 
-#if ENABLE(WML)
-#include "WMLNames.h"
-#endif
-
 using namespace std;
 
 namespace WebCore {
@@ -153,19 +155,20 @@ inline Frame::Frame(Page* page, HTMLFrameOwnerElement* ownerElement, FrameLoader
     , m_ownerElement(ownerElement)
     , m_script(this)
     , m_editor(this)
-    , m_selectionController(this)
+    , m_selection(this)
     , m_eventHandler(this)
     , m_animationController(this)
-    , m_lifeSupportTimer(this, &Frame::lifeSupportTimerFired)
     , m_pageZoomFactor(parentPageZoomFactor(this))
     , m_textZoomFactor(parentTextZoomFactor(this))
-    , m_pageScaleFactor(1)
 #if ENABLE(ORIENTATION_EVENTS)
     , m_orientation(0)
 #endif
     , m_inViewSourceMode(false)
     , m_isDisconnected(false)
     , m_excludeFromTextSearch(false)
+#if ENABLE(MEDIA_STREAM)
+    , m_mediaStreamFrameController(RuntimeEnabledFeatures::mediaStreamEnabled() ? adoptPtr(new MediaStreamFrameController(this)) : PassOwnPtr<MediaStreamFrameController>())
+#endif
 {
     ASSERT(page);
     AtomicString::init();
@@ -177,10 +180,7 @@ inline Frame::Frame(Page* page, HTMLFrameOwnerElement* ownerElement, FrameLoader
     MathMLNames::init();
     XMLNSNames::init();
     XMLNames::init();
-
-#if ENABLE(WML)
-    WMLNames::init();
-#endif
+    WebKitFontFamilyNames::init();
 
     if (!ownerElement) {
 #if ENABLE(TILED_BACKING_STORE)
@@ -216,8 +216,6 @@ Frame::~Frame()
 
     // FIXME: We should not be doing all this work inside the destructor
 
-    ASSERT(!m_lifeSupportTimer.isActive());
-
 #ifndef NDEBUG
     frameCounter.decrement();
 #endif
@@ -226,7 +224,11 @@ Frame::~Frame()
 
     if (m_domWindow)
         m_domWindow->disconnectFrame();
-    script()->clearWindowShell();
+
+#if ENABLE(MEDIA_STREAM)
+    if (m_mediaStreamFrameController)
+        m_mediaStreamFrameController->disconnectFrame();
+#endif
 
     HashSet<DOMWindow*>::iterator end = m_liveFormerWindows.end();
     for (HashSet<DOMWindow*>::iterator it = m_liveFormerWindows.begin(); it != end; ++it)
@@ -240,8 +242,6 @@ Frame::~Frame()
         m_view->hide();
         m_view->clearFrame();
     }
-
-    ASSERT(!m_lifeSupportTimer.isActive());
 }
 
 void Frame::addDestructionObserver(FrameDestructionObserver* observer)
@@ -305,8 +305,11 @@ void Frame::setDocument(PassRefPtr<Document> newDoc)
     // Update the cached 'document' property, which is now stale.
     m_script.updateDocument();
 
-    if (m_page)
+    if (m_page) {
         m_page->updateViewportArguments();
+        if (m_page->mainFrame() == this)
+            notifyChromeClientWheelEventHandlerCountChanged();
+    }
 }
 
 #if ENABLE(ORIENTATION_EVENTS)
@@ -323,7 +326,7 @@ Settings* Frame::settings() const
     return m_page ? m_page->settings() : 0;
 }
 
-static RegularExpression* createRegExpForLabels(const Vector<String>& labels)
+static PassOwnPtr<RegularExpression> createRegExpForLabels(const Vector<String>& labels)
 {
     // REVIEW- version of this call in FrameMac.mm caches based on the NSArray ptrs being
     // the same across calls.  We can't do that.
@@ -354,7 +357,7 @@ static RegularExpression* createRegExpForLabels(const Vector<String>& labels)
             pattern.append("\\b");
     }
     pattern.append(")");
-    return new RegularExpression(pattern, TextCaseInsensitive);
+    return adoptPtr(new RegularExpression(pattern, TextCaseInsensitive));
 }
 
 String Frame::searchForLabelsAboveCell(RegularExpression* regExp, HTMLTableCellElement* cell, size_t* resultDistanceFromStartOfCell)
@@ -500,17 +503,45 @@ String Frame::matchLabelsAgainstElement(const Vector<String>& labels, Element* e
     return matchLabelsAgainstString(labels, element->getAttribute(idAttr));
 }
 
-void Frame::setPrinting(bool printing, const FloatSize& pageSize, float maximumShrinkRatio, AdjustViewSizeOrNot shouldAdjustViewSize)
+void Frame::setPrinting(bool printing, const FloatSize& pageSize, const FloatSize& originalPageSize, float maximumShrinkRatio, AdjustViewSizeOrNot shouldAdjustViewSize)
 {
+    // In setting printing, we should not validate resources already cached for the document.
+    // See https://bugs.webkit.org/show_bug.cgi?id=43704
+    ResourceCacheValidationSuppressor validationSuppressor(m_doc->cachedResourceLoader());
+
     m_doc->setPrinting(printing);
     view()->adjustMediaTypeForPrinting(printing);
 
     m_doc->styleSelectorChanged(RecalcStyleImmediately);
-    view()->forceLayoutForPagination(pageSize, maximumShrinkRatio, shouldAdjustViewSize);
+    if (printing)
+        view()->forceLayoutForPagination(pageSize, originalPageSize, maximumShrinkRatio, shouldAdjustViewSize);
+    else {
+        view()->forceLayout();
+        if (shouldAdjustViewSize == AdjustViewSize)
+            view()->adjustViewSize();
+    }
 
     // Subframes of the one we're printing don't lay out to the page size.
     for (Frame* child = tree()->firstChild(); child; child = child->tree()->nextSibling())
-        child->setPrinting(printing, IntSize(), 0, shouldAdjustViewSize);
+        child->setPrinting(printing, FloatSize(), FloatSize(), 0, shouldAdjustViewSize);
+}
+
+FloatSize Frame::resizePageRectsKeepingRatio(const FloatSize& originalSize, const FloatSize& expectedSize)
+{
+    FloatSize resultSize;
+    if (!contentRenderer())
+        return FloatSize();
+
+    if (contentRenderer()->style()->isHorizontalWritingMode()) {
+        float ratio = originalSize.height() / originalSize.width();
+        resultSize.setWidth(floorf(expectedSize.width()));
+        resultSize.setHeight(floorf(resultSize.width() * ratio));
+    } else {
+        float ratio = originalSize.width() / originalSize.height();
+        resultSize.setHeight(floorf(expectedSize.height()));
+        resultSize.setWidth(floorf(resultSize.height() * ratio));
+    }
+    return resultSize;
 }
 
 void Frame::injectUserScripts(UserScriptInjectionTime injectionTime)
@@ -518,9 +549,9 @@ void Frame::injectUserScripts(UserScriptInjectionTime injectionTime)
     if (!m_page)
         return;
 
-    if (loader()->stateMachine()->creatingInitialEmptyDocument())
+    if (loader()->stateMachine()->creatingInitialEmptyDocument() && !settings()->shouldInjectUserScriptsInInitialEmptyDocument())
         return;
-    
+
     // Walk the hashtable. Inject by world.
     const UserScriptMap* userScripts = m_page->group().userScripts();
     if (!userScripts)
@@ -551,46 +582,6 @@ void Frame::injectUserScriptsForWorld(DOMWrapperWorld* world, const UserScriptVe
     }
 }
 
-#ifndef NDEBUG
-static HashSet<Frame*>& keepAliveSet()
-{
-    DEFINE_STATIC_LOCAL(HashSet<Frame*>, staticKeepAliveSet, ());
-    return staticKeepAliveSet;
-}
-#endif
-
-void Frame::keepAlive()
-{
-    if (m_lifeSupportTimer.isActive())
-        return;
-#ifndef NDEBUG
-    keepAliveSet().add(this);
-#endif
-    ref();
-    m_lifeSupportTimer.startOneShot(0);
-}
-
-#ifndef NDEBUG
-void Frame::cancelAllKeepAlive()
-{
-    HashSet<Frame*>::iterator end = keepAliveSet().end();
-    for (HashSet<Frame*>::iterator it = keepAliveSet().begin(); it != end; ++it) {
-        Frame* frame = *it;
-        frame->m_lifeSupportTimer.stop();
-        frame->deref();
-    }
-    keepAliveSet().clear();
-}
-#endif
-
-void Frame::lifeSupportTimerFired(Timer<Frame>*)
-{
-#ifndef NDEBUG
-    keepAliveSet().remove(this);
-#endif
-    deref();
-}
-
 void Frame::clearDOMWindow()
 {
     if (m_domWindow) {
@@ -668,6 +659,16 @@ void Frame::setDOMWindow(DOMWindow* domWindow)
     m_domWindow = domWindow;
 }
 
+#if ENABLE(PAGE_VISIBILITY_API)
+void Frame::dispatchVisibilityStateChangeEvent()
+{
+    if (m_doc)
+        m_doc->dispatchVisibilityStateChangeEvent();
+    for (Frame* child = tree()->firstChild(); child; child = child->tree()->nextSibling())
+        child->dispatchVisibilityStateChangeEvent();
+}
+#endif
+
 DOMWindow* Frame::domWindow() const
 {
     if (!m_domWindow)
@@ -683,6 +684,9 @@ void Frame::clearFormerDOMWindow(DOMWindow* window)
 
 void Frame::pageDestroyed()
 {
+    // FIXME: Rename this function, since it's called not only from Page destructor, but in several other cases.
+    // This cleanup is needed whenever we remove a frame from page.
+
     if (Frame* parent = tree()->parent())
         parent->loader()->checkLoadComplete();
 
@@ -691,12 +695,16 @@ void Frame::pageDestroyed()
         m_domWindow->pageDestroyed();
     }
 
+#if ENABLE(MEDIA_STREAM)
+    if (m_mediaStreamFrameController)
+        m_mediaStreamFrameController->disconnectPage();
+#endif
+
     // FIXME: It's unclear as to why this is called more than once, but it is,
     // so page() could be NULL.
     if (page() && page()->focusController()->focusedFrame() == this)
         page()->focusController()->setFocusedFrame(0);
 
-    script()->clearWindowShell();
     script()->clearScriptObjects();
     script()->updatePlatformScriptObjects();
 
@@ -741,6 +749,10 @@ void Frame::transferChildFrameToNewDocument()
         if (m_domWindow)
             m_domWindow->resetGeolocation();
 
+#if ENABLE(MEDIA_STREAM)
+        if (m_mediaStreamFrameController)
+            m_mediaStreamFrameController->transferToNewPage(newPage);
+#endif
         m_page = newPage;
 
         if (newPage)
@@ -776,10 +788,15 @@ String Frame::documentTypeString() const
     return String();
 }
 
-VisiblePosition Frame::visiblePositionForPoint(const IntPoint& framePoint)
+String Frame::displayStringModifiedByEncoding(const String& str) const
+{
+    return document() ? document()->displayStringModifiedByEncoding(str) : str;
+}
+
+VisiblePosition Frame::visiblePositionForPoint(const LayoutPoint& framePoint)
 {
     HitTestResult result = eventHandler()->hitTestResultAtPoint(framePoint, true);
-    Node* node = result.innerNode();
+    Node* node = result.innerNonSharedNode();
     if (!node)
         return VisiblePosition();
     RenderObject* renderer = node->renderer();
@@ -791,12 +808,12 @@ VisiblePosition Frame::visiblePositionForPoint(const IntPoint& framePoint)
     return visiblePos;
 }
 
-Document* Frame::documentAtPoint(const IntPoint& point)
+Document* Frame::documentAtPoint(const LayoutPoint& point)
 {
     if (!view())
         return 0;
 
-    IntPoint pt = view()->windowToContents(point);
+    LayoutPoint pt = view()->windowToContents(point);
     HitTestResult result = HitTestResult(pt);
 
     if (contentRenderer())
@@ -804,6 +821,30 @@ Document* Frame::documentAtPoint(const IntPoint& point)
     return result.innerNode() ? result.innerNode()->document() : 0;
 }
 
+PassRefPtr<Range> Frame::rangeForPoint(const LayoutPoint& framePoint)
+{
+    VisiblePosition position = visiblePositionForPoint(framePoint);
+    if (position.isNull())
+        return 0;
+
+    VisiblePosition previous = position.previous();
+    if (previous.isNotNull()) {
+        RefPtr<Range> previousCharacterRange = makeRange(previous, position);
+        LayoutRect rect = editor()->firstRectForRange(previousCharacterRange.get());
+        if (rect.contains(framePoint))
+            return previousCharacterRange.release();
+    }
+
+    VisiblePosition next = position.next();
+    if (RefPtr<Range> nextCharacterRange = makeRange(position, next)) {
+        LayoutRect rect = editor()->firstRectForRange(nextCharacterRange.get());
+        if (rect.contains(framePoint))
+            return nextCharacterRange.release();
+    }
+
+    return 0;
+}
+
 void Frame::createView(const IntSize& viewportSize,
                        const Color& backgroundColor, bool transparent,
                        const IntSize& fixedLayoutSize, bool useFixedLayout,
@@ -854,7 +895,7 @@ void Frame::setTiledBackingStoreEnabled(bool enabled)
     }
     if (m_tiledBackingStore)
         return;
-    m_tiledBackingStore.set(new TiledBackingStore(this));
+    m_tiledBackingStore = adoptPtr(new TiledBackingStore(this));
     if (m_view)
         m_view->setPaintsEntireContents(true);
 }
@@ -906,7 +947,7 @@ Color Frame::tiledBackingStoreBackgroundColor() const
 }
 #endif
 
-String Frame::layerTreeAsText() const
+String Frame::layerTreeAsText(bool showDebugInfo) const
 {
 #if USE(ACCELERATED_COMPOSITING)
     document()->updateLayout();
@@ -914,7 +955,7 @@ String Frame::layerTreeAsText() const
     if (!contentRenderer())
         return String();
 
-    return contentRenderer()->compositor()->layerTreeAsText();
+    return contentRenderer()->compositor()->layerTreeAsText(showDebugInfo);
 #else
     return String();
 #endif
@@ -943,6 +984,8 @@ void Frame::setPageAndTextZoomFactors(float pageZoomFactor, float textZoomFactor
     if (!document)
         return;
 
+    m_editor.dismissCorrectionPanelAsIgnored();
+
 #if ENABLE(SVG)
     // Respect SVGs zoomAndPan="disabled" property in standalone SVG documents.
     // FIXME: How to handle compound documents + zoomAndPan="disabled"? Needs SVG WG clarification.
@@ -957,7 +1000,7 @@ void Frame::setPageAndTextZoomFactors(float pageZoomFactor, float textZoomFactor
     if (m_pageZoomFactor != pageZoomFactor) {
         if (FrameView* view = this->view()) {
             // Update the scroll position when doing a full page zoom, so the content stays in relatively the same position.
-            IntPoint scrollPosition = view->scrollPosition();
+            LayoutPoint scrollPosition = view->scrollPosition();
             float percentDifference = (pageZoomFactor / m_pageZoomFactor);
             view->setScrollPosition(IntPoint(scrollPosition.x() * percentDifference, scrollPosition.y() * percentDifference));
         }
@@ -975,42 +1018,42 @@ void Frame::setPageAndTextZoomFactors(float pageZoomFactor, float textZoomFactor
         if (document->renderer() && document->renderer()->needsLayout() && view->didFirstLayout())
             view->layout();
     }
+
+    if (page->mainFrame() == this)
+        page->backForward()->markPagesForFullStyleRecalc();
+}
+
+float Frame::pageScaleFactor() const
+{
+    Page* page = this->page();
+    if (!page)
+        return 1;
+    return page->pageScaleFactor();
 }
 
 #if USE(ACCELERATED_COMPOSITING)
-void Frame::updateContentsScale(float scale)
+void Frame::deviceOrPageScaleFactorChanged()
 {
     for (Frame* child = tree()->firstChild(); child; child = child->tree()->nextSibling())
-        child->updateContentsScale(scale);
+        child->deviceOrPageScaleFactorChanged();
 
     RenderView* root = contentRenderer();
     if (root && root->compositor())
-        root->compositor()->updateContentsScale(scale);
+        root->compositor()->deviceOrPageScaleFactorChanged();
 }
 #endif
-
-void Frame::scalePage(float scale, const IntPoint& origin)
+void Frame::notifyChromeClientWheelEventHandlerCountChanged() const
 {
-    Document* document = this->document();
-    if (!document)
-        return;
-
-    m_pageScaleFactor = scale;
-
-    if (document->renderer())
-        document->renderer()->setNeedsLayout(true);
-
-    document->recalcStyle(Node::Force);
-
-#if USE(ACCELERATED_COMPOSITING)
-    updateContentsScale(scale);
-#endif
-
-    if (FrameView* view = this->view()) {
-        if (document->renderer() && document->renderer()->needsLayout() && view->didFirstLayout())
-            view->layout();
-        view->setScrollPosition(origin);
+    // Ensure that this method is being called on the main frame of the page.
+    ASSERT(m_page && m_page->mainFrame() == this);
+    
+    unsigned count = 0;
+    for (const Frame* frame = this; frame; frame = frame->tree()->traverseNext()) {
+        if (frame->document())
+            count += frame->document()->wheelEventHandlerCount();
     }
+    
+    m_page->chrome()->client()->numWheelEventHandlersChanged(count);
 }
 
 } // namespace WebCore
