@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006, 2007, 2008, 2009 Apple Inc. All rights reserved.
+ * Copyright (C) 2006, 2007, 2008, 2009, 2011 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -31,6 +31,7 @@
 #include "CookieStorage.h"
 #include "DOMTimer.h"
 #include "Database.h"
+#include "Document.h"
 #include "Frame.h"
 #include "FrameTree.h"
 #include "FrameView.h"
@@ -45,22 +46,41 @@ using namespace std;
 
 namespace WebCore {
 
-static void setNeedsRecalcStyleInAllFrames(Page* page)
-{
-    for (Frame* frame = page->mainFrame(); frame; frame = frame->tree()->traverseNext())
-        frame->document()->styleSelectorChanged(DeferRecalcStyle);
-}
-
 static void setLoadsImagesAutomaticallyInAllFrames(Page* page)
 {
     for (Frame* frame = page->mainFrame(); frame; frame = frame->tree()->traverseNext())
         frame->document()->cachedResourceLoader()->setAutoLoadImages(page->settings()->loadsImagesAutomatically());
 }
 
+static inline void setGenericFontFamilyMap(ScriptFontFamilyMap& fontMap, const AtomicString& family, UScriptCode script, Page* page)
+{
+    ScriptFontFamilyMap::iterator it = fontMap.find(static_cast<int>(script));
+    if (it != fontMap.end() && it->second == family)
+        return;
+    fontMap.set(static_cast<int>(script), family);
+    page->setNeedsRecalcStyleInAllFrames();
+}
+
+static inline const AtomicString& getGenericFontFamilyForScript(const ScriptFontFamilyMap& fontMap, UScriptCode script)
+{
+    ScriptFontFamilyMap::const_iterator it = fontMap.find(static_cast<int>(script));
+    if (it != fontMap.end())
+        return it->second;
+    if (script != USCRIPT_COMMON)
+        return getGenericFontFamilyForScript(fontMap, USCRIPT_COMMON);
+    return emptyAtom;
+}
+
 #if USE(SAFARI_THEME)
 bool Settings::gShouldPaintNativeControls = true;
 #endif
 
+#if USE(AVFOUNDATION)
+bool Settings::gAVFoundationEnabled(false);
+#endif
+
+bool Settings::gMockScrollbarsEnabled = false;
+
 #if PLATFORM(WIN) || (OS(WINDOWS) && PLATFORM(WX))
 bool Settings::gShouldUseHighResolutionTimers = true;
 #endif
@@ -90,19 +110,27 @@ Settings::Settings(Page* page)
     : m_page(page)
     , m_editableLinkBehavior(EditableLinkDefaultBehavior)
     , m_textDirectionSubmenuInclusionBehavior(TextDirectionSubmenuAutomaticallyIncluded)
+    , m_passwordEchoDurationInSeconds(1)
     , m_minimumFontSize(0)
     , m_minimumLogicalFontSize(0)
     , m_defaultFontSize(0)
     , m_defaultFixedFontSize(0)
+    , m_validationMessageTimerMagnification(50)
+    , m_minimumAccelerated2dCanvasSize(128 * 128)
+    , m_layoutFallbackWidth(980)
+    , m_deviceDPI(240)
     , m_maximumDecodedImageSize(numeric_limits<size_t>::max())
+    , m_deviceWidth(480)
+    , m_deviceHeight(854)
 #if ENABLE(DOM_STORAGE)
     , m_sessionStorageQuota(StorageMap::noQuota)
 #endif
-    , m_pluginAllowedRunTime(numeric_limits<unsigned>::max())
     , m_editingBehaviorType(editingBehaviorTypeForPlatform())
+    , m_maximumHTMLParserDOMTreeDepth(defaultMaximumHTMLParserDOMTreeDepth)
     , m_isSpatialNavigationEnabled(false)
     , m_isJavaEnabled(false)
     , m_loadsImagesAutomatically(false)
+    , m_loadsSiteIconsIgnoringImageLoadingSetting(false)
     , m_privateBrowsingEnabled(false)
     , m_caretBrowsingEnabled(false)
     , m_areImagesEnabled(true)
@@ -124,11 +152,11 @@ Settings::Settings(Page* page)
     , m_needsKeyboardEventDisambiguationQuirks(false)
     , m_treatsAnyTextCSSLinkAsStylesheet(false)
     , m_needsLeopardMailQuirks(false)
-    , m_needsTigerMailQuirks(false)
     , m_isDOMPasteAllowed(false)
     , m_shrinksStandaloneImagesToFit(true)
     , m_usesPageCache(false)
     , m_showsURLsInToolTips(false)
+    , m_showsToolTipOverTruncatedText(false)
     , m_forceFTPDirectoryListings(false)
     , m_developerExtrasEnabled(false)
     , m_authorAndUserStylesEnabled(true)
@@ -162,6 +190,7 @@ Settings::Settings(Page* page)
     , m_openGLMultisamplingEnabled(true)
     , m_webAudioEnabled(false)
     , m_acceleratedCanvas2dEnabled(false)
+    , m_legacyAcceleratedCanvas2dEnabled(false)
     , m_loadDeferringEnabled(true)
     , m_tiledBackingStoreEnabled(false)
     , m_paginateDuringLayoutEnabled(false)
@@ -176,64 +205,94 @@ Settings::Settings(Page* page)
     , m_hyperlinkAuditingEnabled(false)
     , m_crossOriginCheckInGetMatchedCSSRulesDisabled(false)
     , m_forceCompositingMode(false)
+    , m_shouldInjectUserScriptsInInitialEmptyDocument(false)
+    , m_allowDisplayOfInsecureContent(true)
+    , m_allowRunningOfInsecureContent(true)
+#if ENABLE(SMOOTH_SCROLLING)
+    , m_scrollAnimatorEnabled(true)
+#endif
+#if ENABLE(WEB_SOCKETS)
+    , m_useHixie76WebSocketProtocol(true)
+#endif
+    , m_mediaPlaybackRequiresUserGesture(false)
+    , m_mediaPlaybackAllowsInline(true)
+    , m_passwordEchoEnabled(false)
+    , m_loadsImagesAutomaticallyTimer(this, &Settings::loadsImagesAutomaticallyTimerFired)
+    , m_zoomAnimatorScale(1)
 {
     // A Frame may not have been created yet, so we initialize the AtomicString 
     // hash before trying to use it.
     AtomicString::init();
 }
 
-void Settings::setStandardFontFamily(const AtomicString& standardFontFamily)
+const AtomicString& Settings::standardFontFamily(UScriptCode script) const
 {
-    if (standardFontFamily == m_standardFontFamily)
-        return;
-
-    m_standardFontFamily = standardFontFamily;
-    setNeedsRecalcStyleInAllFrames(m_page);
+    return getGenericFontFamilyForScript(m_standardFontFamilyMap, script);
 }
 
-void Settings::setFixedFontFamily(const AtomicString& fixedFontFamily)
+void Settings::setStandardFontFamily(const AtomicString& family, UScriptCode script)
 {
-    if (m_fixedFontFamily == fixedFontFamily)
-        return;
-        
-    m_fixedFontFamily = fixedFontFamily;
-    setNeedsRecalcStyleInAllFrames(m_page);
+    setGenericFontFamilyMap(m_standardFontFamilyMap, family, script, m_page);
 }
 
-void Settings::setSerifFontFamily(const AtomicString& serifFontFamily)
+const AtomicString& Settings::fixedFontFamily(UScriptCode script) const
 {
-    if (m_serifFontFamily == serifFontFamily)
-        return;
-        
-    m_serifFontFamily = serifFontFamily;
-    setNeedsRecalcStyleInAllFrames(m_page);
+    return getGenericFontFamilyForScript(m_fixedFontFamilyMap, script);
 }
 
-void Settings::setSansSerifFontFamily(const AtomicString& sansSerifFontFamily)
+void Settings::setFixedFontFamily(const AtomicString& family, UScriptCode script)
 {
-    if (m_sansSerifFontFamily == sansSerifFontFamily)
-        return;
-        
-    m_sansSerifFontFamily = sansSerifFontFamily; 
-    setNeedsRecalcStyleInAllFrames(m_page);
+    setGenericFontFamilyMap(m_fixedFontFamilyMap, family, script, m_page);
 }
 
-void Settings::setCursiveFontFamily(const AtomicString& cursiveFontFamily)
+const AtomicString& Settings::serifFontFamily(UScriptCode script) const
 {
-    if (m_cursiveFontFamily == cursiveFontFamily)
-        return;
-        
-    m_cursiveFontFamily = cursiveFontFamily;
-    setNeedsRecalcStyleInAllFrames(m_page);
+    return getGenericFontFamilyForScript(m_serifFontFamilyMap, script);
 }
 
-void Settings::setFantasyFontFamily(const AtomicString& fantasyFontFamily)
+void Settings::setSerifFontFamily(const AtomicString& family, UScriptCode script)
 {
-    if (m_fantasyFontFamily == fantasyFontFamily)
-        return;
-        
-    m_fantasyFontFamily = fantasyFontFamily;
-    setNeedsRecalcStyleInAllFrames(m_page);
+     setGenericFontFamilyMap(m_serifFontFamilyMap, family, script, m_page);
+}
+
+const AtomicString& Settings::sansSerifFontFamily(UScriptCode script) const
+{
+    return getGenericFontFamilyForScript(m_sansSerifFontFamilyMap, script);
+}
+
+void Settings::setSansSerifFontFamily(const AtomicString& family, UScriptCode script)
+{
+    setGenericFontFamilyMap(m_sansSerifFontFamilyMap, family, script, m_page);
+}
+
+const AtomicString& Settings::cursiveFontFamily(UScriptCode script) const
+{
+    return getGenericFontFamilyForScript(m_cursiveFontFamilyMap, script);
+}
+
+void Settings::setCursiveFontFamily(const AtomicString& family, UScriptCode script)
+{
+    setGenericFontFamilyMap(m_cursiveFontFamilyMap, family, script, m_page);
+}
+
+const AtomicString& Settings::fantasyFontFamily(UScriptCode script) const
+{
+    return getGenericFontFamilyForScript(m_fantasyFontFamilyMap, script);
+}
+
+void Settings::setFantasyFontFamily(const AtomicString& family, UScriptCode script)
+{
+    setGenericFontFamilyMap(m_fantasyFontFamilyMap, family, script, m_page);
+}
+
+const AtomicString& Settings::pictographFontFamily(UScriptCode script) const
+{
+    return getGenericFontFamilyForScript(m_pictographFontFamilyMap, script);
+}
+
+void Settings::setPictographFontFamily(const AtomicString& family, UScriptCode script)
+{
+    setGenericFontFamilyMap(m_pictographFontFamilyMap, family, script, m_page);
 }
 
 void Settings::setMinimumFontSize(int minimumFontSize)
@@ -242,7 +301,7 @@ void Settings::setMinimumFontSize(int minimumFontSize)
         return;
 
     m_minimumFontSize = minimumFontSize;
-    setNeedsRecalcStyleInAllFrames(m_page);
+    m_page->setNeedsRecalcStyleInAllFrames();
 }
 
 void Settings::setMinimumLogicalFontSize(int minimumLogicalFontSize)
@@ -251,7 +310,7 @@ void Settings::setMinimumLogicalFontSize(int minimumLogicalFontSize)
         return;
 
     m_minimumLogicalFontSize = minimumLogicalFontSize;
-    setNeedsRecalcStyleInAllFrames(m_page);
+    m_page->setNeedsRecalcStyleInAllFrames();
 }
 
 void Settings::setDefaultFontSize(int defaultFontSize)
@@ -260,7 +319,7 @@ void Settings::setDefaultFontSize(int defaultFontSize)
         return;
 
     m_defaultFontSize = defaultFontSize;
-    setNeedsRecalcStyleInAllFrames(m_page);
+    m_page->setNeedsRecalcStyleInAllFrames();
 }
 
 void Settings::setDefaultFixedFontSize(int defaultFontSize)
@@ -269,15 +328,33 @@ void Settings::setDefaultFixedFontSize(int defaultFontSize)
         return;
 
     m_defaultFixedFontSize = defaultFontSize;
-    setNeedsRecalcStyleInAllFrames(m_page);
+    m_page->setNeedsRecalcStyleInAllFrames();
 }
 
 void Settings::setLoadsImagesAutomatically(bool loadsImagesAutomatically)
 {
     m_loadsImagesAutomatically = loadsImagesAutomatically;
+    
+    // Changing this setting to true might immediately start new loads for images that had previously had loading disabled.
+    // If this happens while a WebView is being dealloc'ed, and we don't know the WebView is being dealloc'ed, these new loads
+    // can cause crashes downstream when the WebView memory has actually been free'd.
+    // One example where this can happen is in Mac apps that subclass WebView then do work in their overridden dealloc methods.
+    // Starting these loads synchronously is not important.  By putting it on a 0-delay, properly closing the Page cancels them
+    // before they have a chance to really start.
+    // See http://webkit.org/b/60572 for more discussion.
+    m_loadsImagesAutomaticallyTimer.startOneShot(0);
+}
+
+void Settings::loadsImagesAutomaticallyTimerFired(Timer<Settings>*)
+{
     setLoadsImagesAutomaticallyInAllFrames(m_page);
 }
 
+void Settings::setLoadsSiteIconsIgnoringImageLoadingSetting(bool loadsSiteIcons)
+{
+    m_loadsSiteIconsIgnoringImageLoadingSetting = loadsSiteIcons;
+}
+
 void Settings::setJavaScriptEnabled(bool isJavaScriptEnabled)
 {
     m_isJavaScriptEnabled = isJavaScriptEnabled;
@@ -337,16 +414,26 @@ void Settings::setSessionStorageQuota(unsigned sessionStorageQuota)
 
 void Settings::setPrivateBrowsingEnabled(bool privateBrowsingEnabled)
 {
-    if (m_privateBrowsingEnabled == privateBrowsingEnabled)
-        return;
-
+    // FIXME http://webkit.org/b/67870: The private browsing storage session and cookie private
+    // browsing mode (which is used if storage sessions are not available) are global settings, so
+    // it is misleading to have them as per-page settings.
+    // In addition, if they are treated as a per Page settings, the global values can get out of
+    // sync with the per Page value in the following situation:
+    // 1. The global values get set to true when setPrivateBrowsingEnabled(true) is called.
+    // 2. All Pages are closed, so all Settings objects go away.
+    // 3. A new Page is created, and a corresponding new Settings object is created - with
+    //    m_privateBrowsingEnabled starting out as false in the constructor.
+    // 4. The WebPage settings get applied to the new Page and setPrivateBrowsingEnabled(false)
+    //    is called, but an if (m_privateBrowsingEnabled == privateBrowsingEnabled) early return
+    //    prevents the global values from getting changed from true to false.
 #if USE(CFURLSTORAGESESSIONS)
     ResourceHandle::setPrivateBrowsingEnabled(privateBrowsingEnabled);
 #endif
-
-    // FIXME: We can only enable cookie private browsing mode globally, so it's misleading to have it as a per-page setting.
     setCookieStoragePrivateBrowsingEnabled(privateBrowsingEnabled);
 
+    if (m_privateBrowsingEnabled == privateBrowsingEnabled)
+        return;
+
     m_privateBrowsingEnabled = privateBrowsingEnabled;
     m_page->privateBrowsingStateChanged();
 }
@@ -387,7 +474,7 @@ void Settings::setTextAreasAreResizable(bool textAreasAreResizable)
         return;
 
     m_textAreasAreResizable = textAreasAreResizable;
-    setNeedsRecalcStyleInAllFrames(m_page);
+    m_page->setNeedsRecalcStyleInAllFrames();
 }
 
 void Settings::setEditableLinkBehavior(EditableLinkBehavior editableLinkBehavior)
@@ -432,11 +519,6 @@ void Settings::setNeedsLeopardMailQuirks(bool needsQuirks)
     m_needsLeopardMailQuirks = needsQuirks;
 }
 
-void Settings::setNeedsTigerMailQuirks(bool needsQuirks)
-{
-    m_needsTigerMailQuirks = needsQuirks;
-}
-    
 void Settings::setDOMPasteAllowed(bool DOMPasteAllowed)
 {
     m_isDOMPasteAllowed = DOMPasteAllowed;
@@ -487,6 +569,11 @@ void Settings::setShowsURLsInToolTips(bool showsURLsInToolTips)
     m_showsURLsInToolTips = showsURLsInToolTips;
 }
 
+void Settings::setShowsToolTipOverTruncatedText(bool showsToolTipForTruncatedText)
+{
+    m_showsToolTipOverTruncatedText = showsToolTipForTruncatedText;
+}
+
 void Settings::setFTPDirectoryTemplatePath(const String& path)
 {
     m_ftpDirectoryTemplatePath = path;
@@ -508,7 +595,7 @@ void Settings::setAuthorAndUserStylesEnabled(bool authorAndUserStylesEnabled)
         return;
 
     m_authorAndUserStylesEnabled = authorAndUserStylesEnabled;
-    setNeedsRecalcStyleInAllFrames(m_page);
+    m_page->setNeedsRecalcStyleInAllFrames();
 }
 
 void Settings::setFontRenderingMode(FontRenderingMode mode)
@@ -516,7 +603,7 @@ void Settings::setFontRenderingMode(FontRenderingMode mode)
     if (fontRenderingMode() == mode)
         return;
     m_fontRenderingMode = mode;
-    setNeedsRecalcStyleInAllFrames(m_page);
+    m_page->setNeedsRecalcStyleInAllFrames();
 }
 
 FontRenderingMode Settings::fontRenderingMode() const
@@ -618,7 +705,7 @@ void Settings::setAcceleratedCompositingEnabled(bool enabled)
         return;
         
     m_acceleratedCompositingEnabled = enabled;
-    setNeedsRecalcStyleInAllFrames(m_page);
+    m_page->setNeedsRecalcStyleInAllFrames();
 }
 
 void Settings::setCanvasUsesAcceleratedDrawing(bool enabled)
@@ -662,7 +749,7 @@ void Settings::setShowDebugBorders(bool enabled)
         return;
         
     m_showDebugBorders = enabled;
-    setNeedsRecalcStyleInAllFrames(m_page);
+    m_page->setNeedsRecalcStyleInAllFrames();
 }
 
 void Settings::setShowRepaintCounter(bool enabled)
@@ -671,7 +758,7 @@ void Settings::setShowRepaintCounter(bool enabled)
         return;
         
     m_showRepaintCounter = enabled;
-    setNeedsRecalcStyleInAllFrames(m_page);
+    m_page->setNeedsRecalcStyleInAllFrames();
 }
 
 void Settings::setExperimentalNotificationsEnabled(bool enabled)
@@ -679,12 +766,6 @@ void Settings::setExperimentalNotificationsEnabled(bool enabled)
     m_experimentalNotificationsEnabled = enabled;
 }
 
-void Settings::setPluginAllowedRunTime(unsigned runTime)
-{
-    m_pluginAllowedRunTime = runTime;
-    m_page->pluginAllowedRunTimeChanged();
-}
-
 #if PLATFORM(WIN) || (OS(WINDOWS) && PLATFORM(WX))
 void Settings::setShouldUseHighResolutionTimers(bool shouldUseHighResolutionTimers)
 {
@@ -712,6 +793,16 @@ void Settings::setAccelerated2dCanvasEnabled(bool enabled)
     m_acceleratedCanvas2dEnabled = enabled;
 }
 
+void Settings::setLegacyAccelerated2dCanvasEnabled(bool enabled)
+{
+    m_legacyAcceleratedCanvas2dEnabled = enabled;
+}
+
+void Settings::setMinimumAccelerated2dCanvasSize(int numPixels)
+{
+    m_minimumAccelerated2dCanvasSize = numPixels;
+}
+
 void Settings::setLoadDeferringEnabled(bool enabled)
 {
     m_loadDeferringEnabled = enabled;
@@ -726,4 +817,14 @@ void Settings::setTiledBackingStoreEnabled(bool enabled)
 #endif
 }
 
+void Settings::setMockScrollbarsEnabled(bool flag)
+{
+    gMockScrollbarsEnabled = flag;
+}
+
+bool Settings::mockScrollbarsEnabled()
+{
+    return gMockScrollbarsEnabled;
+}
+
 } // namespace WebCore
