@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003, 2006, 2007, 2008, 2009 Apple Inc. All rights reserved.
+ * Copyright (C) 2003, 2006, 2007, 2008, 2009, 2011 Apple Inc. All rights reserved.
  *           (C) 2006 Graham Dennis (graham.dennis@gmail.com)
  *
  * Redistribution and use in source and binary forms, with or without
@@ -30,7 +30,11 @@
 #include "EditingBehaviorTypes.h"
 #include "FontRenderingMode.h"
 #include "KURL.h"
+#include "Timer.h"
+#include <wtf/HashMap.h>
 #include <wtf/text/AtomicString.h>
+#include <wtf/text/AtomicStringHash.h>
+#include <wtf/unicode/Unicode.h>
 
 namespace WebCore {
 
@@ -50,28 +54,42 @@ namespace WebCore {
         TextDirectionSubmenuAlwaysIncluded
     };
 
+    // UScriptCode uses -1 and 0 for UScriptInvalidCode and UScriptCommon.
+    // We need to use -2 and -3 for empty value and deleted value.
+    struct UScriptCodeHashTraits : WTF::GenericHashTraits<int> {
+        static const bool emptyValueIsZero = false;
+        static int emptyValue() { return -2; }
+        static void constructDeletedValue(int& slot) { slot = -3; }
+        static bool isDeletedValue(int value) { return value == -3; }
+    };
+
+    typedef HashMap<int, AtomicString, DefaultHash<int>::Hash, UScriptCodeHashTraits> ScriptFontFamilyMap;
+
     class Settings {
         WTF_MAKE_NONCOPYABLE(Settings); WTF_MAKE_FAST_ALLOCATED;
     public:
         Settings(Page*);
 
-        void setStandardFontFamily(const AtomicString&);
-        const AtomicString& standardFontFamily() const { return m_standardFontFamily; }
+        void setStandardFontFamily(const AtomicString&, UScriptCode = USCRIPT_COMMON);
+        const AtomicString& standardFontFamily(UScriptCode = USCRIPT_COMMON) const;
 
-        void setFixedFontFamily(const AtomicString&);
-        const AtomicString& fixedFontFamily() const { return m_fixedFontFamily; }
+        void setFixedFontFamily(const AtomicString&, UScriptCode = USCRIPT_COMMON);
+        const AtomicString& fixedFontFamily(UScriptCode = USCRIPT_COMMON) const;
 
-        void setSerifFontFamily(const AtomicString&);
-        const AtomicString& serifFontFamily() const { return m_serifFontFamily; }
+        void setSerifFontFamily(const AtomicString&, UScriptCode = USCRIPT_COMMON);
+        const AtomicString& serifFontFamily(UScriptCode = USCRIPT_COMMON) const;
 
-        void setSansSerifFontFamily(const AtomicString&);
-        const AtomicString& sansSerifFontFamily() const { return m_sansSerifFontFamily; }
+        void setSansSerifFontFamily(const AtomicString&, UScriptCode = USCRIPT_COMMON);
+        const AtomicString& sansSerifFontFamily(UScriptCode = USCRIPT_COMMON) const;
 
-        void setCursiveFontFamily(const AtomicString&);
-        const AtomicString& cursiveFontFamily() const { return m_cursiveFontFamily; }
+        void setCursiveFontFamily(const AtomicString&, UScriptCode = USCRIPT_COMMON);
+        const AtomicString& cursiveFontFamily(UScriptCode = USCRIPT_COMMON) const;
 
-        void setFantasyFontFamily(const AtomicString&);
-        const AtomicString& fantasyFontFamily() const { return m_fantasyFontFamily; }
+        void setFantasyFontFamily(const AtomicString&, UScriptCode = USCRIPT_COMMON);
+        const AtomicString& fantasyFontFamily(UScriptCode = USCRIPT_COMMON) const;
+
+        void setPictographFontFamily(const AtomicString&, UScriptCode = USCRIPT_COMMON);
+        const AtomicString& pictographFontFamily(UScriptCode = USCRIPT_COMMON) const;
 
         void setMinimumFontSize(int);
         int minimumFontSize() const { return m_minimumFontSize; }
@@ -90,6 +108,11 @@ namespace WebCore {
         void setLoadsImagesAutomatically(bool);
         bool loadsImagesAutomatically() const { return m_loadsImagesAutomatically; }
 
+        // This setting only affects site icon image loading if loadsImagesAutomatically setting is false and this setting is true.
+        // All other permutations still heed loadsImagesAutomatically setting.
+        void setLoadsSiteIconsIgnoringImageLoadingSetting(bool);
+        bool loadsSiteIconsIgnoringImageLoadingSetting() const { return m_loadsSiteIconsIgnoringImageLoadingSetting; }
+
         void setJavaScriptEnabled(bool);
         // Instead of calling isJavaScriptEnabled directly, please consider calling
         // ScriptController::canExecuteScripts, which takes things like the
@@ -197,9 +220,6 @@ namespace WebCore {
         void setNeedsLeopardMailQuirks(bool);
         bool needsLeopardMailQuirks() const { return m_needsLeopardMailQuirks; }
 
-        void setNeedsTigerMailQuirks(bool);
-        bool needsTigerMailQuirks() const { return m_needsTigerMailQuirks; }
-
         void setDOMPasteAllowed(bool);
         bool isDOMPasteAllowed() const { return m_isDOMPasteAllowed; }
         
@@ -218,6 +238,9 @@ namespace WebCore {
         void setShowsURLsInToolTips(bool);
         bool showsURLsInToolTips() const { return m_showsURLsInToolTips; }
 
+        void setShowsToolTipOverTruncatedText(bool);
+        bool showsToolTipOverTruncatedText() const { return m_showsToolTipOverTruncatedText; }
+
         void setFTPDirectoryTemplatePath(const String&);
         const String& ftpDirectoryTemplatePath() const { return m_ftpDirectoryTemplatePath; }
         
@@ -265,12 +288,6 @@ namespace WebCore {
         void setMaximumDecodedImageSize(size_t size) { m_maximumDecodedImageSize = size; }
         size_t maximumDecodedImageSize() const { return m_maximumDecodedImageSize; }
 
-#if USE(SAFARI_THEME)
-        // Windows debugging pref (global) for switching between the Aqua look and a native windows look.
-        static void setShouldPaintNativeControls(bool);
-        static bool shouldPaintNativeControls() { return gShouldPaintNativeControls; }
-#endif
-
         void setAllowScriptsToCloseWindows(bool);
         bool allowScriptsToCloseWindows() const { return m_allowScriptsToCloseWindows; }
 
@@ -321,9 +338,6 @@ namespace WebCore {
         static bool shouldUseHighResolutionTimers() { return gShouldUseHighResolutionTimers; }
 #endif
 
-        void setPluginAllowedRunTime(unsigned);
-        unsigned pluginAllowedRunTime() const { return m_pluginAllowedRunTime; }
-
         void setWebAudioEnabled(bool);
         bool webAudioEnabled() const { return m_webAudioEnabled; }
 
@@ -336,6 +350,20 @@ namespace WebCore {
         void setAccelerated2dCanvasEnabled(bool);
         bool accelerated2dCanvasEnabled() const { return m_acceleratedCanvas2dEnabled; }
 
+        void setLegacyAccelerated2dCanvasEnabled(bool);
+        bool legacyAccelerated2dCanvasEnabled() const { return m_legacyAcceleratedCanvas2dEnabled; }
+
+        // Number of pixels below which 2D canvas is rendered in software
+        // even if hardware acceleration is enabled.
+        // Hardware acceleration is useful for large canvases where it can avoid the
+        // pixel bandwidth between the CPU and GPU. But GPU acceleration comes at
+        // a price - extra back-buffer and texture copy. Small canvases are also
+        // widely used for stylized fonts. Anti-aliasing text in hardware at that
+        // scale is generally slower. So below a certain size it is better to
+        // draw canvas in software.
+        void setMinimumAccelerated2dCanvasSize(int);
+        int minimumAccelerated2dCanvasSize() const { return m_minimumAccelerated2dCanvasSize; }
+
         void setLoadDeferringEnabled(bool);
         bool loadDeferringEnabled() const { return m_loadDeferringEnabled; }
         
@@ -350,6 +378,11 @@ namespace WebCore {
         bool fullScreenEnabled() const  { return m_fullScreenAPIEnabled; }
 #endif
 
+#if USE(AVFOUNDATION)
+        static void setAVFoundationEnabled(bool flag) { gAVFoundationEnabled = flag; }
+        static bool isAVFoundationEnabled() { return gAVFoundationEnabled; }
+#endif
+
         void setAsynchronousSpellCheckingEnabled(bool flag) { m_asynchronousSpellCheckingEnabled = flag; }
         bool asynchronousSpellCheckingEnabled() const  { return m_asynchronousSpellCheckingEnabled; }
 
@@ -362,19 +395,84 @@ namespace WebCore {
         // and http://webkit.org/b/40908
         void setInteractiveFormValidationEnabled(bool flag) { m_interactiveFormValidation = flag; }
         bool interactiveFormValidationEnabled() const { return m_interactiveFormValidation; }
+
+        // Sets the maginication value for validation message timer.
+        // If the maginication value is N, a validation message disappears
+        // automatically after <message length> * N / 1000 seconds. If N is
+        // equal to or less than 0, a validation message doesn't disappears
+        // automaticaly. The default value is 50.
+        void setValidationMessageTimerMagnification(int newValue) { m_validationMessageTimerMagnification = newValue; }
+        int validationMessageTimerMaginification() const { return m_validationMessageTimerMagnification; }
         
         void setUsePreHTML5ParserQuirks(bool flag) { m_usePreHTML5ParserQuirks = flag; }
         bool usePreHTML5ParserQuirks() const { return m_usePreHTML5ParserQuirks; }
 
+        static const unsigned defaultMaximumHTMLParserDOMTreeDepth = 512;
+        void setMaximumHTMLParserDOMTreeDepth(unsigned maximumHTMLParserDOMTreeDepth) { m_maximumHTMLParserDOMTreeDepth = maximumHTMLParserDOMTreeDepth; }
+        unsigned maximumHTMLParserDOMTreeDepth() const { return m_maximumHTMLParserDOMTreeDepth; }
+
         void setHyperlinkAuditingEnabled(bool flag) { m_hyperlinkAuditingEnabled = flag; }
         bool hyperlinkAuditingEnabled() const { return m_hyperlinkAuditingEnabled; }
 
         void setCrossOriginCheckInGetMatchedCSSRulesDisabled(bool flag) { m_crossOriginCheckInGetMatchedCSSRulesDisabled = flag; }
         bool crossOriginCheckInGetMatchedCSSRulesDisabled() const { return m_crossOriginCheckInGetMatchedCSSRulesDisabled; }
+        
+        void setLayoutFallbackWidth(int width) { m_layoutFallbackWidth = width; }
+        int layoutFallbackWidth() const { return m_layoutFallbackWidth; }
+
+        void setDeviceWidth(int width) { m_deviceWidth = width; }
+        int deviceWidth() const { return m_deviceWidth; }
+
+        void setDeviceHeight(int height) { m_deviceHeight = height; }
+        int deviceHeight() const { return m_deviceHeight; }
+
+        void setDeviceDPI(int deviceDPI) { m_deviceDPI = deviceDPI; }
+        int deviceDPI() const { return m_deviceDPI; }
 
         void setForceCompositingMode(bool flag) { m_forceCompositingMode = flag; }
         bool forceCompositingMode() { return m_forceCompositingMode; }
 
+        void setZoomAnimatorScale(double scale) { m_zoomAnimatorScale = scale; }
+        double zoomAnimatorScale() { return m_zoomAnimatorScale; }
+
+        void setShouldInjectUserScriptsInInitialEmptyDocument(bool flag) { m_shouldInjectUserScriptsInInitialEmptyDocument = flag; }
+        bool shouldInjectUserScriptsInInitialEmptyDocument() { return m_shouldInjectUserScriptsInInitialEmptyDocument; }
+
+        void setAllowDisplayOfInsecureContent(bool flag) { m_allowDisplayOfInsecureContent = flag; }
+        bool allowDisplayOfInsecureContent() const { return m_allowDisplayOfInsecureContent; }
+        void setAllowRunningOfInsecureContent(bool flag) { m_allowRunningOfInsecureContent = flag; }
+        bool allowRunningOfInsecureContent() const { return m_allowRunningOfInsecureContent; }
+
+#if ENABLE(SMOOTH_SCROLLING)
+        void setEnableScrollAnimator(bool flag) { m_scrollAnimatorEnabled = flag; }
+        bool scrollAnimatorEnabled() const { return m_scrollAnimatorEnabled; }
+#endif
+#if ENABLE(WEB_SOCKETS)
+        void setUseHixie76WebSocketProtocol(bool flag) { m_useHixie76WebSocketProtocol = flag; }
+        bool useHixie76WebSocketProtocol() { return m_useHixie76WebSocketProtocol; }
+#endif
+
+        void setMediaPlaybackRequiresUserGesture(bool flag) { m_mediaPlaybackRequiresUserGesture = flag; };
+        bool mediaPlaybackRequiresUserGesture() const { return m_mediaPlaybackRequiresUserGesture; }
+
+        void setMediaPlaybackAllowsInline(bool flag) { m_mediaPlaybackAllowsInline = flag; };
+        bool mediaPlaybackAllowsInline() const { return m_mediaPlaybackAllowsInline; }
+
+        void setPasswordEchoEnabled(bool flag) { m_passwordEchoEnabled = flag; }
+        bool passwordEchoEnabled() const { return m_passwordEchoEnabled; }
+
+        void setPasswordEchoDurationInSeconds(double durationInSeconds) { m_passwordEchoDurationInSeconds = durationInSeconds; }
+        double passwordEchoDurationInSeconds() const { return m_passwordEchoDurationInSeconds; }
+
+#if USE(SAFARI_THEME)
+        // Windows debugging pref (global) for switching between the Aqua look and a native windows look.
+        static void setShouldPaintNativeControls(bool);
+        static bool shouldPaintNativeControls() { return gShouldPaintNativeControls; }
+#endif
+
+        static void setMockScrollbarsEnabled(bool flag);
+        static bool mockScrollbarsEnabled();
+
     private:
         Page* m_page;
 
@@ -382,27 +480,36 @@ namespace WebCore {
         String m_ftpDirectoryTemplatePath;
         String m_localStorageDatabasePath;
         KURL m_userStyleSheetLocation;
-        AtomicString m_standardFontFamily;
-        AtomicString m_fixedFontFamily;
-        AtomicString m_serifFontFamily;
-        AtomicString m_sansSerifFontFamily;
-        AtomicString m_cursiveFontFamily;
-        AtomicString m_fantasyFontFamily;
+        ScriptFontFamilyMap m_standardFontFamilyMap;
+        ScriptFontFamilyMap m_serifFontFamilyMap;
+        ScriptFontFamilyMap m_fixedFontFamilyMap;
+        ScriptFontFamilyMap m_sansSerifFontFamilyMap;
+        ScriptFontFamilyMap m_cursiveFontFamilyMap;
+        ScriptFontFamilyMap m_fantasyFontFamilyMap;
+        ScriptFontFamilyMap m_pictographFontFamilyMap;
         EditableLinkBehavior m_editableLinkBehavior;
         TextDirectionSubmenuInclusionBehavior m_textDirectionSubmenuInclusionBehavior;
+        double m_passwordEchoDurationInSeconds;
         int m_minimumFontSize;
         int m_minimumLogicalFontSize;
         int m_defaultFontSize;
         int m_defaultFixedFontSize;
+        int m_validationMessageTimerMagnification;
+        int m_minimumAccelerated2dCanvasSize;
+        int m_layoutFallbackWidth;
+        int m_deviceDPI;
         size_t m_maximumDecodedImageSize;
+        int m_deviceWidth;
+        int m_deviceHeight;
 #if ENABLE(DOM_STORAGE)
         unsigned m_sessionStorageQuota;
 #endif
-        unsigned m_pluginAllowedRunTime;
         unsigned m_editingBehaviorType;
+        unsigned m_maximumHTMLParserDOMTreeDepth;
         bool m_isSpatialNavigationEnabled : 1;
         bool m_isJavaEnabled : 1;
         bool m_loadsImagesAutomatically : 1;
+        bool m_loadsSiteIconsIgnoringImageLoadingSetting : 1;
         bool m_privateBrowsingEnabled : 1;
         bool m_caretBrowsingEnabled : 1;
         bool m_areImagesEnabled : 1;
@@ -424,11 +531,11 @@ namespace WebCore {
         bool m_needsKeyboardEventDisambiguationQuirks : 1;
         bool m_treatsAnyTextCSSLinkAsStylesheet : 1;
         bool m_needsLeopardMailQuirks : 1;
-        bool m_needsTigerMailQuirks : 1;
         bool m_isDOMPasteAllowed : 1;
         bool m_shrinksStandaloneImagesToFit : 1;
         bool m_usesPageCache: 1;
         bool m_showsURLsInToolTips : 1;
+        bool m_showsToolTipOverTruncatedText : 1;
         bool m_forceFTPDirectoryListings : 1;
         bool m_developerExtrasEnabled : 1;
         bool m_authorAndUserStylesEnabled : 1;
@@ -460,6 +567,7 @@ namespace WebCore {
         bool m_openGLMultisamplingEnabled : 1;
         bool m_webAudioEnabled : 1;
         bool m_acceleratedCanvas2dEnabled : 1;
+        bool m_legacyAcceleratedCanvas2dEnabled : 1;
         bool m_loadDeferringEnabled : 1;
         bool m_tiledBackingStoreEnabled : 1;
         bool m_paginateDuringLayoutEnabled : 1;
@@ -474,6 +582,28 @@ namespace WebCore {
         bool m_hyperlinkAuditingEnabled : 1;
         bool m_crossOriginCheckInGetMatchedCSSRulesDisabled : 1;
         bool m_forceCompositingMode : 1;
+        bool m_shouldInjectUserScriptsInInitialEmptyDocument : 1;
+        bool m_allowDisplayOfInsecureContent : 1;
+        bool m_allowRunningOfInsecureContent : 1;
+#if ENABLE(SMOOTH_SCROLLING)
+        bool m_scrollAnimatorEnabled : 1;
+#endif
+#if ENABLE(WEB_SOCKETS)
+        bool m_useHixie76WebSocketProtocol : 1;
+#endif
+        bool m_mediaPlaybackRequiresUserGesture : 1;
+        bool m_mediaPlaybackAllowsInline : 1;
+        bool m_passwordEchoEnabled : 1;
+
+        Timer<Settings> m_loadsImagesAutomaticallyTimer;
+        void loadsImagesAutomaticallyTimerFired(Timer<Settings>*);
+
+#if USE(AVFOUNDATION)
+        static bool gAVFoundationEnabled;
+#endif
+        static bool gMockScrollbarsEnabled;
+
+        double m_zoomAnimatorScale;
 
 #if USE(SAFARI_THEME)
         static bool gShouldPaintNativeControls;
