@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004, 2007, 2008 Apple Inc. All rights reserved.
+ * Copyright (C) 2004, 2007, 2008, 2011 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -24,13 +24,16 @@
  */
 
 #include "config.h"
-
 #include "KURL.h"
 
+#include "DecodeEscapeSequences.h"
 #include "TextEncoding.h"
-#include <wtf/text/CString.h>
+#include <stdio.h>
 #include <wtf/HashMap.h>
+#include <wtf/HexNumber.h>
 #include <wtf/StdLibExtras.h>
+#include <wtf/text/CString.h>
+#include <wtf/text/StringBuilder.h>
 #include <wtf/text/StringHash.h>
 
 #if USE(ICU_UNICODE)
@@ -42,7 +45,9 @@
 #include "GOwnPtr.h"
 #endif
 
-#include <stdio.h>
+// FIXME: This file makes too much use of the + operator on String.
+// We either have to optimize that operator so it doesn't involve
+// so many allocations, or change this to use StringBuffer instead.
 
 using namespace std;
 using namespace WTF;
@@ -55,11 +60,19 @@ typedef Vector<UChar, 512> UCharBuffer;
 static const unsigned maximumValidPortNumber = 0xFFFE;
 static const unsigned invalidPortNumber = 0xFFFF;
 
+static inline bool isLetterMatchIgnoringCase(UChar character, char lowercaseLetter)
+{
+    ASSERT(isASCIILower(lowercaseLetter));
+    return (character | 0x20) == lowercaseLetter;
+}
+
 #if !USE(GOOGLEURL)
 
-// FIXME: This file makes too much use of the + operator on String.
-// We either have to optimize that operator so it doesn't involve
-// so many allocations, or change this to use Vector<UChar> instead.
+static inline bool isLetterMatchIgnoringCase(char character, char lowercaseLetter)
+{
+    ASSERT(isASCIILower(lowercaseLetter));
+    return (character | 0x20) == lowercaseLetter;
+}
 
 enum URLCharacterClasses {
     // alpha 
@@ -89,8 +102,6 @@ enum URLCharacterClasses {
     BadChar = 1 << 6
 };
 
-static const char hexDigits[17] = "0123456789ABCDEF";
-
 static const unsigned char characterClassTable[256] = {
     /* 0 nul */ PathSegmentEndChar,    /* 1 soh */ BadChar,
     /* 2 stx */ BadChar,    /* 3 etx */ BadChar,
@@ -232,13 +243,13 @@ static inline bool isIPv6Char(unsigned char c) { return characterClassTable[c] &
 static inline bool isPathSegmentEndChar(char c) { return characterClassTable[static_cast<unsigned char>(c)] & PathSegmentEndChar; }
 static inline bool isPathSegmentEndChar(UChar c) { return c <= 0xff && (characterClassTable[c] & PathSegmentEndChar); }
 static inline bool isBadChar(unsigned char c) { return characterClassTable[c] & BadChar; }
-
-static inline int hexDigitValue(UChar c)
+    
+static inline bool isSchemeCharacterMatchIgnoringCase(char character, char schemeCharacter)
 {
-    ASSERT(isASCIIHexDigit(c));
-    if (c < 'A')
-        return c - '0';
-    return (c - 'A' + 10) & 0xF; // handle both upper and lower case without a branch
+    ASSERT(isSchemeChar(character));
+    ASSERT(schemeCharacter & 0x20);
+    ASSERT(isASCIILower(schemeCharacter) || (!isASCIIUpper(schemeCharacter) && isSchemeChar(schemeCharacter)));
+    return (character | 0x20) == schemeCharacter;
 }
 
 // Copies the source to the destination, assuming all the source characters are
@@ -296,7 +307,7 @@ inline bool KURL::protocolIs(const String& string, const char* protocol)
 void KURL::invalidate()
 {
     m_isValid = false;
-    m_protocolInHTTPFamily = false;
+    m_protocolIsInHTTPFamily = false;
     m_schemeEnd = 0;
     m_userStart = 0;
     m_userEnd = 0;
@@ -693,7 +704,7 @@ bool KURL::protocolIs(const char* protocol) const
 
     // Do the comparison without making a new string object.
     for (int i = 0; i < m_schemeEnd; ++i) {
-        if (!protocol[i] || toASCIILower(m_string[i]) != protocol[i])
+        if (!protocol[i] || !isSchemeCharacterMatchIgnoringCase(m_string[i], protocol[i]))
             return false;
     }
     return !protocol[m_schemeEnd]; // We should have consumed all characters in the argument.
@@ -860,118 +871,76 @@ void KURL::setPath(const String& s)
 
     // FIXME: encodeWithURLEscapeSequences does not correctly escape '#' and '?', so fragment and query parts
     // may be inadvertently affected.
-    parse(m_string.left(m_portEnd) + encodeWithURLEscapeSequences(s) + m_string.substring(m_pathEnd));
+    String path = s;
+    if (path.isEmpty() || path[0] != '/')
+        path = "/" + path;
+
+    parse(m_string.left(m_portEnd) + encodeWithURLEscapeSequences(path) + m_string.substring(m_pathEnd));
 }
 
-String KURL::prettyURL() const
+String KURL::deprecatedString() const
 {
     if (!m_isValid)
         return m_string;
 
-    Vector<UChar> result;
+    StringBuilder result;
 
-    append(result, protocol());
+    result.append(protocol());
     result.append(':');
 
-    Vector<UChar> authority;
+    StringBuilder authority;
 
     if (m_hostEnd != m_passwordEnd) {
         if (m_userEnd != m_userStart) {
-            append(authority, user());
+            authority.append(user());
             authority.append('@');
         }
-        append(authority, host());
+        authority.append(host());
         if (hasPort()) {
             authority.append(':');
-            append(authority, String::number(port()));
+            authority.append(String::number(port()));
         }
     }
 
     if (!authority.isEmpty()) {
         result.append('/');
         result.append('/');
-        result.append(authority);
+        result.append(authority.characters(), authority.length());
     } else if (protocolIs("file")) {
         result.append('/');
         result.append('/');
     }
 
-    append(result, path());
+    result.append(path());
 
     if (m_pathEnd != m_queryEnd) {
         result.append('?');
-        append(result, query());
+        result.append(query());
     }
 
     if (m_fragmentEnd != m_queryEnd) {
         result.append('#');
-        append(result, fragmentIdentifier());
+        result.append(fragmentIdentifier());
     }
 
-    return String::adopt(result);
+    return result.toString();
 }
 
-String decodeURLEscapeSequences(const String& str)
+String decodeURLEscapeSequences(const String& string)
 {
-    return decodeURLEscapeSequences(str, UTF8Encoding());
+    return decodeEscapeSequences<URLEscapeSequence>(string, UTF8Encoding());
 }
 
-String decodeURLEscapeSequences(const String& str, const TextEncoding& encoding)
+String decodeURLEscapeSequences(const String& string, const TextEncoding& encoding)
 {
-    Vector<UChar> result;
-
-    CharBuffer buffer;
-
-    unsigned length = str.length();
-    unsigned decodedPosition = 0;
-    unsigned searchPosition = 0;
-    size_t encodedRunPosition;
-    while ((encodedRunPosition = str.find('%', searchPosition)) != notFound) {
-        // Find the sequence of %-escape codes.
-        unsigned encodedRunEnd = encodedRunPosition;
-        while (length - encodedRunEnd >= 3
-                && str[encodedRunEnd] == '%'
-                && isASCIIHexDigit(str[encodedRunEnd + 1])
-                && isASCIIHexDigit(str[encodedRunEnd + 2]))
-            encodedRunEnd += 3;
-        searchPosition = encodedRunEnd;
-        if (encodedRunEnd == encodedRunPosition) {
-            ++searchPosition;
-            continue;
-        }
-
-        // Decode the %-escapes into bytes.
-        unsigned runLength = (encodedRunEnd - encodedRunPosition) / 3;
-        buffer.resize(runLength);
-        char* p = buffer.data();
-        const UChar* q = str.characters() + encodedRunPosition;
-        for (unsigned i = 0; i < runLength; ++i) {
-            *p++ = (hexDigitValue(q[1]) << 4) | hexDigitValue(q[2]);
-            q += 3;
-        }
-
-        // Decode the bytes into Unicode characters.
-        String decoded = (encoding.isValid() ? encoding : UTF8Encoding()).decode(buffer.data(), p - buffer.data());
-        if (decoded.isEmpty())
-            continue;
-
-        // Build up the string with what we just skipped and what we just decoded.
-        result.append(str.characters() + decodedPosition, encodedRunPosition - decodedPosition);
-        result.append(decoded.characters(), decoded.length());
-        decodedPosition = encodedRunEnd;
-    }
-
-    result.append(str.characters() + decodedPosition, length - decodedPosition);
-
-    return String::adopt(result);
+    return decodeEscapeSequences<URLEscapeSequence>(string, encoding);
 }
 
 // Caution: This function does not bounds check.
 static void appendEscapedChar(char*& buffer, unsigned char c)
 {
     *buffer++ = '%';
-    *buffer++ = hexDigits[c >> 4];
-    *buffer++ = hexDigits[c & 0xF];
+    placeByteAsHex(c, buffer);
 }
 
 static void appendEscapingBadChars(char*& buffer, const char* strStart, size_t length)
@@ -1083,11 +1052,6 @@ static inline bool hasSlashDotOrDotDot(const char* str)
     return false;
 }
 
-static inline bool matchLetter(char c, char lowercaseLetter)
-{
-    return (c | 0x20) == lowercaseLetter;
-}
-
 void KURL::parse(const String& string)
 {
     checkEncodedString(string);
@@ -1130,6 +1094,28 @@ static inline bool isDefaultPortForScheme(const char* port, size_t portLength, c
     return false;
 }
 
+static inline bool hostPortIsEmptyButCredentialsArePresent(int hostStart, int portEnd, char userEndChar)
+{
+    return userEndChar == '@' && hostStart == portEnd;
+}
+
+static bool isNonFileHierarchicalScheme(const char* scheme, size_t schemeLength)
+{
+    switch (schemeLength) {
+    case 2:
+        return equal("ws", 2, scheme, schemeLength);
+    case 3:
+        return equal("ftp", 3, scheme, schemeLength) || equal("wss", 3, scheme, schemeLength);
+    case 4:
+        return equal("http", 4, scheme, schemeLength);
+    case 5:
+        return equal("https", 5, scheme, schemeLength);
+    case 6:
+        return equal("gopher", 6, scheme, schemeLength);
+    }
+    return false;
+}
+
 void KURL::parse(const char* url, const String* originalString)
 {
     if (!url || url[0] == '\0') {
@@ -1166,25 +1152,29 @@ void KURL::parse(const char* url, const String* originalString)
     int portEnd;
 
     bool hierarchical = url[schemeEnd + 1] == '/';
+    bool hasSecondSlash = hierarchical && url[schemeEnd + 2] == '/';
 
     bool isFile = schemeEnd == 4
-        && matchLetter(url[0], 'f')
-        && matchLetter(url[1], 'i')
-        && matchLetter(url[2], 'l')
-        && matchLetter(url[3], 'e');
+        && isLetterMatchIgnoringCase(url[0], 'f')
+        && isLetterMatchIgnoringCase(url[1], 'i')
+        && isLetterMatchIgnoringCase(url[2], 'l')
+        && isLetterMatchIgnoringCase(url[3], 'e');
 
-    m_protocolInHTTPFamily = matchLetter(url[0], 'h')
-        && matchLetter(url[1], 't')
-        && matchLetter(url[2], 't')
-        && matchLetter(url[3], 'p')
-        && (url[4] == ':' || (matchLetter(url[4], 's') && url[5] == ':'));
+    m_protocolIsInHTTPFamily = isLetterMatchIgnoringCase(url[0], 'h')
+        && isLetterMatchIgnoringCase(url[1], 't')
+        && isLetterMatchIgnoringCase(url[2], 't')
+        && isLetterMatchIgnoringCase(url[3], 'p')
+        && (url[4] == ':' || (isLetterMatchIgnoringCase(url[4], 's') && url[5] == ':'));
 
-    if (hierarchical && url[schemeEnd + 2] == '/') {
+    if ((hierarchical && hasSecondSlash) || isNonFileHierarchicalScheme(url, schemeEnd)) {
         // The part after the scheme is either a net_path or an abs_path whose first path segment is empty.
         // Attempt to find an authority.
-
         // FIXME: Authority characters may be scanned twice, and it would be nice to be faster.
-        userStart += 2;
+
+        if (hierarchical)
+            userStart++;
+        if (hasSecondSlash)
+            userStart++;
         userEnd = userStart;
 
         int colonPos = 0;
@@ -1255,7 +1245,13 @@ void KURL::parse(const char* url, const String* originalString)
             return;
         }
 
-        if (userStart == portEnd && !m_protocolInHTTPFamily && !isFile) {
+        if (hostPortIsEmptyButCredentialsArePresent(hostStart, portEnd, url[userEnd])) {
+            // in this circumstance, act as if there is an erroneous hostname containing an '@'
+            userEnd = userStart;
+            hostStart = userEnd;
+        }
+
+        if (userStart == portEnd && !m_protocolIsInHTTPFamily && !isFile) {
             // No authority found, which means that this is not a net_path, but rather an abs_path whose first two
             // path segments are empty. For file, http and https only, an empty authority is allowed.
             userStart -= 2;
@@ -1310,15 +1306,15 @@ void KURL::parse(const char* url, const String* originalString)
     m_schemeEnd = p - buffer.data();
 
     bool hostIsLocalHost = portEnd - userStart == 9
-        && matchLetter(url[userStart], 'l')
-        && matchLetter(url[userStart+1], 'o')
-        && matchLetter(url[userStart+2], 'c')
-        && matchLetter(url[userStart+3], 'a')
-        && matchLetter(url[userStart+4], 'l')
-        && matchLetter(url[userStart+5], 'h')
-        && matchLetter(url[userStart+6], 'o')
-        && matchLetter(url[userStart+7], 's')
-        && matchLetter(url[userStart+8], 't');
+        && isLetterMatchIgnoringCase(url[userStart], 'l')
+        && isLetterMatchIgnoringCase(url[userStart+1], 'o')
+        && isLetterMatchIgnoringCase(url[userStart+2], 'c')
+        && isLetterMatchIgnoringCase(url[userStart+3], 'a')
+        && isLetterMatchIgnoringCase(url[userStart+4], 'l')
+        && isLetterMatchIgnoringCase(url[userStart+5], 'h')
+        && isLetterMatchIgnoringCase(url[userStart+6], 'o')
+        && isLetterMatchIgnoringCase(url[userStart+7], 's')
+        && isLetterMatchIgnoringCase(url[userStart+8], 't');
 
     // File URLs need a host part unless it is just file:// or file://localhost
     bool degenFilePath = pathStart == pathEnd && (hostStart == hostEnd || hostIsLocalHost);
@@ -1381,8 +1377,8 @@ void KURL::parse(const char* url, const String* originalString)
         m_userStart = m_userEnd = m_passwordEnd = m_hostEnd = m_portEnd = p - buffer.data();
 
     // For canonicalization, ensure we have a '/' for no path.
-    // Do this only for hierarchical URL with protocol http or https.
-    if (m_protocolInHTTPFamily && hierarchical && pathEnd == pathStart)
+    // Do this only for URL with protocol http or https.
+    if (m_protocolIsInHTTPFamily && pathEnd == pathStart)
         *p++ = '/';
 
     // add path, escaping bad characters
@@ -1739,7 +1735,7 @@ bool protocolIs(const String& url, const char* protocol)
     for (int i = 0; ; ++i) {
         if (!protocol[i])
             return url[i] == ':';
-        if (toASCIILower(url[i]) != protocol[i])
+        if (!isLetterMatchIgnoringCase(url[i], protocol[i]))
             return false;
     }
 }
@@ -1931,4 +1927,17 @@ String mimeTypeFromDataURL(const String& url)
     return "";
 }
 
+bool protocolIsInHTTPFamily(const String& url)
+{
+    unsigned length = url.length();
+    const UChar* characters = url.characters();
+    return length > 4
+        && isLetterMatchIgnoringCase(characters[0], 'h')
+        && isLetterMatchIgnoringCase(characters[1], 't')
+        && isLetterMatchIgnoringCase(characters[2], 't')
+        && isLetterMatchIgnoringCase(characters[3], 'p')
+        && (characters[4] == ':'
+            || (isLetterMatchIgnoringCase(characters[4], 's') && length > 5 && characters[5] == ':'));
+}
+
 }
