@@ -28,6 +28,7 @@
 
 #include "AXObjectCache.h"
 #include "GraphicsContext.h"
+#include "GraphicsLayer.h"
 #include "HostWindow.h"
 #include "PlatformMouseEvent.h"
 #include "PlatformWheelEvent.h"
@@ -56,6 +57,7 @@ ScrollView::ScrollView()
     , m_paintsEntireContents(false)
     , m_clipsRepaints(true)
     , m_delegatesScrolling(false)
+    , m_containsScrollableAreaWithOverlayScrollbars(false)
 {
     platformInit();
 }
@@ -86,9 +88,7 @@ void ScrollView::removeChild(Widget* child)
 
 void ScrollView::setHasHorizontalScrollbar(bool hasBar)
 {
-    if (hasBar && avoidScrollbarCreation())
-        return;
-
+    ASSERT(!hasBar || !avoidScrollbarCreation());
     if (hasBar && !m_horizontalScrollbar) {
         m_horizontalScrollbar = createScrollbar(HorizontalScrollbar);
         addChild(m_horizontalScrollbar.get());
@@ -106,9 +106,7 @@ void ScrollView::setHasHorizontalScrollbar(bool hasBar)
 
 void ScrollView::setHasVerticalScrollbar(bool hasBar)
 {
-    if (hasBar && avoidScrollbarCreation())
-        return;
-
+    ASSERT(!hasBar || !avoidScrollbarCreation());
     if (hasBar && !m_verticalScrollbar) {
         m_verticalScrollbar = createScrollbar(VerticalScrollbar);
         addChild(m_verticalScrollbar.get());
@@ -124,7 +122,7 @@ void ScrollView::setHasVerticalScrollbar(bool hasBar)
         axObjectCache()->handleScrollbarUpdate(this);
 }
 
-#if !USE(NATIVE_GTK_MAIN_FRAME_SCROLLBAR)
+#if !PLATFORM(GTK)
 PassRefPtr<Scrollbar> ScrollView::createScrollbar(ScrollbarOrientation orientation)
 {
     return Scrollbar::createNativeScrollbar(this, orientation, RegularScrollbar);
@@ -224,14 +222,14 @@ void ScrollView::setDelegatesScrolling(bool delegatesScrolling)
     m_delegatesScrolling = delegatesScrolling;
 }
 
-#if !USE(NATIVE_GTK_MAIN_FRAME_SCROLLBAR)
+#if !PLATFORM(GTK)
 IntRect ScrollView::visibleContentRect(bool includeScrollbars) const
 {
     if (platformWidget())
         return platformVisibleContentRect(includeScrollbars);
 
-    if (paintsEntireContents())
-        return IntRect(IntPoint(0, 0), contentsSize());
+    if (!m_fixedVisibleContentRect.isEmpty())
+        return m_fixedVisibleContentRect;
 
     int verticalScrollbarWidth = verticalScrollbar() && !verticalScrollbar()->isOverlayScrollbar()
         && !includeScrollbars ? verticalScrollbar()->width() : 0;
@@ -383,8 +381,6 @@ void ScrollView::setScrollPosition(const IntPoint& scrollPoint)
 #if ENABLE(TILED_BACKING_STORE)
     if (delegatesScrolling()) {
         hostWindow()->delegatedScrollRequested(scrollPoint);
-        if (!m_actualVisibleContentRect.isEmpty())
-            m_actualVisibleContentRect.setLocation(scrollPoint);
         return;
     }
 #endif
@@ -468,9 +464,9 @@ void ScrollView::updateScrollbars(const IntSize& desiredOffset)
         newHasVerticalScrollbar = (vScroll == ScrollbarAlwaysOn);
 
     if (m_scrollbarsSuppressed || (hScroll != ScrollbarAuto && vScroll != ScrollbarAuto)) {
-        if (hasHorizontalScrollbar != newHasHorizontalScrollbar)
+        if (hasHorizontalScrollbar != newHasHorizontalScrollbar && (hasHorizontalScrollbar || !avoidScrollbarCreation()))
             setHasHorizontalScrollbar(newHasHorizontalScrollbar);
-        if (hasVerticalScrollbar != newHasVerticalScrollbar)
+        if (hasVerticalScrollbar != newHasVerticalScrollbar && (hasVerticalScrollbar || !avoidScrollbarCreation()))
             setHasVerticalScrollbar(newHasVerticalScrollbar);
     } else {
         bool sendContentResizedNotification = false;
@@ -496,14 +492,14 @@ void ScrollView::updateScrollbars(const IntSize& desiredOffset)
         if (!newHasVerticalScrollbar && hasVerticalScrollbar && hScroll != ScrollbarAlwaysOn)
             newHasHorizontalScrollbar = false;
 
-        if (hasHorizontalScrollbar != newHasHorizontalScrollbar) {
+        if (hasHorizontalScrollbar != newHasHorizontalScrollbar && (hasHorizontalScrollbar || !avoidScrollbarCreation())) {
             if (m_scrollOrigin.y() && !newHasHorizontalScrollbar)
                 m_scrollOrigin.setY(m_scrollOrigin.y() - m_horizontalScrollbar->height());
             setHasHorizontalScrollbar(newHasHorizontalScrollbar);
             sendContentResizedNotification = true;
         }
 
-        if (hasVerticalScrollbar != newHasVerticalScrollbar) {
+        if (hasVerticalScrollbar != newHasVerticalScrollbar && (hasVerticalScrollbar || !avoidScrollbarCreation())) {
             if (m_scrollOrigin.x() && !newHasVerticalScrollbar)
                 m_scrollOrigin.setX(m_scrollOrigin.x() - m_verticalScrollbar->width());
             setHasVerticalScrollbar(newHasVerticalScrollbar);
@@ -532,9 +528,6 @@ void ScrollView::updateScrollbars(const IntSize& desiredOffset)
 
     m_inUpdateScrollbars = true;
 
-    IntPoint scrollPoint = adjustScrollPositionWithinRange(IntPoint(desiredOffset));
-    IntSize scroll(scrollPoint.x(), scrollPoint.y());
-
     if (m_horizontalScrollbar) {
         int clientWidth = visibleWidth();
         m_horizontalScrollbar->setEnabled(contentsWidth() > clientWidth);
@@ -578,11 +571,15 @@ void ScrollView::updateScrollbars(const IntSize& desiredOffset)
     }
 
     if (hasHorizontalScrollbar != (m_horizontalScrollbar != 0) || hasVerticalScrollbar != (m_verticalScrollbar != 0)) {
+        // FIXME: Is frameRectsChanged really necessary here? Have any frame rects changed?
         frameRectsChanged();
+        positionScrollbarLayers();
         updateScrollCorner();
     }
 
-    ScrollableArea::scrollToOffsetWithoutAnimation(FloatPoint(scroll.width() + m_scrollOrigin.x(), scroll.height() + m_scrollOrigin.y()));
+    IntPoint scrollPoint = adjustScrollPositionWithinRange(IntPoint(desiredOffset)) + IntSize(m_scrollOrigin.x(), m_scrollOrigin.y());
+    if (scrollPoint != scrollPosition())
+        ScrollableArea::scrollToOffsetWithoutAnimation(scrollPoint);
 
     // Make sure the scrollbar offsets are up to date.
     if (m_horizontalScrollbar)
@@ -595,6 +592,19 @@ void ScrollView::updateScrollbars(const IntSize& desiredOffset)
 
 const int panIconSizeLength = 16;
 
+IntRect ScrollView::rectToCopyOnScroll() const
+{
+    IntRect scrollViewRect = convertToContainingWindow(IntRect(0, 0, visibleWidth(), visibleHeight()));
+    if (hasOverlayScrollbars()) {
+        int verticalScrollbarWidth = (verticalScrollbar() && !hasLayerForVerticalScrollbar()) ? verticalScrollbar()->width() : 0;
+        int horizontalScrollbarHeight = (horizontalScrollbar() && !hasLayerForHorizontalScrollbar()) ? horizontalScrollbar()->height() : 0;
+        
+        scrollViewRect.setWidth(scrollViewRect.width() - verticalScrollbarWidth);
+        scrollViewRect.setHeight(scrollViewRect.height() - horizontalScrollbarHeight);
+    }
+    return scrollViewRect;
+}
+
 void ScrollView::scrollContents(const IntSize& scrollDelta)
 {
     if (!hostWindow())
@@ -603,15 +613,7 @@ void ScrollView::scrollContents(const IntSize& scrollDelta)
     // Since scrolling is double buffered, we will be blitting the scroll view's intersection
     // with the clip rect every time to keep it smooth.
     IntRect clipRect = windowClipRect();
-    IntRect scrollViewRect = convertToContainingWindow(IntRect(0, 0, visibleWidth(), visibleHeight()));
-    if (hasOverlayScrollbars()) {
-        int verticalScrollbarWidth = verticalScrollbar() ? verticalScrollbar()->width() : 0;
-        int horizontalScrollbarHeight = horizontalScrollbar() ? horizontalScrollbar()->height() : 0;
-
-        scrollViewRect.setWidth(scrollViewRect.width() - verticalScrollbarWidth);
-        scrollViewRect.setHeight(scrollViewRect.height() - horizontalScrollbarHeight);
-    }
-    
+    IntRect scrollViewRect = rectToCopyOnScroll();    
     IntRect updateRect = clipRect;
     updateRect.intersect(scrollViewRect);
 
@@ -642,6 +644,14 @@ void ScrollView::scrollContents(const IntSize& scrollDelta)
     IntRect horizontalOverhangRect;
     IntRect verticalOverhangRect;
     calculateOverhangAreasForPainting(horizontalOverhangRect, verticalOverhangRect);
+#if USE(ACCELERATED_COMPOSITING) && PLATFORM(CHROMIUM) && ENABLE(RUBBER_BANDING)
+    if (GraphicsLayer* overhangLayer = layerForOverhangAreas()) {
+        bool hasOverhangArea = !horizontalOverhangRect.isEmpty() || !verticalOverhangRect.isEmpty();
+        overhangLayer->setDrawsContent(hasOverhangArea);
+        if (hasOverhangArea)
+            overhangLayer->setNeedsDisplay();
+    }
+#endif
     if (!horizontalOverhangRect.isEmpty())
         hostWindow()->invalidateContentsAndWindow(horizontalOverhangRect, false /*immediate*/);
     if (!verticalOverhangRect.isEmpty())
@@ -777,6 +787,12 @@ Scrollbar* ScrollView::scrollbarAtPoint(const IntPoint& windowPoint)
     return 0;
 }
 
+void ScrollView::setScrollbarOverlayStyle(ScrollbarOverlayStyle overlayStyle)
+{
+    ScrollableArea::setScrollbarOverlayStyle(overlayStyle);
+    platformSetScrollbarOverlayStyle(overlayStyle);
+}
+
 void ScrollView::wheelEvent(PlatformWheelEvent& e)
 {
     // We don't allow mouse wheeling to happen in a ScrollView that has had its scrollbars explicitly disabled.
@@ -809,6 +825,8 @@ void ScrollView::setFrameRect(const IntRect& newRect)
         return;
 
     Widget::setFrameRect(newRect);
+
+    frameRectsChanged();
 }
 
 void ScrollView::setBoundsSize(const IntSize& newSize)
@@ -826,7 +844,7 @@ void ScrollView::setBoundsSize(const IntSize& newSize)
     if (!m_useFixedLayout)
         contentsResized();
 
-    frameRectsChanged();
+    positionScrollbarLayers();
 }
 
 void ScrollView::setInitialBoundsSize(const IntSize& newSize)
@@ -845,6 +863,41 @@ void ScrollView::frameRectsChanged()
         (*current)->frameRectsChanged();
 }
 
+#if USE(ACCELERATED_COMPOSITING)
+static void positionScrollbarLayer(GraphicsLayer* graphicsLayer, Scrollbar* scrollbar)
+{
+    if (!graphicsLayer || !scrollbar)
+        return;
+    graphicsLayer->setDrawsContent(true);
+    IntRect scrollbarRect = scrollbar->frameRect();
+    graphicsLayer->setPosition(scrollbarRect.location());
+    if (scrollbarRect.size() != graphicsLayer->size())
+        graphicsLayer->setNeedsDisplay();
+    graphicsLayer->setSize(scrollbarRect.size());
+}
+
+static void positionScrollCornerLayer(GraphicsLayer* graphicsLayer, const IntRect& cornerRect)
+{
+    if (!graphicsLayer)
+        return;
+    graphicsLayer->setDrawsContent(!cornerRect.isEmpty());
+    graphicsLayer->setPosition(cornerRect.location());
+    if (cornerRect.size() != graphicsLayer->size())
+        graphicsLayer->setNeedsDisplay();
+    graphicsLayer->setSize(cornerRect.size());
+}
+#endif
+
+
+void ScrollView::positionScrollbarLayers()
+{
+#if USE(ACCELERATED_COMPOSITING)
+    positionScrollbarLayer(layerForHorizontalScrollbar(), horizontalScrollbar());
+    positionScrollbarLayer(layerForVerticalScrollbar(), verticalScrollbar());
+    positionScrollCornerLayer(layerForScrollCorner(), scrollCornerRect());
+#endif
+}
+
 void ScrollView::repaintContentRectangle(const IntRect& rect, bool now)
 {
     IntRect paintRect = rect;
@@ -854,6 +907,7 @@ void ScrollView::repaintContentRectangle(const IntRect& rect, bool now)
         return;
 
     if (platformWidget()) {
+        notifyPageThatContentAreaWillPaint();
         platformRepaintContentRectangle(paintRect, now);
         return;
     }
@@ -886,6 +940,17 @@ IntRect ScrollView::scrollCornerRect() const
     return cornerRect;
 }
 
+bool ScrollView::isScrollCornerVisible() const
+{
+    return !scrollCornerRect().isEmpty();
+}
+
+void ScrollView::scrollbarStyleChanged()
+{
+    contentsResized();
+    updateScrollbars(scrollOffset());
+}
+
 void ScrollView::updateScrollCorner()
 {
 }
@@ -895,13 +960,30 @@ void ScrollView::paintScrollCorner(GraphicsContext* context, const IntRect& corn
     ScrollbarTheme::nativeTheme()->paintScrollCorner(this, context, cornerRect);
 }
 
+void ScrollView::invalidateScrollCornerRect(const IntRect& rect)
+{
+    invalidateRect(rect);
+}
+
 void ScrollView::paintScrollbars(GraphicsContext* context, const IntRect& rect)
 {
-    if (m_horizontalScrollbar)
+    if (m_horizontalScrollbar
+#if USE(ACCELERATED_COMPOSITING)
+        && !layerForHorizontalScrollbar()
+#endif
+                                      )
         m_horizontalScrollbar->paint(context, rect);
-    if (m_verticalScrollbar)
+    if (m_verticalScrollbar
+#if USE(ACCELERATED_COMPOSITING)
+        && !layerForVerticalScrollbar()
+#endif
+                                    )
         m_verticalScrollbar->paint(context, rect);
 
+#if USE(ACCELERATED_COMPOSITING)
+    if (layerForScrollCorner())
+        return;
+#endif
     paintScrollCorner(context, scrollCornerRect());
 }
 
@@ -922,44 +1004,51 @@ void ScrollView::paint(GraphicsContext* context, const IntRect& rect)
         return;
 
     notifyPageThatContentAreaWillPaint();
-    
+
+    // If we encounter any overlay scrollbars as we paint, this will be set to true.
+    m_containsScrollableAreaWithOverlayScrollbars = false;
+
+    IntRect clipRect = frameRect();
+    if (verticalScrollbar() && !verticalScrollbar()->isOverlayScrollbar())
+        clipRect.setWidth(clipRect.width() - verticalScrollbar()->width());
+    if (horizontalScrollbar() && !horizontalScrollbar()->isOverlayScrollbar())
+        clipRect.setHeight(clipRect.height() - horizontalScrollbar()->height());
+
     IntRect documentDirtyRect = rect;
-    documentDirtyRect.intersect(frameRect());
+    documentDirtyRect.intersect(clipRect);
 
-    context->save();
+    if (!documentDirtyRect.isEmpty()) {
+        GraphicsContextStateSaver stateSaver(*context);
 
-    context->translate(x(), y());
-    documentDirtyRect.move(-x(), -y());
+        context->translate(x(), y());
+        documentDirtyRect.moveBy(-location());
 
-    if (!paintsEntireContents()) {
-        context->translate(-scrollX(), -scrollY());
-        documentDirtyRect.move(scrollX(), scrollY());
+        if (!paintsEntireContents()) {
+            context->translate(-scrollX(), -scrollY());
+            documentDirtyRect.moveBy(scrollPosition());
 
-        context->clip(visibleContentRect());
+            context->clip(visibleContentRect());
+        }
+
+        paintContents(context, documentDirtyRect);
     }
 
-    paintContents(context, documentDirtyRect);
-
-    context->restore();
-
-    IntRect horizontalOverhangRect;
-    IntRect verticalOverhangRect;
-    calculateOverhangAreasForPainting(horizontalOverhangRect, verticalOverhangRect);
-
-    if (rect.intersects(horizontalOverhangRect) || rect.intersects(verticalOverhangRect))
-        paintOverhangAreas(context, horizontalOverhangRect, verticalOverhangRect, rect);
+#if USE(ACCELERATED_COMPOSITING) && PLATFORM(CHROMIUM) && ENABLE(RUBBER_BANDING)
+    if (!layerForOverhangAreas())
+        calculateAndPaintOverhangAreas(context, rect);
+#else
+    calculateAndPaintOverhangAreas(context, rect);
+#endif
 
     // Now paint the scrollbars.
     if (!m_scrollbarsSuppressed && (m_horizontalScrollbar || m_verticalScrollbar)) {
-        context->save();
+        GraphicsContextStateSaver stateSaver(*context);
         IntRect scrollViewDirtyRect = rect;
         scrollViewDirtyRect.intersect(frameRect());
         context->translate(x(), y());
-        scrollViewDirtyRect.move(-x(), -y());
+        scrollViewDirtyRect.moveBy(-location());
 
         paintScrollbars(context, scrollViewDirtyRect);
-
-        context->restore();
     }
 
     // Paint the panScroll Icon
@@ -1006,17 +1095,19 @@ void ScrollView::calculateOverhangAreasForPainting(IntRect& horizontalOverhangRe
     }
 }
 
-void ScrollView::paintOverhangAreas(GraphicsContext* context, const IntRect& horizontalOverhangRect, const IntRect& verticalOverhangRect, const IntRect&)
+void ScrollView::paintOverhangAreas(GraphicsContext* context, const IntRect& horizontalOverhangRect, const IntRect& verticalOverhangRect, const IntRect& dirtyRect)
 {
-    // FIXME: This should be checking the dirty rect.
+    ScrollbarTheme::nativeTheme()->paintOverhangAreas(this, context, horizontalOverhangRect, verticalOverhangRect, dirtyRect);
+}
 
-    context->setFillColor(Color::white, ColorSpaceDeviceRGB);
-    if (!horizontalOverhangRect.isEmpty())
-        context->fillRect(horizontalOverhangRect);
+void ScrollView::calculateAndPaintOverhangAreas(GraphicsContext* context, const IntRect& dirtyRect)
+{
+    IntRect horizontalOverhangRect;
+    IntRect verticalOverhangRect;
+    calculateOverhangAreasForPainting(horizontalOverhangRect, verticalOverhangRect);
 
-    context->setFillColor(Color::white, ColorSpaceDeviceRGB);
-    if (!verticalOverhangRect.isEmpty())
-        context->fillRect(verticalOverhangRect);
+    if (dirtyRect.intersects(horizontalOverhangRect) || dirtyRect.intersects(verticalOverhangRect))
+        paintOverhangAreas(context, horizontalOverhangRect, verticalOverhangRect, dirtyRect);
 }
 
 bool ScrollView::isPointInScrollbarCorner(const IntPoint& windowPoint)
@@ -1051,7 +1142,7 @@ IntRect ScrollView::convertFromScrollbarToContainingView(const Scrollbar* scroll
 {
     // Scrollbars won't be transformed within us
     IntRect newRect = localRect;
-    newRect.move(scrollbar->x(), scrollbar->y());
+    newRect.moveBy(scrollbar->location());
     return newRect;
 }
 
@@ -1059,7 +1150,7 @@ IntRect ScrollView::convertFromContainingViewToScrollbar(const Scrollbar* scroll
 {
     IntRect newRect = parentRect;
     // Scrollbars won't be transformed within us
-    newRect.move(-scrollbar->x(), -scrollbar->y());
+    newRect.moveBy(-scrollbar->location());
     return newRect;
 }
 
@@ -1068,7 +1159,7 @@ IntPoint ScrollView::convertFromScrollbarToContainingView(const Scrollbar* scrol
 {
     // Scrollbars won't be transformed within us
     IntPoint newPoint = localPoint;
-    newPoint.move(scrollbar->x(), scrollbar->y());
+    newPoint.moveBy(scrollbar->location());
     return newPoint;
 }
 
@@ -1076,7 +1167,7 @@ IntPoint ScrollView::convertFromContainingViewToScrollbar(const Scrollbar* scrol
 {
     IntPoint newPoint = parentPoint;
     // Scrollbars won't be transformed within us
-    newPoint.move(-scrollbar->x(), -scrollbar->y());
+    newPoint.moveBy(-scrollbar->location());
     return newPoint;
 }
 
@@ -1205,6 +1296,10 @@ void ScrollView::platformSetScrollOrigin(const IntPoint&, bool updatePositionAtA
 {
 }
 
+void ScrollView::platformSetScrollbarOverlayStyle(ScrollbarOverlayStyle)
+{
+}
+
 #endif
 
 #if !PLATFORM(MAC) && !PLATFORM(WX)
