@@ -58,6 +58,8 @@ public:
     virtual void setScrollOffset(const IntPoint&);
     virtual void didCompleteRubberBand(const IntSize&) const;
     virtual void notifyPageThatContentAreaWillPaint() const;
+    virtual bool isScrollCornerVisible() const;
+    virtual void scrollbarStyleChanged();
 
     // NOTE: This should only be called by the overriden setScrollOffset from ScrollableArea.
     virtual void scrollTo(const IntSize& newOffset);
@@ -80,6 +82,8 @@ public:
     virtual Scrollbar* verticalScrollbar() const { return m_verticalScrollbar.get(); }
     bool isScrollViewScrollbar(const Widget* child) const { return horizontalScrollbar() == child || verticalScrollbar() == child; }
 
+    void positionScrollbarLayers();
+
     // Functions for setting and retrieving the scrolling mode in each axis (horizontal/vertical). The mode has values of
     // AlwaysOff, AlwaysOn, and Auto. AlwaysOff means never show a scrollbar, AlwaysOn means always show a scrollbar.
     // Auto means show a scrollbar only when one is needed.
@@ -104,6 +108,8 @@ public:
 
     virtual bool avoidScrollbarCreation() const { return false; }
 
+    virtual void setScrollbarOverlayStyle(ScrollbarOverlayStyle);
+
     // By default you only receive paint events for the area that is visible. In the case of using a
     // tiled backing store, this function can be set, so that the view paints the entire contents.
     bool paintsEntireContents() const { return m_paintsEntireContents; }
@@ -136,12 +142,12 @@ public:
     // and height. By default the scrollbars themselves are excluded from this rectangle, but an optional boolean argument allows them to be
     // included.
     // In the situation the client is responsible for the scrolling (ie. with a tiled backing store) it is possible to use
-    // the actualVisibleContentRect instead, though this must be updated manually, e.g after panning ends.
-    IntRect visibleContentRect(bool includeScrollbars = false) const;
-    IntRect actualVisibleContentRect() const { return m_actualVisibleContentRect.isEmpty() ? visibleContentRect() : m_actualVisibleContentRect; }
-    void setActualVisibleContentRect(const IntRect& actualVisibleContentRect) { m_actualVisibleContentRect = actualVisibleContentRect; }
-    int visibleWidth() const { return visibleContentRect().width(); }
-    int visibleHeight() const { return visibleContentRect().height(); }
+    // the setFixedVisibleContentRect instead for the mainframe, though this must be updated manually, e.g just before resuming the page
+    // which usually will happen when panning, pinching and rotation ends, or when scale or position are changed manually.
+    virtual IntRect visibleContentRect(bool includeScrollbars = false) const;
+    void setFixedVisibleContentRect(const IntRect& visibleContentRect) { m_fixedVisibleContentRect = visibleContentRect; }
+    LayoutUnit visibleWidth() const { return visibleContentRect().width(); }
+    LayoutUnit visibleHeight() const { return visibleContentRect().height(); }
 
     // Functions for getting/setting the size webkit should use to layout the contents. By default this is the same as the visible
     // content size. Explicitly setting a layout size value will cause webkit to layout the contents using this size instead.
@@ -171,6 +177,9 @@ public:
 
     IntSize overhangAmount() const;
 
+    void cacheCurrentScrollPosition() { m_cachedScrollPosition = scrollPosition(); }
+    IntPoint cachedScrollPosition() const { return m_cachedScrollPosition; }
+
     // Functions for scrolling the view.
     void setScrollPosition(const IntPoint&);
     void scrollBy(const IntSize& s) { return setScrollPosition(scrollPosition() + s); }
@@ -236,7 +245,7 @@ public:
         IntPoint newPoint = point;
         if (!isScrollViewScrollbar(child))
             newPoint = point - scrollOffset();
-        newPoint.move(child->x(), child->y());
+        newPoint.moveBy(child->location());
         return newPoint;
     }
 
@@ -245,7 +254,7 @@ public:
         IntPoint newPoint = point;
         if (!isScrollViewScrollbar(child))
             newPoint = point + scrollOffset();
-        newPoint.move(-child->x(), -child->y());
+        newPoint.moveBy(-child->location());
         return newPoint;
     }
 
@@ -266,12 +275,19 @@ public:
 
     virtual bool isPointInScrollbarCorner(const IntPoint&);
     virtual bool scrollbarCornerPresent() const;
+    virtual IntRect scrollCornerRect() const;
+    virtual void paintScrollCorner(GraphicsContext*, const IntRect& cornerRect);
 
     virtual IntRect convertFromScrollbarToContainingView(const Scrollbar*, const IntRect&) const;
     virtual IntRect convertFromContainingViewToScrollbar(const Scrollbar*, const IntRect&) const;
     virtual IntPoint convertFromScrollbarToContainingView(const Scrollbar*, const IntPoint&) const;
     virtual IntPoint convertFromContainingViewToScrollbar(const Scrollbar*, const IntPoint&) const;
 
+    bool containsScrollableAreaWithOverlayScrollbars() const { return m_containsScrollableAreaWithOverlayScrollbars; }
+    void setContainsScrollableAreaWithOverlayScrollbars(bool contains) { m_containsScrollableAreaWithOverlayScrollbars = contains; }
+
+    void calculateAndPaintOverhangAreas(GraphicsContext*, const IntRect& dirtyRect);
+
 protected:
     ScrollView();
 
@@ -284,6 +300,8 @@ protected:
     virtual void contentsResized() = 0;
     virtual void visibleContentsResized() = 0;
 
+    IntRect fixedVisibleContentRect() const { return m_fixedVisibleContentRect; }
+
     IntSize boundsSize() const { return m_boundsSize; }
     void setInitialBoundsSize(const IntSize&);
 
@@ -291,9 +309,8 @@ protected:
     void setHasHorizontalScrollbar(bool);
     void setHasVerticalScrollbar(bool);
 
-    IntRect scrollCornerRect() const;
     virtual void updateScrollCorner();
-    virtual void paintScrollCorner(GraphicsContext*, const IntRect& cornerRect);
+    virtual void invalidateScrollCornerRect(const IntRect&);
 
     // Scroll the content by blitting the pixels.
     virtual bool scrollContentsFastPath(const IntSize& scrollDelta, const IntRect& rectToScroll, const IntRect& clipRect);
@@ -301,7 +318,7 @@ protected:
     virtual void scrollContentsSlowPath(const IntRect& updateRect);
 
     void setScrollOrigin(const IntPoint&, bool updatePositionAtAll, bool updatePositionSynchronously);
-    IntPoint scrollOrigin() { return m_scrollOrigin; }
+    IntPoint scrollOrigin() const { return m_scrollOrigin; }
 
     // Subclassed by FrameView to check the writing-mode of the document.
     virtual bool isVerticalDocument() const { return true; }
@@ -324,8 +341,9 @@ private:
     // whether it is safe to blit on scroll.
     bool m_canBlitOnScroll;
 
-    IntRect m_actualVisibleContentRect;
+    IntRect m_fixedVisibleContentRect;
     IntSize m_scrollOffset; // FIXME: Would rather store this as a position, but we will wait to make this change until more code is shared.
+    IntPoint m_cachedScrollPosition;
     IntSize m_fixedLayoutSize;
     IntSize m_contentsSize;
 
@@ -343,6 +361,8 @@ private:
     bool m_clipsRepaints;
     bool m_delegatesScrolling;
 
+    bool m_containsScrollableAreaWithOverlayScrollbars;
+
     IntSize m_boundsSize;
 
     void init();
@@ -350,6 +370,7 @@ private:
 
     // Called to update the scrollbars to accurately reflect the state of the view.
     void updateScrollbars(const IntSize& desiredOffset);
+    IntRect rectToCopyOnScroll() const;
 
     // Called when the scroll position within this view changes.  FrameView overrides this to generate repaint invalidations.
     virtual void repaintFixedElementsAfterScrolling() {}
@@ -372,6 +393,7 @@ private:
     void platformSetScrollbarsSuppressed(bool repaintOnUnsuppress);
     void platformRepaintContentRectangle(const IntRect&, bool now);
     bool platformIsOffscreen() const;
+    void platformSetScrollbarOverlayStyle(ScrollbarOverlayStyle);
    
     void platformSetScrollOrigin(const IntPoint&, bool updatePositionAtAll, bool updatePositionSynchronously);
 
