@@ -26,21 +26,24 @@
 #include "config.h"
 #include "Scrollbar.h"
 
+#include "GraphicsContext.h"
+#include "PlatformMouseEvent.h"
+#include "ScrollAnimator.h"
+#include "ScrollableArea.h"
+#include "ScrollbarTheme.h"
+#include <algorithm>
+
+// FIXME: The following #includes are a layering violation and should be removed.
 #include "AXObjectCache.h"
 #include "AccessibilityScrollbar.h"
+#include "Document.h"
 #include "EventHandler.h"
 #include "Frame.h"
 #include "FrameView.h"
-#include "GraphicsContext.h"
-#include "PlatformMouseEvent.h"
-#include "ScrollableArea.h"
-#include "ScrollbarTheme.h"
-
-#include <algorithm>
 
 using namespace std;
 
-#if (PLATFORM(CHROMIUM) && (OS(LINUX) || OS(FREEBSD))) || PLATFORM(GTK)
+#if (PLATFORM(CHROMIUM) && (OS(UNIX) && !OS(DARWIN))) || PLATFORM(GTK)
 // The position of the scrollbar thumb affects the appearance of the steppers, so
 // when the thumb moves, we have to invalidate them for painting.
 #define THUMB_POSITION_AFFECTS_BUTTONS
@@ -77,6 +80,8 @@ Scrollbar::Scrollbar(ScrollableArea* scrollableArea, ScrollbarOrientation orient
     , m_hoveredPart(NoPart)
     , m_pressedPart(NoPart)
     , m_pressedPos(0)
+    , m_draggingDocument(false)
+    , m_documentDragPos(0)
     , m_enabled(true)
     , m_scrollTimer(this, &Scrollbar::autoscrollTimerFired)
     , m_overlapsResizer(false)
@@ -92,6 +97,9 @@ Scrollbar::Scrollbar(ScrollableArea* scrollableArea, ScrollbarOrientation orient
     // alone when sizing).
     int thickness = m_theme->scrollbarThickness(controlSize);
     Widget::setFrameRect(IntRect(0, 0, thickness, thickness));
+
+    if (m_scrollableArea)
+        m_currentPos = static_cast<float>(m_scrollableArea->scrollPosition(this));
 }
 
 Scrollbar::~Scrollbar()
@@ -253,14 +261,38 @@ ScrollGranularity Scrollbar::pressedPartScrollGranularity()
     return ScrollByPage;
 }
 
-void Scrollbar::moveThumb(int pos)
+void Scrollbar::moveThumb(int pos, bool draggingDocument)
 {
+    if (!m_scrollableArea)
+        return;
+
+    int delta = pos - m_pressedPos;
+
+    if (draggingDocument) {
+        if (m_draggingDocument)
+            delta = pos - m_documentDragPos;
+        m_draggingDocument = true;
+        FloatPoint currentPosition = m_scrollableArea->scrollAnimator()->currentPosition();
+        int destinationPosition = (m_orientation == HorizontalScrollbar ? currentPosition.x() : currentPosition.y()) + delta;
+        if (delta > 0)
+            destinationPosition = min(destinationPosition + delta, maximum());
+        else if (delta < 0)
+            destinationPosition = max(destinationPosition + delta, 0);
+        m_scrollableArea->scrollToOffsetWithoutAnimation(m_orientation, destinationPosition);
+        m_documentDragPos = pos;
+        return;
+    }
+
+    if (m_draggingDocument) {
+        delta += m_pressedPos - m_documentDragPos;
+        m_draggingDocument = false;
+    }
+
     // Drag the thumb.
     int thumbPos = theme()->thumbPosition(this);
     int thumbLen = theme()->thumbLength(this);
     int trackLen = theme()->trackLength(this);
     int maxPos = trackLen - thumbLen;
-    int delta = pos - m_pressedPos;
     if (delta > 0)
         delta = min(maxPos - thumbPos, delta);
     else if (delta < 0)
@@ -268,8 +300,7 @@ void Scrollbar::moveThumb(int pos)
     
     if (delta) {
         float newPosition = static_cast<float>(thumbPos + delta) * maximum() / (trackLen - thumbLen);
-        if (m_scrollableArea)
-            m_scrollableArea->scrollToOffsetWithoutAnimation(m_orientation, newPosition);
+        m_scrollableArea->scrollToOffsetWithoutAnimation(m_orientation, newPosition);
     }
 }
 
@@ -307,7 +338,7 @@ bool Scrollbar::mouseMoved(const PlatformMouseEvent& evt)
         } else {
             moveThumb(m_orientation == HorizontalScrollbar ? 
                       convertFromContainingWindow(evt.pos()).x() :
-                      convertFromContainingWindow(evt.pos()).y());
+                      convertFromContainingWindow(evt.pos()).y(), theme()->shouldDragDocumentInsteadOfThumb(this, evt));
         }
         return true;
     }
@@ -347,6 +378,7 @@ bool Scrollbar::mouseUp()
 {
     setPressedPart(NoPart);
     m_pressedPos = 0;
+    m_draggingDocument = false;
     stopTimerIfNeeded();
 
     if (parent() && parent()->isFrameView())
@@ -430,6 +462,7 @@ void Scrollbar::setEnabled(bool e)
     if (m_enabled == e)
         return;
     m_enabled = e;
+    theme()->updateEnabledState(this);
     invalidate();
 }
 
@@ -448,6 +481,8 @@ AXObjectCache* Scrollbar::axObjectCache() const
     if (!parent() || !parent()->isFrameView())
         return 0;
     
+    // FIXME: Accessing the FrameView and Document here is a layering violation
+    // and should be removed.
     Document* document = static_cast<FrameView*>(parent())->frame()->document();
     return document->axObjectCache();
 }
@@ -456,8 +491,9 @@ void Scrollbar::invalidateRect(const IntRect& rect)
 {
     if (suppressInvalidation())
         return;
+
     if (m_scrollableArea)
-        m_scrollableArea->invalidateScrollbarRect(this, rect);
+        m_scrollableArea->invalidateScrollbar(this, rect);
 }
 
 IntRect Scrollbar::convertToContainingView(const IntRect& localRect) const
