@@ -31,21 +31,28 @@
 #include "SimpleFontData.h"
 #include "TextRun.h"
 #include "WidthIterator.h"
+#include <wtf/MainThread.h>
 #include <wtf/MathExtras.h>
 #include <wtf/unicode/CharacterNames.h>
 #include <wtf/unicode/Unicode.h>
 
 using namespace WTF;
 using namespace Unicode;
+using namespace std;
 
 namespace WebCore {
 
 GlyphData Font::glyphDataForCharacter(UChar32 c, bool mirror, FontDataVariant variant) const
+{
+    return glyphDataAndPageForCharacter(c, mirror, variant).first;
+}
+
+std::pair<GlyphData, GlyphPage*> Font::glyphDataAndPageForCharacter(UChar32 c, bool mirror, FontDataVariant variant) const
 {
     ASSERT(isMainThread());
 
     if (variant == AutoVariant) {
-        if (m_fontDescription.smallCaps()) {
+        if (m_fontDescription.smallCaps() && !primaryFont()->isSVGFont()) {
             UChar32 upperC = toUpper(c);
             if (upperC != c) {
                 c = upperC;
@@ -70,7 +77,7 @@ GlyphData Font::glyphDataForCharacter(UChar32 c, bool mirror, FontDataVariant va
             m_fontList->m_pageZero = node;
     }
 
-    GlyphPage* page;
+    GlyphPage* page = 0;
     if (variant == NormalVariant) {
         // Fastest loop, for the common case (normal variant).
         while (true) {
@@ -78,61 +85,51 @@ GlyphData Font::glyphDataForCharacter(UChar32 c, bool mirror, FontDataVariant va
             if (page) {
                 GlyphData data = page->glyphDataForCharacter(c);
                 if (data.fontData && (data.fontData->platformData().orientation() == Horizontal || data.fontData->isTextOrientationFallback()))
-                    return data;
-                
+                    return make_pair(data, page);
+
                 if (data.fontData) {
                     if (isCJKIdeographOrSymbol(c)) {
                         if (!data.fontData->hasVerticalGlyphs()) {
                             // Use the broken ideograph font data. The broken ideograph font will use the horizontal width of glyphs
                             // to make sure you get a square (even for broken glyphs like symbols used for punctuation).
-                            const SimpleFontData* brokenIdeographFontData = data.fontData->brokenIdeographFontData();
-                            GlyphPageTreeNode* brokenIdeographNode = GlyphPageTreeNode::getRootChild(brokenIdeographFontData, pageNumber);
-                            const GlyphPage* brokenIdeographPage = brokenIdeographNode->page();
-                            if (brokenIdeographPage) {
-                                GlyphData brokenIdeographData = brokenIdeographPage->glyphDataForCharacter(c);
-                                if (brokenIdeographData.fontData)
-                                    return brokenIdeographData;
-                            }
-                            
-                            // Shouldn't be possible to even reach this point.
-                            ASSERT_NOT_REACHED();
-                            return data;
+                            variant = BrokenIdeographVariant;
+                            break;
                         }
                     } else {
                         if (m_fontDescription.textOrientation() == TextOrientationVerticalRight) {
                             const SimpleFontData* verticalRightFontData = data.fontData->verticalRightOrientationFontData();
                             GlyphPageTreeNode* verticalRightNode = GlyphPageTreeNode::getRootChild(verticalRightFontData, pageNumber);
-                            const GlyphPage* verticalRightPage = verticalRightNode->page();
+                            GlyphPage* verticalRightPage = verticalRightNode->page();
                             if (verticalRightPage) {
                                 GlyphData verticalRightData = verticalRightPage->glyphDataForCharacter(c);
                                 // If the glyphs are distinct, we will make the assumption that the font has a vertical-right glyph baked
                                 // into it.
                                 if (data.glyph != verticalRightData.glyph)
-                                    return data;
+                                    return make_pair(data, page);
                                 // The glyphs are identical, meaning that we should just use the horizontal glyph.
                                 if (verticalRightData.fontData)
-                                    return verticalRightData;
+                                    return make_pair(verticalRightData, verticalRightPage);
                             }
                         } else if (m_fontDescription.textOrientation() == TextOrientationUpright) {
                             const SimpleFontData* uprightFontData = data.fontData->uprightOrientationFontData();
                             GlyphPageTreeNode* uprightNode = GlyphPageTreeNode::getRootChild(uprightFontData, pageNumber);
-                            const GlyphPage* uprightPage = uprightNode->page();
+                            GlyphPage* uprightPage = uprightNode->page();
                             if (uprightPage) {
                                 GlyphData uprightData = uprightPage->glyphDataForCharacter(c);
                                 // If the glyphs are the same, then we know we can just use the horizontal glyph rotated vertically to be upright.
                                 if (data.glyph == uprightData.glyph)
-                                    return data;
+                                    return make_pair(data, page);
                                 // The glyphs are distinct, meaning that the font has a vertical-right glyph baked into it. We can't use that
                                 // glyph, so we fall back to the upright data and use the horizontal glyph.
                                 if (uprightData.fontData)
-                                    return uprightData;
+                                    return make_pair(uprightData, uprightPage);
                             }
                         }
 
                         // Shouldn't be possible to even reach this point.
                         ASSERT_NOT_REACHED();
-                        return data;
                     }
+                    return make_pair(data, page);
                 }
 
                 if (node->isSystemFallback())
@@ -146,7 +143,8 @@ GlyphData Font::glyphDataForCharacter(UChar32 c, bool mirror, FontDataVariant va
             else
                 m_fontList->m_pageZero = node;
         }
-    } else {
+    }
+    if (variant != NormalVariant) {
         while (true) {
             page = node->page();
             if (page) {
@@ -156,19 +154,19 @@ GlyphData Font::glyphDataForCharacter(UChar32 c, bool mirror, FontDataVariant va
                     // But if it does, we will just render the capital letter big.
                     const SimpleFontData* variantFontData = data.fontData->variantFontData(m_fontDescription, variant);
                     if (!variantFontData)
-                        return data;
+                        return make_pair(data, page);
 
                     GlyphPageTreeNode* variantNode = GlyphPageTreeNode::getRootChild(variantFontData, pageNumber);
-                    const GlyphPage* variantPage = variantNode->page();
+                    GlyphPage* variantPage = variantNode->page();
                     if (variantPage) {
                         GlyphData data = variantPage->glyphDataForCharacter(c);
                         if (data.fontData)
-                            return data;
+                            return make_pair(data, variantPage);
                     }
 
                     // Do not attempt system fallback off the variantFontData. This is the very unlikely case that
                     // a font has the lowercase character but the small caps font does not have its uppercase version.
-                    return variantFontData->missingGlyphData();
+                    return make_pair(variantFontData->missingGlyphData(), page);
                 }
 
                 if (node->isSystemFallback())
@@ -201,8 +199,12 @@ GlyphData Font::glyphDataForCharacter(UChar32 c, bool mirror, FontDataVariant va
         codeUnitsLength = 2;
     }
     const SimpleFontData* characterFontData = fontCache()->getFontDataForCharacters(*this, codeUnits, codeUnitsLength);
-    if (variant != NormalVariant && characterFontData)
-        characterFontData = characterFontData->variantFontData(m_fontDescription, variant);
+    if (characterFontData) {
+        if (characterFontData->platformData().orientation() == Vertical && !characterFontData->hasVerticalGlyphs() && isCJKIdeographOrSymbol(c))
+            variant = BrokenIdeographVariant;
+        if (variant != NormalVariant)
+            characterFontData = characterFontData->variantFontData(m_fontDescription, variant);
+    }
     if (characterFontData) {
         // Got the fallback glyph and font.
         GlyphPage* fallbackPage = GlyphPageTreeNode::getRootChild(characterFontData, pageNumber)->page();
@@ -215,12 +217,14 @@ GlyphData Font::glyphDataForCharacter(UChar32 c, bool mirror, FontDataVariant va
             // display the character, probably because the font package is not installed correctly.
             // So we just always set the glyph to be same as the character, and let GDI solve it.
             page->setGlyphDataForCharacter(c, c, characterFontData);
-            return page->glyphDataForCharacter(c);
+            characterFontData->setMaxGlyphPageTreeLevel(max(characterFontData->maxGlyphPageTreeLevel(), node->level()));
+            return make_pair(page->glyphDataForCharacter(c), page);
 #else
             page->setGlyphDataForCharacter(c, data.glyph, data.fontData);
+            data.fontData->setMaxGlyphPageTreeLevel(max(data.fontData->maxGlyphPageTreeLevel(), node->level()));
 #endif
         }
-        return data;
+        return make_pair(data, page);
     }
 
     // Even system fallback can fail; use the missing glyph in that case.
@@ -230,12 +234,14 @@ GlyphData Font::glyphDataForCharacter(UChar32 c, bool mirror, FontDataVariant va
 #if OS(WINCE)
         // See comment about WINCE GDI handling near setGlyphDataForCharacter above.
         page->setGlyphDataForCharacter(c, c, data.fontData);
-        return page->glyphDataForCharacter(c);
+        data.fontData->setMaxGlyphPageTreeLevel(max(data.fontData->maxGlyphPageTreeLevel(), node->level()));
+        return make_pair(page->glyphDataForCharacter(c), page);
 #else
         page->setGlyphDataForCharacter(c, data.glyph, data.fontData);
+        data.fontData->setMaxGlyphPageTreeLevel(max(data.fontData->maxGlyphPageTreeLevel(), node->level()));
 #endif
     }
-    return data;
+    return make_pair(data, page);
 }
 
 bool Font::primaryFontHasGlyphForCharacter(UChar32 character) const
@@ -255,12 +261,6 @@ bool Font::getEmphasisMarkGlyphData(const AtomicString& mark, GlyphData& glyphDa
     if (mark.isEmpty())
         return false;
 
-#if ENABLE(SVG_FONTS)
-    // FIXME: Implement for SVG fonts.
-    if (primaryFont()->isSVGFont())
-        return false;
-#endif
-
     UChar32 character = mark[0];
 
     if (U16_IS_SURROGATE(character)) {
@@ -338,8 +338,9 @@ float Font::getGlyphsAndAdvancesForSimpleText(const TextRun& run, int from, int
     float afterWidth = it.m_runWidthSoFar;
 
     if (run.rtl()) {
+        float finalRoundingWidth = it.m_finalRoundingWidth;
         it.advance(run.length());
-        initialAdvance = it.m_runWidthSoFar - afterWidth;
+        initialAdvance = finalRoundingWidth + it.m_runWidthSoFar - afterWidth;
     } else
         initialAdvance = beforeWidth;
 
@@ -362,7 +363,7 @@ void Font::drawSimpleText(GraphicsContext* context, const TextRun& run, const Fl
         return;
 
     FloatPoint startPoint(startX, point.y());
-    drawGlyphBuffer(context, glyphBuffer, startPoint);
+    drawGlyphBuffer(context, run, glyphBuffer, startPoint);
 }
 
 void Font::drawEmphasisMarksForSimpleText(GraphicsContext* context, const TextRun& run, const AtomicString& mark, const FloatPoint& point, int from, int to) const
@@ -373,10 +374,10 @@ void Font::drawEmphasisMarksForSimpleText(GraphicsContext* context, const TextRu
     if (glyphBuffer.isEmpty())
         return;
 
-    drawEmphasisMarks(context, glyphBuffer, mark, FloatPoint(point.x() + initialAdvance, point.y()));
+    drawEmphasisMarks(context, run, glyphBuffer, mark, FloatPoint(point.x() + initialAdvance, point.y()));
 }
 
-void Font::drawGlyphBuffer(GraphicsContext* context, const GlyphBuffer& glyphBuffer, const FloatPoint& point) const
+void Font::drawGlyphBuffer(GraphicsContext* context, const TextRun& run, const GlyphBuffer& glyphBuffer, const FloatPoint& point) const
 {   
     // Draw each contiguous run of glyphs that use the same font data.
     const SimpleFontData* fontData = glyphBuffer.fontDataAt(0);
@@ -385,11 +386,20 @@ void Font::drawGlyphBuffer(GraphicsContext* context, const GlyphBuffer& glyphBuf
     float nextX = startPoint.x();
     int lastFrom = 0;
     int nextGlyph = 0;
+#if ENABLE(SVG_FONTS)
+    TextRun::RenderingContext* renderingContext = run.renderingContext();
+#endif
     while (nextGlyph < glyphBuffer.size()) {
         const SimpleFontData* nextFontData = glyphBuffer.fontDataAt(nextGlyph);
         FloatSize nextOffset = glyphBuffer.offsetAt(nextGlyph);
+
         if (nextFontData != fontData || nextOffset != offset) {
-            drawGlyphs(context, fontData, glyphBuffer, lastFrom, nextGlyph - lastFrom, startPoint);
+#if ENABLE(SVG_FONTS)
+            if (renderingContext && fontData->isSVGFont())
+                renderingContext->drawSVGGlyphs(context, run, fontData, glyphBuffer, lastFrom, nextGlyph - lastFrom, startPoint);
+            else
+#endif
+                drawGlyphs(context, fontData, glyphBuffer, lastFrom, nextGlyph - lastFrom, startPoint);
 
             lastFrom = nextGlyph;
             fontData = nextFontData;
@@ -400,7 +410,12 @@ void Font::drawGlyphBuffer(GraphicsContext* context, const GlyphBuffer& glyphBuf
         nextGlyph++;
     }
 
-    drawGlyphs(context, fontData, glyphBuffer, lastFrom, nextGlyph - lastFrom, startPoint);
+#if ENABLE(SVG_FONTS)
+    if (renderingContext && fontData->isSVGFont())
+        renderingContext->drawSVGGlyphs(context, run, fontData, glyphBuffer, lastFrom, nextGlyph - lastFrom, startPoint);
+    else
+#endif
+        drawGlyphs(context, fontData, glyphBuffer, lastFrom, nextGlyph - lastFrom, startPoint);
 }
 
 inline static float offsetToMiddleOfGlyph(const SimpleFontData* fontData, Glyph glyph)
@@ -418,7 +433,7 @@ inline static float offsetToMiddleOfGlyphAtIndex(const GlyphBuffer& glyphBuffer,
     return offsetToMiddleOfGlyph(glyphBuffer.fontDataAt(i), glyphBuffer.glyphAt(i));
 }
 
-void Font::drawEmphasisMarks(GraphicsContext* context, const GlyphBuffer& glyphBuffer, const AtomicString& mark, const FloatPoint& point) const
+void Font::drawEmphasisMarks(GraphicsContext* context, const TextRun& run, const GlyphBuffer& glyphBuffer, const AtomicString& mark, const FloatPoint& point) const
 {
     GlyphData markGlyphData;
     if (!getEmphasisMarkGlyphData(mark, markGlyphData))
@@ -444,7 +459,7 @@ void Font::drawEmphasisMarks(GraphicsContext* context, const GlyphBuffer& glyphB
     }
     markBuffer.add(glyphBuffer.glyphAt(glyphBuffer.size() - 1) ? markGlyph : spaceGlyph, markFontData, 0);
 
-    drawGlyphBuffer(context, markBuffer, startPoint);
+    drawGlyphBuffer(context, run, markBuffer, startPoint);
 }
 
 float Font::floatWidthForSimpleText(const TextRun& run, GlyphBuffer* glyphBuffer, HashSet<const SimpleFontData*>* fallbackFonts, GlyphOverflow* glyphOverflow) const
@@ -453,8 +468,8 @@ float Font::floatWidthForSimpleText(const TextRun& run, GlyphBuffer* glyphBuffer
     it.advance(run.length(), glyphBuffer);
 
     if (glyphOverflow) {
-        glyphOverflow->top = max<int>(glyphOverflow->top, ceilf(-it.minGlyphBoundingBoxY()) - fontMetrics().ascent());
-        glyphOverflow->bottom = max<int>(glyphOverflow->bottom, ceilf(it.maxGlyphBoundingBoxY()) - fontMetrics().descent());
+        glyphOverflow->top = max<int>(glyphOverflow->top, ceilf(-it.minGlyphBoundingBoxY()) - (glyphOverflow->computeBounds ? 0 : fontMetrics().ascent()));
+        glyphOverflow->bottom = max<int>(glyphOverflow->bottom, ceilf(it.maxGlyphBoundingBoxY()) - (glyphOverflow->computeBounds ? 0 : fontMetrics().descent()));
         glyphOverflow->left = ceilf(it.firstGlyphOverflow());
         glyphOverflow->right = ceilf(it.lastGlyphOverflow());
     }
@@ -474,10 +489,10 @@ FloatRect Font::selectionRectForSimpleText(const TextRun& run, const FloatPoint&
     if (run.rtl()) {
         it.advance(run.length());
         float totalWidth = it.m_runWidthSoFar;
-        return FloatRect(point.x() + floorf(totalWidth - afterWidth), point.y(), roundf(totalWidth - beforeWidth) - floorf(totalWidth - afterWidth), h);
+        return FloatRect(floorf(point.x() + totalWidth - afterWidth), point.y(), roundf(point.x() + totalWidth - beforeWidth) - floorf(point.x() + totalWidth - afterWidth), h);
     }
 
-    return FloatRect(point.x() + floorf(beforeWidth), point.y(), roundf(afterWidth) - floorf(beforeWidth), h);
+    return FloatRect(floorf(point.x() + beforeWidth), point.y(), roundf(point.x() + afterWidth) - floorf(point.x() + beforeWidth), h);
 }
 
 int Font::offsetForPositionForSimpleText(const TextRun& run, float x, bool includePartialGlyphs) const
