@@ -37,13 +37,13 @@
 #include <wtf/Noncopyable.h>
 #include <wtf/PassOwnPtr.h>
 
-#if PLATFORM(CG)
+#if USE(CG)
 typedef struct CGContext PlatformGraphicsContext;
-#elif PLATFORM(CAIRO)
+#elif USE(CAIRO)
 namespace WebCore {
-class ContextShadow;
+class PlatformContextCairo;
 }
-typedef struct _cairo PlatformGraphicsContext;
+typedef WebCore::PlatformContextCairo PlatformGraphicsContext;
 #elif PLATFORM(OPENVG)
 namespace WebCore {
 class SurfaceOpenVG;
@@ -52,7 +52,7 @@ typedef class WebCore::SurfaceOpenVG PlatformGraphicsContext;
 #elif PLATFORM(QT)
 #include <QPainter>
 namespace WebCore {
-class ContextShadow;
+class ShadowBlur;
 }
 typedef QPainter PlatformGraphicsContext;
 #elif PLATFORM(WX)
@@ -77,6 +77,7 @@ class wxWindowDC;
 #elif USE(SKIA)
 namespace WebCore {
 class PlatformContextSkia;
+typedef PlatformContextSkia GraphicsContextPlatformPrivate;
 }
 typedef WebCore::PlatformContextSkia PlatformGraphicsContext;
 #elif PLATFORM(HAIKU)
@@ -90,8 +91,9 @@ typedef void PlatformGraphicsContext;
 #endif
 
 #if PLATFORM(WIN)
+#include "DIBPixelData.h"
 typedef struct HDC__* HDC;
-#if !PLATFORM(CG)
+#if !USE(CG)
 // UInt8 is defined in CoreFoundation/CFBase.h
 typedef unsigned char UInt8;
 #endif
@@ -117,12 +119,14 @@ namespace WebCore {
     class DrawingBuffer;
     class Font;
     class Generator;
+#if !USE(SKIA)
     class GraphicsContextPlatformPrivate;
+#endif
     class ImageBuffer;
     class IntRect;
-    class RoundedIntRect;
+    class RoundedRect;
     class KURL;
-    class SharedGraphicsContext3D;
+    class GraphicsContext3D;
     class TextRun;
 
     enum TextDrawingMode {
@@ -152,9 +156,6 @@ namespace WebCore {
         GraphicsContextState()
             : strokeThickness(0)
             , shadowBlur(0)
-#if PLATFORM(CAIRO)
-            , globalAlpha(1)
-#endif
             , textDrawingMode(TextModeFill)
             , strokeColor(Color::black)
             , fillColor(Color::black)
@@ -168,7 +169,7 @@ namespace WebCore {
             , shouldSmoothFonts(true)
             , paintingDisabled(false)
             , shadowsIgnoreTransforms(false)
-#if PLATFORM(CG)
+#if USE(CG)
             // Core Graphics incorrectly renders shadows with radius > 8px (<rdar://problem/8103442>),
             // but we need to preserve this buggy behavior for canvas and -webkit-box-shadow.
             , shadowsUseLegacyRadius(false)
@@ -187,9 +188,6 @@ namespace WebCore {
         float strokeThickness;
         float shadowBlur;
 
-#if PLATFORM(CAIRO)
-        float globalAlpha;
-#endif
         TextDrawingModeFlags textDrawingMode;
 
         Color strokeColor;
@@ -209,7 +207,7 @@ namespace WebCore {
         bool shouldSmoothFonts : 1;
         bool paintingDisabled : 1;
         bool shadowsIgnoreTransforms : 1;
-#if PLATFORM(CG)
+#if USE(CG)
         bool shadowsUseLegacyRadius : 1;
 #endif
     };
@@ -261,11 +259,13 @@ namespace WebCore {
 
         const GraphicsContextState& state() const;
 
-#if PLATFORM(CG)
+#if USE(CG)
         void applyStrokePattern();
         void applyFillPattern();
         void drawPath(const Path&);
         
+        void drawNativeImage(NativeImagePtr, const FloatSize& selfSize, ColorSpace styleColorSpace, const FloatRect& destRect, const FloatRect& srcRect, CompositeOperator = CompositeSourceOver);
+
         // Allow font smoothing (LCD antialiasing). Not part of the graphics state.
         void setAllowsFontSmoothing(bool);
         
@@ -273,8 +273,8 @@ namespace WebCore {
         bool isCALayerContext() const;
 
         void setIsAcceleratedContext(bool);
-        bool isAcceleratedContext() const;
 #endif
+        bool isAcceleratedContext() const;
 
         void save();
         void restore();
@@ -297,9 +297,10 @@ namespace WebCore {
         void fillRect(const FloatRect&);
         void fillRect(const FloatRect&, const Color&, ColorSpace);
         void fillRect(const FloatRect&, Generator&);
+        void fillRect(const FloatRect&, const Color&, ColorSpace, CompositeOperator);
         void fillRoundedRect(const IntRect&, const IntSize& topLeft, const IntSize& topRight, const IntSize& bottomLeft, const IntSize& bottomRight, const Color&, ColorSpace);
-        void fillRoundedRect(const RoundedIntRect&, const Color&, ColorSpace);
-        void fillRectWithRoundedHole(const IntRect&, const RoundedIntRect& roundedHoleRect, const Color&, ColorSpace);
+        void fillRoundedRect(const RoundedRect&, const Color&, ColorSpace);
+        void fillRectWithRoundedHole(const IntRect&, const RoundedRect& roundedHoleRect, const Color&, ColorSpace);
 
         void clearRect(const FloatRect&);
 
@@ -314,7 +315,7 @@ namespace WebCore {
         void drawTiledImage(Image*, ColorSpace styleColorSpace, const IntRect& destRect, const IntPoint& srcPoint, const IntSize& tileSize,
                        CompositeOperator = CompositeSourceOver, bool useLowQualityScale = false);
         void drawTiledImage(Image*, ColorSpace styleColorSpace, const IntRect& destRect, const IntRect& srcRect,
-                            Image::TileRule hRule = Image::StretchTile, Image::TileRule vRule = Image::StretchTile,
+                            const FloatSize& tileScaleFactor, Image::TileRule hRule = Image::StretchTile, Image::TileRule vRule = Image::StretchTile,
                             CompositeOperator = CompositeSourceOver, bool useLowQualityScale = false);
 
         void drawImageBuffer(ImageBuffer*, ColorSpace styleColorSpace, const IntPoint&, CompositeOperator = CompositeSourceOver);
@@ -329,10 +330,10 @@ namespace WebCore {
 
         void clip(const IntRect&);
         void clip(const FloatRect&);
-        void addRoundedRectClip(const RoundedIntRect&);
+        void addRoundedRectClip(const RoundedRect&);
         void addInnerRoundedRectClip(const IntRect&, int thickness);
         void clipOut(const IntRect&);
-        void clipOutRoundedRect(const RoundedIntRect&);
+        void clipOutRoundedRect(const RoundedRect&);
         void clipPath(const Path&, WindRule);
         void clipConvexPolygon(size_t numPoints, const FloatPoint*, bool antialias = true);
         void clipToImageBuffer(ImageBuffer*, const FloatRect&);
@@ -347,7 +348,11 @@ namespace WebCore {
         void drawBidiText(const Font&, const TextRun&, const FloatPoint&);
         void drawHighlightForText(const Font&, const TextRun&, const FloatPoint&, int h, const Color& backgroundColor, ColorSpace, int from = 0, int to = -1);
 
-        FloatRect roundToDevicePixels(const FloatRect&);
+        enum RoundingMode {
+            RoundAllSides,
+            RoundOriginAndDimensions
+        };
+        FloatRect roundToDevicePixels(const FloatRect&, RoundingMode = RoundAllSides);
 
         void drawLineForText(const FloatPoint&, float width, bool printing);
         enum TextCheckingLineStyle {
@@ -365,6 +370,7 @@ namespace WebCore {
 
         void beginTransparencyLayer(float opacity);
         void endTransparencyLayer();
+        bool isInTransparencyLayer() const;
 
         bool hasShadow() const;
         void setShadow(const FloatSize&, float blur, const Color&, ColorSpace);
@@ -384,9 +390,6 @@ namespace WebCore {
         void setMiterLimit(float);
 
         void setAlpha(float);
-#if PLATFORM(CAIRO)
-        float getAlpha();
-#endif
 
         void setCompositeOperation(CompositeOperator);
         CompositeOperator compositeOperation() const;
@@ -428,7 +431,6 @@ namespace WebCore {
         void drawRoundCorner(bool newClip, RECT clipRect, RECT rectWin, HDC dc, int width, int height);
 #elif PLATFORM(WIN)
         GraphicsContext(HDC, bool hasAlpha = false); // FIXME: To be removed.
-        bool inTransparencyLayer() const;
         HDC getWindowsContext(const IntRect&, bool supportAlphaBlend = true, bool mayCreateBitmap = true); // The passed in rect is used to create a bitmap for compositing inside transparency layers.
         void releaseWindowsContext(HDC, const IntRect&, bool supportAlphaBlend = true, bool mayCreateBitmap = true);    // The passed in HDC should be the one handed back by getWindowsContext.
 
@@ -446,25 +448,24 @@ namespace WebCore {
         class WindowsBitmap {
             WTF_MAKE_NONCOPYABLE(WindowsBitmap);
         public:
-            WindowsBitmap(HDC, IntSize);
+            WindowsBitmap(HDC, const IntSize&);
             ~WindowsBitmap();
 
             HDC hdc() const { return m_hdc; }
-            UInt8* buffer() const { return m_bitmapBuffer; }
-            unsigned bufferLength() const { return m_bitmapBufferLength; }
-            IntSize size() const { return m_size; }
-            unsigned bytesPerRow() const { return m_bytesPerRow; }
+            UInt8* buffer() const { return m_pixelData.buffer(); }
+            unsigned bufferLength() const { return m_pixelData.bufferLength(); }
+            const IntSize& size() const { return m_pixelData.size(); }
+            unsigned bytesPerRow() const { return m_pixelData.bytesPerRow(); }
+            unsigned short bitsPerPixel() const { return m_pixelData.bitsPerPixel(); }
+            const DIBPixelData& windowsDIB() const { return m_pixelData; }
 
         private:
             HDC m_hdc;
             HBITMAP m_bitmap;
-            UInt8* m_bitmapBuffer;
-            unsigned m_bitmapBufferLength;
-            IntSize m_size;
-            unsigned m_bytesPerRow;
+            DIBPixelData m_pixelData;
         };
 
-        WindowsBitmap* createWindowsBitmap(IntSize);
+        PassOwnPtr<WindowsBitmap> createWindowsBitmap(const IntSize&);
         // The bitmap should be non-premultiplied.
         void drawWindowsBitmap(WindowsBitmap*, const IntPoint&);
 #endif
@@ -476,21 +477,22 @@ namespace WebCore {
 #endif
 
 #if PLATFORM(WX)
-        bool inTransparencyLayer() const { return false; }
+        // This is needed because of a bug whereby getting an HDC from a GDI+ context
+        // loses the scale operations applied to the context.
+        FloatSize currentScale(); 
 #endif
 
 #if PLATFORM(QT)
-        bool inTransparencyLayer() const;
         void pushTransparencyLayerInternal(const QRect &rect, qreal opacity, QPixmap& alphaMask);
         void takeOwnershipOfPlatformContext();
 #endif
 
-#if PLATFORM(QT) || PLATFORM(CAIRO)
-        ContextShadow* contextShadow();
+#if PLATFORM(QT)
+        ShadowBlur* shadowBlur();
 #endif
 
-#if PLATFORM(CAIRO)
-        void pushImageMask(cairo_surface_t*, const FloatRect&);
+#if USE(CAIRO)
+        GraphicsContext(cairo_t*);
 #endif
 
 #if PLATFORM(GTK)
@@ -503,9 +505,7 @@ namespace WebCore {
         pattern getHaikuStrokeStyle();
 #endif
 
-        void setSharedGraphicsContext3D(SharedGraphicsContext3D*, DrawingBuffer*, const IntSize&);
-        void syncSoftwareCanvas();
-        void markDirtyRect(const IntRect&); // Hints that a portion of the backing store is dirty.
+        static void adjustLineToPixelBoundaries(FloatPoint& p1, FloatPoint& p2, float strokeWidth, StrokeStyle);
 
     private:
         void platformInit(PlatformGraphicsContext*);
@@ -524,12 +524,8 @@ namespace WebCore {
         void setPlatformStrokeColor(const Color&, ColorSpace);
         void setPlatformStrokeStyle(StrokeStyle);
         void setPlatformStrokeThickness(float);
-        void setPlatformStrokeGradient(Gradient*);
-        void setPlatformStrokePattern(Pattern*);
 
         void setPlatformFillColor(const Color&, ColorSpace);
-        void setPlatformFillGradient(Gradient*);
-        void setPlatformFillPattern(Pattern*);
 
         void setPlatformShouldAntialias(bool);
         void setPlatformShouldSmoothFonts(bool);
@@ -539,13 +535,53 @@ namespace WebCore {
 
         void setPlatformCompositeOperation(CompositeOperator);
 
-        static void adjustLineToPixelBoundaries(FloatPoint& p1, FloatPoint& p2, float strokeWidth, StrokeStyle);
+        void beginPlatformTransparencyLayer(float opacity);
+        void endPlatformTransparencyLayer();
+        static bool supportsTransparencyLayers();
 
         GraphicsContextPlatformPrivate* m_data;
 
         GraphicsContextState m_state;
         Vector<GraphicsContextState> m_stack;
         bool m_updatingControlTints;
+        unsigned m_transparencyCount;
+    };
+
+    class GraphicsContextStateSaver {
+    public:
+        GraphicsContextStateSaver(GraphicsContext& context, bool saveAndRestore = true)
+        : m_context(context)
+        , m_saveAndRestore(saveAndRestore)
+        {
+            if (m_saveAndRestore)
+                m_context.save();
+        }
+        
+        ~GraphicsContextStateSaver()
+        {
+            if (m_saveAndRestore)
+                m_context.restore();
+        }
+        
+        void save()
+        {
+            ASSERT(!m_saveAndRestore);
+            m_context.save();
+            m_saveAndRestore = true;
+        }
+
+        void restore()
+        {
+            ASSERT(m_saveAndRestore);
+            m_context.restore();
+            m_saveAndRestore = false;
+        }
+        
+        GraphicsContext* context() const { return &m_context; }
+
+    private:
+        GraphicsContext& m_context;
+        bool m_saveAndRestore;
     };
 
 } // namespace WebCore
