@@ -32,12 +32,10 @@
 #include "config.h"
 #include "GraphicsContext.h"
 
-#if PLATFORM(CAIRO)
+#if USE(CAIRO)
 
 #include "AffineTransform.h"
-#include "CairoPath.h"
 #include "CairoUtilities.h"
-#include "ContextShadow.h"
 #include "FloatConversion.h"
 #include "FloatRect.h"
 #include "Font.h"
@@ -47,7 +45,10 @@
 #include "NotImplemented.h"
 #include "Path.h"
 #include "Pattern.h"
+#include "PlatformContextCairo.h"
+#include "PlatformPathCairo.h"
 #include "RefPtrCairo.h"
+#include "ShadowBlur.h"
 #include "SimpleFontData.h"
 #include <cairo.h>
 #include <math.h>
@@ -69,58 +70,12 @@ using namespace std;
 
 namespace WebCore {
 
-static inline void setPlatformFill(GraphicsContext* context, cairo_t* cr)
+// A helper which quickly fills a rectangle with a simple color fill.
+static inline void fillRectWithColor(cairo_t* cr, const FloatRect& rect, const Color& color)
 {
-    cairo_pattern_t* pattern = 0;
-    cairo_save(cr);
-    
-    const GraphicsContextState& state = context->state();
-    if (state.fillPattern) {
-        AffineTransform affine;
-        pattern = state.fillPattern->createPlatformPattern(affine);
-        cairo_set_source(cr, pattern);
-    } else if (state.fillGradient)
-        cairo_set_source(cr, state.fillGradient->platformGradient());
-    else
-        setSourceRGBAFromColor(cr, context->fillColor());
-    cairo_clip_preserve(cr);
-    cairo_paint_with_alpha(cr, state.globalAlpha);
-    cairo_restore(cr);
-    if (pattern)
-        cairo_pattern_destroy(pattern);
-}
-
-static inline void setPlatformStroke(GraphicsContext* context, cairo_t* cr)
-{
-    cairo_pattern_t* pattern = 0;
-    cairo_save(cr);
-    
-    const GraphicsContextState& state = context->state();
-    if (state.strokePattern) {
-        AffineTransform affine;
-        pattern = state.strokePattern->createPlatformPattern(affine);
-        cairo_set_source(cr, pattern);
-    } else if (state.strokeGradient)
-        cairo_set_source(cr, state.strokeGradient->platformGradient());
-    else  {
-        Color strokeColor = colorWithOverrideAlpha(context->strokeColor().rgb(), context->strokeColor().alpha() / 255.f * state.globalAlpha);
-        setSourceRGBAFromColor(cr, strokeColor);
-    }
-    if (state.globalAlpha < 1.0f && (state.strokePattern || state.strokeGradient)) {
-        cairo_push_group(cr);
-        cairo_paint_with_alpha(cr, state.globalAlpha);
-        cairo_pop_group_to_source(cr);
-    }
-    cairo_stroke_preserve(cr);
-    cairo_restore(cr);
-    if (pattern)
-        cairo_pattern_destroy(pattern);
-}
-
-// A fillRect helper
-static inline void fillRectSourceOver(cairo_t* cr, const FloatRect& rect, const Color& col)
-{
-    setSourceRGBAFromColor(cr, col);
+    if (!color.alpha())
+        return;
+    setSourceRGBAFromColor(cr, color);
     cairo_rectangle(cr, rect.x(), rect.y(), rect.width(), rect.height());
     cairo_set_operator(cr, CAIRO_OPERATOR_OVER);
     cairo_fill(cr);
@@ -142,14 +97,13 @@ enum PathDrawingStyle {
 
 static inline void drawPathShadow(GraphicsContext* context, PathDrawingStyle drawingStyle)
 {
-    ContextShadow* shadow = context->contextShadow();
-    ASSERT(shadow);
-    if (shadow->m_type == ContextShadow::NoShadow)
+    ShadowBlur& shadow = context->platformContext()->shadowBlur();
+    if (shadow.type() == ShadowBlur::NoShadow)
         return;
 
     // Calculate the extents of the rendered solid paths.
-    cairo_t* cairoContext = context->platformContext();
-    OwnPtr<cairo_path_t> path(cairo_copy_path(cairoContext));
+    cairo_t* cairoContext = context->platformContext()->cr();
+    OwnPtr<cairo_path_t> path = adoptPtr(cairo_copy_path(cairoContext));
 
     FloatRect solidFigureExtents;
     double x0 = 0;
@@ -166,45 +120,73 @@ static inline void drawPathShadow(GraphicsContext* context, PathDrawingStyle dra
         solidFigureExtents.unite(fillExtents);
     }
 
-    cairo_t* shadowContext = shadow->beginShadowLayer(context, solidFigureExtents);
+    GraphicsContext* shadowContext = shadow.beginShadowLayer(context, solidFigureExtents);
     if (!shadowContext)
         return;
 
+    cairo_t* cairoShadowContext = shadowContext->platformContext()->cr();
+
     // It's important to copy the context properties to the new shadow
     // context to preserve things such as the fill rule and stroke width.
-    copyContextProperties(cairoContext, shadowContext);
-    cairo_append_path(shadowContext, path.get());
+    copyContextProperties(cairoContext, cairoShadowContext);
 
-    if (drawingStyle & Fill)
-        setPlatformFill(context, shadowContext);
-    if (drawingStyle & Stroke)
-        setPlatformStroke(context, shadowContext);
+    if (drawingStyle & Fill) {
+        cairo_save(cairoShadowContext);
+        cairo_append_path(cairoShadowContext, path.get());
+        shadowContext->platformContext()->prepareForFilling(context->state(), PlatformContextCairo::NoAdjustment);
+        cairo_clip(cairoShadowContext);
+        cairo_paint(cairoShadowContext);
+        cairo_restore(cairoShadowContext);
+    }
 
-    shadow->endShadowLayer(context);
+    if (drawingStyle & Stroke) {
+        cairo_append_path(cairoShadowContext, path.get());
+        shadowContext->platformContext()->prepareForStroking(context->state(), PlatformContextCairo::DoNotPreserveAlpha);
+        cairo_stroke(cairoShadowContext);
+    }
+
+    shadow.endShadowLayer(context);
+
+    // ShadowBlur::endShadowLayer destroys the current path on the Cairo context. We restore it here.
+    cairo_new_path(cairoContext);
+    cairo_append_path(cairoContext, path.get());
 }
 
-static void fillCurrentCairoPath(GraphicsContext* context, cairo_t* cairoContext)
+static inline void shadowAndFillCurrentCairoPath(GraphicsContext* context)
 {
-    cairo_set_fill_rule(cairoContext, context->fillRule() == RULE_EVENODD ? CAIRO_FILL_RULE_EVEN_ODD : CAIRO_FILL_RULE_WINDING);
+    cairo_t* cr = context->platformContext()->cr();
+    cairo_save(cr);
+
+    context->platformContext()->prepareForFilling(context->state(), PlatformContextCairo::NoAdjustment);
+
     drawPathShadow(context, Fill);
 
-    setPlatformFill(context, cairoContext);
-    cairo_new_path(cairoContext);
+    cairo_clip(cr);
+    cairo_paint_with_alpha(cr, context->platformContext()->globalAlpha());
+    cairo_restore(cr);
 }
 
-static void strokeCurrentCairoPath(GraphicsContext* context,  cairo_t* cairoContext)
+static inline void shadowAndStrokeCurrentCairoPath(GraphicsContext* context)
 {
     drawPathShadow(context, Stroke);
-    setPlatformStroke(context, cairoContext);
-    cairo_new_path(cairoContext);
+    context->platformContext()->prepareForStroking(context->state());
+    cairo_stroke(context->platformContext()->cr());
 }
 
-void GraphicsContext::platformInit(PlatformGraphicsContext* cr)
+GraphicsContext::GraphicsContext(cairo_t* cr)
+    : m_updatingControlTints(false),
+      m_transparencyCount(0)
 {
-    m_data = new GraphicsContextPlatformPrivate;
-    m_data->cr = cairo_reference(cr);
-    m_data->syncContext(cr);
-    setPaintingDisabled(!cr);
+    m_data = new GraphicsContextPlatformPrivateToplevel(new PlatformContextCairo(cr));
+}
+
+void GraphicsContext::platformInit(PlatformContextCairo* platformContext)
+{
+    m_data = new GraphicsContextPlatformPrivate(platformContext);
+    if (platformContext)
+        m_data->syncContext(platformContext->cr());
+    else
+        setPaintingDisabled(true);
 }
 
 void GraphicsContext::platformDestroy()
@@ -214,45 +196,33 @@ void GraphicsContext::platformDestroy()
 
 AffineTransform GraphicsContext::getCTM() const
 {
-    cairo_t* cr = platformContext();
+    cairo_t* cr = platformContext()->cr();
     cairo_matrix_t m;
     cairo_get_matrix(cr, &m);
     return AffineTransform(m.xx, m.yx, m.xy, m.yy, m.x0, m.y0);
 }
 
-cairo_t* GraphicsContext::platformContext() const
+PlatformContextCairo* GraphicsContext::platformContext() const
 {
-    return m_data->cr;
+    return m_data->platformContext;
 }
 
 void GraphicsContext::savePlatformState()
 {
-    cairo_save(m_data->cr);
+    platformContext()->save();
     m_data->save();
-    m_data->shadowStack.append(m_data->shadow);
-    m_data->maskImageStack.append(ImageMaskInformation());
 }
 
 void GraphicsContext::restorePlatformState()
 {
-    cairo_t* cr = m_data->cr;
-    const ImageMaskInformation& maskInformation = m_data->maskImageStack.last();
-    if (maskInformation.isValid()) {
-        const FloatRect& maskRect = maskInformation.maskRect();
-        cairo_pop_group_to_source(cr);
-        cairo_mask_surface(cr, maskInformation.maskSurface(), maskRect.x(), maskRect.y());
-    }
-    m_data->maskImageStack.removeLast();
-
-    if (m_data->shadowStack.isEmpty())
-        m_data->shadow = ContextShadow();
-    else {
-        m_data->shadow = m_data->shadowStack.last();
-        m_data->shadowStack.removeLast();
-    }
-
-    cairo_restore(m_data->cr);
+    platformContext()->restore();
     m_data->restore();
+
+    platformContext()->shadowBlur().setShadowValues(FloatSize(m_state.shadowBlur, m_state.shadowBlur),
+                                                    m_state.shadowOffset,
+                                                    m_state.shadowColor,
+                                                    m_state.shadowColorSpace,
+                                                    m_state.shadowsIgnoreTransforms);
 }
 
 // Draws a filled rectangle with a stroked border.
@@ -261,11 +231,10 @@ void GraphicsContext::drawRect(const IntRect& rect)
     if (paintingDisabled())
         return;
 
-    cairo_t* cr = m_data->cr;
+    cairo_t* cr = platformContext()->cr();
     cairo_save(cr);
 
-    if (fillColor().alpha())
-        fillRectSourceOver(cr, rect, fillColor());
+    fillRectWithColor(cr, rect, fillColor());
 
     if (strokeStyle() != NoStroke) {
         setSourceRGBAFromColor(cr, strokeColor());
@@ -279,95 +248,94 @@ void GraphicsContext::drawRect(const IntRect& rect)
     cairo_restore(cr);
 }
 
-// This is only used to draw borders.
+static double calculateStrokePatternOffset(int distance, int patternWidth)
+{
+    // Example: 80 pixels with a width of 30 pixels. Remainder is 20.
+    // The maximum pixels of line we could paint will be 50 pixels.
+    int remainder = distance % patternWidth;
+    int numSegments = (distance - remainder) / patternWidth;
+
+    // Special case 1px dotted borders for speed.
+    if (patternWidth == 1)
+        return 1;
+
+    bool evenNumberOfSegments = !(numSegments % 2);
+    if (remainder)
+        evenNumberOfSegments = !evenNumberOfSegments;
+
+    if (evenNumberOfSegments) {
+        if (remainder)
+            return (patternWidth - remainder) + (remainder / 2);
+        return patternWidth / 2;
+    }
+
+    // Odd number of segments.
+    if (remainder)
+        return (patternWidth - remainder) / 2.f;
+    return 0;
+}
+
+static void drawLineOnCairoContext(GraphicsContext* graphicsContext, cairo_t* context, const FloatPoint& point1, const FloatPoint& point2)
+{
+    StrokeStyle style = graphicsContext->strokeStyle();
+    if (style == NoStroke)
+        return;
+
+    const Color& strokeColor = graphicsContext->strokeColor();
+    int strokeThickness = floorf(graphicsContext->strokeThickness());
+    if (graphicsContext->strokeThickness() < 1)
+        strokeThickness = 1;
+
+    int patternWidth = 0;
+    if (style == DottedStroke)
+        patternWidth = strokeThickness;
+    else if (style == DashedStroke)
+        patternWidth = 3 * strokeThickness;
+
+    bool isVerticalLine = point1.x() == point2.x();
+    FloatPoint point1OnPixelBoundaries = point1;
+    FloatPoint point2OnPixelBoundaries = point2;
+    GraphicsContext::adjustLineToPixelBoundaries(point1OnPixelBoundaries, point2OnPixelBoundaries, strokeThickness, style);
+
+    cairo_set_antialias(context, CAIRO_ANTIALIAS_NONE);
+    if (patternWidth) {
+        // Do a rect fill of our endpoints.  This ensures we always have the
+        // appearance of being a border.  We then draw the actual dotted/dashed line.
+        FloatRect firstRect(point1OnPixelBoundaries, FloatSize(strokeThickness, strokeThickness));
+        FloatRect secondRect(point2OnPixelBoundaries, FloatSize(strokeThickness, strokeThickness));
+        if (isVerticalLine) {
+            firstRect.move(-strokeThickness / 2, -strokeThickness);
+            secondRect.move(-strokeThickness / 2, 0);
+        } else {
+            firstRect.move(-strokeThickness, -strokeThickness / 2);
+            secondRect.move(0, -strokeThickness / 2);
+        }
+        fillRectWithColor(context, firstRect, strokeColor);
+        fillRectWithColor(context, secondRect, strokeColor);
+
+        int distance = (isVerticalLine ? (point2.y() - point1.y()) : (point2.x() - point1.x())) - 2 * strokeThickness;
+        double patternOffset = calculateStrokePatternOffset(distance, patternWidth);
+        double patternWidthAsDouble = patternWidth;
+        cairo_set_dash(context, &patternWidthAsDouble, 1, patternOffset);
+    }
+
+    setSourceRGBAFromColor(context, strokeColor);
+    cairo_set_line_width(context, strokeThickness);
+    cairo_move_to(context, point1OnPixelBoundaries.x(), point1OnPixelBoundaries.y());
+    cairo_line_to(context, point2OnPixelBoundaries.x(), point2OnPixelBoundaries.y());
+    cairo_stroke(context);
+}
+
+// This is only used to draw borders, so we should not draw shadows.
 void GraphicsContext::drawLine(const IntPoint& point1, const IntPoint& point2)
 {
     if (paintingDisabled())
         return;
 
-    StrokeStyle style = strokeStyle();
-    if (style == NoStroke)
-        return;
-
-    cairo_t* cr = m_data->cr;
-    cairo_save(cr);
-
-    float width = strokeThickness();
-    if (width < 1)
-        width = 1;
-
-    FloatPoint p1 = point1;
-    FloatPoint p2 = point2;
-    bool isVerticalLine = (p1.x() == p2.x());
-
-    adjustLineToPixelBoundaries(p1, p2, width, style);
-    cairo_set_line_width(cr, width);
-
-    int patWidth = 0;
-    switch (style) {
-    case NoStroke:
-    case SolidStroke:
-        break;
-    case DottedStroke:
-        patWidth = static_cast<int>(width);
-        break;
-    case DashedStroke:
-        patWidth = 3*static_cast<int>(width);
-        break;
-    }
-
-    setSourceRGBAFromColor(cr, strokeColor());
-
-    cairo_set_antialias(cr, CAIRO_ANTIALIAS_NONE);
-
-    if (patWidth) {
-        // Do a rect fill of our endpoints.  This ensures we always have the
-        // appearance of being a border.  We then draw the actual dotted/dashed line.
-        if (isVerticalLine) {
-            fillRectSourceOver(cr, FloatRect(p1.x() - width/2, p1.y() - width, width, width), strokeColor());
-            fillRectSourceOver(cr, FloatRect(p2.x() - width/2, p2.y(), width, width), strokeColor());
-        } else {
-            fillRectSourceOver(cr, FloatRect(p1.x() - width, p1.y() - width/2, width, width), strokeColor());
-            fillRectSourceOver(cr, FloatRect(p2.x(), p2.y() - width/2, width, width), strokeColor());
-        }
-
-        // Example: 80 pixels with a width of 30 pixels.
-        // Remainder is 20.  The maximum pixels of line we could paint
-        // will be 50 pixels.
-        int distance = (isVerticalLine ? (point2.y() - point1.y()) : (point2.x() - point1.x())) - 2*static_cast<int>(width);
-        int remainder = distance%patWidth;
-        int coverage = distance-remainder;
-        int numSegments = coverage/patWidth;
-
-        float patternOffset = 0;
-        // Special case 1px dotted borders for speed.
-        if (patWidth == 1)
-            patternOffset = 1.0;
-        else {
-            bool evenNumberOfSegments = !(numSegments % 2);
-            if (remainder)
-                evenNumberOfSegments = !evenNumberOfSegments;
-            if (evenNumberOfSegments) {
-                if (remainder) {
-                    patternOffset += patWidth - remainder;
-                    patternOffset += remainder / 2;
-                } else
-                    patternOffset = patWidth / 2;
-            } else if (!evenNumberOfSegments) {
-                if (remainder)
-                    patternOffset = (patWidth - remainder) / 2;
-            }
-        }
-
-        double dash = patWidth;
-        cairo_set_dash(cr, &dash, 1, patternOffset);
-    }
-
-    cairo_move_to(cr, p1.x(), p1.y());
-    cairo_line_to(cr, p2.x(), p2.y());
-
-    cairo_stroke(cr);
-    cairo_restore(cr);
+    cairo_t* cairoContext = platformContext()->cr();
+    cairo_save(cairoContext);
+    drawLineOnCairoContext(this, cairoContext, point1, point2);
+    cairo_restore(cairoContext);
 }
 
 // This method is only used to draw the little circles used in lists.
@@ -376,7 +344,7 @@ void GraphicsContext::drawEllipse(const IntRect& rect)
     if (paintingDisabled())
         return;
 
-    cairo_t* cr = m_data->cr;
+    cairo_t* cr = platformContext()->cr();
     cairo_save(cr);
     float yRadius = .5 * rect.height();
     float xRadius = .5 * rect.width();
@@ -415,7 +383,7 @@ void GraphicsContext::strokeArc(const IntRect& rect, int startAngle, int angleSp
     float fa = startAngle;
     float falen =  fa + angleSpan;
 
-    cairo_t* cr = m_data->cr;
+    cairo_t* cr = platformContext()->cr();
     cairo_save(cr);
 
     if (w != h)
@@ -426,15 +394,13 @@ void GraphicsContext::strokeArc(const IntRect& rect, int startAngle, int angleSp
     if (w != h)
         cairo_scale(cr, 1., reverseScaleFactor);
 
-    float width = strokeThickness();
-    int patWidth = 0;
-
+    int patternWidth = 0;
     switch (strokeStyle()) {
     case DottedStroke:
-        patWidth = static_cast<int>(width / 2);
+        patternWidth = floorf(strokeThickness() / 2.f);
         break;
     case DashedStroke:
-        patWidth = 3 * static_cast<int>(width / 2);
+        patternWidth = 3 * floorf(strokeThickness() / 2.f);
         break;
     default:
         break;
@@ -442,42 +408,15 @@ void GraphicsContext::strokeArc(const IntRect& rect, int startAngle, int angleSp
 
     setSourceRGBAFromColor(cr, strokeColor());
 
-    if (patWidth) {
-        // Example: 80 pixels with a width of 30 pixels.
-        // Remainder is 20.  The maximum pixels of line we could paint
-        // will be 50 pixels.
-        int distance;
+    if (patternWidth) {
+        float distance = 0;
         if (hRadius == vRadius)
-            distance = static_cast<int>((M_PI * hRadius) / 2.0);
+            distance = (piFloat * hRadius) / 2.f;
         else // We are elliptical and will have to estimate the distance
-            distance = static_cast<int>((M_PI * sqrtf((hRadius * hRadius + vRadius * vRadius) / 2.0)) / 2.0);
-
-        int remainder = distance % patWidth;
-        int coverage = distance - remainder;
-        int numSegments = coverage / patWidth;
-
-        float patternOffset = 0.0;
-        // Special case 1px dotted borders for speed.
-        if (patWidth == 1)
-            patternOffset = 1.0;
-        else {
-            bool evenNumberOfSegments = !(numSegments % 2);
-            if (remainder)
-                evenNumberOfSegments = !evenNumberOfSegments;
-            if (evenNumberOfSegments) {
-                if (remainder) {
-                    patternOffset += patWidth - remainder;
-                    patternOffset += remainder / 2.0;
-                } else
-                    patternOffset = patWidth / 2.0;
-            } else {
-                if (remainder)
-                    patternOffset = (patWidth - remainder) / 2.0;
-            }
-        }
-
-        double dash = patWidth;
-        cairo_set_dash(cr, &dash, 1, patternOffset);
+            distance = (piFloat * sqrtf((hRadius * hRadius + vRadius * vRadius) / 2.f)) / 2.f;
+        double patternOffset = calculateStrokePatternOffset(floorf(distance), patternWidth);
+        double patternWidthAsDouble = patternWidth;
+        cairo_set_dash(cr, &patternWidthAsDouble, 1, patternOffset);
     }
 
     cairo_stroke(cr);
@@ -492,7 +431,7 @@ void GraphicsContext::drawConvexPolygon(size_t npoints, const FloatPoint* points
     if (npoints <= 1)
         return;
 
-    cairo_t* cr = m_data->cr;
+    cairo_t* cr = platformContext()->cr();
 
     cairo_save(cr);
     cairo_set_antialias(cr, shouldAntialias ? CAIRO_ANTIALIAS_DEFAULT : CAIRO_ANTIALIAS_NONE);
@@ -522,7 +461,7 @@ void GraphicsContext::clipConvexPolygon(size_t numPoints, const FloatPoint* poin
     if (numPoints <= 1)
         return;
 
-    cairo_t* cr = m_data->cr;
+    cairo_t* cr = platformContext()->cr();
 
     cairo_new_path(cr);
     cairo_fill_rule_t savedFillRule = cairo_get_fill_rule(cr);
@@ -542,9 +481,9 @@ void GraphicsContext::fillPath(const Path& path)
     if (paintingDisabled())
         return;
 
-    cairo_t* cr = m_data->cr;
+    cairo_t* cr = platformContext()->cr();
     setPathOnCairoContext(cr, path.platformPath()->context());
-    fillCurrentCairoPath(this, cr);
+    shadowAndFillCurrentCairoPath(this);
 }
 
 void GraphicsContext::strokePath(const Path& path)
@@ -552,9 +491,9 @@ void GraphicsContext::strokePath(const Path& path)
     if (paintingDisabled())
         return;
 
-    cairo_t* cr = m_data->cr;
+    cairo_t* cr = platformContext()->cr();
     setPathOnCairoContext(cr, path.platformPath()->context());
-    strokeCurrentCairoPath(this, cr);
+    shadowAndStrokeCurrentCairoPath(this);
 }
 
 void GraphicsContext::fillRect(const FloatRect& rect)
@@ -562,11 +501,9 @@ void GraphicsContext::fillRect(const FloatRect& rect)
     if (paintingDisabled())
         return;
 
-    cairo_t* cr = m_data->cr;
-    cairo_save(cr);
+    cairo_t* cr = platformContext()->cr();
     cairo_rectangle(cr, rect.x(), rect.y(), rect.width(), rect.height());
-    fillCurrentCairoPath(this, cr);
-    cairo_restore(cr);
+    shadowAndFillCurrentCairoPath(this);
 }
 
 void GraphicsContext::fillRect(const FloatRect& rect, const Color& color, ColorSpace)
@@ -575,10 +512,9 @@ void GraphicsContext::fillRect(const FloatRect& rect, const Color& color, ColorS
         return;
 
     if (hasShadow())
-        m_data->shadow.drawRectShadow(this, enclosingIntRect(rect));
+        platformContext()->shadowBlur().drawRectShadow(this, rect, RoundedRect::Radii());
 
-    if (color.alpha())
-        fillRectSourceOver(m_data->cr, rect, color);
+    fillRectWithColor(platformContext()->cr(), rect, color);
 }
 
 void GraphicsContext::clip(const FloatRect& rect)
@@ -586,7 +522,7 @@ void GraphicsContext::clip(const FloatRect& rect)
     if (paintingDisabled())
         return;
 
-    cairo_t* cr = m_data->cr;
+    cairo_t* cr = platformContext()->cr();
     cairo_rectangle(cr, rect.x(), rect.y(), rect.width(), rect.height());
     cairo_fill_rule_t savedFillRule = cairo_get_fill_rule(cr);
     cairo_set_fill_rule(cr, CAIRO_FILL_RULE_WINDING);
@@ -600,12 +536,19 @@ void GraphicsContext::clipPath(const Path& path, WindRule clipRule)
     if (paintingDisabled())
         return;
 
-    cairo_t* cr = m_data->cr;
+    cairo_t* cr = platformContext()->cr();
     setPathOnCairoContext(cr, path.platformPath()->context());
     cairo_set_fill_rule(cr, clipRule == RULE_EVENODD ? CAIRO_FILL_RULE_EVEN_ODD : CAIRO_FILL_RULE_WINDING);
     cairo_clip(cr);
 }
 
+IntRect GraphicsContext::clipBounds() const
+{
+    double x1, x2, y1, y2;
+    cairo_clip_extents(platformContext()->cr(), &x1, &y1, &x2, &y2);
+    return enclosingIntRect(FloatRect(x1, y1, x2 - x1, y2 - y1));
+}
+
 static inline void adjustFocusRingColor(Color& color)
 {
 #if !PLATFORM(GTK)
@@ -638,7 +581,7 @@ void GraphicsContext::drawFocusRing(const Path& path, int width, int /* offset *
     adjustFocusRingColor(ringColor);
     adjustFocusRingLineWidth(width);
 
-    cairo_t* cr = m_data->cr;
+    cairo_t* cr = platformContext()->cr();
     cairo_save(cr);
     appendWebCorePathToCairoContext(cr, path);
     setSourceRGBAFromColor(cr, ringColor);
@@ -655,7 +598,7 @@ void GraphicsContext::drawFocusRing(const Vector<IntRect>& rects, int width, int
 
     unsigned rectCount = rects.size();
 
-    cairo_t* cr = m_data->cr;
+    cairo_t* cr = platformContext()->cr();
     cairo_save(cr);
     cairo_push_group(cr);
     cairo_new_path(cr);
@@ -717,10 +660,21 @@ void GraphicsContext::drawLineForText(const FloatPoint& origin, float width, boo
     if (paintingDisabled())
         return;
 
-    FloatPoint endPoint = origin + FloatSize(width, 0);
-    
-    // FIXME: Loss of precision here. Might consider rounding.
-    drawLine(IntPoint(origin.x(), origin.y()), IntPoint(endPoint.x(), endPoint.y()));
+    cairo_t* cairoContext = platformContext()->cr();
+    cairo_save(cairoContext);
+
+    // This bumping of <1 stroke thicknesses matches the one in drawLineOnCairoContext.
+    FloatPoint endPoint(origin + IntSize(width, 0));
+    FloatRect lineExtents(origin, FloatSize(width, strokeThickness()));
+
+    ShadowBlur& shadow = platformContext()->shadowBlur();
+    if (GraphicsContext* shadowContext = shadow.beginShadowLayer(this, lineExtents)) {
+        drawLineOnCairoContext(this, shadowContext->platformContext()->cr(), origin, endPoint);
+        shadow.endShadowLayer(this);
+    }
+
+    drawLineOnCairoContext(this, cairoContext, origin, endPoint);
+    cairo_restore(cairoContext);
 }
 
 #if !PLATFORM(GTK)
@@ -732,7 +686,7 @@ void GraphicsContext::drawLineForTextChecking(const FloatPoint& origin, float wi
     if (paintingDisabled())
         return;
 
-    cairo_t* cr = m_data->cr;
+    cairo_t* cr = platformContext()->cr();
     cairo_save(cr);
 
     switch (style) {
@@ -757,12 +711,12 @@ void GraphicsContext::drawLineForTextChecking(const FloatPoint& origin, float wi
     cairo_restore(cr);
 }
 
-FloatRect GraphicsContext::roundToDevicePixels(const FloatRect& frect)
+FloatRect GraphicsContext::roundToDevicePixels(const FloatRect& frect, RoundingMode)
 {
     FloatRect result;
     double x = frect.x();
     double y = frect.y();
-    cairo_t* cr = m_data->cr;
+    cairo_t* cr = platformContext()->cr();
     cairo_user_to_device(cr, &x, &y);
     x = round(x);
     y = round(y);
@@ -799,7 +753,7 @@ void GraphicsContext::translate(float x, float y)
     if (paintingDisabled())
         return;
 
-    cairo_t* cr = m_data->cr;
+    cairo_t* cr = platformContext()->cr();
     cairo_translate(cr, x, y);
     m_data->translate(x, y);
 }
@@ -821,7 +775,7 @@ void GraphicsContext::setPlatformStrokeThickness(float strokeThickness)
     if (paintingDisabled())
         return;
 
-    cairo_set_line_width(m_data->cr, strokeThickness);
+    cairo_set_line_width(platformContext()->cr(), strokeThickness);
 }
 
 void GraphicsContext::setPlatformStrokeStyle(StrokeStyle strokeStyle)
@@ -835,16 +789,16 @@ void GraphicsContext::setPlatformStrokeStyle(StrokeStyle strokeStyle)
     switch (strokeStyle) {
     case NoStroke:
         // FIXME: is it the right way to emulate NoStroke?
-        cairo_set_line_width(m_data->cr, 0);
+        cairo_set_line_width(platformContext()->cr(), 0);
         break;
     case SolidStroke:
-        cairo_set_dash(m_data->cr, 0, 0, 0);
+        cairo_set_dash(platformContext()->cr(), 0, 0, 0);
         break;
     case DottedStroke:
-        cairo_set_dash(m_data->cr, dotPattern, 2, 0);
+        cairo_set_dash(platformContext()->cr(), dotPattern, 2, 0);
         break;
     case DashedStroke:
-        cairo_set_dash(m_data->cr, dashPattern, 2, 0);
+        cairo_set_dash(platformContext()->cr(), dashPattern, 2, 0);
         break;
     }
 }
@@ -859,7 +813,7 @@ void GraphicsContext::concatCTM(const AffineTransform& transform)
     if (paintingDisabled())
         return;
 
-    cairo_t* cr = m_data->cr;
+    cairo_t* cr = platformContext()->cr();
     const cairo_matrix_t matrix = cairo_matrix_t(transform);
     cairo_transform(cr, &matrix);
     m_data->concatCTM(transform);
@@ -870,7 +824,7 @@ void GraphicsContext::setCTM(const AffineTransform& transform)
     if (paintingDisabled())
         return;
 
-    cairo_t* cr = m_data->cr;
+    cairo_t* cr = platformContext()->cr();
     const cairo_matrix_t matrix = cairo_matrix_t(transform);
     cairo_set_matrix(cr, &matrix);
     m_data->setCTM(transform);
@@ -881,7 +835,7 @@ void GraphicsContext::addInnerRoundedRectClip(const IntRect& rect, int thickness
     if (paintingDisabled())
         return;
 
-    cairo_t* cr = m_data->cr;
+    cairo_t* cr = platformContext()->cr();
     clip(rect);
 
     Path p;
@@ -901,50 +855,56 @@ void GraphicsContext::addInnerRoundedRectClip(const IntRect& rect, int thickness
 
 void GraphicsContext::setPlatformShadow(FloatSize const& size, float blur, Color const& color, ColorSpace)
 {
-    // Cairo doesn't support shadows natively, they are drawn manually in the draw* functions
+    if (paintingDisabled())
+        return;
+
     if (m_state.shadowsIgnoreTransforms) {
         // Meaning that this graphics context is associated with a CanvasRenderingContext
         // We flip the height since CG and HTML5 Canvas have opposite Y axis
         m_state.shadowOffset = FloatSize(size.width(), -size.height());
-        m_data->shadow = ContextShadow(color, blur, FloatSize(size.width(), -size.height()));
-    } else
-        m_data->shadow = ContextShadow(color, blur, FloatSize(size.width(), size.height()));
+    }
 
-    m_data->shadow.setShadowsIgnoreTransforms(m_state.shadowsIgnoreTransforms);
-}
-
-ContextShadow* GraphicsContext::contextShadow()
-{
-    return &m_data->shadow;
+    // Cairo doesn't support shadows natively, they are drawn manually in the draw* functions using ShadowBlur.
+    platformContext()->shadowBlur().setShadowValues(FloatSize(m_state.shadowBlur, m_state.shadowBlur),
+                                                    m_state.shadowOffset,
+                                                    m_state.shadowColor,
+                                                    m_state.shadowColorSpace,
+                                                    m_state.shadowsIgnoreTransforms);
 }
 
 void GraphicsContext::clearPlatformShadow()
 {
-    m_data->shadow.clear();
+    if (paintingDisabled())
+        return;
+
+    platformContext()->shadowBlur().clear();
 }
 
-void GraphicsContext::beginTransparencyLayer(float opacity)
+void GraphicsContext::beginPlatformTransparencyLayer(float opacity)
 {
     if (paintingDisabled())
         return;
 
-    cairo_t* cr = m_data->cr;
+    cairo_t* cr = platformContext()->cr();
     cairo_push_group(cr);
     m_data->layers.append(opacity);
-    m_data->beginTransparencyLayer();
 }
 
-void GraphicsContext::endTransparencyLayer()
+void GraphicsContext::endPlatformTransparencyLayer()
 {
     if (paintingDisabled())
         return;
 
-    cairo_t* cr = m_data->cr;
+    cairo_t* cr = platformContext()->cr();
 
     cairo_pop_group_to_source(cr);
     cairo_paint_with_alpha(cr, m_data->layers.last());
     m_data->layers.removeLast();
-    m_data->endTransparencyLayer();
+}
+
+bool GraphicsContext::supportsTransparencyLayers()
+{
+    return true;
 }
 
 void GraphicsContext::clearRect(const FloatRect& rect)
@@ -952,7 +912,7 @@ void GraphicsContext::clearRect(const FloatRect& rect)
     if (paintingDisabled())
         return;
 
-    cairo_t* cr = m_data->cr;
+    cairo_t* cr = platformContext()->cr();
 
     cairo_save(cr);
     cairo_rectangle(cr, rect.x(), rect.y(), rect.width(), rect.height());
@@ -966,11 +926,11 @@ void GraphicsContext::strokeRect(const FloatRect& rect, float width)
     if (paintingDisabled())
         return;
 
-    cairo_t* cr = m_data->cr;
+    cairo_t* cr = platformContext()->cr();
     cairo_save(cr);
     cairo_rectangle(cr, rect.x(), rect.y(), rect.width(), rect.height());
     cairo_set_line_width(cr, width);
-    strokeCurrentCairoPath(this, cr);
+    shadowAndStrokeCurrentCairoPath(this);
     cairo_restore(cr);
 }
 
@@ -991,12 +951,12 @@ void GraphicsContext::setLineCap(LineCap lineCap)
         cairoCap = CAIRO_LINE_CAP_SQUARE;
         break;
     }
-    cairo_set_line_cap(m_data->cr, cairoCap);
+    cairo_set_line_cap(platformContext()->cr(), cairoCap);
 }
 
 void GraphicsContext::setLineDash(const DashArray& dashes, float dashOffset)
 {
-    cairo_set_dash(m_data->cr, dashes.data(), dashes.size(), dashOffset);
+    cairo_set_dash(platformContext()->cr(), dashes.data(), dashes.size(), dashOffset);
 }
 
 void GraphicsContext::setLineJoin(LineJoin lineJoin)
@@ -1016,7 +976,7 @@ void GraphicsContext::setLineJoin(LineJoin lineJoin)
         cairoJoin = CAIRO_LINE_JOIN_BEVEL;
         break;
     }
-    cairo_set_line_join(m_data->cr, cairoJoin);
+    cairo_set_line_join(platformContext()->cr(), cairoJoin);
 }
 
 void GraphicsContext::setMiterLimit(float miter)
@@ -1024,17 +984,12 @@ void GraphicsContext::setMiterLimit(float miter)
     if (paintingDisabled())
         return;
 
-    cairo_set_miter_limit(m_data->cr, miter);
+    cairo_set_miter_limit(platformContext()->cr(), miter);
 }
 
 void GraphicsContext::setAlpha(float alpha)
 {
-    m_state.globalAlpha = alpha;
-}
-
-float GraphicsContext::getAlpha()
-{
-    return m_state.globalAlpha;
+    platformContext()->setGlobalAlpha(alpha);
 }
 
 void GraphicsContext::setPlatformCompositeOperation(CompositeOperator op)
@@ -1042,7 +997,7 @@ void GraphicsContext::setPlatformCompositeOperation(CompositeOperator op)
     if (paintingDisabled())
         return;
 
-    cairo_set_operator(m_data->cr, toCairoOperator(op));
+    cairo_set_operator(platformContext()->cr(), toCairoOperator(op));
 }
 
 void GraphicsContext::clip(const Path& path)
@@ -1050,9 +1005,9 @@ void GraphicsContext::clip(const Path& path)
     if (paintingDisabled())
         return;
 
-    cairo_t* cr = m_data->cr;
-    OwnPtr<cairo_path_t> p(cairo_copy_path(path.platformPath()->context()));
-    cairo_append_path(cr, p.get());
+    cairo_t* cr = platformContext()->cr();
+    OwnPtr<cairo_path_t> pathCopy = adoptPtr(cairo_copy_path(path.platformPath()->context()));
+    cairo_append_path(cr, pathCopy.get());
     cairo_fill_rule_t savedFillRule = cairo_get_fill_rule(cr);
     cairo_set_fill_rule(cr, CAIRO_FILL_RULE_WINDING);
     cairo_clip(cr);
@@ -1070,7 +1025,7 @@ void GraphicsContext::clipOut(const Path& path)
     if (paintingDisabled())
         return;
 
-    cairo_t* cr = m_data->cr;
+    cairo_t* cr = platformContext()->cr();
     double x1, y1, x2, y2;
     cairo_clip_extents(cr, &x1, &y1, &x2, &y2);
     cairo_rectangle(cr, x1, y1, x2 - x1, y2 - y1);
@@ -1087,7 +1042,7 @@ void GraphicsContext::rotate(float radians)
     if (paintingDisabled())
         return;
 
-    cairo_rotate(m_data->cr, radians);
+    cairo_rotate(platformContext()->cr(), radians);
     m_data->rotate(radians);
 }
 
@@ -1096,7 +1051,7 @@ void GraphicsContext::scale(const FloatSize& size)
     if (paintingDisabled())
         return;
 
-    cairo_scale(m_data->cr, size.width(), size.height());
+    cairo_scale(platformContext()->cr(), size.width(), size.height());
     m_data->scale(size);
 }
 
@@ -1105,7 +1060,7 @@ void GraphicsContext::clipOut(const IntRect& r)
     if (paintingDisabled())
         return;
 
-    cairo_t* cr = m_data->cr;
+    cairo_t* cr = platformContext()->cr();
     double x1, y1, x2, y2;
     cairo_clip_extents(cr, &x1, &y1, &x2, &y2);
     cairo_rectangle(cr, x1, y1, x2 - x1, y2 - y1);
@@ -1130,9 +1085,9 @@ void GraphicsContext::fillRoundedRect(const IntRect& r, const IntSize& topLeft,
         return;
 
     if (hasShadow())
-        m_data->shadow.drawRectShadow(this, r, topLeft, topRight, bottomLeft, bottomRight);
+        platformContext()->shadowBlur().drawRectShadow(this, r, RoundedRect::Radii(topLeft, topRight, bottomLeft, bottomRight));
 
-    cairo_t* cr = m_data->cr;
+    cairo_t* cr = platformContext()->cr();
     cairo_save(cr);
     Path path;
     path.addRoundedRect(r, topLeft, topRight, bottomLeft, bottomRight);
@@ -1170,45 +1125,19 @@ void GraphicsContext::setPlatformShouldAntialias(bool enable)
     // When true, use the default Cairo backend antialias mode (usually this
     // enables standard 'grayscale' antialiasing); false to explicitly disable
     // antialiasing. This is the same strategy as used in drawConvexPolygon().
-    cairo_set_antialias(m_data->cr, enable ? CAIRO_ANTIALIAS_DEFAULT : CAIRO_ANTIALIAS_NONE);
+    cairo_set_antialias(platformContext()->cr(), enable ? CAIRO_ANTIALIAS_DEFAULT : CAIRO_ANTIALIAS_NONE);
 }
 
-void GraphicsContext::setImageInterpolationQuality(InterpolationQuality)
+void GraphicsContext::setImageInterpolationQuality(InterpolationQuality quality)
 {
+    platformContext()->setImageInterpolationQuality(quality);
 }
 
 InterpolationQuality GraphicsContext::imageInterpolationQuality() const
 {
-    return InterpolationDefault;
-}
-
-void GraphicsContext::pushImageMask(cairo_surface_t* surface, const FloatRect& rect)
-{
-    // We must call savePlatformState at least once before we can use image masking,
-    // since we actually apply the mask in restorePlatformState.
-    ASSERT(!m_data->maskImageStack.isEmpty());
-    m_data->maskImageStack.last().update(surface, rect);
-
-    // Cairo doesn't support the notion of an image clip, so we push a group here
-    // and then paint it to the surface with an image mask (which is an immediate
-    // operation) during restorePlatformState.
-
-    // We want to allow the clipped elements to composite with the surface as it
-    // is now, but they are isolated in another group. To make this work, we're
-    // going to blit the current surface contents onto the new group once we push it.
-    cairo_t* cr = m_data->cr;
-    cairo_surface_t* currentTarget = cairo_get_target(cr);
-    cairo_surface_flush(currentTarget);
-
-    // Pushing a new group ensures that only things painted after this point are clipped.
-    cairo_push_group(cr);
-    cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
-
-    cairo_set_source_surface(cr, currentTarget, 0, 0);
-    cairo_rectangle(cr, rect.x(), rect.y(), rect.width(), rect.height());
-    cairo_fill(cr);
+    return platformContext()->imageInterpolationQuality();
 }
 
 } // namespace WebCore
 
-#endif // PLATFORM(CAIRO)
+#endif // USE(CAIRO)
