@@ -29,7 +29,8 @@
 #if ENABLE(FILTERS)
 #include "FELighting.h"
 
-#include "LightSource.h"
+#include "FELightingNEON.h"
+#include <wtf/ParallelJobs.h>
 
 namespace WebCore {
 
@@ -227,6 +228,70 @@ void FELighting::setPixel(int offset, LightingData& data, LightSource::PaintingD
     inlineSetPixel(offset, data, paintingData, lightX, lightY, factorX, factorY, normalVector);
 }
 
+inline void FELighting::platformApplyGenericPaint(LightingData& data, LightSource::PaintingData& paintingData, int startY, int endY)
+{
+    IntPoint normalVector;
+    int offset = 0;
+
+    for (int y = startY; y < endY; ++y) {
+        offset = y * data.widthMultipliedByPixelSize + cPixelSize;
+        for (int x = 1; x < data.widthDecreasedByOne; ++x, offset += cPixelSize) {
+            data.interior(offset, normalVector);
+            inlineSetPixel(offset, data, paintingData, x, y, cFactor1div4, cFactor1div4, normalVector);
+        }
+    }
+}
+
+#if ENABLE(PARALLEL_JOBS)
+void FELighting::platformApplyGenericWorker(PlatformApplyGenericParameters* parameters)
+{
+    parameters->filter->platformApplyGenericPaint(parameters->data, parameters->paintingData, parameters->yStart, parameters->yEnd);
+}
+#endif // ENABLE(PARALLEL_JOBS)
+
+inline void FELighting::platformApplyGeneric(LightingData& data, LightSource::PaintingData& paintingData)
+{
+#if ENABLE(PARALLEL_JOBS)
+    int optimalThreadNumber = ((data.widthDecreasedByOne - 1) * (data.heightDecreasedByOne - 1)) / s_minimalRectDimension;
+    if (optimalThreadNumber > 1) {
+        // Initialize parallel jobs
+        ParallelJobs<PlatformApplyGenericParameters> parallelJobs(&platformApplyGenericWorker, optimalThreadNumber);
+
+        // Fill the parameter array
+        int job = parallelJobs.numberOfJobs();
+        if (job > 1) {
+            int yStart = 1;
+            int yStep = (data.heightDecreasedByOne - 1) / job;
+            for (--job; job >= 0; --job) {
+                PlatformApplyGenericParameters& params = parallelJobs.parameter(job);
+                params.filter = this;
+                params.data = data;
+                params.paintingData = paintingData;
+                params.yStart = yStart;
+                if (job > 0) {
+                    params.yEnd = yStart + yStep;
+                    yStart += yStep;
+                } else
+                    params.yEnd = data.heightDecreasedByOne;
+            }
+            parallelJobs.execute();
+            return;
+        }
+    }
+#endif
+    platformApplyGenericPaint(data, paintingData, 1, data.heightDecreasedByOne);
+}
+
+inline void FELighting::platformApply(LightingData& data, LightSource::PaintingData& paintingData)
+{
+    // The selection here eventually should happen dynamically on some platforms.
+#if CPU(ARM_NEON) && COMPILER(GCC)
+    platformApplyNeon(data, paintingData);
+#else
+    platformApplyGeneric(data, paintingData);
+#endif
+}
+
 bool FELighting::drawLighting(ByteArray* pixels, int width, int height)
 {
     LightSource::PaintingData paintingData;
@@ -248,35 +313,35 @@ bool FELighting::drawLighting(ByteArray* pixels, int width, int height)
     paintingData.colorVector = FloatPoint3D(m_lightingColor.red(), m_lightingColor.green(), m_lightingColor.blue());
     m_lightSource->initPaintingData(paintingData);
 
-    // Top/Left corner
+    // Top/Left corner.
     IntPoint normalVector;
     int offset = 0;
     data.topLeft(offset, normalVector);
     setPixel(offset, data, paintingData, 0, 0, cFactor2div3, cFactor2div3, normalVector);
 
-    // Top/Right pixel
+    // Top/Right pixel.
     offset = data.widthMultipliedByPixelSize - cPixelSize;
     data.topRight(offset, normalVector);
     setPixel(offset, data, paintingData, data.widthDecreasedByOne, 0, cFactor2div3, cFactor2div3, normalVector);
 
-    // Bottom/Left pixel
+    // Bottom/Left pixel.
     offset = data.heightDecreasedByOne * data.widthMultipliedByPixelSize;
     data.bottomLeft(offset, normalVector);
     setPixel(offset, data, paintingData, 0, data.heightDecreasedByOne, cFactor2div3, cFactor2div3, normalVector);
 
-    // Bottom/Right pixel
+    // Bottom/Right pixel.
     offset = height * data.widthMultipliedByPixelSize - cPixelSize;
     data.bottomRight(offset, normalVector);
     setPixel(offset, data, paintingData, data.widthDecreasedByOne, data.heightDecreasedByOne, cFactor2div3, cFactor2div3, normalVector);
 
     if (width >= 3) {
-        // Top row
+        // Top row.
         offset = cPixelSize;
         for (int x = 1; x < data.widthDecreasedByOne; ++x, offset += cPixelSize) {
             data.topRow(offset, normalVector);
             inlineSetPixel(offset, data, paintingData, x, 0, cFactor1div3, cFactor1div2, normalVector);
         }
-        // Bottom row
+        // Bottom row.
         offset = data.heightDecreasedByOne * data.widthMultipliedByPixelSize + cPixelSize;
         for (int x = 1; x < data.widthDecreasedByOne; ++x, offset += cPixelSize) {
             data.bottomRow(offset, normalVector);
@@ -285,13 +350,13 @@ bool FELighting::drawLighting(ByteArray* pixels, int width, int height)
     }
 
     if (height >= 3) {
-        // Left column
+        // Left column.
         offset = data.widthMultipliedByPixelSize;
         for (int y = 1; y < data.heightDecreasedByOne; ++y, offset += data.widthMultipliedByPixelSize) {
             data.leftColumn(offset, normalVector);
             inlineSetPixel(offset, data, paintingData, 0, y, cFactor1div2, cFactor1div3, normalVector);
         }
-        // Right column
+        // Right column.
         offset = (data.widthMultipliedByPixelSize << 1) - cPixelSize;
         for (int y = 1; y < data.heightDecreasedByOne; ++y, offset += data.widthMultipliedByPixelSize) {
             data.rightColumn(offset, normalVector);
@@ -300,14 +365,8 @@ bool FELighting::drawLighting(ByteArray* pixels, int width, int height)
     }
 
     if (width >= 3 && height >= 3) {
-        // Interior pixels
-        for (int y = 1; y < data.heightDecreasedByOne; ++y) {
-            offset = y * data.widthMultipliedByPixelSize + cPixelSize;
-            for (int x = 1; x < data.widthDecreasedByOne; ++x, offset += cPixelSize) {
-                data.interior(offset, normalVector);
-                inlineSetPixel(offset, data, paintingData, x, y, cFactor1div4, cFactor1div4, normalVector);
-            }
-        }
+        // Interior pixels.
+        platformApply(data, paintingData);
     }
 
     int lastPixel = data.widthMultipliedByPixelSize * height;
