@@ -90,13 +90,14 @@ void SimpleFontData::platformDestroy()
 {
 }
 
-SimpleFontData* SimpleFontData::scaledFontData(const FontDescription& fontDescription, float scaleFactor) const
+PassOwnPtr<SimpleFontData> SimpleFontData::createScaledFontData(const FontDescription& fontDescription, float scaleFactor) const
 {
-    return new SimpleFontData(FontPlatformData(cairo_scaled_font_get_font_face(m_platformData.scaledFont()),
-                                               scaleFactor * fontDescription.computedSize(),
-                                               m_platformData.syntheticBold(),
-                                               m_platformData.syntheticOblique()),
-                              isCustomFont(), false);
+    ASSERT(m_platformData.scaledFont());
+    return adoptPtr(new SimpleFontData(FontPlatformData(cairo_scaled_font_get_font_face(m_platformData.scaledFont()),
+                                                        scaleFactor * fontDescription.computedSize(),
+                                                        m_platformData.syntheticBold(),
+                                                        m_platformData.syntheticOblique()),
+                                       isCustomFont(), false));
 }
 
 SimpleFontData* SimpleFontData::smallCapsFontData(const FontDescription& fontDescription) const
@@ -105,7 +106,7 @@ SimpleFontData* SimpleFontData::smallCapsFontData(const FontDescription& fontDes
         m_derivedFontData = DerivedFontData::create(isCustomFont());
     // FIXME: I think we want to ask FontConfig for the right font again.
     if (!m_derivedFontData->smallCaps)
-        m_derivedFontData->smallCaps = scaledFontData(fontDescription, .7);
+        m_derivedFontData->smallCaps = createScaledFontData(fontDescription, .7);
 
     return m_derivedFontData->smallCaps.get();
 }
@@ -115,7 +116,7 @@ SimpleFontData* SimpleFontData::emphasisMarkFontData(const FontDescription& font
     if (!m_derivedFontData)
         m_derivedFontData = DerivedFontData::create(isCustomFont());
     if (!m_derivedFontData->emphasisMark)
-        m_derivedFontData->emphasisMark = scaledFontData(fontDescription, .5);
+        m_derivedFontData->emphasisMark = createScaledFontData(fontDescription, .5);
 
     return m_derivedFontData->emphasisMark.get();
 }
