@@ -26,20 +26,26 @@
 #include "RenderThemeGtk.h"
 
 #include "CSSValueKeywords.h"
+#include "FileSystem.h"
 #include "GOwnPtr.h"
 #include "Gradient.h"
 #include "GraphicsContext.h"
 #include "GtkVersioning.h"
 #include "HTMLMediaElement.h"
 #include "HTMLNames.h"
+#include "LocalizedStrings.h"
 #include "MediaControlElements.h"
 #include "PaintInfo.h"
+#include "PlatformContextCairo.h"
 #include "RenderBox.h"
 #include "RenderObject.h"
+#include "StringTruncator.h"
 #include "TimeRanges.h"
 #include "UserAgentStyleSheets.h"
 #include <gdk/gdk.h>
+#include <glib.h>
 #include <gtk/gtk.h>
+#include <wtf/text/CString.h>
 
 #if ENABLE(PROGRESS_TAG)
 #include "RenderProgress.h"
@@ -47,6 +53,10 @@
 
 namespace WebCore {
 
+// This would be a static method, except that forward declaring GType is tricky, since its
+// definition depends on including glib.h, negating the benefit of using a forward declaration.
+extern GRefPtr<GdkPixbuf> getStockIconForWidgetType(GType, const char* iconName, gint direction, gint state, gint iconSize);
+
 using namespace HTMLNames;
 
 #if ENABLE(VIDEO)
@@ -54,7 +64,7 @@ static HTMLMediaElement* getMediaElementFromRenderObject(RenderObject* o)
 {
     Node* node = o->node();
     Node* mediaNode = node ? node->shadowAncestorNode() : 0;
-    if (!mediaNode || (!mediaNode->hasTagName(videoTag) && !mediaNode->hasTagName(audioTag)))
+    if (!mediaNode || !mediaNode->isElementNode() || !static_cast<Element*>(mediaNode)->isMediaElement())
         return 0;
 
     return static_cast<HTMLMediaElement*>(mediaNode);
@@ -234,7 +244,7 @@ static void paintGdkPixbuf(GraphicsContext* context, const GdkPixbuf* icon, cons
         icon = scaledIcon.get();
     }
 
-    cairo_t* cr = context->platformContext();
+    cairo_t* cr = context->platformContext()->cr();
     cairo_save(cr);
     gdk_cairo_set_source_pixbuf(cr, icon, iconRect.x(), iconRect.y());
     cairo_paint(cr);
@@ -322,10 +332,10 @@ bool RenderThemeGtk::paintSearchFieldResultsDecoration(RenderObject* renderObjec
     if (iconRect.isEmpty())
         return false;
 
-    GRefPtr<GdkPixbuf> icon = getStockIcon(GTK_TYPE_ENTRY, GTK_STOCK_FIND,
-                                           gtkTextDirection(renderObject->style()->direction()),
-                                           gtkIconState(this, renderObject),
-                                           getIconSizeForPixelSize(rect.height()));
+    GRefPtr<GdkPixbuf> icon = getStockIconForWidgetType(GTK_TYPE_ENTRY, GTK_STOCK_FIND,
+                                                        gtkTextDirection(renderObject->style()->direction()),
+                                                        gtkIconState(this, renderObject),
+                                                        getIconSizeForPixelSize(rect.height()));
     paintGdkPixbuf(paintInfo.context, icon.get(), iconRect);
     return false;
 }
@@ -341,10 +351,10 @@ bool RenderThemeGtk::paintSearchFieldCancelButton(RenderObject* renderObject, co
     if (iconRect.isEmpty())
         return false;
 
-    GRefPtr<GdkPixbuf> icon = getStockIcon(GTK_TYPE_ENTRY, GTK_STOCK_CLEAR,
-                                           gtkTextDirection(renderObject->style()->direction()),
-                                           gtkIconState(this, renderObject),
-                                           getIconSizeForPixelSize(rect.height()));
+    GRefPtr<GdkPixbuf> icon = getStockIconForWidgetType(GTK_TYPE_ENTRY, GTK_STOCK_CLEAR,
+                                                        gtkTextDirection(renderObject->style()->direction()),
+                                                        gtkIconState(this, renderObject),
+                                                        getIconSizeForPixelSize(rect.height()));
     paintGdkPixbuf(paintInfo.context, icon.get(), iconRect);
     return false;
 }
@@ -369,9 +379,9 @@ bool RenderThemeGtk::paintCapsLockIndicator(RenderObject* renderObject, const Pa
         return true;
 
     int iconSize = std::min(rect.width(), rect.height());
-    GRefPtr<GdkPixbuf> icon = getStockIcon(GTK_TYPE_ENTRY, GTK_STOCK_CAPS_LOCK_WARNING,
-                                           gtkTextDirection(renderObject->style()->direction()),
-                                           0, getIconSizeForPixelSize(iconSize));
+    GRefPtr<GdkPixbuf> icon = getStockIconForWidgetType(GTK_TYPE_ENTRY, GTK_STOCK_CAPS_LOCK_WARNING,
+                                                        gtkTextDirection(renderObject->style()->direction()),
+                                                        0, getIconSizeForPixelSize(iconSize));
 
     // Only re-scale the icon when it's smaller than the minimum icon size.
     if (iconSize >= gtkIconSizeMenu)
@@ -388,12 +398,13 @@ bool RenderThemeGtk::paintCapsLockIndicator(RenderObject* renderObject, const Pa
 
 void RenderThemeGtk::adjustSliderTrackStyle(CSSStyleSelector*, RenderStyle* style, Element*) const
 {
-    style->setBoxShadow(0);
+    style->setBoxShadow(nullptr);
 }
 
-void RenderThemeGtk::adjustSliderThumbStyle(CSSStyleSelector*, RenderStyle* style, Element*) const
+void RenderThemeGtk::adjustSliderThumbStyle(CSSStyleSelector* selector, RenderStyle* style, Element* element) const
 {
-    style->setBoxShadow(0);
+    RenderTheme::adjustSliderThumbStyle(selector, style, element);
+    style->setBoxShadow(nullptr);
 }
 
 double RenderThemeGtk::caretBlinkInterval() const
@@ -467,19 +478,26 @@ String RenderThemeGtk::extraMediaControlsStyleSheet()
     return String(mediaControlsGtkUserAgentStyleSheet, sizeof(mediaControlsGtkUserAgentStyleSheet));
 }
 
-void RenderThemeGtk::adjustMediaSliderThumbSize(RenderObject* renderObject) const
+#if ENABLE(FULLSCREEN_API)
+String RenderThemeGtk::extraFullScreenStyleSheet()
 {
-    ASSERT(renderObject->style()->appearance() == MediaSliderThumbPart);
-    renderObject->style()->setWidth(Length(m_mediaSliderThumbWidth, Fixed));
-    renderObject->style()->setHeight(Length(m_mediaSliderThumbHeight, Fixed));
+    return String(fullscreenQuickTimeUserAgentStyleSheet, sizeof(fullscreenQuickTimeUserAgentStyleSheet));
+}
+#endif
+
+void RenderThemeGtk::adjustMediaSliderThumbSize(RenderStyle* style) const
+{
+    ASSERT(style->appearance() == MediaSliderThumbPart);
+    style->setWidth(Length(m_mediaSliderThumbWidth, Fixed));
+    style->setHeight(Length(m_mediaSliderThumbHeight, Fixed));
 }
 
 bool RenderThemeGtk::paintMediaButton(RenderObject* renderObject, GraphicsContext* context, const IntRect& rect, const char* iconName)
 {
-    GRefPtr<GdkPixbuf> icon = getStockIcon(GTK_TYPE_CONTAINER, iconName,
-                                           gtkTextDirection(renderObject->style()->direction()),
-                                           gtkIconState(this, renderObject),
-                                           getMediaButtonIconSize(m_mediaIconSize));
+    GRefPtr<GdkPixbuf> icon = getStockIconForWidgetType(GTK_TYPE_CONTAINER, iconName,
+                                                        gtkTextDirection(renderObject->style()->direction()),
+                                                        gtkIconState(this, renderObject),
+                                                        getMediaButtonIconSize(m_mediaIconSize));
     IntRect iconRect(rect.x() + (rect.width() - m_mediaIconSize) / 2,
                      rect.y() + (rect.height() - m_mediaIconSize) / 2,
                      m_mediaIconSize, m_mediaIconSize);
@@ -623,7 +641,7 @@ bool RenderThemeGtk::paintMediaCurrentTime(RenderObject* renderObject, const Pai
 #if ENABLE(PROGRESS_TAG)
 void RenderThemeGtk::adjustProgressBarStyle(CSSStyleSelector*, RenderStyle* style, Element*) const
 {
-    style->setBoxShadow(0);
+    style->setBoxShadow(nullptr);
 }
 
 // These values have been copied from RenderThemeChromiumSkia.cpp
@@ -671,4 +689,32 @@ IntRect RenderThemeGtk::calculateProgressRect(RenderObject* renderObject, const
 }
 #endif
 
+static bool stringByAdoptingFileSystemRepresentation(gchar* systemFilename, String& result)
+{
+    if (!systemFilename)
+        return false;
+
+    result = filenameToString(systemFilename);
+    g_free(systemFilename);
+
+    return true;
+}
+
+String RenderThemeGtk::fileListNameForWidth(const Vector<String>& filenames, const Font& font, int width)
+{
+    if (width <= 0)
+        return String();
+
+    String string = fileButtonNoFileSelectedLabel();
+
+    if (filenames.size() == 1) {
+        CString systemFilename = fileSystemRepresentation(filenames[0]);
+        gchar* systemBasename = g_path_get_basename(systemFilename.data());
+        stringByAdoptingFileSystemRepresentation(systemBasename, string);
+    } else if (filenames.size() > 1)
+        return StringTruncator::rightTruncate(multipleFileUploadText(filenames.size()), width, font, StringTruncator::EnableRoundingHacks);
+
+    return StringTruncator::centerTruncate(string, width, font, StringTruncator::EnableRoundingHacks);
+}
+
 }
