@@ -5,7 +5,7 @@
  * Copyright (C) 2009 Holger Hans Peter Freyther
  * Copyright (C) 2009 Gustavo Noronha Silva <gns@gnome.org>
  * Copyright (C) 2009 Christian Dywan <christian@imendio.com>
- * Copyright (C) 2009 Igalia S.L.
+ * Copyright (C) 2009, 2010, 2011 Igalia S.L.
  * Copyright (C) 2009 John Kjellberg <john.kjellberg@power.alstom.com>
  *
  * This library is free software; you can redistribute it and/or
@@ -28,7 +28,6 @@
 #include "ResourceHandle.h"
 
 #include "Base64.h"
-#include "CString.h"
 #include "ChromeClient.h"
 #include "CookieJarSoup.h"
 #include "CachedResourceLoader.h"
@@ -57,6 +56,7 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
+#include <wtf/text/CString.h>
 
 namespace WebCore {
 
@@ -69,7 +69,7 @@ public:
     ~WebCoreSynchronousLoader();
 
     virtual void didReceiveResponse(ResourceHandle*, const ResourceResponse&);
-    virtual void didReceiveData(ResourceHandle*, const char*, int, int lengthReceived);
+    virtual void didReceiveData(ResourceHandle*, const char*, int, int encodedDataLength);
     virtual void didFinishLoading(ResourceHandle*, double /*finishTime*/);
     virtual void didFail(ResourceHandle*, const ResourceError&);
 
@@ -135,11 +135,6 @@ ResourceHandleInternal::~ResourceHandleInternal()
 {
     if (m_soupRequest)
         g_object_set_data(G_OBJECT(m_soupRequest.get()), "webkit-resource", 0);
-
-    if (m_idleHandler) {
-        g_source_remove(m_idleHandler);
-        m_idleHandler = 0;
-    }
 }
 
 ResourceHandle::~ResourceHandle()
@@ -149,13 +144,6 @@ ResourceHandle::~ResourceHandle()
 
 static void ensureSessionIsInitialized(SoupSession* session)
 {
-    // Values taken from http://stevesouders.com/ua/index.php following
-    // the rule "Do What Every Other Modern Browser Is Doing". They seem
-    // to significantly improve page loading time compared to soup's
-    // default values.
-    static const int maxConnections = 60;
-    static const int maxConnectionsPerHost = 6;
-
     if (g_object_get_data(G_OBJECT(session), "webkit-init"))
         return;
 
@@ -177,17 +165,12 @@ static void ensureSessionIsInitialized(SoupSession* session)
         g_object_unref(requester);
     }
 
-    g_object_set(session,
-                 SOUP_SESSION_MAX_CONNS, maxConnections,
-                 SOUP_SESSION_MAX_CONNS_PER_HOST, maxConnectionsPerHost,
-                 NULL);
-
     g_object_set_data(G_OBJECT(session), "webkit-init", reinterpret_cast<void*>(0xdeadbeef));
 }
 
-void ResourceHandle::prepareForURL(const KURL &url)
+void ResourceHandle::prepareForURL(const KURL& url)
 {
-    GOwnPtr<SoupURI> soupURI(soup_uri_new(url.prettyURL().utf8().data()));
+    GOwnPtr<SoupURI> soupURI(soup_uri_new(url.string().utf8().data()));
     if (!soupURI)
         return;
     soup_session_prepare_for_uri(ResourceHandle::defaultSession(), soupURI.get());
@@ -206,11 +189,6 @@ static gboolean statusWillBeHandledBySoup(guint statusCode)
     return false;
 }
 
-static void fillResponseFromMessage(SoupMessage* msg, ResourceResponse* response)
-{
-    response->updateFromSoupMessage(msg);
-}
-
 // Called each time the message is going to be sent again except the first time.
 // It's used mostly to let webkit know about redirects.
 static void restartedCallback(SoupMessage* msg, gpointer data)
@@ -230,7 +208,7 @@ static void restartedCallback(SoupMessage* msg, gpointer data)
     ResourceResponse response;
     request.setURL(newURL);
     request.setHTTPMethod(msg->method);
-    fillResponseFromMessage(msg, &response);
+    response.updateFromSoupMessage(msg);
 
     // Should not set Referer after a redirect from a secure resource to non-secure one.
     if (!request.url().protocolIs("https") && protocolIs(request.httpReferrer(), "https")) {
@@ -295,7 +273,7 @@ static void gotHeadersCallback(SoupMessage* msg, gpointer data)
 
     ASSERT(d->m_response.isNull());
 
-    fillResponseFromMessage(msg, &d->m_response);
+    d->m_response.updateFromSoupMessage(msg);
     client->didReceiveResponse(handle.get(), d->m_response);
 }
 
@@ -355,7 +333,7 @@ static void contentSniffedCallback(SoupMessage* msg, const char* sniffedType, GH
         }
     }
 
-    fillResponseFromMessage(msg, &d->m_response);
+    d->m_response.updateFromSoupMessage(msg);
     client->didReceiveResponse(handle.get(), d->m_response);
 }
 
@@ -376,12 +354,18 @@ static void gotChunkCallback(SoupMessage* msg, SoupBuffer* chunk, gpointer data)
 
     ASSERT(!d->m_response.isNull());
 
-    client->didReceiveData(handle.get(), chunk->data, chunk->length, chunk->length);
+    // FIXME: https://bugs.webkit.org/show_bug.cgi?id=19793
+    // -1 means we do not provide any data about transfer size to inspector so it would use
+    // Content-Length headers or content size to show transfer size.
+    client->didReceiveData(handle.get(), chunk->data, chunk->length, -1);
 }
 
-static SoupSession* createSoupSession()
+static void finishedCallback(SoupMessage* msg, gpointer data)
 {
-    return soup_session_async_new();
+    RefPtr<ResourceHandle> handle = static_cast<ResourceHandle*>(data);
+    if (!handle)
+        return;
+    handle->getInternal()->m_finished = true;
 }
 
 static void cleanupSoupRequestOperation(ResourceHandle* handle, bool isDestroying = false)
@@ -415,6 +399,32 @@ static void cleanupSoupRequestOperation(ResourceHandle* handle, bool isDestroyin
         handle->deref();
 }
 
+static bool soupErrorShouldCauseLoadFailure(GError* error, SoupMessage* message)
+{
+    // Libsoup treats some non-error conditions as errors, including redirects and 304 Not Modified responses.
+    return message && SOUP_STATUS_IS_TRANSPORT_ERROR(message->status_code) || error->domain == G_IO_ERROR;
+}
+
+static ResourceError convertSoupErrorToResourceError(GError* error, SoupRequest* request, SoupMessage* message = 0)
+{
+    ASSERT(error);
+    ASSERT(request);
+
+    GOwnPtr<char> uri(soup_uri_to_string(soup_request_get_uri(request), FALSE));
+    if (message && SOUP_STATUS_IS_TRANSPORT_ERROR(message->status_code)) {
+        return ResourceError(g_quark_to_string(SOUP_HTTP_ERROR),
+                             static_cast<gint>(message->status_code),
+                             uri.get(),
+                             String::fromUTF8(message->reason_phrase));
+    }
+
+    // Non-transport errors are handled differently.
+    return ResourceError(g_quark_to_string(G_IO_ERROR),
+                         error->code,
+                         uri.get(),
+                         String::fromUTF8(error->message));
+}
+
 static void sendRequestCallback(GObject* source, GAsyncResult* res, gpointer userData)
 {
     RefPtr<ResourceHandle> handle = static_cast<ResourceHandle*>(g_object_get_data(source, "webkit-resource"));
@@ -438,36 +448,28 @@ static void sendRequestCallback(GObject* source, GAsyncResult* res, gpointer use
 
     GOwnPtr<GError> error;
     GInputStream* in = soup_request_send_finish(d->m_soupRequest.get(), res, &error.outPtr());
-
     if (error) {
-        SoupMessage* soupMsg = d->m_soupMessage.get();
-        gboolean isTransportError = d->m_soupMessage && SOUP_STATUS_IS_TRANSPORT_ERROR(soupMsg->status_code);
-
-        if (isTransportError || (error->domain == G_IO_ERROR)) {
-            SoupURI* uri = soup_request_get_uri(d->m_soupRequest.get());
-            GOwnPtr<char> uriStr(soup_uri_to_string(uri, false));
-            gint errorCode = isTransportError ? static_cast<gint>(soupMsg->status_code) : error->code;
-            const gchar* errorMsg = isTransportError ? soupMsg->reason_phrase : error->message;
-            const gchar* quarkStr = isTransportError ? g_quark_to_string(SOUP_HTTP_ERROR) : g_quark_to_string(G_IO_ERROR);
-            ResourceError resourceError(quarkStr, errorCode, uriStr.get(), String::fromUTF8(errorMsg));
+        SoupMessage* soupMessage = d->m_soupMessage.get();
 
+        if (soupErrorShouldCauseLoadFailure(error.get(), soupMessage)) {
+            client->didFail(handle.get(), convertSoupErrorToResourceError(error.get(), d->m_soupRequest.get(), soupMessage));
             cleanupSoupRequestOperation(handle.get());
-            client->didFail(handle.get(), resourceError);
             return;
         }
 
-        if (d->m_soupMessage && statusWillBeHandledBySoup(d->m_soupMessage->status_code)) {
+        if (soupMessage && statusWillBeHandledBySoup(soupMessage->status_code)) {
             ASSERT(d->m_response.isNull());
 
-            fillResponseFromMessage(soupMsg, &d->m_response);
+            d->m_response.updateFromSoupMessage(soupMessage);
             client->didReceiveResponse(handle.get(), d->m_response);
 
             // WebCore might have cancelled the job in the while. We
             // must check for response_body->length and not
             // response_body->data as libsoup always creates the
             // SoupBuffer for the body even if the length is 0
-            if (!d->m_cancelled && soupMsg->response_body->length)
-                client->didReceiveData(handle.get(), soupMsg->response_body->data, soupMsg->response_body->length, soupMsg->response_body->length);
+            if (!d->m_cancelled && soupMessage->response_body->length)
+                client->didReceiveData(handle.get(), soupMessage->response_body->data,
+                                       soupMessage->response_body->length, soupMessage->response_body->length);
         }
 
         // didReceiveData above might have canceled this operation. If not, inform the client we've finished loading.
@@ -570,6 +572,8 @@ static bool startHTTPRequest(ResourceHandle* handle)
     url.removeFragmentIdentifier();
     request.setURL(url);
 
+    d->m_finished = false;
+
     GOwnPtr<GError> error;
     d->m_soupRequest = adoptGRef(soup_requester_request(requester, url.string().utf8().data(), &error.outPtr()));
     if (error) {
@@ -595,6 +599,7 @@ static bool startHTTPRequest(ResourceHandle* handle)
     g_signal_connect(soupMessage, "got-headers", G_CALLBACK(gotHeadersCallback), handle);
     g_signal_connect(soupMessage, "wrote-body-data", G_CALLBACK(wroteBodyDataCallback), handle);
     d->m_gotChunkHandler = g_signal_connect(soupMessage, "got-chunk", G_CALLBACK(gotChunkCallback), handle);
+    d->m_finishedHandler = g_signal_connect(soupMessage, "finished", G_CALLBACK(finishedCallback), handle);
 
     String firstPartyString = request.firstPartyForCookies().string();
     if (!firstPartyString.isEmpty()) {
@@ -680,17 +685,6 @@ void ResourceHandle::cancel()
         g_cancellable_cancel(d->m_cancellable.get());
 }
 
-PassRefPtr<SharedBuffer> ResourceHandle::bufferedData()
-{
-    ASSERT_NOT_REACHED();
-    return 0;
-}
-
-bool ResourceHandle::supportsBufferedData()
-{
-    return false;
-}
-
 void ResourceHandle::platformSetDefersLoading(bool defersLoading)
 {
     // Initial implementation of this method was required for bug #44157.
@@ -709,8 +703,9 @@ void ResourceHandle::platformSetDefersLoading(bool defersLoading)
     if (!d->m_soupMessage)
         return;
 
+    // Avoid any operation on not yet started messages and completed messages.
     SoupMessage* soupMessage = d->m_soupMessage.get();
-    if (soupMessage->status_code != SOUP_STATUS_NONE)
+    if (d->m_finished || soupMessage->status_code == SOUP_STATUS_NONE)
         return;
 
     if (defersLoading)
@@ -773,25 +768,18 @@ static void readCallback(GObject* source, GAsyncResult* asyncResult, gpointer da
     }
 
     GOwnPtr<GError> error;
-
     gssize bytesRead = g_input_stream_read_finish(d->m_inputStream.get(), asyncResult, &error.outPtr());
     if (error) {
-        SoupURI* uri = soup_request_get_uri(d->m_soupRequest.get());
-        GOwnPtr<char> uriStr(soup_uri_to_string(uri, false));
-        ResourceError resourceError(g_quark_to_string(G_IO_ERROR), error->code, uriStr.get(),
-                                    error ? String::fromUTF8(error->message) : String());
+        client->didFail(handle.get(), convertSoupErrorToResourceError(error.get(), d->m_soupRequest.get()));
         cleanupSoupRequestOperation(handle.get());
-        client->didFail(handle.get(), resourceError);
         return;
     }
 
     if (!bytesRead) {
-        // Finish the load. We do not wait for the stream to
-        // close. Instead we better notify WebCore as soon as possible
+        // We inform WebCore of load completion now instead of waiting for the input
+        // stream to close because the input stream is closed asynchronously.
         client->didFinishLoading(handle.get(), 0);
-
-        g_input_stream_close_async(d->m_inputStream.get(), G_PRIORITY_DEFAULT,
-                                   0, closeCallback, 0);
+        g_input_stream_close_async(d->m_inputStream.get(), G_PRIORITY_DEFAULT, 0, closeCallback, 0);
         return;
     }
 
@@ -850,7 +838,21 @@ static bool startNonHTTPRequest(ResourceHandle* handle, KURL url)
 
 SoupSession* ResourceHandle::defaultSession()
 {
-    static SoupSession* session = createSoupSession();
+    static SoupSession* session = 0;
+    // Values taken from http://www.browserscope.org/  following
+    // the rule "Do What Every Other Modern Browser Is Doing". They seem
+    // to significantly improve page loading time compared to soup's
+    // default values.
+    static const int maxConnections = 35;
+    static const int maxConnectionsPerHost = 6;
+
+    if (!session) {
+        session = soup_session_async_new();
+        g_object_set(session,
+                     SOUP_SESSION_MAX_CONNS, maxConnections,
+                     SOUP_SESSION_MAX_CONNS_PER_HOST, maxConnectionsPerHost, 
+                     NULL);
+    }
 
     return session;
 }
