@@ -59,7 +59,6 @@
 #include "ProxyServer.h"
 #include "RenderBox.h"
 #include "RenderObject.h"
-#include "ScriptController.h"
 #include "ScriptValue.h"
 #include "SecurityOrigin.h"
 #include "Settings.h"
@@ -272,9 +271,6 @@ bool PluginView::start()
     if (m_status != PluginStatusLoadedSuccessfully)
         return false;
 
-    if (parentFrame()->page())
-        parentFrame()->page()->didStartPlugin(this);
-
     return true;
 }
 
@@ -319,9 +315,6 @@ void PluginView::stop()
     if (!m_isStarted)
         return;
 
-    if (parentFrame()->page())
-        parentFrame()->page()->didStopPlugin(this);
-
     LOG(Plugins, "PluginView::stop(): Stopping plug-in '%s'", m_plugin->name().utf8().data());
 
     HashSet<RefPtr<PluginStream> > streams = m_streams;
@@ -340,7 +333,7 @@ void PluginView::stop()
 #endif
 
 #if ENABLE(NETSCAPE_PLUGIN_API)
-#ifdef XP_WIN
+#if defined(XP_WIN) && !PLATFORM(GTK)
     // Unsubclass the window
     if (m_isWindowed) {
 #if OS(WINCE)
@@ -355,7 +348,7 @@ void PluginView::stop()
             SetWindowLongPtr(platformPluginWidget(), GWLP_WNDPROC, (LONG_PTR)m_pluginWndProc);
 #endif
     }
-#endif // XP_WIN
+#endif // !defined(XP_WIN) || PLATFORM(GTK)
 #endif // ENABLE(NETSCAPE_PLUGIN_API)
 
 #if !defined(XP_MACOSX)
@@ -830,7 +823,7 @@ PluginView::PluginView(Frame* parentFrame, const IntSize& size, PluginPackage* p
     , m_element(element)
     , m_isStarted(false)
     , m_url(url)
-    , m_baseURL(m_parentFrame->loader()->completeURL(m_parentFrame->document()->baseURL().string()))
+    , m_baseURL(m_parentFrame->document()->baseURL()) // FIXME: No need for this member variable!
     , m_status(PluginStatusLoadedSuccessfully)
     , m_requestTimer(this, &PluginView::requestTimerFired)
     , m_invalidateTimer(this, &PluginView::invalidateTimerFired)
@@ -859,7 +852,7 @@ PluginView::PluginView(Frame* parentFrame, const IntSize& size, PluginPackage* p
     , m_wmPrintHDC(0)
     , m_haveUpdatedPluginWidget(false)
 #endif
-#if (PLATFORM(QT) && OS(WINDOWS)) || defined(XP_MACOSX)
+#if (PLATFORM(QT) && OS(WINDOWS)) || defined(XP_MACOSX) || PLATFORM(EFL)
     , m_window(0)
 #endif
 #if defined(XP_MACOSX)
@@ -881,8 +874,6 @@ PluginView::PluginView(Frame* parentFrame, const IntSize& size, PluginPackage* p
     , m_loadManually(loadManually)
     , m_manualStream(0)
     , m_isJavaScriptPaused(false)
-    , m_isHalted(false)
-    , m_hasBeenHalted(false)
     , m_haveCalledSetWindow(false)
 {
     if (!m_plugin) {
@@ -910,8 +901,9 @@ void PluginView::focusPluginElement()
 {
     // Focus the plugin
     if (Page* page = m_parentFrame->page())
-        page->focusController()->setFocusedFrame(m_parentFrame);
-    m_parentFrame->document()->setFocusedNode(m_element);
+        page->focusController()->setFocusedNode(m_element, m_parentFrame);
+    else
+        m_parentFrame->document()->setFocusedNode(m_element);
 }
 
 void PluginView::didReceiveResponse(const ResourceResponse& response)
@@ -1290,16 +1282,6 @@ const char* PluginView::userAgentStatic()
 #endif
 
 
-Node* PluginView::node() const
-{
-    return m_element;
-}
-
-String PluginView::pluginName() const
-{
-    return m_plugin->name();
-}
-
 void PluginView::lifeSupportTimerFired(Timer<PluginView>*)
 {
     deref();
