@@ -29,7 +29,6 @@
 #define PluginView_h
 
 #include "FrameLoadRequest.h"
-#include "HaltablePlugin.h"
 #include "IntRect.h"
 #include "MediaCanStartListener.h"
 #include "PluginViewBase.h"
@@ -58,12 +57,21 @@ typedef PlatformWidget PlatformPluginWidget;
 #endif
 #endif
 #if PLATFORM(QT)
+#if USE(TEXTURE_MAPPER)
+#include "TextureMapperPlatformLayer.h"
+#endif
+
 #include <QGraphicsItem>
 #include <QImage>
 QT_BEGIN_NAMESPACE
 class QPainter;
 QT_END_NAMESPACE
 #endif
+#if PLATFORM(QT) && USE(ACCELERATED_COMPOSITING) && ENABLE(NETSCAPE_PLUGIN_API) && (defined(XP_UNIX) || OS(SYMBIAN))
+#ifndef WTF_USE_ACCELERATED_COMPOSITING_PLUGIN_LAYER
+#define WTF_USE_ACCELERATED_COMPOSITING_PLUGIN_LAYER 1
+#endif
+#endif
 #if PLATFORM(GTK)
 typedef struct _GtkSocket GtkSocket;
 #endif
@@ -132,7 +140,6 @@ namespace WebCore {
                      , private PluginStreamClient
 #endif
                      , public PluginManualLoader
-                     , private HaltablePlugin
                      , private MediaCanStartListener {
     public:
         static PassRefPtr<PluginView> create(Frame* parentFrame, const IntSize&, Element*, const KURL&, const Vector<String>& paramNames, const Vector<String>& paramValues, const String& mimeType, bool loadManually);
@@ -235,16 +242,6 @@ namespace WebCore {
         void didFinishLoading();
         void didFail(const ResourceError&);
 
-        // HaltablePlugin
-        virtual void halt();
-        virtual void restart();
-        virtual Node* node() const;
-        virtual bool isWindowed() const { return m_isWindowed; }
-        virtual String pluginName() const;
-
-        bool isHalted() const { return m_isHalted; }
-        bool hasBeenHalted() const { return m_hasBeenHalted; }
-
         static bool isCallingPlugin();
 
         bool start();
@@ -255,8 +252,9 @@ namespace WebCore {
         void keepAlive();
 
 #if USE(ACCELERATED_COMPOSITING)
-#if defined(XP_UNIX) && ENABLE(NETSCAPE_PLUGIN_API) && PLATFORM(QT)
+#if USE(ACCELERATED_COMPOSITING_PLUGIN_LAYER)
         virtual PlatformLayer* platformLayer() const;
+        bool shouldUseAcceleratedCompositing() const;
 #else
         virtual PlatformLayer* platformLayer() const { return 0; }
 #endif
@@ -390,7 +388,7 @@ public:
 
 private:
 
-#if defined(XP_UNIX) || OS(SYMBIAN)
+#if defined(XP_UNIX) || OS(SYMBIAN) || PLATFORM(GTK)
         void setNPWindowIfNeeded();
 #elif defined(XP_MACOSX)
         NP_CGContext m_npCgContext;
@@ -428,11 +426,11 @@ private:
 #endif
 #if defined(XP_UNIX) && ENABLE(NETSCAPE_PLUGIN_API)
         void paintUsingXPixmap(QPainter* painter, const QRect &exposedRect);
-#if USE(ACCELERATED_COMPOSITING)
+#endif
+#if USE(ACCELERATED_COMPOSITING_PLUGIN_LAYER)
         OwnPtr<PlatformLayer> m_platformLayer;
         friend class PluginGraphicsLayerQt;
-#endif // USE(ACCELERATED_COMPOSITING)
-#endif
+#endif // USE(ACCELERATED_COMPOSITING_PLUGIN_LAYER)
 #endif // PLATFORM(QT)
 
 #if PLATFORM(GTK)
@@ -451,9 +449,6 @@ private:
 
         bool m_isJavaScriptPaused;
 
-        bool m_isHalted;
-        bool m_hasBeenHalted;
-
         bool m_haveCalledSetWindow;
 
         static PluginView* s_currentPluginView;
