@@ -27,7 +27,10 @@
 #include "config.h"
 #include "RenderImage.h"
 
+#include "BitmapImage.h"
+#include "FontCache.h"
 #include "Frame.h"
+#include "FrameSelection.h"
 #include "GraphicsContext.h"
 #include "HTMLAreaElement.h"
 #include "HTMLImageElement.h"
@@ -38,15 +41,8 @@
 #include "Page.h"
 #include "RenderLayer.h"
 #include "RenderView.h"
-#include "SelectionController.h"
-#include "TextRun.h"
 #include <wtf/UnusedParam.h>
 
-#if ENABLE(WML)
-#include "WMLImageElement.h"
-#include "WMLNames.h"
-#endif
-
 using namespace std;
 
 namespace WebCore {
@@ -56,10 +52,9 @@ using namespace HTMLNames;
 RenderImage::RenderImage(Node* node)
     : RenderReplaced(node, IntSize(0, 0))
     , m_needsToSetSizeForAltText(false)
+    , m_didIncrementVisuallyNonEmptyPixelCount(false)
 {
     updateAltText();
-
-    view()->frameView()->setIsVisuallyNonEmpty();
 }
 
 RenderImage::~RenderImage()
@@ -89,9 +84,18 @@ IntSize RenderImage::imageSizeForError(CachedImage* newImage) const
     ASSERT_ARG(newImage, newImage);
     ASSERT_ARG(newImage, newImage->image());
 
+    IntSize imageSize;
+    if (newImage->willPaintBrokenImage()) {
+        float deviceScaleFactor = Page::deviceScaleFactor(frame());
+        pair<Image*, float> brokenImageAndImageScaleFactor = newImage->brokenImage(deviceScaleFactor);
+        imageSize = brokenImageAndImageScaleFactor.first->size();
+        imageSize.scale(1 / brokenImageAndImageScaleFactor.second);
+    } else
+        imageSize = newImage->image()->size();
+
     // imageSize() returns 0 for the error image. We need the true size of the
     // error image, so we have to get it by grabbing image() directly.
-    return IntSize(paddingWidth + newImage->image()->width() * style()->effectiveZoom(), paddingHeight + newImage->image()->height() * style()->effectiveZoom());
+    return IntSize(paddingWidth + imageSize.width() * style()->effectiveZoom(), paddingHeight + imageSize.height() * style()->effectiveZoom());
 }
 
 // Sets the image height and width to fit the alt text.  Returns true if the
@@ -108,8 +112,10 @@ bool RenderImage::setImageSizeForAltText(CachedImage* newImage /* = 0 */)
 
     // we have an alt and the user meant it (its not a text we invented)
     if (!m_altText.isEmpty()) {
+        FontCachePurgePreventer fontCachePurgePreventer;
+
         const Font& font = style()->font();
-        IntSize textSize(min(font.width(TextRun(m_altText.characters(), m_altText.length())), maxAltTextWidth), min(font.fontMetrics().height(), maxAltTextHeight));
+        IntSize textSize(min(font.width(RenderBlock::constructTextRun(this, font, m_altText, style())), maxAltTextWidth), min(font.fontMetrics().height(), maxAltTextHeight));
         imageSize = imageSize.expandedTo(textSize);
     }
 
@@ -143,6 +149,11 @@ void RenderImage::imageChanged(WrappedImagePtr newImage, const IntRect* rect)
 
     if (newImage != m_imageResource->imagePtr() || !newImage)
         return;
+    
+    if (!m_didIncrementVisuallyNonEmptyPixelCount) {
+        view()->frameView()->incrementVisuallyNonEmptyPixelCount(m_imageResource->imageSize(1.0f));
+        m_didIncrementVisuallyNonEmptyPixelCount = true;
+    }
 
     bool imageSizeChanged = false;
 
@@ -234,14 +245,14 @@ void RenderImage::notifyFinished(CachedResource* newImage)
 #endif
 }
 
-void RenderImage::paintReplaced(PaintInfo& paintInfo, int tx, int ty)
+void RenderImage::paintReplaced(PaintInfo& paintInfo, const LayoutPoint& paintOffset)
 {
-    int cWidth = contentWidth();
-    int cHeight = contentHeight();
-    int leftBorder = borderLeft();
-    int topBorder = borderTop();
-    int leftPad = paddingLeft();
-    int topPad = paddingTop();
+    LayoutUnit cWidth = contentWidth();
+    LayoutUnit cHeight = contentHeight();
+    LayoutUnit leftBorder = borderLeft();
+    LayoutUnit topBorder = borderTop();
+    LayoutUnit leftPad = paddingLeft();
+    LayoutUnit topPad = paddingTop();
 
     GraphicsContext* context = paintInfo.context;
 
@@ -254,50 +265,54 @@ void RenderImage::paintReplaced(PaintInfo& paintInfo, int tx, int ty)
             context->setStrokeStyle(SolidStroke);
             context->setStrokeColor(Color::lightGray, style()->colorSpace());
             context->setFillColor(Color::transparent, style()->colorSpace());
-            context->drawRect(IntRect(tx + leftBorder + leftPad, ty + topBorder + topPad, cWidth, cHeight));
+            context->drawRect(LayoutRect(paintOffset.x() + leftBorder + leftPad, paintOffset.y() + topBorder + topPad, cWidth, cHeight));
 
             bool errorPictureDrawn = false;
-            int imageX = 0;
-            int imageY = 0;
+            LayoutSize imageOffset;
             // When calculating the usable dimensions, exclude the pixels of
             // the ouline rect so the error image/alt text doesn't draw on it.
-            int usableWidth = cWidth - 2;
-            int usableHeight = cHeight - 2;
+            LayoutUnit usableWidth = cWidth - 2;
+            LayoutUnit usableHeight = cHeight - 2;
 
             RefPtr<Image> image = m_imageResource->image();
 
             if (m_imageResource->errorOccurred() && !image->isNull() && usableWidth >= image->width() && usableHeight >= image->height()) {
+                float deviceScaleFactor = Page::deviceScaleFactor(frame());
+                // Call brokenImage() explicitly to ensure we get the broken image icon at the appropriate resolution.
+                pair<Image*, float> brokenImageAndImageScaleFactor = m_imageResource->cachedImage()->brokenImage(deviceScaleFactor);
+                image = brokenImageAndImageScaleFactor.first;
+                IntSize imageSize = image->size();
+                imageSize.scale(1 / brokenImageAndImageScaleFactor.second);
                 // Center the error image, accounting for border and padding.
-                int centerX = (usableWidth - image->width()) / 2;
+                LayoutUnit centerX = (usableWidth - imageSize.width()) / 2;
                 if (centerX < 0)
                     centerX = 0;
-                int centerY = (usableHeight - image->height()) / 2;
+                LayoutUnit centerY = (usableHeight - imageSize.height()) / 2;
                 if (centerY < 0)
                     centerY = 0;
-                imageX = leftBorder + leftPad + centerX + 1;
-                imageY = topBorder + topPad + centerY + 1;
-                context->drawImage(image.get(), style()->colorSpace(), IntPoint(tx + imageX, ty + imageY));
+                imageOffset = LayoutSize(leftBorder + leftPad + centerX + 1, topBorder + topPad + centerY + 1);
+                context->drawImage(image.get(), style()->colorSpace(), IntRect(paintOffset + imageOffset, imageSize));
                 errorPictureDrawn = true;
             }
 
             if (!m_altText.isEmpty()) {
                 String text = document()->displayStringModifiedByEncoding(m_altText);
                 context->setFillColor(style()->visitedDependentColor(CSSPropertyColor), style()->colorSpace());
-                int ax = tx + leftBorder + leftPad;
-                int ay = ty + topBorder + topPad;
                 const Font& font = style()->font();
                 const FontMetrics& fontMetrics = font.fontMetrics();
-                int ascent = fontMetrics.ascent();
+                LayoutUnit ascent = fontMetrics.ascent();
+                LayoutPoint altTextOffset = paintOffset;
+                altTextOffset.move(leftBorder + leftPad, topBorder + topPad + ascent);
 
                 // Only draw the alt text if it'll fit within the content box,
                 // and only if it fits above the error image.
-                TextRun textRun(text.characters(), text.length());
-                int textWidth = font.width(textRun);
+                TextRun textRun = RenderBlock::constructTextRun(this, font, text, style());
+                LayoutUnit textWidth = font.width(textRun);
                 if (errorPictureDrawn) {
-                    if (usableWidth >= textWidth && fontMetrics.height() <= imageY)
-                        context->drawText(font, textRun, IntPoint(ax, ay + ascent));
+                    if (usableWidth >= textWidth && fontMetrics.height() <= imageOffset.height())
+                        context->drawText(font, textRun, altTextOffset);
                 } else if (usableWidth >= textWidth && cHeight >= fontMetrics.height())
-                    context->drawText(font, textRun, IntPoint(ax, ay + ascent));
+                    context->drawText(font, textRun, altTextOffset);
             }
         }
     } else if (m_imageResource->hasImage() && cWidth > 0 && cHeight > 0) {
@@ -307,18 +322,19 @@ void RenderImage::paintReplaced(PaintInfo& paintInfo, int tx, int ty)
 
 #if PLATFORM(MAC)
         if (style()->highlight() != nullAtom && !paintInfo.context->paintingDisabled())
-            paintCustomHighlight(tx - x(), ty - y(), style()->highlight(), true);
+            paintCustomHighlight(toPoint(paintOffset - location()), style()->highlight(), true);
 #endif
 
-        IntSize contentSize(cWidth, cHeight);
-        IntRect rect(IntPoint(tx + leftBorder + leftPad, ty + topBorder + topPad), contentSize);
-        paintIntoRect(context, rect);
+        LayoutSize contentSize(cWidth, cHeight);
+        LayoutPoint contentLocation = paintOffset;
+        contentLocation.move(leftBorder + leftPad, topBorder + topPad);
+        paintIntoRect(context, LayoutRect(contentLocation, contentSize));
     }
 }
 
-void RenderImage::paint(PaintInfo& paintInfo, int tx, int ty)
+void RenderImage::paint(PaintInfo& paintInfo, const LayoutPoint& paintOffset)
 {
-    RenderReplaced::paint(paintInfo, tx, ty);
+    RenderReplaced::paint(paintInfo, paintOffset);
     
     if (paintInfo.phase == PaintPhaseOutline)
         paintAreaElementFocusRing(paintInfo);
@@ -352,7 +368,12 @@ void RenderImage::paintAreaElementFocusRing(PaintInfo& paintInfo)
     // FIXME: Do we need additional code to clip the path to the image's bounding box?
 
     RenderStyle* areaElementStyle = areaElement->computedStyle();
-    paintInfo.context->drawFocusRing(path, areaElementStyle->outlineWidth(), areaElementStyle->outlineOffset(),
+    unsigned short outlineWidth = areaElementStyle->outlineWidth();
+    if (!outlineWidth)
+        return;
+
+    paintInfo.context->drawFocusRing(path, outlineWidth,
+        areaElementStyle->outlineOffset(),
         areaElementStyle->visitedDependentColor(CSSPropertyOutlineColor));
 }
 
@@ -382,6 +403,32 @@ void RenderImage::paintIntoRect(GraphicsContext* context, const IntRect& rect)
     context->drawImage(m_imageResource->image(rect.width(), rect.height()).get(), style()->colorSpace(), rect, compositeOperator, useLowQualityScaling);
 }
 
+bool RenderImage::backgroundIsObscured() const
+{
+    if (!m_imageResource->hasImage() || m_imageResource->errorOccurred())
+        return false;
+
+    if (m_imageResource->cachedImage() && !m_imageResource->cachedImage()->isLoaded())
+        return false;
+
+    EFillBox backgroundClip = style()->backgroundClip();
+
+    // Background paints under borders.
+    if (backgroundClip == BorderFillBox && style()->hasBorder() && !borderObscuresBackground())
+        return false;
+
+    // Background shows in padding area.
+    if ((backgroundClip == BorderFillBox || backgroundClip == PaddingFillBox) && style()->hasPadding())
+        return false;
+
+    // Check for bitmap image with alpha.
+    Image* image = m_imageResource->image().get();
+    if (!image || !image->isBitmapImage() || image->currentFrameHasAlpha())
+        return false;
+        
+    return true;
+}
+
 int RenderImage::minimumReplacedHeight() const
 {
     return m_imageResource->errorOccurred() ? intrinsicSize().height() : 0;
@@ -390,21 +437,22 @@ int RenderImage::minimumReplacedHeight() const
 HTMLMapElement* RenderImage::imageMap() const
 {
     HTMLImageElement* i = node() && node()->hasTagName(imgTag) ? static_cast<HTMLImageElement*>(node()) : 0;
-    return i ? i->document()->getImageMap(i->fastGetAttribute(usemapAttr)) : 0;
+    return i ? i->treeScope()->getImageMap(i->fastGetAttribute(usemapAttr)) : 0;
 }
 
-bool RenderImage::nodeAtPoint(const HitTestRequest& request, HitTestResult& result, int x, int y, int tx, int ty, HitTestAction hitTestAction)
+bool RenderImage::nodeAtPoint(const HitTestRequest& request, HitTestResult& result, const LayoutPoint& pointInContainer, const LayoutPoint& accumulatedOffset, HitTestAction hitTestAction)
 {
     HitTestResult tempResult(result.point(), result.topPadding(), result.rightPadding(), result.bottomPadding(), result.leftPadding());
-    bool inside = RenderReplaced::nodeAtPoint(request, tempResult, x, y, tx, ty, hitTestAction);
+    bool inside = RenderReplaced::nodeAtPoint(request, tempResult, pointInContainer, accumulatedOffset, hitTestAction);
 
     if (tempResult.innerNode() && node()) {
         if (HTMLMapElement* map = imageMap()) {
             IntRect contentBox = contentBoxRect();
-            float zoom = style()->effectiveZoom();
-            int mapX = lroundf((x - tx - this->x() - contentBox.x()) / zoom);
-            int mapY = lroundf((y - ty - this->y() - contentBox.y()) / zoom);
-            if (map->mapMouseEvent(mapX, mapY, contentBox.size(), tempResult))
+            float scaleFactor = 1 / style()->effectiveZoom();
+            LayoutPoint mapLocation(pointInContainer.x() - accumulatedOffset.x() - this->x() - contentBox.x(), pointInContainer.y() - accumulatedOffset.y() - this->y() - contentBox.y());
+            mapLocation.scale(scaleFactor, scaleFactor);
+            
+            if (map->mapMouseEvent(mapLocation, contentBox.size(), tempResult))
                 tempResult.setInnerNonSharedNode(node());
         }
     }
@@ -425,10 +473,6 @@ void RenderImage::updateAltText()
         m_altText = static_cast<HTMLInputElement*>(node())->altText();
     else if (node()->hasTagName(imgTag))
         m_altText = static_cast<HTMLImageElement*>(node())->altText();
-#if ENABLE(WML)
-    else if (node()->hasTagName(WMLNames::imgTag))
-        m_altText = static_cast<WMLImageElement*>(node())->altText();
-#endif
 }
 
 bool RenderImage::isLogicalWidthSpecified() const
@@ -442,8 +486,11 @@ bool RenderImage::isLogicalWidthSpecified() const
         case Intrinsic:
         case MinIntrinsic:
             return false;
+        case Undefined:
+            ASSERT_NOT_REACHED();
+            return false;
     }
-    ASSERT(false);
+    ASSERT_NOT_REACHED();
     return false;
 }
 
@@ -458,53 +505,50 @@ bool RenderImage::isLogicalHeightSpecified() const
         case Intrinsic:
         case MinIntrinsic:
             return false;
+        case Undefined:
+            ASSERT_NOT_REACHED();
+            return false;
     }
-    ASSERT(false);
+    ASSERT_NOT_REACHED();
     return false;
 }
 
-int RenderImage::computeReplacedLogicalWidth(bool includeMaxWidth) const
+LayoutUnit RenderImage::computeReplacedLogicalWidth(bool includeMaxWidth) const
 {
     if (m_imageResource->imageHasRelativeWidth())
         if (RenderObject* cb = isPositioned() ? container() : containingBlock()) {
             if (cb->isBox())
-                m_imageResource->setImageContainerSize(IntSize(toRenderBox(cb)->availableWidth(), toRenderBox(cb)->availableHeight()));
+                m_imageResource->setImageContainerSize(LayoutSize(toRenderBox(cb)->availableWidth(), toRenderBox(cb)->availableHeight()));
         }
 
-    int logicalWidth;
+    LayoutUnit logicalWidth;
     if (isLogicalWidthSpecified())
         logicalWidth = computeReplacedLogicalWidthUsing(style()->logicalWidth());
     else if (m_imageResource->usesImageContainerSize()) {
-        IntSize size = m_imageResource->imageSize(style()->effectiveZoom());
+        LayoutSize size = m_imageResource->imageSize(style()->effectiveZoom());
         logicalWidth = style()->isHorizontalWritingMode() ? size.width() : size.height();
     } else if (m_imageResource->imageHasRelativeWidth())
         logicalWidth = 0; // If the image is relatively-sized, set the width to 0 until there is a set container size.
     else
         logicalWidth = calcAspectRatioLogicalWidth();
 
-    int minLogicalWidth = computeReplacedLogicalWidthUsing(style()->logicalMinWidth());
-    int maxLogicalWidth = !includeMaxWidth || style()->logicalMaxWidth().isUndefined() ? logicalWidth : computeReplacedLogicalWidthUsing(style()->logicalMaxWidth());
-
-    return max(minLogicalWidth, min(logicalWidth, maxLogicalWidth));
+    return computeReplacedLogicalWidthRespectingMinMaxWidth(logicalWidth, includeMaxWidth);
 }
 
-int RenderImage::computeReplacedLogicalHeight() const
+LayoutUnit RenderImage::computeReplacedLogicalHeight() const
 {
-    int logicalHeight;
+    LayoutUnit logicalHeight;
     if (isLogicalHeightSpecified())
         logicalHeight = computeReplacedLogicalHeightUsing(style()->logicalHeight());
     else if (m_imageResource->usesImageContainerSize()) {
-        IntSize size = m_imageResource->imageSize(style()->effectiveZoom());
+        LayoutSize size = m_imageResource->imageSize(style()->effectiveZoom());
         logicalHeight = style()->isHorizontalWritingMode() ? size.height() : size.width();
     } else if (m_imageResource->imageHasRelativeHeight())
         logicalHeight = 0; // If the image is relatively-sized, set the height to 0 until there is a set container size.
     else
         logicalHeight = calcAspectRatioLogicalHeight();
 
-    int minLogicalHeight = computeReplacedLogicalHeightUsing(style()->logicalMinHeight());
-    int maxLogicalHeight = style()->logicalMaxHeight().isUndefined() ? logicalHeight : computeReplacedLogicalHeightUsing(style()->logicalMaxHeight());
-
-    return max(minLogicalHeight, min(logicalHeight, maxLogicalHeight));
+    return computeReplacedLogicalHeightRespectingMinMaxHeight(logicalHeight);
 }
 
 int RenderImage::calcAspectRatioLogicalWidth() const
