@@ -31,12 +31,14 @@
 #include "ContentData.h"
 #include "RenderBlock.h"
 #include "RenderCounter.h"
+#include "RenderFlowThread.h"
 #include "RenderImage.h"
 #include "RenderImageResourceStyleImage.h"
 #include "RenderInline.h"
 #include "RenderLayer.h"
 #include "RenderListItem.h"
 #include "RenderQuote.h"
+#include "RenderRegion.h"
 #include "RenderStyle.h"
 #include "RenderTextFragment.h"
 #include "RenderView.h"
@@ -61,6 +63,14 @@ void RenderObjectChildList::destroyLeftoverChildren()
     }
 }
 
+static RenderFlowThread* renderFlowThreadContainer(RenderObject* object)
+{
+    while (object && object->isAnonymousBlock() && !object->isRenderFlowThread())
+        object = object->parent();
+
+    return object && object->isRenderFlowThread() ? toRenderFlowThread(object) : 0;
+}
+
 RenderObject* RenderObjectChildList::removeChildNode(RenderObject* owner, RenderObject* oldChild, bool fullRemove)
 {
     ASSERT(oldChild->parent() == owner);
@@ -84,8 +94,8 @@ RenderObject* RenderObjectChildList::removeChildNode(RenderObject* owner, Render
         // if we remove visible child from an invisible parent, we don't know the layer visibility any more
         RenderLayer* layer = 0;
         if (owner->style()->visibility() != VISIBLE && oldChild->style()->visibility() == VISIBLE && !oldChild->hasLayer()) {
-            layer = owner->enclosingLayer();
-            layer->dirtyVisibleContentStatus();
+            if ((layer = owner->enclosingLayer()))
+                layer->dirtyVisibleContentStatus();
         }
 
          // Keep our layer hierarchy updated.
@@ -101,6 +111,12 @@ RenderObject* RenderObjectChildList::removeChildNode(RenderObject* owner, Render
         if (oldChild->isPositioned() && owner->childrenInline())
             owner->dirtyLinesFromChangedChild(oldChild);
 
+        if (oldChild->isRenderRegion())
+            toRenderRegion(oldChild)->detachRegion();
+
+        if (RenderFlowThread* containerFlowThread = renderFlowThreadContainer(owner))
+            containerFlowThread->removeFlowChild(oldChild);
+
 #if ENABLE(SVG)
         // Update cached boundaries in SVG renderers, if a child is removed.
         owner->setNeedsBoundariesUpdate();
@@ -109,7 +125,7 @@ RenderObject* RenderObjectChildList::removeChildNode(RenderObject* owner, Render
     
     // If oldChild is the start or end of the selection, then clear the selection to
     // avoid problems of invalid pointers.
-    // FIXME: The SelectionController should be responsible for this when it
+    // FIXME: The FrameSelection should be responsible for this when it
     // is notified of DOM mutations.
     if (!owner->documentBeingDestroyed() && oldChild->isSelectionBorder())
         owner->view()->clearSelection();
@@ -129,8 +145,7 @@ RenderObject* RenderObjectChildList::removeChildNode(RenderObject* owner, Render
     oldChild->setNextSibling(0);
     oldChild->setParent(0);
 
-    if (oldChild->m_hasCounterNodeMap)
-        RenderCounter::destroyCounterNodes(oldChild);
+    RenderCounter::rendererRemovedFromTree(oldChild);
     RenderQuote::rendererRemovedFromTree(oldChild);
 
     if (AXObjectCache::accessibilityEnabled())
@@ -161,7 +176,7 @@ void RenderObjectChildList::appendChildNode(RenderObject* owner, RenderObject* n
         RenderLayer* layer = 0;
         if (newChild->firstChild() || newChild->hasLayer()) {
             layer = owner->enclosingLayer();
-            newChild->addLayers(layer, newChild);
+            newChild->addLayers(layer);
         }
 
         // if the new child is visible but this object was not, tell the layer it has some visible content
@@ -176,8 +191,14 @@ void RenderObjectChildList::appendChildNode(RenderObject* owner, RenderObject* n
         if (newChild->isListItem())
             toRenderListItem(newChild)->updateListMarkerNumbers();
 
-        if (!newChild->isFloatingOrPositioned() && owner->childrenInline())
+        if (!newChild->isFloating() && owner->childrenInline())
             owner->dirtyLinesFromChangedChild(newChild);
+
+        if (newChild->isRenderRegion())
+            toRenderRegion(newChild)->attachRegion();
+
+        if (RenderFlowThread* containerFlowThread = renderFlowThreadContainer(owner))
+            containerFlowThread->addFlowChild(newChild);
     }
     RenderCounter::rendererSubtreeAttached(newChild);
     RenderQuote::rendererSubtreeAttached(newChild);
@@ -221,7 +242,7 @@ void RenderObjectChildList::insertChildNode(RenderObject* owner, RenderObject* c
         RenderLayer* layer = 0;
         if (child->firstChild() || child->hasLayer()) {
             layer = owner->enclosingLayer();
-            child->addLayers(layer, child);
+            child->addLayers(layer);
         }
 
         // if the new child is visible but this object was not, tell the layer it has some visible content
@@ -238,6 +259,12 @@ void RenderObjectChildList::insertChildNode(RenderObject* owner, RenderObject* c
 
         if (!child->isFloating() && owner->childrenInline())
             owner->dirtyLinesFromChangedChild(child);
+
+        if (child->isRenderRegion())
+            toRenderRegion(child)->attachRegion();
+
+        if (RenderFlowThread* containerFlowThread = renderFlowThreadContainer(owner))
+            containerFlowThread->addFlowChild(child, beforeChild);
     }
 
     RenderCounter::rendererSubtreeAttached(child);
@@ -259,32 +286,7 @@ static RenderObject* findBeforeAfterParent(RenderObject* object)
     RenderObject* beforeAfterParent = object;
     while (beforeAfterParent && !(beforeAfterParent->isText() || beforeAfterParent->isImage()))
         beforeAfterParent = beforeAfterParent->firstChild();
-    return beforeAfterParent;
-}
-
-static void invalidateCountersInContainer(RenderObject* container, const AtomicString& identifier)
-{
-    if (!container)
-        return;
-    container = findBeforeAfterParent(container);
-    if (!container)
-        return;
-    // Sometimes the counter is attached directly on the container.
-    if (container->isCounter()) {
-        toRenderCounter(container)->invalidate(identifier);
-        return;
-    }
-    for (RenderObject* content = container->firstChild(); content; content = content->nextSibling()) {
-        if (content->isCounter())
-            toRenderCounter(content)->invalidate(identifier);
-    }
-}
-
-void RenderObjectChildList::invalidateCounters(const RenderObject* owner, const AtomicString& identifier)
-{
-    ASSERT(!owner->documentBeingDestroyed());
-    invalidateCountersInContainer(beforePseudoElementRenderer(owner), identifier);
-    invalidateCountersInContainer(afterPseudoElementRenderer(owner), identifier);
+    return beforeAfterParent ? beforeAfterParent->parent() : 0;
 }
 
 RenderObject* RenderObjectChildList::beforePseudoElementRenderer(const RenderObject* owner) const
@@ -297,7 +299,12 @@ RenderObject* RenderObjectChildList::beforePseudoElementRenderer(const RenderObj
     do {
         // Skip list markers and generated run-ins
         first = first->firstChild();
-        while (first && (first->isListMarker() || (first->isRenderInline() && first->isRunIn() && first->isAnonymous())))
+        while (first && first->isListMarker()) {
+            if (first->parent() != owner && first->parent()->isAnonymousBlock())
+                first = first->parent();
+            first = first->nextSibling();
+        }
+        while (first && first->isRenderInline() && first->isRunIn())
             first = first->nextSibling();
     } while (first && first->isAnonymous() && first->style()->styleType() == NOPSEUDO);
 
@@ -319,7 +326,7 @@ RenderObject* RenderObjectChildList::beforePseudoElementRenderer(const RenderObj
         // We still need to skip any list markers that could exist before the run-in.
         while (first && first->isListMarker())
             first = first->nextSibling();
-        if (first && first->style()->styleType() == BEFORE && first->isRenderInline() && first->isRunIn() && first->isAnonymous())
+        if (first && first->style()->styleType() == BEFORE && first->isRenderInline() && first->isRunIn())
             return first;
     }
     return 0;
@@ -395,7 +402,7 @@ void RenderObjectChildList::updateBeforeAfterContent(RenderObject* owner, Pseudo
     if (!newContentWanted)
         return;
 
-    if (owner->isRenderInline() && !pseudoElementStyle->isDisplayInlineType() && pseudoElementStyle->floating() == FNONE &&
+    if (owner->isRenderInline() && !pseudoElementStyle->isDisplayInlineType() && !pseudoElementStyle->isFloating() &&
         !(pseudoElementStyle->position() == AbsolutePosition || pseudoElementStyle->position() == FixedPosition))
         // According to the CSS2 spec (the end of section 12.1), the only allowed
         // display values for the pseudo style are NONE and INLINE for inline flows.
@@ -414,6 +421,21 @@ void RenderObjectChildList::updateBeforeAfterContent(RenderObject* owner, Pseudo
             if (!beforeAfterParent)
                 return;
 
+            // When beforeAfterParent is not equal to child (e.g. in tables),
+            // we need to create new styles inheriting from pseudoElementStyle 
+            // on all the intermediate parents (leaving their display same).
+            if (beforeAfterParent != child) {
+                RenderObject* curr = beforeAfterParent;
+                while (curr && curr != child) {
+                    ASSERT(curr->isAnonymous());
+                    RefPtr<RenderStyle> newStyle = RenderStyle::create();
+                    newStyle->inheritFrom(pseudoElementStyle);
+                    newStyle->setDisplay(curr->style()->display());
+                    curr->setStyle(newStyle);
+                    curr = curr->parent();
+                }
+            }
+
             // Note that if we ever support additional types of generated content (which should be way off
             // in the future), this code will need to be patched.
             for (RenderObject* genChild = beforeAfterParent->firstChild(); genChild; genChild = genChild->nextSibling()) {
@@ -436,6 +458,14 @@ void RenderObjectChildList::updateBeforeAfterContent(RenderObject* owner, Pseudo
     }
     
     RenderObject* insertBefore = (type == BEFORE) ? owner->virtualChildren()->firstChild() : 0;
+    if (insertBefore && insertBefore->isAnonymousBlock() && insertBefore->childrenInline() && !insertBefore->isEmpty()) {
+        // We are going to add the "before" element. We have to check whether the "insertBefore" element
+        // is an anonymous block with inline children. If it is, then we should insert the "before" element
+        // before the first inline child of the anonymous block, otherwise we will end up with the "before"
+        // element in a different block. We do this only when the anonymous block has children, otherwise
+        // we end up with the before element in a wrong block.
+        insertBefore = insertBefore->firstChild();
+    }
 
     // Generated content consists of a single container that houses multiple children (specified
     // by the content property).  This generated content container gets the pseudo-element style set on it.
@@ -448,7 +478,7 @@ void RenderObjectChildList::updateBeforeAfterContent(RenderObject* owner, Pseudo
             case CONTENT_NONE:
                 break;
             case CONTENT_TEXT:
-                renderer = new (owner->renderArena()) RenderTextFragment(owner->document() /* anonymous object */, content->text());
+                renderer = new (owner->renderArena()) RenderTextFragment(owner->document() /* anonymous object */, static_cast<const TextContentData*>(content)->text().impl());
                 renderer->setStyle(pseudoElementStyle);
                 break;
             case CONTENT_OBJECT: {
@@ -456,19 +486,19 @@ void RenderObjectChildList::updateBeforeAfterContent(RenderObject* owner, Pseudo
                 RefPtr<RenderStyle> style = RenderStyle::create();
                 style->inheritFrom(pseudoElementStyle);
                 image->setStyle(style.release());
-                if (StyleImage* styleImage = content->image())
-                    image->setImageResource(RenderImageResourceStyleImage::create(styleImage));
+                if (const StyleImage* styleImage = static_cast<const ImageContentData*>(content)->image())
+                    image->setImageResource(RenderImageResourceStyleImage::create(const_cast<StyleImage*>(styleImage)));
                 else
                     image->setImageResource(RenderImageResource::create());
                 renderer = image;
                 break;
             }
         case CONTENT_COUNTER:
-            renderer = new (owner->renderArena()) RenderCounter(owner->document(), *content->counter());
+            renderer = new (owner->renderArena()) RenderCounter(owner->document(), *static_cast<const CounterContentData*>(content)->counter());
             renderer->setStyle(pseudoElementStyle);
             break;
         case CONTENT_QUOTE:
-            renderer = new (owner->renderArena()) RenderQuote(owner->document(), content->quote());
+            renderer = new (owner->renderArena()) RenderQuote(owner->document(), static_cast<const QuoteContentData*>(content)->quote());
             renderer->setStyle(pseudoElementStyle);
             break;
         }
@@ -481,6 +511,12 @@ void RenderObjectChildList::updateBeforeAfterContent(RenderObject* owner, Pseudo
                 ASSERT(styledObject->node()); // The styled object cannot be anonymous or else it could not have ':before' or ':after' pseudo elements.
                 generatedContentContainer->setNode(styledObject->node()); // This allows access to the generatingNode.
                 generatedContentContainer->setStyle(pseudoElementStyle);
+                if (!owner->isChildAllowed(generatedContentContainer, pseudoElementStyle)) {
+                    // The generated content container is not allowed here -> abort.
+                    generatedContentContainer->destroy();
+                    renderer->destroy();
+                    return;
+                }
                 owner->addChild(generatedContentContainer, insertBefore);
             }
             if (generatedContentContainer->isChildAllowed(renderer, pseudoElementStyle))
