@@ -29,6 +29,8 @@
 #include "GraphicsContext.h"
 #include "HTMLFrameOwnerElement.h"
 #include "HitTestResult.h"
+#include "Page.h"
+#include "RenderFlowThread.h"
 #include "RenderLayer.h"
 #include "RenderSelectionInfo.h"
 #include "RenderWidget.h"
@@ -51,8 +53,10 @@ RenderView::RenderView(Node* node, FrameView* view)
     , m_maximalOutlineSize(0)
     , m_pageLogicalHeight(0)
     , m_pageLogicalHeightChanged(false)
+    , m_isRenderFlowThreadOrderDirty(false)
     , m_layoutState(0)
     , m_layoutStateDisableCount(0)
+    , m_currentRenderFlowThread(0)
 {
     // Clear our anonymous bit, set because RenderObject assumes
     // any renderer with document as the node is anonymous.
@@ -123,28 +127,33 @@ void RenderView::layout()
     state.m_clipped = false;
     state.m_pageLogicalHeight = m_pageLogicalHeight;
     state.m_pageLogicalHeightChanged = m_pageLogicalHeightChanged;
+    state.m_isPaginated = state.m_pageLogicalHeight;
     m_pageLogicalHeightChanged = false;
     m_layoutState = &state;
 
-    if (needsLayout())
+    if (needsLayout()) {
         RenderBlock::layout();
+        if (hasRenderFlowThreads())
+            layoutRenderFlowThreads();
+    }
 
-    ASSERT(layoutDelta() == IntSize());
+    ASSERT(layoutDelta() == LayoutSize());
     ASSERT(m_layoutStateDisableCount == 0);
     ASSERT(m_layoutState == &state);
     m_layoutState = 0;
     setNeedsLayout(false);
 }
 
-void RenderView::mapLocalToContainer(RenderBoxModelObject* repaintContainer, bool fixed, bool useTransforms, TransformState& transformState) const
+void RenderView::mapLocalToContainer(RenderBoxModelObject* repaintContainer, bool fixed, bool useTransforms, TransformState& transformState, bool* wasFixed) const
 {
     // If a container was specified, and was not 0 or the RenderView,
     // then we should have found it by now.
-    ASSERT_UNUSED(repaintContainer, !repaintContainer || repaintContainer == this);
+    ASSERT_ARG(repaintContainer, !repaintContainer || repaintContainer == this);
+    ASSERT_UNUSED(wasFixed, !wasFixed || *wasFixed == fixed);
 
-    if (useTransforms && shouldUseTransformFromContainer(0)) {
+    if (!repaintContainer && useTransforms && shouldUseTransformFromContainer(0)) {
         TransformationMatrix t;
-        getTransformFromContainer(0, IntSize(), t);
+        getTransformFromContainer(0, LayoutSize(), t);
         transformState.applyTransform(t);
     }
     
@@ -155,20 +164,20 @@ void RenderView::mapLocalToContainer(RenderBoxModelObject* repaintContainer, boo
 void RenderView::mapAbsoluteToLocalPoint(bool fixed, bool useTransforms, TransformState& transformState) const
 {
     if (fixed && m_frameView)
-        transformState.move(-m_frameView->scrollOffsetForFixedPosition());
+        transformState.move(m_frameView->scrollOffsetForFixedPosition());
 
     if (useTransforms && shouldUseTransformFromContainer(0)) {
         TransformationMatrix t;
-        getTransformFromContainer(0, IntSize(), t);
+        getTransformFromContainer(0, LayoutSize(), t);
         transformState.applyTransform(t);
     }
 }
 
-void RenderView::paint(PaintInfo& paintInfo, int tx, int ty)
+void RenderView::paint(PaintInfo& paintInfo, const LayoutPoint& paintOffset)
 {
     // If we ever require layout but receive a paint anyway, something has gone horribly wrong.
     ASSERT(!needsLayout());
-    paintObject(paintInfo, tx, ty);
+    paintObject(paintInfo, paintOffset);
 }
 
 static inline bool isComposited(RenderObject* object)
@@ -184,7 +193,7 @@ static inline bool rendererObscuresBackground(RenderObject* object)
         && !isComposited(object);
 }
     
-void RenderView::paintBoxDecorations(PaintInfo& paintInfo, int, int)
+void RenderView::paintBoxDecorations(PaintInfo& paintInfo, const LayoutPoint&)
 {
     // Check to see if we are enclosed by a layer that requires complex painting rules.  If so, we cannot blit
     // when scrolling, and we need to use slow repaints.  Examples of layers that require this are transparent layers,
@@ -194,15 +203,15 @@ void RenderView::paintBoxDecorations(PaintInfo& paintInfo, int, int)
     Element* elt;
     for (elt = document()->ownerElement(); view() && elt && elt->renderer(); elt = elt->document()->ownerElement()) {
         RenderLayer* layer = elt->renderer()->enclosingLayer();
-        if (layer->requiresSlowRepaints()) {
-            frameView()->setUseSlowRepaints();
+        if (layer->cannotBlitToWindow()) {
+            frameView()->setCannotBlitToWindow();
             break;
         }
 
 #if USE(ACCELERATED_COMPOSITING)
         if (RenderLayer* compositingLayer = layer->enclosingCompositingLayer()) {
             if (!compositingLayer->backing()->paintingGoesToWindow()) {
-                frameView()->setUseSlowRepaints();
+                frameView()->setCannotBlitToWindow();
                 break;
             }
         }
@@ -220,16 +229,19 @@ void RenderView::paintBoxDecorations(PaintInfo& paintInfo, int, int)
         rootFillsViewport = rootBox && !rootBox->x() && !rootBox->y() && rootBox->width() >= width() && rootBox->height() >= height();
     }
     
+    Page* page = document()->page();
+    float pageScaleFactor = page ? page->pageScaleFactor() : 1;
+
     // If painting will entirely fill the view, no need to fill the background.
-    if (rootFillsViewport && rendererObscuresBackground(firstChild()))
+    if (rootFillsViewport && rendererObscuresBackground(firstChild()) && pageScaleFactor >= 1)
         return;
 
     // This code typically only executes if the root element's visibility has been set to hidden,
-    // or there is a transform on the <html>.
+    // if there is a transform on the <html>, or if there is a page scale factor less than 1.
     // Only fill with the base background color (typically white) if we're the root document, 
     // since iframes/frames with no background in the child document should show the parent's background.
     if (frameView()->isTransparent()) // FIXME: This needs to be dynamic.  We should be able to go back to blitting if we ever stop being transparent.
-        frameView()->setUseSlowRepaints(); // The parent must show behind the child.
+        frameView()->setCannotBlitToWindow(); // The parent must show behind the child.
     else {
         Color baseColor = frameView()->baseBackgroundColor();
         if (baseColor.alpha() > 0) {
@@ -253,6 +265,11 @@ bool RenderView::shouldRepaint(const IntRect& r) const
     return true;
 }
 
+RenderBlock* RenderView::containingBlock() const
+{
+    return const_cast<RenderView*>(this);
+}
+
 void RenderView::repaintViewRectangle(const IntRect& ur, bool immediate)
 {
     if (!shouldRepaint(ur))
@@ -269,7 +286,7 @@ void RenderView::repaintViewRectangle(const IntRect& ur, bool immediate)
         
         // Subtract out the contentsX and contentsY offsets to get our coords within the viewing
         // rectangle.
-        r.move(-vr.x(), -vr.y());
+        r.moveBy(-vr.location());
         
         // FIXME: Hardcoded offsets here are not good.
         r.move(obj->borderLeft() + obj->paddingLeft(),
@@ -291,11 +308,11 @@ void RenderView::repaintRectangleInViewAndCompositedLayers(const IntRect& ur, bo
 #endif
 }
 
-void RenderView::computeRectForRepaint(RenderBoxModelObject* repaintContainer, IntRect& rect, bool fixed)
+void RenderView::computeRectForRepaint(RenderBoxModelObject* repaintContainer, IntRect& rect, bool fixed) const
 {
     // If a container was specified, and was not 0 or the RenderView,
     // then we should have found it by now.
-    ASSERT_UNUSED(repaintContainer, !repaintContainer || repaintContainer == this);
+    ASSERT_ARG(repaintContainer, !repaintContainer || repaintContainer == this);
 
     if (printing())
         return;
@@ -313,18 +330,20 @@ void RenderView::computeRectForRepaint(RenderBoxModelObject* repaintContainer, I
         rect.move(m_frameView->scrollXForFixedPosition(), m_frameView->scrollYForFixedPosition());
         
     // Apply our transform if we have one (because of full page zooming).
-    if (m_layer && m_layer->transform())
+    if (!repaintContainer && m_layer && m_layer->transform())
         rect = m_layer->transform()->mapRect(rect);
 }
 
-void RenderView::absoluteRects(Vector<IntRect>& rects, int tx, int ty)
+void RenderView::absoluteRects(Vector<LayoutRect>& rects, const LayoutPoint& accumulatedOffset)
 {
-    rects.append(IntRect(tx, ty, m_layer->width(), m_layer->height()));
+    rects.append(LayoutRect(accumulatedOffset, m_layer->size()));
 }
 
-void RenderView::absoluteQuads(Vector<FloatQuad>& quads)
+void RenderView::absoluteQuads(Vector<FloatQuad>& quads, bool* wasFixed)
 {
-    quads.append(FloatRect(0, 0, m_layer->width(), m_layer->height()));
+    if (wasFixed)
+        *wasFixed = false;
+    quads.append(FloatRect(FloatPoint(), m_layer->size()));
 }
 
 static RenderObject* rendererAfterPosition(RenderObject* object, unsigned offset)
@@ -649,40 +668,20 @@ IntRect RenderView::viewRect() const
     return IntRect();
 }
 
-int RenderView::docTop() const
-{
-    IntRect overflowRect(0, minYLayoutOverflow(), 0, maxYLayoutOverflow() - minYLayoutOverflow());
-    flipForWritingMode(overflowRect);
-    if (hasTransform())
-        overflowRect = layer()->currentTransform().mapRect(overflowRect);
-    return overflowRect.y();
-}
 
-int RenderView::docBottom() const
+IntRect RenderView::unscaledDocumentRect() const
 {
     IntRect overflowRect(layoutOverflowRect());
     flipForWritingMode(overflowRect);
-    if (hasTransform())
-        overflowRect = layer()->currentTransform().mapRect(overflowRect);
-    return overflowRect.maxY();
+    return overflowRect;
 }
 
-int RenderView::docLeft() const
+IntRect RenderView::documentRect() const
 {
-    IntRect overflowRect(layoutOverflowRect());
-    flipForWritingMode(overflowRect);
+    IntRect overflowRect(unscaledDocumentRect());
     if (hasTransform())
         overflowRect = layer()->currentTransform().mapRect(overflowRect);
-    return overflowRect.x();
-}
-
-int RenderView::docRight() const
-{
-    IntRect overflowRect(layoutOverflowRect());
-    flipForWritingMode(overflowRect);
-    if (hasTransform())
-        overflowRect = layer()->currentTransform().mapRect(overflowRect);
-    return overflowRect.maxX();
+    return overflowRect;
 }
 
 int RenderView::viewHeight() const
@@ -719,6 +718,11 @@ void RenderView::pushLayoutState(RenderObject* root)
     m_layoutState = new (renderArena()) LayoutState(root);
 }
 
+void RenderView::pushLayoutState(RenderFlowThread* flowThread)
+{
+    m_layoutState = new (renderArena()) LayoutState(m_layoutState, flowThread);
+}
+
 bool RenderView::shouldDisableLayoutStateForSubtree(RenderObject* renderer) const
 {
     RenderObject* o = renderer;
@@ -730,7 +734,7 @@ bool RenderView::shouldDisableLayoutStateForSubtree(RenderObject* renderer) cons
     return false;
 }
 
-void RenderView::updateHitTestResult(HitTestResult& result, const IntPoint& point)
+void RenderView::updateHitTestResult(HitTestResult& result, const LayoutPoint& point)
 {
     if (result.innerNode())
         return;
@@ -778,7 +782,7 @@ bool RenderView::usesCompositing() const
 RenderLayerCompositor* RenderView::compositor()
 {
     if (!m_compositor)
-        m_compositor.set(new RenderLayerCompositor(this));
+        m_compositor = adoptPtr(new RenderLayerCompositor(this));
 
     return m_compositor.get();
 }
@@ -800,4 +804,62 @@ void RenderView::willMoveOffscreen()
 #endif
 }
 
+void RenderView::styleDidChange(StyleDifference diff, const RenderStyle* oldStyle)
+{
+    RenderBlock::styleDidChange(diff, oldStyle);
+    
+    for (RenderObject* renderer = firstChild(); renderer; renderer = renderer->nextSibling()) {
+        if (renderer->isRenderFlowThread()) {
+            RenderFlowThread* flowRenderer = toRenderFlowThread(renderer);
+            flowRenderer->setStyle(RenderFlowThread::createFlowThreadStyle(style()));
+        }
+    }
+}
+
+RenderFlowThread* RenderView::renderFlowThreadWithName(const AtomicString& flowThread)
+{
+    if (!m_renderFlowThreadList)
+        m_renderFlowThreadList = adoptPtr(new RenderFlowThreadList());
+    else {
+        for (RenderFlowThreadList::iterator iter = m_renderFlowThreadList->begin(); iter != m_renderFlowThreadList->end(); ++iter) {
+            RenderFlowThread* flowRenderer = *iter;
+            if (flowRenderer->flowThread() == flowThread)
+                return flowRenderer;
+        }
+    }
+
+    RenderFlowThread* flowRenderer = new (renderArena()) RenderFlowThread(document(), flowThread);
+    flowRenderer->setStyle(RenderFlowThread::createFlowThreadStyle(style()));
+    addChild(flowRenderer);
+    
+    m_renderFlowThreadList->add(flowRenderer);
+    setIsRenderFlowThreadOrderDirty(true);
+
+    return flowRenderer;
+}
+
+void RenderView::layoutRenderFlowThreads()
+{
+    ASSERT(m_renderFlowThreadList);
+
+    if (isRenderFlowThreadOrderDirty()) {
+        // Arrange the thread list according to dependencies.
+        RenderFlowThreadList sortedList;
+        for (RenderFlowThreadList::iterator iter = m_renderFlowThreadList->begin(); iter != m_renderFlowThreadList->end(); ++iter) {
+            RenderFlowThread* flowRenderer = *iter;
+            if (sortedList.contains(flowRenderer))
+                continue;
+            flowRenderer->pushDependencies(sortedList);
+            sortedList.add(flowRenderer);
+        }
+        m_renderFlowThreadList->swap(sortedList);
+        setIsRenderFlowThreadOrderDirty(false);
+    }
+
+    for (RenderFlowThreadList::iterator iter = m_renderFlowThreadList->begin(); iter != m_renderFlowThreadList->end(); ++iter) {
+        RenderFlowThread* flowRenderer = *iter;
+        flowRenderer->layoutIfNeeded();
+    }
+}
+
 } // namespace WebCore
