@@ -188,35 +188,13 @@ static TextStream& operator<<(TextStream& ts, const WindRule rule)
 
 static TextStream& operator<<(TextStream& ts, const SVGUnitTypes::SVGUnitType& unitType)
 {
-    switch (unitType) {
-    case SVGUnitTypes::SVG_UNIT_TYPE_UNKNOWN:
-        ts << "unknown";
-        break;
-    case SVGUnitTypes::SVG_UNIT_TYPE_USERSPACEONUSE:
-        ts << "userSpaceOnUse";
-        break;
-    case SVGUnitTypes::SVG_UNIT_TYPE_OBJECTBOUNDINGBOX:
-        ts << "objectBoundingBox";
-        break;
-    }
-
+    ts << SVGPropertyTraits<SVGUnitTypes::SVGUnitType>::toString(unitType);
     return ts;
 }
 
-static TextStream& operator<<(TextStream& ts, const SVGMarkerElement::SVGMarkerUnitsType& markerUnit)
+static TextStream& operator<<(TextStream& ts, const SVGMarkerUnitsType& markerUnit)
 {
-    switch (markerUnit) {
-    case SVGMarkerElement::SVG_MARKERUNITS_UNKNOWN:
-        ts << "unknown";
-        break;
-    case SVGMarkerElement::SVG_MARKERUNITS_USERSPACEONUSE:
-        ts << "userSpaceOnUse";
-        break;
-    case SVGMarkerElement::SVG_MARKERUNITS_STROKEWIDTH:
-        ts << "strokeWidth";
-        break;
-    }
-
+    ts << SVGPropertyTraits<SVGMarkerUnitsType>::toString(markerUnit);
     return ts;
 }
 
@@ -273,21 +251,9 @@ static TextStream& operator<<(TextStream& ts, LineJoin style)
     return ts;
 }
 
-// FIXME: Maybe this should be in Gradient.cpp
-static TextStream& operator<<(TextStream& ts, GradientSpreadMethod mode)
+static TextStream& operator<<(TextStream& ts, const SVGSpreadMethodType& type)
 {
-    switch (mode) {
-    case SpreadMethodPad:
-        ts << "PAD";
-        break;
-    case SpreadMethodRepeat:
-        ts << "REPEAT";
-        break;
-    case SpreadMethodReflect:
-        ts << "REFLECT";
-        break;
-    }
-
+    ts << SVGPropertyTraits<SVGSpreadMethodType>::toString(type).upper();
     return ts;
 }
 
@@ -321,7 +287,7 @@ static void writeStyle(TextStream& ts, const RenderObject& object)
 
     if (!object.localTransform().isIdentity())
         writeNameValuePair(ts, "transform", object.localTransform());
-    writeIfNotDefault(ts, "image rendering", svgStyle->imageRendering(), SVGRenderStyle::initialImageRendering());
+    writeIfNotDefault(ts, "image rendering", style->imageRendering(), RenderStyle::initialImageRendering());
     writeIfNotDefault(ts, "opacity", style->opacity(), RenderStyle::initialOpacity());
     if (object.isSVGPath()) {
         const RenderSVGPath& path = static_cast<const RenderSVGPath&>(object);
@@ -506,8 +472,12 @@ static inline void writeSVGInlineTextBox(TextStream& ts, SVGInlineTextBox* textB
 
 static inline void writeSVGInlineTextBoxes(TextStream& ts, const RenderText& text, int indent)
 {
-    for (InlineTextBox* box = text.firstTextBox(); box; box = box->nextTextBox())
+    for (InlineTextBox* box = text.firstTextBox(); box; box = box->nextTextBox()) {
+        if (!box->isSVGInlineTextBox())
+            continue;
+
         writeSVGInlineTextBox(ts, static_cast<SVGInlineTextBox*>(box), indent);
+    }
 }
 
 static void writeStandardPrefix(TextStream& ts, const RenderObject& object, int indent)
@@ -530,11 +500,11 @@ static inline String boundingBoxModeString(bool boundingBoxMode)
     return boundingBoxMode ? "objectBoundingBox" : "userSpaceOnUse";
 }
 
-static inline void writeCommonGradientProperties(TextStream& ts, GradientSpreadMethod spreadMethod, const AffineTransform& gradientTransform, bool boundingBoxMode)
+static inline void writeCommonGradientProperties(TextStream& ts, SVGSpreadMethodType spreadMethod, const AffineTransform& gradientTransform, bool boundingBoxMode)
 {
     writeNameValuePair(ts, "gradientUnits", boundingBoxModeString(boundingBoxMode));
 
-    if (spreadMethod != SpreadMethodPad)
+    if (spreadMethod != SVGSpreadMethodPad)
         ts << " [spreadMethod=" << spreadMethod << "]";
 
     if (!gradientTransform.isIdentity())
