@@ -33,13 +33,14 @@
 #include "SVGRenderStyle.h"
 #include "SVGURIReference.h"
 
+#ifndef NDEBUG
+#include <stdio.h>
+#endif
+
 namespace WebCore {
 
 SVGResources::SVGResources()
-    : m_clipperFilterMaskerData(0)
-    , m_markerData(0)
-    , m_fillStrokeData(0)
-    , m_linkedResource(0)
+    : m_linkedResource(0)
 {
 }
 
@@ -149,7 +150,7 @@ static inline String targetReferenceFromResource(SVGElement* element)
     else
         ASSERT_NOT_REACHED();
 
-    return SVGURIReference::getTarget(target);
+    return SVGURIReference::fragmentIdentifierFromIRIString(target, element->document());
 }
 
 static inline RenderSVGResourceContainer* paintingResourceFromSVGPaint(Document* document, const SVGPaint::SVGPaintType& paintType, const String& paintUri, AtomicString& id, bool& hasPendingResource)
@@ -157,7 +158,7 @@ static inline RenderSVGResourceContainer* paintingResourceFromSVGPaint(Document*
     if (paintType != SVGPaint::SVG_PAINTTYPE_URI && paintType != SVGPaint::SVG_PAINTTYPE_URI_RGBCOLOR)
         return 0;
 
-    id = SVGURIReference::getTarget(paintUri);
+    id = SVGURIReference::fragmentIdentifierFromIRIString(paintUri, document);
     RenderSVGResourceContainer* container = getRenderSVGResourceContainerById(document, id);
     if (!container) {
         hasPendingResource = true;
@@ -174,9 +175,7 @@ static inline RenderSVGResourceContainer* paintingResourceFromSVGPaint(Document*
 static inline void registerPendingResource(SVGDocumentExtensions* extensions, const AtomicString& id, SVGElement* element)
 {
     ASSERT(element);
-    if (!element->isStyled())
-        return;
-
+    ASSERT(element->isStyled());
     extensions->addPendingResource(id, static_cast<SVGStyledElement*>(element));
 }
 
