@@ -26,6 +26,8 @@
 #include "config.h"
 #include "StorageTracker.h"
 
+#if ENABLE(DOM_STORAGE)
+
 #include "DatabaseThread.h"
 #include "FileSystem.h"
 #include "LocalStorageTask.h"
@@ -46,14 +48,24 @@ namespace WebCore {
 
 static StorageTracker* storageTracker = 0;
 
-void StorageTracker::initializeTracker(const String& storagePath)
+void StorageTracker::initializeTracker(const String& storagePath, StorageTrackerClient* client)
 {
     ASSERT(isMainThread());
-    ASSERT(!storageTracker);
+    ASSERT(!storageTracker || !storageTracker->m_client);
     
     if (!storageTracker)
         storageTracker = new StorageTracker(storagePath);
     
+    storageTracker->m_client = client;
+    storageTracker->m_needsInitialization = true;
+}
+
+void StorageTracker::internalInitialize()
+{
+    m_needsInitialization = false;
+
+    ASSERT(isMainThread());
+
     // Make sure text encoding maps have been built on the main thread, as the StorageTracker thread might try to do it there instead.
     // FIXME (<rdar://problem/9127819>): Is there a more explicit way of doing this besides accessing the UTF8Encoding?
     UTF8Encoding();
@@ -68,24 +80,19 @@ StorageTracker& StorageTracker::tracker()
 {
     if (!storageTracker)
         storageTracker = new StorageTracker("");
+    if (storageTracker->m_needsInitialization)
+        storageTracker->internalInitialize();
     
     return *storageTracker;
 }
 
 StorageTracker::StorageTracker(const String& storagePath)
-    : m_client(0)
+    : m_storageDirectoryPath(storagePath.threadsafeCopy())
+    , m_client(0)
     , m_thread(LocalStorageThread::create())
     , m_isActive(false)
+    , m_needsInitialization(false)
 {
-    setStorageDirectoryPath(storagePath);
-}
-
-void StorageTracker::setStorageDirectoryPath(const String& path)
-{
-    MutexLocker lockDatabase(m_databaseGuard);
-    ASSERT(!m_database.isOpen());
-    
-    m_storageDirectoryPath = path.threadsafeCopy();
 }
 
 String StorageTracker::trackerDatabasePath()
@@ -193,7 +200,7 @@ void StorageTracker::syncFileSystemAndTrackerDatabase()
     m_databaseGuard.lock();
     DEFINE_STATIC_LOCAL(const String, fileMatchPattern, ("*.localstorage"));
     DEFINE_STATIC_LOCAL(const String, fileExt, (".localstorage"));
-    DEFINE_STATIC_LOCAL(const unsigned, fileExtLength, (fileExt.length()));
+    static const unsigned fileExtLength = fileExt.length();
     m_databaseGuard.unlock();
 
     Vector<String> paths;
@@ -230,8 +237,10 @@ void StorageTracker::syncFileSystemAndTrackerDatabase()
     // Delete stale StorageTracker records.
     OriginSet::const_iterator setEnd = originSetCopy.end();
     for (OriginSet::const_iterator it = originSetCopy.begin(); it != setEnd; ++it) {
-        if (!foundOrigins.contains(*it))
-            syncDeleteOrigin(*it);
+        if (!foundOrigins.contains(*it)) {
+            RefPtr<StringImpl> originIdentifier = (*it).threadsafeCopy().impl();
+            callOnMainThread(deleteOriginOnMainThread, originIdentifier.release().leakRef());
+        }
     }
 }
 
@@ -374,10 +383,33 @@ void StorageTracker::syncDeleteAllOrigins()
     if (m_database.isOpen())
         m_database.close();
     
-    SQLiteFileSystem::deleteDatabaseFile(trackerDatabasePath());
+    if (!SQLiteFileSystem::deleteDatabaseFile(trackerDatabasePath())) {
+        // In the case where it is not possible to delete the database file (e.g some other program
+        // like a virus scanner is accessing it), make sure to remove all entries.
+        openTrackerDatabase(false);
+        if (!m_database.isOpen())
+            return;
+        SQLiteStatement deleteStatement(m_database, "DELETE FROM Origins");
+        if (deleteStatement.prepare() != SQLResultOk) {
+            LOG_ERROR("Unable to prepare deletion of all origins");
+            return;
+        }
+        if (!deleteStatement.executeCommand()) {
+            LOG_ERROR("Unable to execute deletion of all origins");
+            return;
+        }
+    }
     SQLiteFileSystem::deleteEmptyDatabaseDirectory(m_storageDirectoryPath);
 }
 
+void StorageTracker::deleteOriginOnMainThread(void* originIdentifier)
+{
+    ASSERT(isMainThread());
+
+    String identifier = adoptRef(reinterpret_cast<StringImpl*>(originIdentifier));
+    tracker().deleteOrigin(identifier);
+}
+
 void StorageTracker::deleteOrigin(const String& originIdentifier)
 {
     deleteOrigin(SecurityOrigin::createFromDatabaseIdentifier(originIdentifier).get());
@@ -425,23 +457,13 @@ void StorageTracker::syncDeleteOrigin(const String& originIdentifier)
     openTrackerDatabase(false);
     if (!m_database.isOpen())
         return;
-    
-    // Get origin's db file path, delete entry in tracker's db, then delete db file.
-    SQLiteStatement pathStatement(m_database, "SELECT path FROM Origins WHERE origin=?");
-    if (pathStatement.prepare() != SQLResultOk) {
-        LOG_ERROR("Unable to prepare selection of path for origin '%s'", originIdentifier.ascii().data());
-        return;
-    }
-    pathStatement.bindText(1, originIdentifier);
-    int result = pathStatement.step();
-    if (result != SQLResultRow) {
-        LOG_ERROR("Unable to find origin '%s' in Origins table", originIdentifier.ascii().data());
-        return;
-    }
-    
-    String path = pathStatement.getColumnText(0);
 
-    ASSERT(!path.isEmpty());
+    String path = databasePathForOrigin(originIdentifier);
+    if (path.isEmpty()) {
+        // It is possible to get a request from the API to delete the storage for an origin that
+        // has no such storage.
+        return;
+    }
     
     SQLiteStatement deleteStatement(m_database, "DELETE FROM Origins where origin=?");
     if (deleteStatement.prepare() != SQLResultOk) {
@@ -531,5 +553,42 @@ void StorageTracker::setIsActive(bool flag)
 {
     m_isActive = flag;
 }
+    
+String StorageTracker::databasePathForOrigin(const String& originIdentifier)
+{
+    ASSERT(!m_databaseGuard.tryLock());
+    ASSERT(m_isActive);
+    
+    if (!m_database.isOpen())
+        return String();
+    
+    SQLiteStatement pathStatement(m_database, "SELECT path FROM Origins WHERE origin=?");
+    if (pathStatement.prepare() != SQLResultOk) {
+        LOG_ERROR("Unable to prepare selection of path for origin '%s'", originIdentifier.ascii().data());
+        return String();
+    }
+    pathStatement.bindText(1, originIdentifier);
+    int result = pathStatement.step();
+    if (result != SQLResultRow)
+        return String();
 
+    return pathStatement.getColumnText(0);
+}
+    
+long long StorageTracker::diskUsageForOrigin(SecurityOrigin* origin)
+{
+    if (!m_isActive)
+        return 0;
+
+    MutexLocker lock(m_databaseGuard);
+
+    String path = databasePathForOrigin(origin->databaseIdentifier());
+    if (path.isEmpty())
+        return 0;
+
+    return SQLiteFileSystem::getDatabaseFileSize(path);
+}
+    
 } // namespace WebCore
+
+#endif // ENABLE(DOM_STORAGE)
