@@ -25,7 +25,7 @@
 
 #include "config.h"
 
-#if ENABLE(SVG_ANIMATION)
+#if ENABLE(SVG)
 #include "SVGSMILElement.h"
 
 #include "Attribute.h"
@@ -119,10 +119,10 @@ SVGSMILElement::SVGSMILElement(const QualifiedName& tagName, Document* doc)
     , m_targetElement(0)
     , m_conditionsConnected(false)
     , m_hasEndEventConditions(false)
+    , m_isWaitingForFirstInterval(true)
     , m_intervalBegin(SMILTime::unresolved())
     , m_intervalEnd(SMILTime::unresolved())
     , m_previousIntervalBegin(SMILTime::unresolved())
-    , m_isWaitingForFirstInterval(true)
     , m_activeState(Inactive)
     , m_lastPercent(0)
     , m_lastRepeat(0)
@@ -141,6 +141,8 @@ SVGSMILElement::~SVGSMILElement()
     disconnectConditions();
     if (m_timeContainer)
         m_timeContainer->unschedule(this);
+    if (m_targetElement)
+        document()->accessSVGExtensions()->removeAnimationElementFromTarget(this, m_targetElement);
 }
 
 static inline QualifiedName constructQualifiedName(const SVGElement* svgElement, const String& attributeName)
@@ -171,16 +173,24 @@ void SVGSMILElement::insertedIntoDocument()
 #ifndef NDEBUG
     // Verify we are not in <use> instance tree.
     for (ContainerNode* n = this; n; n = n->parentNode())
-        ASSERT(!n->isShadowRoot());
+        ASSERT(!n->isSVGShadowRoot());
 #endif
-    m_attributeName = constructQualifiedName(this, getAttribute(SVGNames::attributeNameAttr));
+    m_attributeName = constructQualifiedName(this, fastGetAttribute(SVGNames::attributeNameAttr));
     SVGSVGElement* owner = ownerSVGElement();
     if (!owner)
         return;
     m_timeContainer = owner->timeContainer();
     ASSERT(m_timeContainer);
     m_timeContainer->setDocumentOrderIndexesDirty();
-    reschedule();
+
+    // "If no attribute is present, the default begin value (an offset-value of 0) must be evaluated."
+    if (!fastHasAttribute(SVGNames::beginAttr))
+        m_beginTimes.append(0);
+
+    if (m_isWaitingForFirstInterval) {
+        resolveFirstInterval();
+        reschedule();
+    }
 }
 
 void SVGSMILElement::removedFromDocument()
@@ -201,20 +211,6 @@ void SVGSMILElement::removedFromDocument()
     SVGElement::removedFromDocument();
 }
    
-void SVGSMILElement::finishParsingChildren()
-{
-    SVGElement::finishParsingChildren();
-
-    // "If no attribute is present, the default begin value (an offset-value of 0) must be evaluated."
-    if (!hasAttribute(SVGNames::beginAttr))
-        m_beginTimes.append(0);
-
-    if (m_isWaitingForFirstInterval) {
-        resolveFirstInterval();
-        reschedule();
-    }
-}
-
 SMILTime SVGSMILElement::parseOffsetValue(const String& data)
 {
     bool ok;
@@ -377,7 +373,7 @@ void SVGSMILElement::parseMappedAttribute(Attribute* attr)
         if (!m_conditions.isEmpty()) {
             disconnectConditions();
             m_conditions.clear();
-            parseBeginOrEnd(getAttribute(SVGNames::endAttr), End);
+            parseBeginOrEnd(fastGetAttribute(SVGNames::endAttr), End);
         }
         parseBeginOrEnd(attr->value().string(), Begin);
         if (inDocument())
@@ -386,7 +382,7 @@ void SVGSMILElement::parseMappedAttribute(Attribute* attr)
         if (!m_conditions.isEmpty()) {
             disconnectConditions();
             m_conditions.clear();
-            parseBeginOrEnd(getAttribute(SVGNames::beginAttr), Begin);
+            parseBeginOrEnd(fastGetAttribute(SVGNames::beginAttr), Begin);
         }
         parseBeginOrEnd(attr->value().string(), End);
         if (inDocument())
@@ -417,15 +413,15 @@ void SVGSMILElement::attributeChanged(Attribute* attr, bool preserveDecls)
 
     if (inDocument()) {
         if (attrName == SVGNames::beginAttr)
-            beginListChanged();
+            beginListChanged(elapsed());
         else if (attrName == SVGNames::endAttr)
-            endListChanged();
+            endListChanged(elapsed());
     }
 }
 
-inline Element* SVGSMILElement::eventBaseFor(const Condition& condition) const
+inline Element* SVGSMILElement::eventBaseFor(const Condition& condition)
 {
-    return condition.m_baseID.isEmpty() ? targetElement() : document()->getElementById(condition.m_baseID);
+    return condition.m_baseID.isEmpty() ? targetElement() : treeScope()->getElementById(condition.m_baseID);
 }
 
 void SVGSMILElement::connectConditions()
@@ -445,7 +441,7 @@ void SVGSMILElement::connectConditions()
             eventBase->addEventListener(condition.m_name, condition.m_eventListener, false);
         } else if (condition.m_type == Condition::Syncbase) {
             ASSERT(!condition.m_baseID.isEmpty());
-            condition.m_syncbase = document()->getElementById(condition.m_baseID);
+            condition.m_syncbase = treeScope()->getElementById(condition.m_baseID);
             if (!isSMILElement(condition.m_syncbase.get())) {
                 condition.m_syncbase = 0;
                 continue;
@@ -493,18 +489,18 @@ void SVGSMILElement::reschedule()
         m_timeContainer->schedule(this);
 }
 
-SVGElement* SVGSMILElement::targetElement() const
+SVGElement* SVGSMILElement::targetElement()
 {
     if (m_targetElement)
         return m_targetElement;
 
     String href = xlinkHref();
-    ContainerNode* target = href.isEmpty() ? parentNode() : document()->getElementById(SVGURIReference::getTarget(href));
+    ContainerNode* target = href.isEmpty() ? parentNode() : SVGURIReference::targetElementFromIRIString(href, document());
     if (!target || !target->isSVGElement())
         return 0;
     
     m_targetElement = static_cast<SVGElement*>(target);
-    document()->accessSVGExtensions()->addAnimationElementToTarget(const_cast<SVGSMILElement*>(this), m_targetElement);
+    document()->accessSVGExtensions()->addAnimationElementToTarget(this, m_targetElement);
     return m_targetElement;
 }
 
@@ -527,7 +523,7 @@ SVGSMILElement::Restart SVGSMILElement::restart() const
 {    
     DEFINE_STATIC_LOCAL(const AtomicString, never, ("never"));
     DEFINE_STATIC_LOCAL(const AtomicString, whenNotActive, ("whenNotActive"));
-    const AtomicString& value = getAttribute(SVGNames::restartAttr);
+    const AtomicString& value = fastGetAttribute(SVGNames::restartAttr);
     if (value == never)
         return RestartNever;
     if (value == whenNotActive)
@@ -538,20 +534,20 @@ SVGSMILElement::Restart SVGSMILElement::restart() const
 SVGSMILElement::FillMode SVGSMILElement::fill() const
 {   
     DEFINE_STATIC_LOCAL(const AtomicString, freeze, ("freeze"));
-    const AtomicString& value = getAttribute(SVGNames::fillAttr);
+    const AtomicString& value = fastGetAttribute(SVGNames::fillAttr);
     return value == freeze ? FillFreeze : FillRemove;
 }
     
 String SVGSMILElement::xlinkHref() const
 {    
-    return getAttribute(XLinkNames::hrefAttr);
+    return fastGetAttribute(XLinkNames::hrefAttr);
 }
     
 SMILTime SVGSMILElement::dur() const
 {   
     if (m_cachedDur != invalidCachedTime)
         return m_cachedDur;
-    const AtomicString& value = getAttribute(SVGNames::durAttr);
+    const AtomicString& value = fastGetAttribute(SVGNames::durAttr);
     SMILTime clockValue = parseClockValue(value);
     return m_cachedDur = clockValue <= 0 ? SMILTime::unresolved() : clockValue;
 }
@@ -560,7 +556,7 @@ SMILTime SVGSMILElement::repeatDur() const
 {    
     if (m_cachedRepeatDur != invalidCachedTime)
         return m_cachedRepeatDur;
-    const AtomicString& value = getAttribute(SVGNames::repeatDurAttr);
+    const AtomicString& value = fastGetAttribute(SVGNames::repeatDurAttr);
     SMILTime clockValue = parseClockValue(value);
     m_cachedRepeatDur = clockValue <= 0 ? SMILTime::unresolved() : clockValue;
     return m_cachedRepeatDur;
@@ -571,7 +567,7 @@ SMILTime SVGSMILElement::repeatCount() const
 {    
     if (m_cachedRepeatCount != invalidCachedTime)
         return m_cachedRepeatCount;
-    const AtomicString& value = getAttribute(SVGNames::repeatCountAttr);
+    const AtomicString& value = fastGetAttribute(SVGNames::repeatCountAttr);
     if (value.isNull())
         return SMILTime::unresolved();
 
@@ -587,7 +583,7 @@ SMILTime SVGSMILElement::maxValue() const
 {    
     if (m_cachedMax != invalidCachedTime)
         return m_cachedMax;
-    const AtomicString& value = getAttribute(SVGNames::maxAttr);
+    const AtomicString& value = fastGetAttribute(SVGNames::maxAttr);
     SMILTime result = parseClockValue(value);
     return m_cachedMax = (result.isUnresolved() || result < 0) ? SMILTime::indefinite() : result;
 }
@@ -596,7 +592,7 @@ SMILTime SVGSMILElement::minValue() const
 {    
     if (m_cachedMin != invalidCachedTime)
         return m_cachedMin;
-    const AtomicString& value = getAttribute(SVGNames::minAttr);
+    const AtomicString& value = fastGetAttribute(SVGNames::minAttr);
     SMILTime result = parseClockValue(value);
     return m_cachedMin = (result.isUnresolved() || result < 0) ? 0 : result;
 }
@@ -606,39 +602,66 @@ SMILTime SVGSMILElement::simpleDuration() const
     return min(dur(), SMILTime::indefinite());
 }
     
-void SVGSMILElement::addBeginTime(SMILTime time)
+void SVGSMILElement::addBeginTime(SMILTime eventTime, SMILTime beginTime)
 {
-    m_beginTimes.append(time);
+    m_beginTimes.append(beginTime);
     sortTimeList(m_beginTimes);
-    beginListChanged();
+    beginListChanged(eventTime);
 }
     
-void SVGSMILElement::addEndTime(SMILTime time)
+void SVGSMILElement::addEndTime(SMILTime eventTime, SMILTime endTime)
 {
-    m_endTimes.append(time);
+    m_endTimes.append(endTime);
     sortTimeList(m_endTimes);
-    endListChanged();
+    endListChanged(eventTime);
 }
     
+inline SMILTime extractTimeFromVector(const SMILTime* position)
+{
+    return *position;
+}
+
 SMILTime SVGSMILElement::findInstanceTime(BeginOrEnd beginOrEnd, SMILTime minimumTime, bool equalsMinimumOK) const
 {
-    // FIXME: This searches from the beginning which is inefficient. The list is usually not long
-    // (one entry in common cases) but you can construct a case where it does grow.
     const Vector<SMILTime>& list = beginOrEnd == Begin ? m_beginTimes : m_endTimes;
-    for (unsigned n = 0; n < list.size(); ++n) {
-        SMILTime time = list[n];
-        ASSERT(!time.isUnresolved());
-        if (time.isIndefinite() && beginOrEnd == Begin) {
-            // "The special value "indefinite" does not yield an instance time in the begin list."
-            continue;
-        }
-        if (equalsMinimumOK) {
-            if (time >= minimumTime)
-                return time;
-        } else if (time > minimumTime)
-            return time;
+    int sizeOfList = list.size();
+
+    if (!sizeOfList)
+        return beginOrEnd == Begin ? SMILTime::unresolved() : SMILTime::indefinite();
+
+    const SMILTime* result = binarySearch<const SMILTime, SMILTime, extractTimeFromVector>(list.begin(), sizeOfList, minimumTime, WTF::KeyMustNotBePresentInArray);
+    int indexOfResult = result - list.begin();
+
+    if (sizeOfList - 1 > indexOfResult && list[indexOfResult] < minimumTime)
+        ++indexOfResult;
+
+    ASSERT(indexOfResult < sizeOfList);
+
+    // "The special value "indefinite" does not yield an instance time in the begin list."
+    if (list[indexOfResult].isIndefinite() && beginOrEnd == Begin)
+        return SMILTime::unresolved();
+
+    if (list[indexOfResult] < minimumTime)
+        return beginOrEnd == Begin ? SMILTime::unresolved() : SMILTime::indefinite();
+
+    // If the equals is NOT accepted, we have to find a bigger one.
+    if (list[indexOfResult] == minimumTime && !equalsMinimumOK) {
+        if (indexOfResult + 1 >= sizeOfList)
+            return beginOrEnd == Begin ? SMILTime::unresolved() : SMILTime::indefinite();
     }
-    return SMILTime::unresolved();
+
+    while (indexOfResult < sizeOfList - 1 && list[indexOfResult] == list[indexOfResult + 1])
+        ++indexOfResult;
+
+    if (list[indexOfResult] > minimumTime)
+        return list[indexOfResult];
+    if (list[indexOfResult] == minimumTime) {
+        if (indexOfResult + 1 < sizeOfList - 1)
+            return list[indexOfResult + 1];
+        return beginOrEnd == Begin ? SMILTime::unresolved() : SMILTime::indefinite();
+    }
+
+    return list[indexOfResult];
 }
 
 SMILTime SVGSMILElement::repeatingDuration() const
@@ -752,28 +775,27 @@ SMILTime SVGSMILElement::nextProgressTime() const
     return m_nextProgressTime;
 }
     
-void SVGSMILElement::beginListChanged()
+void SVGSMILElement::beginListChanged(SMILTime eventTime)
 {
-    SMILTime elapsed = this->elapsed();
     if (m_isWaitingForFirstInterval)
         resolveFirstInterval();
-    else if (elapsed < m_intervalBegin) {
-        SMILTime newBegin = findInstanceTime(Begin, elapsed, false);
-        if (newBegin < m_intervalBegin) {
+    else {
+        SMILTime newBegin = findInstanceTime(Begin, eventTime, true);
+        if (newBegin.isFinite() && (m_intervalEnd <= eventTime || newBegin < m_intervalBegin)) {
             // Begin time changed, re-resolve the interval.
             SMILTime oldBegin = m_intervalBegin;
-            m_intervalBegin = elapsed;
+            m_intervalEnd = eventTime;
             resolveInterval(false, m_intervalBegin, m_intervalEnd);  
             ASSERT(!m_intervalBegin.isUnresolved());
             if (m_intervalBegin != oldBegin)
                 notifyDependentsIntervalChanged(ExistingInterval);
         }
     }
-    m_nextProgressTime = elapsed;
+    m_nextProgressTime = elapsed();
     reschedule();
 }
 
-void SVGSMILElement::endListChanged()
+void SVGSMILElement::endListChanged(SMILTime)
 {
     SMILTime elapsed = this->elapsed();
     if (m_isWaitingForFirstInterval)
@@ -834,6 +856,11 @@ float SVGSMILElement::calculateAnimationPercentAndRepeat(SMILTime elapsed, unsig
         repeat = static_cast<unsigned>(repeatingDuration.value() / simpleDuration.value());
         if (fmod(repeatingDuration.value(), !simpleDuration.value()))
             repeat--;
+
+        SMILTime simpleEndTime = fmod(m_intervalEnd.value() - m_intervalBegin.value(), simpleDuration.value());
+        if (simpleEndTime.value())
+            return narrowPrecisionToFloat(simpleEndTime.value() / simpleDuration.value());
+
         return 1.f;
     }
     repeat = static_cast<unsigned>(activeTime.value() / simpleDuration.value());
@@ -847,7 +874,6 @@ SMILTime SVGSMILElement::calculateNextProgressTime(SMILTime elapsed) const
         // If duration is indefinite the value does not actually change over time. Same is true for <set>.
         SMILTime simpleDuration = this->simpleDuration();
         if (simpleDuration.isIndefinite() || hasTagName(SVGNames::setTag)) {
-            SMILTime repeatCount = this->repeatCount();
             SMILTime repeatingDurationEnd = m_intervalBegin + repeatingDuration();
             // We are supposed to do freeze semantics when repeating ends, even if the element is still active. 
             // Take care that we get a timer callback at that point.
@@ -961,9 +987,9 @@ void SVGSMILElement::createInstanceTimesFromSyncbase(SVGSMILElement* syncbase, N
                 time = syncbase->m_intervalEnd + condition.m_offset;
             ASSERT(time.isFinite());
             if (condition.m_beginOrEnd == Begin)
-                addBeginTime(time);
+                addBeginTime(elapsed(), time);
             else
-                addEndTime(time);
+                addEndTime(elapsed(), time);
         }
     }
 }
@@ -982,15 +1008,17 @@ void SVGSMILElement::removeTimeDependent(SVGSMILElement* animation)
     
 void SVGSMILElement::handleConditionEvent(Event*, Condition* condition)
 {
+    SMILTime elapsed = this->elapsed();
     if (condition->m_beginOrEnd == Begin)
-        addBeginTime(elapsed() + condition->m_offset);
+        addBeginTime(elapsed, elapsed + condition->m_offset);
     else
-        addEndTime(elapsed() + condition->m_offset);
+        addEndTime(elapsed, elapsed + condition->m_offset);
 }
 
 void SVGSMILElement::beginByLinkActivation()
 {
-    addBeginTime(elapsed());
+    SMILTime elapsed = this->elapsed();
+    addBeginTime(elapsed, elapsed);
 }
     
 }
