@@ -58,7 +58,7 @@
 
 namespace WebCore {
 
-class SharedWorkerProxy : public ThreadSafeShared<SharedWorkerProxy>, public WorkerLoaderProxy, public WorkerReportingProxy {
+class SharedWorkerProxy : public ThreadSafeRefCounted<SharedWorkerProxy>, public WorkerLoaderProxy, public WorkerReportingProxy {
 public:
     static PassRefPtr<SharedWorkerProxy> create(const String& name, const KURL& url, PassRefPtr<SecurityOrigin> origin) { return adoptRef(new SharedWorkerProxy(name, url, origin)); }
 
@@ -81,6 +81,9 @@ public:
     // WorkerReportingProxy
     virtual void postExceptionToWorkerObject(const String& errorMessage, int lineNumber, const String& sourceURL);
     virtual void postConsoleMessageToWorkerObject(MessageSource, MessageType, MessageLevel, const String& message, int lineNumber, const String& sourceURL);
+#if ENABLE(INSPECTOR)
+    virtual void postMessageToPageInspector(const String&);
+#endif
     virtual void workerContextClosed();
     virtual void workerContextDestroyed();
 
@@ -178,6 +181,12 @@ void SharedWorkerProxy::postConsoleMessageToWorkerObject(MessageSource source, M
         (*iter)->postTask(createCallbackTask(&postConsoleMessageTask, source, type, level, message, lineNumber, sourceURL));
 }
 
+#if ENABLE(INSPECTOR)
+void SharedWorkerProxy::postMessageToPageInspector(const String&)
+{
+}
+#endif
+
 void SharedWorkerProxy::workerContextClosed()
 {
     if (isClosing())
@@ -225,7 +234,7 @@ class SharedWorkerConnectTask : public ScriptExecutionContext::Task {
 public:
     static PassOwnPtr<SharedWorkerConnectTask> create(PassOwnPtr<MessagePortChannel> channel)
     {
-        return new SharedWorkerConnectTask(channel);
+        return adoptPtr(new SharedWorkerConnectTask(channel));
     }
 
 private:
@@ -256,13 +265,14 @@ public:
     void load(const KURL&);
 
 private:
-    // WorkerScriptLoaderClient callback
+    // WorkerScriptLoaderClient callbacks
+    virtual void didReceiveResponse(unsigned long identifier, const ResourceResponse&);
     virtual void notifyFinished();
 
     RefPtr<SharedWorker> m_worker;
     OwnPtr<MessagePortChannel> m_port;
     RefPtr<SharedWorkerProxy> m_proxy;
-    OwnPtr<WorkerScriptLoader> m_scriptLoader;
+    RefPtr<WorkerScriptLoader> m_scriptLoader;
 };
 
 SharedWorkerScriptLoader::SharedWorkerScriptLoader(PassRefPtr<SharedWorker> worker, PassOwnPtr<MessagePortChannel> port, PassRefPtr<SharedWorkerProxy> proxy)
@@ -274,13 +284,21 @@ SharedWorkerScriptLoader::SharedWorkerScriptLoader(PassRefPtr<SharedWorker> work
 
 void SharedWorkerScriptLoader::load(const KURL& url)
 {
-    // Mark this object as active for the duration of the load.
-    m_scriptLoader = new WorkerScriptLoader(ResourceRequestBase::TargetIsSharedWorker);
-    m_scriptLoader->loadAsynchronously(m_worker->scriptExecutionContext(), url, DenyCrossOriginRequests, this);
-
     // Stay alive (and keep the SharedWorker and JS wrapper alive) until the load finishes.
     this->ref();
     m_worker->setPendingActivity(m_worker.get());
+
+    // Mark this object as active for the duration of the load.
+    m_scriptLoader = WorkerScriptLoader::create();
+#if PLATFORM(CHROMIUM)
+    m_scriptLoader->setTargetType(ResourceRequest::TargetIsSharedWorker);
+#endif
+    m_scriptLoader->loadAsynchronously(m_worker->scriptExecutionContext(), url, DenyCrossOriginRequests, this);
+}
+
+void SharedWorkerScriptLoader::didReceiveResponse(unsigned long identifier, const ResourceResponse&)
+{
+    InspectorInstrumentation::didReceiveScriptResponse(m_worker->scriptExecutionContext(), identifier);
 }
 
 void SharedWorkerScriptLoader::notifyFinished()
