@@ -29,10 +29,12 @@
 #include "DumpRenderTreeSupportGtk.h"
 #include "Element.h"
 #include "FileChooser.h"
+#include "FileIconLoader.h"
 #include "FileSystem.h"
 #include "FloatRect.h"
 #include "FrameLoadRequest.h"
 #include "FrameView.h"
+#include "GtkUtilities.h"
 #include "GtkVersioning.h"
 #include "HTMLNames.h"
 #include "HitTestResult.h"
@@ -61,7 +63,7 @@
 #include <gtk/gtk.h>
 #include <wtf/text/CString.h>
 
-#if ENABLE(DATABASE)
+#if ENABLE(SQL_DATABASE)
 #include "DatabaseTracker.h"
 #endif
 
@@ -135,12 +137,6 @@ FloatRect ChromeClient::pageRect()
     return IntRect(allocation.x, allocation.y, allocation.width, allocation.height);
 }
 
-float ChromeClient::scaleFactor()
-{
-    // Not implementable
-    return 1.0;
-}
-
 void ChromeClient::focus()
 {
     gtk_widget_grab_focus(GTK_WIDGET(m_webView));
@@ -374,6 +370,13 @@ IntRect ChromeClient::windowResizerRect() const
     return IntRect();
 }
 
+#if ENABLE(REGISTER_PROTOCOL_HANDLER) 
+void ChromeClient::registerProtocolHandler(const String& scheme, const String& baseURL, const String& url, const String& title) 
+{ 
+    notImplemented(); 
+} 
+#endif 
+
 void ChromeClient::invalidateWindow(const IntRect&, bool immediate)
 {
     // If we've invalidated regions for scrolling, force GDK to process those invalidations
@@ -402,6 +405,7 @@ void ChromeClient::invalidateContentsAndWindow(const IntRect& updateRect, bool i
 void ChromeClient::invalidateContentsForSlowScroll(const IntRect& updateRect, bool immediate)
 {
     invalidateContentsAndWindow(updateRect, immediate);
+    m_adjustmentWatcher.updateAdjustmentsFromScrollbarsLater();
 }
 
 void ChromeClient::scroll(const IntSize& delta, const IntRect& rectToScroll, const IntRect& clipRect)
@@ -453,37 +457,16 @@ void ChromeClient::scroll(const IntSize& delta, const IntRect& rectToScroll, con
     m_adjustmentWatcher.updateAdjustmentsFromScrollbarsLater();
 }
 
-// FIXME: this does not take into account the WM decorations
-static IntPoint widgetScreenPosition(GtkWidget* widget)
-{
-    GtkWidget* window = gtk_widget_get_toplevel(widget);
-    int widgetX = 0, widgetY = 0;
-
-    gtk_widget_translate_coordinates(widget, window, 0, 0, &widgetX, &widgetY);
-
-    IntPoint result(widgetX, widgetY);
-    int originX, originY;
-    gdk_window_get_origin(gtk_widget_get_window(window), &originX, &originY);
-    result.move(originX, originY);
-
-    return result;
-}
-
 IntRect ChromeClient::windowToScreen(const IntRect& rect) const
 {
-    IntRect result(rect);
-    IntPoint screenPosition = widgetScreenPosition(GTK_WIDGET(m_webView));
-    result.move(screenPosition.x(), screenPosition.y());
-
-    return result;
+    return IntRect(convertWidgetPointToScreenPoint(GTK_WIDGET(m_webView), rect.location()), rect.size());
 }
 
 IntPoint ChromeClient::screenToWindow(const IntPoint& point) const
 {
+    IntPoint widgetPositionOnScreen = convertWidgetPointToScreenPoint(GTK_WIDGET(m_webView), IntPoint());
     IntPoint result(point);
-    IntPoint screenPosition = widgetScreenPosition(GTK_WIDGET(m_webView));
-    result.move(-screenPosition.x(), -screenPosition.y());
-
+    result.move(-widgetPositionOnScreen.x(), -widgetPositionOnScreen.y());
     return result;
 }
 
@@ -559,7 +542,7 @@ void ChromeClient::mouseDidMoveOverElement(const HitTestResult& hit, unsigned mo
         if (!url.isEmpty() && url != m_hoveredLinkURL) {
             TextDirection dir;
             CString titleString = hit.title(dir).utf8();
-            CString urlString = url.prettyURL().utf8();
+            CString urlString = url.string().utf8();
             g_signal_emit_by_name(m_webView, "hovering-over-link", titleString.data(), urlString.data());
             m_hoveredLinkURL = url;
         }
@@ -593,7 +576,7 @@ void ChromeClient::print(Frame* frame)
     webkit_web_frame_print(webFrame);
 }
 
-#if ENABLE(DATABASE)
+#if ENABLE(SQL_DATABASE)
 void ChromeClient::exceededDatabaseQuota(Frame* frame, const String& databaseName)
 {
     guint64 defaultQuota = webkit_get_default_web_database_quota();
@@ -613,7 +596,7 @@ void ChromeClient::reachedMaxAppCacheSize(int64_t spaceNeeded)
     notImplemented();
 }
 
-void ChromeClient::reachedApplicationCacheOriginQuota(SecurityOrigin*)
+void ChromeClient::reachedApplicationCacheOriginQuota(SecurityOrigin*, int64_t)
 {
     notImplemented();
 }
@@ -630,7 +613,7 @@ void ChromeClient::runOpenPanel(Frame*, PassRefPtr<FileChooser> prpFileChooser)
                                                     GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
                                                     NULL);
 
-    gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(dialog), chooser->allowsMultipleFiles());
+    gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(dialog), chooser->settings().allowsMultipleFiles);
 
     if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
         if (gtk_file_chooser_get_select_multiple(GTK_FILE_CHOOSER(dialog))) {
@@ -654,9 +637,9 @@ void ChromeClient::runOpenPanel(Frame*, PassRefPtr<FileChooser> prpFileChooser)
     gtk_widget_destroy(dialog);
 }
 
-void ChromeClient::chooseIconForFiles(const Vector<WTF::String>& filenames, WebCore::FileChooser* chooser)
+void ChromeClient::loadIconForFiles(const Vector<WTF::String>& filenames, WebCore::FileIconLoader* loader)
 {
-    chooser->iconLoaded(Icon::createIconForFiles(filenames));
+    loader->notifyFinished(Icon::createIconForFiles(filenames));
 }
 
 void ChromeClient::dispatchViewportDataDidChange(const ViewportArguments& arguments) const
@@ -678,6 +661,11 @@ void ChromeClient::setCursor(const Cursor& cursor)
         gdk_window_set_cursor(window, newCursor);
 }
 
+void ChromeClient::setCursorHiddenUntilMouseMoves(bool)
+{
+    notImplemented();
+}
+
 void ChromeClient::requestGeolocationPermissionForFrame(Frame* frame, Geolocation* geolocation)
 {
     WebKitWebFrame* webFrame = kit(frame);
@@ -696,12 +684,12 @@ void ChromeClient::cancelGeolocationPermissionRequestForFrame(WebCore::Frame* fr
 
 bool ChromeClient::selectItemWritingDirectionIsNatural()
 {
-    return true;
+    return false;
 }
 
 bool ChromeClient::selectItemAlignmentFollowsMenuWritingDirection()
 {
-    return false;
+    return true;
 }
 
 PassRefPtr<WebCore::PopupMenu> ChromeClient::createPopupMenu(WebCore::PopupMenuClient* client) const
@@ -733,8 +721,11 @@ void ChromeClient::exitFullscreenForNode(Node* node)
 #endif
 
 #if ENABLE(FULLSCREEN_API)
-bool ChromeClient::supportsFullScreenForElement(const WebCore::Element* element)
+bool ChromeClient::supportsFullScreenForElement(const WebCore::Element* element, bool withKeyboard)
 {
+    if (withKeyboard)
+        return false;
+
     return true;
 }
 
@@ -751,4 +742,26 @@ void ChromeClient::exitFullScreenForElement(WebCore::Element* element)
 }
 #endif
 
+#if USE(ACCELERATED_COMPOSITING)
+void ChromeClient::attachRootGraphicsLayer(Frame* frame, GraphicsLayer* rootLayer)
+{
+    notImplemented();
+}
+
+void ChromeClient::setNeedsOneShotDrawingSynchronization()
+{
+    notImplemented();
+}
+
+void ChromeClient::scheduleCompositingLayerSync()
+{
+    notImplemented();
+}
+
+ChromeClient::CompositingTriggerFlags ChromeClient::allowedCompositingTriggers() const
+{
+    return 0;
+}
+#endif
+
 }
