@@ -38,7 +38,7 @@ namespace WebKit {
     class ChromeClient : public WebCore::ChromeClient {
     public:
         ChromeClient(WebKitWebView*);
-        WebKitWebView* webView() const { return m_webView; }
+        virtual void* webView() const { return m_webView; }
         GtkAdjustmentWatcher* adjustmentWatcher() { return &m_adjustmentWatcher; }
 
         virtual void chromeDestroyed();
@@ -48,8 +48,6 @@ namespace WebKit {
 
         virtual WebCore::FloatRect pageRect();
 
-        virtual float scaleFactor();
-
         virtual void focus();
         virtual void unfocus();
 
@@ -96,7 +94,9 @@ namespace WebKit {
         virtual WebCore::KeyboardUIMode keyboardUIMode();
 
         virtual WebCore::IntRect windowResizerRect() const;
-
+#if ENABLE(REGISTER_PROTOCOL_HANDLER) 
+        virtual void registerProtocolHandler(const WTF::String&, const WTF::String&, const WTF::String&, const WTF::String&); 
+#endif 
         virtual void invalidateWindow(const WebCore::IntRect&, bool);
         virtual void invalidateContentsAndWindow(const WebCore::IntRect&, bool);
         virtual void invalidateContentsForSlowScroll(const WebCore::IntRect&, bool);
@@ -115,24 +115,25 @@ namespace WebKit {
         virtual void dispatchViewportDataDidChange(const WebCore::ViewportArguments& arguments) const;
 
         virtual void print(WebCore::Frame*);
-#if ENABLE(DATABASE)
+#if ENABLE(SQL_DATABASE)
         virtual void exceededDatabaseQuota(WebCore::Frame*, const WTF::String&);
 #endif
 #if ENABLE(OFFLINE_WEB_APPLICATIONS)
         virtual void reachedMaxAppCacheSize(int64_t spaceNeeded);
-        virtual void reachedApplicationCacheOriginQuota(WebCore::SecurityOrigin*);
+        virtual void reachedApplicationCacheOriginQuota(WebCore::SecurityOrigin*, int64_t totalSpaceNeeded);
 #endif
 #if ENABLE(CONTEXT_MENUS)
         virtual void showContextMenu() { }
 #endif
         virtual void runOpenPanel(WebCore::Frame*, PassRefPtr<WebCore::FileChooser>);
-        virtual void chooseIconForFiles(const Vector<WTF::String>&, WebCore::FileChooser*);
+        virtual void loadIconForFiles(const Vector<WTF::String>&, WebCore::FileIconLoader*);
 
         virtual void formStateDidChange(const WebCore::Node*) { }
 
         virtual void setCursor(const WebCore::Cursor&);
+        virtual void setCursorHiddenUntilMouseMoves(bool);
 
-        virtual void scrollRectIntoView(const WebCore::IntRect&, const WebCore::ScrollView*) const {}
+        virtual void scrollRectIntoView(const WebCore::IntRect&) const { }
         virtual void requestGeolocationPermissionForFrame(WebCore::Frame*, WebCore::Geolocation*);
         virtual void cancelGeolocationPermissionRequestForFrame(WebCore::Frame*, WebCore::Geolocation*);
 
@@ -147,11 +148,21 @@ namespace WebKit {
 #endif
 
 #if ENABLE(FULLSCREEN_API)
-        virtual bool supportsFullScreenForElement(const WebCore::Element*);
+        virtual bool supportsFullScreenForElement(const WebCore::Element*, bool withKeyboard);
         virtual void enterFullScreenForElement(WebCore::Element*);
         virtual void exitFullScreenForElement(WebCore::Element*);
 #endif
 
+        virtual bool shouldRubberBandInDirection(WebCore::ScrollDirection) const { return true; }
+        virtual void numWheelEventHandlersChanged(unsigned) { }
+
+#if USE(ACCELERATED_COMPOSITING) 
+        virtual void attachRootGraphicsLayer(WebCore::Frame*, WebCore::GraphicsLayer*);
+        virtual void setNeedsOneShotDrawingSynchronization();
+        virtual void scheduleCompositingLayerSync();
+        virtual CompositingTriggerFlags allowedCompositingTriggers() const;
+#endif 
+
     private:
         WebKitWebView* m_webView;
         GtkAdjustmentWatcher m_adjustmentWatcher;
