@@ -28,6 +28,8 @@
 #include "AnimationController.h"
 #include "DOMWrapperWorld.h"
 #include "Document.h"
+#include "EditorClientGtk.h"
+#include "Element.h"
 #include "FocusController.h"
 #include "FrameLoaderClientGtk.h"
 #include "FrameTree.h"
@@ -35,12 +37,12 @@
 #include "GCController.h"
 #include "GraphicsContext.h"
 #include "HTMLInputElement.h"
-#include "InputElement.h"
 #include "JSDOMWindow.h"
 #include "JSDocument.h"
 #include "JSElement.h"
 #include "JSLock.h"
 #include "JSNodeList.h"
+#include "JSRange.h"
 #include "JSValue.h"
 #include "NodeList.h"
 #include "PageGroup.h"
@@ -52,6 +54,7 @@
 #include "SecurityOrigin.h"
 #include "Settings.h"
 #include "TextIterator.h"
+#include "WebKitDOMRangePrivate.h"
 #include "WorkerThread.h"
 #include "webkitglobalsprivate.h"
 #include "webkitwebframe.h"
@@ -126,6 +129,20 @@ JSValueRef DumpRenderTreeSupportGtk::nodesFromRect(JSContextRef context, JSValue
     return toRef(exec, toJS(exec, jsDocument->globalObject(), nodes.get()));
 }
 
+WebKitDOMRange* DumpRenderTreeSupportGtk::jsValueToDOMRange(JSContextRef context, JSValueRef value)
+{
+    if (!value)
+        return 0;
+
+    JSLock lock(SilenceAssertionsOnly);
+    ExecState* exec = toJS(context);
+
+    Range* range = toRange(toJS(exec, value));
+    if (!range)
+        return 0;
+    return kit(range);
+}
+
 /**
  * getFrameChildren:
  * @frame: a #WebKitWebFrame
@@ -166,11 +183,12 @@ CString DumpRenderTreeSupportGtk::getInnerText(WebKitWebFrame* frame)
         return CString("");
 
     FrameView* view = coreFrame->view();
-
     if (view && view->layoutPending())
         view->layout();
 
     Element* documentElement = coreFrame->document()->documentElement();
+    if (!documentElement)
+        return CString("");
     return documentElement->innerText().utf8();
 }
 
@@ -337,7 +355,7 @@ void DumpRenderTreeSupportGtk::addUserStyleSheet(WebKitWebFrame* frame, const ch
 
     WebKitWebView* webView = getViewFromFrame(frame);
     Page* page = core(webView);
-    page->group().addUserStyleSheetToWorld(mainThreadNormalWorld(), sourceCode, KURL(), 0, 0, allFrames ? InjectInAllFrames : InjectInTopFrameOnly); 
+    page->group().addUserStyleSheetToWorld(mainThreadNormalWorld(), sourceCode, KURL(), nullptr, nullptr, allFrames ? InjectInAllFrames : InjectInTopFrameOnly); 
 }
 
 /**
@@ -403,7 +421,7 @@ unsigned int DumpRenderTreeSupportGtk::numberOfActiveAnimations(WebKitWebFrame*
     if (!coreFrame)
         return 0;
 
-    return coreFrame->animation()->numberOfActiveAnimations();
+    return coreFrame->animation()->numberOfActiveAnimations(coreFrame->document());
 }
 
 void DumpRenderTreeSupportGtk::suspendAnimations(WebKitWebFrame* frame)
@@ -483,7 +501,7 @@ bool DumpRenderTreeSupportGtk::isCommandEnabled(WebKitWebView* webView, const gc
     return core(webView)->focusController()->focusedOrMainFrame()->editor()->command(name).isEnabled();
 }
 
-void DumpRenderTreeSupportGtk::setComposition(WebKitWebView* webView, const char* text, int start, int end)
+void DumpRenderTreeSupportGtk::setComposition(WebKitWebView* webView, const char* text, int start, int length)
 {
     g_return_if_fail(WEBKIT_IS_WEB_VIEW(webView));
     g_return_if_fail(text);
@@ -493,15 +511,45 @@ void DumpRenderTreeSupportGtk::setComposition(WebKitWebView* webView, const char
         return;
 
     Editor* editor = frame->editor();
-    if (!editor)
-        return;
-    if (!editor->canEdit() && !editor->hasComposition())
+    if (!editor || (!editor->canEdit() && !editor->hasComposition()))
         return;
 
     String compositionString = String::fromUTF8(text);
     Vector<CompositionUnderline> underlines;
     underlines.append(CompositionUnderline(0, compositionString.length(), Color(0, 0, 0), false));
-    editor->setComposition(compositionString, underlines, start, end);
+    editor->setComposition(compositionString, underlines, start, start + length);
+}
+
+bool DumpRenderTreeSupportGtk::hasComposition(WebKitWebView* webView)
+{
+    g_return_val_if_fail(WEBKIT_IS_WEB_VIEW(webView), false);
+    Frame* frame = core(webView)->focusController()->focusedOrMainFrame();
+    if (!frame)
+        return false;
+    Editor* editor = frame->editor();
+    if (!editor)
+        return false;
+
+    return editor->hasComposition();
+}
+
+bool DumpRenderTreeSupportGtk::compositionRange(WebKitWebView* webView, int* start, int* length)
+{
+    g_return_val_if_fail(start && length, false);
+    *start = *length = 0;
+
+    g_return_val_if_fail(WEBKIT_IS_WEB_VIEW(webView), false);
+    Frame* frame = core(webView)->focusController()->focusedOrMainFrame();
+    if (!frame)
+        return false;
+
+    Editor* editor = frame->editor();
+    if (!editor || !editor->hasComposition())
+        return false;
+
+    *start = editor->compositionStart();
+    *length = editor->compositionEnd() - *start;
+    return true;
 }
 
 void DumpRenderTreeSupportGtk::confirmComposition(WebKitWebView* webView, const char* text)
@@ -513,19 +561,21 @@ void DumpRenderTreeSupportGtk::confirmComposition(WebKitWebView* webView, const
         return;
 
     Editor* editor = frame->editor();
-    if (!editor || (!editor->hasComposition() && !text))
+    if (!editor)
         return;
 
-    if (editor->hasComposition()) {
-        if (text)
-            editor->confirmComposition(String::fromUTF8(text));
-        else
-            editor->confirmComposition();
-    } else
+    if (!editor->hasComposition()) {
         editor->insertText(String::fromUTF8(text), 0);
+        return;
+    }
+    if (text) {
+        editor->confirmComposition(String::fromUTF8(text));
+        return;
+    }
+    editor->confirmComposition();
 }
 
-bool DumpRenderTreeSupportGtk::firstRectForCharacterRange(WebKitWebView* webView, int location, int length, GdkRectangle* rect)
+bool DumpRenderTreeSupportGtk::firstRectForCharacterRange(WebKitWebView* webView, int location, int length, cairo_rectangle_int_t* rect)
 {
     g_return_val_if_fail(WEBKIT_IS_WEB_VIEW(webView), false);
     g_return_val_if_fail(rect, false);
@@ -548,21 +598,21 @@ bool DumpRenderTreeSupportGtk::firstRectForCharacterRange(WebKitWebView* webView
         return false;
 
     *rect = editor->firstRectForRange(range.get());
-
     return true;
 }
 
-bool DumpRenderTreeSupportGtk::selectedRange(WebKitWebView* webView, int* start, int* end)
+bool DumpRenderTreeSupportGtk::selectedRange(WebKitWebView* webView, int* start, int* length)
 {
     g_return_val_if_fail(WEBKIT_IS_WEB_VIEW(webView), false);
-    g_return_val_if_fail(start, false);
-    g_return_val_if_fail(end, false);
+    g_return_val_if_fail(start && length, false);
 
     Frame* frame = core(webView)->focusController()->focusedOrMainFrame();
     if (!frame)
         return false;
 
     RefPtr<Range> range = frame->selection()->toNormalizedRange().get();
+    if (!range)
+        return false;
 
     Element* selectionRoot = frame->selection()->rootEditableElement();
     Element* scope = selectionRoot ? selectionRoot : frame->document()->documentElement();
@@ -574,11 +624,20 @@ bool DumpRenderTreeSupportGtk::selectedRange(WebKitWebView* webView, int* start,
     ExceptionCode ec;
     testRange->setEnd(range->endContainer(), range->endOffset(), ec);
     ASSERT(testRange->startContainer() == scope);
-    *end = TextIterator::rangeLength(testRange.get());
+    *length = TextIterator::rangeLength(testRange.get());
 
     return true;
 }
 
+void DumpRenderTreeSupportGtk::setSmartInsertDeleteEnabled(WebKitWebView* webView, bool enabled)
+{
+    g_return_if_fail(WEBKIT_IS_WEB_VIEW(webView));
+    g_return_if_fail(webView);
+
+    WebKit::EditorClient* client = static_cast<WebKit::EditorClient*>(core(webView)->editorClient());
+    client->setSmartInsertDeleteEnabled(enabled);
+}
+
 void DumpRenderTreeSupportGtk::whiteListAccessFromOrigin(const gchar* sourceOrigin, const gchar* destinationProtocol, const gchar* destinationHost, bool allowDestinationSubdomains)
 {
     SecurityOrigin::addOriginAccessWhitelistEntry(*SecurityOrigin::createFromString(sourceOrigin), destinationProtocol, destinationHost, allowDestinationSubdomains);
@@ -649,7 +708,7 @@ bool DumpRenderTreeSupportGtk::webkitWebFrameSelectionHasSpellingMarker(WebKitWe
 {
     g_return_val_if_fail(WEBKIT_IS_WEB_FRAME(frame), FALSE);
 
-    return core(frame)->editor()->selectionStartHasSpellingMarkerFor(from, length);
+    return core(frame)->editor()->selectionStartHasMarkerFor(DocumentMarker::Spelling, from, length);
 }
 
 bool DumpRenderTreeSupportGtk::findString(WebKitWebView* webView, const gchar* targetString, WebKitFindOptions findOptions)
@@ -667,17 +726,42 @@ void DumpRenderTreeSupportGtk::setMinimumTimerInterval(WebKitWebView* webView, d
     core(webView)->settings()->setMinDOMTimerInterval(interval);
 }
 
+static void modifyAccessibilityValue(AtkObject* axObject, bool increment)
+{
+    if (!axObject || !WEBKIT_IS_ACCESSIBLE(axObject))
+        return;
+
+    AccessibilityObject* coreObject = webkit_accessible_get_accessibility_object(WEBKIT_ACCESSIBLE(axObject));
+    if (!coreObject)
+        return;
+
+    if (increment)
+        coreObject->increment();
+    else
+        coreObject->decrement();
+}
+
+void DumpRenderTreeSupportGtk::incrementAccessibilityValue(AtkObject* axObject)
+{
+    modifyAccessibilityValue(axObject, true);
+}
+
+void DumpRenderTreeSupportGtk::decrementAccessibilityValue(AtkObject* axObject)
+{
+    modifyAccessibilityValue(axObject, false);
+}
+
 void DumpRenderTreeSupportGtk::setAutofilled(JSContextRef context, JSValueRef nodeObject, bool autofilled)
 {
     JSC::ExecState* exec = toJS(context);
     Element* element = toElement(toJS(exec, nodeObject));
     if (!element)
         return;
-    InputElement* inputElement = element->toInputElement();
+    HTMLInputElement* inputElement = element->toInputElement();
     if (!inputElement)
         return;
 
-    static_cast<HTMLInputElement*>(inputElement)->setAutofilled(autofilled);
+    inputElement->setAutofilled(autofilled);
 }
 
 void DumpRenderTreeSupportGtk::setValueForUser(JSContextRef context, JSValueRef nodeObject, JSStringRef value)
@@ -686,7 +770,7 @@ void DumpRenderTreeSupportGtk::setValueForUser(JSContextRef context, JSValueRef
     Element* element = toElement(toJS(exec, nodeObject));
     if (!element)
         return;
-    InputElement* inputElement = element->toInputElement();
+    HTMLInputElement* inputElement = element->toInputElement();
     if (!inputElement)
         return;
 
@@ -695,3 +779,29 @@ void DumpRenderTreeSupportGtk::setValueForUser(JSContextRef context, JSValueRef
     JSStringGetUTF8CString(value, valueBuffer.get(), bufferSize);
     inputElement->setValueForUser(String::fromUTF8(valueBuffer.get()));
 }
+
+void DumpRenderTreeSupportGtk::rectangleForSelection(WebKitWebFrame* frame, cairo_rectangle_int_t* rectangle)
+{
+    Frame* coreFrame = core(frame);
+    if (!coreFrame)
+        return;
+
+    IntRect bounds = enclosingIntRect(coreFrame->selection()->bounds());
+    rectangle->x = bounds.x();
+    rectangle->y = bounds.y();
+    rectangle->width = bounds.width();
+    rectangle->height = bounds.height();
+}
+
+bool DumpRenderTreeSupportGtk::shouldClose(WebKitWebFrame* frame)
+{
+    Frame* coreFrame = core(frame);
+    if (!coreFrame)
+        return true;
+    return coreFrame->loader()->shouldClose();
+}
+
+void DumpRenderTreeSupportGtk::scalePageBy(WebKitWebView* webView, float scaleFactor, float x, float y)
+{
+    core(webView)->setPageScaleFactor(scaleFactor, IntPoint(x, y));
+}
