@@ -28,13 +28,6 @@
 #define WEBKIT_PARAM_READABLE ((GParamFlags)(G_PARAM_READABLE|G_PARAM_STATIC_NAME|G_PARAM_STATIC_NICK|G_PARAM_STATIC_BLURB))
 #define WEBKIT_PARAM_READWRITE ((GParamFlags)(G_PARAM_READWRITE|G_PARAM_STATIC_NAME|G_PARAM_STATIC_NICK|G_PARAM_STATIC_BLURB))
 
-namespace WebKit {
-
-class PasteboardHelperGtk;
-PasteboardHelperGtk* pasteboardHelperInstance();
-
-}
-
 extern "C" {
 
 void webkitInit();
