@@ -31,10 +31,12 @@
 #include "AccessibilityObjectWrapperAtk.h"
 #include "AnimationController.h"
 #include "DOMObjectCache.h"
+#include "DocumentFragment.h"
 #include "DocumentLoader.h"
 #include "DocumentLoaderGtk.h"
 #include "FrameLoader.h"
 #include "FrameLoaderClientGtk.h"
+#include "FrameSelection.h"
 #include "FrameTree.h"
 #include "FrameView.h"
 #include "GCController.h"
@@ -44,12 +46,17 @@
 #include "JSDOMBinding.h"
 #include "JSDOMWindow.h"
 #include "JSElement.h"
+#include "PlatformContextCairo.h"
 #include "PrintContext.h"
 #include "RenderListItem.h"
 #include "RenderTreeAsText.h"
 #include "RenderView.h"
+#include "ReplaceSelectionCommand.h"
 #include "ScriptController.h"
 #include "SubstituteData.h"
+#include "TextIterator.h"
+#include "markup.h"
+#include "webkit/WebKitDOMRangePrivate.h"
 #include "webkitenumtypes.h"
 #include "webkitglobalsprivate.h"
 #include "webkitmarshal.h"
@@ -82,7 +89,7 @@
  * <informalexample><programlisting>
  * /<!-- -->* Get the frame from the #WebKitWebView *<!-- -->/
  * WebKitWebFrame *frame = webkit_web_view_get_main_frame (WEBKIT_WEB_VIEW(my_view));
- * g_print("The URI of this frame is '%s'", webkit_web_frame_get_uri (frame));
+ * g_print ("The URI of this frame is '%s'", webkit_web_frame_get_uri (frame));
  * </programlisting></informalexample>
  */
 
@@ -425,7 +432,7 @@ WebKitWebFrame* webkit_web_frame_new(WebKitWebView* webView)
  *
  * Return value: the title of @frame
  */
-G_CONST_RETURN gchar* webkit_web_frame_get_title(WebKitWebFrame* frame)
+const gchar* webkit_web_frame_get_title(WebKitWebFrame* frame)
 {
     g_return_val_if_fail(WEBKIT_IS_WEB_FRAME(frame), 0);
 
@@ -441,7 +448,7 @@ G_CONST_RETURN gchar* webkit_web_frame_get_title(WebKitWebFrame* frame)
  *
  * Return value: the URI of @frame
  */
-G_CONST_RETURN gchar* webkit_web_frame_get_uri(WebKitWebFrame* frame)
+const gchar* webkit_web_frame_get_uri(WebKitWebFrame* frame)
 {
     g_return_val_if_fail(WEBKIT_IS_WEB_FRAME(frame), 0);
 
@@ -478,7 +485,7 @@ WebKitWebView* webkit_web_frame_get_web_view(WebKitWebFrame* frame)
  * the #WebKitWebFrame is invalid or an empty string if it is not backed
  * by a live WebCore frame.
  */
-G_CONST_RETURN gchar* webkit_web_frame_get_name(WebKitWebFrame* frame)
+const gchar* webkit_web_frame_get_name(WebKitWebFrame* frame)
 {
     g_return_val_if_fail(WEBKIT_IS_WEB_FRAME(frame), 0);
     Frame* coreFrame = core(frame);
@@ -765,17 +772,17 @@ static void begin_print_callback(GtkPrintOperation* op, GtkPrintContext* context
     gtk_print_operation_set_n_pages(op, printContext->pageCount());
 }
 
-static void draw_page_callback(GtkPrintOperation* op, GtkPrintContext* context, gint page_nr, gpointer user_data)
+static void draw_page_callback(GtkPrintOperation*, GtkPrintContext* gtkPrintContext, gint pageNumber, PrintContext* corePrintContext)
 {
-    PrintContext* printContext = reinterpret_cast<PrintContext*>(user_data);
-
-    if (page_nr >= static_cast<gint>(printContext->pageCount()))
+    if (pageNumber >= static_cast<gint>(corePrintContext->pageCount()))
         return;
 
-    cairo_t* cr = gtk_print_context_get_cairo_context(context);
-    GraphicsContext ctx(cr);
-    float width = gtk_print_context_get_width(context);
-    printContext->spoolPage(ctx, page_nr, width);
+    cairo_t* cr = gtk_print_context_get_cairo_context(gtkPrintContext);
+    float pageWidth = gtk_print_context_get_width(gtkPrintContext);
+
+    PlatformContextCairo platformContext(cr);
+    GraphicsContext graphicsContext(&platformContext);
+    corePrintContext->spoolPage(graphicsContext, pageNumber, pageWidth);
 }
 
 static void end_print_callback(GtkPrintOperation* op, GtkPrintContext* context, gpointer user_data)
@@ -973,6 +980,51 @@ WebKitNetworkResponse* webkit_web_frame_get_network_response(WebKitWebFrame* fra
     return kitNew(loader->response());
 }
 
+/**
+ * webkit_web_frame_replace_selection:
+ * @frame: a #WebKitWeFrame
+ * @text: the text to insert in place of the current selection
+ *
+ * Replaces the current selection in @frame, if any, with @text.
+ *
+ * Since: 1.5.1
+ **/
+void webkit_web_frame_replace_selection(WebKitWebFrame* frame, const char* text)
+{
+    Frame* coreFrame = core(frame);
+    RefPtr<DocumentFragment> fragment = createFragmentFromText(
+        coreFrame->selection()->toNormalizedRange().get(), text);
+    applyCommand(ReplaceSelectionCommand::create(coreFrame->document(), fragment.get(),
+                                                 ReplaceSelectionCommand::SmartReplace | ReplaceSelectionCommand::MatchStyle | ReplaceSelectionCommand::PreventNesting));
+}
+
+/**
+ * webkit_web_frame_get_range_for_word_around_caret:
+ * @frame: a #WebKitWebFrame
+ *
+ * Returns a #WebKitDOMRange for the word where the caret is currently
+ * positioned.
+ *
+ * Returns: a #WebKitDOMRange spanning the word where the caret
+ * currently is positioned. If there is no caret %NULL will be
+ * returned.
+ *
+ * Since: 1.5.1.
+ **/
+WebKitDOMRange* webkit_web_frame_get_range_for_word_around_caret(WebKitWebFrame* frame)
+{
+    g_return_val_if_fail(WEBKIT_IS_WEB_FRAME(frame), 0);
+
+    Frame* coreFrame = core(frame);
+    FrameSelection* selection = coreFrame->selection();
+    if (selection->isNone() || selection->isRange())
+        return 0;
+    VisibleSelection visibleSelection(selection->selection().visibleStart());
+    visibleSelection.expandUsingGranularity(WordGranularity);
+
+    return kit(visibleSelection.firstRange().get());
+}
+
 namespace WebKit {
 
 WebKitWebView* getViewFromFrame(WebKitWebFrame* frame)
