@@ -25,6 +25,7 @@
 
 #include "DataObjectGtk.h"
 #include "FullscreenVideoController.h"
+#include "GtkClickCounter.h"
 #include "GOwnPtr.h"
 #include "Page.h"
 #include "ResourceHandle.h"
@@ -85,7 +86,6 @@ struct _WebKitWebViewPrivate {
     CString iconURI;
 
     gboolean disposing;
-    gboolean usePrimaryForPaste;
 
 #if ENABLE(VIDEO)
     FullscreenVideoController* fullscreenVideoController;
@@ -99,12 +99,11 @@ struct _WebKitWebViewPrivate {
     CString tooltipText;
     WebCore::IntRect tooltipArea;
 
-    int currentClickCount;
-    WebCore::IntPoint previousClickPoint;
-    guint previousClickButton;
-    guint32 previousClickTime;
+    WebCore::GtkClickCounter clickCounter;
     HashMap<GdkDragContext*, RefPtr<WebCore::DataObjectGtk> > draggingDataObjects;
     HashMap<GdkDragContext*, WebKit::DroppingContext*> droppingContexts;
+
+    bool selfScrolling;
 };
 
 void webkit_web_view_notify_ready(WebKitWebView*);
@@ -122,9 +121,6 @@ GList* webkit_web_view_get_subresources(WebKitWebView*);
 void webkit_web_view_set_tooltip_text(WebKitWebView*, const char*);
 GtkMenu* webkit_web_view_get_context_menu(WebKitWebView*);
 
-WEBKIT_API gchar* webkit_web_view_get_selected_text(WebKitWebView*);
-bool webkit_web_view_use_primary_for_paste(WebKitWebView*);
-
 void webViewEnterFullscreen(WebKitWebView* webView, WebCore::Node*);
 void webViewExitFullscreen(WebKitWebView* webView);
 
