@@ -34,54 +34,78 @@
 #include "NPRuntimeUtilities.h"
 #include "NPVariantData.h"
 #include "NetscapePlugin.h"
+#include "PluginCreationParameters.h"
 #include "PluginProcess.h"
 #include "PluginProxyMessages.h"
 #include "ShareableBitmap.h"
 #include "WebCoreArgumentCoders.h"
 #include "WebProcessConnection.h"
 #include <WebCore/GraphicsContext.h>
+#include <WebCore/IdentifierRep.h>
+#include <WebCore/NotImplemented.h>
 #include <wtf/text/WTFString.h>
 
 using namespace WebCore;
 
 namespace WebKit {
 
-PassOwnPtr<PluginControllerProxy> PluginControllerProxy::create(WebProcessConnection* connection, uint64_t pluginInstanceID, const String& userAgent, bool isPrivateBrowsingEnabled, bool isAcceleratedCompositingEnabled)
+PassOwnPtr<PluginControllerProxy> PluginControllerProxy::create(WebProcessConnection* connection, const PluginCreationParameters& creationParameters)
 {
-    return adoptPtr(new PluginControllerProxy(connection, pluginInstanceID, userAgent, isPrivateBrowsingEnabled, isAcceleratedCompositingEnabled));
+    return adoptPtr(new PluginControllerProxy(connection, creationParameters));
 }
 
-PluginControllerProxy::PluginControllerProxy(WebProcessConnection* connection, uint64_t pluginInstanceID, const String& userAgent, bool isPrivateBrowsingEnabled, bool isAcceleratedCompositingEnabled)
+PluginControllerProxy::PluginControllerProxy(WebProcessConnection* connection, const PluginCreationParameters& creationParameters)
     : m_connection(connection)
-    , m_pluginInstanceID(pluginInstanceID)
-    , m_userAgent(userAgent)
-    , m_isPrivateBrowsingEnabled(isPrivateBrowsingEnabled)
-    , m_isAcceleratedCompositingEnabled(isAcceleratedCompositingEnabled)
+    , m_pluginInstanceID(creationParameters.pluginInstanceID)
+    , m_userAgent(creationParameters.userAgent)
+    , m_isPrivateBrowsingEnabled(creationParameters.isPrivateBrowsingEnabled)
+#if USE(ACCELERATED_COMPOSITING)
+    , m_isAcceleratedCompositingEnabled(creationParameters.isAcceleratedCompositingEnabled)
+#endif
     , m_paintTimer(RunLoop::main(), this, &PluginControllerProxy::paint)
     , m_pluginDestructionProtectCount(0)
     , m_pluginDestroyTimer(RunLoop::main(), this, &PluginControllerProxy::destroy)
+    , m_pluginCreationParameters(0)
     , m_waitingForDidUpdate(false)
     , m_pluginCanceledManualStreamLoad(false)
 #if PLATFORM(MAC)
     , m_isComplexTextInputEnabled(false)
 #endif
+    , m_windowNPObject(0)
+    , m_pluginElementNPObject(0)
 {
 }
 
 PluginControllerProxy::~PluginControllerProxy()
 {
     ASSERT(!m_plugin);
+
+    if (m_windowNPObject)
+        releaseNPObject(m_windowNPObject);
+
+    if (m_pluginElementNPObject)
+        releaseNPObject(m_pluginElementNPObject);
 }
 
-bool PluginControllerProxy::initialize(const Plugin::Parameters& parameters)
+bool PluginControllerProxy::initialize(const PluginCreationParameters& creationParameters)
 {
     ASSERT(!m_plugin);
 
     m_plugin = NetscapePlugin::create(PluginProcess::shared().netscapePluginModule());
-    if (!m_plugin)
+    if (!m_plugin) {
+        // This will delete the plug-in controller proxy object.
+        m_connection->removePluginControllerProxy(this, 0);
         return false;
+    }
 
-    if (!m_plugin->initialize(this, parameters)) {
+    m_windowNPObject = m_connection->npRemoteObjectMap()->createNPObjectProxy(creationParameters.windowNPObjectID, m_plugin.get());
+    ASSERT(m_windowNPObject);
+
+    m_pluginCreationParameters = &creationParameters;
+    bool returnValue = m_plugin->initialize(this, creationParameters.parameters);
+    m_pluginCreationParameters = 0;
+
+    if (!returnValue) {
         // Get the plug-in so we can pass it to removePluginControllerProxy. The pointer is only
         // used as an identifier so it's OK to just get a weak reference.
         Plugin* plugin = m_plugin.get();
@@ -113,7 +137,7 @@ void PluginControllerProxy::destroy()
     // used as an identifier so it's OK to just get a weak reference.
     Plugin* plugin = m_plugin.get();
 
-    m_plugin->destroy();
+    m_plugin->destroyPlugin();
     m_plugin = 0;
 
     platformDestroy();
@@ -133,11 +157,16 @@ void PluginControllerProxy::paint()
     IntRect dirtyRect = m_dirtyRect;
     m_dirtyRect = IntRect();
 
+    ASSERT(m_plugin);
+
     // Create a graphics context.
     OwnPtr<GraphicsContext> graphicsContext = m_backingStore->createGraphicsContext();
+
     graphicsContext->translate(-m_frameRect.x(), -m_frameRect.y());
 
-    ASSERT(m_plugin);
+    if (m_plugin->isTransparent())
+        graphicsContext->clearRect(dirtyRect);
+
     m_plugin->paint(graphicsContext.get(), dirtyRect);
 
     m_connection->connection()->send(Messages::PluginProxy::Update(dirtyRect), m_pluginInstanceID);
@@ -164,6 +193,13 @@ void PluginControllerProxy::startPaintTimer()
     m_waitingForDidUpdate = true;
 }
 
+bool PluginControllerProxy::isPluginVisible()
+{
+    // FIXME: Implement this.
+    notImplemented();
+    return false;
+}
+
 void PluginControllerProxy::invalidate(const IntRect& rect)
 {
     // Convert the dirty rect to window coordinates.
@@ -202,32 +238,34 @@ void PluginControllerProxy::cancelManualStreamLoad()
 
 NPObject* PluginControllerProxy::windowScriptNPObject()
 {
-    uint64_t windowScriptNPObjectID = 0;
-
-    if (!m_connection->connection()->sendSync(Messages::PluginProxy::GetWindowScriptNPObject(), Messages::PluginProxy::GetWindowScriptNPObject::Reply(windowScriptNPObjectID), m_pluginInstanceID))
-        return 0;
-
-    if (!windowScriptNPObjectID)
-        return 0;
-
-    return m_connection->npRemoteObjectMap()->createNPObjectProxy(windowScriptNPObjectID, m_plugin.get());
+    retainNPObject(m_windowNPObject);
+    return m_windowNPObject;
 }
 
 NPObject* PluginControllerProxy::pluginElementNPObject()
 {
-    uint64_t pluginElementNPObjectID = 0;
+    if (!m_pluginElementNPObject) {
+        uint64_t pluginElementNPObjectID = 0;
 
-    if (!m_connection->connection()->sendSync(Messages::PluginProxy::GetPluginElementNPObject(), Messages::PluginProxy::GetPluginElementNPObject::Reply(pluginElementNPObjectID), m_pluginInstanceID))
-        return 0;
+        if (!m_connection->connection()->sendSync(Messages::PluginProxy::GetPluginElementNPObject(), Messages::PluginProxy::GetPluginElementNPObject::Reply(pluginElementNPObjectID), m_pluginInstanceID))
+            return 0;
 
-    if (!pluginElementNPObjectID)
-        return 0;
+        if (!pluginElementNPObjectID)
+            return 0;
 
-    return m_connection->npRemoteObjectMap()->createNPObjectProxy(pluginElementNPObjectID, m_plugin.get());
+        m_pluginElementNPObject = m_connection->npRemoteObjectMap()->createNPObjectProxy(pluginElementNPObjectID, m_plugin.get());
+        ASSERT(m_pluginElementNPObject);
+    }
+
+    retainNPObject(m_pluginElementNPObject);
+    return m_pluginElementNPObject;
 }
 
 bool PluginControllerProxy::evaluate(NPObject* npObject, const String& scriptString, NPVariant* result, bool allowPopups)
 {
+    if (tryToShortCircuitEvaluate(npObject, scriptString, result))
+        return true;
+
     PluginDestructionProtector protector(this);
 
     NPVariant npObjectAsNPVariant;
@@ -249,6 +287,58 @@ bool PluginControllerProxy::evaluate(NPObject* npObject, const String& scriptStr
     return true;
 }
 
+bool PluginControllerProxy::tryToShortCircuitInvoke(NPObject* npObject, NPIdentifier methodName, const NPVariant* arguments, uint32_t argumentCount, bool& returnValue, NPVariant& result)
+{
+    // Only try to short circuit evaluate for plug-ins that have the quirk specified.
+#if PLUGIN_ARCHITECTURE(MAC)
+    if (!PluginProcess::shared().netscapePluginModule()->pluginQuirks().contains(PluginQuirks::CanShortCircuitSomeNPRuntimeCallsDuringInitialization))
+        return false;
+#else
+    return false;
+#endif
+
+    // And only when we're in initialize.
+    if (!inInitialize())
+        return false;
+    
+    // And only when the NPObject is the window NPObject.
+    if (npObject != m_windowNPObject)
+        return false;
+
+    // And only when we don't have any arguments.
+    if (argumentCount)
+        return false;
+
+    IdentifierRep* methodNameRep = static_cast<IdentifierRep*>(methodName);
+    if (!methodNameRep->isString())
+        return false;
+
+    if (!strcmp(methodNameRep->string(), "__flash_getWindowLocation")) {
+        result.type = NPVariantType_String;
+        result.value.stringValue = createNPString(m_pluginCreationParameters->parameters.documentURL.utf8()); 
+        returnValue = true;
+        return true;
+    }
+    
+    if (!strcmp(methodNameRep->string(), "__flash_getTopLocation")) {
+        if (m_pluginCreationParameters->parameters.toplevelDocumentURL.isNull()) {
+            // If the toplevel document is URL it means that the frame that the plug-in is in doesn't have access to the toplevel document.
+            // In this case, just pass the string "[object]" to Flash.
+            result.type = NPVariantType_String;
+            result.value.stringValue = createNPString("[object]");
+            returnValue = true;
+            return true;
+        }
+
+        result.type = NPVariantType_String;
+        result.value.stringValue = createNPString(m_pluginCreationParameters->parameters.toplevelDocumentURL.utf8()); 
+        returnValue = true;
+        return true;
+    }
+
+    return false;
+}
+
 void PluginControllerProxy::setStatusbarText(const String& statusbarText)
 {
     m_connection->connection()->send(Messages::PluginProxy::SetStatusbarText(statusbarText), m_pluginInstanceID);
@@ -265,19 +355,10 @@ void PluginControllerProxy::pluginProcessCrashed()
     ASSERT_NOT_REACHED();
 }
 
-void PluginControllerProxy::setComplexTextInputEnabled(bool complexTextInputEnabled)
+void PluginControllerProxy::willSendEventToPlugin()
 {
-    if (m_isComplexTextInputEnabled == complexTextInputEnabled)
-        return;
-
-    m_isComplexTextInputEnabled = complexTextInputEnabled;
-
-    m_connection->connection()->send(Messages::PluginProxy::SetComplexTextInputEnabled(complexTextInputEnabled), m_pluginInstanceID);
-}
-
-mach_port_t PluginControllerProxy::compositingRenderServerPort()
-{
-    return PluginProcess::shared().compositingRenderServerPort();
+    // This is only used when running plugins in the web process.
+    ASSERT_NOT_REACHED();
 }
 
 String PluginControllerProxy::proxiesForURL(const String& urlString)
@@ -310,6 +391,15 @@ bool PluginControllerProxy::isPrivateBrowsingEnabled()
     return m_isPrivateBrowsingEnabled;
 }
 
+bool PluginControllerProxy::getAuthenticationInfo(const ProtectionSpace& protectionSpace, String& username, String& password)
+{
+    bool returnValue;
+    if (!m_connection->connection()->sendSync(Messages::PluginProxy::GetAuthenticationInfo(protectionSpace), Messages::PluginProxy::GetAuthenticationInfo::Reply(returnValue, username, password), m_pluginInstanceID))
+        return false;
+
+    return returnValue;
+}
+
 void PluginControllerProxy::protectPluginFromDestruction()
 {
     m_pluginDestructionProtectCount++;
@@ -332,26 +422,26 @@ void PluginControllerProxy::frameDidFail(uint64_t requestID, bool wasCancelled)
     m_plugin->frameDidFail(requestID, wasCancelled);
 }
 
-void PluginControllerProxy::geometryDidChange(const IntRect& frameRect, const IntRect& clipRect, const SharedMemory::Handle& backingStoreHandle)
+void PluginControllerProxy::geometryDidChange(const IntRect& frameRect, const IntRect& clipRect, const ShareableBitmap::Handle& backingStoreHandle)
 {
     m_frameRect = frameRect;
     m_clipRect = clipRect;
 
     ASSERT(m_plugin);
 
+    platformGeometryDidChange();
+
     if (!backingStoreHandle.isNull()) {
         // Create a new backing store.
-        m_backingStore = ShareableBitmap::create(frameRect.size(), backingStoreHandle);
+        m_backingStore = ShareableBitmap::create(backingStoreHandle);
     }
 
     m_plugin->geometryDidChange(frameRect, clipRect);
-
-    platformGeometryDidChange(frameRect, clipRect);
 }
 
-void PluginControllerProxy::didEvaluateJavaScript(uint64_t requestID, const String& requestURLString, const String& result)
+void PluginControllerProxy::didEvaluateJavaScript(uint64_t requestID, const String& result)
 {
-    m_plugin->didEvaluateJavaScript(requestID, requestURLString, result);
+    m_plugin->didEvaluateJavaScript(requestID, result);
 }
 
 void PluginControllerProxy::streamDidReceiveResponse(uint64_t streamID, const String& responseURLString, uint32_t streamLength, uint32_t lastModifiedTime, const String& mimeType, const String& headers)
@@ -406,9 +496,17 @@ void PluginControllerProxy::manualStreamDidFail(bool wasCancelled)
     m_plugin->manualStreamDidFail(wasCancelled);
 }
     
-void PluginControllerProxy::handleMouseEvent(const WebMouseEvent& mouseEvent, bool& handled)
+void PluginControllerProxy::handleMouseEvent(const WebMouseEvent& mouseEvent, PassRefPtr<Messages::PluginControllerProxy::HandleMouseEvent::DelayedReply> reply)
 {
-    handled = m_plugin->handleMouseEvent(mouseEvent);
+    // Always let the web process think that we've handled this mouse event, even before passing it along to the plug-in.
+    // This is a workaround for 
+    // <rdar://problem/9299901> UI process thinks the page is unresponsive when a plug-in is showing a context menu.
+    // The web process sends a synchronous HandleMouseEvent message and the plug-in process spawns a nested
+    // run loop when showing the context menu, so eventually the unresponsiveness timer kicks in in the UI process.
+    // FIXME: We should come up with a better way to do this.
+    reply->send(true);
+
+    m_plugin->handleMouseEvent(mouseEvent);
 }
 
 void PluginControllerProxy::handleWheelEvent(const WebWheelEvent& wheelEvent, bool& handled)
@@ -440,7 +538,7 @@ void PluginControllerProxy::paintEntirePlugin()
     paint();
 }
 
-void PluginControllerProxy::snapshot(WebCore::IntSize& bufferSize, SharedMemory::Handle& backingStoreHandle)
+void PluginControllerProxy::snapshot(ShareableBitmap::Handle& backingStoreHandle)
 {
     ASSERT(m_plugin);
     RefPtr<ShareableBitmap> bitmap = m_plugin->snapshot();
@@ -448,7 +546,6 @@ void PluginControllerProxy::snapshot(WebCore::IntSize& bufferSize, SharedMemory:
         return;
 
     bitmap->createHandle(backingStoreHandle);
-    bufferSize = bitmap->size();
 }
 
 void PluginControllerProxy::setFocus(bool hasFocus)
@@ -474,34 +571,45 @@ void PluginControllerProxy::getPluginScriptableNPObject(uint64_t& pluginScriptab
     releaseNPObject(pluginScriptableNPObject);
 }
 
-#if PLATFORM(MAC)
-void PluginControllerProxy::windowFocusChanged(bool hasFocus)
-{
-    m_plugin->windowFocusChanged(hasFocus);
-}
-
-void PluginControllerProxy::windowAndViewFramesChanged(const IntRect& windowFrameInScreenCoordinates, const IntRect& viewFrameInWindowCoordinates)
-{
-    m_plugin->windowAndViewFramesChanged(windowFrameInScreenCoordinates, viewFrameInWindowCoordinates);
-}
-
-void PluginControllerProxy::windowVisibilityChanged(bool isVisible)
-{
-    m_plugin->windowVisibilityChanged(isVisible);
-}
-
-void PluginControllerProxy::sendComplexTextInput(const String& textInput)
-{
-    m_plugin->sendComplexTextInput(textInput);
-}
-
-#endif
-
 void PluginControllerProxy::privateBrowsingStateChanged(bool isPrivateBrowsingEnabled)
 {
+    m_isPrivateBrowsingEnabled = isPrivateBrowsingEnabled;
+
     m_plugin->privateBrowsingStateChanged(isPrivateBrowsingEnabled);
 }
 
+void PluginControllerProxy::getFormValue(bool& returnValue, String& formValue)
+{
+    returnValue = m_plugin->getFormValue(formValue);
+}
+
+bool PluginControllerProxy::tryToShortCircuitEvaluate(NPObject* npObject, const String& scriptString, NPVariant* result)
+{
+    // Only try to short circuit evaluate for plug-ins that have the quirk specified.
+#if PLUGIN_ARCHITECTURE(MAC)
+    if (!PluginProcess::shared().netscapePluginModule()->pluginQuirks().contains(PluginQuirks::CanShortCircuitSomeNPRuntimeCallsDuringInitialization))
+        return false;
+#else
+    return false;
+#endif
+
+    // And only when we're in initialize.
+    if (!inInitialize())
+        return false;
+
+    // And only when the NPObject is the window NPObject.
+    if (npObject != m_windowNPObject)
+        return false;
+
+    // Now, check for the right strings.
+    if (scriptString != "function __flash_getWindowLocation() { return window.location; }"
+        && scriptString != "function __flash_getTopLocation() { return top.location; }")
+        return false;
+
+    VOID_TO_NPVARIANT(*result);
+    return true;
+}
+
 } // namespace WebKit
 
 #endif // ENABLE(PLUGIN_PROCESS)
