@@ -31,10 +31,14 @@
 #include "ContextMenuState.h"
 #include "DragControllerAction.h"
 #include "DrawingAreaProxy.h"
+#include "EditorState.h"
 #include "GeolocationPermissionRequestManagerProxy.h"
+#if ENABLE(TOUCH_EVENTS)
+#include "NativeWebTouchEvent.h"
+#endif
+#include "PlatformProcessIdentifier.h"
 #include "SandboxExtension.h"
-#include "SelectionState.h"
-#include "SharedMemory.h"
+#include "ShareableBitmap.h"
 #include "WKBase.h"
 #include "WKPagePrivate.h"
 #include "WebContextMenuItemData.h"
@@ -42,19 +46,28 @@
 #include "WebFormClient.h"
 #include "WebFrameProxy.h"
 #include "WebHistoryClient.h"
+#include "WebHitTestResult.h"
 #include "WebLoaderClient.h"
 #include "WebPageContextMenuClient.h"
 #include "WebPolicyClient.h"
 #include "WebPopupMenuProxy.h"
 #include "WebResourceLoadClient.h"
 #include "WebUIClient.h"
+#include <WebCore/DragActions.h>
+#include <WebCore/HitTestResult.h>
+#include <WebCore/ScrollTypes.h>
+#include <WebCore/TextChecking.h>
 #include <wtf/HashMap.h>
 #include <wtf/HashSet.h>
 #include <wtf/OwnPtr.h>
 #include <wtf/PassOwnPtr.h>
 #include <wtf/PassRefPtr.h>
 #include <wtf/RefPtr.h>
+#include <wtf/Vector.h>
 #include <wtf/text/WTFString.h>
+#if PLATFORM(EFL)
+#include <Evas.h>
+#endif
 
 namespace CoreIPC {
     class ArgumentDecoder;
@@ -74,9 +87,19 @@ namespace WebCore {
     struct WindowFeatures;
 }
 
+#if PLATFORM(MAC)
+#ifdef __OBJC__
+@class WKView;
+#else
+class WKView;
+#endif
+#endif
+
 namespace WebKit {
 
 class NativeWebKeyboardEvent;
+class NativeWebMouseEvent;
+class NativeWebWheelEvent;
 class PageClient;
 class PlatformCertificateInfo;
 class StringPairVector;
@@ -93,12 +116,18 @@ class WebPageGroup;
 class WebProcessProxy;
 class WebURLRequest;
 class WebWheelEvent;
+struct AttributedString;
 struct DictionaryPopupInfo;
+struct EditorState;
 struct PlatformPopupMenuData;
 struct PrintInfo;
 struct WebPageCreationParameters;
 struct WebPopupItem;
 
+#if PLATFORM(WIN)
+struct WindowGeometry;
+#endif
+
 #if ENABLE(GESTURE_EVENTS)
 class WebGestureEvent;
 #endif
@@ -106,6 +135,17 @@ class WebGestureEvent;
 typedef GenericCallback<WKStringRef, StringImpl*> StringCallback;
 typedef GenericCallback<WKSerializedScriptValueRef, WebSerializedScriptValue*> ScriptValueCallback;
 
+#if ENABLE(TOUCH_EVENTS)
+struct QueuedTouchEvents {
+    QueuedTouchEvents(const NativeWebTouchEvent& event)
+        : forwardedEvent(event)
+    {
+    }
+    NativeWebTouchEvent forwardedEvent;
+    Vector<NativeWebTouchEvent> deferredTouchEvents;
+};
+#endif
+
 // FIXME: Make a version of CallbackBase with three arguments, and define ValidateCommandCallback as a specialization.
 class ValidateCommandCallback : public CallbackBase {
 public:
@@ -155,7 +195,7 @@ class WebPageProxy : public APIObject, public WebPopupMenuProxy::Client {
 public:
     static const Type APIType = TypePage;
 
-    static PassRefPtr<WebPageProxy> create(PageClient*, WebContext*, WebPageGroup*, uint64_t pageID);
+    static PassRefPtr<WebPageProxy> create(PageClient*, PassRefPtr<WebProcessProxy>, WebPageGroup*, uint64_t pageID);
     virtual ~WebPageProxy();
 
     uint64_t pageID() const { return m_pageID; }
@@ -164,10 +204,9 @@ public:
     WebFrameProxy* focusedFrame() const { return m_focusedFrame.get(); }
     WebFrameProxy* frameSetLargestFrame() const { return m_frameSetLargestFrame.get(); }
 
-    DrawingAreaProxy* drawingArea() { return m_drawingArea.get(); }
-    void setDrawingArea(PassOwnPtr<DrawingAreaProxy>);
+    DrawingAreaProxy* drawingArea() const { return m_drawingArea.get(); }
 
-    WebBackForwardList* backForwardList() { return m_backForwardList.get(); }
+    WebBackForwardList* backForwardList() const { return m_backForwardList.get(); }
 
 #if ENABLE(INSPECTOR)
     WebInspectorProxy* inspector();
@@ -206,9 +245,12 @@ public:
     bool canGoBack() const;
 
     void goToBackForwardItem(WebBackForwardListItem*);
+    void tryRestoreScrollPosition();
     void didChangeBackForwardList(WebBackForwardListItem* addedItem, Vector<RefPtr<APIObject> >* removedItems);
     void shouldGoToBackForwardListItem(uint64_t itemID, bool& shouldGoToBackForwardListItem);
 
+    bool willHandleHorizontalScrollEvents() const;
+
     bool canShowMIMEType(const String& mimeType) const;
 
     bool drawsBackground() const { return m_drawsBackground; }
@@ -222,6 +264,8 @@ public:
 
     void setInitialFocus(bool);
     void setWindowResizerSize(const WebCore::IntSize&);
+    
+    void clearSelection();
 
     void setViewNeedsDisplay(const WebCore::IntRect&);
     void displayView();
@@ -231,30 +275,47 @@ public:
         ViewWindowIsActive = 1 << 0,
         ViewIsFocused = 1 << 1,
         ViewIsVisible = 1 << 2,
-        ViewIsInWindow = 1 << 3
+        ViewIsInWindow = 1 << 3,
     };
     typedef unsigned ViewStateFlags;
     void viewStateDidChange(ViewStateFlags flags);
 
     WebCore::IntSize viewSize() const;
     bool isViewVisible() const { return m_isVisible; }
+    bool isViewWindowActive() const;
 
     void executeEditCommand(const String& commandName);
     void validateCommand(const String& commandName, PassRefPtr<ValidateCommandCallback>);
 
-    const SelectionState& selectionState() const { return m_selectionState; }
+    const EditorState& editorState() const { return m_editorState; }
     bool canDelete() const { return hasSelectedRange() && isContentEditable(); }
-    bool hasSelectedRange() const { return m_selectionState.selectedRangeLength; }
-    bool isContentEditable() const { return m_selectionState.isContentEditable; }
+    bool hasSelectedRange() const { return m_editorState.selectionIsRange; }
+    bool isContentEditable() const { return m_editorState.isContentEditable; }
+    
+    bool maintainsInactiveSelection() const { return m_maintainsInactiveSelection; }
+    void setMaintainsInactiveSelection(bool);
 
 #if PLATFORM(MAC)
     void updateWindowIsVisible(bool windowIsVisible);
     void windowAndViewFramesChanged(const WebCore::IntRect& windowFrameInScreenCoordinates, const WebCore::IntRect& viewFrameInWindowCoordinates, const WebCore::IntPoint& accessibilityViewCoordinates);
+
+    void setComposition(const String& text, Vector<WebCore::CompositionUnderline> underlines, uint64_t selectionStart, uint64_t selectionEnd, uint64_t replacementRangeStart, uint64_t replacementRangeEnd);
+    void confirmComposition();
+    void cancelComposition();
+    bool insertText(const String& text, uint64_t replacementRangeStart, uint64_t replacementRangeEnd);
     void getMarkedRange(uint64_t& location, uint64_t& length);
+    void getSelectedRange(uint64_t& location, uint64_t& length);
+    void getAttributedSubstringFromRange(uint64_t location, uint64_t length, AttributedString&);
     uint64_t characterIndexForPoint(const WebCore::IntPoint);
     WebCore::IntRect firstRectForCharacterRange(uint64_t, uint64_t);
+    bool executeKeypressCommands(const Vector<WebCore::KeypressCommand>&);
+
     void sendComplexTextInputToPlugin(uint64_t pluginComplexTextInputIdentifier, const String& textInput);
     CGContextRef containingWindowGraphicsContext();
+    bool shouldDelayWindowOrderingForEvent(const WebMouseEvent&);
+    bool acceptsFirstMouse(int eventNumber, const WebMouseEvent&);
+    
+    WKView* wkView() const;
 #endif
 #if PLATFORM(WIN)
     void didChangeCompositionSelection(bool);
@@ -262,21 +323,38 @@ public:
     void setComposition(const String&, Vector<WebCore::CompositionUnderline>&, int);
     WebCore::IntRect firstRectForCharacterInSelectedRange(int);
     String getSelectedText();
+
+    bool gestureWillBegin(const WebCore::IntPoint&);
+    void gestureDidScroll(const WebCore::IntSize&);
+    void gestureDidEnd();
+
+    void setGestureReachedScrollingLimit(bool);
+
+    HWND nativeWindow() const;
+#endif
+#if PLATFORM(GTK)
+    GtkWidget* viewWidget();
+#endif
+#if PLATFORM(EFL)
+    Evas_Object* viewObject();
 #endif
 #if ENABLE(TILED_BACKING_STORE)
-    void setActualVisibleContentRect(const WebCore::IntRect& rect);
+    void setFixedVisibleContentRect(const WebCore::IntRect&);
 #endif
 
-    void handleMouseEvent(const WebMouseEvent&);
-    void handleWheelEvent(const WebWheelEvent&);
+    void handleMouseEvent(const NativeWebMouseEvent&);
+    void handleWheelEvent(const NativeWebWheelEvent&);
     void handleKeyboardEvent(const NativeWebKeyboardEvent&);
 #if ENABLE(GESTURE_EVENTS)
     void handleGestureEvent(const WebGestureEvent&);
 #endif
 #if ENABLE(TOUCH_EVENTS)
-    void handleTouchEvent(const WebTouchEvent&);
+    void handleTouchEvent(const NativeWebTouchEvent&);
 #endif
 
+    void scrollBy(WebCore::ScrollDirection, WebCore::ScrollGranularity);
+    void centerSelectionInVisibleArea();
+
     String pageTitle() const;
     const String& toolTip() const { return m_toolTip; }
 
@@ -286,6 +364,7 @@ public:
     const String& applicationNameForUserAgent() const { return m_applicationNameForUserAgent; }
     void setCustomUserAgent(const String&);
     const String& customUserAgent() const { return m_customUserAgent; }
+    static String standardUserAgent(const String& applicationName = String());
 
     bool supportsTextEncoding() const;
     void setCustomTextEncodingName(const String&);
@@ -306,8 +385,12 @@ public:
     void setPageZoomFactor(double);
     void setPageAndTextZoomFactors(double pageZoomFactor, double textZoomFactor);
 
-    void scaleWebView(double scale, const WebCore::IntPoint& origin);
-    double viewScaleFactor() const { return m_viewScaleFactor; }
+    void scalePage(double scale, const WebCore::IntPoint& origin);
+    double pageScaleFactor() const { return m_pageScaleFactor; }
+
+    float deviceScaleFactor() const;
+    void setIntrinsicDeviceScaleFactor(float);
+    void setCustomDeviceScaleFactor(float);
 
     void setUseFixedLayout(bool);
     void setFixedLayoutSize(const WebCore::IntSize&);
@@ -326,9 +409,11 @@ public:
     // Called by the UI process when it is ready to send its tokens to the web process.
     void registerUIProcessAccessibilityTokens(const CoreIPC::DataReference& elemenToken, const CoreIPC::DataReference& windowToken);
     bool writeSelectionToPasteboard(const String& pasteboardName, const Vector<String>& pasteboardTypes);
+    bool readSelectionFromPasteboard(const String& pasteboardName);
+    void makeFirstResponder();
 #endif
 
-    void viewScaleFactorDidChange(double);
+    void pageScaleFactorDidChange(double);
 
     void setMemoryCacheClientCallsEnabled(bool);
 
@@ -336,6 +421,13 @@ public:
     void findString(const String&, FindOptions, unsigned maxMatchCount);
     void hideFindUI();
     void countStringMatches(const String&, FindOptions, unsigned maxMatchCount);
+    void didCountStringMatches(const String&, uint32_t matchCount);
+    void setFindIndicator(const WebCore::FloatRect& selectionRectInWindowCoordinates, const Vector<WebCore::FloatRect>& textRectsInSelectionRectCoordinates, float contentImageScaleFactor, const ShareableBitmap::Handle& contentImageHandle, bool fadeOut);
+    void didFindString(const String&, uint32_t matchCount);
+    void didFailToFindString(const String&);
+#if PLATFORM(WIN)
+    void didInstallOrUninstallPageOverlay(bool);
+#endif
 
     void getContentsAsString(PassRefPtr<StringCallback>);
     void getMainResourceDataOfFrame(WebFrameProxy*, PassRefPtr<DataCallback>);
@@ -362,17 +454,24 @@ public:
     void backForwardRemovedItem(uint64_t itemID);
 
     // Drag and drop support.
-    void performDragControllerAction(DragControllerAction, WebCore::DragData*, const String& = String());
+    void dragEntered(WebCore::DragData*, const String& dragStorageName = String());
+    void dragUpdated(WebCore::DragData*, const String& dragStorageName = String());
+    void dragExited(WebCore::DragData*, const String& dragStorageName = String());
+    void performDrag(WebCore::DragData*, const String& dragStorageName, const SandboxExtension::Handle&);
+
     void didPerformDragControllerAction(uint64_t resultOperation);
     void dragEnded(const WebCore::IntPoint& clientPosition, const WebCore::IntPoint& globalPosition, uint64_t operation);
 #if PLATFORM(MAC)
-    void setDragImage(const WebCore::IntPoint& clientPosition, const WebCore::IntSize& imageSize, const SharedMemory::Handle& dragImageHandle, bool isLinkDrag);
+    void setDragImage(const WebCore::IntPoint& clientPosition, const ShareableBitmap::Handle& dragImageHandle, bool isLinkDrag);
 #endif
 #if PLATFORM(WIN)
-    void startDragDrop(const WebCore::IntPoint& imagePoint, const WebCore::IntPoint& dragPoint, uint64_t okEffect, const HashMap<UINT, Vector<String> >& dataMap, const WebCore::IntSize& dragImageSize, const SharedMemory::Handle& dragImageHandle, bool isLinkDrag);
+    void startDragDrop(const WebCore::IntPoint& imagePoint, const WebCore::IntPoint& dragPoint, uint64_t okEffect, const HashMap<UINT, Vector<String> >& dataMap, uint64_t fileSize, const String& pathname, const SharedMemory::Handle& fileContentHandle, const WebCore::IntSize& dragImageSize, const SharedMemory::Handle& dragImageHandle, bool isLinkDrag);
+#endif
+#if PLATFORM(QT)
+    void startDrag(const WebCore::DragData&, const ShareableBitmap::Handle& dragImage);
 #endif
     void didReceiveMessage(CoreIPC::Connection*, CoreIPC::MessageID, CoreIPC::ArgumentDecoder*);
-    void didReceiveSyncMessage(CoreIPC::Connection*, CoreIPC::MessageID, CoreIPC::ArgumentDecoder*, CoreIPC::ArgumentEncoder*);
+    void didReceiveSyncMessage(CoreIPC::Connection*, CoreIPC::MessageID, CoreIPC::ArgumentDecoder*, OwnPtr<CoreIPC::ArgumentEncoder>&);
 
     void processDidBecomeUnresponsive();
     void processDidBecomeResponsive();
@@ -392,7 +491,7 @@ public:
     void registerEditCommand(PassRefPtr<WebEditCommandProxy>, UndoOrRedo);
 
     WebProcessProxy* process() const;
-    WebContext* context() const { return m_context.get(); }
+    PlatformProcessIdentifier processIdentifier() const;
 
     WebPageGroup* pageGroup() const { return m_pageGroup.get(); }
 
@@ -401,9 +500,6 @@ public:
     WebCore::DragOperation dragOperation() { return m_currentDragOperation; }
     void resetDragOperation() { m_currentDragOperation = WebCore::DragOperationNone; }
 
-    // REMOVE: For demo purposes only.
-    const String& urlAtProcessExit() const { return m_urlAtProcessExit; }
-
     void preferencesDidChange();
 
 #if ENABLE(TILED_BACKING_STORE)
@@ -423,10 +519,8 @@ public:
     void findZoomableAreaForPoint(const WebCore::IntPoint&);
 #endif
 
-    void advanceToNextMisspelling(bool startBeforeSelection);
-    void changeSpellingToWord(const String& word);
-    void unmarkAllMisspellings();
-    void unmarkAllBadGrammar();
+    void advanceToNextMisspelling(bool startBeforeSelection) const;
+    void changeSpellingToWord(const String& word) const;
 #if PLATFORM(MAC)
     void uppercaseWord();
     void lowercaseWord();
@@ -448,6 +542,10 @@ public:
 
     void flashBackingStoreUpdates(const Vector<WebCore::IntRect>& updateRects);
 
+#if PLATFORM(MAC)
+    void handleCorrectionPanelResult(const String& result);
+#endif
+
     static void setDebugPaintFlags(WKPageDebugPaintFlags flags) { s_debugPaintFlags = flags; }
     static WKPageDebugPaintFlags debugPaintFlags() { return s_debugPaintFlags; }
 
@@ -459,25 +557,34 @@ public:
 
     void saveDataToFileInDownloadsFolder(const String& suggestedFilename, const String& mimeType, const String& originatingURLString, WebData*);
 
+    void linkClicked(const String&, const WebMouseEvent&);
+
+    WebCore::IntRect visibleScrollerThumbRect() const { return m_visibleScrollerThumbRect; }
+
+    uint64_t renderTreeSize() const { return m_renderTreeSize; }
+
+    void setShouldSendKeyboardEventSynchronously(bool sync) { m_shouldSendKeyboardEventSynchronously = sync; };
+
 private:
-    WebPageProxy(PageClient*, WebContext*, WebPageGroup*, uint64_t pageID);
+    WebPageProxy(PageClient*, PassRefPtr<WebProcessProxy>, WebPageGroup*, uint64_t pageID);
 
     virtual Type type() const { return APIType; }
 
     // WebPopupMenuProxy::Client
     virtual void valueChangedForPopupMenu(WebPopupMenuProxy*, int32_t newSelectedIndex);
     virtual void setTextFromItemForPopupMenu(WebPopupMenuProxy*, int32_t index);
+    virtual NativeWebMouseEvent* currentlyProcessedMouseDownEvent();
 
     // Implemented in generated WebPageProxyMessageReceiver.cpp
     void didReceiveWebPageProxyMessage(CoreIPC::Connection*, CoreIPC::MessageID, CoreIPC::ArgumentDecoder*);
-    CoreIPC::SyncReplyMode didReceiveSyncWebPageProxyMessage(CoreIPC::Connection*, CoreIPC::MessageID, CoreIPC::ArgumentDecoder*, CoreIPC::ArgumentEncoder*);
+    void didReceiveSyncWebPageProxyMessage(CoreIPC::Connection*, CoreIPC::MessageID, CoreIPC::ArgumentDecoder*, OwnPtr<CoreIPC::ArgumentEncoder>&);
 
     void didCreateMainFrame(uint64_t frameID);
     void didCreateSubframe(uint64_t frameID, uint64_t parentFrameID);
     void didSaveFrameToPageCache(uint64_t frameID);
     void didRestoreFrameFromPageCache(uint64_t frameID, uint64_t parentFrameID);
 
-    void didStartProvisionalLoadForFrame(uint64_t frameID, const String&, bool loadingSubstituteDataForUnreachableURL, CoreIPC::ArgumentDecoder*);
+    void didStartProvisionalLoadForFrame(uint64_t frameID, const String& url, const String& unreachableURL, CoreIPC::ArgumentDecoder*);
     void didReceiveServerRedirectForProvisionalLoadForFrame(uint64_t frameID, const String&, CoreIPC::ArgumentDecoder*);
     void didFailProvisionalLoadForFrame(uint64_t frameID, const WebCore::ResourceError&, CoreIPC::ArgumentDecoder*);
     void didCommitLoadForFrame(uint64_t frameID, const String& mimeType, bool frameHasCustomRepresentation, const PlatformCertificateInfo&, CoreIPC::ArgumentDecoder*);
@@ -512,14 +619,15 @@ private:
     void didFailLoadForResource(uint64_t frameID, uint64_t resourceIdentifier, const WebCore::ResourceError&);
 
     // UI client
-    void createNewPage(const WebCore::WindowFeatures&, uint32_t modifiers, int32_t mouseButton, uint64_t& newPageID, WebPageCreationParameters&);
+    void createNewPage(const WebCore::ResourceRequest&, const WebCore::WindowFeatures&, uint32_t modifiers, int32_t mouseButton, uint64_t& newPageID, WebPageCreationParameters&);
     void showPage();
-    void closePage();
+    void closePage(bool stopResponsivenessTimer);
     void runJavaScriptAlert(uint64_t frameID, const String&);
     void runJavaScriptConfirm(uint64_t frameID, const String&, bool& result);
     void runJavaScriptPrompt(uint64_t frameID, const String&, const String&, String& result);
+    void shouldInterruptJavaScript(bool& result);
     void setStatusText(const String&);
-    void mouseDidMoveOverElement(uint32_t modifiers, CoreIPC::ArgumentDecoder*);
+    void mouseDidMoveOverElement(const WebHitTestResult::Data& hitTestResultData, uint32_t modifiers, CoreIPC::ArgumentDecoder*);
     void missingPluginButtonClicked(const String& mimeType, const String& url, const String& pluginsPageURL);
     void setToolbarsAreVisible(bool toolbarsAreVisible);
     void getToolbarsAreVisible(bool& toolbarsAreVisible);
@@ -531,18 +639,22 @@ private:
     void getIsResizable(bool& isResizable);
     void setWindowFrame(const WebCore::FloatRect&);
     void getWindowFrame(WebCore::FloatRect&);
+    void screenToWindow(const WebCore::IntPoint& screenPoint, WebCore::IntPoint& windowPoint);
     void windowToScreen(const WebCore::IntRect& viewRect, WebCore::IntRect& result);
     void runBeforeUnloadConfirmPanel(const String& message, uint64_t frameID, bool& shouldClose);
     void didChangeViewportData(const WebCore::ViewportArguments&);
     void pageDidScroll();
     void runOpenPanel(uint64_t frameID, const WebOpenPanelParameters::Data&);
     void printFrame(uint64_t frameID);
-    void exceededDatabaseQuota(uint64_t frameID, const String& originIdentifier, const String& databaseName, const String& displayName, uint64_t currentQuota, uint64_t currentUsage, uint64_t expectedUsage, uint64_t& newQuota);
+    void exceededDatabaseQuota(uint64_t frameID, const String& originIdentifier, const String& databaseName, const String& displayName, uint64_t currentQuota, uint64_t currentOriginUsage, uint64_t currentDatabaseUsage, uint64_t expectedUsage, uint64_t& newQuota);
     void requestGeolocationPermissionForFrame(uint64_t geolocationID, uint64_t frameID, String originIdentifier);
-    void runModal() { m_uiClient.runModal(this); }
+    void runModal();
     void didCompleteRubberBandForMainFrame(const WebCore::IntSize&);
+    void notifyScrollerThumbIsVisibleInRect(const WebCore::IntRect&);
     void didChangeScrollbarsForMainFrame(bool hasHorizontalScrollbar, bool hasVerticalScrollbar);
     void didChangeScrollOffsetPinningForMainFrame(bool pinnedToLeftSide, bool pinnedToRightSide);
+    void didFailToInitializePlugin(const String& mimeType);
+    void numWheelEventHandlersChanged(unsigned count) { m_wheelEventHandlerCount = count; }
 
     void reattachToWebProcess();
     void reattachToWebProcessWithItem(WebBackForwardListItem*);
@@ -553,11 +665,13 @@ private:
 
 #if PLATFORM(QT)
     void didChangeContentsSize(const WebCore::IntSize&);
-    void didFindZoomableArea(const WebCore::IntRect&);
+    void didFindZoomableArea(const WebCore::IntPoint&, const WebCore::IntRect&);
+#endif
+#if ENABLE(TOUCH_EVENTS)
+    void needTouchEvents(bool);
 #endif
 
-    // Selection
-    void selectionStateChanged(const SelectionState&);
+    void editorStateChanged(const EditorState&);
 
     // Back/Forward list management
     void backForwardAddItem(uint64_t itemID);
@@ -570,24 +684,36 @@ private:
     // Undo management
     void registerEditCommandForUndo(uint64_t commandID, uint32_t editAction);
     void clearAllEditCommands();
+    void canUndoRedo(uint32_t action, bool& result);
+    void executeUndoRedo(uint32_t action, bool& result);
 
     // Keyboard handling
 #if PLATFORM(MAC)
-    void interpretKeyEvent(uint32_t eventType, Vector<WebCore::KeypressCommand>&, uint32_t selectionStart, uint32_t selectionEnd, Vector<WebCore::CompositionUnderline>& underlines);
+    void interpretQueuedKeyEvent(const EditorState&, bool& handled, Vector<WebCore::KeypressCommand>&);
+    void executeSavedCommandBySelector(const String& selector, bool& handled);
+#endif
+
+#if PLATFORM(GTK)
+    void getEditorCommandsForKeyEvent(const AtomicString&, Vector<String>&);
+#endif
+#if PLATFORM(EFL)
+    void getEditorCommandsForKeyEvent(Vector<String>&);
 #endif
-    
-    // Find.
-    void didCountStringMatches(const String&, uint32_t matchCount);
-    void setFindIndicator(const WebCore::FloatRect& selectionRectInWindowCoordinates, const Vector<WebCore::FloatRect>& textRectsInSelectionRectCoordinates, const SharedMemory::Handle& contentImageHandle, bool fadeOut);
-    void didFindString(const String&, uint32_t matchCount);
-    void didFailToFindString(const String&);
 
     // Popup Menu.
     void showPopupMenu(const WebCore::IntRect& rect, uint64_t textDirection, const Vector<WebPopupItem>& items, int32_t selectedIndex, const PlatformPopupMenuData&);
     void hidePopupMenu();
+#if PLATFORM(WIN)
+    void setPopupMenuSelectedIndex(int32_t);
+#endif
 
     // Context Menu.
     void showContextMenu(const WebCore::IntPoint& menuLocation, const ContextMenuState&, const Vector<WebContextMenuItemData>&, CoreIPC::ArgumentDecoder*);
+    void internalShowContextMenu(const WebCore::IntPoint& menuLocation, const ContextMenuState&, const Vector<WebContextMenuItemData>&, CoreIPC::ArgumentDecoder*);
+
+    // Search popup results
+    void saveRecentSearches(const String&, const Vector<String>&);
+    void loadRecentSearches(const String&, Vector<String>&);
 
 #if PLATFORM(MAC)
     // Speech.
@@ -595,13 +721,21 @@ private:
     void speak(const String&);
     void stopSpeaking();
 
+    // Spotlight.
+    void searchWithSpotlight(const String&);
+
     // Dictionary.
     void didPerformDictionaryLookup(const String&, const DictionaryPopupInfo&);
 #endif
 
     // Spelling and grammar.
     int64_t spellDocumentTag();
+#if USE(UNIFIED_TEXT_CHECKING)
     void checkTextOfParagraph(const String& text, uint64_t checkingTypes, Vector<WebCore::TextCheckingResult>& results);
+#endif
+    void checkSpellingOfString(const String& text, int32_t& misspellingLocation, int32_t& misspellingLength);
+    void checkGrammarOfString(const String& text, Vector<WebCore::GrammarDetail>&, int32_t& badGrammarLocation, int32_t& badGrammarLength);
+    void spellingUIIsShowing(bool&);
     void updateSpellingUIWithMisspelledWord(const String& misspelledWord);
     void updateSpellingUIWithGrammarString(const String& badGrammarPhrase, const WebCore::GrammarDetail&);
     void getGuessesForWord(const String& word, const String& context, Vector<String>& guesses);
@@ -609,11 +743,13 @@ private:
     void ignoreWord(const String& word);
 
     void setFocus(bool focused);
-    void takeFocus(bool direction);
+    void takeFocus(uint32_t direction);
     void setToolTip(const String&);
     void setCursor(const WebCore::Cursor&);
+    void setCursorHiddenUntilMouseMoves(bool);
 
     void didReceiveEvent(uint32_t opaqueType, bool handled);
+    void stopResponsivenessTimer();
 
     void voidCallback(uint64_t);
     void dataCallback(const CoreIPC::DataReference&, uint64_t);
@@ -631,16 +767,37 @@ private:
     void didFinishLoadingDataForCustomRepresentation(const String& suggestedFilename, const CoreIPC::DataReference&);
 
 #if PLATFORM(MAC)
-    void setComplexTextInputEnabled(uint64_t pluginComplexTextInputIdentifier, bool complexTextInputEnabled);
+    void pluginFocusOrWindowFocusChanged(uint64_t pluginComplexTextInputIdentifier, bool pluginHasFocusAndWindowHasFocus);
+    void setPluginComplexTextInputState(uint64_t pluginComplexTextInputIdentifier, uint64_t complexTextInputState);
 #endif
 
-    static String standardUserAgent(const String& applicationName = String());
-
     void clearPendingAPIRequestURL() { m_pendingAPIRequestURL = String(); }
     void setPendingAPIRequestURL(const String& pendingAPIRequestURL) { m_pendingAPIRequestURL = pendingAPIRequestURL; }
 
     void initializeSandboxExtensionHandle(const WebCore::KURL&, SandboxExtension::Handle&);
 
+#if PLATFORM(MAC)
+    void substitutionsPanelIsShowing(bool&);
+#if !defined(BUILDING_ON_SNOW_LEOPARD)
+    void showCorrectionPanel(int32_t panelType, const WebCore::FloatRect& boundingBoxOfReplacedString, const String& replacedString, const String& replacementString, const Vector<String>& alternativeReplacementStrings);
+    void dismissCorrectionPanel(int32_t reason);
+    void dismissCorrectionPanelSoon(int32_t reason, String& result);
+    void recordAutocorrectionResponse(int32_t responseType, const String& replacedString, const String& replacementString);
+#endif // !defined(BUILDING_ON_SNOW_LEOPARD)
+#endif // PLATFORM(MAC)
+
+    void clearLoadDependentCallbacks();
+
+    void performDragControllerAction(DragControllerAction, WebCore::DragData*, const String& dragStorageName, const SandboxExtension::Handle&);
+
+    void updateBackingStoreDiscardableState();
+
+#if PLATFORM(WIN)
+    void scheduleChildWindowGeometryUpdate(const WindowGeometry&);
+#endif
+
+    void setRenderTreeSize(uint64_t treeSize) { m_renderTreeSize = treeSize; }
+
     PageClient* m_pageClient;
     WebLoaderClient m_loaderClient;
     WebPolicyClient m_policyClient;
@@ -651,7 +808,7 @@ private:
     WebPageContextMenuClient m_contextMenuClient;
 
     OwnPtr<DrawingAreaProxy> m_drawingArea;
-    RefPtr<WebContext> m_context;
+    RefPtr<WebProcessProxy> m_process;
     RefPtr<WebPageGroup> m_pageGroup;
     RefPtr<WebFrameProxy> m_mainFrame;
     RefPtr<WebFrameProxy> m_focusedFrame;
@@ -673,6 +830,7 @@ private:
     HashMap<uint64_t, RefPtr<VoidCallback> > m_voidCallbacks;
     HashMap<uint64_t, RefPtr<DataCallback> > m_dataCallbacks;
     HashMap<uint64_t, RefPtr<StringCallback> > m_stringCallbacks;
+    HashSet<uint64_t> m_loadDependentStringCallbackIDs;
     HashMap<uint64_t, RefPtr<ScriptValueCallback> > m_scriptValueCallbacks;
     HashMap<uint64_t, RefPtr<ComputedPagesCallback> > m_computedPagesCallbacks;
     HashMap<uint64_t, RefPtr<ValidateCommandCallback> > m_validateCommandCallbacks;
@@ -696,17 +854,18 @@ private:
     bool m_canGoBack;
     bool m_canGoForward;
     RefPtr<WebBackForwardList> m_backForwardList;
+    
+    bool m_maintainsInactiveSelection;
 
     String m_toolTip;
 
-    SelectionState m_selectionState;
-
-    // REMOVE: For demo purposes only.
-    String m_urlAtProcessExit;
+    EditorState m_editorState;
 
     double m_textZoomFactor;
     double m_pageZoomFactor;
-    double m_viewScaleFactor;
+    double m_pageScaleFactor;
+    float m_intrinsicDeviceScaleFactor;
+    float m_customDeviceScaleFactor;
 
     bool m_drawsBackground;
     bool m_drawsTransparentBackground;
@@ -736,11 +895,17 @@ private:
     uint64_t m_syncNavigationActionPolicyDownloadID;
 
     Deque<NativeWebKeyboardEvent> m_keyEventQueue;
-    bool m_processingWheelEvent;
-    OwnPtr<WebWheelEvent> m_nextWheelEvent;
+    Deque<NativeWebWheelEvent> m_wheelEventQueue;
+    Vector<NativeWebWheelEvent> m_currentlyProcessedWheelEvents;
 
     bool m_processingMouseMoveEvent;
-    OwnPtr<WebMouseEvent> m_nextMouseMoveEvent;
+    OwnPtr<NativeWebMouseEvent> m_nextMouseMoveEvent;
+    OwnPtr<NativeWebMouseEvent> m_currentlyProcessedMouseDownEvent;
+
+#if ENABLE(TOUCH_EVENTS)
+    bool m_needTouchEvents;
+    Deque<QueuedTouchEvents> m_touchEventQueue;
+#endif
 
     uint64_t m_pageID;
 
@@ -759,11 +924,18 @@ private:
 
     bool m_mainFrameHasHorizontalScrollbar;
     bool m_mainFrameHasVerticalScrollbar;
+    int m_wheelEventHandlerCount;
 
     bool m_mainFrameIsPinnedToLeftSide;
     bool m_mainFrameIsPinnedToRightSide;
 
+    WebCore::IntRect m_visibleScrollerThumbRect;
+
+    uint64_t m_renderTreeSize;
+
     static WKPageDebugPaintFlags s_debugPaintFlags;
+
+    bool m_shouldSendKeyboardEventSynchronously;
 };
 
 } // namespace WebKit
