@@ -32,6 +32,7 @@
 #include <WebCore/HTTPHeaderMap.h>
 #include <WebCore/IdentifierRep.h>
 #include <WebCore/NotImplemented.h>
+#include <WebCore/ProtectionSpace.h>
 #include <WebCore/SharedBuffer.h>
 #include <utility>
 
@@ -407,9 +408,12 @@ static NPError NPN_PostURLNotify(NPP npp, const char* url, const char* target, u
 }
 
 #if PLATFORM(MAC)
-// true if the browser supports hardware compositing of Core Animation plug-ins.
+// Whether the browser supports compositing of Core Animation plug-ins.
 static const unsigned WKNVSupportsCompositingCoreAnimationPluginsBool = 74656;
 
+// Whether the browser expects a non-retained Core Animation layer.
+static const unsigned WKNVExpectsNonretainedLayer = 74657;
+
 // The Core Animation render server port.
 static const unsigned WKNVCALayerRenderServerPort = 71879;
 
@@ -461,13 +465,36 @@ static NPError NPN_GetValue(NPP npp, NPNVariable variable, void *value)
             *(NPBool*)value = true;
             break;
 
+        case NPNVsupportsUpdatedCocoaTextInputBool: {
+            // The plug-in is asking whether we support the updated Cocoa text input model.
+            // If we haven't yet delivered a key down event to the plug-in, we can opt into the updated
+            // model and say that we support it. Otherwise, we'll just fall back and say that we don't support it.
+            RefPtr<NetscapePlugin> plugin = NetscapePlugin::fromNPP(npp);
+
+            bool supportsUpdatedTextInput = !plugin->hasHandledAKeyDownEvent();
+            if (supportsUpdatedTextInput)
+                plugin->setPluginWantsLegacyCocoaTextInput(false);
+
+            *reinterpret_cast<NPBool*>(value) = supportsUpdatedTextInput;
+            break;
+        }
+
         case WKNVCALayerRenderServerPort: {
             RefPtr<NetscapePlugin> plugin = NetscapePlugin::fromNPP(npp);
 
             *(mach_port_t*)value = plugin->compositingRenderServerPort();
             break;
         }
-        
+
+        case WKNVExpectsNonretainedLayer: {
+            RefPtr<NetscapePlugin> plugin = NetscapePlugin::fromNPP(npp);
+
+            // Asking for this will make us expect a non-retained layer from the plug-in.
+            plugin->setPluginReturnsNonretainedLayer(true);
+            *(NPBool*)value = true;
+            break;
+        }
+
 #ifndef NP_NO_QUICKDRAW
         case NPNVsupportsQuickDrawBool:
             // We don't support the QuickDraw drawing model.
@@ -476,8 +503,7 @@ static NPError NPN_GetValue(NPP npp, NPNVariable variable, void *value)
 #endif
 #ifndef NP_NO_CARBON
        case NPNVsupportsCarbonBool:
-            // FIXME: We should support the Carbon event model.
-            *(NPBool*)value = false;
+            *(NPBool*)value = true;
             break;
 #endif
 #elif PLATFORM(WIN)
@@ -490,7 +516,9 @@ static NPError NPN_GetValue(NPP npp, NPNVariable variable, void *value)
            *(NPBool*)value = true;
            break;
 #elif PLUGIN_ARCHITECTURE(X11)
-       case NPNVxDisplay:
+       case NPNVxDisplay: {
+           if (!npp)
+               return NPERR_GENERIC_ERROR;
 #if PLATFORM(QT)
            *reinterpret_cast<Display**>(value) = QX11Info::display();
            break;
@@ -500,6 +528,7 @@ static NPError NPN_GetValue(NPP npp, NPNVariable variable, void *value)
 #else
            goto default;
 #endif
+       }
        case NPNVSupportsXEmbedBool:
            *static_cast<NPBool*>(value) = true;
            break;
@@ -508,11 +537,23 @@ static NPError NPN_GetValue(NPP npp, NPNVariable variable, void *value)
            break;
 
        case NPNVToolkit: {
+#if PLATFORM(GTK)
+           *reinterpret_cast<uint32_t*>(value) = 2;
+#else
            const uint32_t expectedGTKToolKitVersion = 2;
 
-           RefPtr<NetscapePlugin> plugin = NetscapePlugin::fromNPP(npp);
-           *reinterpret_cast<uint32_t*>(value) = plugin->quirks().contains(PluginQuirks::RequiresGTKToolKit) ?
-                                                 expectedGTKToolKitVersion : 0;
+           // Set the expected GTK version if we know that this plugin needs it or if the plugin call us
+           // with a null instance. The latter is the case with NSPluginWrapper plugins.
+           bool requiresGTKToolKitVersion;
+           if (!npp)
+               requiresGTKToolKitVersion = true;
+           else {
+               RefPtr<NetscapePlugin> plugin = NetscapePlugin::fromNPP(npp);
+               requiresGTKToolKitVersion = plugin->quirks().contains(PluginQuirks::RequiresGTKToolKit);
+           }
+
+           *reinterpret_cast<uint32_t*>(value) = requiresGTKToolKitVersion ? expectedGTKToolKitVersion : 0;
+#endif
            break;
        }
 
@@ -551,7 +592,12 @@ static NPError NPN_SetValue(NPP npp, NPPVariable variable, void *value)
             return NPERR_NO_ERROR;
         }
 
-        case NPPVpluginTransparentBool:
+        case NPPVpluginTransparentBool: {
+            RefPtr<NetscapePlugin> plugin = NetscapePlugin::fromNPP(npp);
+            plugin->setIsTransparent(value);
+            return NPERR_NO_ERROR;
+        }
+
         default:
             notImplemented();
             return NPERR_GENERIC_ERROR;
@@ -644,8 +690,14 @@ static void NPN_ReleaseObject(NPObject *npObject)
     releaseNPObject(npObject);
 }
 
-static bool NPN_Invoke(NPP, NPObject *npObject, NPIdentifier methodName, const NPVariant* arguments, uint32_t argumentCount, NPVariant* result)
+static bool NPN_Invoke(NPP npp, NPObject *npObject, NPIdentifier methodName, const NPVariant* arguments, uint32_t argumentCount, NPVariant* result)
 {
+    if (RefPtr<NetscapePlugin> plugin = NetscapePlugin::fromNPP(npp)) {
+        bool returnValue;
+        if (plugin->tryToShortCircuitInvoke(npObject, methodName, arguments, argumentCount, returnValue, *result))
+            return returnValue;
+    }
+
     if (npObject->_class->invoke)
         return npObject->_class->invoke(npObject, methodName, arguments, argumentCount, result);
 
@@ -758,9 +810,11 @@ static bool NPN_Enumerate(NPP npp, NPObject* npObject, NPIdentifier** identifier
     return false;
 }
 
-static void NPN_PluginThreadAsyncCall(NPP instance, void (*func) (void*), void* userData)
+static void NPN_PluginThreadAsyncCall(NPP npp, void (*function)(void*), void* userData)
 {
-    notImplemented();
+    RefPtr<NetscapePlugin> plugin = NetscapePlugin::fromNPP(npp);
+
+    plugin->pluginThreadAsyncCall(function, userData);
 }
 
 static bool NPN_Construct(NPP npp, NPObject* npObject, const NPVariant* arguments, uint32_t argumentCount, NPVariant* result)
@@ -843,22 +897,71 @@ static NPError NPN_SetValueForURL(NPP npp, NPNURLVariable variable, const char*
     }
 }
 
-static NPError NPN_GetAuthenticationInfo(NPP instance, const char* protocol, const char* host, int32_t port, const char* scheme, 
-                                         const char* realm, char** username, uint32_t* ulen, char** password, uint32_t* plen)
+static bool initializeProtectionSpace(const char* protocol, const char* host, int port, const char* scheme, const char* realm, ProtectionSpace& protectionSpace)
 {
-    notImplemented();
-    return NPERR_GENERIC_ERROR;
+    ProtectionSpaceServerType serverType;
+    if (!strcasecmp(protocol, "http"))
+        serverType = ProtectionSpaceServerHTTP;
+    else if (!strcasecmp(protocol, "https"))
+        serverType = ProtectionSpaceServerHTTPS;
+    else {
+        // We only care about http and https.
+        return false;
+    }
+
+    ProtectionSpaceAuthenticationScheme authenticationScheme = ProtectionSpaceAuthenticationSchemeDefault;
+    if (serverType == ProtectionSpaceServerHTTP) {
+        if (!strcasecmp(scheme, "basic"))
+            authenticationScheme = ProtectionSpaceAuthenticationSchemeHTTPBasic;
+        else if (!strcmp(scheme, "digest"))
+            authenticationScheme = ProtectionSpaceAuthenticationSchemeHTTPDigest;
+    }
+
+    protectionSpace = ProtectionSpace(host, port, serverType, realm, authenticationScheme);
+    return true;
 }
 
-static uint32_t NPN_ScheduleTimer(NPP instance, uint32_t interval, NPBool repeat, void (*timerFunc)(NPP npp, uint32_t timerID))
+static NPError NPN_GetAuthenticationInfo(NPP npp, const char* protocol, const char* host, int32_t port, const char* scheme, 
+                                         const char* realm, char** username, uint32_t* usernameLength, char** password, uint32_t* passwordLength)
 {
-    notImplemented();
-    return NPERR_GENERIC_ERROR;
+    if (!protocol || !host || !scheme || !realm || !username || !usernameLength || !password || !passwordLength)
+        return NPERR_GENERIC_ERROR;
+
+    ProtectionSpace protectionSpace;
+    if (!initializeProtectionSpace(protocol, host, port, scheme, realm, protectionSpace))
+        return NPERR_GENERIC_ERROR;
+
+    RefPtr<NetscapePlugin> plugin = NetscapePlugin::fromNPP(npp);
+    String usernameString;
+    String passwordString;
+    if (!plugin->getAuthenticationInfo(protectionSpace, usernameString, passwordString))
+        return NPERR_GENERIC_ERROR;
+
+    NPError result = copyCString(usernameString.utf8(), username, usernameLength);
+    if (result != NPERR_NO_ERROR)
+        return result;
+
+    result = copyCString(passwordString.utf8(), password, passwordLength);
+    if (result != NPERR_NO_ERROR) {
+        npnMemFree(*username);
+        return result;
+    }
+
+    return NPERR_NO_ERROR;
 }
 
-static void NPN_UnscheduleTimer(NPP instance, uint32_t timerID)
+static uint32_t NPN_ScheduleTimer(NPP npp, uint32_t interval, NPBool repeat, void (*timerFunc)(NPP npp, uint32_t timerID))
 {
-    notImplemented();
+    RefPtr<NetscapePlugin> plugin = NetscapePlugin::fromNPP(npp);
+
+    return plugin->scheduleTimer(interval, repeat, timerFunc);
+}
+
+static void NPN_UnscheduleTimer(NPP npp, uint32_t timerID)
+{
+    RefPtr<NetscapePlugin> plugin = NetscapePlugin::fromNPP(npp);
+
+    plugin->unscheduleTimer(timerID);
 }
 
 #if PLATFORM(MAC)
