@@ -27,9 +27,11 @@
 #include "NetscapePlugin.h"
 
 #include "NPRuntimeObjectMap.h"
+#include "NPRuntimeUtilities.h"
 #include "NetscapePluginStream.h"
 #include "PluginController.h"
 #include "ShareableBitmap.h"
+#include "WorkItem.h"
 #include <WebCore/GraphicsContext.h>
 #include <WebCore/HTTPHeaderMap.h>
 #include <WebCore/IntRect.h>
@@ -54,8 +56,7 @@ PassRefPtr<NetscapePlugin> NetscapePlugin::create(PassRefPtr<NetscapePluginModul
 }
     
 NetscapePlugin::NetscapePlugin(PassRefPtr<NetscapePluginModule> pluginModule)
-    : m_pluginController(0)
-    , m_nextRequestID(0)
+    : m_nextRequestID(0)
     , m_pluginModule(pluginModule)
     , m_npWindow()
     , m_isStarted(false)
@@ -64,14 +65,21 @@ NetscapePlugin::NetscapePlugin(PassRefPtr<NetscapePluginModule> pluginModule)
 #else
     , m_isWindowed(true)
 #endif
+    , m_isTransparent(false)
     , m_inNPPNew(false)
     , m_loadManually(false)
+    , m_nextTimerID(0)
 #if PLATFORM(MAC)
     , m_drawingModel(static_cast<NPDrawingModel>(-1))
     , m_eventModel(static_cast<NPEventModel>(-1))
+    , m_pluginReturnsNonretainedLayer(!m_pluginModule->pluginQuirks().contains(PluginQuirks::ReturnsRetainedCoreAnimationLayer))
     , m_currentMouseEvent(0)
     , m_pluginHasFocus(false)
     , m_windowHasFocus(false)
+    , m_pluginWantsLegacyCocoaTextInput(true)
+    , m_isComplexTextInputEnabled(false)
+    , m_hasHandledAKeyDownEvent(false)
+    , m_ignoreNextKeyUpEventCounter(0)
 #ifndef NP_NO_CARBON
     , m_nullEventTimer(RunLoop::main(), this, &NetscapePlugin::nullEventTimerFired)
     , m_npCGContext()
@@ -90,6 +98,7 @@ NetscapePlugin::NetscapePlugin(PassRefPtr<NetscapePluginModule> pluginModule)
 NetscapePlugin::~NetscapePlugin()
 {
     ASSERT(!m_isStarted);
+    ASSERT(m_timers.isEmpty());
 
     m_pluginModule->decrementLoadCount();
 }
@@ -117,7 +126,7 @@ void NetscapePlugin::invalidate(const NPRect* invalidRect)
     if (platformInvalidate(rect))
         return;
 
-    m_pluginController->invalidate(rect);
+    controller()->invalidate(rect);
 }
 
 const char* NetscapePlugin::userAgent(NPP npp)
@@ -133,9 +142,23 @@ const char* NetscapePlugin::userAgent(NPP npp)
 
 const char* NetscapePlugin::userAgent()
 {
+#if PLUGIN_ARCHITECTURE(WIN)
+    static const char* MozillaUserAgent = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1) Gecko/20061010 Firefox/2.0";
+    
+    if (quirks().contains(PluginQuirks::WantsMozillaUserAgent))
+        return MozillaUserAgent;
+#endif
+
     if (m_userAgent.isNull()) {
-        m_userAgent = m_pluginController->userAgent().utf8();
-        ASSERT(!m_userAgent.isNull());
+        String userAgent = controller()->userAgent();
+        ASSERT(!userAgent.isNull());
+
+#if PLUGIN_ARCHITECTURE(MAC)
+        if (quirks().contains(PluginQuirks::AppendVersion3UserAgent))
+            userAgent += " Version/3.2.1";
+#endif
+
+        m_userAgent = userAgent.utf8();
     }
     return m_userAgent.data();
 }
@@ -145,11 +168,11 @@ void NetscapePlugin::loadURL(const String& method, const String& urlString, cons
 {
     uint64_t requestID = ++m_nextRequestID;
     
-    m_pluginController->loadURL(requestID, method, urlString, target, headerFields, httpBody, allowPopups());
+    controller()->loadURL(requestID, method, urlString, target, headerFields, httpBody, allowPopups());
 
     if (target.isNull()) {
         // The browser is going to send the data in a stream, create a plug-in stream.
-        RefPtr<NetscapePluginStream> pluginStream = NetscapePluginStream::create(this, requestID, sendNotification, notificationData);
+        RefPtr<NetscapePluginStream> pluginStream = NetscapePluginStream::create(this, requestID, urlString, sendNotification, notificationData);
         ASSERT(!m_streams.contains(requestID));
 
         m_streams.set(requestID, pluginStream.release());
@@ -192,47 +215,64 @@ void NetscapePlugin::setIsWindowed(bool isWindowed)
     m_isWindowed = isWindowed;
 }
 
+void NetscapePlugin::setIsTransparent(bool isTransparent)
+{
+    m_isTransparent = isTransparent;
+}
+
 void NetscapePlugin::setStatusbarText(const String& statusbarText)
 {
-    m_pluginController->setStatusbarText(statusbarText);
+    controller()->setStatusbarText(statusbarText);
+}
+
+static void (*setExceptionFunction)(const String&);
+
+void NetscapePlugin::setSetExceptionFunction(void (*function)(const String&))
+{
+    ASSERT(!setExceptionFunction || setExceptionFunction == function);
+    setExceptionFunction = function;
 }
 
 void NetscapePlugin::setException(const String& exceptionString)
 {
-    // FIXME: If the plug-in is running in its own process, this needs to send a CoreIPC message instead of
-    // calling the runtime object map directly.
-    NPRuntimeObjectMap::setGlobalException(exceptionString);
+    ASSERT(setExceptionFunction);
+    setExceptionFunction(exceptionString);
 }
 
 bool NetscapePlugin::evaluate(NPObject* npObject, const String& scriptString, NPVariant* result)
 {
-    return m_pluginController->evaluate(npObject, scriptString, result, allowPopups());
+    return controller()->evaluate(npObject, scriptString, result, allowPopups());
 }
 
 bool NetscapePlugin::isPrivateBrowsingEnabled()
 {
-    return m_pluginController->isPrivateBrowsingEnabled();
+    return controller()->isPrivateBrowsingEnabled();
 }
 
 NPObject* NetscapePlugin::windowScriptNPObject()
 {
-    return m_pluginController->windowScriptNPObject();
+    return controller()->windowScriptNPObject();
 }
 
 NPObject* NetscapePlugin::pluginElementNPObject()
 {
-    return m_pluginController->pluginElementNPObject();
+    return controller()->pluginElementNPObject();
+}
+
+bool NetscapePlugin::tryToShortCircuitInvoke(NPObject* npObject, NPIdentifier methodName, const NPVariant* arguments, uint32_t argumentCount, bool& returnValue, NPVariant& result)
+{
+    return controller()->tryToShortCircuitInvoke(npObject, methodName, arguments, argumentCount, returnValue, result);
 }
 
 void NetscapePlugin::cancelStreamLoad(NetscapePluginStream* pluginStream)
 {
     if (pluginStream == m_manualStream) {
-        m_pluginController->cancelManualStreamLoad();
+        controller()->cancelManualStreamLoad();
         return;
     }
 
     // Ask the plug-in controller to cancel this stream load.
-    m_pluginController->cancelStreamLoad(pluginStream->streamID());
+    controller()->cancelStreamLoad(pluginStream->streamID());
 }
 
 void NetscapePlugin::removePluginStream(NetscapePluginStream* pluginStream)
@@ -249,7 +289,7 @@ void NetscapePlugin::removePluginStream(NetscapePluginStream* pluginStream)
 bool NetscapePlugin::isAcceleratedCompositingEnabled()
 {
 #if USE(ACCELERATED_COMPOSITING)
-    return m_pluginController->isAcceleratedCompositingEnabled();
+    return controller()->isAcceleratedCompositingEnabled();
 #else
     return false;
 #endif
@@ -267,19 +307,108 @@ void NetscapePlugin::popPopupsEnabledState()
     m_popupEnabledStates.removeLast();
 }
 
+void NetscapePlugin::pluginThreadAsyncCall(void (*function)(void*), void* userData)
+{
+    RunLoop::main()->scheduleWork(WorkItem::create(this, &NetscapePlugin::handlePluginThreadAsyncCall, function, userData));
+}
+    
+void NetscapePlugin::handlePluginThreadAsyncCall(void (*function)(void*), void* userData)
+{
+    if (!m_isStarted)
+        return;
+
+    function(userData);
+}
+
+PassOwnPtr<NetscapePlugin::Timer> NetscapePlugin::Timer::create(NetscapePlugin* netscapePlugin, unsigned timerID, unsigned interval, bool repeat, TimerFunc timerFunc)
+{
+    return adoptPtr(new Timer(netscapePlugin, timerID, interval, repeat, timerFunc));
+}
+
+NetscapePlugin::Timer::Timer(NetscapePlugin* netscapePlugin, unsigned timerID, unsigned interval, bool repeat, TimerFunc timerFunc)
+    : m_netscapePlugin(netscapePlugin)
+    , m_timerID(timerID)
+    , m_interval(interval)
+    , m_repeat(repeat)
+    , m_timerFunc(timerFunc)
+    , m_timer(RunLoop::main(), this, &Timer::timerFired)
+{
+}
+
+NetscapePlugin::Timer::~Timer()
+{
+}
+
+void NetscapePlugin::Timer::start()
+{
+    double timeInterval = m_interval / 1000.0;
+
+    if (m_repeat)
+        m_timer.startRepeating(timeInterval);
+    else
+        m_timer.startOneShot(timeInterval);
+}
+
+void NetscapePlugin::Timer::stop()
+{
+    m_timer.stop();
+}
+
+void NetscapePlugin::Timer::timerFired()
+{
+    m_timerFunc(&m_netscapePlugin->m_npp, m_timerID);
+
+    if (!m_repeat)
+        m_netscapePlugin->unscheduleTimer(m_timerID);
+}
+
+uint32_t NetscapePlugin::scheduleTimer(unsigned interval, bool repeat, void (*timerFunc)(NPP, unsigned timerID))
+{
+    if (!timerFunc)
+        return 0;
+
+    // FIXME: Handle wrapping around.
+    unsigned timerID = ++m_nextTimerID;
+
+    OwnPtr<Timer> timer = Timer::create(this, timerID, interval, repeat, timerFunc);
+    
+    // FIXME: Based on the plug-in visibility, figure out if we should throttle the timer, or if we should start it at all.
+    timer->start();
+    m_timers.set(timerID, timer.leakPtr());
+
+    return timerID;
+}
+
+void NetscapePlugin::unscheduleTimer(unsigned timerID)
+{
+    TimerMap::iterator it = m_timers.find(timerID);
+    if (it == m_timers.end())
+        return;
+
+    OwnPtr<Timer> timer = adoptPtr(it->second);
+    m_timers.remove(it);
+
+    timer->stop();
+}
+
 String NetscapePlugin::proxiesForURL(const String& urlString)
 {
-    return m_pluginController->proxiesForURL(urlString);
+    return controller()->proxiesForURL(urlString);
 }
     
 String NetscapePlugin::cookiesForURL(const String& urlString)
 {
-    return m_pluginController->cookiesForURL(urlString);
+    return controller()->cookiesForURL(urlString);
 }
 
 void NetscapePlugin::setCookiesForURL(const String& urlString, const String& cookieString)
 {
-    m_pluginController->setCookiesForURL(urlString, cookieString);
+    controller()->setCookiesForURL(urlString, cookieString);
+}
+
+bool NetscapePlugin::getAuthenticationInfo(const ProtectionSpace& protectionSpace, String& username, String& password)
+{
+    return controller()->getAuthenticationInfo(protectionSpace, username, password);
 }
 
 NPError NetscapePlugin::NPP_New(NPMIMEType pluginType, uint16_t mode, int16_t argc, char* argn[], char* argv[], NPSavedData* savedData)
@@ -406,13 +535,8 @@ bool NetscapePlugin::allowPopups() const
     return false;
 }
 
-bool NetscapePlugin::initialize(PluginController* pluginController, const Parameters& parameters)
+bool NetscapePlugin::initialize(const Parameters& parameters)
 {
-    ASSERT(!m_pluginController);
-    ASSERT(pluginController);
-
-    m_pluginController = pluginController;
-    
     uint16_t mode = parameters.loadManually ? NP_FULL : NP_EMBED;
     
     m_loadManually = parameters.loadManually;
@@ -424,7 +548,14 @@ bool NetscapePlugin::initialize(PluginController* pluginController, const Parame
     Vector<CString> paramNames;
     Vector<CString> paramValues;
     for (size_t i = 0; i < parameters.names.size(); ++i) {
-        paramNames.append(parameters.names[i].utf8());
+        String parameterName = parameters.names[i];
+
+#if PLUGIN_ARCHITECTURE(MAC)
+        if (m_pluginModule->pluginQuirks().contains(PluginQuirks::WantsLowercaseParameterNames))
+            parameterName = parameterName.lower();
+#endif
+
+        paramNames.append(parameterName.utf8());
         paramValues.append(parameters.values[i].utf8());
     }
 
@@ -436,6 +567,17 @@ bool NetscapePlugin::initialize(PluginController* pluginController, const Parame
         values.append(paramValues[i].data());
     }
 
+#if PLUGIN_ARCHITECTURE(MAC)
+    if (m_pluginModule->pluginQuirks().contains(PluginQuirks::MakeTransparentIfBackgroundAttributeExists)) {
+        for (size_t i = 0; i < parameters.names.size(); ++i) {
+            if (equalIgnoringCase(parameters.names[i], "background")) {
+                setIsTransparent(true);
+                break;
+            }
+        }
+    }
+#endif
+
     NetscapePlugin* previousNPPNewPlugin = currentNPPNewPlugin;
     
     m_inNPPNew = true;
@@ -474,12 +616,19 @@ void NetscapePlugin::destroy()
     // Stop all streams.
     stopAllStreams();
 
+#if !PLUGIN_ARCHITECTURE(MAC) && !PLUGIN_ARCHITECTURE(X11)
+    m_npWindow.window = 0;
+    callSetWindow();
+#endif
+
     NPP_Destroy(0);
 
     m_isStarted = false;
-    m_pluginController = 0;
 
     platformDestroy();
+
+    deleteAllValues(m_timers);
+    m_timers.clear();
 }
     
 void NetscapePlugin::paint(GraphicsContext* context, const IntRect& dirtyRect)
@@ -496,7 +645,7 @@ PassRefPtr<ShareableBitmap> NetscapePlugin::snapshot()
 
     ASSERT(m_isStarted);
     
-    RefPtr<ShareableBitmap> bitmap = ShareableBitmap::createShareable(m_frameRect.size());
+    RefPtr<ShareableBitmap> bitmap = ShareableBitmap::createShareable(m_frameRect.size(), ShareableBitmap::SupportsAlpha);
     OwnPtr<GraphicsContext> context = bitmap->createGraphicsContext();
 
     context->translate(-m_frameRect.x(), -m_frameRect.y());
@@ -506,6 +655,11 @@ PassRefPtr<ShareableBitmap> NetscapePlugin::snapshot()
     return bitmap.release();
 }
 
+bool NetscapePlugin::isTransparent()
+{
+    return m_isTransparent;
+}
+
 void NetscapePlugin::geometryDidChange(const IntRect& frameRect, const IntRect& clipRect)
 {
     ASSERT(m_isStarted);
@@ -522,6 +676,13 @@ void NetscapePlugin::geometryDidChange(const IntRect& frameRect, const IntRect&
     callSetWindow();
 }
 
+void NetscapePlugin::visibilityDidChange()
+{
+    ASSERT(m_isStarted);
+
+    platformVisibilityDidChange();
+}
+
 void NetscapePlugin::frameDidFinishLoading(uint64_t requestID)
 {
     ASSERT(m_isStarted);
@@ -554,12 +715,12 @@ void NetscapePlugin::frameDidFail(uint64_t requestID, bool wasCancelled)
     NPP_URLNotify(url.utf8().data(), wasCancelled ? NPRES_USER_BREAK : NPRES_NETWORK_ERR, notificationData);
 }
 
-void NetscapePlugin::didEvaluateJavaScript(uint64_t requestID, const String& requestURLString, const String& result)
+void NetscapePlugin::didEvaluateJavaScript(uint64_t requestID, const String& result)
 {
     ASSERT(m_isStarted);
     
     if (NetscapePluginStream* pluginStream = streamFromID(requestID))
-        pluginStream->sendJavaScriptStream(requestURLString, result);
+        pluginStream->sendJavaScriptStream(result);
 }
 
 void NetscapePlugin::streamDidReceiveResponse(uint64_t streamID, const KURL& responseURL, uint32_t streamLength, 
@@ -602,7 +763,7 @@ void NetscapePlugin::manualStreamDidReceiveResponse(const KURL& responseURL, uin
     ASSERT(m_loadManually);
     ASSERT(!m_manualStream);
     
-    m_manualStream = NetscapePluginStream::create(this, 0, false, 0);
+    m_manualStream = NetscapePluginStream::create(this, 0, responseURL.string(), false, 0);
     m_manualStream->didReceiveResponse(responseURL, streamLength, lastModifiedTime, mimeType, headers);
 }
 
@@ -682,7 +843,12 @@ NPObject* NetscapePlugin::pluginScriptableNPObject()
     
     if (NPP_GetValue(NPPVpluginScriptableNPObject, &scriptableNPObject) != NPERR_NO_ERROR)
         return 0;
-    
+
+#if PLUGIN_ARCHITECTURE(MAC)
+    if (m_pluginModule->pluginQuirks().contains(PluginQuirks::ReturnsNonRetainedScriptableNPObject))
+        retainNPObject(scriptableNPObject);        
+#endif    
+
     return scriptableNPObject;
 }
 
@@ -699,6 +865,21 @@ void NetscapePlugin::privateBrowsingStateChanged(bool privateBrowsingEnabled)
     NPP_SetValue(NPNVprivateModeBool, &value);
 }
 
+bool NetscapePlugin::getFormValue(String& formValue)
+{
+    ASSERT(m_isStarted);
+
+    char* formValueString = 0;
+    if (NPP_GetValue(NPPVformValue, &formValueString) != NPERR_NO_ERROR)
+        return false;
+
+    formValue = String::fromUTF8(formValueString);
+
+    // The plug-in allocates the form value string with NPN_MemAlloc so it needs to be freed with NPN_MemFree.
+    npnMemFree(formValueString);
+    return true;
+}
+
 bool NetscapePlugin::supportsSnapshotting() const
 {
 #if PLATFORM(MAC)
@@ -707,9 +888,4 @@ bool NetscapePlugin::supportsSnapshotting() const
     return false;
 }
 
-PluginController* NetscapePlugin::controller()
-{
-    return m_pluginController;
-}
-
 } // namespace WebKit
