@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010 Apple Inc. All rights reserved.
+ * Copyright (C) 2010, 2011 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -44,7 +44,6 @@
 #include "WebPage.h"
 #include "WebPageProxyMessages.h"
 #include "WebProcess.h"
-#include "WebProcessProxyMessageKinds.h"
 #include "WebProcessProxyMessages.h"
 #include <JavaScriptCore/APICast.h>
 #include <JavaScriptCore/JSObject.h>
@@ -223,14 +222,14 @@ void WebFrameLoaderClient::dispatchDidReceiveResponse(DocumentLoader*, unsigned
     webPage->send(Messages::WebPageProxy::DidReceiveResponseForResource(m_frame->frameID(), identifier, response));
 }
 
-void WebFrameLoaderClient::dispatchDidReceiveContentLength(DocumentLoader*, unsigned long identifier, int lengthReceived)
+void WebFrameLoaderClient::dispatchDidReceiveContentLength(DocumentLoader*, unsigned long identifier, int dataLength)
 {
     WebPage* webPage = m_frame->page();
     if (!webPage)
         return;
 
-    webPage->injectedBundleResourceLoadClient().didReceiveContentLengthForResource(webPage, m_frame, identifier, lengthReceived);
-    webPage->send(Messages::WebPageProxy::DidReceiveContentLengthForResource(m_frame->frameID(), identifier, lengthReceived));
+    webPage->injectedBundleResourceLoadClient().didReceiveContentLengthForResource(webPage, m_frame, identifier, dataLength);
+    webPage->send(Messages::WebPageProxy::DidReceiveContentLengthForResource(m_frame->frameID(), identifier, dataLength));
 }
 
 void WebFrameLoaderClient::dispatchDidFinishLoading(DocumentLoader*, unsigned long identifier)
@@ -386,8 +385,10 @@ void WebFrameLoaderClient::dispatchDidStartProvisionalLoad()
     WebPage* webPage = m_frame->page();
     if (!webPage)
         return;
+
     webPage->findController().hideFindUI();
-    
+    webPage->sandboxExtensionTracker().didStartProvisionalLoad(m_frame);
+
     DocumentLoader* provisionalLoader = m_frame->coreFrame()->loader()->provisionalDocumentLoader();
     const String& url = provisionalLoader->url().string();
     RefPtr<APIObject> userData;
@@ -395,15 +396,13 @@ void WebFrameLoaderClient::dispatchDidStartProvisionalLoad()
     // Notify the bundle client.
     webPage->injectedBundleLoaderClient().didStartProvisionalLoadForFrame(webPage, m_frame, userData);
 
-    bool loadingSubstituteDataForUnreachableURL = !provisionalLoader->unreachableURL().isNull();
-
-    webPage->sandboxExtensionTracker().didStartProvisionalLoad(m_frame);
+    String unreachableURL = provisionalLoader->unreachableURL().string();
 
     // Notify the UIProcess.
-    webPage->send(Messages::WebPageProxy::DidStartProvisionalLoadForFrame(m_frame->frameID(), url, loadingSubstituteDataForUnreachableURL, InjectedBundleUserMessageEncoder(userData.get())));
+    webPage->send(Messages::WebPageProxy::DidStartProvisionalLoadForFrame(m_frame->frameID(), url, unreachableURL, InjectedBundleUserMessageEncoder(userData.get())));
 }
 
-void WebFrameLoaderClient::dispatchDidReceiveTitle(const String& title)
+void WebFrameLoaderClient::dispatchDidReceiveTitle(const StringWithDirection& title)
 {
     WebPage* webPage = m_frame->page();
     if (!webPage)
@@ -412,13 +411,14 @@ void WebFrameLoaderClient::dispatchDidReceiveTitle(const String& title)
     RefPtr<APIObject> userData;
 
     // Notify the bundle client.
-    webPage->injectedBundleLoaderClient().didReceiveTitleForFrame(webPage, title, m_frame, userData);
+    // FIXME: use direction of title.
+    webPage->injectedBundleLoaderClient().didReceiveTitleForFrame(webPage, title.string(), m_frame, userData);
 
     // Notify the UIProcess.
-    webPage->send(Messages::WebPageProxy::DidReceiveTitleForFrame(m_frame->frameID(), title, InjectedBundleUserMessageEncoder(userData.get())));
+    webPage->send(Messages::WebPageProxy::DidReceiveTitleForFrame(m_frame->frameID(), title.string(), InjectedBundleUserMessageEncoder(userData.get())));
 }
 
-void WebFrameLoaderClient::dispatchDidChangeIcons()
+void WebFrameLoaderClient::dispatchDidChangeIcons(WebCore::IconType)
 {
     notImplemented();
 }
@@ -443,8 +443,9 @@ void WebFrameLoaderClient::dispatchDidCommitLoad()
 
     // Only restore the scale factor for standard frame loads (of the main frame).
     if (m_frame->isMainFrame() && m_frame->coreFrame()->loader()->loadType() == FrameLoadTypeStandard) {
-        if (m_frame->coreFrame()->pageScaleFactor() != 1)
-            webPage->scaleWebView(1, IntPoint());
+        Page* page = m_frame->coreFrame()->page();
+        if (page && page->pageScaleFactor() != 1)
+            webPage->scalePage(1, IntPoint());
     }
 }
 
@@ -535,6 +536,9 @@ void WebFrameLoaderClient::dispatchDidFirstLayout()
 
     // Notify the UIProcess.
     webPage->send(Messages::WebPageProxy::DidFirstLayoutForFrame(m_frame->frameID(), InjectedBundleUserMessageEncoder(userData.get())));
+
+    if (m_frame == m_frame->page()->mainFrame())
+        webPage->drawingArea()->setLayerTreeStateIsFrozen(false);
 }
 
 void WebFrameLoaderClient::dispatchDidFirstVisuallyNonEmptyLayout()
@@ -552,6 +556,23 @@ void WebFrameLoaderClient::dispatchDidFirstVisuallyNonEmptyLayout()
     webPage->send(Messages::WebPageProxy::DidFirstVisuallyNonEmptyLayoutForFrame(m_frame->frameID(), InjectedBundleUserMessageEncoder(userData.get())));
 }
 
+void WebFrameLoaderClient::dispatchDidLayout()
+{
+    WebPage* webPage = m_frame->page();
+    if (!webPage)
+        return;
+
+    // Notify the bundle client.
+    webPage->injectedBundleLoaderClient().didLayoutForFrame(webPage, m_frame);
+
+    // NOTE: Unlike the other layout notifications, this does not notify the
+    // the UIProcess for every call.
+
+    // FIXME: Remove at the soonest possible time.
+    if (m_frame == m_frame->page()->mainFrame())
+        webPage->send(Messages::WebPageProxy::SetRenderTreeSize(webPage->renderTreeSize()));
+}
+
 Frame* WebFrameLoaderClient::dispatchCreatePage(const NavigationAction& navigationAction)
 {
     WebPage* webPage = m_frame->page();
@@ -581,8 +602,10 @@ void WebFrameLoaderClient::dispatchDecidePolicyForResponse(FramePolicyFunction f
     if (!webPage)
         return;
 
-    if (!request.url().string())
+    if (!request.url().string()) {
+        (m_frame->coreFrame()->loader()->policyChecker()->*function)(PolicyUse);
         return;
+    }
 
     RefPtr<APIObject> userData;
 
@@ -767,7 +790,7 @@ void WebFrameLoaderClient::setMainFrameDocumentReady(bool)
     notImplemented();
 }
 
-void WebFrameLoaderClient::startDownload(const ResourceRequest& request)
+void WebFrameLoaderClient::startDownload(const ResourceRequest& request, const String& /* suggestedName */)
 {
     m_frame->startDownload(request);
 }
@@ -839,14 +862,16 @@ void WebFrameLoaderClient::finishedLoading(DocumentLoader* loader)
 void WebFrameLoaderClient::updateGlobalHistory()
 {
     WebPage* webPage = m_frame->page();
-    if (!webPage)
+    if (!webPage || !webPage->pageGroup()->isVisibleToHistoryClient())
         return;
 
     DocumentLoader* loader = m_frame->coreFrame()->loader()->documentLoader();
 
     WebNavigationDataStore data;
     data.url = loader->urlForHistory().string();
-    data.title = loader->title();
+    // FIXME: use direction of title.
+    data.title = loader->title().string();
+    data.originalRequest = loader->originalRequestCopy();
 
     WebProcess::shared().connection()->send(Messages::WebContext::DidNavigateWithNavigationData(webPage->pageID(), data, m_frame->frameID()), 0);
 }
@@ -854,7 +879,7 @@ void WebFrameLoaderClient::updateGlobalHistory()
 void WebFrameLoaderClient::updateGlobalHistoryRedirectLinks()
 {
     WebPage* webPage = m_frame->page();
-    if (!webPage)
+    if (!webPage || !webPage->pageGroup()->isVisibleToHistoryClient())
         return;
 
     DocumentLoader* loader = m_frame->coreFrame()->loader()->documentLoader();
@@ -898,21 +923,6 @@ bool WebFrameLoaderClient::shouldStopLoadingForHistoryItem(HistoryItem* item) co
     return true;
 }
 
-void WebFrameLoaderClient::dispatchDidAddBackForwardItem(HistoryItem*) const
-{
-    notImplemented();
-}
-
-void WebFrameLoaderClient::dispatchDidRemoveBackForwardItem(HistoryItem*) const
-{
-    notImplemented();
-}
-
-void WebFrameLoaderClient::dispatchDidChangeBackForwardIndex() const
-{
-    notImplemented();
-}
-
 void WebFrameLoaderClient::didDisplayInsecureContent()
 {
     WebPage* webPage = m_frame->page();
@@ -954,9 +964,9 @@ ResourceError WebFrameLoaderClient::cannotShowURLError(const ResourceRequest& re
     return WebKit::cannotShowURLError(request);
 }
 
-ResourceError WebFrameLoaderClient::interruptForPolicyChangeError(const ResourceRequest& request)
+ResourceError WebFrameLoaderClient::interruptedForPolicyChangeError(const ResourceRequest& request)
 {
-    return WebKit::interruptForPolicyChangeError(request);
+    return WebKit::interruptedForPolicyChangeError(request);
 }
 
 ResourceError WebFrameLoaderClient::cannotShowMIMETypeError(const ResourceResponse& response)
@@ -1019,7 +1029,12 @@ String WebFrameLoaderClient::generatedMIMETypeForURLScheme(const String& URLSche
 
 void WebFrameLoaderClient::frameLoadCompleted()
 {
-    notImplemented();
+    WebPage* webPage = m_frame->page();
+    if (!webPage)
+        return;
+
+    if (m_frame == m_frame->page()->mainFrame())
+        webPage->drawingArea()->setLayerTreeStateIsFrozen(false);
 }
 
 void WebFrameLoaderClient::saveViewStateToItem(HistoryItem*)
@@ -1031,7 +1046,7 @@ void WebFrameLoaderClient::restoreViewState()
 {
     // Inform the UI process of the scale factor.
     double scaleFactor = m_frame->coreFrame()->loader()->history()->currentItem()->pageScaleFactor();
-    m_frame->page()->send(Messages::WebPageProxy::ViewScaleFactorDidChange(scaleFactor));
+    m_frame->page()->send(Messages::WebPageProxy::PageScaleFactorDidChange(scaleFactor));
 
     // FIXME: This should not be necessary. WebCore should be correctly invalidating
     // the view on restores from the back/forward cache.
@@ -1041,7 +1056,12 @@ void WebFrameLoaderClient::restoreViewState()
 
 void WebFrameLoaderClient::provisionalLoadStarted()
 {
-    notImplemented();
+    WebPage* webPage = m_frame->page();
+    if (!webPage)
+        return;
+
+    if (m_frame == m_frame->page()->mainFrame())
+        webPage->drawingArea()->setLayerTreeStateIsFrozen(true);
 }
 
 void WebFrameLoaderClient::didFinishLoad()
@@ -1061,14 +1081,15 @@ PassRefPtr<DocumentLoader> WebFrameLoaderClient::createDocumentLoader(const Reso
     return DocumentLoader::create(request, data);
 }
 
-void WebFrameLoaderClient::setTitle(const String& title, const KURL& url)
+void WebFrameLoaderClient::setTitle(const StringWithDirection& title, const KURL& url)
 {
     WebPage* webPage = m_frame->page();
-    if (!webPage)
+    if (!webPage || !webPage->pageGroup()->isVisibleToHistoryClient())
         return;
 
+    // FIXME: use direction of title.
     WebProcess::shared().connection()->send(Messages::WebContext::DidUpdateHistoryTitle(webPage->pageID(),
-        title, url.string(), m_frame->frameID()), 0);
+        title.string(), url.string(), m_frame->frameID()), 0);
 }
 
 String WebFrameLoaderClient::userAgent(const KURL&)
@@ -1086,6 +1107,11 @@ void WebFrameLoaderClient::savePlatformDataToCachedFrame(CachedFrame*)
 
 void WebFrameLoaderClient::transitionToCommittedFromCachedFrame(CachedFrame*)
 {
+    WebPage* webPage = m_frame->page();
+    bool isMainFrame = webPage->mainFrame() == m_frame;
+    
+    const ResourceResponse& response = m_frame->coreFrame()->loader()->documentLoader()->response();
+    m_frameHasCustomRepresentation = isMainFrame && WebProcess::shared().shouldUseCustomRepresentationForResponse(response);
 }
 
 void WebFrameLoaderClient::transitionToCommittedForNewPage()
@@ -1096,19 +1122,19 @@ void WebFrameLoaderClient::transitionToCommittedForNewPage()
     bool isMainFrame = webPage->mainFrame() == m_frame;
 
 #if ENABLE(TILED_BACKING_STORE)
-    IntSize currentVisibleContentSize = m_frame->coreFrame()->view() ? m_frame->coreFrame()->view()->actualVisibleContentRect().size() : IntSize();
+    IntSize currentVisibleContentSize = m_frame->coreFrame()->view() ? m_frame->coreFrame()->view()->visibleContentRect().size() : IntSize();
     m_frame->coreFrame()->createView(webPage->size(), backgroundColor, false, webPage->resizesToContentsLayoutSize(), isMainFrame && webPage->resizesToContentsEnabled());
 
     if (isMainFrame && webPage->resizesToContentsEnabled()) {
         m_frame->coreFrame()->view()->setDelegatesScrolling(true);
         m_frame->coreFrame()->view()->setPaintsEntireContents(true);
+        // The HistoryController will update the scroll position later if needed.
+        m_frame->coreFrame()->view()->setFixedVisibleContentRect(IntRect(IntPoint::zero(), currentVisibleContentSize));
     }
 
-    // The HistoryController will update the scroll position later if needed.
-    m_frame->coreFrame()->view()->setActualVisibleContentRect(IntRect(IntPoint::zero(), currentVisibleContentSize));
 #else
-    const String& mimeType = m_frame->coreFrame()->loader()->documentLoader()->response().mimeType();
-    m_frameHasCustomRepresentation = isMainFrame && WebProcess::shared().shouldUseCustomRepresentationForMIMEType(mimeType);
+    const ResourceResponse& response = m_frame->coreFrame()->loader()->documentLoader()->response();
+    m_frameHasCustomRepresentation = isMainFrame && WebProcess::shared().shouldUseCustomRepresentationForResponse(response);
 
     m_frame->coreFrame()->createView(webPage->size(), backgroundColor, false, IntSize(), false);
 #endif
@@ -1170,8 +1196,13 @@ PassRefPtr<Frame> WebFrameLoaderClient::createFrame(const KURL& url, const Strin
     RefPtr<WebFrame> subframe = WebFrame::createSubframe(webPage, name, ownerElement);
 
     Frame* coreSubframe = subframe->coreFrame();
+    if (!coreSubframe)
+        return 0;
+
+    // The creation of the frame may have run arbitrary JavaScript that removed it from the page already.
+    if (!coreSubframe->page())
+        return 0;
 
-     // The creation of the frame may have run arbitrary JavaScript that removed it from the page already.
     m_frame->coreFrame()->loader()->loadURLIntoChildFrame(url, referrer, coreSubframe);
 
     // The frame's onload handler may have removed it from the document.
@@ -1186,7 +1217,7 @@ void WebFrameLoaderClient::didTransferChildFrameToNewDocument(Page*)
     notImplemented();
 }
 
-void WebFrameLoaderClient::transferLoadingResourceFromPage(unsigned long, DocumentLoader*, const ResourceRequest&, Page*)
+void WebFrameLoaderClient::transferLoadingResourceFromPage(ResourceLoader*, const ResourceRequest&, Page*)
 {
     notImplemented();
 }
@@ -1204,29 +1235,30 @@ PassRefPtr<Widget> WebFrameLoaderClient::createPlugin(const IntSize&, HTMLPlugIn
     parameters.values = paramValues;
     parameters.mimeType = mimeType;
     parameters.loadManually = loadManually;
+    parameters.documentURL = m_frame->coreFrame()->document()->url().string();
 
-    // <rdar://problem/8440903>: AppleConnect has a bug where it does not
-    // understand the parameter names specified in the <object> element that
-    // embeds its plug-in. This hack works around the issue by converting the
-    // parameter names to lowercase before passing them to the plug-in.
-    // FIXME: This workaround should be dependent on site-specific quirks being
-    // enabled. This requires adding this setting to WebKit2's WebPreferences
-    // implementation. See <https://bugs.webkit.org/show_bug.cgi?id=46076>.
-    if (equalIgnoringCase(mimeType, "application/x-snkp")) {
-        for (size_t i = 0; i < paramNames.size(); ++i)
-            parameters.names[i] = paramNames[i].lower();
+    Frame* mainFrame = webPage->mainFrame()->coreFrame();
+    if (m_frame->coreFrame() == mainFrame)
+        parameters.toplevelDocumentURL = parameters.documentURL;
+    else if (m_frame->coreFrame()->document()->securityOrigin()->canAccess(mainFrame->document()->securityOrigin())) {
+        // We only want to set the toplevel document URL if the plug-in has access to it.
+        parameters.toplevelDocumentURL = mainFrame->document()->url().string();
     }
 
 #if PLUGIN_ARCHITECTURE(X11)
+    // FIXME: This should really be X11-specific plug-in quirks.
     if (equalIgnoringCase(mimeType, "application/x-shockwave-flash")) {
         // Currently we don't support transparency and windowed mode.
         // Inject wmode=opaque to make Flash work in these conditions.
         size_t wmodeIndex = parameters.names.find("wmode");
-        if (wmodeIndex == -1) {
+        if (wmodeIndex == notFound) {
             parameters.names.append("wmode");
             parameters.values.append("opaque");
         } else if (equalIgnoringCase(parameters.values[wmodeIndex], "window"))
             parameters.values[wmodeIndex] = "opaque";
+    } else if (equalIgnoringCase(mimeType, "application/x-webkit-test-netscape")) {
+        parameters.names.append("windowedPlugin");
+        parameters.values.append("false");
     }
 #endif
 
@@ -1247,30 +1279,67 @@ void WebFrameLoaderClient::redirectDataToPlugin(Widget* pluginWidget)
 
 PassRefPtr<Widget> WebFrameLoaderClient::createJavaAppletWidget(const IntSize& pluginSize, HTMLAppletElement* appletElement, const KURL& baseURL, const Vector<String>& paramNames, const Vector<String>& paramValues)
 {
-    return createPlugin(pluginSize, appletElement, KURL(), paramNames, paramValues, "application/x-java-applet", false);
+    const String mimeType = "application/x-java-applet";
+    RefPtr<Widget> plugin = createPlugin(pluginSize, appletElement, KURL(), paramNames, paramValues, mimeType, false);
+    if (!plugin) {
+        if (WebPage* webPage = m_frame->page())
+            webPage->send(Messages::WebPageProxy::DidFailToInitializePlugin(mimeType));
+    }
+    return plugin.release();
 }
 
-ObjectContentType WebFrameLoaderClient::objectContentType(const KURL& url, const String& mimeTypeIn)
+static bool pluginSupportsExtension(PluginData* pluginData, const String& extension)
+{
+    ASSERT(extension.lower() == extension);
+
+    for (size_t i = 0; i < pluginData->mimes().size(); ++i) {
+        const MimeClassInfo& mimeClassInfo = pluginData->mimes()[i];
+
+        if (mimeClassInfo.extensions.contains(extension))
+            return true;
+    }
+    return false;
+}
+
+ObjectContentType WebFrameLoaderClient::objectContentType(const KURL& url, const String& mimeTypeIn, bool shouldPreferPlugInsForImages)
 {
     // FIXME: This should be merged with WebCore::FrameLoader::defaultObjectContentType when the plugin code
     // is consolidated.
 
     String mimeType = mimeTypeIn;
-    if (mimeType.isEmpty())
-        mimeType = MIMETypeRegistry::getMIMETypeForExtension(url.path().substring(url.path().reverseFind('.') + 1));
+    if (mimeType.isEmpty()) {
+        String extension = url.path().substring(url.path().reverseFind('.') + 1).lower();
+
+        // Try to guess the MIME type from the extension.
+        mimeType = MIMETypeRegistry::getMIMETypeForExtension(extension);
+
+        if (mimeType.isEmpty()) {
+            // Check if there's a plug-in around that can handle the extension.
+            if (WebPage* webPage = m_frame->page()) {
+                if (PluginData* pluginData = webPage->corePage()->pluginData()) {
+                    if (pluginSupportsExtension(pluginData, extension))
+                        return ObjectContentNetscapePlugin;
+                }
+            }
+        }
+    }
 
     if (mimeType.isEmpty())
         return ObjectContentFrame;
 
-    if (MIMETypeRegistry::isSupportedImageMIMEType(mimeType))
-        return WebCore::ObjectContentImage;
-
+    bool plugInSupportsMIMEType = false;
     if (WebPage* webPage = m_frame->page()) {
         if (PluginData* pluginData = webPage->corePage()->pluginData()) {
             if (pluginData->supportsMimeType(mimeType))
-                return ObjectContentNetscapePlugin;
+                plugInSupportsMIMEType = true;
         }
     }
+    
+    if (MIMETypeRegistry::isSupportedImageMIMEType(mimeType))
+        return shouldPreferPlugInsForImages && plugInSupportsMIMEType ? ObjectContentNetscapePlugin : ObjectContentImage;
+
+    if (plugInSupportsMIMEType)
+        return ObjectContentNetscapePlugin;
 
     if (MIMETypeRegistry::isSupportedNonImageMIMEType(mimeType))
         return ObjectContentFrame;
@@ -1324,7 +1393,7 @@ NSCachedURLResponse* WebFrameLoaderClient::willCacheResponse(DocumentLoader*, un
 }
 
 #endif
-#if USE(CFNETWORK)
+#if PLATFORM(WIN) && USE(CFNETWORK)
 bool WebFrameLoaderClient::shouldCacheResponse(DocumentLoader*, unsigned long identifier, const ResourceResponse&, const unsigned char* data, unsigned long long length)
 {
     return true;
