@@ -46,15 +46,21 @@
 #include <WebCore/Frame.h>
 #include <WebCore/FrameView.h>
 #include <WebCore/HTMLFrameOwnerElement.h>
+#include <WebCore/HTMLNames.h>
 #include <WebCore/JSCSSStyleDeclaration.h>
 #include <WebCore/JSElement.h>
 #include <WebCore/JSRange.h>
 #include <WebCore/Page.h>
 #include <WebCore/RenderTreeAsText.h>
+#include <WebCore/SecurityOrigin.h>
 #include <WebCore/TextIterator.h>
 #include <WebCore/TextResourceDecoder.h>
 #include <wtf/text/StringBuilder.h>
 
+#if PLATFORM(MAC) || PLATFORM(WIN)
+#include <WebCore/LegacyWebArchive.h>
+#endif
+
 #ifndef NDEBUG
 #include <wtf/RefCountedLeakCounter.h>
 #endif
@@ -382,7 +388,7 @@ unsigned WebFrame::numberOfActiveAnimations() const
     if (!controller)
         return 0;
 
-    return controller->numberOfActiveAnimations();
+    return controller->numberOfActiveAnimations(m_coreFrame->document());
 }
 
 bool WebFrame::pauseAnimationOnElementWithId(const String& animationName, const String& elementID, double time)
@@ -404,6 +410,25 @@ bool WebFrame::pauseAnimationOnElementWithId(const String& animationName, const
     return controller->pauseAnimationAtTime(coreNode->renderer(), animationName, time);
 }
 
+bool WebFrame::pauseTransitionOnElementWithId(const String& propertyName, const String& elementID, double time)
+{
+    if (!m_coreFrame)
+        return false;
+
+    AnimationController* controller = m_coreFrame->animation();
+    if (!controller)
+        return false;
+
+    if (!m_coreFrame->document())
+        return false;
+
+    Node* coreNode = m_coreFrame->document()->getElementById(elementID);
+    if (!coreNode || !coreNode->renderer())
+        return false;
+
+    return controller->pauseTransitionAtTime(coreNode->renderer(), propertyName, time);
+}
+
 void WebFrame::suspendAnimations()
 {
     if (!m_coreFrame)
@@ -512,29 +537,63 @@ IntSize WebFrame::scrollOffset() const
     return view->scrollOffset();
 }
 
+bool WebFrame::hasHorizontalScrollbar() const
+{
+    if (!m_coreFrame)
+        return false;
+
+    FrameView* view = m_coreFrame->view();
+    if (!view)
+        return false;
+
+    return view->horizontalScrollbar();
+}
+
+bool WebFrame::hasVerticalScrollbar() const
+{
+    if (!m_coreFrame)
+        return false;
+
+    FrameView* view = m_coreFrame->view();
+    if (!view)
+        return false;
+
+    return view->verticalScrollbar();
+}
+
 bool WebFrame::getDocumentBackgroundColor(double* red, double* green, double* blue, double* alpha)
 {
     if (!m_coreFrame)
         return false;
+
+    FrameView* view = m_coreFrame->view();
+    if (!view)
+        return false;
+
+    Color bgColor = view->documentBackgroundColor();
+    if (!bgColor.isValid())
+        return false;
+
+    bgColor.getRGBA(*red, *green, *blue, *alpha);
+    return true;
+}
+
+bool WebFrame::containsAnyFormElements() const
+{
+    if (!m_coreFrame)
+        return false;
+    
     Document* document = m_coreFrame->document();
     if (!document)
         return false;
 
-    Element* rootElementToUse = document->body();
-    if (!rootElementToUse)
-        rootElementToUse = document->documentElement();
-    if (!rootElementToUse)
-        return false;
-
-    RenderObject* renderer = rootElementToUse->renderer();
-    if (!renderer)
-        return false;
-    Color color = renderer->style()->visitedDependentColor(CSSPropertyBackgroundColor);
-    if (!color.isValid())
-        return false;
-
-    color.getRGBA(*red, *green, *blue, *alpha);
-    return true;
+    for (Node* node = document->documentElement(); node; node = node->traverseNextNode()) {
+        if (!node->isElementNode())
+            continue;
+        if (static_cast<Element*>(node)->hasTagName(HTMLNames::formTag))
+            return true;
+    }
+    return false;
 }
 
 WebFrame* WebFrame::frameForContext(JSContextRef context)
@@ -550,6 +609,9 @@ WebFrame* WebFrame::frameForContext(JSContextRef context)
 
 JSValueRef WebFrame::jsWrapperForWorld(InjectedBundleNodeHandle* nodeHandle, InjectedBundleScriptWorld* world)
 {
+    if (!m_coreFrame)
+        return 0;
+
     JSDOMWindow* globalObject = m_coreFrame->script()->globalObject(world->coreWorld());
     ExecState* exec = globalObject->globalExec();
 
@@ -559,6 +621,9 @@ JSValueRef WebFrame::jsWrapperForWorld(InjectedBundleNodeHandle* nodeHandle, Inj
 
 JSValueRef WebFrame::jsWrapperForWorld(InjectedBundleRangeHandle* rangeHandle, InjectedBundleScriptWorld* world)
 {
+    if (!m_coreFrame)
+        return 0;
+
     JSDOMWindow* globalObject = m_coreFrame->script()->globalObject(world->coreWorld());
     ExecState* exec = globalObject->globalExec();
 
@@ -622,10 +687,10 @@ String WebFrame::suggestedFilenameForResourceWithURL(const KURL& url) const
 
     // Next, try subresources.
     RefPtr<ArchiveResource> resource = loader->subresource(url);
-    if (!resource)
-        return String();
-    
-    return resource->response().suggestedFilename();
+    if (resource)
+        return resource->response().suggestedFilename();
+
+    return page()->cachedSuggestedFilenameForURL(url);
 }
 
 String WebFrame::mimeTypeForResourceWithURL(const KURL& url) const
@@ -649,4 +714,27 @@ String WebFrame::mimeTypeForResourceWithURL(const KURL& url) const
     return page()->cachedResponseMIMETypeForURL(url);
 }
 
+void WebFrame::setTextDirection(const String& direction)
+{
+    if (!m_coreFrame || !m_coreFrame->editor())
+        return;
+
+    if (direction == "auto")
+        m_coreFrame->editor()->setBaseWritingDirection(NaturalWritingDirection);
+    else if (direction == "ltr")
+        m_coreFrame->editor()->setBaseWritingDirection(LeftToRightWritingDirection);
+    else if (direction == "rtl")
+        m_coreFrame->editor()->setBaseWritingDirection(RightToLeftWritingDirection);
+}
+
+#if PLATFORM(MAC) || PLATFORM(WIN)
+RetainPtr<CFDataRef> WebFrame::webArchiveData() const
+{
+    if (RefPtr<LegacyWebArchive> archive = LegacyWebArchive::create(coreFrame()->document()))
+        return archive->rawDataRepresentation();
+    
+    return 0;
+}
+#endif
+    
 } // namespace WebKit
