@@ -34,21 +34,21 @@
 
 #include "DumpRenderTree.h"
 #include "WebCoreSupport/DumpRenderTreeSupportGtk.h"
-
+#include <GOwnPtrGtk.h>
+#include <GRefPtrGtk.h>
 #include <GtkVersioning.h>
 #include <JavaScriptCore/JSObjectRef.h>
 #include <JavaScriptCore/JSRetainPtr.h>
 #include <JavaScriptCore/JSStringRef.h>
+#include <cstring>
+#include <gdk/gdk.h>
+#include <gdk/gdkkeysyms.h>
 #include <webkit/webkitwebframe.h>
 #include <webkit/webkitwebview.h>
 #include <wtf/ASCIICType.h>
 #include <wtf/Platform.h>
 #include <wtf/text/CString.h>
 
-#include <gdk/gdk.h>
-#include <gdk/gdkkeysyms.h>
-#include <string.h>
-
 extern "C" {
     extern GtkMenu* webkit_web_view_get_context_menu(WebKitWebView*);
 }
@@ -62,7 +62,7 @@ static int lastClickPositionX;
 static int lastClickPositionY;
 static int lastClickTimeOffset;
 static int lastClickButton;
-static int buttonCurrentlyDown;
+static unsigned buttonCurrentlyDown;
 static int clickCount;
 GdkDragContext* currentDragSourceContext;
 
@@ -229,6 +229,29 @@ static JSValueRef contextClickCallback(JSContextRef context, JSObjectRef functio
     return valueRef;
 }
 
+static gboolean sendClick(gpointer)
+{
+    GdkEvent* pressEvent = gdk_event_new(GDK_BUTTON_PRESS);
+
+    if (!prepareMouseButtonEvent(pressEvent, 1, 0)) {
+        gdk_event_free(pressEvent);
+        return FALSE;
+    }
+
+    GdkEvent* releaseEvent = gdk_event_copy(pressEvent);
+    dispatchEvent(pressEvent);
+    releaseEvent->type = GDK_BUTTON_RELEASE;
+    dispatchEvent(releaseEvent);
+
+    return FALSE;
+}
+
+static JSValueRef scheduleAsynchronousClickCallback(JSContextRef context, JSObjectRef function, JSObjectRef thisObject, size_t argumentCount, const JSValueRef arguments[], JSValueRef* exception)
+{
+    g_idle_add(sendClick, 0);
+    return JSValueMakeUndefined(context);
+}
+
 static void updateClickCount(int button)
 {
     if (lastClickPositionX != lastMousePositionX
@@ -240,33 +263,42 @@ static void updateClickCount(int button)
         clickCount++;
 }
 
+static guint gdkModifierFromJSValue(JSContextRef context, const JSValueRef value)
+{
+    JSStringRef string = JSValueToStringCopy(context, value, 0);
+    guint gdkModifier = 0;
+    if (JSStringIsEqualToUTF8CString(string, "ctrlKey")
+        || JSStringIsEqualToUTF8CString(string, "addSelectionKey"))
+        gdkModifier = GDK_CONTROL_MASK;
+    else if (JSStringIsEqualToUTF8CString(string, "shiftKey")
+             || JSStringIsEqualToUTF8CString(string, "rangeSelectionKey"))
+        gdkModifier = GDK_SHIFT_MASK;
+    else if (JSStringIsEqualToUTF8CString(string, "altKey"))
+        gdkModifier = GDK_MOD1_MASK;
+    
+    // Currently the metaKey as defined in WebCore/platform/gtk/MouseEventGtk.cpp
+    // is GDK_MOD2_MASK. This code must be kept in sync with that file.
+    else if (JSStringIsEqualToUTF8CString(string, "metaKey"))
+        gdkModifier = GDK_MOD2_MASK;
+    
+    JSStringRelease(string);
+    return gdkModifier;
+}
+
 static guint gdkModifersFromJSValue(JSContextRef context, const JSValueRef modifiers)
 {
+    // The value may either be a string with a single modifier or an array of modifiers.
+    if (JSValueIsString(context, modifiers))
+        return gdkModifierFromJSValue(context, modifiers);
+
     JSObjectRef modifiersArray = JSValueToObject(context, modifiers, 0);
     if (!modifiersArray)
         return 0;
 
     guint gdkModifiers = 0;
     int modifiersCount = JSValueToNumber(context, JSObjectGetProperty(context, modifiersArray, JSStringCreateWithUTF8CString("length"), 0), 0);
-    for (int i = 0; i < modifiersCount; ++i) {
-        JSValueRef value = JSObjectGetPropertyAtIndex(context, modifiersArray, i, 0);
-        JSStringRef string = JSValueToStringCopy(context, value, 0);
-        if (JSStringIsEqualToUTF8CString(string, "ctrlKey")
-            || JSStringIsEqualToUTF8CString(string, "addSelectionKey"))
-            gdkModifiers |= GDK_CONTROL_MASK;
-        else if (JSStringIsEqualToUTF8CString(string, "shiftKey")
-                 || JSStringIsEqualToUTF8CString(string, "rangeSelectionKey"))
-            gdkModifiers |= GDK_SHIFT_MASK;
-        else if (JSStringIsEqualToUTF8CString(string, "altKey"))
-            gdkModifiers |= GDK_MOD1_MASK;
-
-        // Currently the metaKey as defined in WebCore/platform/gtk/MouseEventGtk.cpp
-        // is GDK_MOD2_MASK. This code must be kept in sync with that file.
-        else if (JSStringIsEqualToUTF8CString(string, "metaKey"))
-            gdkModifiers |= GDK_MOD2_MASK;
-
-        JSStringRelease(string);
-    }
+    for (int i = 0; i < modifiersCount; ++i)
+        gdkModifiers |= gdkModifierFromJSValue(context, JSObjectGetPropertyAtIndex(context, modifiersArray, i, 0));
     return gdkModifiers;
 }
 
@@ -283,6 +315,10 @@ static JSValueRef mouseDownCallback(JSContextRef context, JSObjectRef function,
     if (!prepareMouseButtonEvent(event, button, modifiers))
         return JSValueMakeUndefined(context);
 
+    // If the same mouse button is already in the down position don't send another event as it may confuse Xvfb.
+    if (buttonCurrentlyDown == event->button.button)
+        return JSValueMakeUndefined(context);
+
     buttonCurrentlyDown = event->button.button;
 
     // Normally GDK will send both GDK_BUTTON_PRESS and GDK_2BUTTON_PRESS for
@@ -416,12 +452,85 @@ static JSValueRef continuousMouseScrollByCallback(JSContextRef context, JSObject
     return JSValueMakeUndefined(context);
 }
 
+static void dragWithFilesDragDataGetCallback(GtkWidget*, GdkDragContext*, GtkSelectionData *data, guint, guint, gpointer userData)
+{
+    gtk_selection_data_set_uris(data, static_cast<gchar**>(userData));
+}
+
+static void dragWithFilesDragEndCallback(GtkWidget* widget, GdkDragContext*, gpointer userData)
+{
+    g_signal_handlers_disconnect_by_func(widget, reinterpret_cast<void*>(dragWithFilesDragEndCallback), userData);
+    g_signal_handlers_disconnect_by_func(widget, reinterpret_cast<void*>(dragWithFilesDragDataGetCallback), userData);
+    g_strfreev(static_cast<gchar**>(userData));
+}
+
 static JSValueRef beginDragWithFilesCallback(JSContextRef context, JSObjectRef function, JSObjectRef thisObject, size_t argumentCount, const JSValueRef arguments[], JSValueRef* exception)
 {
     if (argumentCount < 1)
         return JSValueMakeUndefined(context);
 
-    // FIXME: Implement this completely once WebCore has complete drag and drop support
+    JSObjectRef filesArray = JSValueToObject(context, arguments[0], exception);
+    ASSERT(!exception || !*exception);
+
+    const gchar* mainFrameURI = webkit_web_frame_get_uri(mainFrame);
+    GRefPtr<GFile> testFile(adoptGRef(g_file_new_for_uri(mainFrameURI)));
+    GRefPtr<GFile> parentDirectory(g_file_get_parent(testFile.get()));
+    if (!parentDirectory)
+        return JSValueMakeUndefined(context);
+
+    // If this is an HTTP test, we still need to pass a local file path
+    // to WebCore. Even though the file doesn't exist, this should be fine
+    // for most tests.
+    GOwnPtr<gchar> scheme(g_file_get_uri_scheme(parentDirectory.get()));
+    if (g_str_equal(scheme.get(), "http") || g_str_equal(scheme.get(), "https")) {
+        GOwnPtr<gchar> currentDirectory(g_get_current_dir());
+        parentDirectory = g_file_new_for_path(currentDirectory.get());
+    }
+
+    JSStringRef lengthProperty = JSStringCreateWithUTF8CString("length");
+    int filesArrayLength = JSValueToNumber(context, JSObjectGetProperty(context, filesArray, lengthProperty, 0), 0);
+    JSStringRelease(lengthProperty);
+
+    gchar** draggedFilesURIList = g_new0(gchar*, filesArrayLength + 1);
+    for (int i = 0; i < filesArrayLength; ++i) {
+        JSStringRef filenameString = JSValueToStringCopy(context,
+                                                         JSObjectGetPropertyAtIndex(context, filesArray, i, 0), 0);
+        size_t bufferSize = JSStringGetMaximumUTF8CStringSize(filenameString);
+        GOwnPtr<gchar> filenameBuffer(static_cast<gchar*>(g_malloc(bufferSize)));
+        JSStringGetUTF8CString(filenameString, filenameBuffer.get(), bufferSize);
+        JSStringRelease(filenameString);
+
+        GRefPtr<GFile> dragFile(g_file_get_child(parentDirectory.get(), filenameBuffer.get()));
+        draggedFilesURIList[i] = g_file_get_uri(dragFile.get());
+    }
+
+    GtkWidget* view = GTK_WIDGET(webkit_web_frame_get_web_view(mainFrame));
+    g_object_connect(G_OBJECT(view),
+        "signal::drag-end", dragWithFilesDragEndCallback, draggedFilesURIList,
+        "signal::drag-data-get", dragWithFilesDragDataGetCallback, draggedFilesURIList,
+        NULL);
+
+    GdkEvent event;
+    GdkWindow* viewGDKWindow = gtk_widget_get_window(view);
+    memset(&event, 0, sizeof(event));
+    event.type = GDK_MOTION_NOTIFY;
+    event.motion.x = lastMousePositionX;
+    event.motion.y = lastMousePositionY;
+    event.motion.time = GDK_CURRENT_TIME;
+    event.motion.window = viewGDKWindow;
+    event.motion.device = getDefaultGDKPointerDevice(viewGDKWindow);
+    event.motion.state = GDK_BUTTON1_MASK;
+
+    int xRoot, yRoot;
+    gdk_window_get_root_coords(viewGDKWindow, lastMousePositionX, lastMousePositionY, &xRoot, &yRoot);
+    event.motion.x_root = xRoot;
+    event.motion.y_root = yRoot;
+
+    GtkTargetList* targetList = gtk_target_list_new(0, 0);
+    gtk_target_list_add_uri_targets(targetList, 0);
+    gtk_drag_begin(view, targetList, GDK_ACTION_COPY, 1, &event);
+    gtk_target_list_unref(targetList);
+
     return JSValueMakeUndefined(context);
 }
 
@@ -562,6 +671,8 @@ static JSValueRef keyDownCallback(JSContextRef context, JSObjectRef function, JS
             gdkKeySym = GDK_Delete;
         else if (JSStringIsEqualToUTF8CString(character, "printScreen"))
             gdkKeySym = GDK_Print;
+        else if (JSStringIsEqualToUTF8CString(character, "menu"))
+            gdkKeySym = GDK_Menu;
         else if (JSStringIsEqualToUTF8CString(character, "F1"))
             gdkKeySym = GDK_F1;
         else if (JSStringIsEqualToUTF8CString(character, "F2"))
@@ -680,6 +791,24 @@ static JSValueRef zoomPageOutCallback(JSContextRef context, JSObjectRef function
     return JSValueMakeUndefined(context);
 }
 
+static JSValueRef scalePageByCallback(JSContextRef context, JSObjectRef function, JSObjectRef thisObject, size_t argumentCount, const JSValueRef arguments[], JSValueRef* exception)
+{
+    if (argumentCount < 3)
+        return JSValueMakeUndefined(context);
+
+    float scaleFactor = JSValueToNumber(context, arguments[0], exception);
+    float x = JSValueToNumber(context, arguments[1], exception);
+    float y = JSValueToNumber(context, arguments[2], exception);
+
+    WebKitWebView* view = webkit_web_frame_get_web_view(mainFrame);
+    if (!view)
+        return JSValueMakeUndefined(context);
+
+    DumpRenderTreeSupportGtk::scalePageBy(view, scaleFactor, x, y);
+
+    return JSValueMakeUndefined(context);
+}
+
 static JSStaticFunction staticFunctions[] = {
     { "mouseScrollBy", mouseScrollByCallback, kJSPropertyAttributeReadOnly | kJSPropertyAttributeDontDelete },
     { "continuousMouseScrollBy", continuousMouseScrollByCallback, kJSPropertyAttributeReadOnly | kJSPropertyAttributeDontDelete },
@@ -694,6 +823,9 @@ static JSStaticFunction staticFunctions[] = {
     { "textZoomOut", textZoomOutCallback, kJSPropertyAttributeReadOnly | kJSPropertyAttributeDontDelete },
     { "zoomPageIn", zoomPageInCallback, kJSPropertyAttributeReadOnly | kJSPropertyAttributeDontDelete },
     { "zoomPageOut", zoomPageOutCallback, kJSPropertyAttributeReadOnly | kJSPropertyAttributeDontDelete },
+    { "scheduleAsynchronousClick", scheduleAsynchronousClickCallback, kJSPropertyAttributeReadOnly | kJSPropertyAttributeDontDelete },
+    { "scalePageBy", scalePageByCallback, kJSPropertyAttributeReadOnly | kJSPropertyAttributeDontDelete },
+
     { 0, 0, 0 }
 };
 
